@@ -33,10 +33,20 @@ DECLARE_bool(cache_force_single_shard);
 DECLARE_string(codegen_cache_capacity);
 
 namespace impala {
+
+// The object cache size for an engine containing a single compiled
+// function in our test case. The value is in bytes.
+const int ENGINE_CACHE_SIZE = 1100;
+// Capacity for large codegen cache. 256KB.
+const int64_t CODEGEN_CACHE_CAPACITY = 256 * 1024;
+
 class LlvmCodeGenCacheTest : public testing::Test {
  public:
   virtual void SetUp() {
     FLAGS_codegen_cache_capacity = "0";
+    // Using single shard makes the logic of scenarios simple for capacity and
+    // eviction-related behavior.
+    FLAGS_cache_force_single_shard = true;
     metrics_.reset(new MetricGroup("codegen-cache-test"));
     profile_ = RuntimeProfile::Create(&obj_pool_, "codegen-cache-test");
     test_env_.reset(new TestEnv);
@@ -51,6 +61,7 @@ class LlvmCodeGenCacheTest : public testing::Test {
   }
 
   virtual void TearDown() {
+    FLAGS_cache_force_single_shard = false;
     fragment_state_->ReleaseResources();
     fragment_state_ = nullptr;
     codegen_cache_.reset();
@@ -69,18 +80,12 @@ class LlvmCodeGenCacheTest : public testing::Test {
     return codegen->AddFunctionToJitInternal(fn, fn_ptr);
   }
 
-  static Status FinalizeModule(LlvmCodeGen* codegen) { return codegen->FinalizeModule(); }
-
-  static void CheckResult(CodeGenCacheEntry& entry, bool is_double = false) {
-    ASSERT_TRUE(!entry.Empty());
-    ASSERT_TRUE(entry.engine_pointer != nullptr);
-    CheckResult(entry.engine_pointer, is_double);
+  static Status FinalizeModule(LlvmCodeGen* codegen, string* module_id = nullptr) {
+    return codegen->FinalizeModule(module_id);
   }
 
   static void CheckResult(LlvmCodeGen* codegen, bool is_double = false) {
-    ASSERT_TRUE(codegen->execution_engine_wrapper_cached_ != nullptr);
-    llvm::ExecutionEngine* cached_execution_engine =
-        codegen->execution_engine_wrapper_cached_->execution_engine();
+    llvm::ExecutionEngine* cached_execution_engine = codegen->execution_engine();
     ASSERT_TRUE(cached_execution_engine != nullptr);
     CheckResult(cached_execution_engine, is_double);
   }
@@ -101,9 +106,13 @@ class LlvmCodeGenCacheTest : public testing::Test {
     }
   }
 
-  void AddLlvmCodegenEcho(LlvmCodeGen* codegen);
-  void AddLlvmCodegenDouble(LlvmCodeGen* codegen);
-  void GetLlvmEmptyFunction(LlvmCodeGen* codegen, llvm::Function**);
+  void CheckResult(
+      CodeGenCacheEntry& entry, const string& module_id, bool is_double = false);
+  shared_ptr<CodeGenObjectCache> CreateObjCache(bool is_double, string* module_id);
+  void AddLlvmCodegenEcho(LlvmCodeGen* codegen, string* module_id, bool doFinalizeModule);
+  void AddLlvmCodegenDouble(
+      LlvmCodeGen* codegen, string* module_id, bool doFinalizeModule);
+  void GetLlvmEmptyFunction(LlvmCodeGen* codegen, llvm::Function** func);
   typedef int (*TestEcho)(int);
   typedef int (*TestDouble)(int);
   typedef void (*TestEmpty)();
@@ -113,13 +122,14 @@ class LlvmCodeGenCacheTest : public testing::Test {
   void CheckMetrics(CodeGenCache*, int, int, int);
   void CheckInUseMetrics(CodeGenCache*, int, int64_t);
   void CheckEvictMetrics(CodeGenCache*, int);
+  void CheckObjCacheExists(LlvmCodeGen*);
   int64_t GetMemCharge(LlvmCodeGen* codegen, string key_str, bool is_normal_mode);
-  void CheckEngineCount(LlvmCodeGen*, int expect_count);
+  void CheckEngineCacheCount(LlvmCodeGen*, int expect_count);
   void CheckToInsertMap();
   void TestSwitchModeHelper(TCodeGenCacheMode::type mode, string key,
-      int expect_entry_num, int expect_engine_num, llvm::ExecutionEngine** engine);
+      int expect_entry_num, int expect_engine_num, CodeGenObjectCache** cached_engine);
   bool CheckKeyExist(TCodeGenCacheMode::type mode, string key);
-  bool CheckEngineExist(llvm::ExecutionEngine* engine);
+  bool CheckEngineExist(CodeGenObjectCache* cached_engine);
   void ExpectNumEngineSameAsEntry();
   void StoreHelper(TCodeGenCacheMode::type mode, string key);
   void TestConcurrentStore(int num_threads);
@@ -134,12 +144,68 @@ class LlvmCodeGenCacheTest : public testing::Test {
   RuntimeProfile* profile_;
   scoped_ptr<TestEnv> test_env_;
   scoped_ptr<CodeGenCache> codegen_cache_;
-  shared_ptr<LlvmExecutionEngineWrapper> exec_engine_wrapper_;
+  shared_ptr<CodeGenObjectCache> codegen_obj_cache_;
   TQueryOptions query_options_;
+  TCodeGenOptLevel::type opt_level_ = TCodeGenOptLevel::O2;
 };
 
-void LlvmCodeGenCacheTest::AddLlvmCodegenEcho(LlvmCodeGen* codegen) {
-  ASSERT_TRUE(codegen != NULL);
+void LlvmCodeGenCacheTest::CheckResult(
+    CodeGenCacheEntry& entry, const string& module_id, bool is_double) {
+  ASSERT_TRUE(!entry.Empty());
+  ASSERT_TRUE(entry.cached_engine_pointer != nullptr);
+  scoped_ptr<LlvmCodeGen> codegen;
+  ASSERT_OK(
+      LlvmCodeGen::CreateImpalaCodegen(fragment_state_, nullptr, "test1", &codegen));
+  if (is_double) {
+    AddLlvmCodegenDouble(codegen.get(), nullptr, false /*doFinalizeModule*/);
+  } else {
+    AddLlvmCodegenEcho(codegen.get(), nullptr, false /*doFinalizeModule*/);
+  }
+  ASSERT_OK(codegen->FinalizeLazyMaterialization());
+  codegen->module_->setModuleIdentifier(module_id);
+  // Use the specific cache for finalizing the module and expect the compiled functions
+  // in the execution engine is from the cache, then verify the correctness.
+  codegen->execution_engine()->setObjectCache(entry.cached_engine_pointer);
+  codegen->execution_engine()->finalizeObject();
+  codegen->DestroyModule();
+  CheckResult(codegen.get(), is_double);
+  codegen->Close();
+}
+
+// The function is to create and return a CodeGenObjectCache which contains a specific
+// compiled codegened function.
+shared_ptr<CodeGenObjectCache> LlvmCodeGenCacheTest::CreateObjCache(
+    bool is_double, string* module_id) {
+  scoped_ptr<LlvmCodeGen> codegen;
+  shared_ptr<CodeGenObjectCache> engine_cache = make_shared<CodeGenObjectCache>();
+  EXPECT_TRUE(
+      LlvmCodeGen::CreateImpalaCodegen(fragment_state_, nullptr, "test", &codegen).ok());
+  string m_id;
+  if (is_double) {
+    AddLlvmCodegenDouble(codegen.get(), nullptr, false /*doFinalizeModule*/);
+    m_id = "module_double";
+  } else {
+    AddLlvmCodegenEcho(codegen.get(), nullptr, false /*doFinalizeModule*/);
+    m_id = "module_echo";
+  }
+  EXPECT_TRUE(codegen->FinalizeLazyMaterialization().ok());
+  codegen->PruneModule();
+  EXPECT_TRUE(codegen->OptimizeModule().ok());
+  codegen->module_->setModuleIdentifier(m_id);
+  codegen->engine_cache_ = engine_cache;
+  codegen->execution_engine()->setObjectCache(engine_cache.get());
+  codegen->execution_engine()->finalizeObject();
+  CheckObjCacheExists(codegen.get());
+  codegen->DestroyModule();
+  CheckResult(codegen.get(), is_double);
+  codegen->Close();
+  if (module_id != nullptr) *module_id = m_id;
+  return engine_cache;
+}
+
+void LlvmCodeGenCacheTest::AddLlvmCodegenEcho(
+    LlvmCodeGen* codegen, string* module_id = nullptr, bool doFinalizeModule = true) {
+  ASSERT_TRUE(codegen != nullptr);
   LlvmCodeGen::FnPrototype prototype(codegen, "Echo", codegen->i32_type());
   prototype.AddArgument(LlvmCodeGen::NamedVariable("n", codegen->i32_type()));
   LlvmBuilder builder(codegen->context());
@@ -147,29 +213,32 @@ void LlvmCodeGenCacheTest::AddLlvmCodegenEcho(LlvmCodeGen* codegen) {
   llvm::Function* fn = prototype.GeneratePrototype(&builder, args);
   builder.CreateRet(args[0]);
   fn = codegen->FinalizeFunction(fn);
-  ASSERT_TRUE(fn != NULL);
+  ASSERT_TRUE(fn != nullptr);
   CodegenFnPtr<TestEcho> jitted_fn;
   AddFunctionToJit(codegen, fn, &jitted_fn);
-  ASSERT_OK(FinalizeModule(codegen));
-  ASSERT_TRUE(jitted_fn.load() != nullptr);
-  TestEcho test_fn = jitted_fn.load();
-  ASSERT_EQ(test_fn(1), 1);
+  if (doFinalizeModule) {
+    ASSERT_OK(FinalizeModule(codegen, module_id));
+    ASSERT_TRUE(jitted_fn.load() != nullptr);
+    TestEcho test_fn = jitted_fn.load();
+    ASSERT_EQ(test_fn(1), 1);
+  }
 }
 
 void LlvmCodeGenCacheTest::GetLlvmEmptyFunction(
     LlvmCodeGen* codegen, llvm::Function** func) {
-  ASSERT_TRUE(codegen != NULL);
+  ASSERT_TRUE(codegen != nullptr);
   LlvmCodeGen::FnPrototype prototype(codegen, "TestEmpty", codegen->void_type());
   LlvmBuilder builder(codegen->context());
   llvm::Function* fn = prototype.GeneratePrototype(&builder, nullptr);
   builder.CreateRetVoid();
   fn = codegen->FinalizeFunction(fn);
-  ASSERT_TRUE(fn != NULL);
+  ASSERT_TRUE(fn != nullptr);
   *func = fn;
 }
 
-void LlvmCodeGenCacheTest::AddLlvmCodegenDouble(LlvmCodeGen* codegen) {
-  ASSERT_TRUE(codegen != NULL);
+void LlvmCodeGenCacheTest::AddLlvmCodegenDouble(
+    LlvmCodeGen* codegen, string* module_id = nullptr, bool doFinalizeModule = true) {
+  ASSERT_TRUE(codegen != nullptr);
   LlvmCodeGen::FnPrototype prototype(codegen, "Double", codegen->i32_type());
   prototype.AddArgument(LlvmCodeGen::NamedVariable("n", codegen->i32_type()));
   LlvmBuilder builder(codegen->context());
@@ -179,59 +248,69 @@ void LlvmCodeGenCacheTest::AddLlvmCodegenDouble(LlvmCodeGen* codegen) {
   args[0] = builder.CreateMul(args[0], mul);
   builder.CreateRet(args[0]);
   fn = codegen->FinalizeFunction(fn);
-  ASSERT_TRUE(fn != NULL);
+  ASSERT_TRUE(fn != nullptr);
   CodegenFnPtr<TestDouble> jitted_fn;
   AddFunctionToJit(codegen, fn, &jitted_fn);
-  ASSERT_OK(FinalizeModule(codegen));
-  ASSERT_TRUE(jitted_fn.load() != nullptr);
-  TestEcho test_fn = jitted_fn.load();
-  ASSERT_EQ(test_fn(1), 2);
+  if (doFinalizeModule) {
+    ASSERT_OK(FinalizeModule(codegen, module_id));
+    ASSERT_TRUE(jitted_fn.load() != nullptr);
+    TestEcho test_fn = jitted_fn.load();
+    ASSERT_EQ(test_fn(1), 2);
+  }
 }
 
 /// Test the basic function of a codegen cache.
 void LlvmCodeGenCacheTest::TestBasicFunction(TCodeGenCacheMode::type mode) {
-  int64_t codegen_cache_capacity = 256 * 1024; // 256KB
-  codegen_cache_.reset(new CodeGenCache(metrics_.get()));
   bool is_normal_mode = !CodeGenCacheModeAnalyzer::is_optimal(mode);
-  EXPECT_OK(codegen_cache_->Init(codegen_cache_capacity));
 
   // Create a LlvmCodeGen containing a codegen function Echo.
-  scoped_ptr<LlvmCodeGen> codegen;
-  ASSERT_OK(LlvmCodeGen::CreateImpalaCodegen(fragment_state_, NULL, "test", &codegen));
-  AddLlvmCodegenEcho(codegen.get());
+  scoped_ptr<LlvmCodeGen> codegen_echo;
+  ASSERT_OK(
+      LlvmCodeGen::CreateImpalaCodegen(fragment_state_, nullptr, "test", &codegen_echo));
+  string module_id_echo;
+  codegen_echo->engine_cache_ = CreateObjCache(false /*is_double*/, &module_id_echo);
   // Create a LlvmCodeGen containing a codegen function Double.
   scoped_ptr<LlvmCodeGen> codegen_double;
   ASSERT_OK(LlvmCodeGen::CreateImpalaCodegen(
-      fragment_state_, NULL, "test_double", &codegen_double));
-  AddLlvmCodegenDouble(codegen_double.get());
+      fragment_state_, nullptr, "test_double", &codegen_double));
+  string module_id_double;
+  codegen_double->engine_cache_ = CreateObjCache(true /*is_double*/, &module_id_double);
+  CheckObjCacheExists(codegen_echo.get());
+  CheckObjCacheExists(codegen_double.get());
 
   CodeGenCacheKey cache_key;
   CodeGenCacheEntry entry;
   string key = "key";
   CodeGenCacheKeyConstructor::construct(key, &cache_key);
-  int64_t mem_charge = GetMemCharge(codegen.get(), cache_key.data(), is_normal_mode);
+  int64_t mem_charge_echo =
+      GetMemCharge(codegen_echo.get(), cache_key.data(), is_normal_mode);
   int64_t mem_charge_double =
       GetMemCharge(codegen_double.get(), cache_key.data(), is_normal_mode);
 
+  scoped_ptr<MetricGroup> metrics;
+  metrics.reset(new MetricGroup("codegen-cache-test-basic"));
+  codegen_cache_.reset(new CodeGenCache(metrics.get()));
+  EXPECT_OK(codegen_cache_->Init(CODEGEN_CACHE_CAPACITY));
   // Store and lookup the entry by the key.
-  EXPECT_OK(codegen_cache_->Store(cache_key, codegen.get(), mode));
+  EXPECT_OK(codegen_cache_->Store(cache_key, codegen_echo.get(), mode, opt_level_));
   CheckInUseMetrics(
-      codegen_cache_.get(), 1 /*num_entry_in_use*/, mem_charge /*bytes_in_use*/);
-  EXPECT_OK(codegen_cache_->Lookup(cache_key, mode, &entry, &exec_engine_wrapper_));
-  CheckResult(entry);
-  codegen->Close();
+      codegen_cache_.get(), 1 /*num_entry_in_use*/, mem_charge_echo /*bytes_in_use*/);
+  EXPECT_OK(codegen_cache_->Lookup(cache_key, mode, &entry, &codegen_obj_cache_));
+  CheckResult(entry, module_id_echo);
+  codegen_echo->Close();
   // Close the LlvmCodeGen, but should not affect the stored cache.
-  EXPECT_OK(codegen_cache_->Lookup(cache_key, mode, &entry, &exec_engine_wrapper_));
-  CheckResult(entry);
+  EXPECT_OK(codegen_cache_->Lookup(cache_key, mode, &entry, &codegen_obj_cache_));
+  CheckResult(entry, module_id_echo);
   // Override the entry with a different function, should be able to find the new
   // function from the new entry.
-  EXPECT_OK(codegen_cache_->Store(cache_key, codegen_double.get(), mode));
+  EXPECT_OK(codegen_cache_->Store(cache_key, codegen_double.get(), mode, opt_level_));
   CheckInUseMetrics(
       codegen_cache_.get(), 1 /*num_entry_in_use*/, mem_charge_double /*bytes_in_use*/);
-  EXPECT_OK(codegen_cache_->Lookup(cache_key, mode, &entry, &exec_engine_wrapper_));
-  CheckResult(entry, true /*is_double*/);
+  EXPECT_OK(codegen_cache_->Lookup(cache_key, mode, &entry, &codegen_obj_cache_));
+  CheckResult(entry, module_id_double, true /*is_double*/);
   EXPECT_EQ(codegen_cache_->codegen_cache_entries_evicted_->GetValue(), 1);
   codegen_double->Close();
+  codegen_cache_.reset();
 }
 
 void LlvmCodeGenCacheTest::CheckMetrics(
@@ -253,38 +332,47 @@ void LlvmCodeGenCacheTest::CheckEvictMetrics(CodeGenCache* codegen_cache, int ev
   EXPECT_EQ(codegen_cache->codegen_cache_entries_evicted_->GetValue(), evict);
 }
 
+void LlvmCodeGenCacheTest::CheckObjCacheExists(LlvmCodeGen* codegen) {
+  // If a cache is written, the size of it should be larger than size of the struct.
+  EXPECT_GT(codegen->engine_cache()->objSize(), sizeof(CodeGenObjectCache));
+}
+
 int64_t LlvmCodeGenCacheTest::GetMemCharge(
     LlvmCodeGen* codegen, string key_str, bool is_normal_mode) {
   if (is_normal_mode) {
-    return codegen->memory_manager_->bytes_allocated() + key_str.size()
+    return codegen->engine_cache()->objSize() + key_str.size()
         + sizeof(CodeGenCacheEntry);
   }
   // Optimal mode would use hash code and length as the key.
-  return codegen->memory_manager_->bytes_allocated() + CodeGenCacheKey::OptimalKeySize
+  return codegen->engine_cache()->objSize() + CodeGenCacheKey::OptimalKeySize
       + sizeof(CodeGenCacheEntry);
 }
 
 /// Test the situation that the codegen cache hits the limit of capacity, in this case,
 /// eviction is needed when new insertion comes.
 void LlvmCodeGenCacheTest::TestAtCapacity(TCodeGenCacheMode::type mode) {
-  int64_t codegen_cache_capacity = 196; // 196B
+  // Allocates memory for small examples.
+  int64_t codegen_cache_capacity = ENGINE_CACHE_SIZE + sizeof(CodeGenObjectCache)
+      + sizeof(CodeGenCacheEntry) + CodeGenCacheKey::OptimalKeySize + sizeof(std::string);
   bool is_normal_mode = !CodeGenCacheModeAnalyzer::is_optimal(mode);
-  // 128B for optimal mode
-  if (!is_normal_mode) codegen_cache_capacity = 128;
-  // Using single shard makes the logic of scenarios simple for capacity and
-  // eviction-related behavior.
-  FLAGS_cache_force_single_shard = true;
 
   // Create two LlvmCodeGen objects containing a different codegen function separately.
   scoped_ptr<LlvmCodeGen> codegen;
-  ASSERT_OK(LlvmCodeGen::CreateImpalaCodegen(fragment_state_, NULL, "test", &codegen));
+
+  test_env_->ResetCodegenCache(metrics_.get());
+  CodeGenCache* cache = test_env_->codegen_cache();
+  EXPECT_OK(cache->Init(codegen_cache_capacity));
+
+  ASSERT_OK(LlvmCodeGen::CreateImpalaCodegen(fragment_state_, nullptr, "test", &codegen));
   AddLlvmCodegenEcho(codegen.get());
   codegen->GenerateFunctionNamesHashCode();
   scoped_ptr<LlvmCodeGen> codegen_double;
   ASSERT_OK(LlvmCodeGen::CreateImpalaCodegen(
-      fragment_state_, NULL, "test_double", &codegen_double));
+      fragment_state_, nullptr, "test_double", &codegen_double));
   AddLlvmCodegenDouble(codegen_double.get());
   codegen_double->GenerateFunctionNamesHashCode();
+  CheckObjCacheExists(codegen.get());
+  CheckObjCacheExists(codegen_double.get());
 
   CodeGenCacheKey cache_key_1;
   CodeGenCacheKey cache_key_2;
@@ -300,10 +388,15 @@ void LlvmCodeGenCacheTest::TestAtCapacity(TCodeGenCacheMode::type mode) {
   ASSERT_LE(mem_charge_2, codegen_cache_capacity);
   ASSERT_GE(mem_charge_1 + mem_charge_2, codegen_cache_capacity);
 
-  test_env_->ResetCodegenCache(metrics_.get());
-  CodeGenCache* cache = test_env_->codegen_cache();
+  scoped_ptr<MetricGroup> metrics;
+  metrics.reset(new MetricGroup("codegen-test-capacity"));
+  test_env_->ResetCodegenCache(metrics.get());
+  cache = test_env_->codegen_cache();
   EXPECT_OK(cache->Init(codegen_cache_capacity));
 
+  // Expect init metrics.
+  CheckMetrics(cache, 0 /*hit*/, 0 /*miss*/, 0 /*evict*/);
+
   // Store key_1 and lookup.
   EXPECT_OK(codegen->StoreCache(cache_key_1));
   EXPECT_TRUE(codegen->LookupCache(cache_key_1));
@@ -333,6 +426,7 @@ void LlvmCodeGenCacheTest::TestAtCapacity(TCodeGenCacheMode::type mode) {
 
   codegen->Close();
   codegen_double->Close();
+  test_env_->ResetCodegenCache();
 }
 
 /// Test the case if we have a cache but doesn't contain the function
@@ -340,17 +434,17 @@ void LlvmCodeGenCacheTest::TestAtCapacity(TCodeGenCacheMode::type mode) {
 void LlvmCodeGenCacheTest::TestSkipCache() {
   // Initial a LlvmCodeGen object with a normal function.
   scoped_ptr<LlvmCodeGen> codegen;
-  ASSERT_OK(LlvmCodeGen::CreateImpalaCodegen(fragment_state_, NULL, "test", &codegen));
+  ASSERT_OK(LlvmCodeGen::CreateImpalaCodegen(fragment_state_, nullptr, "test", &codegen));
   AddLlvmCodegenEcho(codegen.get());
   // Create an empty function from other LlvmCodeGen to create the failure later.
   scoped_ptr<LlvmCodeGen> codegen_empty;
   ASSERT_OK(LlvmCodeGen::CreateImpalaCodegen(
-      fragment_state_, NULL, "test_empty", &codegen_empty));
+      fragment_state_, nullptr, "test_empty", &codegen_empty));
   llvm::Function* empty_func;
   GetLlvmEmptyFunction(codegen_empty.get(), &empty_func);
 
   test_env_->ResetCodegenCache(metrics_.get());
-  EXPECT_OK(test_env_->codegen_cache()->Init(256 * 1024 /*capacity*/));
+  EXPECT_OK(test_env_->codegen_cache()->Init(CODEGEN_CACHE_CAPACITY));
 
   CheckMetrics(test_env_->codegen_cache(), 0 /*hit*/, 0 /*miss*/, 0 /*evict*/);
   CodeGenCacheKey cache_key;
@@ -414,12 +508,10 @@ TEST_F(LlvmCodeGenCacheTest, ModeAnalyzer) {
   EXPECT_FALSE(CodeGenCacheModeAnalyzer::is_optimal(TCodeGenCacheMode::NORMAL_DEBUG));
 }
 
-// Check the number of execution engine wrappers stored in the cache. Because the shared
-// pointer of the execution engine wrapper needs to be stored in the codegen cache while
-// the entry using the execution engine is stored in the cache.
-void LlvmCodeGenCacheTest::CheckEngineCount(LlvmCodeGen* codegen, int expect_count) {
+// Check the number of engine caches stored in the global cache.
+void LlvmCodeGenCacheTest::CheckEngineCacheCount(LlvmCodeGen* codegen, int expect_count) {
   lock_guard<mutex> lock(codegen_cache_->cached_engines_lock_);
-  auto engine_it = codegen_cache_->cached_engines_.find(codegen->execution_engine());
+  auto engine_it = codegen_cache_->cached_engines_.find(codegen->engine_cache());
   EXPECT_TRUE(engine_it != codegen_cache_->cached_engines_.end());
   EXPECT_EQ(codegen_cache_->cached_engines_.size(), expect_count);
 }
@@ -434,13 +526,13 @@ bool LlvmCodeGenCacheTest::CheckKeyExist(TCodeGenCacheMode::type mode, string ke
   CodeGenCacheKey cache_key;
   CodeGenCacheEntry entry;
   CodeGenCacheKeyConstructor::construct(key, &cache_key);
-  EXPECT_OK(codegen_cache_->Lookup(cache_key, mode, &entry, &exec_engine_wrapper_));
+  EXPECT_OK(codegen_cache_->Lookup(cache_key, mode, &entry, &codegen_obj_cache_));
   return !entry.Empty();
 }
 
 // Return true if the provided engine exists.
-bool LlvmCodeGenCacheTest::CheckEngineExist(llvm::ExecutionEngine* engine) {
-  auto engine_it = codegen_cache_->cached_engines_.find(engine);
+bool LlvmCodeGenCacheTest::CheckEngineExist(CodeGenObjectCache* cached_engine) {
+  auto engine_it = codegen_cache_->cached_engines_.find(cached_engine);
   return engine_it != codegen_cache_->cached_engines_.end();
 }
 
@@ -455,33 +547,39 @@ void LlvmCodeGenCacheTest::ExpectNumEngineSameAsEntry() {
 /// and mode.
 void LlvmCodeGenCacheTest::TestSwitchModeHelper(TCodeGenCacheMode::type mode, string key,
     int expect_entry_num = -1, int expect_engine_num = -1,
-    llvm::ExecutionEngine** engine = nullptr) {
+    CodeGenObjectCache** engine_cache = nullptr) {
   // Create a LlvmCodeGen containing a codegen function Echo.
   scoped_ptr<LlvmCodeGen> codegen;
-  ASSERT_OK(LlvmCodeGen::CreateImpalaCodegen(fragment_state_, NULL, "test", &codegen));
-  AddLlvmCodegenEcho(codegen.get());
+  ASSERT_OK(LlvmCodeGen::CreateImpalaCodegen(fragment_state_, nullptr, "test", &codegen));
+  string module_id_echo;
+  codegen->engine_cache_ = CreateObjCache(false /*is_double*/, &module_id_echo);
+  ASSERT_TRUE(codegen->engine_cache_ != nullptr);
+  CheckObjCacheExists(codegen.get());
 
   CodeGenCacheKey cache_key;
   CodeGenCacheEntry entry;
   CodeGenCacheKeyConstructor::construct(key, &cache_key);
 
   // Store and lookup the entry by the key.
-  EXPECT_OK(codegen_cache_->Store(cache_key, codegen.get(), mode));
+  EXPECT_OK(codegen_cache_->Store(cache_key, codegen.get(), mode, opt_level_));
   if (expect_entry_num != -1) {
     CheckInUseMetrics(codegen_cache_.get(), expect_entry_num /*num_entry_in_use*/);
   }
   if (expect_engine_num != -1) {
-    CheckEngineCount(codegen.get(), expect_engine_num);
+    CheckEngineCacheCount(codegen.get(), expect_engine_num);
   }
-  EXPECT_OK(codegen_cache_->Lookup(cache_key, mode, &entry, &exec_engine_wrapper_));
-  CheckResult(entry);
-  if (engine) *engine = codegen->execution_engine();
+  EXPECT_OK(codegen_cache_->Lookup(cache_key, mode, &entry, &codegen_obj_cache_));
+  CheckResult(entry, module_id_echo);
+  if (engine_cache) *engine_cache = codegen->engine_cache();
   codegen->Close();
 }
 
 // Test to switch among different modes.
 TEST_F(LlvmCodeGenCacheTest, SwitchMode) {
-  int64_t codegen_cache_capacity = 512; // 512B
+  // Storage for 2 entries.
+  int64_t codegen_cache_capacity = 2
+      * (ENGINE_CACHE_SIZE + sizeof(CodeGenCacheEntry) + CodeGenCacheKey::OptimalKeySize
+            + sizeof(std::string));
   codegen_cache_.reset(new CodeGenCache(metrics_.get()));
   EXPECT_OK(codegen_cache_->Init(codegen_cache_capacity));
   string key = "key";
@@ -496,7 +594,7 @@ TEST_F(LlvmCodeGenCacheTest, SwitchMode) {
   TestSwitchModeHelper(TCodeGenCacheMode::NORMAL_DEBUG, key, 2, 2);
   // Try again, the new insertion should replace the old ones, so the entry number and
   // engine number won't change.
-  llvm::ExecutionEngine *engine_opt, *engine_opt_dbg, *engine_normal, *engine_normal_dbg;
+  CodeGenObjectCache *engine_opt, *engine_opt_dbg, *engine_normal, *engine_normal_dbg;
   TestSwitchModeHelper(TCodeGenCacheMode::OPTIMAL, key, 2, 2, &engine_opt);
   TestSwitchModeHelper(TCodeGenCacheMode::OPTIMAL_DEBUG, key, 2, 2, &engine_opt_dbg);
   // Expect the engines with the same key should be different, because the engine is
@@ -535,12 +633,15 @@ TEST_F(LlvmCodeGenCacheTest, SwitchMode) {
 void LlvmCodeGenCacheTest::StoreHelper(TCodeGenCacheMode::type mode, string key) {
   // Create a LlvmCodeGen containing a codegen function Echo.
   scoped_ptr<LlvmCodeGen> codegen;
-  ASSERT_OK(LlvmCodeGen::CreateImpalaCodegen(fragment_state_, NULL, "test", &codegen));
-  AddLlvmCodegenEcho(codegen.get());
+  ASSERT_OK(LlvmCodeGen::CreateImpalaCodegen(fragment_state_, nullptr, "test", &codegen));
+  shared_ptr<CodeGenObjectCache> engine_cache =
+      CreateObjCache(true /*is_double*/, nullptr /*module_id*/);
+  codegen->engine_cache_ = engine_cache;
   CodeGenCacheKey cache_key;
   CodeGenCacheEntry entry;
   CodeGenCacheKeyConstructor::construct(key, &cache_key);
-  EXPECT_OK(codegen_cache_->Store(cache_key, codegen.get(), mode));
+  EXPECT_OK(codegen_cache_->Store(cache_key, codegen.get(), mode, opt_level_));
+  CheckObjCacheExists(codegen.get());
   codegen->Close();
 }
 
@@ -572,7 +673,10 @@ void LlvmCodeGenCacheTest::TestConcurrentStore(int num_threads) {
 }
 
 TEST_F(LlvmCodeGenCacheTest, ConcurrentStore) {
-  int64_t codegen_cache_capacity = 512; // 512B
+  // Storage for 2 entries.
+  int64_t codegen_cache_capacity = 2
+      * (ENGINE_CACHE_SIZE + sizeof(CodeGenCacheEntry) + CodeGenCacheKey::OptimalKeySize
+            + sizeof(std::string));
   codegen_cache_.reset(new CodeGenCache(metrics_.get()));
   EXPECT_OK(codegen_cache_->Init(codegen_cache_capacity));
   TestConcurrentStore(8);
