@@ -52,6 +52,8 @@ DEFINE_string(principal, "", "Kerberos principal. If set, both client and backen
 DEFINE_string(be_principal, "", "Kerberos principal for backend network connections only,"
     "overriding --principal if set. Must not be set if --principal is not set.");
 DEFINE_string(keytab_file, "", "Absolute path to Kerberos keytab file");
+DEFINE_string(spnego_keytab_file, "", "Absolute path to Kerberos keytab file "
+    "for HTTP spnego. If it is empty, --keytab_file flag will be used.");
 DEFINE_string(krb5_ccname, "/tmp/krb5cc_impala_internal", "Absolute path to the file "
     "based credentials cache that we pass to the KRB5CCNAME environment variable.");
 DEFINE_string(krb5_conf, "", "Absolute path to Kerberos krb5.conf if in a non-standard "
@@ -230,6 +232,12 @@ DEFINE_bool(redirect_stdout_stderr, true,
 DEFINE_int32(max_log_files, 10, "Maximum number of log files to retain per severity "
     "level. The most recent log files are retained. If set to 0, all log files are "
     "retained.");
+DEFINE_bool(log_rotation_match_pid, false,
+    "If set to True, Impala log rotation will only consider log files that match with "
+    "PID of currently running service. Otherwise, log rotation will ignore the PID in "
+    "log file names and may remove older log files from previous PID run. "
+    "Set to True if log files from prior run must be retained or when running multiple "
+    "instances of same service with common log directory. Default to False.");
 
 static const string re2_mem_limit_help_msg =
     "Maximum bytes of memory to be used by re2's regex engine "
@@ -413,6 +421,24 @@ DEFINE_int32(iceberg_reload_new_files_threshold, 100, "(Advanced) If during a ta
     "reload all file metadata. If number of new files are less or equal to this, "
     "catalogd will only load the metadata of the newly added files.");
 
+DEFINE_bool(iceberg_allow_datafiles_in_table_location_only, true, "If true, Impala "
+    "does not allow Iceberg data file locations outside of the table directory during "
+    "reads");
+
+// Host and port of Statestore Service
+DEFINE_string(state_store_host, "localhost",
+    "hostname where StatestoreService is running");
+DEFINE_int32(state_store_port, 24000, "port where StatestoreService is running");
+
+// Starting flags for Statestore HA
+DEFINE_string(state_store_2_host, "localhost",
+    "hostname where second StatestoreService instance is running");
+DEFINE_int32(state_store_2_port, 24001,
+    "port where second StatestoreService instance is running");
+// Port for RPC communication between two statestore instances for Statestore HA
+DEFINE_int32(state_store_ha_port, 24020,
+    "port where StatestoreHaService is running");
+
 // TGeospatialLibrary's values are mapped here as constants
 static const string geo_lib_none = "NONE";
 static const string geo_lib_hive_esri = "HIVE_ESRI";
