@@ -83,7 +83,8 @@ DECLARE_string(re2_mem_limit);
 DECLARE_string(reserved_words_version);
 DECLARE_bool(symbolize_stacktrace);
 DECLARE_string(debug_actions);
-DECLARE_int32(thrift_rpc_max_message_size);
+DECLARE_int64(thrift_rpc_max_message_size);
+DECLARE_int64(thrift_external_rpc_max_message_size);
 
 DEFINE_int32(memory_maintenance_sleep_time_ms, 10000, "Sleep time in milliseconds "
     "between memory maintenance iterations");
@@ -376,9 +377,10 @@ static Status JavaAddOpens(bool useSizeOf) {
   }
 
   for (const string& param : {
-    // Needed for jamm and ehcache
+    // Needed for jamm and ehcache (jamm needs it to access lambdas)
     "--add-opens=java.base/java.lang=ALL-UNNAMED",
     "--add-opens=java.base/java.nio=ALL-UNNAMED",
+    "--add-opens=java.base/java.util.regex=ALL-UNNAMED",
     "--add-opens=java.base/sun.nio.ch=ALL-UNNAMED"
   }) {
     val_out << " " << param;
@@ -406,6 +408,7 @@ static Status JavaAddOpens(bool useSizeOf) {
       "--add-opens=java.base/jdk.internal.module=ALL-UNNAMED",
       "--add-opens=java.base/jdk.internal.perf=ALL-UNNAMED",
       "--add-opens=java.base/jdk.internal.platform=ALL-UNNAMED",
+      "--add-opens=java.base/jdk.internal.platform.cgroupv1=ALL-UNNAMED",
       "--add-opens=java.base/jdk.internal.ref=ALL-UNNAMED",
       "--add-opens=java.base/jdk.internal.reflect=ALL-UNNAMED",
       "--add-opens=java.base/jdk.internal.util.jar=ALL-UNNAMED",
@@ -535,10 +538,24 @@ void impala::InitCommonRuntime(int argc, char** argv, bool init_jvm,
         FLAGS_reserved_words_version));
   }
 
+  // Enforce a minimum value for thrift_max_message_size, as configuring the limit to
+  // a small value is very unlikely to work.
   if (!impala::TestInfo::is_test() && FLAGS_thrift_rpc_max_message_size > 0
       && FLAGS_thrift_rpc_max_message_size < ThriftDefaultMaxMessageSize()) {
-    CLEAN_EXIT_WITH_ERROR(Substitute("thrift_rpc_max_message_size must be >= $0 or <= 0.",
-        ThriftDefaultMaxMessageSize()));
+    CLEAN_EXIT_WITH_ERROR(
+        Substitute("Invalid $0: $1 is less than the minimum value of $2.",
+            "thrift_rpc_max_message_size", FLAGS_thrift_rpc_max_message_size,
+            ThriftDefaultMaxMessageSize()));
+  }
+
+  // Enforce a minimum value for thrift_external_max_message_size, as configuring the
+  // limit to a small value is very unlikely to work.
+  if (!impala::TestInfo::is_test() && FLAGS_thrift_external_rpc_max_message_size > 0
+      && FLAGS_thrift_external_rpc_max_message_size < ThriftDefaultMaxMessageSize()) {
+    CLEAN_EXIT_WITH_ERROR(
+        Substitute("Invalid $0: $1 is less than the minimum value of $2.",
+            "thrift_external_rpc_max_message_size",
+            FLAGS_thrift_external_rpc_max_message_size, ThriftDefaultMaxMessageSize()));
   }
 
   impala::InitGoogleLoggingSafe(argv[0]);
