@@ -27,7 +27,8 @@
 #include "exec/hbase/hbase-table-sink.h"
 #include "exec/hdfs-table-sink.h"
 #include "exec/iceberg-delete-builder.h"
-#include "exec/iceberg-delete-sink.h"
+#include "exec/iceberg-delete-sink-config.h"
+#include "exec/multi-table-sink.h"
 #include "exec/kudu/kudu-table-sink.h"
 #include "exec/kudu/kudu-util.h"
 #include "exec/nested-loop-join-builder.h"
@@ -125,6 +126,10 @@ Status DataSinkConfig::CreateConfig(const TDataSink& thrift_sink,
       *data_sink = pool->Add(new IcebergDeleteBuilderConfig());
       break;
     }
+    case TDataSinkType::MULTI_DATA_SINK: {
+      *data_sink = pool->Add(new MultiTableSinkConfig());
+      break;
+    }
     default:
       stringstream error_msg;
       map<int, const char*>::const_iterator i =
