@@ -38,6 +38,7 @@
 #include "exec/hdfs-scan-node-mt.h"
 #include "exec/hdfs-scan-node.h"
 #include "exec/iceberg-delete-node.h"
+#include "exec/iceberg-metadata/iceberg-metadata-scan-node.h"
 #include "exec/kudu/kudu-scan-node-mt.h"
 #include "exec/kudu/kudu-scan-node.h"
 #include "exec/kudu/kudu-util.h"
@@ -50,6 +51,7 @@
 #include "exec/streaming-aggregation-node.h"
 #include "exec/subplan-node.h"
 #include "exec/topn-node.h"
+#include "exec/tuple-cache-node.h"
 #include "exec/union-node.h"
 #include "exec/unnest-node.h"
 #include "exprs/expr.h"
@@ -174,6 +176,7 @@ Status PlanNode::CreatePlanNode(
     case TPlanNodeType::HBASE_SCAN_NODE:
     case TPlanNodeType::DATA_SOURCE_NODE:
     case TPlanNodeType::KUDU_SCAN_NODE:
+    case TPlanNodeType::SYSTEM_TABLE_SCAN_NODE:
       *node = pool->Add(new ScanPlanNode());
       break;
     case TPlanNodeType::AGGREGATION_NODE:
@@ -226,6 +229,12 @@ Status PlanNode::CreatePlanNode(
     case TPlanNodeType::ICEBERG_DELETE_NODE:
       *node = pool->Add(new IcebergDeletePlanNode());
       break;
+    case TPlanNodeType::ICEBERG_METADATA_SCAN_NODE:
+      *node = pool->Add(new IcebergMetadataScanPlanNode());
+      break;
+    case TPlanNodeType::TUPLE_CACHE_NODE:
+      *node = pool->Add(new TupleCachePlanNode());
+      break;
     default:
       map<int, const char*>::const_iterator i =
           _TPlanNodeType_VALUES_TO_NAMES.find(tnode.node_type);
