@@ -136,9 +136,9 @@ class HdfsTableSink : public TableSinkBase {
   /// 'output_partition->raw_partition_names' is a vector of partition key-values in a
   /// non-encoded format.
   /// Staying with the above example this would hold ["a=12/31/11", "b=10"].
-  void ConstructPartitionInfo(
+  Status ConstructPartitionInfo(
       const TupleRow* row,
-      OutputPartition* output_partition) override;
+      OutputPartition* output_partition);
 
   /// Returns partition descriptor object for the given key.
   const HdfsPartitionDescriptor* GetPartitionDescriptor(const std::string& key);
