@@ -406,9 +406,10 @@ Status HdfsOrcScanner::Open(ScannerContext* context) {
     row_batches_need_validation_ = rows_valid == ValidWriteIdList::SOME;
   }
 
-  if (UNLIKELY(scan_node_->optimize_parquet_count_star())) {
-    DCHECK(false);
-    return Status("Internal ERROR: ORC scanner cannot optimize count star slot.");
+  if (scan_node_->optimize_count_star()) {
+    DCHECK(!row_batches_need_validation_);
+    template_tuple_ = template_tuple_map_[scan_node_->tuple_desc()];
+    return Status::OK();
   }
 
   // Update 'row_reader_options_' based on the tuple descriptor so the ORC lib can skip
@@ -689,6 +690,7 @@ void HdfsOrcScanner::SetSyntheticAcidFieldForOriginalFile(const SlotDescriptor*
       break;
     case ACID_FIELD_ROWID_INDEX:
       file_position_ = slot_desc;
+      break;
     default:
       break;
   }
@@ -778,10 +780,39 @@ Status HdfsOrcScanner::ProcessSplit() {
 }
 
 Status HdfsOrcScanner::GetNextInternal(RowBatch* row_batch) {
-  // In case 'row_batches_need_validation_' is true, we need to look at the row
-  // batches and check their validity. In that case 'currentTransaction' is the only
-  // selected field from the file (in case of zero slot scans).
-  if (scan_node_->IsZeroSlotTableScan() && !row_batches_need_validation_) {
+  if (scan_node_->optimize_count_star()) {
+    // There are no materialized slots, e.g. count(*) over the table.  We can serve
+    // this query from just the file metadata.  We don't need to read the column data.
+    // Only scanner of the footer split will run in this case. See the logic in
+    // HdfsScanner::IssueFooterRanges() and HdfsScanNodeBase::ReadsFileMetadataOnly().
+    DCHECK(!row_batches_need_validation_);
+    DCHECK(is_footer_scanner_);
+    uint64_t file_rows = reader_->getNumberOfRows();
+    DCHECK_LT(stripe_rows_read_, file_rows);
+    COUNTER_ADD(num_file_metadata_read_, 1);
+    int64_t tuple_buffer_size;
+    uint8_t* tuple_buffer;
+    int capacity = 1;
+    RETURN_IF_ERROR(row_batch->ResizeAndAllocateTupleBuffer(state_,
+        row_batch->tuple_data_pool(), row_batch->row_desc()->GetRowSize(), &capacity,
+        &tuple_buffer_size, &tuple_buffer));
+    Tuple* dst_tuple = reinterpret_cast<Tuple*>(tuple_buffer);
+    InitTuple(template_tuple_, dst_tuple);
+    int64_t* dst_slot = dst_tuple->GetBigIntSlot(scan_node_->count_star_slot_offset());
+    *dst_slot = file_rows;
+    TupleRow* dst_row = row_batch->GetRow(row_batch->AddRow());
+    dst_row->SetTuple(0, dst_tuple);
+    row_batch->CommitLastRow();
+    stripe_rows_read_ += file_rows;
+    COUNTER_ADD(scan_node_->rows_read_counter(), file_rows);
+    eos_ = true;
+    return Status::OK();
+  } else if (scan_node_->IsZeroSlotTableScan() && !row_batches_need_validation_) {
+    // In case 'row_batches_need_validation_' is true, we need to look at the row
+    // batches and check their validity. In that case 'currentTransaction' is the only
+    // selected field from the file (in case of zero slot scans).
+    // This block only handle case when 'row_batches_need_validation_' is false.
+    DCHECK(is_footer_scanner_);
     uint64_t file_rows = reader_->getNumberOfRows();
     // There are no materialized slots, e.g. count(*) over the table.  We can serve
     // this query from just the file metadata.  We don't need to read the column data.
@@ -789,6 +820,7 @@ Status HdfsOrcScanner::GetNextInternal(RowBatch* row_batch) {
       eos_ = true;
       return Status::OK();
     }
+    DCHECK_LT(stripe_rows_read_, file_rows);
     COUNTER_ADD(num_file_metadata_read_, 1);
     assemble_rows_timer_.Start();
     DCHECK_LT(stripe_rows_read_, file_rows);
@@ -1160,7 +1192,7 @@ orc::Literal HdfsOrcScanner::GetSearchArgumentLiteral(ScalarExprEvaluator* eval,
     case TYPE_STRING: {
       if (UNLIKELY(!val)) return orc::Literal(predicate_type);
       const StringValue* sv = reinterpret_cast<StringValue*>(val);
-      return orc::Literal(sv->ptr, sv->len);
+      return orc::Literal(sv->Ptr(), sv->Len());
     }
     // Predicates on CHAR/VARCHAR are currently skipped in FE. We will focus on them in
     // IMPALA-10882.
@@ -1172,16 +1204,17 @@ orc::Literal HdfsOrcScanner::GetSearchArgumentLiteral(ScalarExprEvaluator* eval,
       DCHECK(false) << "Char predicate is not supported: IMPALA-10882";
       if (UNLIKELY(!val)) return orc::Literal(predicate_type);
       const StringValue* sv = reinterpret_cast<StringValue*>(val);
+      StringValue::SimpleString s = sv->ToSimpleString();
       char* dst_ptr;
-      if (dst_type.len > sv->len) {
+      if (dst_type.len > s.len) {
         dst_ptr = reinterpret_cast<char*>(search_args_pool_->TryAllocate(dst_type.len));
         if (dst_ptr == nullptr) return orc::Literal(predicate_type);
-        memcpy(dst_ptr, sv->ptr, sv->len);
-        StringValue::PadWithSpaces(dst_ptr, dst_type.len, sv->len);
+        memcpy(dst_ptr, s.ptr, s.len);
+        StringValue::PadWithSpaces(dst_ptr, dst_type.len, s.len);
       } else {
-        dst_ptr = sv->ptr;
+        dst_ptr = s.ptr;
       }
-      return orc::Literal(dst_ptr, sv->len);
+      return orc::Literal(dst_ptr, s.len);
     }
     case TYPE_DECIMAL: {
       if (!val) return orc::Literal(predicate_type);
