@@ -764,8 +764,9 @@ template<>
 inline StringValue* HdfsParquetTableWriter::ColumnWriter<StringValue>::CastValue(
     void* value) {
   if (type().type == TYPE_CHAR) {
-    temp_.ptr = reinterpret_cast<char*>(value);
-    temp_.len = StringValue::UnpaddedCharLength(temp_.ptr, type().len);
+    char* s = reinterpret_cast<char*>(value);
+    int len = StringValue::UnpaddedCharLength(s, type().len);
+    temp_.Assign(reinterpret_cast<char*>(value), len);
     return &temp_;
   }
   return reinterpret_cast<StringValue*>(value);
@@ -1503,7 +1504,7 @@ Status HdfsParquetTableWriter::CreateSchema() {
     const int field_id = col_desc.field_id();
     if (field_id != -1) col_schema.__set_field_id(field_id);
     ParquetMetadataUtils::FillSchemaElement(col_type, string_utf8_, timestamp_type_,
-                                            col_desc.auxType(), &col_schema);
+        &col_schema);
   }
 
   return Status::OK();
