@@ -415,7 +415,6 @@ parquet::Type::type ParquetMetadataUtils::ConvertInternalToParquetType(
 
 void ParquetMetadataUtils::FillSchemaElement(const ColumnType& col_type,
     bool string_utf8, TParquetTimestampType::type timestamp_type,
-    const AuxColumnType& aux_type,
     parquet::SchemaElement* col_schema) {
   col_schema->__set_type(ConvertInternalToParquetType(col_type.type, timestamp_type));
   col_schema->__set_repetition_type(parquet::FieldRepetitionType::OPTIONAL);
@@ -431,7 +430,7 @@ void ParquetMetadataUtils::FillSchemaElement(const ColumnType& col_type,
     case TYPE_STRING:
       // By default STRING has no logical type, see IMPALA-5982.
       // VARCHAR and CHAR are always set to UTF8.
-      if (string_utf8 && !aux_type.IsBinaryStringSubtype()) {
+      if (string_utf8 && !col_type.IsBinaryType()) {
         SetUtf8ConvertedAndLogicalType(col_schema);
       }
       break;
