@@ -18,6 +18,8 @@
 #include "exec/partitioned-hash-join-node.inline.h"
 
 #include "codegen/impala-ir.h"
+#include "exec/blocking-join-node.inline.h"
+#include "exec/exec-node.inline.h"
 #include "exec/hash-table.inline.h"
 #include "runtime/row-batch.h"
 
@@ -303,7 +305,7 @@ bool IR_ALWAYS_INLINE PartitionedHashJoinNode::NextProbeRow(
         hash_tbl_iterator_ = hash_tbl->FindProbeRow(ht_ctx);
       } else {
         // The build partition is either empty or spilled.
-        PhjBuilder::Partition* build_partition =
+        PhjBuilderPartition* build_partition =
             (*build_hash_partitions_.hash_partitions)[partition_idx].get();
         ProbePartition* probe_partition = probe_hash_partitions_[partition_idx].get();
         DCHECK((build_partition->IsClosed() && probe_partition == NULL)
