@@ -98,7 +98,7 @@ Status TableSinkBase::ClosePartitionFile(
 string TableSinkBase::GetPartitionName(int i) {
   if (IsIceberg()) {
     DCHECK_LT(i, partition_key_expr_evals_.size());
-    return table_desc_->IcebergNonVoidPartitionNames()[i];
+    return table_desc_->IcebergNonVoidPartitionFields()[i].field_name;
   } else {
     DCHECK_LT(i, table_desc_->num_clustering_cols());
     return table_desc_->col_descs()[i].name();
@@ -192,12 +192,8 @@ void TableSinkBase::BuildHdfsFileNames(
 }
 
 Status TableSinkBase::InitOutputPartition(RuntimeState* state,
-    const HdfsPartitionDescriptor& partition_descriptor, const TupleRow* row,
+    const HdfsPartitionDescriptor& partition_descriptor,
     OutputPartition* output_partition, bool empty_partition) {
-  // Build the unique name for this partition from the partition keys, e.g. "j=1/f=foo/"
-  // etc.
-  ConstructPartitionInfo(row, output_partition);
-
   BuildHdfsFileNames(partition_descriptor, output_partition);
 
   if (ShouldSkipStaging(state, output_partition)) {
@@ -366,16 +362,16 @@ Status TableSinkBase::CreateNewTmpFile(RuntimeState* state,
 }
 
 Status TableSinkBase::WriteRowsToPartition(
-    RuntimeState* state, RowBatch* batch, PartitionPair* partition_pair) {
+    RuntimeState* state, RowBatch* batch, OutputPartition* output_partition,
+    const std::vector<int32_t>& indices) {
   // The rows of this batch may span multiple files. We repeatedly pass the row batch to
   // the writer until it sets new_file to false, indicating that all rows have been
   // written. The writer tracks where it is in the batch when it returns with new_file
   // set.
   bool new_file;
   while (true) {
-    OutputPartition* output_partition = partition_pair->first.get();
     Status status =
-        output_partition->writer->AppendRows(batch, partition_pair->second, &new_file);
+        output_partition->writer->AppendRows(batch, indices, &new_file);
     if (!status.ok()) {
       // IMPALA-10607: Deletes partition file if staging is skipped when appending rows
       // fails. Otherwise, it leaves the file in un-finalized state.
@@ -390,7 +386,6 @@ Status TableSinkBase::WriteRowsToPartition(
     RETURN_IF_ERROR(FinalizePartitionFile(state, output_partition));
     RETURN_IF_ERROR(CreateNewTmpFile(state, output_partition));
   }
-  partition_pair->second.clear();
   return Status::OK();
 }
 
@@ -414,18 +409,26 @@ bool TableSinkBase::ShouldSkipStaging(RuntimeState* state, OutputPartition* part
 }
 
 Status TableSinkBase::FinalizePartitionFile(
-    RuntimeState* state, OutputPartition* partition) {
+    RuntimeState* state, OutputPartition* partition, bool is_delete,
+    DmlExecState* dml_exec_state) {
+  if (dml_exec_state == nullptr) dml_exec_state = state->dml_exec_state();
   if (partition->tmp_hdfs_file == nullptr && !is_overwrite()) return Status::OK();
   SCOPED_TIMER(ADD_TIMER(profile(), "FinalizePartitionFileTimer"));
 
   // OutputPartition writer could be nullptr if there is no row to output.
   if (partition->writer.get() != nullptr) {
     RETURN_IF_ERROR(partition->writer->Finalize());
-    state->dml_exec_state()->UpdatePartition(
+    dml_exec_state->UpdatePartition(
         partition->partition_name, partition->current_file_rows,
-        &partition->writer->stats());
-    state->dml_exec_state()->AddCreatedFile(*partition, IsIceberg(),
-        partition->writer->iceberg_file_stats());
+        &partition->writer->stats(), is_delete);
+    if (is_delete) {
+      DCHECK(IsIceberg());
+      dml_exec_state->AddCreatedDeleteFile(*partition,
+          partition->writer->iceberg_file_stats());
+    } else {
+      dml_exec_state->AddCreatedFile(*partition, IsIceberg(),
+          partition->writer->iceberg_file_stats());
+    }
   }
 
   RETURN_IF_ERROR(ClosePartitionFile(state, partition));
