@@ -39,8 +39,7 @@ namespace impala {
 
 /// Note: this function has a codegen'd version.  Changing this function requires
 /// corresponding changes to CodegenWriteSlot().
-inline bool TextConverter::WriteSlot(const SlotDescriptor* slot_desc,
-    const AuxColumnType* auxType, Tuple* tuple,
+inline bool TextConverter::WriteSlot(const SlotDescriptor* slot_desc, Tuple* tuple,
     const char* data, int len, bool copy_string, bool need_escape, MemPool* pool) {
   if ((len == 0 && !slot_desc->type().IsStringType()) || data == NULL) {
     tuple->SetNull(slot_desc->null_indicator_offset());
@@ -68,7 +67,7 @@ inline bool TextConverter::WriteSlot(const SlotDescriptor* slot_desc,
           !(len != 0 && (copy_string || need_escape));
 
       bool base64_decode = false;
-      if (auxType->IsBinaryStringSubtype()) {
+      if (type.IsBinaryType()) {
         base64_decode = true;
         reuse_data = false;
         int64_t out_len;
@@ -76,7 +75,8 @@ inline bool TextConverter::WriteSlot(const SlotDescriptor* slot_desc,
         buffer_len = out_len;
       }
 
-      StringValue str;
+      StringValue::SimpleString str;
+      str.ptr = nullptr;
       str.len = std::min(buffer_len, len);
       if (reuse_data) {
         str.ptr = const_cast<char*>(data);
@@ -92,9 +92,9 @@ inline bool TextConverter::WriteSlot(const SlotDescriptor* slot_desc,
         str.ptr = type.IsVarLenStringType() ?
             reinterpret_cast<char*>(pool->TryAllocateUnaligned(buffer_len)) :
             reinterpret_cast<char*>(slot);
-        if (UNLIKELY(str.ptr == NULL)) return false;
+        if (UNLIKELY(str.ptr == nullptr)) return false;
         if (base64_decode) {
-          int64_t out_len;
+          unsigned out_len;
           if(!Base64Decode(data, len, buffer_len, str.ptr, &out_len)) return false;
           DCHECK_LE(out_len, buffer_len);
           str.len = out_len;
@@ -108,6 +108,7 @@ inline bool TextConverter::WriteSlot(const SlotDescriptor* slot_desc,
           memcpy(str.ptr, data, str.len);
         }
       }
+      DCHECK_NE(str.ptr, nullptr);
 
       if (type.type == TYPE_CHAR) {
         StringValue::PadWithSpaces(str.ptr, buffer_len, str.len);
@@ -116,7 +117,7 @@ inline bool TextConverter::WriteSlot(const SlotDescriptor* slot_desc,
       // write back to the slot, if !IsVarLenStringType() we already wrote to the slot
       if (type.IsVarLenStringType()) {
         StringValue* str_slot = reinterpret_cast<StringValue*>(slot);
-        *str_slot = str;
+        str_slot->Assign(str.ptr, str.len);
       }
       break;
     }
