@@ -27,6 +27,7 @@
 
 #include "exprs/anyval-util.h"
 #include "exprs/scalar-expr.h"
+#include "gen-cpp/Metrics_types.h"
 #include "gutil/strings/charset.h"
 #include "gutil/strings/substitute.h"
 #include "runtime/string-value.inline.h"
@@ -42,6 +43,7 @@
 
 using namespace impala_udf;
 using std::bitset;
+using std::any_of;
 
 // NOTE: be careful not to use string::append.  It is not performant.
 namespace impala {
@@ -522,28 +524,30 @@ StringVal StringFunctions::Replace(FunctionContext* context, const StringVal& st
   // No match?  Skip everything.
   if (match_pos < 0) return str;
 
+  StringValue::SimpleString haystack_s = haystack.ToSimpleString();
+
   DCHECK_GT(pattern.len, 0);
-  DCHECK_GE(haystack.len, pattern.len);
+  DCHECK_GE(haystack_s.len, pattern.len);
   int buffer_space;
   const int delta = replace.len - pattern.len;
   // MAX_LENGTH is unsigned, so convert back to int to do correctly signed compare
   DCHECK_LE(delta, static_cast<int>(StringVal::MAX_LENGTH) - 1);
-  if ((delta > 0 && delta < 128) && haystack.len <= 128) {
+  if ((delta > 0 && delta < 128) && haystack_s.len <= 128) {
     // Quick estimate for potential matches - this heuristic is needed to win
     // over regexp_replace on expanding patterns.  128 is arbitrarily chosen so
     // we can't massively over-estimate the buffer size.
     int matches_possible = 0;
     char c = pattern.ptr[0];
-    for (int i = 0; i <= haystack.len - pattern.len; ++i) {
-      if (haystack.ptr[i] == c) ++matches_possible;
+    for (int i = 0; i <= haystack_s.len - pattern.len; ++i) {
+      if (haystack_s.ptr[i] == c) ++matches_possible;
     }
-    buffer_space = haystack.len + matches_possible * delta;
+    buffer_space = haystack_s.len + matches_possible * delta;
   } else {
     // Note - cannot overflow because pattern.len is at least one
     static_assert(StringVal::MAX_LENGTH - 1 + StringVal::MAX_LENGTH <=
         std::numeric_limits<decltype(buffer_space)>::max(),
         "Buffer space computation can overflow");
-    buffer_space = haystack.len + delta;
+    buffer_space = haystack_s.len + delta;
   }
 
   StringVal result(context, buffer_space);
@@ -552,10 +556,10 @@ StringVal StringFunctions::Replace(FunctionContext* context, const StringVal& st
 
   uint8_t* ptr = result.ptr;
   int consumed = 0;
-  while (match_pos + pattern.len <= haystack.len) {
+  while (match_pos + pattern.len <= haystack_s.len) {
     // Copy in original string
     const int unmatched_bytes = match_pos - consumed;
-    memcpy(ptr, &haystack.ptr[consumed], unmatched_bytes);
+    memcpy(ptr, &haystack_s.ptr[consumed], unmatched_bytes);
     DCHECK_LE(ptr - result.ptr + unmatched_bytes, buffer_space);
     ptr += unmatched_bytes;
 
@@ -577,7 +581,7 @@ StringVal StringFunctions::Replace(FunctionContext* context, const StringVal& st
     // If we had an enlarging pattern, we may need more space
     if (delta > 0) {
       const int bytes_produced = ptr - result.ptr;
-      const int bytes_remaining = haystack.len - consumed;
+      const int bytes_remaining = haystack_s.len - consumed;
       DCHECK_LE(bytes_produced, StringVal::MAX_LENGTH);
       DCHECK_LE(bytes_remaining, StringVal::MAX_LENGTH - 1);
       // Note: by above, cannot overflow
@@ -610,10 +614,10 @@ StringVal StringFunctions::Replace(FunctionContext* context, const StringVal& st
   }
 
   // Copy in remainder and re-adjust size
-  const int bytes_remaining = haystack.len - consumed;
+  const int bytes_remaining = haystack_s.len - consumed;
   result.len = ptr - result.ptr + bytes_remaining;
   DCHECK_LE(result.len, buffer_space);
-  memcpy(ptr, &haystack.ptr[consumed], bytes_remaining);
+  memcpy(ptr, &haystack_s.ptr[consumed], bytes_remaining);
 
   return result;
 }
@@ -688,35 +692,90 @@ StringVal StringFunctions::Translate(FunctionContext* context, const StringVal&
   return result;
 }
 
-void StringFunctions::TrimPrepare(
-    FunctionContext* context, FunctionContext::FunctionStateScope scope) {
+void StringFunctions::TrimContext::Reset(const StringVal& chars_to_trim) {
+  single_byte_chars_.reset();
+  double_byte_chars_.clear();
+  triple_byte_chars_.clear();
+  quadruple_byte_chars_.clear();
+
+  if (!utf8_mode_) {
+    for (size_t i = 0; i < chars_to_trim.len; ++i) {
+      single_byte_chars_.set(chars_to_trim.ptr[i], true);
+    }
+    return;
+  }
+
+  for (size_t i = 0, char_size = 0; i < chars_to_trim.len; i += char_size) {
+    char_size = BitUtil::NumBytesInUtf8Encoding(chars_to_trim.ptr[i]);
+
+    // If the remaining number of bytes does not match the number of bytes specified by
+    // the UTF-8 character, we may have encountered an illegal UTF-8 character.
+    // In order to prevent subsequent data access from going out of bounds, restrictions
+    // are placed here to ensure that accessing pointers to multi-byte characters is
+    // always safe.
+    if (UNLIKELY(i + char_size > chars_to_trim.len)) {
+      char_size = chars_to_trim.len - i;
+    }
+
+    switch (char_size) {
+      case 1: single_byte_chars_.set(chars_to_trim.ptr[i], true); break;
+      case 2: double_byte_chars_.push_back(&chars_to_trim.ptr[i]); break;
+      case 3: triple_byte_chars_.push_back(&chars_to_trim.ptr[i]); break;
+      case 4: quadruple_byte_chars_.push_back(&chars_to_trim.ptr[i]); break;
+      default: DCHECK(false); break;
+    }
+  }
+}
+
+bool StringFunctions::TrimContext::Contains(const uint8_t* utf8_char, int len) const {
+  auto eq = [&](const uint8_t* c){ return memcmp(c, utf8_char, len) == 0; };
+  switch (len) {
+    case 1: return single_byte_chars_.test(*utf8_char);
+    case 2: return any_of(double_byte_chars_.begin(), double_byte_chars_.end(), eq);
+    case 3: return any_of(triple_byte_chars_.begin(), triple_byte_chars_.end(), eq);
+    case 4: return any_of(quadruple_byte_chars_.begin(), quadruple_byte_chars_.end(), eq);
+    default: DCHECK(false); return false;
+  }
+}
+
+void StringFunctions::TrimPrepare(FunctionContext* context,
+    FunctionContext::FunctionStateScope scope) {
+  bool utf8_mode = context->impl()->GetConstFnAttr(FunctionContextImpl::UTF8_MODE);
+  DoTrimPrepare(context, scope, utf8_mode);
+}
+
+void StringFunctions::Utf8TrimPrepare(FunctionContext* context,
+    FunctionContext::FunctionStateScope scope) {
+  DoTrimPrepare(context, scope, true /* utf8_mode */);
+}
+
+void StringFunctions::DoTrimPrepare(FunctionContext* context,
+    FunctionContext::FunctionStateScope scope, bool utf8_mode) {
   if (scope != FunctionContext::THREAD_LOCAL) return;
-  // Create a bitset to hold the unique characters to trim.
-  bitset<256>* unique_chars = new bitset<256>();
-  context->SetFunctionState(scope, unique_chars);
+  TrimContext* trim_ctx = new TrimContext(utf8_mode);
+  context->SetFunctionState(scope, trim_ctx);
+
   // If the caller didn't specify the set of characters to trim, it means
   // that we're only trimming whitespace. Return early in that case.
   // There can be either 1 or 2 arguments.
   DCHECK(context->GetNumArgs() == 1 || context->GetNumArgs() == 2);
   if (context->GetNumArgs() == 1) {
-    unique_chars->set(static_cast<int>(' '), true);
+    trim_ctx->Reset(StringVal(" "));
     return;
   }
   if (!context->IsArgConstant(1)) return;
   DCHECK_EQ(context->GetArgType(1)->type, FunctionContext::TYPE_STRING);
   StringVal* chars_to_trim = reinterpret_cast<StringVal*>(context->GetConstantArg(1));
   if (chars_to_trim->is_null) return; // We shouldn't peek into Null StringVals
-  for (int32_t i = 0; i < chars_to_trim->len; ++i) {
-    unique_chars->set(static_cast<int>(chars_to_trim->ptr[i]), true);
-  }
+  trim_ctx->Reset(*chars_to_trim);
 }
 
 void StringFunctions::TrimClose(
     FunctionContext* context, FunctionContext::FunctionStateScope scope) {
   if (scope != FunctionContext::THREAD_LOCAL) return;
-  bitset<256>* unique_chars = reinterpret_cast<bitset<256>*>(
-      context->GetFunctionState(scope));
-  delete unique_chars;
+  TrimContext* trim_ctx =
+      reinterpret_cast<TrimContext*>(context->GetFunctionState(scope));
+  delete trim_ctx;
   context->SetFunctionState(scope, nullptr);
 }
 
@@ -724,36 +783,69 @@ template <StringFunctions::TrimPosition D, bool IS_IMPLICIT_WHITESPACE>
 StringVal StringFunctions::DoTrimString(FunctionContext* ctx,
     const StringVal& str, const StringVal& chars_to_trim) {
   if (str.is_null) return StringVal::null();
-  bitset<256>* unique_chars = reinterpret_cast<bitset<256>*>(
+  TrimContext* trim_ctx = reinterpret_cast<TrimContext*>(
       ctx->GetFunctionState(FunctionContext::THREAD_LOCAL));
-  // When 'chars_to_trim' is unique for each element (e.g. when 'chars_to_trim'
-  // is each element of a table column), we need to prepare a bitset of unique
-  // characters here instead of using the bitset from function context.
+
+  // When 'chars_to_trim' is not a constant, we need to reset TrimContext with new
+  // 'chars_to_trim'.
   if (!IS_IMPLICIT_WHITESPACE && !ctx->IsArgConstant(1)) {
     if (chars_to_trim.is_null) return str;
-    unique_chars->reset();
-    for (int32_t i = 0; i < chars_to_trim.len; ++i) {
-      unique_chars->set(static_cast<int>(chars_to_trim.ptr[i]), true);
-    }
+    trim_ctx->Reset(chars_to_trim);
   }
-  // Find new starting position.
+
+  // When dealing with UTF-8 characters in UTF-8 mode, use DoUtf8TrimString().
+  if (trim_ctx->utf8_mode()) {
+    return DoUtf8TrimString<D>(str, *trim_ctx);
+  }
+
+  // Otherwise, we continue to maintain the old behavior.
   int32_t begin = 0;
   int32_t end = str.len - 1;
-  if (D == LEADING || D == BOTH) {
-    while (begin < str.len &&
-        unique_chars->test(static_cast<int>(str.ptr[begin]))) {
+  // Find new starting position.
+  if constexpr (D == LEADING || D == BOTH) {
+    while (begin < str.len && trim_ctx->Contains(str.ptr[begin])) {
       ++begin;
     }
   }
   // Find new ending position.
-  if (D == TRAILING || D == BOTH) {
-    while (end >= begin && unique_chars->test(static_cast<int>(str.ptr[end]))) {
+  if constexpr (D == TRAILING || D == BOTH) {
+    while (end >= begin && trim_ctx->Contains(str.ptr[end])) {
       --end;
     }
   }
   return StringVal(str.ptr + begin, end - begin + 1);
 }
 
+template <StringFunctions::TrimPosition D>
+StringVal StringFunctions::DoUtf8TrimString(const StringVal& str,
+    const TrimContext& trim_ctx) {
+  if (UNLIKELY(str.len == 0)) return str;
+
+  const uint8_t* begin = str.ptr;
+  const uint8_t* end = begin + str.len;
+  // Find new starting position.
+  if constexpr (D == LEADING || D == BOTH) {
+    while (begin < end) {
+      size_t char_size = BitUtil::NumBytesInUtf8Encoding(*begin);
+      if (UNLIKELY(begin + char_size > end)) char_size = end - begin;
+      if (!trim_ctx.Contains(begin, char_size)) break;
+      begin += char_size;
+    }
+  }
+  // Find new ending position.
+  if constexpr (D == TRAILING || D == BOTH) {
+    while (begin < end) {
+      int char_index = FindUtf8PosBackward(begin, end - begin, 0);
+      DCHECK_NE(char_index, -1);
+      const uint8_t* char_begin = begin + char_index;
+      if (!trim_ctx.Contains(char_begin, end - char_begin)) break;
+      end = char_begin;
+    }
+  }
+
+  return StringVal(const_cast<uint8_t*>(begin), end - begin);
+}
+
 StringVal StringFunctions::Trim(FunctionContext* context, const StringVal& str) {
   return DoTrimString<BOTH, true>(context, str, StringVal(" "));
 }
@@ -803,7 +895,9 @@ IntVal StringFunctions::Instr(FunctionContext* context, const StringVal& str,
 
   bool utf8_mode = context->impl()->GetConstFnAttr(FunctionContextImpl::UTF8_MODE);
   StringValue haystack = StringValue::FromStringVal(str);
+  StringValue::SimpleString haystack_s = haystack.ToSimpleString();
   StringValue needle = StringValue::FromStringVal(substr);
+  StringValue::SimpleString needle_s = needle.ToSimpleString();
   StringSearch search(&needle);
   int match_pos = -1;
   if (start_position.val > 0) {
@@ -812,9 +906,9 @@ IntVal StringFunctions::Instr(FunctionContext* context, const StringVal& str,
     if (utf8_mode) {
       search_start_pos = FindUtf8PosForward(str.ptr, str.len, search_start_pos);
     }
-    if (search_start_pos >= haystack.len) return IntVal(0);
+    if (search_start_pos >= haystack_s.len) return IntVal(0);
     for (int match_num = 0; match_num < occurrence.val; ++match_num) {
-      DCHECK_LE(search_start_pos, haystack.len);
+      DCHECK_LE(search_start_pos, haystack_s.len);
       StringValue haystack_substring = haystack.Substring(search_start_pos);
       int match_pos_in_substring = search.Search(&haystack_substring);
       if (match_pos_in_substring < 0) return IntVal(0);
@@ -825,17 +919,17 @@ IntVal StringFunctions::Instr(FunctionContext* context, const StringVal& str,
     // A negative starting position indicates searching from the right.
     int search_start_pos = utf8_mode ?
         FindUtf8PosBackward(str.ptr, str.len, -start_position.val - 1) :
-        haystack.len + start_position.val;
+        haystack_s.len + start_position.val;
     // The needle must fit between search_start_pos and the end of the string
-    if (search_start_pos + needle.len > haystack.len) {
-      search_start_pos = haystack.len - needle.len;
+    if (search_start_pos + needle_s.len > haystack_s.len) {
+      search_start_pos = haystack_s.len - needle_s.len;
     }
     if (search_start_pos < 0) return IntVal(0);
     for (int match_num = 0; match_num < occurrence.val; ++match_num) {
-      DCHECK_GE(search_start_pos + needle.len, 0);
-      DCHECK_LE(search_start_pos + needle.len, haystack.len);
+      DCHECK_GE(search_start_pos + needle_s.len, 0);
+      DCHECK_LE(search_start_pos + needle_s.len, haystack_s.len);
       StringValue haystack_substring =
-          haystack.Substring(0, search_start_pos + needle.len);
+          haystack.Substring(0, search_start_pos + needle_s.len);
       match_pos = search.RSearch(&haystack_substring);
       if (match_pos < 0) return IntVal(0);
       search_start_pos = match_pos - 1;
@@ -1435,7 +1529,7 @@ StringVal StringFunctions::Base64Encode(FunctionContext* ctx, const StringVal& s
   }
   StringVal result(ctx, out_max);
   if (UNLIKELY(result.is_null)) return result;
-  int64_t out_len = 0;
+  unsigned out_len = 0;
   if (UNLIKELY(!impala::Base64Encode(
           reinterpret_cast<const char*>(str.ptr), str.len,
           out_max, reinterpret_cast<char*>(result.ptr), &out_len))) {
@@ -1464,7 +1558,7 @@ StringVal StringFunctions::Base64Decode(FunctionContext* ctx, const StringVal& s
   }
   StringVal result(ctx, out_max);
   if (UNLIKELY(result.is_null)) return result;
-  int64_t out_len = 0;
+  unsigned out_len = 0;
   if (UNLIKELY(!impala::Base64Decode(
           reinterpret_cast<const char*>(str.ptr), static_cast<int64_t>(str.len),
           out_max, reinterpret_cast<char*>(result.ptr), &out_len))) {
@@ -1795,4 +1889,42 @@ IntVal StringFunctions::DamerauLevenshtein(
   ctx->Free(reinterpret_cast<uint8_t*>(rows));
   return IntVal(result);
 }
-}
\ No newline at end of file
+
+template <typename T>
+static StringVal prettyPrint(FunctionContext* context, const T& int_val,
+    const TUnit::type& unit) {
+  if (int_val.is_null) {
+    return StringVal::null();
+  }
+
+  const string& fmt_str = PrettyPrinter::Print(int_val.val, unit);
+
+  StringVal result(context, fmt_str.size());
+  if (UNLIKELY(result.is_null)) return StringVal::null();
+  uint8_t* ptr = result.ptr;
+  memcpy(ptr, fmt_str.c_str(), fmt_str.size());
+
+  return result;
+}
+
+StringVal StringFunctions::PrettyPrintMemory(FunctionContext* context,
+    const BigIntVal& bytes) {
+  return prettyPrint(context, bytes, TUnit::BYTES);
+}
+
+StringVal StringFunctions::PrettyPrintMemory(FunctionContext* context,
+    const IntVal& bytes) {
+  return prettyPrint(context, bytes, TUnit::BYTES);
+}
+
+StringVal StringFunctions::PrettyPrintMemory(FunctionContext* context,
+    const SmallIntVal& bytes) {
+  return prettyPrint(context, bytes, TUnit::BYTES);
+}
+
+StringVal StringFunctions::PrettyPrintMemory(FunctionContext* context,
+    const TinyIntVal& bytes) {
+  return prettyPrint(context, bytes, TUnit::BYTES);
+}
+
+}
