@@ -19,8 +19,10 @@
 
 #include <cerrno>
 #include <iomanip>
+#include <list>
 #include <sstream>
 #include <unordered_set>
+#include <utility>
 
 #include <thrift/protocol/TDebugProtocol.h>
 #include <boost/algorithm/string/join.hpp>
@@ -423,13 +425,6 @@ void Coordinator::AddFilterSource(const FragmentExecParamsPB& src_fragment_param
     int num_instances, int num_backends, const TRuntimeFilterDesc& filter,
     int join_node_id) {
   FilterState* f = filter_routing_table_->GetOrCreateFilterState(filter);
-  // Set the 'pending_count_' to zero to indicate that for a filter with
-  // local-only targets the coordinator does not expect to receive any filter
-  // updates. We expect to receive a single aggregated filter from each backend
-  // for partitioned joins.
-  int pending_count = filter.is_broadcast_join
-      ? (filter.has_remote_targets ? 1 : 0) : num_backends;
-  f->set_pending_count(pending_count);
 
   // Determine which instances will produce the filters.
   // TODO: IMPALA-9333: having a shared RuntimeFilterBank between all fragments on
@@ -457,10 +452,60 @@ void Coordinator::AddFilterSource(const FragmentExecParamsPB& src_fragment_param
     random_shuffle(src_idxs.begin(), src_idxs.end());
     src_idxs.resize(MAX_BROADCAST_FILTER_PRODUCERS);
   }
-  for (int src_idx : src_idxs) {
+
+  bool has_intermediate_aggregator = src_fragment_params.has_filter_agg_info()
+      && !filter.has_local_targets && !filter.is_broadcast_join
+      && filter.type == TRuntimeFilterType::BLOOM;
+
+  if (has_intermediate_aggregator) {
+    const RuntimeFilterAggregatorInfoPB& agg_info = src_fragment_params.filter_agg_info();
+    // Set the 'pending_count_' to num_aggregators from RuntimeFilterAggregatorInfoPB.
+    int num_agg = agg_info.num_aggregators();
+    DCHECK_EQ(
+        src_fragment_params.instances_size(), agg_info.aggregator_idx_to_report_size());
+    DCHECK_EQ(num_agg, agg_info.aggregator_krpc_addresses_size());
+    DCHECK_EQ(num_agg, agg_info.aggregator_krpc_backends_size());
+    DCHECK_EQ(num_agg, agg_info.num_reporter_per_aggregator_size());
+    for (int i = 0; i < num_agg; i++) {
+      VLOG(2) << "Filter " << filter.filter_id << " backend aggregator " << (i + 1)
+              << " krpc_address=" << agg_info.aggregator_krpc_addresses(i)
+              << " krpc_backend=" << agg_info.aggregator_krpc_backends(i);
+    }
+    f->set_pending_count(num_agg);
+  } else {
+    // Set the 'pending_count_' to zero to indicate that for a filter with
+    // local-only targets the coordinator does not expect to receive any filter
+    // updates. We expect to receive a single aggregated filter from each backend
+    // for partitioned joins.
+    int pending_count =
+        filter.is_broadcast_join ? (filter.has_remote_targets ? 1 : 0) : num_backends;
+    f->set_pending_count(pending_count);
+  }
+
+  for (int i = 0; i < src_idxs.size(); i++) {
+    int src_idx = src_idxs[i];
     TRuntimeFilterSource filter_src;
     filter_src.src_node_id = join_node_id;
     filter_src.filter_id = filter.filter_id;
+    if (has_intermediate_aggregator) {
+      // Find target aggregator for fragment instance i.
+      const RuntimeFilterAggregatorInfoPB& agg_info =
+          src_fragment_params.filter_agg_info();
+      int agg_idx = agg_info.aggregator_idx_to_report(i);
+      string agg_hostname = agg_info.aggregator_krpc_addresses(agg_idx).hostname();
+      int num_reporter = agg_info.num_reporter_per_aggregator(agg_idx);
+      TNetworkAddress agg_address =
+          FromNetworkAddressPB(agg_info.aggregator_krpc_backends(agg_idx));
+
+      // Populate TRuntimeFilterAggDesc for fragment instance i.
+      TRuntimeFilterAggDesc agg_desc;
+      agg_desc.__set_krpc_hostname(agg_hostname);
+      agg_desc.__set_krpc_address(agg_address);
+      agg_desc.__set_num_reporting_hosts(num_reporter);
+      filter_src.__set_aggregator_desc(agg_desc);
+      VLOG(3) << "Instance " << src_fragment_params.instances(i)
+              << " report to backend aggregator " << (agg_idx + 1);
+    }
     filter_routing_table_->finstance_filters_produced[src_idx].emplace_back(
         filter_src);
   }
@@ -625,14 +670,14 @@ string Coordinator::FilterDebugString() {
     row.push_back(join(partition_filter, ", "));
 
     if (filter_mode_ == TRuntimeFilterMode::GLOBAL) {
-      int pending_count = state.completion_time() != 0L ? 0 : state.pending_count();
+      int pending_count = state.has_completion_time() ? 0 : state.pending_count();
       row.push_back(Substitute("$0 ($1)", pending_count, state.num_producers()));
-      if (state.first_arrival_time() == 0L) {
+      if (!state.has_first_arrival_time()) {
         row.push_back("N/A");
       } else {
         row.push_back(PrettyPrinter::Print(state.first_arrival_time(), TUnit::TIME_NS));
       }
-      if (state.completion_time() == 0L) {
+      if (!state.has_completion_time()) {
         row.push_back("N/A");
       } else {
         row.push_back(PrettyPrinter::Print(state.completion_time(), TUnit::TIME_NS));
@@ -1169,7 +1214,7 @@ Status Coordinator::UpdateBlacklistWithAuxErrorInfo(
   // that contains a valid RPCErrorInfoPB object.
   for (const auto& aux_error : *aux_error_info) {
     if (aux_error.has_rpc_error_info()) {
-      RPCErrorInfoPB rpc_error_info = aux_error.rpc_error_info();
+      const RPCErrorInfoPB& rpc_error_info = aux_error.rpc_error_info();
       DCHECK(rpc_error_info.has_dest_node());
       DCHECK(rpc_error_info.has_posix_error_code());
       const NetworkAddressPB& dest_node = rpc_error_info.dest_node();
@@ -1462,6 +1507,20 @@ vector<NetworkAddressPB> Coordinator::GetActiveBackends(
   return result;
 }
 
+list<pair<NetworkAddressPB, Coordinator::ResourceUtilization>>
+    Coordinator::BackendResourceUtilization() {
+  DCHECK(exec_rpcs_complete_.Load()) << "Exec() must be called first.";
+  list<pair<NetworkAddressPB, Coordinator::ResourceUtilization>> result;
+
+  lock_guard<SpinLock> l(backend_states_init_lock_);
+  for (BackendState* backend_state : backend_states_) {
+    result.push_back(make_pair(backend_state->impalad_address(),
+        backend_state->GetResourceUtilization()));
+  }
+
+  return result;
+}
+
 void Coordinator::UpdateFilter(const UpdateFilterParamsPB& params, RpcContext* context) {
   VLOG(2) << "Coordinator::UpdateFilter(filter_id=" << params.filter_id() << ")";
   shared_lock<shared_mutex> lock(filter_routing_table_->lock);
@@ -1478,7 +1537,7 @@ void Coordinator::UpdateFilter(const UpdateFilterParamsPB& params, RpcContext* c
   std::unordered_set<int> target_fragment_idxs;
   if (!IsExecuting()) {
     LOG(INFO) << "Filter update received for non-executing query with id: "
-        << query_id();
+        << PrintId(query_id());
     return;
   }
   auto it = filter_routing_table_->id_to_filter.find(params.filter_id());
@@ -1571,8 +1630,8 @@ void Coordinator::FilterState::ApplyUpdate(
     const UpdateFilterParamsPB& params, Coordinator* coord, RpcContext* context) {
   DCHECK(enabled());
   DCHECK_GT(pending_count_, 0);
-  DCHECK_EQ(completion_time_, 0L);
-  if (first_arrival_time_ == 0L) {
+  DCHECK(!has_completion_time());
+  if (!has_first_arrival_time()) {
     first_arrival_time_ = coord->query_events_->ElapsedTime();
   }
 
