@@ -17,9 +17,11 @@
 
 #pragma once
 
+#include <list>
 #include <memory>
 #include <string>
 #include <vector>
+#include <utility>
 #include <boost/unordered_map.hpp>
 #include <rapidjson/document.h>
 
@@ -264,6 +266,10 @@ class Coordinator { // NOLINT: The member variables could be re-ordered to save
   /// latest status reports received from those backends).
   ResourceUtilization ComputeQueryResourceUtilization();
 
+  /// Return the backends and their associated resource utilization.
+  std::list<std::pair<NetworkAddressPB, ResourceUtilization>>
+      BackendResourceUtilization();
+
   /// Return the backends in 'candidates' that still have at least one fragment instance
   /// executing on them. The returned backends may not be in the same order as the input.
   std::vector<NetworkAddressPB> GetActiveBackends(
