@@ -28,6 +28,7 @@
 #include "common/logging.h"
 #include "common/object-pool.h"
 #include "exec/kudu/kudu-util.h"
+#include "exprs/ai-functions.h"
 #include "kudu/rpc/service_if.h"
 #include "rpc/rpc-mgr.h"
 #include "runtime/bufferpool/buffer-pool.h"
@@ -43,6 +44,7 @@
 #include "runtime/query-exec-mgr.h"
 #include "runtime/thread-resource-mgr.h"
 #include "runtime/tmp-file-mgr.h"
+#include "runtime/tuple-cache-mgr.h"
 #include "scheduling/admission-controller.h"
 #include "scheduling/cluster-membership-mgr.h"
 #include "scheduling/request-pool-service.h"
@@ -84,8 +86,6 @@ DEFINE_bool(enable_webserver, true, "If true, debug webserver is enabled");
 DEFINE_bool(ping_expose_webserver_url, true,
     "If true, debug webserver url is exposed via PingImpalaService/PingImpalaHS2Service "
     "RPC calls");
-DEFINE_string(state_store_host, "localhost",
-    "hostname where StatestoreService is running");
 DEFINE_int32(state_store_subscriber_port, 23000,
     "port where StatestoreSubscriberService should be exported");
 DEFINE_int32(num_hdfs_worker_threads, 16,
@@ -123,8 +123,11 @@ DEFINE_int32(local_catalog_max_fetch_retries, 40,
     "If --use_local_catalog is enabled, configures the maximum number of times "
     "the frontend retries when fetching a metadata object from the impalad "
     "coordinator's local catalog cache.");
+DEFINE_int32(local_catalog_cache_concurrency_level, 4,
+    "If --use_local_catalog is enabled, configures the local cache's concurrency "
+    "level to avoid lock contention, the default value 4 is consistent with the "
+    "default value of the original cache.");
 
-DECLARE_int32(state_store_port);
 DECLARE_int32(num_threads_per_core);
 DECLARE_int32(num_cores);
 DECLARE_int32(krpc_port);
@@ -142,9 +145,16 @@ DECLARE_string(admission_service_host);
 DECLARE_int32(admission_service_port);
 DECLARE_string(catalog_service_host);
 DECLARE_int32(catalog_service_port);
+DECLARE_string(state_store_host);
+DECLARE_int32(state_store_port);
+DECLARE_string(state_store_2_host);
+DECLARE_int32(state_store_2_port);
 
+DECLARE_string(debug_actions);
 DECLARE_string(ssl_client_ca_certificate);
 
+DECLARE_string(ai_api_key_jceks_secret);
+
 DEFINE_int32(backend_client_connection_num_retries, 3, "Retry backend connections.");
 // When network is unstable, TCP will retry and sending could take longer time.
 // Choose 5 minutes as default timeout because we don't want RPC timeout be triggered
@@ -155,8 +165,13 @@ DEFINE_int32(backend_client_rpc_timeout_ms, 300000, "(Advanced) The underlying "
 
 DEFINE_int32(catalog_client_connection_num_retries, 10, "The number of times connections "
     "or RPCs to the catalog should be retried.");
-DEFINE_int32(catalog_client_rpc_timeout_ms, 0, "(Advanced) The underlying TSocket "
-    "send/recv timeout in milliseconds for a catalog client RPC.");
+DEFINE_int32(catalog_client_rpc_timeout_ms, 36000000, "(Advanced) The underlying TSocket "
+    "send/recv timeout in milliseconds for a catalog client RPC. The default is 10 hours."
+    " Operations take longer than this are usually abnormal and hanging.");
+DEFINE_int32(catalog_lightweight_rpc_timeout_ms, 1800000, "(Advanced) The underlying "
+    "TSocket send/recv timeout in milliseconds for a lightweight catalog RPC which "
+    "shouldn't take long in catalogd, e.g. fetching db/table list. The default is 30 "
+    "minutes which is long enough to tolerate TCP timeout due to retransmission.");
 DEFINE_int32(catalog_client_rpc_retry_interval_ms, 3000, "(Advanced) The time to wait "
     "before retrying when the catalog RPC client fails to connect to catalogd or when "
     "RPCs to the catalogd fail.");
@@ -204,10 +219,12 @@ ExecEnv* ExecEnv::exec_env_ = nullptr;
 
 ExecEnv::ExecEnv(bool external_fe)
   : ExecEnv(FLAGS_krpc_port, FLAGS_state_store_subscriber_port, FLAGS_webserver_port,
-        FLAGS_state_store_host, FLAGS_state_store_port, external_fe) {}
+      FLAGS_state_store_host, FLAGS_state_store_port, FLAGS_state_store_2_host,
+      FLAGS_state_store_2_port, external_fe) {}
 
 ExecEnv::ExecEnv(int krpc_port, int subscriber_port, int webserver_port,
-    const string& statestore_host, int statestore_port, bool external_fe)
+    const string& statestore_host, int statestore_port,
+    const std::string& statestore2_host, int statestore2_port, bool external_fe)
   : obj_pool_(new ObjectPool),
     metrics_(new MetricGroup("impala-metrics")),
     // Create the CatalogServiceClientCache with num_retries = 1 and wait_ms = 0.
@@ -217,6 +234,10 @@ ExecEnv::ExecEnv(int krpc_port, int subscriber_port, int webserver_port,
     catalogd_client_cache_(new CatalogServiceClientCache(1, 0,
         FLAGS_catalog_client_rpc_timeout_ms, FLAGS_catalog_client_rpc_timeout_ms, "",
         !FLAGS_ssl_client_ca_certificate.empty())),
+    catalogd_lightweight_req_client_cache_(new CatalogServiceClientCache(1, 0,
+        FLAGS_catalog_lightweight_rpc_timeout_ms,
+        FLAGS_catalog_lightweight_rpc_timeout_ms, "",
+        !FLAGS_ssl_client_ca_certificate.empty())),
     htable_factory_(new HBaseTableFactory()),
     disk_io_mgr_(new io::DiskIoMgr()),
     webserver_(new Webserver(FLAGS_webserver_interface, webserver_port, metrics_.get())),
@@ -226,6 +247,7 @@ ExecEnv::ExecEnv(int krpc_port, int subscriber_port, int webserver_port,
     frontend_(external_fe ? nullptr : new Frontend()),
     async_rpc_pool_(new CallableThreadPool("rpc-pool", "async-rpc-sender", 8, 10000)),
     query_exec_mgr_(new QueryExecMgr()),
+    tuple_cache_mgr_(new TupleCacheMgr(metrics_.get())),
     rpc_metrics_(metrics_->GetOrCreateChildGroup("rpc")),
     enable_webserver_(FLAGS_enable_webserver && webserver_port > 0),
     external_fe_(external_fe),
@@ -247,6 +269,8 @@ ExecEnv::ExecEnv(int krpc_port, int subscriber_port, int webserver_port,
       MakeNetworkAddress(FLAGS_hostname, subscriber_port);
   TNetworkAddress statestore_address =
       MakeNetworkAddress(statestore_host, statestore_port);
+  TNetworkAddress statestore2_address =
+      MakeNetworkAddress(statestore2_host, statestore2_port);
 
   catalogd_address_ = std::make_shared<const TNetworkAddress>(
       MakeNetworkAddress(FLAGS_catalog_service_host, FLAGS_catalog_service_port));
@@ -262,7 +286,7 @@ ExecEnv::ExecEnv(int krpc_port, int subscriber_port, int webserver_port,
   // Set StatestoreSubscriber::subscriber_id as hostname + krpc_port.
   statestore_subscriber_.reset(new StatestoreSubscriber(
       Substitute("impalad@$0:$1", FLAGS_hostname, FLAGS_krpc_port), subscriber_address,
-      statestore_address, metrics_.get(), subscriber_type));
+      statestore_address, statestore2_address, metrics_.get(), subscriber_type));
 
   if (FLAGS_is_coordinator) {
     hdfs_op_thread_pool_.reset(
@@ -371,6 +395,9 @@ Status ExecEnv::Init() {
     LOG(INFO) << "CodeGen Cache is disabled.";
   }
 
+  // Initialize the tuple cache
+  RETURN_IF_ERROR(tuple_cache_mgr_->Init());
+
   LOG(INFO) << "Admit memory limit: "
             << PrettyPrinter::Print(admit_mem_limit_, TUnit::BYTES);
 
@@ -419,6 +446,8 @@ Status ExecEnv::Init() {
     RETURN_IF_ERROR(metrics_webserver_->Start());
   }
   catalogd_client_cache_->InitMetrics(metrics_.get(), "catalog.server");
+  catalogd_lightweight_req_client_cache_->InitMetrics(
+      metrics_.get(), "catalog.server", "for-lightweight-rpc");
   RETURN_IF_ERROR(RegisterMemoryMetrics(
       metrics_.get(), true, buffer_reservation_.get(), buffer_pool_.get()));
   // Initialize impalad metrics
@@ -491,6 +520,16 @@ Status ExecEnv::Init() {
 
   RETURN_IF_ERROR(admission_controller_->Init());
   RETURN_IF_ERROR(InitHadoopConfig());
+
+  // If 'ai_api_key_jceks_secret' is set then extract the api_key and populate
+  // AIFunctions::ai_api_key_
+  if (frontend_ != nullptr && FLAGS_ai_api_key_jceks_secret != "") {
+    string api_key;
+    RETURN_IF_ERROR(
+        frontend_->GetSecretFromKeyStore(FLAGS_ai_api_key_jceks_secret, &api_key));
+    AiFunctions::set_api_key(api_key);
+  }
+
   return Status::OK();
 }
 
@@ -628,7 +667,7 @@ void ExecEnv::InitSystemStateInfo() {
   system_state_info_.reset(new SystemStateInfo());
   PeriodicCounterUpdater::RegisterUpdateFunction([s = system_state_info_.get()]() {
     s->CaptureSystemStateSnapshot();
-  });
+  }, true);
 }
 
 Status ExecEnv::GetKuduClient(const vector<string>& master_addresses,
@@ -703,6 +742,8 @@ void ExecEnv::UpdateActiveCatalogd(bool is_registration_reply,
   bool is_matching = (catalogd_registration.address.port == catalogd_address_->port
       && catalogd_registration.address.hostname == catalogd_address_->hostname);
   if (!is_matching) {
+    RETURN_VOID_IF_ERROR(
+        DebugAction(FLAGS_debug_actions, "IGNORE_NEW_ACTIVE_CATALOGD_ADDR"));
     LOG(INFO) << "The address of Catalog service is changed from "
               << TNetworkAddressToString(*catalogd_address_.get())
               << " to " << TNetworkAddressToString(catalogd_registration.address);
