@@ -70,6 +70,7 @@ class StatestoreSubscriber;
 class SystemStateInfo;
 class ThreadResourceMgr;
 class TmpFileMgr;
+class TupleCacheMgr;
 class Webserver;
 class CodeGenCache;
 class TCatalogRegistration;
@@ -92,7 +93,9 @@ class ExecEnv {
   ExecEnv(bool external_fe = false);
 
   ExecEnv(int krpc_port, int subscriber_port, int webserver_port,
-      const std::string& statestore_host, int statestore_port, bool external_fe = false);
+      const std::string& statestore_host, int statestore_port,
+      const std::string& statestore2_host = "", int statestore2_port = 0,
+      bool external_fe = false);
 
   /// Returns the most recently created exec env instance. In a normal impalad, this is
   /// the only instance. In test setups with multiple ExecEnv's per process,
@@ -125,6 +128,9 @@ class ExecEnv {
   CatalogServiceClientCache* catalogd_client_cache() {
     return catalogd_client_cache_.get();
   }
+  CatalogServiceClientCache* catalogd_lightweight_req_client_cache() {
+    return catalogd_lightweight_req_client_cache_.get();
+  }
   HBaseTableFactory* htable_factory() { return htable_factory_.get(); }
   io::DiskIoMgr* disk_io_mgr() { return disk_io_mgr_.get(); }
   Webserver* webserver() { return webserver_.get(); }
@@ -158,6 +164,8 @@ class ExecEnv {
   CodeGenCache* codegen_cache() const { return codegen_cache_.get(); }
   bool codegen_cache_enabled() const { return codegen_cache_ != nullptr; }
 
+  TupleCacheMgr* tuple_cache_mgr() const { return tuple_cache_mgr_.get(); }
+
   const TNetworkAddress& configured_backend_address() const {
     return configured_backend_address_;
   }
@@ -230,6 +238,7 @@ class ExecEnv {
   boost::scoped_ptr<AdmissionController> admission_controller_;
   boost::scoped_ptr<StatestoreSubscriber> statestore_subscriber_;
   boost::scoped_ptr<CatalogServiceClientCache> catalogd_client_cache_;
+  boost::scoped_ptr<CatalogServiceClientCache> catalogd_lightweight_req_client_cache_;
   boost::scoped_ptr<HBaseTableFactory> htable_factory_;
   boost::scoped_ptr<io::DiskIoMgr> disk_io_mgr_;
   boost::scoped_ptr<Webserver> webserver_;
@@ -264,6 +273,9 @@ class ExecEnv {
   /// Singleton cache for codegen functions.
   boost::scoped_ptr<CodeGenCache> codegen_cache_;
 
+  /// Singleton cache for tuple caching
+  boost::scoped_ptr<TupleCacheMgr> tuple_cache_mgr_;
+
   /// Not owned by this class
   ImpalaServer* impala_server_ = nullptr;
   MetricGroup* rpc_metrics_ = nullptr;
