@@ -23,8 +23,17 @@
 #include <string.h>
 #include <unistd.h>
 #include <sstream>
+#include <fstream>
+#include <unordered_map>
+#include <unordered_set>
+#include <sys/stat.h>
 
 #include <glog/logging.h>
+#include <boost/archive/binary_oarchive.hpp>
+#include <boost/archive/binary_iarchive.hpp>
+#include <boost/serialization/access.hpp>
+#include <boost/serialization/vector.hpp>
+#include <boost/thread/pthread/shared_mutex.hpp>
 
 #include "common/compiler-util.h"
 #include "exec/kudu/kudu-util.h"
@@ -66,6 +75,7 @@ using kudu::Slice;
 using kudu::WritableFile;
 using strings::SkipEmpty;
 using strings::Split;
+using boost::shared_mutex;
 
 #ifdef NDEBUG
 #define ENABLE_CHECKSUMMING (false)
@@ -116,12 +126,23 @@ DEFINE_string(data_cache_eviction_policy, "LRU",
     "(Advanced) The cache eviction policy to use for the data cache. "
     "Either 'LRU' (default) or 'LIRS' (experimental)");
 
+DEFINE_string(data_cache_async_write_buffer_limit, "1GB",
+    "(Experimental) Limit of the total buffer size used by asynchronous store tasks.");
+
+DECLARE_bool(data_cache_keep_across_restarts);
+
 namespace impala {
 namespace io {
 
 static const int64_t PAGE_SIZE = 1L << 12;
 const char* DataCache::Partition::CACHE_FILE_PREFIX = "impala-cache-file-";
+const char* DataCache::Partition::DUMP_FILE_NAME = "impala-cache-metadata";
 const int MAX_FILE_DELETER_QUEUE_SIZE = 500;
+
+// This large value for the queue size is harmless because the total size of the entries
+// on the queue are bound by --data_cache_async_write_bufer_limit.
+const int MAX_STORE_TASK_QUEUE_SIZE = 1 << 20;
+
 static const char* PARTITION_PATH_METRIC_KEY_TEMPLATE =
     "impala-server.io-mgr.remote-data-cache-partition-$0.path";
 static const char* PARTITION_READ_LATENCY_METRIC_KEY_TEMPLATE =
@@ -172,6 +193,20 @@ class DataCache::CacheFile {
     return Status::OK();
   }
 
+  static Status Open(std::string path, bool allow_append, int64_t current_offset,
+      std::unique_ptr<CacheFile>* cache_file_ptr) {
+    unique_ptr<CacheFile> cache_file(new CacheFile(path));
+    kudu::RWFileOptions opts;
+    opts.mode = kudu::Env::OpenMode::MUST_EXIST;
+    KUDU_RETURN_IF_ERROR(
+        kudu::Env::Default()->NewRWFile(opts, path, &cache_file->file_),
+        "Failed to open cache file");
+    cache_file->allow_append_ = allow_append;
+    cache_file->current_offset_.Store(current_offset);
+    *cache_file_ptr = std::move(cache_file);
+    return Status::OK();
+  }
+
   // Close the underlying file so it cannot be read or written to anymore.
   void Close() {
     // Explicitly hold the lock in write mode to block all readers. This ensures that
@@ -191,6 +226,7 @@ class DataCache::CacheFile {
   // Close the underlying file and delete it from the filesystem.
   void DeleteFile() {
     Close();
+    if (readonly_.Load()) return;
     DCHECK(!file_);
     kudu::Status status = kudu::Env::Default()->DeleteFile(path_);
     if (!status.ok()) {
@@ -244,9 +280,10 @@ class DataCache::CacheFile {
   bool Write(int64_t offset, const uint8_t* buffer, int64_t buffer_len) {
     DCHECK_EQ(offset % PAGE_SIZE, 0);
     DCHECK_LE(offset, current_offset_.Load());
+    if (UNLIKELY(readonly_.Load())) return false;
     // Hold the lock in shared mode to check if 'file_' is not closed already.
     kudu::shared_lock<rw_spinlock> lock(lock_.get_lock());
-    if (UNLIKELY(!file_)) return false;
+    if (UNLIKELY(!file_ || readonly_.Load())) return false;
     DCHECK_LE(offset + buffer_len, current_offset_.Load());
     kudu::Status status = file_->Write(offset, Slice(buffer, buffer_len));
     if (UNLIKELY(!status.ok())) {
@@ -260,9 +297,10 @@ class DataCache::CacheFile {
   void PunchHole(int64_t offset, int64_t hole_size) {
     DCHECK_EQ(offset % PAGE_SIZE, 0);
     DCHECK_EQ(hole_size % PAGE_SIZE, 0);
+    if (UNLIKELY(readonly_.Load())) return;
     // Hold the lock in shared mode to check if 'file_' is not closed already.
     kudu::shared_lock<rw_spinlock> lock(lock_.get_lock());
-    if (UNLIKELY(!file_)) return;
+    if (UNLIKELY(!file_ || readonly_.Load())) return;
     DCHECK_LE(offset + hole_size, current_offset_.Load());
     kudu::Status status = file_->PunchHole(offset, hole_size);
     if (UNLIKELY(!status.ok())) {
@@ -271,8 +309,49 @@ class DataCache::CacheFile {
     }
   }
 
+  Status Flush() {
+    std::unique_lock<percpu_rwlock> lock(lock_);
+    // If the file is already closed, nothing to do.
+    if (!file_) return Status::OK();
+    kudu::Status status = file_->Sync();
+    if (UNLIKELY(!status.ok())) {
+      return Status(status.message().ToString());
+    }
+    return Status::OK();
+  }
+
+  void SetReadOnly() {
+    // Setting this without holding a lock tells other threads not to start new
+    // modifications.
+    readonly_.Store(true);
+    // Other threads get lock_ in shared mode for critical sections. Getting this
+    // in exclusive mode forces this thread to wait until all other threads have
+    // left their critical sections. That means that no writes should happen after
+    // this function returns (until RevokeReadOnly() runs).
+    std::unique_lock<percpu_rwlock> lock(lock_);
+  }
+
+  void RevokeReadOnly() {
+    // Going from readonly to read/write doesn't need a full lock
+    readonly_.Store(false);
+  }
+
+  int64_t mtime() {
+    // If the file is already closed, don't care about its mtime.
+    if (!file_) return -1;
+    struct stat buf;
+    stat(path_.c_str(), &buf);
+    return buf.st_mtime;
+  }
+
   const string& path() const { return path_; }
 
+  bool closed() const { return file_ == nullptr; }
+
+  bool allow_append() const { return allow_append_; }
+
+  int64_t current_offset() const { return current_offset_.Load(); }
+
  private:
   /// Full path of the backing file in the local storage.
   const string path_;
@@ -283,15 +362,19 @@ class DataCache::CacheFile {
   /// True iff it's okay to append to this backing file.
   bool allow_append_ = true;
 
+  /// The backing file cannot be written or punched hole if this is true.
+  AtomicBool readonly_{false};
+
   /// The current offset in the file to append to on next insert.
   AtomicInt64 current_offset_;
 
   /// This is a reader-writer lock used for synchronization with the deleter thread.
-  /// It is taken in write mode in Close() and shared mode everywhere else. It's expected
-  /// that all places except for Close() check that 'file_' is not NULL with the lock held
-  /// in shared mode while Close() ensures that no thread is holding the lock in shared
-  /// mode so it's safe to close the file. The file can no longer be read, written or hole
-  /// punched after it has been closed. The only operation allowed is to deletion.
+  /// It is taken in write mode in Close()/SetReadOnly() and shared mode
+  /// everywhere else. It's expected that all places except for
+  /// Close()/SetReadOnly() check that 'file_' is not NULL with the lock held in
+  /// shared mode while Close() ensures that no thread is holding the lock in
+  /// shared mode so it's safe to close the file. The file can no longer be read, written
+  /// or hole punched after it has been closed. The only operation allowed is deletion.
   percpu_rwlock lock_;
 
   /// C'tor of CacheFile to be called by Create() only.
@@ -378,6 +461,137 @@ struct DataCache::CacheKey {
   faststring key_;
 };
 
+/// The helper class to dump/load cache metadata. DumpData is the intermediary between
+/// cache metadata and dump file. Data cache partition can push data into DumpData and
+/// serialize it to disk or deserialize a DumpData from disk to load the metadata.
+class DataCache::Partition::DumpData {
+ public:
+
+  void serialize(std::ofstream& os) {
+    boost::archive::binary_oarchive oa(os);
+    oa << *this;
+  }
+
+  void deserialize(std::ifstream& is) {
+    boost::archive::binary_iarchive ia(is);
+    ia >> *this;
+  }
+
+ private:
+  friend class Partition;
+  friend class boost::serialization::access;
+
+  /// The dump struct of CacheFile.
+  struct CacheFileData {
+    CacheFileData() = default;
+
+    CacheFileData(string path, bool allow_append, int64_t current_offset, int64_t mtime)
+      : path(path), allow_append(allow_append), current_offset(current_offset),
+        mtime(mtime) { }
+
+    template<class Archive>
+    void serialize(Archive& ar, const unsigned int version) {
+      ar & path;
+      ar & allow_append;
+      ar & current_offset;
+      ar & mtime;
+    }
+
+    /// Corresponds to 'CacheFile::path_'.
+    string path;
+
+    /// Corresponds to 'CacheFile::allow_append_'.
+    bool allow_append;
+
+    /// Corresponds to 'CacheFile::current_offset_'.
+    int64_t current_offset;
+
+    /// The last modification time of the cache file, for check when load to prevent
+    /// accidental modification of the cache file.
+    int64_t mtime;
+  };
+
+  /// The dump struct of a pair of CacheKey and CacheEntry.
+  struct CacheKeyEntryData {
+    CacheKeyEntryData() = default;
+
+    CacheKeyEntryData(string key, int64_t index, int64_t offset, int64_t len,
+        uint64_t checksum)
+      : key(key), index(index), offset(offset), len(len), checksum(checksum) { }
+
+    template<class Archive>
+    void serialize(Archive& ar, const unsigned int version) {
+      ar & key;
+      ar & index;
+      ar & offset;
+      ar & len;
+      ar & checksum;
+    }
+
+    /// Corresponds to CacheKey.
+    string key;
+
+    /// Corresponds to CacheEntry, but uses 'index' instead of the 'file_' to point to the
+    /// cache file. When we dump the cache metadata, the 'file_' is meaningless because
+    /// the cache file objects cannot still be at the same address after reloading. So
+    /// during the dump, we use the corresponding file's index in the 'cache_files_data'
+    /// instead of file pointer, so that when reloading, the correct file pointer can be
+    /// find based on the index.
+    int64_t index;
+    int64_t offset;
+    int64_t len;
+    uint64_t checksum;
+  };
+
+  template<class Archive>
+  void serialize(Archive& ar, const unsigned int version) {
+    ar & cache_files_data;
+    ar & meta_cache_data;
+  }
+
+  /// Corresponds to 'Partition::cache_files_', but does not include closed files, because
+  /// closed files can no longer be read or written, and the underlying file should have
+  /// been deleted already.
+  vector<CacheFileData> cache_files_data;
+
+  /// Corresponds to 'Partition::meta_cache_', but does not include entries that the cache
+  /// file is closed, because these entries can no longer be hit and do not need to be
+  /// dumped.
+  vector<CacheKeyEntryData> meta_cache_data;
+
+  /// The mapping between the cache file pointer and the index of 'cache_files_data' to
+  /// convert the file pointer to the index when the cache entry is dumped. It is built
+  /// when cache files are dumped, only used when 'meta_cache_' is dumped and will not be
+  /// dumped to disk.
+  std::unordered_map<CacheFile*, int64_t> file_indexs;
+};
+
+/// The class to abstract store behavior, holds a temporary buffer copied from the source
+/// buffer until store complete.
+class DataCache::StoreTask {
+ public:
+  explicit StoreTask(const std::string& filename, int64_t mtime, int64_t offset,
+      uint8_t* buffer, int64_t buffer_len, DataCache* cache)
+    : key_(filename, mtime, offset),
+      buffer_(buffer),
+      buffer_len_(buffer_len),
+      cache_(cache) { }
+
+  ~StoreTask() { cache_->CompleteStoreTask(*this); }
+
+  const CacheKey& key() const { return key_; }
+  const uint8_t* buffer() const { return buffer_.get(); }
+  int64_t buffer_len() const { return buffer_len_; }
+
+ private:
+  DISALLOW_COPY_AND_ASSIGN(StoreTask);
+
+  CacheKey key_;
+  std::unique_ptr<uint8_t[]> buffer_;
+  int64_t buffer_len_;
+  DataCache* cache_;
+};
+
 static Cache::EvictionPolicy GetCacheEvictionPolicy(const std::string& policy_string) {
   Cache::EvictionPolicy policy = Cache::ParseEvictionPolicy(policy_string);
   if (policy != Cache::EvictionPolicy::LRU && policy != Cache::EvictionPolicy::LIRS) {
@@ -439,14 +653,19 @@ Status DataCache::Partition::CreateCacheFile() {
   return Status::OK();
 }
 
-Status DataCache::Partition::DeleteExistingFiles() const {
+Status DataCache::Partition::DeleteUntrackedFiles() const {
   DCHECK(!trace_replay_);
+  std::unordered_set<string> tracked_file_paths;
+  for (const auto& file : cache_files_) {
+    tracked_file_paths.insert(file->path());
+  }
   vector<string> entries;
   RETURN_IF_ERROR(FileSystemUtil::Directory::GetEntryNames(path_, &entries, 0,
       FileSystemUtil::Directory::EntryType::DIR_ENTRY_REG));
   for (const string& entry : entries) {
-    if (entry.find(CACHE_FILE_PREFIX) == 0) {
+    if (entry.find(CACHE_FILE_PREFIX) == 0 || entry == DUMP_FILE_NAME) {
       const string file_path = JoinPathSegments(path_, entry);
+      if (tracked_file_paths.find(file_path) != tracked_file_paths.end()) continue;
       KUDU_RETURN_IF_ERROR(kudu::Env::Default()->DeleteFile(file_path),
           Substitute("Failed to delete old cache file $0", file_path));
       LOG(INFO) << Substitute("Deleted old cache file $0", file_path);
@@ -470,20 +689,6 @@ Status DataCache::Partition::Init() {
   }
   RETURN_IF_ERROR(FileSystemUtil::VerifyIsDirectory(path_));
 
-  // Delete all existing backing files left over from previous runs.
-  RETURN_IF_ERROR(DeleteExistingFiles());
-
-  // Check if there is enough space available at this point in time.
-  uint64_t available_bytes;
-  RETURN_IF_ERROR(FileSystemUtil::GetSpaceAvailable(path_, &available_bytes));
-  if (available_bytes < capacity_) {
-    const string& err = Substitute("Insufficient space for $0. Required $1. Only $2 is "
-        "available", path_, PrettyPrinter::PrintBytes(capacity_),
-        PrettyPrinter::PrintBytes(available_bytes));
-    LOG(ERROR) << err;
-    return Status(err);
-  }
-
   // Make sure hole punching is supported for the caching directory.
   RETURN_IF_ERROR(FileSystemUtil::CheckHolePunch(path_));
 
@@ -515,9 +720,32 @@ Status DataCache::Partition::Init() {
   // Create metrics for this partition
   InitMetrics();
 
-  // Create a backing file for the partition.
-  RETURN_IF_ERROR(CreateCacheFile());
-  oldest_opened_file_ = 0;
+  // Only if loading is not enabled, or if loading is enabled but fails, do we need to
+  // create new cache file.
+  if (!(FLAGS_data_cache_keep_across_restarts && Load().ok())) {
+    // Create a backing file for the partition.
+    RETURN_IF_ERROR(CreateCacheFile());
+    oldest_opened_file_ = 0;
+  }
+
+  // Delete all cache files that are not listed in cache_files_ (including dumped metadata
+  // file). These files were left over from previous runs and are now no longer needed.
+  RETURN_IF_ERROR(DeleteUntrackedFiles());
+
+  // Check if there is enough space available at this point in time.
+  int64_t used_bytes = ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_TOTAL_BYTES->GetValue();
+  uint64_t available_bytes;
+  RETURN_IF_ERROR(FileSystemUtil::GetSpaceAvailable(path_, &available_bytes));
+  if (used_bytes + available_bytes < capacity_) {
+    const string& err = Substitute("Insufficient space for $0. Total required $1 and "
+      "cache already used $2. Still required $3, but only $4 is available",
+      path_, PrettyPrinter::PrintBytes(capacity_), PrettyPrinter::PrintBytes(used_bytes),
+      PrettyPrinter::PrintBytes(capacity_ - used_bytes),
+      PrettyPrinter::PrintBytes(available_bytes));
+    LOG(ERROR) << err;
+    return Status(err);
+  }
+
   return Status::OK();
 }
 
@@ -697,6 +925,17 @@ bool DataCache::Partition::InsertIntoCache(const Slice& key, CacheFile* cache_fi
     }
   }
 
+  // IMPALA-10971: These metrics need to be incremented prior to the Insert(), because
+  // the Insert() can fail and instantly evict the entry. When that happens,
+  // the total bytes and num entries are decremented. Without this corresponding
+  // increment, the counts will be incorrect.
+  // Trace replays do not keep metrics
+  if (LIKELY(!trace_replay_)) {
+    ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_TOTAL_BYTES->Increment(charge_len);
+    ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_NUM_ENTRIES->Increment(1);
+    ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_NUM_WRITES->Increment(1);
+  }
+
   // Insert the new entry into the cache.
   CacheEntry entry(cache_file, insertion_offset, buffer_len, checksum);
   memcpy(meta_cache_->MutableValue(&pending_handle), &entry, sizeof(CacheEntry));
@@ -705,16 +944,11 @@ bool DataCache::Partition::InsertIntoCache(const Slice& key, CacheFile* cache_fi
   if (UNLIKELY(handle.get() == nullptr)){
     // Trace replays do not keep metrics
     if (LIKELY(!trace_replay_)) {
+      // EvictedEntry() already ran and decremented the other counters.
       ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_INSTANT_EVICTIONS->Increment(1);
     }
     return false;
   }
-  // Trace replays do not keep metrics
-  if (LIKELY(!trace_replay_)) {
-    ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_TOTAL_BYTES->Increment(charge_len);
-    ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_NUM_ENTRIES->Increment(1);
-    ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_NUM_WRITES->Increment(1);
-  }
   return true;
 }
 
@@ -742,9 +976,6 @@ bool DataCache::Partition::Store(const CacheKey& cache_key, const uint8_t* buffe
     // Limit the write concurrency to avoid blocking the caller (which could be calling
     // from the critical path of an IO read) when the cache becomes IO bound due to either
     // limited memory for page cache or the cache is undersized which leads to eviction.
-    //
-    // TODO: defer the writes to another thread which writes asynchronously. Need to bound
-    // the extra memory consumption for holding the temporary buffer though.
     const bool exceed_concurrency =
         pending_insert_set_.size() >= data_cache_write_concurrency_;
     if (exceed_concurrency ||
@@ -800,6 +1031,7 @@ bool DataCache::Partition::Store(const CacheKey& cache_key, const uint8_t* buffe
 
 void DataCache::Partition::DeleteOldFiles() {
   std::unique_lock<SpinLock> partition_lock(lock_);
+  if (UNLIKELY(files_readonly_)) return;
   DCHECK_GE(oldest_opened_file_, 0);
   int target = cache_files_.size() - FLAGS_data_cache_max_opened_files;
   while (oldest_opened_file_ < target) {
@@ -807,6 +1039,156 @@ void DataCache::Partition::DeleteOldFiles() {
   }
 }
 
+void DataCache::Partition::SetCacheFilesReadOnly() {
+  std::unique_lock<SpinLock> partition_lock(lock_);
+  for (auto& file : cache_files_) file->SetReadOnly();
+  files_readonly_ = true;
+}
+
+void DataCache::Partition::RevokeCacheFilesReadOnly() {
+  std::unique_lock<SpinLock> partition_lock(lock_);
+  for (auto& file : cache_files_) file->RevokeReadOnly();
+  files_readonly_ = false;
+}
+
+Status DataCache::Partition::Dump() {
+  std::unique_lock<SpinLock> partition_lock(lock_);
+  LOG(INFO) << Substitute("Partition $0 start dumping.", index_);
+  DumpData dump_data;
+  RETURN_IF_ERROR(DumpCacheFiles(dump_data));
+  RETURN_IF_ERROR(DumpMetaCache(dump_data));
+  std::ofstream file;
+  file.open(JoinPathSegments(path_, DUMP_FILE_NAME), std::ofstream::binary);
+  if (!file) return Status(Substitute("Failed to open $0", DUMP_FILE_NAME));
+  try {
+    dump_data.serialize(file);
+  } catch (boost::archive::archive_exception& e) {
+    LOG(ERROR) << Substitute("Partition $0 failed to serialize dump file.", index_)
+               << e.what();
+    return Status("Failed to serialize dump file.");
+  }
+  file.close();
+  LOG(INFO) << Substitute("Partition $0 dump successfully.", index_);
+  return Status::OK();
+}
+
+Status DataCache::Partition::Load() {
+  std::ifstream file;
+  file.open(JoinPathSegments(path_, DUMP_FILE_NAME), std::ofstream::binary);
+  if (!file) return Status::Expected(Substitute("Failed to open $0", DUMP_FILE_NAME));
+  LOG(INFO) << Substitute("Partition $0 start loading.", index_);
+  DumpData dump_data;
+  try {
+    dump_data.deserialize(file);
+  } catch (boost::archive::archive_exception& e) {
+    LOG(ERROR) << Substitute("Partition $0 failed to deserialize dump file.", index_)
+               << e.what();
+    return Status("Failed to deserialize dump file.");
+  }
+  file.close();
+
+  Status status = LoadCacheFiles(dump_data);
+  if (!status.ok()) {
+    cache_files_.clear();
+    return status;
+  }
+
+  status = LoadMetaCache(dump_data);
+  if (!status.ok()) {
+    cache_files_.clear();
+    meta_cache_.reset(NewCache(GetCacheEvictionPolicy(FLAGS_data_cache_eviction_policy),
+        capacity_, path_));
+    // We have already successfully initialized meta_cache_ once before calling Load(), so
+    // there is no reason for it to fail this time.
+    ABORT_IF_ERROR(meta_cache_->Init());
+    return status;
+  }
+
+  LOG(INFO) << Substitute("Partition $0 load successfully.", index_);
+  return Status::OK();
+}
+
+Status DataCache::Partition::DumpCacheFiles(DumpData& dump_data) {
+  lock_.DCheckLocked();
+  int64_t index = 0;
+  for (const auto& file : cache_files_) {
+    // There is no need to dump closed file, it can no longer be read or written or even
+    // deleted.
+    if (file->closed()) continue;
+
+    // Just to be safe, flush the file.
+    RETURN_IF_ERROR(file->Flush());
+
+    dump_data.cache_files_data.emplace_back(file->path(), file->allow_append(),
+        file->current_offset(), file->mtime());
+    auto result = dump_data.file_indexs.emplace(file.get(), index++);
+    DCHECK(result.second);
+  }
+  return Status::OK();
+}
+
+Status DataCache::Partition::LoadCacheFiles(const DumpData& dump_data) {
+  lock_.DCheckLocked();
+  cache_files_.clear();
+  for (const DumpData::CacheFileData& file_data : dump_data.cache_files_data) {
+    unique_ptr<CacheFile> cache_file;
+    RETURN_IF_ERROR(CacheFile::Open(file_data.path, file_data.allow_append,
+        file_data.current_offset, &cache_file));
+    if (file_data.mtime != cache_file->mtime()) {
+      return Status("The actual mtime of the cache file does not match the dumped data.");
+    }
+    cache_files_.emplace_back(std::move(cache_file));
+  }
+  if (cache_files_.empty()) {
+    return Status("No cache files loaded.");
+  }
+  oldest_opened_file_ = 0;
+  return Status::OK();
+}
+
+Status DataCache::Partition::DumpMetaCache(DumpData& dump_data) {
+  lock_.DCheckLocked();
+  DCHECK(dump_data.cache_files_data.size() == dump_data.file_indexs.size());
+
+  // Walk through the meta cache, taking all keys and entries and dumping them one by one.
+  vector<Cache::UniqueHandle> handles = meta_cache_->Dump();
+  for (const auto& handle : handles) {
+    CacheEntry entry(meta_cache_->Value(handle));
+    auto index_iter = dump_data.file_indexs.find(entry.file());
+    // If the entry's cache file is not in the list, we do not need to dump it.
+    if (index_iter == dump_data.file_indexs.end()) continue;
+    dump_data.meta_cache_data.emplace_back(meta_cache_->Key(handle).ToString(),
+        index_iter->second, entry.offset(), entry.len(), entry.checksum());
+  }
+  return Status::OK();
+}
+
+Status DataCache::Partition::LoadMetaCache(const DumpData& dump_data) {
+  lock_.DCheckLocked();
+  for (const DumpData::CacheKeyEntryData& key_entry : dump_data.meta_cache_data) {
+    // Usually this shouldn't happen, and if it did, the dump file may be corrupted and
+    // loading should be aborted.
+    if (UNLIKELY(key_entry.index >= cache_files_.size())) {
+      return Status("Failed to load meta cache, the dump file may be corrupted.");
+    }
+
+    // According to the index, find the corresponding file pointer, build entry and insert
+    // into the cache like normal store. If the cache capacity has been changed, the entry
+    // may be evicted, but that's no matter.
+    CacheFile* file = cache_files_[key_entry.index].get();
+    CacheEntry entry(file, key_entry.offset, key_entry.len, key_entry.checksum);
+    const int64_t charge_len = BitUtil::RoundUp(entry.len(), PAGE_SIZE);
+    Cache::UniquePendingHandle pending_handle(
+        meta_cache_->Allocate(Slice(key_entry.key), sizeof(CacheEntry), charge_len));
+    if (UNLIKELY(pending_handle.get() == nullptr)) continue;
+    memcpy(meta_cache_->MutableValue(&pending_handle), &entry, sizeof(CacheEntry));
+    meta_cache_->Insert(std::move(pending_handle), this);
+    ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_TOTAL_BYTES->Increment(charge_len);
+    ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_NUM_ENTRIES->Increment(1);
+  }
+  return Status::OK();
+}
+
 void DataCache::Partition::EvictedEntry(Slice key, Slice value) {
   if (closed_) return;
   if (UNLIKELY(trace_replay_)) return;
@@ -840,6 +1222,14 @@ bool DataCache::Partition::VerifyChecksum(const string& ops_name, const CacheEnt
   return true;
 }
 
+DataCache::DataCache(const std::string config, int32_t num_async_write_threads,
+    bool trace_replay)
+  : config_(config),
+    trace_replay_(trace_replay),
+    num_async_write_threads_(num_async_write_threads) { }
+
+DataCache::~DataCache() { ReleaseResources(); }
+
 Status DataCache::Init() {
   // Verifies all the configured flags are sane.
   if (FLAGS_data_cache_file_max_size_bytes < PAGE_SIZE) {
@@ -861,12 +1251,12 @@ Status DataCache::Init() {
 
   // Parse the capacity string to make sure it's well-formed.
   bool is_percent;
-  int64_t capacity = ParseUtil::ParseMemSpec(all_cache_configs[1], &is_percent, 0);
+  per_partition_capacity_ = ParseUtil::ParseMemSpec(all_cache_configs[1], &is_percent, 0);
   if (is_percent) {
     return Status(Substitute("Malformed data cache capacity configuration $0",
         all_cache_configs[1]));
   }
-  if (capacity < PAGE_SIZE) {
+  if (per_partition_capacity_ < PAGE_SIZE) {
     return Status(Substitute("Configured data cache capacity $0 is too small",
         all_cache_configs[1]));
   }
@@ -882,9 +1272,9 @@ Status DataCache::Init() {
   int32_t partition_idx = 0;
   for (const string& dir_path : cache_dirs) {
     LOG(INFO) << "Adding partition " << dir_path << " with capacity "
-              << PrettyPrinter::PrintBytes(capacity);
+              << PrettyPrinter::PrintBytes(per_partition_capacity_);
     std::unique_ptr<Partition> partition =
-        make_unique<Partition>(partition_idx, dir_path, capacity,
+        make_unique<Partition>(partition_idx, dir_path, per_partition_capacity_,
             max_opened_files_per_partition, trace_replay_);
     RETURN_IF_ERROR(partition->Init());
     partitions_.emplace_back(move(partition));
@@ -897,16 +1287,46 @@ Status DataCache::Init() {
     // will enqueue a request (i.e. a partition index) when it notices the number of files
     // in a partition exceeds the per-partition limit. The files in a partition will be
     // closed in the order they are created until it's within the per-partition limit.
-    file_deleter_pool_.reset(new ThreadPool<int>("impala-server",
+    file_deleter_pool_.reset(new ThreadPool<int>("data-cache",
         "data-cache-file-deleter", 1, MAX_FILE_DELETER_QUEUE_SIZE,
         bind<void>(&DataCache::DeleteOldFiles, this, _1, _2)));
     RETURN_IF_ERROR(file_deleter_pool_->Init());
   }
 
+  /// If --data_cache_num_async_write_threads has been set above 0, the store behavior
+  /// will be asynchronous. A task queue and thread pool will be started for that.
+  if (num_async_write_threads_ > 0) {
+    if (UNLIKELY(trace_replay_)) {
+      return Status("Data cache does not support asynchronous writes when doing trace "
+          "replay. Please set 'data_cache_num_async_write_threads' to 0.");
+    }
+    // Parse the buffer limit string to make sure it's well-formed.
+    bool is_percent;
+    int64_t buffer_limit = ParseUtil::ParseMemSpec(
+        FLAGS_data_cache_async_write_buffer_limit, &is_percent, 0);
+    if (is_percent) {
+      return Status(Substitute("Malformed data cache write buffer limit configuration $0",
+          FLAGS_data_cache_async_write_buffer_limit));
+    }
+    if (!TestInfo::is_be_test() && buffer_limit < (1 << 23 /* 8MB */)) {
+      return Status(Substitute("Configured data cache write buffer limit $0 is too small "
+          "(less than 8MB)", FLAGS_data_cache_async_write_buffer_limit));
+    }
+    store_buffer_capacity_ = buffer_limit;
+    storer_pool_.reset(new ThreadPool<StoreTaskHandle>("data-cache", "data-cache-storer",
+        num_async_write_threads_, MAX_STORE_TASK_QUEUE_SIZE,
+        bind<void>(&DataCache::HandleStoreTask, this, _1, _2)));
+    RETURN_IF_ERROR(storer_pool_->Init());
+  }
+
   return Status::OK();
 }
 
 void DataCache::ReleaseResources() {
+  if (storer_pool_) {
+    storer_pool_->Shutdown();
+    storer_pool_->Join();
+  }
   if (file_deleter_pool_) file_deleter_pool_->Shutdown();
   for (auto& partition : partitions_) partition->ReleaseResources();
 }
@@ -938,6 +1358,10 @@ int64_t DataCache::Lookup(const string& filename, int64_t mtime, int64_t offset,
 bool DataCache::Store(const string& filename, int64_t mtime, int64_t offset,
     const uint8_t* buffer, int64_t buffer_len) {
   DCHECK(!partitions_.empty());
+
+  // Check early that the cache is read-only.
+  if (UNLIKELY(readonly_.Load())) return false;
+
   // Bail out early for uncacheable ranges or invalid requests.
   if (mtime < 0 || offset < 0 || buffer_len < 0) {
     VLOG(3) << Substitute("Skipping insertion of invalid entry $0 mtime: $1 offset: $2 "
@@ -945,19 +1369,14 @@ bool DataCache::Store(const string& filename, int64_t mtime, int64_t offset,
     return false;
   }
 
+  // If the storer thread pool is available, data will be stored asynchronously.
+  if (num_async_write_threads_ > 0) {
+    return SubmitStoreTask(filename, mtime, offset, buffer, buffer_len);
+  }
+
   // Construct a cache key. The cache key is also hashed to compute the partition index.
   const CacheKey key(filename, mtime, offset);
-  int idx = key.Hash() % partitions_.size();
-  bool start_reclaim;
-  bool stored = partitions_[idx]->Store(key, buffer, buffer_len, &start_reclaim);
-  if (VLOG_IS_ON(3)) {
-    stringstream ss;
-    ss << std::hex << reinterpret_cast<int64_t>(buffer);
-    LOG(INFO) << Substitute("Storing $0 mtime: $1 offset: $2 bytes_to_read: $3 "
-        "buffer: 0x$4 stored: $5", filename, mtime, offset, buffer_len, ss.str(), stored);
-  }
-  if (start_reclaim) file_deleter_pool_->Offer(idx);
-  return stored;
+  return StoreInternal(key, buffer, buffer_len);
 }
 
 Status DataCache::CloseFilesAndVerifySizes() {
@@ -967,11 +1386,124 @@ Status DataCache::CloseFilesAndVerifySizes() {
   return Status::OK();
 }
 
+int64_t DataCache::SetDataCacheReadOnly() {
+  if (readonly_.Load()) {
+    return ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_NUM_WRITES->GetValue();
+  }
+
+  // First set the read-only flag to reject new writes. The exclusive lock is then
+  // acquired, because the shared lock will be acquired before any writes begin, so it
+  // blocks here until all ongoing writes have completed and all shared lock has been
+  // released. This ensures that there will be no more change to the cache after the
+  // function returns.
+  readonly_.Store(true);
+  std::unique_lock<shared_mutex> lock(readonly_lock_);
+  for (auto& partition : partitions_) {
+    partition->SetCacheFilesReadOnly();
+  }
+
+  return ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_NUM_WRITES->GetValue();
+}
+
+int64_t DataCache::RevokeDataCacheReadOnly() {
+  if (!readonly_.Load()) {
+    return ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_NUM_WRITES->GetValue();
+  }
+
+  std::unique_lock<shared_mutex> lock(readonly_lock_);
+  for (auto& partition : partitions_) {
+    partition->RevokeCacheFilesReadOnly();
+  }
+  readonly_.Store(false);
+
+  return ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_NUM_WRITES->GetValue();
+}
+
+Status DataCache::Dump() {
+  SetDataCacheReadOnly();
+  for (auto& partition : partitions_) {
+    RETURN_IF_ERROR(partition->Dump());
+  }
+  return Status::OK();
+}
+
 void DataCache::DeleteOldFiles(uint32_t thread_id, int partition_idx) {
   DCHECK_LT(partition_idx, partitions_.size());
   partitions_[partition_idx]->DeleteOldFiles();
 }
 
+bool DataCache::SubmitStoreTask(const std::string& filename, int64_t mtime,
+    int64_t offset, const uint8_t* buffer, int64_t buffer_len) {
+  const int64_t charge_len = BitUtil::RoundUp(buffer_len, PAGE_SIZE);
+  if (UNLIKELY(charge_len > per_partition_capacity_)) return false;
+
+  // Tries to increase the current_buffer_size_ by buffer_len before allocate buffer.
+  // If new size exceeds store_buffer_capacity_, return false and current_buffer_size_ is
+  // not changed.
+  while (true) {
+    int64_t current_size = current_buffer_size_.Load();
+    int64_t new_size = current_size + buffer_len;
+    if (UNLIKELY(new_size > store_buffer_capacity_)) {
+      VLOG(2) << Substitute("Failed to create store task due to buffer size limitation, "
+          "current buffer size: $0 size limitation: $1 require: $2",
+          current_size, store_buffer_capacity_, buffer_len);
+      ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_ASYNC_WRITES_DROPPED_BYTES->
+          Increment(buffer_len);
+      ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_ASYNC_WRITES_DROPPED_ENTRIES->Increment(1);
+      return false;
+    }
+    if (LIKELY(current_buffer_size_.CompareAndSwap(current_size, new_size))) {
+      ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_ASYNC_WRITES_OUTSTANDING_BYTES->SetValue(
+          current_buffer_size_.Load());
+      break;
+    }
+  }
+
+  DCHECK(buffer != nullptr);
+  // TODO: Should we use buffer pool instead of piecemeal memory allocate?
+  uint8_t* task_buffer = new uint8_t[buffer_len];
+  memcpy(task_buffer, buffer, buffer_len);
+
+  const StoreTask* task =
+      new StoreTask(filename, mtime, offset, task_buffer, buffer_len, this);
+  storer_pool_->Offer(StoreTaskHandle(task));
+  ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_NUM_ASYNC_WRITES_SUBMITTED->Increment(1);
+  return true;
+}
+
+void DataCache::CompleteStoreTask(const StoreTask& task) {
+  current_buffer_size_.Add(-task.buffer_len());
+  DCHECK_GE(current_buffer_size_.Load(), 0);
+  ImpaladMetrics::IO_MGR_REMOTE_DATA_CACHE_ASYNC_WRITES_OUTSTANDING_BYTES->SetValue(
+      current_buffer_size_.Load());
+}
+
+void DataCache::HandleStoreTask(uint32_t thread_id, const StoreTaskHandle& task) {
+    StoreInternal(task->key(), task->buffer(), task->buffer_len());
+}
+
+bool DataCache::StoreInternal(const CacheKey& key, const uint8_t* buffer,
+    int64_t buffer_len) {
+  // Here, a shared lock is acquired with try_to_lock, while a unique lock is acquired in
+  // SetDataCacheReadOnly(). Therefore, when setting the read-only status, the lock
+  // acquisition here can quickly fail, then return.
+  kudu::shared_lock<shared_mutex> lock(readonly_lock_, std::try_to_lock);
+  if (UNLIKELY(!lock.owns_lock() || readonly_.Load())) return false;
+
+  int idx = key.Hash() % partitions_.size();
+  bool start_reclaim;
+  bool stored = partitions_[idx]->Store(key, buffer, buffer_len, &start_reclaim);
+  if (VLOG_IS_ON(3)) {
+    stringstream ss;
+    ss << std::hex << reinterpret_cast<int64_t>(buffer);
+    LOG(INFO) << Substitute("Storing $0 mtime: $1 offset: $2 bytes_to_read: $3 "
+        "buffer: 0x$4 stored: $5", key.filename().ToString(), key.mtime(), key.offset(),
+        buffer_len, ss.str(), stored);
+  }
+  if (start_reclaim) file_deleter_pool_->Offer(idx);
+  return stored;
+}
+
 void DataCache::Partition::Trace(
     const trace::EventType& type, const DataCache::CacheKey& key,
     int64_t lookup_len, int64_t entry_len) {
