@@ -74,6 +74,19 @@ DEFINE_string(data_cache, "", "The configuration string for IO data cache. "
     "a capacity quota per directory. For example /data/0,/data/1:1TB means the cache "
     "may use up to 2TB, with 1TB max in /data/0 and /data/1 respectively. Please note "
     "that each Impala daemon on a host must have a unique caching directory.");
+DEFINE_int32(data_cache_num_async_write_threads, 0,
+    "(Experimental) Number of data cache async write threads. Write threads will write "
+    "the cache asynchronously after IO thread read data, so IO thread will return more "
+    "quickly. The extra memory for temporary buffers is limited by "
+    "--data_cache_async_write_buffer_limit. If this's 0, then write will be "
+    "synchronous.");
+
+DEFINE_bool(data_cache_keep_across_restarts, false,
+    "(Experimental) If this is true, the data cache metadata is dumped to the same "
+    "directory as the cached files on disk when the process gracefully shutdowns. The "
+    "metadata will be reloaded the next time the process starts, so that the previous "
+    "cached data can be reused as if the process had never shutdown. If loading fails, "
+    "it will proceed with regular initialization.");
 
 // Rotational disks should have 1 thread per disk to minimize seeks.  Non-rotational
 // don't have this penalty and benefit from multiple concurrent IO requests.
@@ -144,6 +157,9 @@ DEFINE_int32(num_ozone_io_threads, 16, "Number of Ozone I/O threads");
 // The maximum number of SFS I/O threads.
 DEFINE_int32(num_sfs_io_threads, 16, "Number of SFS I/O threads");
 
+// The maximum number of OBS I/O threads.
+DEFINE_int32(num_obs_io_threads, 16, "Number of OBS I/O threads");
+
 // The number of cached file handles defines how much memory can be used per backend for
 // caching frequently used file handles. Measurements indicate that a single file handle
 // uses about 6kB of memory. 20k file handles will thus reserve ~120MB of memory.
@@ -190,7 +206,7 @@ DEFINE_bool(cache_s3_file_handles, true, "Enable the file handle cache for "
 DEFINE_bool(cache_abfs_file_handles, true, "Enable the file handle cache for "
     "ABFS files.");
 
-DEFINE_bool(cache_ozone_file_handles, false, "Enable the file handle cache for Ozone "
+DEFINE_bool(cache_ozone_file_handles, true, "Enable the file handle cache for Ozone "
     "files.");
 
 DECLARE_int64(min_buffer_size);
@@ -575,6 +591,9 @@ Status DiskIoMgr::Init() {
     } else if (i == RemoteSFSDiskId()) {
       num_threads_per_disk = FLAGS_num_sfs_io_threads;
       device_name = "SFS remote";
+    } else if (i == RemoteOBSDiskId()) {
+      num_threads_per_disk = FLAGS_num_obs_io_threads;
+      device_name = "OBS remote";
     } else if (DiskInfo::is_rotational(i)) {
       num_threads_per_disk = num_io_threads_per_rotational_disk_;
       // During tests, i may not point to an existing disk.
@@ -676,7 +695,8 @@ Status DiskIoMgr::Init() {
   DCHECK_EQ(ret, 0);
 
   if (!FLAGS_data_cache.empty()) {
-    remote_data_cache_.reset(new DataCache(FLAGS_data_cache));
+    remote_data_cache_.reset(
+        new DataCache(FLAGS_data_cache, FLAGS_data_cache_num_async_write_threads));
     RETURN_IF_ERROR(remote_data_cache_->Init());
   }
   return Status::OK();
@@ -718,6 +738,13 @@ Status DiskIoMgr::AllocateBuffersForRange(
       min_buffer_size_, max_buffer_size_);
 }
 
+Status DiskIoMgr::DumpDataCache() {
+  if (FLAGS_data_cache_keep_across_restarts && remote_data_cache_) {
+    return remote_data_cache_->Dump();
+  }
+  return Status::Expected("No cache dump is required.");
+}
+
 vector<int64_t> DiskIoMgr::ChooseBufferSizes(int64_t scan_range_len, int64_t max_bytes) {
   DCHECK_GE(max_bytes, min_buffer_size_);
   vector<int64_t> buffer_sizes;
@@ -917,6 +944,7 @@ int DiskIoMgr::AssignQueue(
     if (IsCosPath(file, check_default_fs)) return RemoteCosDiskId();
     if (IsOzonePath(file, check_default_fs)) return RemoteOzoneDiskId();
     if (IsSFSPath(file, check_default_fs)) return RemoteSFSDiskId();
+    if (IsOBSPath(file, check_default_fs)) return RemoteOBSDiskId();
   }
   // Assign to a local disk queue.
   DCHECK(!IsS3APath(file, check_default_fs)); // S3 is always remote.
@@ -926,6 +954,7 @@ int DiskIoMgr::AssignQueue(
   DCHECK(!IsGcsPath(file, check_default_fs)); // GCS is always remote.
   DCHECK(!IsCosPath(file, check_default_fs)); // COS is always remote.
   DCHECK(!IsSFSPath(file, check_default_fs)); // SFS is always remote.
+  DCHECK(!IsOBSPath(file, check_default_fs)); // OBS is always remote.
   if (disk_id == -1) {
     // disk id is unknown, assign it an arbitrary one.
     disk_id = next_disk_id_.Add(1);
