@@ -46,6 +46,7 @@
 #include "runtime/tuple-row.h"
 #include "service/data-stream-service.h"
 #include "util/aligned-new.h"
+#include "util/compress.h"
 #include "util/debug-util.h"
 #include "util/network-util.h"
 #include "util/pretty-printer.h"
@@ -59,6 +60,10 @@
 DEFINE_int64(data_stream_sender_buffer_size, 16 * 1024,
     "(Advanced) Max size in bytes which a row batch in a data stream sender's channel "
     "can accumulate before the row batch is sent over the wire.");
+DEFINE_int64_hidden(data_stream_sender_eos_timeout_ms, 60*60*1000,
+    "Timeout for EndDataStream (EOS) RPCs. Setting a timeout prioritizes them over other "
+    "DataStreamService RPCs. Defaults to 1 hour. Set to 0 or negative value to disable "
+    "the timeout.");
 
 using std::condition_variable_any;
 using namespace apache::thrift;
@@ -118,12 +123,18 @@ void KrpcDataStreamSenderConfig::Close() {
 // calls above will return before the RPC has completed but they may block if there
 // is already an in-flight RPC.
 //
-// Each channel internally has two OutboundRowBatch to serialize to. They are reused
-// across multiple RPC calls. Having two OutboundRowBatch allows client to serialize
-// the next row batch while the current row batch is being sent. Upon completion of
-// a RPC, the callback TransmitDataCompleteCb() is invoked. If the RPC fails due to
-// remote service's queue being full, TransmitDataCompleteCb() will schedule the retry
-// callback RetryCb() after some delay dervied from 'FLAGS_rpc_retry_internal_ms'.
+// Each channel owns a single OutboundRowBatch outbound_batch_. This holds the buffers
+// backing the in-flight RPC or kept as reserve if there is no ongoing RPC. The actual
+// serialization happens to a shared OutboundRowBatch (see IMPALA-12433 for details) which
+// can be swapped with outbound_batch_ once the previous in-flight RPC is finished.
+// This means that the client can serialize the next row batch while the current row batch
+// is being sent. outbound_batch_ is only used in the partitioned case - in the
+// unpartitioned case the shared outbound_batch_ holds the data for the the in-flight RPC.
+//
+// Upon completion of a RPC, the callback TransmitDataCompleteCb() is invoked. If the RPC
+// fails due to remote service's queue being full, TransmitDataCompleteCb() will schedule
+// the retry callback RetryCb() after some delay dervied from
+// 'FLAGS_rpc_retry_internal_ms'.
 //
 // When a data stream sender is shut down, it will call Teardown() on all channels to
 // release resources. Teardown() will cancel any in-flight RPC and wait for the
@@ -175,7 +186,10 @@ class KrpcDataStreamSender::Channel : public CacheLineAligned {
   // preceding RPC is still in-flight. This is expected to be called from the fragment
   // instance execution thread. Return error status if initialization of the RPC request
   // parameters failed or if the preceding RPC failed. Returns OK otherwise.
-  Status TransmitData(const OutboundRowBatch* outbound_batch);
+  // If 'swap_batch' is true, 'outbound_batch' is swapped with the Channel's
+  // 'outbound_batch_'. This happens after the previous RPC is finished so its buffers
+  // can be reused.
+  Status TransmitData(std::unique_ptr<OutboundRowBatch>* outbound_batch, bool swap_batch);
 
   // Copies a single row into this channel's row batch and flushes the row batch once
   // it reaches capacity. This call may block if the row batch's capacity is reached
@@ -231,20 +245,14 @@ class KrpcDataStreamSender::Channel : public CacheLineAligned {
   // Only used if the partitioning scheme is "KUDU" or "HASH_PARTITIONED".
   scoped_ptr<RowBatch> batch_;
 
-  // The outbound row batches are double-buffered so that we can serialize the next
-  // batch while the other is still referenced by the in-flight RPC. Each entry contains
+  // Owns an outbound row batch that can be referenced by the in-flight RPC. Contains
   // a RowBatchHeaderPB and the buffers for the serialized tuple offsets and data.
   //
-  // TODO: replace this with an actual queue. Schedule another RPC callback in the
+  // TODO: replace this with a queue. Schedule another RPC callback in the
   // completion callback if the queue is not empty.
-  // TODO: rethink whether to keep per-channel buffers vs having all buffers in the
-  // datastream sender and sharing them across all channels. These buffers are not used in
+  // datastream sender and sharing them across all channels. This buffer is not used in
   // "UNPARTITIONED" scheme.
-  std::vector<OutboundRowBatch> outbound_batches_;
-
-  // Index into 'outbound_batches_' for the next available OutboundRowBatch to serialize
-  // into. This is read and written by the main execution thread.
-  int next_batch_idx_ = 0;
+  std::unique_ptr<OutboundRowBatch> outbound_batch_;
 
   // Synchronize accesses to the following fields between the main execution thread and
   // the KRPC reactor thread. Note that there should be only one reactor thread invoking
@@ -383,10 +391,9 @@ Status KrpcDataStreamSender::Channel::Init(
   // Create a DataStreamService proxy to the destination.
   RETURN_IF_ERROR(DataStreamService::GetProxy(address_, hostname_, &proxy_));
 
-  // Init outbound_batches_.
-  for (int i = 0; i < NUM_OUTBOUND_BATCHES; ++i) {
-    outbound_batches_.emplace_back(allocator, is_local_);
-  }
+  // Init outbound_batch_.
+  outbound_batch_.reset(new OutboundRowBatch(allocator));
+
   return Status::OK();
 }
 
@@ -584,10 +591,10 @@ Status KrpcDataStreamSender::Channel::DoTransmitDataRpc() {
 }
 
 Status KrpcDataStreamSender::Channel::TransmitData(
-    const OutboundRowBatch* outbound_batch) {
+    unique_ptr<OutboundRowBatch>* outbound_batch, bool swap_batch) {
   VLOG_ROW << "Channel::TransmitData() fragment_instance_id="
            << PrintId(fragment_instance_id_) << " dest_node=" << dest_node_id_
-           << " #rows=" << outbound_batch->header()->num_rows();
+           << " #rows=" << outbound_batch->get()->header()->num_rows();
   std::unique_lock<SpinLock> l(lock_);
   RETURN_IF_ERROR(WaitForRpcLocked(&l));
   DCHECK(!rpc_in_flight_);
@@ -596,20 +603,25 @@ Status KrpcDataStreamSender::Channel::TransmitData(
   // TODO: Needs better solution for IMPALA-3990 in the long run.
   if (UNLIKELY(remote_recvr_closed_)) return Status::OK();
   rpc_in_flight_ = true;
-  rpc_in_flight_batch_ = outbound_batch;
+  rpc_in_flight_batch_ = outbound_batch->get();
   RETURN_IF_ERROR(DoTransmitDataRpc());
+  // At this point the previous RPC must be already finished and the previous buffer
+  // in outbound_batch_ can be reused.
+  DCHECK_NE(rpc_in_flight_batch_, outbound_batch_.get());
+  if (swap_batch) outbound_batch_.swap(*outbound_batch);
   return Status::OK();
 }
 
 Status KrpcDataStreamSender::Channel::SerializeAndSendBatch(RowBatch* batch) {
-  OutboundRowBatch* outbound_batch = &outbound_batches_[next_batch_idx_];
+  unique_ptr<OutboundRowBatch>* serialization_batch = &parent_->serialization_batch_;
+  DCHECK(serialization_batch->get() != nullptr);
   // Reads 'rpc_in_flight_batch_' without acquiring 'lock_', so reads can be racey.
   ANNOTATE_IGNORE_READS_BEGIN();
-  DCHECK(outbound_batch != rpc_in_flight_batch_);
+  DCHECK(serialization_batch->get() != rpc_in_flight_batch_);
   ANNOTATE_IGNORE_READS_END();
-  RETURN_IF_ERROR(parent_->SerializeBatch(batch, outbound_batch));
-  RETURN_IF_ERROR(TransmitData(outbound_batch));
-  next_batch_idx_ = (next_batch_idx_ + 1) % NUM_OUTBOUND_BATCHES;
+  RETURN_IF_ERROR(parent_->SerializeBatch(batch, serialization_batch->get(), !is_local_));
+  // Swap serialization_batch with outbound_batch_ once the old RPC is finished.
+  RETURN_IF_ERROR(TransmitData(serialization_batch, true /*swap_batch*/));
   return Status::OK();
 }
 
@@ -668,6 +680,12 @@ Status KrpcDataStreamSender::Channel::DoEndDataStreamRpc() {
   DCHECK(rpc_in_flight_);
   EndDataStreamRequestPB eos_req;
   rpc_controller_.Reset();
+  if (FLAGS_data_stream_sender_eos_timeout_ms > 0) {
+    // Provide a timeout so EOS RPCs are prioritized over others, as completing a stream
+    // can help free up resources.
+    rpc_controller_.set_timeout(
+        MonoDelta::FromMilliseconds(FLAGS_data_stream_sender_eos_timeout_ms));
+  }
   *eos_req.mutable_dest_fragment_instance_id() = fragment_instance_id_;
   eos_req.set_sender_id(parent_->sender_id_);
   eos_req.set_dest_node_id(dest_node_id_);
@@ -722,7 +740,7 @@ void KrpcDataStreamSender::Channel::Teardown(RuntimeState* state) {
     while (rpc_in_flight_) rpc_done_cv_.wait(l);
   }
   batch_.reset();
-  outbound_batches_.clear();
+  outbound_batch_.reset(nullptr);
 }
 
 KrpcDataStreamSender::KrpcDataStreamSender(TDataSinkId sink_id, int sender_id,
@@ -738,12 +756,14 @@ KrpcDataStreamSender::KrpcDataStreamSender(TDataSinkId sink_id, int sender_id,
     dest_node_id_(sink.dest_node_id),
     next_unknown_partition_(0),
     exchange_hash_seed_(sink_config.exchange_hash_seed_),
-    hash_and_add_rows_fn_(sink_config.hash_and_add_rows_fn_) {
+    hash_and_add_rows_fn_(sink_config.hash_and_add_rows_fn_),
+    filepath_to_hosts_(sink_config.filepath_to_hosts_) {
   DCHECK_GT(destinations.size(), 0);
   DCHECK(sink.output_partition.type == TPartitionType::UNPARTITIONED
       || sink.output_partition.type == TPartitionType::HASH_PARTITIONED
       || sink.output_partition.type == TPartitionType::RANDOM
-      || sink.output_partition.type == TPartitionType::KUDU);
+      || sink.output_partition.type == TPartitionType::KUDU
+      || sink.output_partition.type == TPartitionType::DIRECTED);
 
   string process_address =
       NetworkAddressPBToString(ExecEnv::GetInstance()->krpc_address());
@@ -753,6 +773,11 @@ KrpcDataStreamSender::KrpcDataStreamSender(TDataSinkId sink_id, int sender_id,
     channels_.emplace_back(new Channel(this, row_desc_, destination.address().hostname(),
         destination.krpc_backend(), destination.fragment_instance_id(), sink.dest_node_id,
         per_channel_buffer_size, is_local));
+
+    if (IsDirectedMode()) {
+      DCHECK(host_to_channel_.find(destination.address()) == host_to_channel_.end());
+      host_to_channel_[destination.address()] = channels_.back().get();
+    }
   }
 
   if (partition_type_ == TPartitionType::UNPARTITIONED
@@ -761,6 +786,8 @@ KrpcDataStreamSender::KrpcDataStreamSender(TDataSinkId sink_id, int sender_id,
     random_shuffle(channels_.begin(), channels_.end());
   }
 
+  DCHECK(filepath_to_hosts_.empty() || partition_type_ == TPartitionType::DIRECTED) <<
+      " TPartitionType: " << partition_type_ << " dest ID: " << dest_node_id_;
 }
 
 KrpcDataStreamSender::~KrpcDataStreamSender() {
@@ -797,16 +824,17 @@ Status KrpcDataStreamSender::Prepare(
       new MemTracker(-1, "RowBatchSerialization", mem_tracker_.get()));
   char_mem_tracker_allocator_.reset(
       new CharMemTrackerAllocator(outbound_rb_mem_tracker_));
+
   string process_address =
        NetworkAddressPBToString(ExecEnv::GetInstance()->krpc_address());
-  for (int i = 0; i < NUM_OUTBOUND_BATCHES; ++i) {
-    // Only skip compression if there is a single channel and destination is in the same
-    // process. TODO: could be optimized to send the uncompressed buffer to the local
-    // targets to avoid decompression cost at the receiver.
-    bool is_local = channels_.size() == 1 && channels_[0]->IsLocal();
-    outbound_batches_.emplace_back(char_mem_tracker_allocator_, is_local);
+
+  serialization_batch_.reset(new OutboundRowBatch(char_mem_tracker_allocator_));
+  if (partition_type_ == TPartitionType::UNPARTITIONED) {
+    in_flight_batch_.reset(new OutboundRowBatch(char_mem_tracker_allocator_));
   }
 
+  compression_scratch_.reset(new TrackedString(*char_mem_tracker_allocator_.get()));
+
   for (int i = 0; i < channels_.size(); ++i) {
     RETURN_IF_ERROR(channels_[i]->Init(state, char_mem_tracker_allocator_));
   }
@@ -952,6 +980,8 @@ string KrpcDataStreamSenderConfig::PartitionTypeName() const {
     return "Random Partitioned";
   case TPartitionType::KUDU:
     return "Kudu Partitioned";
+  case TPartitionType::DIRECTED:
+    return "Directed distribution mode";
   default:
     DCHECK(false) << partition_type_;
     return "";
@@ -1023,14 +1053,22 @@ Status KrpcDataStreamSender::Send(RuntimeState* state, RowBatch* batch) {
 
   if (batch->num_rows() == 0) return Status::OK();
   if (partition_type_ == TPartitionType::UNPARTITIONED) {
-    OutboundRowBatch* outbound_batch = &outbound_batches_[next_batch_idx_];
-    RETURN_IF_ERROR(SerializeBatch(batch, outbound_batch, channels_.size()));
+    // Only skip compression if there is a single channel and destination is in the same
+    // process. TODO: could be optimized to send the uncompressed buffer to the local
+    // targets to avoid decompression cost at the receiver.
+    bool is_local = channels_.size() == 1 && channels_[0]->IsLocal();
+    RETURN_IF_ERROR(SerializeBatch(
+        batch, serialization_batch_.get(), !is_local,  channels_.size()));
     // TransmitData() will block if there are still in-flight rpcs (and those will
-    // reference the previously written serialized batch).
+    // reference the previously written in_flight_batch_).
     for (int i = 0; i < channels_.size(); ++i) {
-      RETURN_IF_ERROR(channels_[i]->TransmitData(outbound_batch));
+      // Do not swap serialization_batch_ with the channel's outbound_batch_ to allow
+      // multiple channels to use the data backing serialization_batch_ in parallel.
+      RETURN_IF_ERROR(
+          channels_[i]->TransmitData(&serialization_batch_, false /*swap_batch*/));
     }
-    next_batch_idx_ = (next_batch_idx_ + 1) % NUM_OUTBOUND_BATCHES;
+    // At this point no RPCs can still refer to the old in_flight_batch_.
+    in_flight_batch_.swap(serialization_batch_);
   } else if (partition_type_ == TPartitionType::RANDOM || channels_.size() == 1) {
     // Round-robin batches among channels. Wait for the current channel to finish its
     // rpc before overwriting its batch.
@@ -1064,6 +1102,67 @@ Status KrpcDataStreamSender::Send(RuntimeState* state, RowBatch* batch) {
         RETURN_IF_ERROR(channels_[channel_ids[i]]->AddRow(row));
       }
     }
+  } else if (partition_type_ == TPartitionType::DIRECTED) {
+    const int num_rows = batch->num_rows();
+    char* prev_filename_ptr = nullptr;
+    vector<Channel*> prev_channels;
+    bool skipped_prev_row = false;
+    for (int row_idx = 0; row_idx < num_rows; ++row_idx) {
+      DCHECK_EQ(batch->num_tuples_per_row(), 1);
+      TupleRow* tuple_row = batch->GetRow(row_idx);
+      Tuple* row = batch->GetRow(row_idx)->GetTuple(0);
+      StringValue* filename_value = row->GetStringSlot(0);
+      DCHECK(filename_value != nullptr);
+      StringValue::SimpleString filename_value_ss = filename_value->ToSimpleString();
+      if (filename_value_ss.ptr == prev_filename_ptr) {
+        // If the filename pointer is the same as the previous one then we can instantly
+        // send the row to the same channels as the previous row.
+        DCHECK(skipped_prev_row || !prev_channels.empty() ||
+            (filename_value_ss.len == 0 && prev_channels.empty()));
+        for (Channel* ch : prev_channels) RETURN_IF_ERROR(ch->AddRow(tuple_row));
+        continue;
+      }
+      prev_filename_ptr = filename_value_ss.ptr;
+      string filename(filename_value_ss.ptr, filename_value_ss.len);
+
+      const auto filepath_to_hosts_it = filepath_to_hosts_.find(filename);
+      if (filepath_to_hosts_it == filepath_to_hosts_.end()) {
+        // This can happen when e.g. compaction removed some data files from a snapshot
+        // but a delete file referencing them remained because it references other data
+        // files that remains in the new snapshot.
+        // Another use-case is table sampling where we read only a subset of the data
+        // files.
+        // A third case is when the delete record is invalid.
+        if (UNLIKELY(filename_value_ss.len == 0)) {
+          state->LogError(
+            ErrorMsg(TErrorCode::GENERAL, "NULL found as file_path in delete file"));
+        } else {
+          VLOG(3) << "Row from delete file refers to a non-existing data file: " <<
+              filename;
+        }
+        skipped_prev_row = true;
+        continue;
+      }
+      skipped_prev_row = false;
+
+      prev_channels.clear();
+      for (const NetworkAddressPB& host_addr : filepath_to_hosts_it->second) {
+        const auto channel_map_it = host_to_channel_.find(host_addr);
+        if (channel_map_it == host_to_channel_.end()) {
+          DumpFilenameToHostsMapping();
+          DumpDestinationHosts();
+
+          stringstream ss;
+          ss << "Failed to distribute Iceberg delete file content"
+              " in DIRECTED distribution mode. Host not found " << host_addr <<
+              ". Try 'SET DISABLE_OPTIMIZED_ICEBERG_V2_READ=1' as a workaround.";
+          return Status(ss.str());
+        }
+
+        prev_channels.push_back(channel_map_it->second);
+        RETURN_IF_ERROR(channel_map_it->second->AddRow(tuple_row));
+      }
+    }
   } else {
     DCHECK_EQ(partition_type_, TPartitionType::HASH_PARTITIONED);
     const KrpcDataStreamSenderConfig::HashAndAddRowsFn hash_and_add_rows_fn =
@@ -1080,6 +1179,26 @@ Status KrpcDataStreamSender::Send(RuntimeState* state, RowBatch* batch) {
   return Status::OK();
 }
 
+void KrpcDataStreamSender::DumpFilenameToHostsMapping() const {
+  VLOG(3) << "Dumping the contents of the filename to hosts mapping";
+  if (filepath_to_hosts_.empty()) {
+    VLOG(3) << "The mapping is empty";
+    return;
+  }
+  for (const auto& file_to_hosts : filepath_to_hosts_) {
+    for (const auto& host_addr : file_to_hosts.second) {
+      VLOG(3) << "Filename: " << file_to_hosts.first << " host address: " << host_addr;
+    }
+  }
+}
+
+void KrpcDataStreamSender::DumpDestinationHosts() const {
+  VLOG(3) << "Dumping the destination hosts";
+  for (const auto& host : host_to_channel_) {
+    VLOG(3) << "Network Address: " << host.first;
+  }
+}
+
 Status KrpcDataStreamSender::FlushFinal(RuntimeState* state) {
   SCOPED_TIMER(profile()->total_time_counter());
   DCHECK(!flushed_);
@@ -1112,7 +1231,10 @@ void KrpcDataStreamSender::Close(RuntimeState* state) {
     channels_[i]->Teardown(state);
   }
 
-  outbound_batches_.clear();
+  serialization_batch_.reset(nullptr);
+  in_flight_batch_.reset(nullptr);
+  compression_scratch_.reset(nullptr);
+
   if (outbound_rb_mem_tracker_.get() != nullptr) {
     outbound_rb_mem_tracker_->Close();
   }
@@ -1123,11 +1245,12 @@ void KrpcDataStreamSender::Close(RuntimeState* state) {
 }
 
 Status KrpcDataStreamSender::SerializeBatch(
-    RowBatch* src, OutboundRowBatch* dest, int num_receivers) {
+    RowBatch* src, OutboundRowBatch* dest, bool compress, int num_receivers) {
   VLOG_ROW << "serializing " << src->num_rows() << " rows";
   {
     SCOPED_TIMER(serialize_batch_timer_);
-    RETURN_IF_ERROR(src->Serialize(dest));
+    RETURN_IF_ERROR(
+        src->Serialize(dest, compress ? compression_scratch_.get() : nullptr));
     int64_t uncompressed_bytes = RowBatch::GetDeserializedSize(*dest);
     COUNTER_ADD(uncompressed_bytes_counter_, uncompressed_bytes * num_receivers);
   }
