@@ -20,6 +20,7 @@
 #define IMPALA_RUNTIME_KRPC_DATA_STREAM_SENDER_H
 
 #include <string>
+#include <unordered_map>
 #include <vector>
 
 #include "codegen/impala-ir.h"
@@ -30,17 +31,21 @@
 #include "exprs/scalar-expr.h"
 #include "runtime/mem-tracker.h"
 #include "runtime/outbound-row-batch.h"
+#include "util/container-util.h"
 #include "util/runtime-profile.h"
 
+#include "gen-cpp/common.pb.h"
+
 namespace impala {
 
 class KrpcDataStreamSender;
 class MemTracker;
+class NetworkAddressPB;
+class PlanFragmentDestinationPB;
 class RowBatch;
 class RowDescriptor;
 class TDataStreamSink;
 class TNetworkAddress;
-class PlanFragmentDestinationPB;
 
 class KrpcDataStreamSenderConfig : public DataSinkConfig {
  public:
@@ -105,7 +110,7 @@ class KrpcDataStreamSender : public DataSink {
   /// 'per_channel_buffer_size' is the soft limit in bytes of the buffering into the
   /// per-channel's accumulating row batch before it will be sent.
   /// NOTE: supported partition types are UNPARTITIONED (broadcast), HASH_PARTITIONED,
-  /// and RANDOM.
+  /// RANDOM, and DIRECTED (used for sending rows from Iceberg delete files).
   KrpcDataStreamSender(TDataSinkId sink_id, int sender_id,
       const KrpcDataStreamSenderConfig& sink_config, const TDataStreamSink& sink,
       const google::protobuf::RepeatedPtrField<PlanFragmentDestinationPB>& destinations,
@@ -161,9 +166,11 @@ class KrpcDataStreamSender : public DataSink {
 
   /// Serializes the src batch into the serialized row batch 'dest' and updates
   /// various stat counters.
+  /// 'compress' decides whether compression is attempted after serialization.
   /// 'num_receivers' is the number of receivers this batch will be sent to. Used for
   /// updating the stat counters.
-  Status SerializeBatch(RowBatch* src, OutboundRowBatch* dest, int num_receivers = 1);
+  Status SerializeBatch(
+      RowBatch* src, OutboundRowBatch* dest, bool compress, int num_receivers = 1);
 
   /// Returns 'partition_expr_evals_[i]'. Used by the codegen'd HashRow() IR function.
   ScalarExprEvaluator* GetPartitionExprEvaluator(int i);
@@ -185,6 +192,12 @@ class KrpcDataStreamSender : public DataSink {
   /// Adds the given row to 'channels_[channel_id]'.
   Status AddRowToChannel(const int channel_id, TupleRow* row);
 
+  /// Functions to dump the content of the "filename to hosts" related mappings into logs.
+  void DumpFilenameToHostsMapping() const;
+  void DumpDestinationHosts() const;
+
+  bool IsDirectedMode() const { return !filepath_to_hosts_.empty(); }
+
   /// Sender instance id, unique within a fragment.
   const int sender_id_;
 
@@ -200,15 +213,20 @@ class KrpcDataStreamSender : public DataSink {
   /// Index of the current channel to send to if random_ == true.
   int current_channel_idx_ = 0;
 
-  /// Index of the next OutboundRowBatch to use for serialization.
-  int next_batch_idx_ = 0;
+  /// Pointer to OutboundRowBatch that will be used for serialization.
+  /// Swapped with in_flight_batch_ (UNPARTITIONED case) or the channel's outbound_batch_
+  /// (PARTITIONED case) after serialization.
+  std::unique_ptr<OutboundRowBatch> serialization_batch_;
 
-  /// The outbound row batches are double-buffered so that we can serialize the next
-  /// batch while the other is still referenced by the in-flight RPC. Each entry contains
-  /// a RowBatchHeaderPB and buffers for the serialized tuple offsets and data. Used only
-  /// when the partitioning strategy is UNPARTITIONED.
-  static const int NUM_OUTBOUND_BATCHES = 2;
-  std::vector<OutboundRowBatch> outbound_batches_;
+  /// Buffer used for compression after serialization. Swapped with the OutboundRowBatch's
+  /// tuple_data_ if the compressed data is smaller.
+  std::unique_ptr<TrackedString> compression_scratch_;
+
+  /// Pointer to OutboundRowBatch referenced by the in-flight RPC(s). Used only
+  /// when the partitioning strategy is UNPARTITIONED. In the broadcasting case
+  /// multiple channels use it at the same time to hold the buffers backing the RPC
+  /// sidecars.
+  std::unique_ptr<OutboundRowBatch> in_flight_batch_;
 
   /// If true, this sender has called FlushFinal() successfully.
   /// Not valid to call Send() anymore.
@@ -278,6 +296,14 @@ class KrpcDataStreamSender : public DataSink {
   /// Pointer for the codegen'd HashAndAddRows() function.
   /// NULL if codegen is disabled or failed.
   const CodegenFnPtr<KrpcDataStreamSenderConfig::HashAndAddRowsFn>& hash_and_add_rows_fn_;
+
+  /// Mapping to store which data file is read on which hosts.
+  const std::unordered_map<std::string, std::vector<NetworkAddressPB>>&
+      filepath_to_hosts_;
+
+  /// A mapping between host addresses to channels. Used for DIRECTED distribution mode
+  /// where only one channel is associated with each host address.
+  std::unordered_map<NetworkAddressPB, Channel*> host_to_channel_;
 };
 
 } // namespace impala
