@@ -218,35 +218,35 @@ Status QueryState::Init(const ExecQueryFInstancesRequestPB* exec_rpc_params,
   // Initialize resource tracking counters.
   if (query_ctx().trace_resource_usage) {
     SystemStateInfo* system_state_info = exec_env->system_state_info();
-    host_profile_->AddChunkedTimeSeriesCounter(
+    host_profile_->AddSamplingTimeSeriesCounter(
         "HostCpuUserPercentage", TUnit::BASIS_POINTS, [system_state_info] () {
-        return system_state_info->GetCpuUsageRatios().user;
+        return system_state_info->GetCpuUsageRatios().user.Load();
         });
-    host_profile_->AddChunkedTimeSeriesCounter(
+    host_profile_->AddSamplingTimeSeriesCounter(
         "HostCpuSysPercentage", TUnit::BASIS_POINTS, [system_state_info] () {
-        return system_state_info->GetCpuUsageRatios().system;
+        return system_state_info->GetCpuUsageRatios().system.Load();
         });
-    host_profile_->AddChunkedTimeSeriesCounter(
+    host_profile_->AddSamplingTimeSeriesCounter(
         "HostCpuIoWaitPercentage", TUnit::BASIS_POINTS, [system_state_info] () {
-        return system_state_info->GetCpuUsageRatios().iowait;
+        return system_state_info->GetCpuUsageRatios().iowait.Load();
         });
     // Add network usage
-    host_profile_->AddChunkedTimeSeriesCounter(
+    host_profile_->AddSamplingTimeSeriesCounter(
         "HostNetworkRx", TUnit::BYTES_PER_SECOND, [system_state_info] () {
-        return system_state_info->GetNetworkUsage().rx_rate;
+        return system_state_info->GetNetworkUsage().rx_rate.Load();
         });
-    host_profile_->AddChunkedTimeSeriesCounter(
+    host_profile_->AddSamplingTimeSeriesCounter(
         "HostNetworkTx", TUnit::BYTES_PER_SECOND, [system_state_info] () {
-        return system_state_info->GetNetworkUsage().tx_rate;
+        return system_state_info->GetNetworkUsage().tx_rate.Load();
         });
     // Add disk stats
-    host_profile_->AddChunkedTimeSeriesCounter(
+    host_profile_->AddSamplingTimeSeriesCounter(
         "HostDiskReadThroughput", TUnit::BYTES_PER_SECOND, [system_state_info] () {
-        return system_state_info->GetDiskStats().read_rate;
+        return system_state_info->GetDiskStats().read_rate.Load();
         });
-    host_profile_->AddChunkedTimeSeriesCounter(
+    host_profile_->AddSamplingTimeSeriesCounter(
         "HostDiskWriteThroughput", TUnit::BYTES_PER_SECOND, [system_state_info] () {
-        return system_state_info->GetDiskStats().write_rate;
+        return system_state_info->GetDiskStats().write_rate.Load();
         });
   }
 
@@ -268,6 +268,17 @@ Status QueryState::Init(const ExecQueryFInstancesRequestPB* exec_rpc_params,
   RETURN_IF_ERROR(
       ControlService::GetProxy(coord_addr, query_ctx().coord_hostname, &proxy_));
 
+  // Making a copy of the "filepath to hosts" mapping into std library types.
+  for (const auto& nodes : exec_rpc_params->by_node_filepath_to_hosts()) {
+    for (const auto& files : nodes.second.filepath_to_hosts()) {
+      FileSchedulingInfo file_schedule_info(files.first, files.second.is_relative());
+      for (const auto& host : files.second.hosts()) {
+        file_schedule_info.hosts.push_back(host);
+      }
+      node_to_file_schedulings_[nodes.first].push_back(file_schedule_info);
+    }
+  }
+
   // don't copy query_ctx, it's large and we already did that in the c'tor
   exec_rpc_params_.set_coord_state_idx(exec_rpc_params->coord_state_idx());
   exec_rpc_params_.mutable_fragment_ctxs()->Swap(
@@ -368,6 +379,7 @@ bool VerifyFiltersProduced(const vector<TPlanFragmentInstanceCtx>& instance_ctxs
 }
 
 Status QueryState::InitFilterBank() {
+  const NetworkAddressPB& this_krpc_address = ExecEnv::GetInstance()->krpc_address();
   int64_t runtime_filters_reservation_bytes = 0;
   int fragment_ctx_idx = -1;
   const vector<TPlanFragment>& fragments = fragment_info_.fragments;
@@ -417,9 +429,32 @@ Status QueryState::InitFilterBank() {
     for (const TRuntimeFilterSource& produced_filter : instance_ctx.filters_produced) {
       auto it = filters.find(produced_filter.filter_id);
       DCHECK(it != filters.end());
-      ++it->second.num_producers;
+      FilterRegistration& reg = it->second;
+      ++reg.num_producers;
+
+      if (produced_filter.__isset.aggregator_desc) {
+        TRuntimeFilterAggDesc agg_desc = produced_filter.aggregator_desc;
+        if (reg.need_subaggregation) {
+          // This filter registration is already set before.
+          // Do sanity check to make sure it match with the rest of TRuntimeFilterSource.
+          DCHECK_EQ(reg.num_reporting_hosts, agg_desc.num_reporting_hosts);
+          DCHECK_EQ(reg.krpc_hostname_to_report, agg_desc.krpc_hostname);
+          DCHECK_EQ(reg.krpc_backend_to_report, agg_desc.krpc_address);
+        } else {
+          // This filter bank is a backend aggregator for 'filter'.
+          reg.need_subaggregation = true;
+          reg.num_reporting_hosts = agg_desc.num_reporting_hosts;
+          reg.krpc_hostname_to_report = agg_desc.krpc_hostname;
+          reg.krpc_backend_to_report = agg_desc.krpc_address;
+          if (KrpcAddressEqual(
+                  this_krpc_address, FromTNetworkAddress(agg_desc.krpc_address))) {
+            reg.is_intermediate_aggregator = true;
+          }
+        }
+      }
     }
   }
+
   filter_bank_.reset(
       new RuntimeFilterBank(this, filters, runtime_filters_reservation_bytes));
   return filter_bank_->ClaimBufferReservation();
@@ -806,6 +841,11 @@ bool QueryState::codegen_cache_enabled() const {
       && ExecEnv::GetInstance()->codegen_cache_enabled();
 }
 
+bool QueryState::is_initialized() {
+  std::lock_guard<std::mutex> l(init_lock_);
+  return is_initialized_;
+}
+
 bool QueryState::StartFInstances() {
   VLOG(2) << "StartFInstances(): query_id=" << PrintId(query_id())
           << " #instances=" << fragment_info_.fragment_instance_ctxs.size();
@@ -993,6 +1033,12 @@ void QueryState::PublishFilter(const PublishFilterParamsPB& params, RpcContext*
   filter_bank_->PublishGlobalFilter(params, context);
 }
 
+void QueryState::UpdateFilterFromRemote(
+    const UpdateFilterParamsPB& params, RpcContext* context) {
+  if (!WaitForPrepare().ok()) return;
+  filter_bank_->UpdateFilterFromRemote(params, context);
+}
+
 Status QueryState::StartSpilling(RuntimeState* runtime_state, MemTracker* mem_tracker) {
   // Return an error message with the root cause of why spilling is disabled.
   if (query_options().scratch_limit == 0) {
