@@ -21,6 +21,7 @@
 #include <memory>
 #include <string>
 #include <unordered_map>
+#include <vector>
 
 #include "common/atomic.h"
 #include "common/compiler-util.h"
@@ -63,6 +64,7 @@ class ScalarExpr;
 class ScannerMemLimiter;
 class TmpFileGroup;
 class TRuntimeProfileForest;
+class UpdateFilterParamsPB;
 
 /// Central class for all backend execution state (example: the FragmentInstanceStates
 /// of the individual fragment instances) created for a particular query.
@@ -119,6 +121,20 @@ class TRuntimeProfileForest;
 /// - set up kudu clients in Init(), remove related locking
 class QueryState {
  public:
+  /// Stores information about which file is scheduled to which hosts.
+  struct FileSchedulingInfo {
+    std::string file_path;
+    std::vector<NetworkAddressPB> hosts;
+    bool is_relative_path;
+
+    FileSchedulingInfo(const string& fn, bool is_rel)
+        : file_path(fn), is_relative_path(is_rel) {}
+  };
+
+  /// Stores information about which file is scheduled to which hosts, grouped by scan
+  /// node ID.
+  typedef std::unordered_map<int, std::vector<FileSchedulingInfo>> NodeToFileSchedulings;
+
   /// Use this class to obtain a QueryState for the duration of a function/block,
   /// rather than manually via QueryExecMgr::Get-/ReleaseQueryState().
   /// Pattern:
@@ -152,8 +168,12 @@ class QueryState {
     return query_ctx_.client_request.query_options;
   }
   bool codegen_cache_enabled() const;
+  bool is_initialized();
   MemTracker* query_mem_tracker() const { return query_mem_tracker_; }
   RuntimeProfile* host_profile() const { return host_profile_; }
+  const NodeToFileSchedulings* node_to_file_schedulings() const {
+    return &node_to_file_schedulings_;
+  }
   UniqueIdPB GetCoordinatorBackendId() const;
 
   /// The following getters are only valid after Init().
@@ -217,6 +237,10 @@ class QueryState {
   /// Blocks until all fragment instances have finished their Prepare phase.
   void PublishFilter(const PublishFilterParamsPB& params, kudu::rpc::RpcContext* context);
 
+  /// Handle filter update that comes from remote backend executor.
+  void UpdateFilterFromRemote(
+      const UpdateFilterParamsPB& params, kudu::rpc::RpcContext* context);
+
   /// Cancels all actively executing fragment instances. Blocks until all fragment
   /// instances have finished their Prepare phase. Idempotent.
   /// For uninitialized QueryState, just set is_cancelled_ and don't need to cancel
@@ -297,6 +321,16 @@ class QueryState {
     return fragment_state_map_;
   }
 
+  /// Returns true if the query has reached a terminal state.
+  bool IsTerminalState() const {
+    // Read into local variable to protect against concurrent modification
+    // of backend_exec_state_.
+    BackendExecState exec_state = backend_exec_state_;
+    return exec_state == BackendExecState::FINISHED
+        || exec_state == BackendExecState::CANCELLED
+        || exec_state == BackendExecState::ERROR;
+  }
+
  private:
   friend class QueryExecMgr;
 
@@ -379,6 +413,9 @@ class QueryState {
   ExecQueryFInstancesRequestPB exec_rpc_params_;
   TExecPlanFragmentInfo fragment_info_;
 
+  /// Stores which data file is scheduled to which host, grouped by scan node ID.
+  NodeToFileSchedulings node_to_file_schedulings_;
+
   /// Buffer reservation for this query (owned by obj_pool_). Set in Init().
   ReservationTracker* buffer_reservation_ = nullptr;
 
@@ -522,13 +559,6 @@ class QueryState {
     return !overall_status_.ok() && !overall_status_.IsCancelled();
   }
 
-  /// Returns true if the query has reached a terminal state.
-  bool IsTerminalState() const {
-    return backend_exec_state_ == BackendExecState::FINISHED
-        || backend_exec_state_ == BackendExecState::CANCELLED
-        || backend_exec_state_ == BackendExecState::ERROR;
-  }
-
   /// Updates the BackendExecState based on 'overall_status_'. Should only be called when
   /// the current state is a non-terminal state. The transition can either be to the next
   /// legal state or ERROR if 'overall_status_' is an error. Called by the query state
