@@ -81,8 +81,9 @@ class RowBatchSerializeTest : public testing::Test {
       bool print_batches, bool full_dedup = false) {
     if (print_batches) cout << PrintBatch(batch) << endl;
 
+    TrackedString compression_scratch(*char_mem_tracker_allocator_.get());
     OutboundRowBatch row_batch(char_mem_tracker_allocator_);
-    RETURN_IF_ERROR(batch->Serialize(&row_batch, full_dedup));
+    RETURN_IF_ERROR(batch->Serialize(&row_batch, full_dedup, &compression_scratch));
 
     RowBatch deserialized_batch(&row_desc, row_batch, tracker_.get());
     if (print_batches) cout << PrintBatch(&deserialized_batch) << endl;
@@ -213,7 +214,7 @@ class RowBatchSerializeTest : public testing::Test {
         StringValue* string_value = reinterpret_cast<StringValue*>(slot);
         StringValue* deserialized_string_value =
             reinterpret_cast<StringValue*>(deserialized_slot);
-        EXPECT_NE(string_value->ptr, deserialized_string_value->ptr);
+        EXPECT_NE(string_value->Ptr(), deserialized_string_value->Ptr());
       }
 
       if (type.IsCollectionType()) {
@@ -613,8 +614,10 @@ void RowBatchSerializeTest::TestDupRemoval(bool full_dedup) {
   vector<Tuple*> tuples;
   CreateTuples(tuple_desc, batch->tuple_data_pool(), num_distinct_tuples, 0, 10, &tuples);
   AddTuplesToRowBatch(num_rows, tuples, repeats, batch);
+
+  TrackedString compression_scratch(*char_mem_tracker_allocator_.get());
   OutboundRowBatch row_batch(char_mem_tracker_allocator_);
-  EXPECT_OK(batch->Serialize(&row_batch, full_dedup));
+  EXPECT_OK(batch->Serialize(&row_batch, full_dedup, &compression_scratch));
   // Serialized data should only have one copy of each tuple.
   int64_t total_byte_size = 0; // Total size without duplication
   for (int i = 0; i < tuples.size(); ++i) {
@@ -751,8 +754,10 @@ TEST_F(RowBatchSerializeTest, DedupPathologicalFull) {
   // Full dedup should be automatically enabled because of row batch structure.
   EXPECT_TRUE(UseFullDedup(batch));
   LOG(INFO) << "Serializing row batch";
+
+  TrackedString compression_scratch(*char_mem_tracker_allocator_.get());
   OutboundRowBatch row_batch(char_mem_tracker_allocator_);
-  EXPECT_OK(batch->Serialize(&row_batch));
+  EXPECT_OK(batch->Serialize(&row_batch, &compression_scratch));
   LOG(INFO) << "Serialized batch size: " << row_batch.TupleDataAsSlice().size();
   LOG(INFO) << "Serialized batch uncompressed size: "
             << row_batch.header()->uncompressed_size();
