@@ -42,6 +42,7 @@
 namespace impala {
 
 const char* Tuple::LLVM_CLASS_NAME = "class.impala::Tuple";
+const char* Tuple::CodegenTypes::LLVM_CLASS_NAME = "struct.impala::Tuple::CodegenTypes";
 const char* SlotOffsets::LLVM_CLASS_NAME = "struct.impala::SlotOffsets";
 
 const char* Tuple::MATERIALIZE_EXPRS_SYMBOL = "MaterializeExprsILb0ELb0";
@@ -63,7 +64,9 @@ int64_t Tuple::VarlenByteSize(const TupleDescriptor& desc) const {
     DCHECK((*slot)->type().IsVarLenStringType());
     if (IsNull((*slot)->null_indicator_offset())) continue;
     const StringValue* string_val = GetStringSlot((*slot)->tuple_offset());
-    result += string_val->len;
+    // Small strings don't require extra storage space in the varlen section.
+    if (string_val->IsSmall()) continue;
+    result += string_val->Len();
   }
 
   slot = desc.collection_slots().begin();
@@ -81,6 +84,19 @@ int64_t Tuple::VarlenByteSize(const TupleDescriptor& desc) const {
   return result;
 }
 
+inline void Tuple::SmallifyStrings(const TupleDescriptor& desc) {
+  for (const SlotDescriptor* slot : desc.string_slots()) {
+    DCHECK(slot->type().IsVarLenStringType());
+    if (IsNull(slot->null_indicator_offset())) continue;
+    StringValue* string_v = GetStringSlot(slot->tuple_offset());
+    // StringValues are only smallified on a on-demand basis. And we only smallify
+    // them in batches for whole tuples. I.e. if we encounter the first small string
+    // in a tuple we can assume that the rest of the strings are already smallified.
+    if (string_v->IsSmall()) return;
+    string_v->Smallify();
+  }
+}
+
 Tuple* Tuple::DeepCopy(const TupleDescriptor& desc, MemPool* pool) {
   Tuple* result = reinterpret_cast<Tuple*>(pool->Allocate(desc.byte_size()));
   DeepCopy(result, desc, pool);
@@ -91,8 +107,14 @@ Tuple* Tuple::DeepCopy(const TupleDescriptor& desc, MemPool* pool) {
 // memory is allocated - can we templatise it somehow to avoid redundancy without runtime
 // overhead.
 void Tuple::DeepCopy(Tuple* dst, const TupleDescriptor& desc, MemPool* pool) {
-  memcpy(dst, this, desc.byte_size());
-  if (desc.HasVarlenSlots()) dst->DeepCopyVarlenData(desc, pool);
+  if (desc.HasVarlenSlots()) {
+    memcpy(dst, this, desc.byte_size());
+    // 'dst' is a new tuple, so it is safe to smallify its string values.
+    dst->SmallifyStrings(desc);
+    dst->DeepCopyVarlenData(desc, pool);
+  } else {
+    memcpy(dst, this, desc.byte_size());
+  }
 }
 
 void Tuple::DeepCopyVarlenData(const TupleDescriptor& desc, MemPool* pool) {
@@ -102,9 +124,10 @@ void Tuple::DeepCopyVarlenData(const TupleDescriptor& desc, MemPool* pool) {
     DCHECK((*slot)->type().IsVarLenStringType());
     if (IsNull((*slot)->null_indicator_offset())) continue;
     StringValue* string_v = GetStringSlot((*slot)->tuple_offset());
-    char* string_copy = reinterpret_cast<char*>(pool->Allocate(string_v->len));
-    Ubsan::MemCpy(string_copy, string_v->ptr, string_v->len);
-    string_v->ptr = string_copy;
+    if (string_v->IsSmall()) continue;
+    char* string_copy = reinterpret_cast<char*>(pool->Allocate(string_v->Len()));
+    Ubsan::MemCpy(string_copy, string_v->Ptr(), string_v->Len());
+    string_v->SetPtr(string_copy);
   }
 
   for (vector<SlotDescriptor*>::const_iterator slot = desc.collection_slots().begin();
@@ -144,10 +167,12 @@ void Tuple::DeepCopyVarlenData(const TupleDescriptor& desc, char** data, int* of
     if (IsNull((*slot)->null_indicator_offset())) continue;
 
     StringValue* string_v = GetStringSlot((*slot)->tuple_offset());
-    Ubsan::MemCpy(*data, string_v->ptr, string_v->len);
-    string_v->ptr = convert_ptrs ? reinterpret_cast<char*>(*offset) : *data;
-    *data += string_v->len;
-    *offset += string_v->len;
+    if (string_v->IsSmall()) continue;
+    unsigned int len = string_v->Len();
+    Ubsan::MemCpy(*data, string_v->Ptr(), len);
+    string_v->SetPtr(convert_ptrs ? reinterpret_cast<char*>(*offset) : *data);
+    *data += len;
+    *offset += len;
   }
 
   slot = desc.collection_slots().begin();
@@ -183,8 +208,9 @@ void Tuple::ConvertOffsetsToPointers(const TupleDescriptor& desc, uint8_t* tuple
     if (IsNull((*slot)->null_indicator_offset())) continue;
 
     StringValue* string_val = GetStringSlot((*slot)->tuple_offset());
-    int offset = reinterpret_cast<intptr_t>(string_val->ptr);
-    string_val->ptr = reinterpret_cast<char*>(tuple_data + offset);
+    if (string_val->IsSmall()) continue;
+    int offset = reinterpret_cast<intptr_t>(string_val->Ptr());
+    string_val->SetPtr(reinterpret_cast<char*>(tuple_data + offset));
   }
 
   slot = desc.collection_slots().begin();
@@ -217,13 +243,41 @@ void Tuple::SetNullIndicators(NullIndicatorOffset offset, int64_t num_tuples,
   }
 }
 
+// Note: 'noexcept' is added because otherwise in LLVM code this function is called with
+// the 'invoke' instruction, not the 'call' instruction. This is bad because we can't
+// replace functions called with 'invoke' with a codegen'd function.
+//
+// The 'invoke' instruction is for functions that can throw exceptions. It would be chosen
+// here - if we didn't add 'noexcept' - because std::vector<...> (the actual type of
+// 'non_null_string_values' and 'non_null_collection_values') can throw if it fails to
+// allocate memory when it tries to grow. If a noexcept function throws, std::terminate()
+// is called (so no Undefined Behaviour).
+//
+// Currently the hand-crafted function we replace this function with doesn't use the
+// vectors (COLLECT_VAR_LEN_VALS is always false), so no exception will be thrown in
+// codegen code.
+//
+// In interpreted mode, if we crash because of std::terminate(), it is the same as what we
+// did before changing the type of these parameters to std::vector<...>: vector resizing
+// was done in the caller, but there was no catch block for it. This will also be the case
+// if the hand-crafted function in codegen code starts using the vectors.
 template <bool COLLECT_VAR_LEN_VALS, bool NO_POOL>
 void Tuple::MaterializeExprs(TupleRow* row, const TupleDescriptor& desc,
     ScalarExprEvaluator* const* evals, MemPool* pool,
-    StringValue** non_null_string_values, CollValueAndSize* non_null_collection_values,
+    CodegenTypes::StringValuePtrVecType* non_null_string_values,
+    CodegenTypes::CollValuePtrAndSizeVecType* non_null_collection_values,
     int* total_varlen_lengths,
-    int* num_non_null_string_values, int* num_non_null_collection_values) {
+    int* num_non_null_string_values, int* num_non_null_collection_values) noexcept {
+  if constexpr (COLLECT_VAR_LEN_VALS) {
+    DCHECK(non_null_string_values != nullptr);
+    DCHECK(non_null_collection_values != nullptr);
+    DCHECK(total_varlen_lengths != nullptr);
+    DCHECK(num_non_null_string_values != nullptr);
+    DCHECK(num_non_null_collection_values != nullptr);
+  }
+
   ClearNullBits(desc);
+
   // Evaluate the materialize_expr_evals and place the results in the tuple.
   for (int i = 0; i < desc.slots().size(); ++i) {
     SlotDescriptor* slot_desc = desc.slots()[i];
@@ -234,26 +288,30 @@ void Tuple::MaterializeExprs(TupleRow* row, const TupleDescriptor& desc,
         slot_desc->type() == evals[i]->root().type());
     void* src = evals[i]->GetValue(row);
 
-    vector<StringValue*> string_values;
-    vector<pair<CollectionValue*, int64_t>> collection_values;
+    const size_t old_num_string_values = COLLECT_VAR_LEN_VALS ?
+        non_null_string_values->size() : 0;
+    const size_t old_num_collection_values = COLLECT_VAR_LEN_VALS ?
+        non_null_collection_values->size() : 0;
     RawValue::Write<COLLECT_VAR_LEN_VALS>(src, this, slot_desc, pool,
-        &string_values, &collection_values);
-    if (string_values.size() > 0) {
-      for (StringValue* string_val : string_values) {
-        *(non_null_string_values++) = string_val;
-        *total_varlen_lengths += string_val->len;
-      }
-      (*num_non_null_string_values) += string_values.size();
-    }
+        non_null_string_values, non_null_collection_values);
 
-    if (collection_values.size() > 0) {
-      for (const pair<CollectionValue*, int64_t>& collection_val_pair
-          : collection_values) {
-        CollValueAndSize cvs(collection_val_pair.first, collection_val_pair.second);
-        *(non_null_collection_values++) = cvs;
+    if constexpr (COLLECT_VAR_LEN_VALS) {
+      const size_t new_num_string_values = non_null_string_values->size();
+      for (size_t i = old_num_string_values; i < new_num_string_values; i++) {
+        const StringValue* string_val = (*non_null_string_values)[i];
+        DCHECK(!string_val->IsSmall());
+        *total_varlen_lengths += string_val->Len();
+      }
+      (*num_non_null_string_values) += new_num_string_values - old_num_string_values;
+
+      const size_t new_num_collection_values = non_null_collection_values->size();
+      for (size_t i = old_num_collection_values; i < new_num_collection_values; i++) {
+        const pair<CollectionValue*, int64_t>& collection_val_pair =
+            (*non_null_collection_values)[i];
         *total_varlen_lengths += collection_val_pair.second;
       }
-      (*num_non_null_collection_values) += collection_values.size();
+      (*num_non_null_collection_values) +=
+          new_num_collection_values - old_num_collection_values;
     }
   }
 }
@@ -290,14 +348,14 @@ char* Tuple::AllocateStrings(const char* err_ctx, RuntimeState* state,
 //   order by l_comment
 //   limit 10;
 //
+//
 // define void @MaterializeExprs(%"class.impala::Tuple"* %opaque_tuple,
 //     %"class.impala::TupleRow"* %row, %"class.impala::TupleDescriptor"* %desc,
 //     %"class.impala::ScalarExprEvaluator"** %slot_materialize_expr_evals,
 //     %"class.impala::MemPool"* %pool,
-//     %"struct.impala::StringValue"** %non_null_string_values,
-//     %"struct.impala::CollValueAndSize"* %non_null_collection_values,
-//     i32* %total_varlen_lengths, i32* %num_non_null_string_values,
-//     i32* %num_non_null_collection_values) #50 {
+//     %"class.std::vector.1154"* %non_null_string_values,
+//     %"class.std::vector.1159"* %non_null_collection_values, i32* %total_varlen_lengths,
+//     i32* %num_non_null_string_values, i32* %num_non_null_collection_values) #50 {
 // entry:
 //   %tuple = bitcast %"class.impala::Tuple"* %opaque_tuple
 //       to <{ %"struct.impala::StringValue", i8 }>*
@@ -311,7 +369,8 @@ char* Tuple::AllocateStrings(const char* err_ctx, RuntimeState* state,
 //   %src = call { i64, i8* } @GetSlotRef.4(
 //       %"class.impala::ScalarExprEvaluator"* %expr_eval,
 //       %"class.impala::TupleRow"* %row)
-//   ; -- generated by CodegenAnyVal::ToReadWriteInfo() and SlotDescriptor::WriteToSlot()
+//   ; -- generated by CodegenAnyVal::ToReadWriteInfo() and
+//   ; -- SlotDescriptor::CodegenWriteToSlot()
 //   br label %entry1
 //
 // entry1:                                           ; preds = %entry
@@ -328,11 +387,11 @@ char* Tuple::AllocateStrings(const char* err_ctx, RuntimeState* state,
 //       <{ %"struct.impala::StringValue", i8 }>* %tuple, i32 0, i32 0
 //   %5 = insertvalue %"struct.impala::StringValue" zeroinitializer, i32 %4, 1
 //   %6 = sext i32 %4 to i64
-//   %new_ptr = call i8* @_ZN6impala7MemPool8AllocateILb0EEEPhli(
+//   %new_coll_val_ptr = call i8* @_ZN6impala7MemPool8AllocateILb0EEEPhli(
 //       %"class.impala::MemPool"* %pool, i64 %6, i32 8)
-//   call void @llvm.memcpy.p0i8.p0i8.i32(i8* %new_ptr, i8* %src2, i32 %4, i32 0,
+//   call void @llvm.memcpy.p0i8.p0i8.i32(i8* %new_coll_val_ptr, i8* %src2, i32 %4, i32 0,
 //       i1 false)
-//   %7 = insertvalue %"struct.impala::StringValue" %5, i8* %new_ptr, 0
+//   %7 = insertvalue %"struct.impala::StringValue" %5, i8* %new_coll_val_ptr, 0
 //   store %"struct.impala::StringValue" %7, %"struct.impala::StringValue"* %slot
 //   br label %end_write
 //
@@ -345,7 +404,7 @@ char* Tuple::AllocateStrings(const char* err_ctx, RuntimeState* state,
 //   br label %end_write
 //
 // end_write:                                        ; preds = %null, %non_null
-//   ; -- end CodegenAnyVal::ToReadWriteInfo() and SlotDescriptor::WriteToSlot() --------
+//   ; -- end CodegenAnyVal::ToReadWriteInfo() and SlotDescriptor::CodegenWriteToSlot()
 //   ret void
 // }
 Status Tuple::CodegenMaterializeExprs(LlvmCodeGen* codegen, bool collect_varlen_vals,
@@ -373,8 +432,8 @@ Status Tuple::CodegenMaterializeExprs(LlvmCodeGen* codegen, bool collect_varlen_
   // used in xcompiled IR). With 'pool':
   // void MaterializeExprs(Tuple* opaque_tuple, TupleRow* row,
   //     const TupleDescriptor& desc, ScalarExprEvaluator* const* evals, MemPool* pool,
-  //     StringValue** non_null_string_values,
-  //     CollValueAndSize* non_null_collection_values,
+  //     CodegenTypes::StringValuePtrVecType* non_null_string_values,
+  //     CodegenTypes::CollValuePtrAndSizeVecType* non_null_collection_values,
   //     int* total_varlen_lengths, int* num_non_null_string_values,
   //     int* num_non_null_collection_values);
   llvm::PointerType* opaque_tuple_type = codegen->GetStructPtrType<Tuple>();
@@ -383,10 +442,12 @@ Status Tuple::CodegenMaterializeExprs(LlvmCodeGen* codegen, bool collect_varlen_
   llvm::PointerType* expr_evals_type =
       codegen->GetStructPtrPtrType<ScalarExprEvaluator>();
   llvm::PointerType* pool_type = codegen->GetStructPtrType<MemPool>();
-  llvm::PointerType* string_values_type = codegen->GetStructPtrPtrType<StringValue>();
-  llvm::PointerType* coll_values_and_sizes_type =
-      codegen->GetStructPtrType<CollValueAndSize>();
+  llvm::Type* string_values_type =
+      CodegenTypes::getStringValuePtrVecType(codegen)->getPointerTo();
+  llvm::Type* coll_values_and_sizes_type =
+      CodegenTypes::getCollValuePtrAndSizeVecType(codegen)->getPointerTo();
   llvm::PointerType* int_ptr_type = codegen->i32_ptr_type();
+
   LlvmCodeGen::FnPrototype prototype(codegen, "MaterializeExprs", codegen->void_type());
   prototype.AddArgument("opaque_tuple", opaque_tuple_type);
   prototype.AddArgument("row", row_type);
@@ -416,9 +477,8 @@ Status Tuple::CodegenMaterializeExprs(LlvmCodeGen* codegen, bool collect_varlen_
 
   // Cast the opaque Tuple* argument to the generated struct type
   llvm::Type* tuple_struct_type = desc.GetLlvmStruct(codegen);
-  if (tuple_struct_type == NULL) {
-    return Status("CodegenMaterializeExprs(): failed to generate tuple desc");
-  }
+  DCHECK(tuple_struct_type != nullptr);
+
   llvm::PointerType* tuple_type = codegen->GetPtrType(tuple_struct_type);
   llvm::Value* tuple = builder.CreateBitCast(opaque_tuple_arg, tuple_type, "tuple");
 
@@ -516,16 +576,26 @@ llvm::Constant* SlotOffsets::ToIR(LlvmCodeGen* codegen) const {
           codegen->GetI32Constant(tuple_offset)});
 }
 
+llvm::Type* Tuple::CodegenTypes::getStringValuePtrVecType(LlvmCodeGen* codegen) {
+  llvm::StructType* codegenTypes = codegen->GetStructType<Tuple::CodegenTypes>();
+  return codegenTypes->getElementType(0);
+}
+
+llvm::Type* Tuple::CodegenTypes::getCollValuePtrAndSizeVecType(LlvmCodeGen* codegen) {
+  llvm::StructType* codegenTypes = codegen->GetStructType<Tuple::CodegenTypes>();
+  return codegenTypes->getElementType(1);
+}
+
 template void Tuple::MaterializeExprs<false, false>(TupleRow*, const TupleDescriptor&,
-    ScalarExprEvaluator* const*, MemPool*, StringValue**, CollValueAndSize*,
-    int*, int*, int*);
+    ScalarExprEvaluator* const*, MemPool*, Tuple::CodegenTypes::StringValuePtrVecType*,
+    Tuple::CodegenTypes::CollValuePtrAndSizeVecType*, int*, int*, int*);
 template void Tuple::MaterializeExprs<false, true>(TupleRow*, const TupleDescriptor&,
-    ScalarExprEvaluator* const*, MemPool*, StringValue**, CollValueAndSize*,
-    int*, int*, int*);
+    ScalarExprEvaluator* const*, MemPool*, Tuple::CodegenTypes::StringValuePtrVecType*,
+    Tuple::CodegenTypes::CollValuePtrAndSizeVecType*, int*, int*, int*);
 template void Tuple::MaterializeExprs<true, false>(TupleRow*, const TupleDescriptor&,
-    ScalarExprEvaluator* const*, MemPool*, StringValue**, CollValueAndSize*,
-    int*, int*, int*);
+    ScalarExprEvaluator* const*, MemPool*, Tuple::CodegenTypes::StringValuePtrVecType*,
+    Tuple::CodegenTypes::CollValuePtrAndSizeVecType*, int*, int*, int*);
 template void Tuple::MaterializeExprs<true, true>(TupleRow*, const TupleDescriptor&,
-    ScalarExprEvaluator* const*, MemPool*, StringValue**, CollValueAndSize*,
-    int*, int*, int*);
+    ScalarExprEvaluator* const*, MemPool*, Tuple::CodegenTypes::StringValuePtrVecType*,
+    Tuple::CodegenTypes::CollValuePtrAndSizeVecType*, int*, int*, int*);
 }
