@@ -339,6 +339,7 @@ class AdmissionController {
   static const std::string PROFILE_INFO_KEY_LAST_QUEUED_REASON;
   static const std::string PROFILE_INFO_KEY_ADMITTED_MEM;
   static const std::string PROFILE_INFO_KEY_EXECUTOR_GROUP;
+  static const std::string PROFILE_INFO_KEY_EXECUTOR_GROUP_QUERY_LOAD;
   static const std::string PROFILE_INFO_KEY_STALENESS_WARNING;
   static const std::string PROFILE_TIME_SINCE_LAST_UPDATE_COUNTER_NAME;
 
@@ -947,6 +948,8 @@ class AdmissionController {
   /// an error if no executor groups are available for scheduling, but will set
   /// 'queue_node->not_admitted_reason' and leave 'queue_node->group_states' empty in
   /// that case.
+  /// If the associated TQueryExecRequest sets include_all_coordinators, an extra group
+  /// containing all active coordinators will be added to ExecutorConfig for scheduling.
   Status ComputeGroupScheduleStates(
       ClusterMembershipMgr::SnapshotPtr membership_snapshot, QueueNode* queue_node);
 
@@ -1167,6 +1170,10 @@ class AdmissionController {
   /// admitted or released.
   void UpdateExecGroupMetric(const string& grp_name, int64_t delta);
 
+  /// Returns the query load for the given executor group.
+  /// If no query load information is available, returns 0.
+  int64_t GetExecGroupQueryLoad(const string& grp_name);
+
   /// A helper type to glue information together to compute the topN queries out of <n>
   /// topM queries through a priority queue. Each object of the type represents a query.
   ///
@@ -1209,6 +1216,7 @@ class AdmissionController {
   FRIEND_TEST(AdmissionControllerTest, CanAdmitRequestMemory);
   FRIEND_TEST(AdmissionControllerTest, CanAdmitRequestCount);
   FRIEND_TEST(AdmissionControllerTest, CanAdmitRequestSlots);
+  FRIEND_TEST(AdmissionControllerTest, CanAdmitRequestSlotsDefault);
   FRIEND_TEST(AdmissionControllerTest, GetMaxToDequeue);
   FRIEND_TEST(AdmissionControllerTest, QueryRejection);
   FRIEND_TEST(AdmissionControllerTest, DedicatedCoordScheduleState);
