@@ -69,8 +69,6 @@ void RemoveExecutorAndGroup(const BackendDescriptorPB& be_desc,
 
 namespace impala {
 
-static const string EMPTY_GROUP_NAME("empty group (using coordinator only)");
-
 static const string LIVE_EXEC_GROUP_KEY("cluster-membership.executor-groups.total");
 static const string HEALTHY_EXEC_GROUP_KEY(
     "cluster-membership.executor-groups.total-healthy");
@@ -83,6 +81,9 @@ static const string HEALTHY_EXEC_GROUP_KEY_FORMAT(
 static const string TOTAL_BACKENDS_KEY_FORMAT(
     "cluster-membership.group-set.backends.total.$0");
 
+const string ClusterMembershipMgr::EMPTY_GROUP_NAME(
+    "empty group (using coordinator only)");
+
 ClusterMembershipMgr::ClusterMembershipMgr(
     string local_backend_id, StatestoreSubscriber* subscriber, MetricGroup* metrics)
   : empty_exec_group_(EMPTY_GROUP_NAME),
@@ -161,6 +162,33 @@ ClusterMembershipMgr::SnapshotPtr ClusterMembershipMgr::GetSnapshot() const {
   return state;
 }
 
+static bool is_active_coordinator(const BackendDescriptorPB& be) {
+  return be.has_is_coordinator() && be.is_coordinator() &&
+      !(be.has_is_quiescing() && be.is_quiescing());
+}
+
+ExecutorGroup ClusterMembershipMgr::Snapshot::GetCoordinators() const {
+  ExecutorGroup coordinators("all-coordinators");
+  for (const auto& it : current_backends) {
+    if (is_active_coordinator(it.second)) {
+      coordinators.AddExecutor(it.second);
+    }
+  }
+  return coordinators;
+}
+
+vector<TNetworkAddress> ClusterMembershipMgr::Snapshot::GetCoordinatorAddresses() const {
+  vector<TNetworkAddress> coordinators;
+  for (const auto& it : current_backends) {
+    if (is_active_coordinator(it.second)) {
+      VLOG_QUERY << "Found coordinator "
+                 << it.second.address().hostname() << ":" << it.second.address().port();
+      coordinators.emplace_back(FromNetworkAddressPB(it.second.address()));
+    }
+  }
+  return coordinators;
+}
+
 void ClusterMembershipMgr::UpdateMembership(
     const StatestoreSubscriber::TopicDeltaMap& incoming_topic_deltas,
     vector<TTopicDelta>* subscriber_topic_updates) {
@@ -185,8 +213,8 @@ void ClusterMembershipMgr::UpdateMembership(
   BeDescSharedPtr local_be_desc = GetLocalBackendDescriptor();
   bool needs_local_be_update = NeedsLocalBackendUpdate(*base_snapshot, local_be_desc);
 
-  // We consider the statestore to be recovering from a connection failure until its post
-  // recovery grace period has elapsed.
+  // We consider the statestore service to be recovering from a connection failure or
+  // fail-over until its post recovery grace period has elapsed.
   bool ss_is_recovering = statestore_subscriber_ != nullptr
       && statestore_subscriber_->IsInPostRecoveryGracePeriod();
 
