@@ -27,6 +27,9 @@ class ExecutorGroupDescPB;
 
 namespace test {
 
+static const int64_t MEGABYTE = 1024L * 1024L;
+static const int64_t GIGABYTE = 1024L * MEGABYTE;
+
 /// Convert a host index to a hostname.
 std::string HostIdxToHostname(int host_idx);
 
@@ -40,14 +43,17 @@ std::string HostIdxToIpAddr(int host_idx);
 ///
 /// Make a backend descriptor for group 'group_desc'.
 BackendDescriptorPB MakeBackendDescriptor(
-    int idx, const ExecutorGroupDescPB& group_desc, int port_offset = 0);
+    int idx, const ExecutorGroupDescPB& group_desc, int port_offset = 0,
+    int64_t admit_mem_limit = 4L * MEGABYTE);
 
 /// Make a backend descriptor for 'group'.
 BackendDescriptorPB MakeBackendDescriptor(
-    int idx, const ExecutorGroup& group, int port_offset = 0);
+    int idx, const ExecutorGroup& group, int port_offset = 0,
+    int64_t admit_mem_limit = 4L * MEGABYTE);
 
 /// Make a backend descriptor for the default executor group.
-BackendDescriptorPB MakeBackendDescriptor(int idx, int port_offset = 0);
+BackendDescriptorPB MakeBackendDescriptor(int idx, int port_offset = 0,
+    int64_t admit_mem_limit = 4L * MEGABYTE);
 
 }  // end namespace test
 }  // end namespace impala
