@@ -111,6 +111,25 @@ TEST_F(SchedulerTest, ExecAtCoord) {
   EXPECT_EQ(0, result.NumTotalAssignedBytes(2));
 }
 
+/// Test cluster configuration with one coordinator that can't process scan ranges
+/// when scheduling to coordinators is enabled.
+TEST_F(SchedulerTest, UseDedicatedCoordinator) {
+  Cluster cluster;
+  cluster.AddHost(true, true, false);
+  cluster.AddHost(true, true, true);
+  cluster.AddHost(true, true, true);
+
+  Schema schema(cluster);
+  Plan plan(schema);
+  plan.AddSystemTableScan();
+
+  Result result(plan);
+  SchedulerWrapper scheduler(plan);
+  ASSERT_OK(scheduler.Compute(&result, true));
+
+  EXPECT_EQ(3, result.NumDistinctBackends());
+}
+
 /// Test scanning a simple table twice.
 TEST_F(SchedulerTest, ScanTableTwice) {
   Cluster cluster;
@@ -642,6 +661,27 @@ TEST_F(SchedulerTest, TestGeneratedVariableSizeSplit) {
   EXPECT_EQ(300, result.NumTotalAssignedBytes());
 }
 
+TEST_F(SchedulerTest, TestGeneratedVariableSizeSplitFooterOnly) {
+  Cluster cluster;
+
+  cluster.AddHosts(3, true, true);
+
+  Schema schema(cluster);
+  schema.AddFileSplitGeneratorSpecs(
+      "T", {{100, 100, false, false}, {100, 1, true, true}, {100, 10, true, true}});
+
+  Plan plan(schema);
+  plan.AddTableScan("T");
+  plan.SetRandomReplica(true);
+
+  Result result(plan);
+  SchedulerWrapper scheduler(plan);
+  ASSERT_OK(scheduler.Compute(&result));
+
+  EXPECT_EQ(3, result.NumTotalAssignments());
+  EXPECT_EQ(111, result.NumTotalAssignedBytes());
+}
+
 TEST_F(SchedulerTest, TestBlockAndGenerateSplit) {
   Cluster cluster;
 
