@@ -56,6 +56,10 @@ using namespace apache::thrift;
 using namespace org::apache::impala::fb;
 using namespace strings;
 
+DEFINE_bool_hidden(sort_runtime_filter_aggregator_candidates, false,
+    "Control whether to sort intermediate runtime filter aggregator candidates based on "
+    "their KRPC address. Only used for testing.");
+
 namespace impala {
 
 static const string LOCAL_ASSIGNMENTS_KEY("simple-scheduler.local-assignments.total");
@@ -65,7 +69,8 @@ static const string SCHEDULER_WARNING_KEY("Scheduler Warning");
 
 static const vector<TPlanNodeType::type> SCAN_NODE_TYPES{TPlanNodeType::HDFS_SCAN_NODE,
     TPlanNodeType::HBASE_SCAN_NODE, TPlanNodeType::DATA_SOURCE_NODE,
-    TPlanNodeType::KUDU_SCAN_NODE};
+    TPlanNodeType::KUDU_SCAN_NODE, TPlanNodeType::ICEBERG_METADATA_SCAN_NODE,
+    TPlanNodeType::SYSTEM_TABLE_SCAN_NODE};
 
 // Consistent scheduling requires picking up to k distinct candidates out of n nodes.
 // Since each iteration can pick a node that it already picked (i.e. it is sampling with
@@ -92,11 +97,18 @@ const BackendDescriptorPB& Scheduler::LookUpBackendDesc(
     const ExecutorConfig& executor_config, const NetworkAddressPB& host) {
   const BackendDescriptorPB* desc = executor_config.group.LookUpBackendDesc(host);
   if (desc == nullptr) {
-    // Coordinator host may not be in executor_config's executor group if it's a dedicated
-    // coordinator, or if it is configured to be in a different executor group.
-    const BackendDescriptorPB& coord_desc = executor_config.coord_desc;
-    DCHECK(host == coord_desc.address());
-    desc = &coord_desc;
+    if (executor_config.all_coords.NumExecutors() > 0) {
+      // Group containing all coordinators was provided, so use that for lookup.
+      desc = executor_config.all_coords.LookUpBackendDesc(host);
+      DCHECK_NE(nullptr, desc);
+    } else {
+      // Coordinator host may not be in executor_config's executor group if it's a
+      // dedicated coordinator, or if it is configured to be in a different executor
+      // group.
+      const BackendDescriptorPB& coord_desc = executor_config.coord_desc;
+      DCHECK(host == coord_desc.address());
+      desc = &coord_desc;
+    }
   }
   return *desc;
 }
@@ -121,8 +133,17 @@ Status Scheduler::GenerateScanRanges(const vector<TFileSplitGeneratorSpec>& spec
 
     long scan_range_offset = 0;
     long remaining = fb_desc->length();
-    long scan_range_length = std::min(spec.max_block_size, fb_desc->length());
-    if (!spec.is_splittable) scan_range_length = fb_desc->length();
+    long scan_range_length = fb_desc->length();
+
+    if (spec.is_splittable) {
+      scan_range_length = std::min(spec.max_block_size, fb_desc->length());
+      if (spec.is_footer_only) {
+        scan_range_offset = fb_desc->length() - scan_range_length;
+        remaining = scan_range_length;
+      }
+    } else {
+      DCHECK(!spec.is_footer_only);
+    }
 
     while (remaining > 0) {
       THdfsFileSplit hdfs_scan_range;
@@ -203,7 +224,8 @@ Status Scheduler::ComputeScanRangeAssignment(
       RETURN_IF_ERROR(
           ComputeScanRangeAssignment(executor_config, node_id, node_replica_preference,
               node_random_replica, *locations, exec_request.host_list, exec_at_coord,
-              state->query_options(), total_assignment_timer, state->rng(), assignment));
+              state->query_options(), total_assignment_timer, state->rng(),
+              state->summary_profile(), assignment));
       state->IncNumScanRanges(locations->size());
     }
   }
@@ -226,12 +248,23 @@ Status Scheduler::ComputeFragmentExecParams(
     // Set destinations, per_exch_num_senders, sender_id.
     for (const TPlanFragment& src_fragment : plan_exec_info.fragments) {
       VLOG(3) << "Computing exec params for fragment " << src_fragment.display_name;
+      if (!src_fragment.output_sink.__isset.stream_sink
+          && !src_fragment.output_sink.__isset.join_build_sink) {
+        continue;
+      }
+
+      FragmentScheduleState* src_state =
+          state->GetFragmentScheduleState(src_fragment.idx);
+      if (state->query_options().runtime_filter_mode == TRuntimeFilterMode::GLOBAL
+          && src_fragment.produced_runtime_filters_reservation_bytes > 0) {
+        ComputeRandomKrpcForAggregation(executor_config, state, src_state,
+            state->query_options().max_num_filters_aggregated_per_host);
+      }
+
       if (!src_fragment.output_sink.__isset.stream_sink) continue;
       FragmentIdx dest_idx =
           state->GetFragmentIdx(src_fragment.output_sink.stream_sink.dest_node_id);
       FragmentScheduleState* dest_state = state->GetFragmentScheduleState(dest_idx);
-      FragmentScheduleState* src_state =
-          state->GetFragmentScheduleState(src_fragment.idx);
 
       // populate src_state->destinations
       for (int i = 0; i < dest_state->instance_states.size(); ++i) {
@@ -252,7 +285,8 @@ Status Scheduler::ComputeFragmentExecParams(
       DCHECK(sink.output_partition.type == TPartitionType::UNPARTITIONED
           || sink.output_partition.type == TPartitionType::HASH_PARTITIONED
           || sink.output_partition.type == TPartitionType::RANDOM
-          || sink.output_partition.type == TPartitionType::KUDU);
+          || sink.output_partition.type == TPartitionType::KUDU
+          || sink.output_partition.type == TPartitionType::DIRECTED);
       PlanNodeId exch_id = sink.dest_node_id;
       google::protobuf::Map<int32_t, int32_t>* per_exch_num_senders =
           dest_state->exec_params->mutable_per_exch_num_senders();
@@ -267,18 +301,121 @@ Status Scheduler::ComputeFragmentExecParams(
   return Status::OK();
 }
 
+// Helper type to map instance indexes to aggregator indexes.
+typedef vector<pair<int, int>> InstanceToAggPairs;
+
+void Scheduler::ComputeRandomKrpcForAggregation(const ExecutorConfig& executor_config,
+    ScheduleState* state, FragmentScheduleState* src_state, int num_filters_per_host) {
+  if (num_filters_per_host <= 1) return;
+  // src_state->instance_states organize fragment instances as one dimension vector
+  // where instances scheduled in same host is placed in adjacent indices.
+  // Group instance indices from common host to 'instance_groups' by comparing
+  // krpc address between adjacent element of instance_states.
+  const NetworkAddressPB& coord_address = executor_config.coord_desc.krpc_address();
+  vector<InstanceToAggPairs> instance_groups;
+  InstanceToAggPairs coordinator_instances;
+  for (int i = 0; i < src_state->instance_states.size(); ++i) {
+    const NetworkAddressPB& krpc_address = src_state->instance_states[i].krpc_host;
+    if (KrpcAddressEqual(krpc_address, coord_address)) {
+      coordinator_instances.emplace_back(i, 0);
+    } else {
+      if (i == 0
+          || !KrpcAddressEqual(
+              krpc_address, src_state->instance_states[i - 1].krpc_host)) {
+        // This is the first fragment instance scheduled in a host.
+        // Append empty InstanceToAggPairs to 'instance_groups'.
+        instance_groups.emplace_back();
+      }
+      instance_groups.back().emplace_back(i, 0);
+    }
+  }
+
+  int num_non_coordinator_host = instance_groups.size();
+  if (num_non_coordinator_host == 0) return;
+
+  // Select number of intermediate aggregator so that each aggregator will receive
+  // runtime filter update from at most 'num_filters_per_host' executors.
+  int num_agg = (int)ceil((double)num_non_coordinator_host / num_filters_per_host);
+  DCHECK_GT(num_agg, 0);
+
+  if (UNLIKELY(FLAGS_sort_runtime_filter_aggregator_candidates)) {
+    sort(instance_groups.begin(), instance_groups.end(),
+        [src_state](InstanceToAggPairs a, InstanceToAggPairs b) {
+          int idx_a = a[0].first;
+          int idx_b = b[0].first;
+          return CompareNetworkAddressPB(src_state->instance_states[idx_a].krpc_host,
+                     src_state->instance_states[idx_b].krpc_host)
+              < 0;
+        });
+  } else {
+    std::shuffle(instance_groups.begin(), instance_groups.end(), *state->rng());
+  }
+  if (coordinator_instances.size() > 0) {
+    // Put coordinator group behind so that coordinator won't be selected as intermediate
+    // aggregator.
+    instance_groups.push_back(coordinator_instances);
+  }
+
+  RuntimeFilterAggregatorInfoPB* agg_info =
+      src_state->exec_params->mutable_filter_agg_info();
+  agg_info->set_num_aggregators(num_agg);
+
+  int group_idx = 0;
+  int agg_idx = -1;
+  InstanceToAggPairs instance_to_agg;
+  vector<int> num_reporting_hosts(num_agg, 0);
+  for (auto group : instance_groups) {
+    const NetworkAddressPB& host = src_state->instance_states[group[0].first].host;
+    if (group_idx < num_agg) {
+      // First 'num_agg' of 'instance_groups' host selected as intermediate aggregator.
+      const BackendDescriptorPB& desc = LookUpBackendDesc(executor_config, host);
+      NetworkAddressPB* address = agg_info->add_aggregator_krpc_addresses();
+      *address = desc.address();
+      DCHECK(desc.has_krpc_address());
+      DCHECK(IsResolvedAddress(desc.krpc_address()));
+      NetworkAddressPB* krpc_address = agg_info->add_aggregator_krpc_backends();
+      *krpc_address = desc.krpc_address();
+    }
+    agg_idx = (agg_idx + 1) % num_agg;
+    num_reporting_hosts[agg_idx] += 1;
+    for (auto entry : group) {
+      entry.second = agg_idx;
+      instance_to_agg.push_back(entry);
+    }
+    ++group_idx;
+  }
+
+  // Sort 'instance_to_agg' based on the original instance order to populate
+  // 'aggregator_idx_to_report'.
+  sort(instance_to_agg.begin(), instance_to_agg.end());
+  DCHECK_EQ(src_state->instance_states.size(), instance_to_agg.size());
+  for (auto entry : instance_to_agg) {
+    agg_info->add_aggregator_idx_to_report(entry.second);
+  }
+
+  // Write how many host is expected to report to each intermediate aggregator,
+  // including the aggregator host itself.
+  for (auto num_reporters : num_reporting_hosts) {
+    agg_info->add_num_reporter_per_aggregator(num_reporters);
+  }
+}
+
+static inline void initializeSchedulerWarning(RuntimeProfile* summary_profile) {
+  if (summary_profile->GetInfoString(SCHEDULER_WARNING_KEY) == nullptr) {
+    summary_profile->AddInfoString(SCHEDULER_WARNING_KEY,
+        "Cluster membership might changed between planning and scheduling");
+  }
+}
+
 Status Scheduler::CheckEffectiveInstanceCount(
     const FragmentScheduleState* fragment_state, ScheduleState* state) {
   // These checks are only intended if COMPUTE_PROCESSING_COST=true.
   if (!state->query_options().compute_processing_cost) return Status::OK();
 
   int effective_instance_count = fragment_state->fragment.effective_instance_count;
+  DCHECK_GT(effective_instance_count, 0);
   if (effective_instance_count < fragment_state->instance_states.size()) {
-    if (state->summary_profile()->GetInfoString(SCHEDULER_WARNING_KEY) == nullptr) {
-      state->summary_profile()->AddInfoString(SCHEDULER_WARNING_KEY,
-          "Cluster membership might changed between planning and scheduling");
-    }
-
+    initializeSchedulerWarning(state->summary_profile());
     string warn_message = Substitute(
         "$0 scheduled instance count ($1) is higher than its effective count ($2)",
         fragment_state->fragment.display_name, fragment_state->instance_states.size(),
@@ -358,13 +495,19 @@ Status Scheduler::ComputeFragmentExecParams(const ExecutorConfig& executor_confi
     // the plan that must be executed at the coordinator or an unpartitioned fragment
     // that can be executed anywhere.
     VLOG(3) << "Computing exec params for "
-            << (fragment_state->is_coord_fragment ? "coordinator" : "unpartitioned")
+            << (fragment_state->is_root_coord_fragment
+                ? "root coordinator" : "unpartitioned")
             << " fragment " << fragment_state->fragment.display_name;
     NetworkAddressPB host;
     NetworkAddressPB krpc_host;
-    if (fragment_state->is_coord_fragment || executor_config.group.NumExecutors() == 0) {
-      // The coordinator fragment must be scheduled on the coordinator. Otherwise if
-      // no executors are available, we need to schedule on the coordinator.
+    if (fragment_state->is_root_coord_fragment || fragment.is_coordinator_only ||
+        executor_config.group.NumExecutors() == 0) {
+      // 1. The root coordinator fragment ('fragment_state->is_root_coord_fragment') must
+      // be scheduled on the coordinator.
+      // 2. Some other fragments ('fragment.is_coordinator_only'), such as
+      // Iceberg metadata scanner fragments, must also run on the coordinator.
+      // 3. Otherwise if no executors are available, we need to schedule on the
+      // coordinator.
       const BackendDescriptorPB& coord_desc = executor_config.coord_desc;
       host = coord_desc.address();
       DCHECK(coord_desc.has_krpc_address());
@@ -393,8 +536,8 @@ Status Scheduler::ComputeFragmentExecParams(const ExecutorConfig& executor_confi
     }
     VLOG(3) << "Scheduled unpartitioned fragment on " << krpc_host;
     DCHECK(IsResolvedAddress(krpc_host));
-    // make sure that the coordinator instance ends up with instance idx 0
-    UniqueIdPB instance_id = fragment_state->is_coord_fragment ?
+    // make sure that the root coordinator instance ends up with instance idx 0
+    UniqueIdPB instance_id = fragment_state->is_root_coord_fragment ?
         state->query_id() :
         state->GetNextInstanceId();
     fragment_state->instance_states.emplace_back(
@@ -521,7 +664,7 @@ void Scheduler::CreateCollocatedAndScanInstances(const ExecutorConfig& executor_
   if (request_query_option.compute_processing_cost) {
     // Numbers should follow 'effective_instance_count' from planner.
     // 'effective_instance_count' is total instances across all executors.
-    DCHECK(fragment.effective_instance_count > 0);
+    DCHECK_GT(fragment.effective_instance_count, 0);
     int inst_per_host =
         ceil((float)fragment.effective_instance_count / instances_per_host.size());
     max_num_instances = max(1, inst_per_host);
@@ -706,7 +849,7 @@ Status Scheduler::ComputeScanRangeAssignment(const ExecutorConfig& executor_conf
     bool node_random_replica, const vector<TScanRangeLocationList>& locations,
     const vector<TNetworkAddress>& host_list, bool exec_at_coord,
     const TQueryOptions& query_options, RuntimeProfile::Counter* timer, std::mt19937* rng,
-    FragmentScanRangeAssignment* assignment) {
+    RuntimeProfile* summary_profile, FragmentScanRangeAssignment* assignment) {
   const ExecutorGroup& executor_group = executor_config.group;
   if (executor_group.NumExecutors() == 0 && !exec_at_coord) {
     return Status(TErrorCode::NO_REGISTERED_BACKENDS);
@@ -755,6 +898,28 @@ Status Scheduler::ComputeScanRangeAssignment(const ExecutorConfig& executor_conf
           coord_desc.address().hostname(), nullptr));
       assignment_ctx.RecordScanRangeAssignment(
           coord_desc, node_id, host_list, scan_range_locations, assignment);
+    } else if (scan_range_locations.scan_range.__isset.is_system_scan &&
+               scan_range_locations.scan_range.is_system_scan) {
+      // Must run on a coordinator, lookup in executor_config.all_coords
+      DCHECK_EQ(1, scan_range_locations.locations.size());
+      const TNetworkAddress& coordinator =
+          host_list[scan_range_locations.locations[0].host_idx];
+      const BackendDescriptorPB* coordinatorPB =
+          executor_config.all_coords.LookUpBackendDesc(FromTNetworkAddress(coordinator));
+      if (coordinatorPB == nullptr) {
+        // Coordinator is no longer available, skip this range.
+        string warn_message = Substitute(
+            "Coordinator $0 is no longer available for system table scan assignment",
+            TNetworkAddressToString(coordinator));
+        if (LIKELY(summary_profile != nullptr)) {
+          initializeSchedulerWarning(summary_profile);
+          summary_profile->AppendInfoString(SCHEDULER_WARNING_KEY, warn_message);
+        }
+        LOG(WARNING) << warn_message;
+        continue;
+      }
+      assignment_ctx.RecordScanRangeAssignment(
+        *coordinatorPB, node_id, host_list, scan_range_locations, assignment);
     } else {
       // Collect executor candidates with smallest memory distance.
       vector<IpAddr> executor_candidates;
@@ -937,12 +1102,52 @@ bool Scheduler::IsCoordinatorOnlyQuery(const TQueryExecRequest& exec_request) {
       && type == TPartitionType::UNPARTITIONED;
 }
 
+void Scheduler::PopulateFilepathToHostsMapping(const FInstanceScheduleState& finst,
+    ScheduleState* state, ByNodeFilepathToHosts* duplicate_check) {
+  for (const auto& per_node_ranges : finst.exec_params.per_node_scan_ranges()) {
+    const TPlanNode& node = state->GetNode(per_node_ranges.first);
+    if (node.node_type != TPlanNodeType::HDFS_SCAN_NODE) continue;
+    if (!node.hdfs_scan_node.__isset.deleteFileScanNodeId) continue;
+    const TPlanNodeId delete_file_node_id = node.hdfs_scan_node.deleteFileScanNodeId;
+
+    for (const auto& scan_ranges : per_node_ranges.second.scan_ranges()) {
+      string file_path;
+      bool is_relative = false;
+      const HdfsFileSplitPB& hdfs_file_split = scan_ranges.scan_range().hdfs_file_split();
+      if (hdfs_file_split.has_relative_path() &&
+          !hdfs_file_split.relative_path().empty()) {
+        file_path = hdfs_file_split.relative_path();
+        is_relative = true;
+      } else {
+        file_path = hdfs_file_split.absolute_path();
+      }
+      DCHECK(!file_path.empty());
+
+      std::unordered_set<NetworkAddressPB>& current_hosts =
+          (*duplicate_check)[delete_file_node_id][file_path];
+      if (current_hosts.find(finst.host) != current_hosts.end()) continue;
+      current_hosts.insert(finst.host);
+
+      auto* by_node_filepath_to_hosts =
+          state->query_schedule_pb()->mutable_by_node_filepath_to_hosts();
+      auto* filepath_to_hosts =
+          (*by_node_filepath_to_hosts)[delete_file_node_id].mutable_filepath_to_hosts();
+      (*filepath_to_hosts)[file_path].set_is_relative(is_relative);
+      auto* hosts = (*filepath_to_hosts)[file_path].add_hosts();
+      *hosts = finst.host;
+    }
+  }
+}
+
 void Scheduler::ComputeBackendExecParams(
     const ExecutorConfig& executor_config, ScheduleState* state) {
+  ByNodeFilepathToHosts duplicate_check;
   for (FragmentScheduleState& f : state->fragment_schedule_states()) {
     const NetworkAddressPB* prev_host = nullptr;
     int num_hosts = 0;
     for (FInstanceScheduleState& i : f.instance_states) {
+      PopulateFilepathToHostsMapping(i, state, &duplicate_check);
+
       BackendScheduleState& be_state = state->GetOrCreateBackendScheduleState(i.host);
       be_state.exec_params->add_instance_params()->Swap(&i.exec_params);
       // Different fragments do not synchronize their Open() and Close(), so the backend
@@ -976,23 +1181,64 @@ void Scheduler::ComputeBackendExecParams(
   }
 
   // Compute 'slots_to_use' for each backend based on the max # of instances of
-  // any fragment on that backend.
+  // any fragment on that backend. If 'compute_processing_cost' is on, and Planner
+  // set 'max_slot_per_executor', pick the min between 'dominant_instance_count' and
+  // 'max_slot_per_executor'.
+  bool cap_slots = state->query_options().compute_processing_cost
+      && state->query_options().slot_count_strategy == TSlotCountStrategy::PLANNER_CPU_ASK
+      && state->request().__isset.max_slot_per_executor
+      && state->request().max_slot_per_executor > 0;
   for (auto& backend : state->per_backend_schedule_states()) {
-    int be_max_instances = 0;
+    int be_max_instances = 0; // max # of instances of any fragment.
+    int dominant_instance_count = 0; // sum of all dominant fragment instances.
+
     // Instances for a fragment are clustered together because of how the vector is
-    // constructed above. So we can compute the max # of instances of any fragment
-    // with a single pass over the vector.
+    // constructed above. For example, 3 fragments with 2 instances each will be in this
+    // order inside exec_params->instance_params() vector.
+    //   [F00_a, F00_b, F01_c, F01_d, F02_e, F02_f]
+    // So we can compute the max # of instances of any fragment with a single pass over
+    // the vector.
     int curr_fragment_idx = -1;
     int curr_instance_count = 0; // Number of instances of the current fragment seen.
+    bool is_dominant = false;
     for (auto& finstance : backend.second.exec_params->instance_params()) {
       if (curr_fragment_idx == -1 || curr_fragment_idx != finstance.fragment_idx()) {
+        // We arrived at new fragment group. Update 'be_max_instances'.
+        be_max_instances = max(be_max_instances, curr_instance_count);
+        // Reset 'curr_fragment_idx' and other counting related variables.
         curr_fragment_idx = finstance.fragment_idx();
         curr_instance_count = 0;
+        is_dominant =
+            state->GetFragmentScheduleState(curr_fragment_idx)->fragment.is_dominant;
       }
       ++curr_instance_count;
-      be_max_instances = max(be_max_instances, curr_instance_count);
+      if (is_dominant) ++dominant_instance_count;
     }
-    backend.second.exec_params->set_slots_to_use(be_max_instances);
+    // Update 'be_max_instances' one last time.
+    be_max_instances = max(be_max_instances, curr_instance_count);
+
+    // Default slots to use number from IMPALA-8998.
+    // For fragment with largest num of instances running in this backend, it
+    // ensures allocation of 1 slot for each instance of that fragment.
+    int slots_to_use = be_max_instances;
+
+    // Done looping exec_params->instance_params(). Derived 'slots_to_use' based on
+    // finalized 'be_max_instances' and 'dominant_instance_count'.
+    if (cap_slots) {
+      if (dominant_instance_count >= be_max_instances) {
+        // One case where it is possible to have 'dominant_instance_count' <
+        // 'be_max_instances' is with dedicated coordinator setup. The schedule would
+        // only assign one coordinator fragment instance to the coordinator node,
+        // but 'dominant_instance_count' can be 0 if fragment.is_dominant == false.
+        // In that case, ignore 'dominant_instance_count' and continue with
+        // 'be_max_instances'.
+        // However, if 'dominant_instance_count' >= 'be_max_instances',
+        // continue with 'dominant_instance_count'.
+        slots_to_use = dominant_instance_count;
+      }
+      slots_to_use = min(slots_to_use, state->request().max_slot_per_executor);
+    }
+    backend.second.exec_params->set_slots_to_use(slots_to_use);
   }
 
   // This also ensures an entry always exists for the coordinator backend.
@@ -1238,6 +1484,9 @@ void TScanRangeToScanRangePB(const TScanRange& tscan_range, ScanRangePB* scan_ra
   if (tscan_range.__isset.file_metadata) {
     scan_range_pb->set_file_metadata(tscan_range.file_metadata);
   }
+  if (tscan_range.__isset.is_system_scan) {
+    scan_range_pb->set_is_system_scan(tscan_range.is_system_scan);
+  }
 }
 
 void Scheduler::AssignmentCtx::RecordScanRangeAssignment(
@@ -1245,6 +1494,23 @@ void Scheduler::AssignmentCtx::RecordScanRangeAssignment(
     const vector<TNetworkAddress>& host_list,
     const TScanRangeLocationList& scan_range_locations,
     FragmentScanRangeAssignment* assignment) {
+  if (scan_range_locations.scan_range.__isset.is_system_scan &&
+      scan_range_locations.scan_range.is_system_scan) {
+    // Assigned to a coordinator.
+    PerNodeScanRanges* scan_ranges =
+        FindOrInsert(assignment, executor.address(), PerNodeScanRanges());
+    vector<ScanRangeParamsPB>* scan_range_params_list =
+        FindOrInsert(scan_ranges, node_id, vector<ScanRangeParamsPB>());
+    ScanRangeParamsPB scan_range_params;
+    TScanRangeToScanRangePB(
+        scan_range_locations.scan_range, scan_range_params.mutable_scan_range());
+    scan_range_params_list->push_back(scan_range_params);
+
+    VLOG_FILE << "Scheduler assignment of system table scan to coordinator: "
+              << executor.address();
+    return;
+  }
+
   int64_t scan_range_length = 0;
   if (scan_range_locations.scan_range.__isset.hdfs_file_split) {
     scan_range_length = scan_range_locations.scan_range.hdfs_file_split.length;
