@@ -66,6 +66,7 @@ class Scheduler {
   struct ExecutorConfig {
     const ExecutorGroup& group;
     const BackendDescriptorPB& coord_desc;
+    const ExecutorGroup& all_coords;
   };
 
   /// Populates given query schedule and assigns fragments to hosts based on scan
@@ -83,6 +84,11 @@ class Scheduler {
   typedef boost::unordered_map<IpAddr, ExecutorGroup::Executors::const_iterator>
       NextExecutorPerHost;
 
+  /// Map from file paths to hosts where those files are scheduled, grouped by scan node
+  /// ID.
+  typedef std::unordered_map<int, std::unordered_map<
+      std::string, std::unordered_set<NetworkAddressPB>>> ByNodeFilepathToHosts;
+
   /// Internal structure to track scan range assignments for an executor host. This struct
   /// is used as the heap element in and maintained by AddressableAssignmentHeap.
   struct ExecutorAssignmentInfo {
@@ -346,13 +352,15 @@ class Scheduler {
   /// query_options:           Query options for the current query.
   /// timer:                   Tracks execution time of ComputeScanRangeAssignment.
   /// rng:                     Random number generated used for any random decisions
+  /// summary_profile:         Summary profile for any scheduler warnings.
   /// assignment:              Output parameter, to which new assignments will be added.
   Status ComputeScanRangeAssignment(const ExecutorConfig& executor_config,
       PlanNodeId node_id, const TReplicaPreference::type* node_replica_preference,
       bool node_random_replica, const std::vector<TScanRangeLocationList>& locations,
       const std::vector<TNetworkAddress>& host_list, bool exec_at_coord,
       const TQueryOptions& query_options, RuntimeProfile::Counter* timer,
-      std::mt19937* rng, FragmentScanRangeAssignment* assignment);
+      std::mt19937* rng, RuntimeProfile* summary_profile,
+      FragmentScanRangeAssignment* assignment);
 
   /// Computes execution parameters for all backends assigned in the query and always one
   /// for the coordinator backend since it participates in execution regardless. Must be
@@ -375,6 +383,12 @@ class Scheduler {
       const TPlanExecInfo& plan_exec_info, FragmentScheduleState* fragment_state,
       ScheduleState* state);
 
+  /// For a given 'src_state' and 'num_filters_per_host', select few backend as
+  /// intermediate filter aggregator before final aggregation in coordinator.
+  /// Do nothing if 'num_filters_per_host' <= 1.
+  void ComputeRandomKrpcForAggregation(const ExecutorConfig& executor_config,
+      ScheduleState* state, FragmentScheduleState* src_state, int num_filters_per_host);
+
   /// Create instances of the fragment corresponding to fragment_state, which contains
   /// either a Union node, one or more scan nodes, or both.
   ///
@@ -414,6 +428,14 @@ class Scheduler {
   void CreateCollocatedJoinBuildInstances(
       FragmentScheduleState* fragment_state, ScheduleState* state);
 
+  /// This is called during the execution of Schedule() and populates
+  /// 'ScheduleState::query_schedule_pb_::by_node_filepath_to_hosts' mapping with the
+  /// information of what files are scheduled to what hosts grouped by scan node.
+  /// 'duplicate_check' keeps track of the previously added items and used for preventing
+  /// duplicates being added.
+  void PopulateFilepathToHostsMapping(const FInstanceScheduleState& finst,
+      ScheduleState* state, ByNodeFilepathToHosts* duplicate_check);
+
   /// Add all hosts that the scans identified by 'scan_ids' are executed on to
   /// 'scan_hosts'.
   void GetScanHosts(const std::vector<TPlanNodeId>& scan_ids,
