@@ -28,6 +28,7 @@
 #include "runtime/hdfs-fs-cache.h"
 #include "util/condition-variable.h"
 #include "util/runtime-profile.h"
+#include "util/runtime-profile-counters.h"
 #include "gen-cpp/Frontend_types.h"
 #include "gen-cpp/ImpalaHiveServer2Service.h"
 
@@ -78,8 +79,7 @@ class QuerySchedulePB;
 class ClientRequestState {
  public:
   ClientRequestState(const TQueryCtx& query_ctx, Frontend* frontend, ImpalaServer* server,
-      std::shared_ptr<ImpalaServer::SessionState> session, TExecRequest* exec_request,
-      QueryDriver* query_driver);
+      std::shared_ptr<ImpalaServer::SessionState> session, QueryDriver* query_driver);
 
   ~ClientRequestState();
 
@@ -87,6 +87,12 @@ class ClientRequestState {
 
   enum class RetryState { RETRYING, RETRIED, NOT_RETRIED };
 
+  /// Initialize TExecRequest. Prior to this, exec_request() returns a place-holder.
+  void SetExecRequest(const TExecRequest* exec_request) {
+    DCHECK(exec_request_.Load()->stmt_type == TStmtType::UNKNOWN);
+    exec_request_.Store(exec_request);
+  }
+
   /// Sets the profile that is produced by the frontend. The frontend creates the
   /// profile during planning and returns it to the backend via TExecRequest,
   /// which then sets the frontend profile.
@@ -167,11 +173,18 @@ class ClientRequestState {
   /// If current status is already != ok, no update is made (we preserve the first error)
   /// If called with a non-ok argument, the expectation is that the query will be aborted
   /// quickly.
+  /// If 'log_error' is true, log the error when query status become non-ok. This is used
+  /// when running DDLs in async threads. The async thread should take care of the logging
+  /// since the main thread only checks whether the thread creation succeeds.
+  /// E.g. ExecDdlRequest() returns the query status in sync mode, but returns the thread
+  /// creation status in async mode. So LOG_AND_RETURN_IF_ERROR(ExecDdlRequest()) doesn't
+  /// log the query failure in async mode.
   /// Returns the status argument (so we can write
   /// RETURN_IF_ERROR(UpdateQueryStatus(SomeOperation())).
   /// Does not take lock_, but requires it: caller must ensure lock_
   /// is taken before calling UpdateQueryStatus
-  Status UpdateQueryStatus(const Status& status) WARN_UNUSED_RESULT;
+  Status UpdateQueryStatus(const Status& status, bool log_error=false)
+      WARN_UNUSED_RESULT;
 
   /// Cancels the child queries and the coordinator with the given cause.
   /// If cause is NULL, it assumes this was deliberately cancelled by the user while in
@@ -181,19 +194,13 @@ class ClientRequestState {
   /// until cancellaton of 'coord_' finishes and it is finalized.
   /// 'wait_until_finalized' should only used by the single thread finalizing the query,
   /// to avoid many threads piling up waiting for query cancellation.
-  ///
-  /// Only returns an error if 'check_inflight' is true and the query is not yet
-  /// in-flight. Otherwise, proceed and return Status::OK() even if the query isn't
-  /// in-flight (for cleaning up after an error on the query issuing path).
-  Status Cancel(bool check_inflight, const Status* cause, bool wait_until_finalized=false);
+  void Cancel(const Status* cause, bool wait_until_finalized=false);
 
   /// This is called when the query is done (finished, cancelled, or failed). This runs
   /// synchronously within the last client RPC and does any work that is required before
   /// the query is finished from the client's point of view, including cancelling the
-  /// query with 'cause'. Returns an error if 'check_inflight' is true and the query is
-  /// not yet in-flight, or if another thread has already started finalizing the query.
-  /// Takes lock_: callers must not hold lock() before calling.
-  Status Finalize(bool check_inflight, const Status* cause);
+  /// query with 'cause'. Takes lock_: callers must not hold lock() before calling.
+  void Finalize(const Status* cause);
 
   /// Sets the API-specific (Beeswax, HS2) result cache and its size bound.
   /// The given cache is owned by this client request state, even if an error is returned.
@@ -233,11 +240,16 @@ class ClientRequestState {
   /// Queries are run and authorized on behalf of the effective_user.
   const std::string& effective_user() const;
   const std::string& connected_user() const { return query_ctx_.session.connected_user; }
-  bool user_has_profile_access() const { return user_has_profile_access_; }
+  bool user_has_profile_access() const { return user_has_profile_access_.Load(); }
   const std::string& do_as_user() const { return query_ctx_.session.delegated_user; }
   TSessionType::type session_type() const { return query_ctx_.session.session_type; }
   const TUniqueId& session_id() const { return query_ctx_.session.session_id; }
   const std::string& default_db() const { return query_ctx_.session.database; }
+  const std::string& redacted_sql() const {
+    return
+      query_ctx_.client_request.__isset.redacted_stmt ?
+      query_ctx_.client_request.redacted_stmt : query_ctx_.client_request.stmt;
+  }
   bool eos() const { return eos_.Load(); }
   const QuerySchedulePB* schedule() const { return schedule_.get(); }
 
@@ -252,14 +264,15 @@ class ClientRequestState {
   /// control.
   /// Admission control resource pool associated with this query.
   std::string request_pool() const {
-    if (is_planning_done_.load() && exec_request_ != nullptr
-        && exec_request_->query_exec_request.query_ctx.__isset.request_pool) {
-      // If the request pool has been set by Planner, return the request pool selected
-      // by Planner.
-      return exec_request_->query_exec_request.query_ctx.request_pool;
-    } else {
-      return query_ctx_.__isset.request_pool ? query_ctx_.request_pool : "";
+    if (is_planning_done_.load()) {
+      const TExecRequest& exec_req = exec_request();
+      if (exec_req.query_exec_request.query_ctx.__isset.request_pool) {
+        // If the request pool has been set by Planner, return the request pool selected
+        // by Planner.
+        return exec_req.query_exec_request.query_ctx.request_pool;
+      }
     }
+    return query_ctx_.__isset.request_pool ? query_ctx_.request_pool : "";
   }
 
   int num_rows_fetched() const { return num_rows_fetched_; }
@@ -268,19 +281,27 @@ class ClientRequestState {
   bool returns_result_set() { return !result_metadata_.columns.empty(); }
   const TResultSetMetadata* result_metadata() const { return &result_metadata_; }
   const TUniqueId& query_id() const { return query_ctx_.query_id; }
+
+  /// Return values from counters. See explanation on declaration of
+  /// num_rows_fetched_counter_, row_materialization_rate_, and row_materialization_timer_
+  /// for details.
+  int64_t num_rows_fetched_counter() const;
+  int64_t row_materialization_rate() const;
+  int64_t row_materialization_timer() const;
+
   /// Returns the TExecRequest for the query associated with this ClientRequestState.
-  /// Contents are only valid after InitExecRequest(TQueryCtx) initializes the
+  /// Contents are a place-holder until GetExecRequest(TQueryCtx) initializes the
   /// TExecRequest.
   const TExecRequest& exec_request() const {
-    DCHECK(exec_request_ != nullptr);
-    return *exec_request_;
+    DCHECK(exec_request_.Load() != nullptr);
+    return *exec_request_.Load();
   }
-  TStmtType::type stmt_type() const { return exec_request_->stmt_type; }
+  TStmtType::type stmt_type() const { return exec_request().stmt_type; }
   TCatalogOpType::type catalog_op_type() const {
-    return exec_request_->catalog_op_request.op_type;
+    return exec_request().catalog_op_request.op_type;
   }
   TDdlType::type ddl_type() const {
-    return exec_request_->catalog_op_request.ddl_params.ddl_type;
+    return exec_request().catalog_op_request.ddl_params.ddl_type;
   }
   std::mutex* lock() { return &lock_; }
   std::mutex* fetch_rows_lock() { return &fetch_rows_lock_; }
@@ -301,21 +322,24 @@ class ClientRequestState {
   const Status& query_status() const { return query_status_; }
   void set_result_metadata(const TResultSetMetadata& md) { result_metadata_ = md; }
   void set_user_profile_access(bool user_has_profile_access) {
-    user_has_profile_access_ = user_has_profile_access;
+    user_has_profile_access_.Store(user_has_profile_access);
   }
   const RuntimeProfile* profile() const { return profile_; }
   const RuntimeProfile* summary_profile() const { return summary_profile_; }
+  const RuntimeProfile* frontend_profile() const { return frontend_profile_; }
   int64_t start_time_us() const { return start_time_us_; }
   int64_t end_time_us() const { return end_time_us_.Load(); }
   const std::string& sql_stmt() const { return query_ctx_.client_request.stmt; }
   const TQueryOptions& query_options() const {
     return query_ctx_.client_request.query_options;
   }
+  bool hs2_metadata_op() const { return query_ctx_.client_request.hs2_metadata_op; }
   /// Returns 0:0 if this is a root query
   TUniqueId parent_query_id() const { return query_ctx_.parent_query_id; }
+  const vector<TTableName>& tables() const { return exec_request().tables; }
 
   const std::vector<std::string>& GetAnalysisWarnings() const {
-    return exec_request_->analysis_warnings;
+    return exec_request().analysis_warnings;
   }
 
   inline int64_t last_active_ms() const {
@@ -329,6 +353,15 @@ class ClientRequestState {
     return ref_count_ > 0;
   }
 
+  inline bool is_inflight() const {
+    // If the query is in 'inflight_queries' it means that the query has actually started
+    // executing. It is ok if the query is removed from 'inflight_queries' later, so we
+    // can release the session lock before returning.
+    std::lock_guard<std::mutex> session_lock(session_->lock);
+    return session_->inflight_queries.find(query_id())
+        != session_->inflight_queries.end();
+  }
+
   bool is_expired() const {
     std::lock_guard<std::mutex> l(expiration_data_lock_);
     return is_expired_;
@@ -462,6 +495,17 @@ class ClientRequestState {
   void UnRegisterRemainingRPCs();
   // Copy pending RPCs for a retried query
   void CopyRPCs(ClientRequestState& from_request);
+
+  // Update the profile counter 'GetInFlightProfileTimeStats' which is the summary stats
+  // of time spent in dumping the profile before the query is archived.
+  void UpdateGetInFlightProfileTime(int64_t elapsed_time_ns) {
+    get_inflight_profile_time_stats_->UpdateCounter(elapsed_time_ns);
+  }
+  // Update the profile counter 'ClientFetchLockWaitTimer' which is the cumulative time
+  // that client fetch requests waiting for locks.
+  void AddClientFetchLockWaitTime(int64_t lock_wait_time_ns) {
+    client_fetch_lock_wait_timer_->Add(lock_wait_time_ns);
+  }
  protected:
   /// Updates the end_time_us_ of this query if it isn't set. The end time is determined
   /// when this function is called for the first time, calling it multiple times does not
@@ -621,6 +665,11 @@ class ClientRequestState {
   MonotonicStopWatch client_wait_sw_;
   int64_t last_client_wait_time_ = 0;
 
+  // Tracks time spent in dumping the profile before the query is archived.
+  RuntimeProfile::SummaryStatsCounter* get_inflight_profile_time_stats_;
+  // Tracks time client fetch requests waiting for locks.
+  RuntimeProfile::Counter* client_fetch_lock_wait_timer_;
+
   // Tracks time spent by client calls reading RPC arguments
   RuntimeProfile::Counter* rpc_read_timer_;
   // Tracks time spent by client calls writing RPC results
@@ -650,14 +699,16 @@ class ClientRequestState {
   /// UpdateQueryStatus(Status) or MarkAsRetrying(Status).
   Status query_status_;
 
+  /// Default TExecRequest to return until SetExecRequest is called.
+  static const TExecRequest unknown_exec_request_;
+
   /// The TExecRequest for the query tracked by this ClientRequestState. The TExecRequest
-  /// is initialized in QueryDriver::RunFrontendPlanner(TQueryCtx).The TExecRequest is
-  /// owned by the parent QueryDriver.
-  TExecRequest* exec_request_;
+  /// is initialized once it's finalized. It's owned by the parent QueryDriver.
+  AtomicPtr<const TExecRequest> exec_request_{&unknown_exec_request_};
 
   /// If true, effective_user() has access to the runtime profile and execution
   /// summary.
-  bool user_has_profile_access_ = true;
+  AtomicBool user_has_profile_access_{true};
 
   TResultSetMetadata result_metadata_; // metadata for select query
   int num_rows_fetched_ = 0; // number of rows fetched by client for the entire query
@@ -800,6 +851,11 @@ class ClientRequestState {
   /// the transaction will be closed when this function returns.
   Status UpdateCatalog() WARN_UNUSED_RESULT;
 
+  /// Fills 'cat_ice_op' based on 'finalize_params'. Returns false if there is no
+  /// need to update the Catalog (no records changed), returns true otherwise.
+  bool CreateIcebergCatalogOps(const TFinalizeParams& finalize_params,
+      TIcebergOperationParam* cat_ice_op);
+
   /// Copies results into request_result_set_
   /// TODO: Have the FE return list<Data.TResultRow> so that this isn't necessary
   void SetResultSet(const TDdlExecResponse* ddl_resp);
@@ -887,5 +943,8 @@ class ClientRequestState {
   /// hints about how to reset the original table name. 'params' holds the SQL query
   /// string the user should run.
   void AddTableResetHints(const TConvertTableRequest& params, Status* status) const;
+
+  /// Adds the catalog execution timeline returned from catalog RPCs into the profile.
+  void AddCatalogTimeline();
 };
 }
