@@ -42,6 +42,8 @@ class MetricGroup;
 /// appropriate receivers. Metrics exposed by the service will be added to 'metric_group'.
 class DataStreamService : public DataStreamServiceIf {
  public:
+  static constexpr std::string_view END_DATA_STREAM = "EndDataStream";
+
   DataStreamService(MetricGroup* metric_group);
 
   /// Initializes the service by registering it with the singleton RPC manager.
@@ -69,6 +71,11 @@ class DataStreamService : public DataStreamServiceIf {
   virtual void UpdateFilter(const UpdateFilterParamsPB* req, UpdateFilterResultPB* resp,
       kudu::rpc::RpcContext* context);
 
+  /// Called by fragment instances that produce local runtime filters to deliver them to
+  /// the aggregator backend for intermediate aggregation.
+  virtual void UpdateFilterFromRemote(const UpdateFilterParamsPB* req,
+      UpdateFilterResultPB* resp, kudu::rpc::RpcContext* context);
+
   /// Called by the coordinator to deliver global runtime filters to fragments for
   /// application at plan nodes.
   virtual void PublishFilter(const PublishFilterParamsPB* req,
