@@ -20,23 +20,18 @@
 #include "service/impala-server.inline.h"
 
 #include <algorithm>
-#include <type_traits>
 
 #include <boost/algorithm/string.hpp>
 #include <boost/algorithm/string/join.hpp>
 #include <boost/bind.hpp>
-#include <boost/date_time/posix_time/posix_time_types.hpp>
 #include <boost/unordered_set.hpp>
-#include <jni.h>
 #include <gperftools/heap-profiler.h>
 #include <gperftools/malloc_extension.h>
 #include <gtest/gtest.h>
 #include <gutil/strings/substitute.h>
-#include <thrift/protocol/TDebugProtocol.h>
 
 #include "common/logging.h"
 #include "common/version.h"
-#include "rpc/thrift-util.h"
 #include "runtime/coordinator.h"
 #include "runtime/exec-env.h"
 #include "runtime/raw-value.h"
@@ -96,6 +91,7 @@ DECLARE_string(hostname);
 DECLARE_int32(webserver_port);
 DECLARE_int32(idle_session_timeout);
 DECLARE_int32(disconnected_session_timeout);
+DECLARE_int32(max_hs2_sessions_per_user);
 DECLARE_bool(ping_expose_webserver_url);
 DECLARE_string(anonymous_user_name);
 
@@ -149,9 +145,13 @@ void ImpalaServer::ExecuteMetadataOp(const THandleIdentifier& session_handle,
   // from an RPC. As a best effort, we use the type of the operation.
   map<int, const char*>::const_iterator query_text_it =
       _TMetadataOpcode_VALUES_TO_NAMES.find(request->opcode);
-  const string& query_text = query_text_it == _TMetadataOpcode_VALUES_TO_NAMES.end() ?
-      "N/A" : query_text_it->second;
-  query_ctx.client_request.stmt = query_text;
+  if (query_text_it == _TMetadataOpcode_VALUES_TO_NAMES.end()) {
+    query_ctx.client_request.stmt = "N/A";
+  } else {
+    query_ctx.client_request.stmt = query_text_it->second;
+    query_ctx.client_request.__set_hs2_metadata_op(true);
+  }
+
   QueryHandle query_handle;
   QueryDriver::CreateNewDriver(this, &query_handle, query_ctx, session);
   Status register_status = RegisterQuery(query_ctx.query_id, session, &query_handle);
@@ -203,8 +203,11 @@ Status ImpalaServer::FetchInternal(TUniqueId query_id, SessionState* session,
     return Status::OK();
   }
 
+  int64_t start_time_ns = MonotonicNanos();
   lock_guard<mutex> frl(*query_handle->fetch_rows_lock());
   lock_guard<mutex> l(*query_handle->lock());
+  int64_t lock_wait_time_ns = MonotonicNanos() - start_time_ns;
+  query_handle->AddClientFetchLockWaitTime(lock_wait_time_ns);
 
   // Check for cancellation or an error.
   RETURN_IF_ERROR(query_handle->query_status());
@@ -422,6 +425,11 @@ void ImpalaServer::OpenSession(TOpenSessionResp& return_val,
     }
   }
 
+  Status status = IncrementAndCheckSessionCount(state->connected_user);
+  HS2_RETURN_IF_ERROR(return_val, status, SQLSTATE_GENERAL_ERROR);
+
+  // At this point all checks are complete, store session status information.
+
   RegisterSessionTimeout(state->session_timeout);
   TQueryOptionsToMap(state->QueryOptions(), &return_val.configuration);
 
@@ -454,6 +462,57 @@ void ImpalaServer::OpenSession(TOpenSessionResp& return_val,
              << "<" << TNetworkAddressToString(state->network_address) << ">.";
 }
 
+void ImpalaServer::DecrementCount(
+    std::map<std::string, int64>& loads, const std::string& key) {
+  // Check if key is present as dereferencing the map will insert it.
+  // FIXME C++20: use contains().
+  if (!loads.count(key)) {
+    string msg = Substitute("Missing key $0 when decrementing count", key);
+    LOG(WARNING) << msg;
+    DCHECK(false) << msg;
+    return;
+  }
+  int64& current_value = loads[key];
+  if (current_value == 1) {
+    // Remove the entry from the map if the current_value will go to zero.
+    loads.erase(key);
+    return;
+  }
+  if (current_value < 1) {
+    // Don't allow decrement below zero.
+    string msg = Substitute("Attempt to decrement below zero with key $0 ", key);
+    LOG(WARNING) << msg;
+    return;
+  }
+  loads[key]--;
+}
+
+void ImpalaServer::DecrementSessionCount(const string& user_name) {
+  if (FLAGS_max_hs2_sessions_per_user > 0) {
+    lock_guard<mutex> l(per_user_session_count_lock_);
+    DecrementCount(per_user_session_count_map_, user_name);
+  }
+}
+
+Status ImpalaServer::IncrementAndCheckSessionCount(const string& user_name) {
+  if (FLAGS_max_hs2_sessions_per_user > 0) {
+    lock_guard<mutex> l(per_user_session_count_lock_);
+    // Only check user limit if there is already a session for the user.
+    if (per_user_session_count_map_.count(user_name)) {
+      int64 load = per_user_session_count_map_[user_name];
+      if (load >= FLAGS_max_hs2_sessions_per_user) {
+        const string& err_msg =
+            Substitute("Number of sessions for user $0 exceeds coordinator limit $1",
+                user_name, FLAGS_max_hs2_sessions_per_user);
+        VLOG(1) << err_msg;
+        return Status::Expected(err_msg);
+      }
+    }
+    per_user_session_count_map_[user_name]++;
+  }
+  return Status::OK();
+}
+
 void ImpalaServer::CloseSession(TCloseSessionResp& return_val,
     const TCloseSessionReq& request) {
   VLOG_QUERY << "CloseSession(): request=" << RedactedDebugString(request);
@@ -493,8 +552,12 @@ void ImpalaServer::GetInfo(TGetInfoResp& return_val,
       return_val.infoValue.__set_stringValue(GetDaemonBuildVersion());
       break;
     default:
-      HS2_RETURN_ERROR(return_val, "Unsupported operation",
-          SQLSTATE_OPTIONAL_FEATURE_NOT_IMPLEMENTED);
+      return_val.status.__set_statusCode(thrift::TStatusCode::ERROR_STATUS);
+      return_val.status.__set_errorMessage(("Unsupported operation"));
+      return_val.status.__set_sqlState((SQLSTATE_OPTIONAL_FEATURE_NOT_IMPLEMENTED));
+      // 'infoValue' is a required field of TGetInfoResp
+      return_val.infoValue.__set_stringValue("");
+      return;
   }
   return_val.status.__set_statusCode(thrift::TStatusCode::SUCCESS_STATUS);
 }
@@ -1031,8 +1094,12 @@ void ImpalaServer::GetLog(TGetLogResp& return_val, const TGetLogReq& request) {
   // Report the query status, if the query failed or has been retried.
   if (query_handle->IsRetriedQuery()) {
     QueryHandle original_query_handle;
-    HS2_RETURN_IF_ERROR(return_val, GetQueryHandle(query_id, &original_query_handle),
-        SQLSTATE_GENERAL_ERROR);
+    Status status = GetQueryHandle(query_id, &original_query_handle);
+    if (UNLIKELY(!status.ok())) {
+      VLOG(1) << "Error in GetLog, could not get query handle: " << status.GetDetail();
+      HS2_RETURN_ERROR(return_val, status.GetDetail(), SQLSTATE_GENERAL_ERROR);
+      return;
+    }
     DCHECK(!original_query_handle->query_status().ok());
     ss << Substitute(GET_LOG_QUERY_RETRY_INFO_FORMAT,
         original_query_handle->query_status().GetDetail(),
