@@ -20,7 +20,6 @@
 #include <algorithm>
 #include <mutex>
 #include <sstream>
-#include <boost/lexical_cast.hpp>
 #include <boost/unordered_set.hpp>
 #include <rapidjson/prettywriter.h>
 #include <rapidjson/stringbuffer.h>
@@ -43,8 +42,8 @@
 #include "service/client-request-state.h"
 #include "service/frontend.h"
 #include "service/impala-server.h"
+#include "service/query-state-record.h"
 #include "thrift/protocol/TDebugProtocol.h"
-#include "util/coding-util.h"
 #include "util/debug-util.h"
 #include "util/logging-support.h"
 #include "util/pretty-printer.h"
@@ -64,6 +63,7 @@ using namespace rapidjson;
 using namespace strings;
 
 DECLARE_int32(query_log_size);
+DECLARE_int64(query_log_size_in_bytes);
 DECLARE_int32(query_stmt_size);
 DECLARE_bool(use_local_catalog);
 DECLARE_string(admission_service_host);
@@ -230,11 +230,17 @@ void ImpalaHttpHandler::HadoopVarzHandler(const Webserver::WebRequest& req,
     Document* document) {
   TGetAllHadoopConfigsResponse response;
   Status status  = server_->exec_env_->frontend()->GetAllHadoopConfigs(&response);
-  if (!status.ok()) return;
+  if (!status.ok()) {
+    LOG(ERROR) << "Error getting cluster configuration for hadoop-varz: "
+               << status.GetDetail();
+    Value error(status.GetDetail().c_str(), document->GetAllocator());
+    document->AddMember("error", error, document->GetAllocator());
+    return;
+  }
 
   Value configs(kArrayType);
   typedef map<string, string> ConfigMap;
-  for (const ConfigMap::value_type& config: response.configs) {
+  for (const auto& config: response.configs) {
     Value key(config.first.c_str(), document->GetAllocator());
     Value value(config.second.c_str(), document->GetAllocator());
     Value config_json(kObjectType);
@@ -504,7 +510,7 @@ std::string ImpalaHttpHandler::ProgressToString(int64_t num_completed, int64_t t
   return ss.str();
 }
 
-void ImpalaHttpHandler::QueryStateToJson(const ImpalaServer::QueryStateRecord& record,
+void ImpalaHttpHandler::QueryStateToJson(const QueryStateRecord& record,
     Value* value, Document* document, bool inflight) {
   Value user(record.effective_user.c_str(), document->GetAllocator());
   value->AddMember("effective_user", user, document->GetAllocator());
@@ -617,19 +623,19 @@ void ImpalaHttpHandler::QueryStateHandler(const Webserver::WebRequest& req,
     Document* document) {
   AddQueryRecordTips(document);
 
-  set<ImpalaServer::QueryStateRecord, ImpalaServer::QueryStateRecordLessThan>
+  set<QueryStateRecord, QueryStateRecord::StartTimeComparator>
       sorted_query_records;
 
   server_->query_driver_map_.DoFuncForAllEntries(
       [&](const std::shared_ptr<QueryDriver>& query_driver) {
         sorted_query_records.insert(
-            ImpalaServer::QueryStateRecord(*query_driver->GetActiveClientRequestState()));
+            QueryStateRecord(*query_driver->GetActiveClientRequestState()));
       });
 
   unordered_set<TUniqueId> in_flight_query_ids;
   Value in_flight_queries(kArrayType);
   int64_t num_waiting_queries = 0;
-  for (const ImpalaServer::QueryStateRecord& record: sorted_query_records) {
+  for (const QueryStateRecord& record: sorted_query_records) {
     Value record_json(kObjectType);
     QueryStateToJson(record, &record_json, document, true);
 
@@ -656,7 +662,7 @@ void ImpalaHttpHandler::QueryStateHandler(const Webserver::WebRequest& req,
   Value completed_queries(kArrayType);
   {
     lock_guard<mutex> l(server_->query_log_lock_);
-    for (const shared_ptr<ImpalaServer::QueryStateRecord>& log_entry :
+    for (const shared_ptr<QueryStateRecord>& log_entry :
         server_->query_log_) {
       // Don't show duplicated entries between in-flight and completed queries.
       if (in_flight_query_ids.find(log_entry->id) != in_flight_query_ids.end()) continue;
@@ -668,6 +674,8 @@ void ImpalaHttpHandler::QueryStateHandler(const Webserver::WebRequest& req,
   document->AddMember("completed_queries", completed_queries, document->GetAllocator());
   document->AddMember("completed_log_size", FLAGS_query_log_size,
       document->GetAllocator());
+  document->AddMember("completed_log_size_in_bytes", FLAGS_query_log_size_in_bytes,
+      document->GetAllocator());
 
   Value query_locations(kArrayType);
   {
@@ -701,6 +709,9 @@ void ImpalaHttpHandler::SessionsHandler(const Webserver::WebRequest& req,
 
   VLOG(1) << "Step3: Fill the connections information into the document.";
   FillConnectionsInfo(document, connection_contexts);
+
+  VLOG(1) << "Step4: Fill the hs2 users information into the document.";
+  FillUsersInfo(document);
 }
 
 void ImpalaHttpHandler::FillSessionsInfo(Document* document) {
@@ -766,6 +777,29 @@ void ImpalaHttpHandler::FillSessionsInfo(Document* document) {
       document->GetAllocator());
 }
 
+// Comparer that will sort the users array by the session_count field.
+bool SessionCountComparer(const Value& a, const Value& b) {
+  return a["session_count"].GetInt64() < b["session_count"].GetInt64();
+}
+
+void ImpalaHttpHandler::FillUsersInfo(Document* document) {
+  Value users(kArrayType);
+  {
+    lock_guard<mutex> l(server_->per_user_session_count_lock_);
+    for (auto const& it : server_->per_user_session_count_map_) {
+      const string& name = it.first;
+      const int64& session_count = it.second;
+      Value users_json(kObjectType);
+      Value user_name(name.c_str(), document->GetAllocator());
+      users_json.AddMember("user", user_name, document->GetAllocator());
+      users_json.AddMember("session_count", session_count, document->GetAllocator());
+      users.PushBack(users_json, document->GetAllocator());
+    }
+  }
+  sort(users.Begin(), users.End(), SessionCountComparer);
+  document->AddMember("users", users, document->GetAllocator());
+}
+
 void ImpalaHttpHandler::FillClientHostsInfo(
     Document* document, const ThriftServer::ConnectionContextList& connection_contexts) {
   lock_guard<mutex> session_state_map_l(server_->session_state_map_lock_);
@@ -927,8 +961,8 @@ void ImpalaHttpHandler::CatalogHandler(const Webserver::WebRequest& req,
     database.AddMember("name", str, document->GetAllocator());
 
     TGetTablesResult get_table_results;
-    Status status = server_->exec_env_->frontend()->GetTableNames(
-        db.db_name, NULL, NULL, &get_table_results);
+    status = server_->exec_env_->frontend()->GetTableNames(
+        db.db_name, nullptr, nullptr, &get_table_results);
     if (!status.ok()) {
       Value error(status.GetDetail().c_str(), document->GetAllocator());
       database.AddMember("error", error, document->GetAllocator());
@@ -1140,6 +1174,8 @@ void ImpalaHttpHandler::QueryBackendsHandler(
       return;
     }
     query_handle->GetCoordinator()->BackendsToJson(document);
+  } else {
+    VLOG(1) << "Failed to get backends of the query: " << status.GetDetail();
   }
 }
 
@@ -1163,6 +1199,8 @@ void ImpalaHttpHandler::QueryFInstancesHandler(
       return;
     }
     query_handle->GetCoordinator()->FInstanceStatsToJson(document);
+  } else {
+    VLOG(1) << "Failed to get fragment instances of the query: " << status.GetDetail();
   }
 }
 
@@ -1179,7 +1217,7 @@ void ImpalaHttpHandler::QuerySummaryHandler(bool include_json_plan, bool include
     return;
   }
 
-  shared_ptr<ImpalaServer::QueryStateRecord> query_record = nullptr;
+  shared_ptr<QueryStateRecord> query_record = nullptr;
   TExecSummary summary;
   string stmt;
   string plan;
@@ -1193,7 +1231,7 @@ void ImpalaHttpHandler::QuerySummaryHandler(bool include_json_plan, bool include
     status = server_->GetQueryHandle(query_id, &query_handle);
     if (status.ok()) {
       inflight = true;
-      query_record = make_shared<ImpalaServer::QueryStateRecord>(*query_handle);
+      query_record = make_shared<QueryStateRecord>(*query_handle);
       // If the query plan isn't generated, avoid waiting for the request
       // state lock to be acquired, since it could potentially be an expensive
       // call, if the table Catalog metadata loading is in progress. Instead
@@ -1206,14 +1244,15 @@ void ImpalaHttpHandler::QuerySummaryHandler(bool include_json_plan, bool include
       lock_guard<mutex> l(*(*query_handle).lock());
       query_status = query_handle->query_status();
       stmt = query_handle->sql_stmt();
-      plan = query_handle->exec_request().query_exec_request.query_plan;
+      const TExecRequest& exec_request = query_handle->exec_request();
+      plan = exec_request.query_exec_request.query_plan;
       if ((include_json_plan || include_summary)
           && query_handle->GetCoordinator() != nullptr) {
         query_handle->GetCoordinator()->GetTExecSummary(&summary);
       }
       if (include_json_plan) {
         for (const TPlanExecInfo& plan_exec_info:
-            query_handle->exec_request().query_exec_request.plan_exec_info) {
+            exec_request.query_exec_request.plan_exec_info) {
           for (const TPlanFragment& fragment: plan_exec_info.fragments) {
             fragments.push_back(fragment);
           }
