@@ -18,6 +18,7 @@
 #pragma once
 
 #include <atomic>
+#include <memory>
 #include <mutex>
 #include <unordered_map>
 #include <boost/random/random_device.hpp>
@@ -34,10 +35,14 @@
 #include "gen-cpp/ImpalaHiveServer2Service.h"
 #include "gen-cpp/ImpalaService.h"
 #include "gen-cpp/control_service.pb.h"
+#include "gen-cpp/Query_types.h"
 #include "kudu/util/random.h"
 #include "rpc/thrift-server.h"
 #include "runtime/types.h"
+#include "service/internal-server.h"
 #include "service/query-options.h"
+#include "service/query-state-record.h"
+#include "service/workload-management.h"
 #include "statestore/statestore-subscriber.h"
 #include "util/condition-variable.h"
 #include "util/container-util.h"
@@ -45,6 +50,7 @@
 #include "util/sharded-query-map-util.h"
 #include "util/simple-logger.h"
 #include "util/thread-pool.h"
+#include "util/ticker.h"
 #include "util/time.h"
 
 namespace kudu {
@@ -75,6 +81,7 @@ class TGetExecSummaryReq;
 class ClientRequestState;
 class QueryDriver;
 struct QueryHandle;
+class QueryScanner;
 class SimpleLogger;
 class UpdateFilterParamsPB;
 class UpdateFilterResultPB;
@@ -178,10 +185,11 @@ class TQueryExecRequest;
 /// 1. session_state_map_lock_
 /// 2. SessionState::lock
 /// 3. query_expiration_lock_
-/// 4. ClientRequestState::fetch_rows_lock
-/// 5. ClientRequestState::lock
-/// 6. ClientRequestState::expiration_data_lock_
-/// 7. Coordinator::exec_summary_lock
+/// 4. idle_query_statuses_lock_
+/// 5. ClientRequestState::fetch_rows_lock
+/// 6. ClientRequestState::lock
+/// 7. ClientRequestState::expiration_data_lock_
+/// 8. Coordinator::exec_summary_lock
 ///
 /// The following locks are not held in conjunction with other locks:
 /// * query_log_lock_
@@ -190,6 +198,7 @@ class TQueryExecRequest;
 /// * uuid_lock_
 /// * catalog_version_lock_
 /// * connection_to_sessions_map_lock_
+/// * per_user_session_count_lock_
 ///
 /// TODO: The same doesn't apply to the execution state of an individual plan
 /// fragment: the originating coordinator might die, but we can get notified of
@@ -197,6 +206,7 @@ class TQueryExecRequest;
 class ImpalaServer : public ImpalaServiceIf,
                      public ImpalaHiveServer2ServiceIf,
                      public ThriftServer::ConnectionHandlerIf,
+                     public InternalServer,
                      public std::enable_shared_from_this<ImpalaServer>,
                      public CacheLineAligned {
  public:
@@ -382,7 +392,7 @@ class ImpalaServer : public ImpalaServiceIf,
   static void PrepareQueryContext(const std::string& hostname,
       const NetworkAddressPB& krpc_addr, TQueryCtx* query_ctx);
 
-  /// SessionHandlerIf methods
+  /// ThriftServer::ConnectionHandlerIf methods
 
   /// Called when a Beeswax or HS2 connection starts. For Beeswax, registers a new
   /// SessionState associated with the new connection. For HS2, this is a no-op (HS2
@@ -398,6 +408,29 @@ class ImpalaServer : public ImpalaServiceIf,
   /// Called when a client has been inactive for --idle_client_poll_period_s seconds.
   virtual bool IsIdleConnection(const ThriftServer::ConnectionContext& session_context);
 
+  /// InternalServer methods, see internal-server.h for details
+  virtual Status OpenSession(const std::string& user_name, TUniqueId& new_session_id,
+      const QueryOptionMap& query_opts = {});
+  virtual bool CloseSession(const impala::TUniqueId& session_id);
+  virtual Status ExecuteIgnoreResults(const std::string& user_name,
+      const std::string& sql, const QueryOptionMap& query_opts = {},
+      const bool persist_in_db = true, TUniqueId* query_id = nullptr);
+  virtual Status ExecuteAndFetchAllText(const std::string& user_name,
+      const std::string& sql, query_results& results, results_columns* columns = nullptr,
+      TUniqueId* query_id = nullptr);
+  virtual Status SubmitAndWait(const std::string& user_name, const std::string& sql,
+      TUniqueId& new_session_id, TUniqueId& new_query_id,
+      const QueryOptionMap& query_opts = {}, const bool persist_in_db = true);
+  virtual Status WaitForResults(TUniqueId& query_id);
+  virtual Status SubmitQuery(const std::string& sql, const impala::TUniqueId& session_id,
+      TUniqueId& new_query_id, const bool persist_in_db = true);
+  virtual Status FetchAllRows(const TUniqueId& query_id, query_results& results,
+      results_columns* columns = nullptr);
+  virtual void CloseQuery(const TUniqueId& query_id);
+  virtual void GetConnectionContextList(
+      ThriftServer::ConnectionContextList* connection_contexts);
+  /// end of InternalServer methods
+
   void CatalogUpdateCallback(const StatestoreSubscriber::TopicDeltaMap& topic_deltas,
       std::vector<TTopicDelta>* topic_updates);
 
@@ -473,6 +506,11 @@ class ImpalaServer : public ImpalaServiceIf,
   /// the server has started successfully.
   int GetHS2Port();
 
+  /// Return the number of live queries managed by this server. Acquires
+  /// completed_queries_lock_ to check for completed queries that have not been written.
+  /// (implemented in workload-management.cc)
+  size_t NumLiveQueries();
+
   /// Returns a current snapshot of the local backend descriptor.
   std::shared_ptr<const BackendDescriptorPB> GetLocalBackendDescriptor();
 
@@ -613,6 +651,15 @@ class ImpalaServer : public ImpalaServiceIf,
     /// that need to be closed if the session closes or expires.
     boost::unordered_set<TUniqueId> inflight_queries;
 
+    /// Queries are added to inflight_queries via SetQueryInflight. Execution is async
+    /// and can start before SetQueryInflight is called. If a query is retried before
+    /// SetQueryInflight is called, the original may be cleaned up before it is added to
+    /// inflight_queries. In that case we add it to prestopped_queries instead.
+    std::set<TUniqueId> prestopped_queries;
+
+    /// Unregistered queries we need to clear from idle_query_statuses_ on closure.
+    std::vector<TUniqueId> idled_queries;
+
     /// Total number of queries run as part of this session.
     int64_t total_queries;
 
@@ -655,6 +702,10 @@ class ImpalaServer : public ImpalaServiceIf,
     TQueryOptions QueryOptions();
   };
 
+  /// Helper function that decrements the value associated with the given key.
+  /// Removes the entry from the map if the value becomes zero.
+  static void DecrementCount(std::map<std::string, int64>& loads, const std::string& key);
+
  private:
   struct ExpirationEvent;
   class SecretArg;
@@ -664,10 +715,12 @@ class ImpalaServer : public ImpalaServiceIf,
   friend struct SessionState;
   friend class ImpalaServerTest;
   friend class QueryDriver;
+  friend class QueryScanner;
 
   static const string BEESWAX_SERVER_NAME;
   static const string HS2_SERVER_NAME;
   static const string HS2_HTTP_SERVER_NAME;
+  static const string INTERNAL_SERVER_NAME;
   // Used to identify external frontend RPC calls
   static const string EXTERNAL_FRONTEND_SERVER_NAME;
 
@@ -707,8 +760,8 @@ class ImpalaServer : public ImpalaServiceIf,
   /// external_exec_request is a statement that was prepared by an external frontend using
   /// Impala PlanNodes or null if the external frontend isn't being used.
   Status Execute(TQueryCtx* query_ctx, std::shared_ptr<SessionState> session_state,
-      QueryHandle* query_handle,
-      const TExecRequest* external_exec_request) WARN_UNUSED_RESULT;
+      QueryHandle* query_handle, const TExecRequest* external_exec_request,
+      const bool include_in_query_log = true) WARN_UNUSED_RESULT;
 
   /// Implements Execute() logic, but doesn't unregister query on error.
   Status ExecuteInternal(const TQueryCtx& query_ctx,
@@ -731,13 +784,18 @@ class ImpalaServer : public ImpalaServiceIf,
       QueryHandle* query_handle) WARN_UNUSED_RESULT;
 
   /// Adds the query to the set of in-flight queries for the session. The query remains
-  /// in-flight until the query is unregistered.  Until a query is in-flight, an attempt
-  /// to cancel or close the query by the user will return an error status.  If the
-  /// session is closed before a query is in-flight, then the query cancellation is
-  /// deferred until after the issuing path has completed initializing the query.  Once
+  /// in-flight until the query is unregistered. Until a query is in-flight, an attempt
+  /// to cancel or close the query by the user will return an error status. A query that
+  /// retries may close the original query state before SetQueryInflight is called; in
+  /// that case it adds the query to prestopped_queries, which will bypass unnecessary
+  /// setup when SetQueryInflight is later called on it.
+  ///
+  /// If the session is closed before a query is in-flight, then the query cancellation
+  /// is deferred until after the issuing path has completed initializing the query. Once
   /// a query is in-flight, it can be cancelled/closed asynchronously by the user
   /// (e.g. via an RPC) and the session close path can close (cancel and unregister) it.
-  /// The query must have already been registered using RegisterQuery().  The caller
+  ///
+  /// The query must have already been registered using RegisterQuery(). The caller
   /// must have checked out the session state.
   Status SetQueryInflight(std::shared_ptr<SessionState> session_state,
       const QueryHandle& query_handle) WARN_UNUSED_RESULT;
@@ -906,142 +964,10 @@ class ImpalaServer : public ImpalaServiceIf,
   /// which are returned as strings (see IMPALA-11041).
   std::string ColumnTypeToBeeswaxTypeString(const TColumnType& type);
 
-  /// Snapshot of a query's state, archived in the query log. Not mutated after
-  /// construction.
-  struct QueryStateRecord {
-    /// Compressed representation of profile returned by RuntimeProfile::Compress().
-    /// Must be initialised to a valid value if this is a completed query.
-    /// Empty if this was initialised from a running query.
-    const std::vector<uint8_t> compressed_profile;
-
-    /// Query id
-    TUniqueId id;
-
-    /// Queries are run and authorized on behalf of the effective_user.
-    /// If there is no delegated user, this will be the connected user. Otherwise, it
-    /// will be set to the delegated user.
-    std::string effective_user;
-
-    /// If true, effective_user has access to the runtime profile and execution
-    /// summary.
-    bool user_has_profile_access;
-
-    /// default db for this query
-    std::string default_db;
-
-    /// SQL statement text
-    std::string stmt;
-
-    /// Text representation of plan
-    std::string plan;
-
-    /// DDL, DML etc.
-    TStmtType::type stmt_type;
-
-    /// True if the query required a coordinator fragment
-    bool has_coord;
-
-    /// The number of scan ranges that have completed.
-    int64_t num_completed_scan_ranges;
-
-    /// The total number of scan ranges.
-    int64_t total_scan_ranges;
-
-    /// The number of fragment instances that have completed.
-    int64_t num_completed_fragment_instances;
-
-    /// The total number of fragment instances.
-    int64_t total_fragment_instances;
-
-    /// The number of rows fetched by the client
-    int64_t num_rows_fetched;
-
-    /// The state of the query as of this snapshot. The possible values for the
-    /// query_state = union(beeswax::QueryState, ClientRequestState::RetryState). This is
-    /// necessary so that the query_state can accurately reflect if a query has been
-    /// retried or not. This string is not displayed in the runtime profiles, it is only
-    /// displayed on the /queries endpoint of the Web UI when listing out the state of
-    /// each query. This is necessary so that users can clearly see if a query has been
-    /// retried or not.
-    std::string query_state;
-
-    /// The beeswax::QueryState of the query as of this snapshot.
-    beeswax::QueryState::type beeswax_query_state;
-
-    /// Start and end time of the query, in Unix microseconds.
-    /// A query whose end_time_us is 0 indicates that it is an in-flight query.
-    /// These two variables are initialized with the corresponding values from
-    /// ClientRequestState.
-    int64_t start_time_us, end_time_us;
-
-    /// The request waited time in ms for queued.
-    int64_t wait_time_ms;
-
-    /// Total peak memory usage by this query at all backends.
-    int64_t total_peak_mem_usage;
-
-    /// The cluster wide estimated memory usage of this query.
-    int64_t cluster_mem_est;
-
-    /// Total bytes read by this query at all backends.
-    int64_t bytes_read;
-
-    /// The total number of bytes sent (across the network) by this query in exchange
-    /// nodes. Does not include remote reads, data written to disk, or data sent to the
-    /// client.
-    int64_t bytes_sent;
-
-    // Query timeline from summary profile.
-    std::string timeline;
-
-    /// Summary of execution for this query.
-    TExecSummary exec_summary;
-
-    Status query_status;
-
-    /// Timeline of important query events
-    TEventSequence event_sequence;
-
-    /// Save the query plan fragments so that the plan tree can be rendered on the debug
-    /// webpages.
-    vector<TPlanFragment> fragments;
-
-    // If true, this query has no more rows to return
-    bool all_rows_returned;
-
-    // The most recent time this query was actively being processed, in Unix milliseconds.
-    int64_t last_active_time_ms;
-
-    /// Resource pool to which the request was submitted for admission, or an empty
-    /// string if this request doesn't go through admission control.
-    std::string resource_pool;
-
-    /// True if this query was retried, false otherwise.
-    bool was_retried = false;
-
-    /// If this query was retried, the query id of the retried query.
-    std::unique_ptr<const TUniqueId> retried_query_id;
-
-    /// Initialise from 'exec_state' of a completed query. 'compressed_profile' must be
-    /// a runtime profile decompressed with RuntimeProfile::Compress().
-    QueryStateRecord(
-        const ClientRequestState& exec_state, std::vector<uint8_t>&& compressed_profile);
-
-    /// Initialize from 'exec_state' of a running query
-    QueryStateRecord(const ClientRequestState& exec_state);
-
-    /// Default constructor used only when participating in collections
-    QueryStateRecord() { }
-
-   private:
-    // Common initialization for constructors.
-    void Init(const ClientRequestState& exec_state);
-  };
-
-  struct QueryStateRecordLessThan {
-    /// Comparator that sorts by start time.
-    bool operator() (const QueryStateRecord& lhs, const QueryStateRecord& rhs) const;
-  };
+  /// Places a completed query into the in-memory queue of completed queries.
+  /// (implemented in workload-management.cc)
+  void EnqueueCompletedQuery(const QueryHandle& query_handle,
+      const std::shared_ptr<QueryStateRecord> qs_rec);
 
   /// Returns the active QueryHandle for this query id. The QueryHandle contains the
   /// active ClientRequestState. Returns an error Status if the query id cannot be found.
@@ -1207,14 +1133,32 @@ class ImpalaServer : public ImpalaServiceIf,
   /// current query ids to the admissiond.
   [[noreturn]] void AdmissionHeartbeatThread();
 
+  /// If workload management is enabled, starts workload management threads.
+  /// (implemented in workload-management.cc)
+  Status InitWorkloadManagement();
+
+  /// Blocks until running workload management threads are shut down.
+  /// (implemented in workload-management.cc)
+  void ShutdownWorkloadManagement();
+
+  /// Periodically writes out completed queries (if configured)
+  /// (implemented in workload-management.cc)
+  void CompletedQueriesThread();
+
+  /// Returns a list of completed queries that have not yet been written to storage.
+  /// Acquires completed_queries_lock_ to make a copy of completed_queries_ state.
+  /// (implemented in workload-management.cc)
+  std::vector<std::shared_ptr<QueryStateExpanded>> GetCompletedQueries();
+
   /// Called from ExpireQueries() to check query resource limits for 'crs'. If the query
   /// exceeded a resource limit, returns a non-OK status with information about what
   /// limit was exceeded. Returns OK if the query will continue running and expiration
   /// check should be rescheduled for a later time.
   Status CheckResourceLimits(ClientRequestState* crs);
 
-  /// Expire 'crs' and cancel it with status 'status'.
-  void ExpireQuery(ClientRequestState* crs, const Status& status);
+  /// Expire 'crs' and cancel it with status 'status'. Optionally unregisters the query.
+  void ExpireQuery(ClientRequestState* crs, const Status& status,
+      bool unregister = false);
 
   typedef boost::unordered_map<std::string, boost::unordered_set<std::string>>
       AuthorizedProxyMap;
@@ -1234,7 +1178,7 @@ class ImpalaServer : public ImpalaServiceIf,
   /// Random number generator for use in this class, thread safe.
   static ThreadSafeRandom rng_;
 
-  /// Guards query_log_ and query_log_index_
+  /// Guards query_log_, query_log_index_, and query_log_est_sizes_.
   std::mutex query_log_lock_;
 
   /// FIFO list of query records, which are written after the query finishes executing.
@@ -1253,6 +1197,9 @@ class ImpalaServer : public ImpalaServiceIf,
       QueryLogIndex;
   QueryLogIndex query_log_index_;
 
+  /// Estimated individual size of records in query_log_ in bytes.
+  std::list<int64_t> query_log_est_sizes_;
+
   /// Sets the given query_record (and retried_query_record too if given) for the given
   /// query_id. Returns an error Status if the given query_id cannot be found in the
   /// QueryLogIndex.
@@ -1268,6 +1215,9 @@ class ImpalaServer : public ImpalaServiceIf,
   void WaitForNewCatalogServiceId(TUniqueId cur_service_id,
       std::unique_lock<std::mutex>* ver_lock);
 
+  /// Random `impala::TUniqueID` generator. Use wherever a new `TUniqueId` is needed.
+  TUniqueId RandomUniqueID();
+
   /// Logger for writing encoded query profiles, one per line with the following format:
   /// <ms-since-epoch> <query-id> <thrift query profile URL encoded and gzipped>
   boost::scoped_ptr<SimpleLogger> profile_logger_;
@@ -1477,10 +1427,32 @@ class ImpalaServer : public ImpalaServiceIf,
   typedef boost::unordered_map<TUniqueId, std::shared_ptr<SessionState>> SessionStateMap;
   SessionStateMap session_state_map_;
 
+  /// Protects idle_query_statuses_;
+  std::mutex idle_query_statuses_lock_;
+
+  /// A map of queries that were stopped due to idle timeout and the status they had when
+  /// unregistered. Used to return a more useful error when looking up unregistered IDs.
+  std::map<TUniqueId, Status> idle_query_statuses_;
+
   /// Protects connection_to_sessions_map_. See "Locking" in the class comment for lock
   /// acquisition order.
   std::mutex connection_to_sessions_map_lock_;
 
+  /// A map from user to a count of sessions created by the user.
+  typedef std::map<std::string, int64> SessionCounts;
+  SessionCounts per_user_session_count_map_;
+
+  /// Protects per_user_session_count_map_. See "Locking" in the class comment for lock
+  /// acquisition order.
+  std::mutex per_user_session_count_lock_;
+
+  /// Increment the count of HS2 sessions used by the user.
+  /// If max_hs2_sessions_per_user is greater than zero, and the count of HS2 sessions
+  /// used by the user would be above that value, then an error status is returned.
+  Status IncrementAndCheckSessionCount(const string& user_name);
+  /// Decrement the count of HS2 sessions used by the user.
+  void DecrementSessionCount(const string& user_name);
+
   /// Map from a connection ID to the associated list of sessions so that all can be
   /// closed when the connection ends. HS2 allows for multiplexing several sessions across
   /// a single connection. If a session has already been closed (only possible via HS2) it
@@ -1488,6 +1460,14 @@ class ImpalaServer : public ImpalaServiceIf,
   typedef boost::unordered_map<TUniqueId, std::set<TUniqueId>> ConnectionToSessionMap;
   ConnectionToSessionMap connection_to_sessions_map_;
 
+  /// Map storing connections opened by the InternalServer functions. Key is the session
+  /// id, value is a shared pointer holding the connection's ConnectionContext. Use the
+  /// `internal_server_connections_lock_` mutex whenever accessing this map
+  typedef std::map<TUniqueId, std::shared_ptr<ThriftServer::ConnectionContext>>
+      SessionToConnectionContext;
+  SessionToConnectionContext internal_server_connections_;
+  std::mutex internal_server_connections_lock_;
+
   /// Returns session state for given session_id.
   /// If not found or validation of 'secret' against the stored secret in the
   /// SessionState fails, session_state will be NULL and an error status will be returned.
@@ -1670,6 +1650,7 @@ class ImpalaServer : public ImpalaServiceIf,
   boost::scoped_ptr<ThriftServer> hs2_server_;
   boost::scoped_ptr<ThriftServer> hs2_http_server_;
   boost::scoped_ptr<ThriftServer> external_fe_server_;
+  std::shared_ptr<InternalServer> internal_server_;
 
   /// Flag that records if backend and/or client services have been started. The flag is
   /// set after all services required for the server have been started.
@@ -1687,5 +1668,31 @@ class ImpalaServer : public ImpalaServiceIf,
   /// Stores the last version number for the admission heartbeat that was sent.
   /// Incremented every time a new admission heartbeat is sent.
   int64_t admission_heartbeat_version_ = 0;
+
+  /// Workload Management Related Declarations.
+  /// Coordinate periodic execution of the completed queries queue processing thread.
+  std::condition_variable completed_queries_cv_;
+
+  /// Coordinate shutdown of the completed queries queue processing thread.
+  std::condition_variable completed_queries_shutdown_cv_;
+
+  /// Tracks the state of the thread that drains the completed queries queue to the table.
+  /// The associated lock must be held before reading/modifying this variable.
+  impala::workload_management::ThreadState completed_queries_thread_state_ =
+      impala::workload_management::NOT_STARTED;
+  std::mutex completed_queries_threadstate_mu_;
+
+  /// Thread that runs CompletedQueriesThread().
+  std::unique_ptr<Thread> completed_queries_thread_;
+
+  /// Ticker that wakes up the completed_queried_thread at set intervals to process the
+  /// queued completed queries. Uses the completed_queries_lock_ to synchonize access to
+  /// the completed_queries_ list.
+  std::unique_ptr<TickerSecondsBool> completed_queries_ticker_;
+
+  /// Queue of completed queries and the lock to synchronize access to it.
+  std::list<impala::workload_management::CompletedQuery> completed_queries_;
+  std::mutex completed_queries_lock_;
+
 };
 }
