@@ -161,6 +161,7 @@ TEST(QueryOptions, SetByteOptions) {
       {MAKE_OPTIONDEF(max_spilled_result_spooling_mem), {-1, I64_MAX}},
       {MAKE_OPTIONDEF(large_agg_mem_threshold), {-1, I64_MAX}},
       {MAKE_OPTIONDEF(mem_limit_coordinators), {-1, I64_MAX}},
+      {MAKE_OPTIONDEF(hdfs_scanner_non_reserved_bytes), {-1, I64_MAX}},
   };
   vector<pair<OptionDef<int32_t>, Range<int32_t>>> case_set_i32{
       {MAKE_OPTIONDEF(runtime_filter_min_size),
@@ -230,7 +231,7 @@ TEST(QueryOptions, SetEnumOptions) {
       (THREE_LEVEL, TWO_LEVEL, TWO_LEVEL_THEN_THREE_LEVEL)), true);
   TestEnumCase(options, CASE(default_file_format, THdfsFileFormat,
       (TEXT, RC_FILE, SEQUENCE_FILE, AVRO, PARQUET, KUDU, ORC, HUDI_PARQUET, ICEBERG,
-      JSON)), true);
+      JSON, JDBC)), true);
   TestEnumCase(options, CASE(runtime_filter_mode, TRuntimeFilterMode,
       (OFF, LOCAL, GLOBAL)), true);
   TestEnumCase(options, CASE(kudu_read_mode, TKuduReadMode,
@@ -267,6 +268,9 @@ TEST(QueryOptions, SetIntOptions) {
           {1, qc.MAX_FRAGMENT_INSTANCES_PER_NODE}},
       {MAKE_OPTIONDEF(max_fragment_instances_per_node),
           {1, qc.MAX_FRAGMENT_INSTANCES_PER_NODE}},
+      {MAKE_OPTIONDEF(max_num_filters_aggregated_per_host),
+          {-1, I32_MAX}},
+      {MAKE_OPTIONDEF(num_nodes),                      {0, 1}},
   };
   for (const auto& test_case : case_set) {
     const OptionDef<int32_t>& option_def = test_case.first;
@@ -309,6 +313,45 @@ TEST(QueryOptions, SetBigIntOptions) {
   }
 }
 
+// Test double options with expected value between 0 and 1 (inclusive or exclusive).
+TEST(QueryOptions, SetFractionalOptions) {
+  TQueryOptions options;
+  // List of pairs of Key and boolean flag on whether the option is inclusive of 0 and 1.
+  pair<OptionDef<double>, bool> case_set[]{
+      {MAKE_OPTIONDEF(resource_trace_ratio), true},
+      {MAKE_OPTIONDEF(runtime_filter_error_rate), false},
+      {MAKE_OPTIONDEF(minmax_filter_threshold), true},
+      {MAKE_OPTIONDEF(join_selectivity_correlation_factor), true},
+      {MAKE_OPTIONDEF(agg_mem_correlation_factor), true},
+      {MAKE_OPTIONDEF(runtime_filter_cardinality_reduction_scale), true},
+  };
+  for (const auto& test_case : case_set) {
+    const OptionDef<double>& option_def = test_case.first;
+    const bool& is_inclusive = test_case.second;
+    auto TestOk = MakeTestOkFn(options, option_def);
+    auto TestError = MakeTestErrFn(options, option_def);
+    TestOk("0.5", 0.5);
+    TestOk("0.01", 0.01);
+    TestOk("0.001", 0.001);
+    TestOk("0.0001", 0.0001);
+    TestOk("0.0000000001", 0.0000000001);
+    TestOk("0.999999999", 0.999999999);
+    TestOk(" 0.9", 0.9);
+    if (is_inclusive) {
+      TestOk("1", 1.0);
+      TestOk("0", 0.0);
+    } else {
+      TestError("1");
+      TestError("0");
+    }
+    // Out of range values
+    TestError("-1");
+    TestError("-0.1");
+    TestError("1.1");
+    TestError("Not a number!");
+  }
+}
+
 // Test options with non regular validation rule
 TEST(QueryOptions, SetSpecialOptions) {
   // REPLICA_PREFERENCE has unsettable enum values: cache_rack(1) & disk_rack(3)
@@ -386,28 +429,41 @@ TEST(QueryOptions, SetSpecialOptions) {
     TestError("8191"); // default value of FLAGS_min_buffer_size is 8KB
     TestOk("64KB", 64 * 1024);
   }
+  // QUERY_CPU_COUNT_DIVISOR should be greater than 0.0.
   {
-    // RUNTIME_FILTER_ERROR_RATE is a double in range (0.0, 1.0)
-    OptionDef<double> key_def = MAKE_OPTIONDEF(runtime_filter_error_rate);
+    OptionDef<double> key_def = MAKE_OPTIONDEF(query_cpu_count_divisor);
     auto TestOk = MakeTestOkFn(options, key_def);
     auto TestError = MakeTestErrFn(options, key_def);
     TestOk("0.5", 0.5);
-    TestOk("0.01", 0.01);
-    TestOk("0.001", 0.001);
-    TestOk("0.0001", 0.0001);
     TestOk("0.0000000001", 0.0000000001);
     TestOk("0.999999999", 0.999999999);
     TestOk(" 0.9", 0.9);
-    // Out of range values
-    TestError("1");
+    TestOk("1", 1.0);
+    TestOk("1.1", 1.1);
+    TestOk("1000.00", 1000.0);
     TestError("0");
     TestError("-1");
     TestError("-0.1");
-    TestError("1.1");
     TestError("Not a number!");
   }
 }
 
+void VerifyFilterTypes(const set<TRuntimeFilterType::type>& types,
+    const std::initializer_list<TRuntimeFilterType::type>& expects) {
+  EXPECT_EQ(expects.size(), types.size());
+  for (const auto t : expects) {
+    EXPECT_NE(types.end(), types.find(t));
+  }
+}
+
+void VerifyFilterIds(
+    const set<int32_t>& types, const std::initializer_list<int32_t>& expects) {
+  EXPECT_EQ(expects.size(), types.size());
+  for (const auto t : expects) {
+    EXPECT_NE(types.end(), types.find(t));
+  }
+}
+
 TEST(QueryOptions, ParseQueryOptions) {
   QueryOptionsMask expectedMask;
   expectedMask.set(TImpalaQueryOptions::NUM_NODES);
@@ -433,6 +489,36 @@ TEST(QueryOptions, ParseQueryOptions) {
     EXPECT_FALSE(status.ok());
     EXPECT_EQ(status.msg().details().size(), 2);
   }
+
+  QueryOptionsMask expectedMask2;
+  expectedMask2.set(TImpalaQueryOptions::ENABLED_RUNTIME_FILTER_TYPES);
+  expectedMask2.set(TImpalaQueryOptions::RUNTIME_FILTER_IDS_TO_SKIP);
+
+  {
+    TQueryOptions options;
+    QueryOptionsMask mask;
+    Status status = ParseQueryOptions("enabled_runtime_filter_types=\"bloom,min_max\","
+                                      "runtime_filter_ids_to_skip=\"1,2\"",
+        &options, &mask);
+    VerifyFilterTypes(options.enabled_runtime_filter_types,
+        {TRuntimeFilterType::BLOOM, TRuntimeFilterType::MIN_MAX});
+    VerifyFilterIds(options.runtime_filter_ids_to_skip, {1, 2});
+    EXPECT_EQ(mask, expectedMask2);
+    EXPECT_TRUE(status.ok());
+  }
+
+  {
+    TQueryOptions options;
+    QueryOptionsMask mask;
+    Status status = ParseQueryOptions("enabled_runtime_filter_types=bloom,min_max,"
+                                      "runtime_filter_ids_to_skip=1,2",
+        &options, &mask);
+    VerifyFilterTypes(options.enabled_runtime_filter_types, {TRuntimeFilterType::BLOOM});
+    VerifyFilterIds(options.runtime_filter_ids_to_skip, {1});
+    EXPECT_EQ(mask, expectedMask2);
+    EXPECT_FALSE(status.ok());
+    EXPECT_EQ(status.msg().details().size(), 2);
+  }
 }
 
 TEST(QueryOptions, MapOptionalDefaultlessToEmptyString) {
@@ -562,14 +648,6 @@ TEST(QueryOptions, CompressionCodec) {
 #undef ENTRY
 }
 
-void VerifyFilterTypes(const set<TRuntimeFilterType::type>& types,
-    const std::initializer_list<TRuntimeFilterType::type>& expects) {
-  EXPECT_EQ(expects.size(), types.size());
-  for (const auto t : expects) {
-    EXPECT_NE(types.end(), types.find(t));
-  }
-}
-
 // Tests for setting of ENABLED_RUNTIME_FILTER_TYPES.
 TEST(QueryOptions, EnabledRuntimeFilterTypes) {
   const string KEY = "enabled_runtime_filter_types";
@@ -607,6 +685,15 @@ TEST(QueryOptions, EnabledRuntimeFilterTypes) {
             TRuntimeFilterType::MIN_MAX
         });
   }
+  {
+    TQueryOptions options;
+    EXPECT_TRUE(SetQueryOption(KEY, "bloom , , min_max", &options, nullptr).ok());
+    VerifyFilterTypes(options.enabled_runtime_filter_types,
+        {
+            TRuntimeFilterType::BLOOM,
+            TRuntimeFilterType::MIN_MAX
+        });
+  }
   {
     TQueryOptions options;
     EXPECT_TRUE(SetQueryOption(KEY, "in_list,bloom", &options, nullptr).ok());
@@ -618,6 +705,59 @@ TEST(QueryOptions, EnabledRuntimeFilterTypes) {
   }
 }
 
+// Tests for setting of RUNTIME_FILTER_IDS_TO_SKIP.
+TEST(QueryOptions, RuntimeFilterIdsToSkip) {
+  const string KEY = "runtime_filter_ids_to_skip";
+  {
+    TQueryOptions options;
+    EXPECT_TRUE(SetQueryOption(KEY, "0", &options, nullptr).ok());
+    VerifyFilterIds(options.runtime_filter_ids_to_skip, {0});
+  }
+  {
+    TQueryOptions options;
+    EXPECT_TRUE(SetQueryOption(KEY, "0,1", &options, nullptr).ok());
+    VerifyFilterIds(options.runtime_filter_ids_to_skip, {0, 1});
+  }
+  {
+    TQueryOptions options;
+    EXPECT_TRUE(SetQueryOption(KEY, "111,2,33", &options, nullptr).ok());
+    VerifyFilterIds(options.runtime_filter_ids_to_skip, {2, 33, 111});
+  }
+  {
+    TQueryOptions options;
+    EXPECT_TRUE(SetQueryOption(KEY, "-1,0,1", &options, nullptr).ok());
+    VerifyFilterIds(options.runtime_filter_ids_to_skip, {-1, 0, 1});
+  }
+  {
+    TQueryOptions options;
+    EXPECT_TRUE(SetQueryOption(KEY, "1,6.9", &options, nullptr).ok());
+    VerifyFilterIds(options.runtime_filter_ids_to_skip, {1, 6});
+  }
+  {
+    TQueryOptions options;
+    EXPECT_TRUE(SetQueryOption(KEY, "0, 1", &options, nullptr).ok());
+    VerifyFilterIds(options.runtime_filter_ids_to_skip, {0, 1});
+  }
+  {
+    TQueryOptions options;
+    EXPECT_TRUE(SetQueryOption(KEY, "0,,1", &options, nullptr).ok());
+    VerifyFilterIds(options.runtime_filter_ids_to_skip, {0, 1});
+  }
+  {
+    TQueryOptions options;
+    EXPECT_TRUE(SetQueryOption(KEY, "0, 1, , 2", &options, nullptr).ok());
+    VerifyFilterIds(options.runtime_filter_ids_to_skip, {0, 1, 2});
+  }
+  {
+    TQueryOptions options;
+    EXPECT_FALSE(SetQueryOption(KEY, "1,b", &options, nullptr).ok());
+  }
+  {
+    TQueryOptions options;
+    EXPECT_FALSE(SetQueryOption(KEY, "1,4294967295", &options, nullptr).ok());
+  }
+}
+
 // Tests for setting of MAX_RESULT_SPOOLING_MEM and
 // MAX_SPILLED_RESULT_SPOOLING_MEM. Setting of these options must maintain the
 // condition 'MAX_RESULT_SPOOLING_MEM <= MAX_SPILLED_RESULT_SPOOLING_MEM'.
