@@ -18,7 +18,9 @@
 #include "service/query-options.h"
 
 #include <limits>
+#include <regex>
 #include <sstream>
+#include <string>
 
 #include <boost/algorithm/string.hpp>
 #include <gutil/strings/strip.h>
@@ -45,6 +47,7 @@ using namespace impala;
 using namespace strings;
 
 DECLARE_int32(idle_session_timeout);
+DECLARE_bool(allow_tuple_caching);
 
 void impala::OverlayQueryOptions(
     const TQueryOptions& src, const QueryOptionsMask& mask, TQueryOptions* dst) {
@@ -100,6 +103,22 @@ const string PrintQueryOptionValue(
   return val.str();
 }
 
+std::ostream& impala::operator<<(std::ostream& out, const std::set<int32_t>& filter_ids) {
+  bool first = true;
+  for (const auto& t : filter_ids) {
+    if (!first) out << ",";
+    out << t;
+    first = false;
+  }
+  return out;
+}
+
+const string PrintQueryOptionValue(const std::set<int32_t>& filter_ids) {
+  std::stringstream val;
+  val << filter_ids;
+  return val.str();
+}
+
 void impala::TQueryOptionsToMap(
     const TQueryOptions& query_options, std::map<string, string>* configuration) {
 #define QUERY_OPT_FN(NAME, ENUM, LEVEL)                                    \
@@ -160,6 +179,18 @@ string impala::DebugQueryOptions(const TQueryOptions& query_options) {
   return ss.str();
 }
 
+inline void TrimAndRemoveEmptyString(vector<string>& values) {
+  int i = 0;
+  while (i < values.size()) {
+    trim(values[i]);
+    if (values[i].length() == 0) {
+      values.erase(values.begin() + i);
+    } else {
+      i++;
+    }
+  }
+}
+
 // Returns the TImpalaQueryOptions enum for the given "key". Input is case insensitive.
 // Return -1 if the input is an invalid option.
 static int GetQueryOptionForKey(const string& key) {
@@ -200,9 +231,13 @@ Status impala::SetQueryOption(const string& key, const string& value,
   if (option_int < 0) {
     return Status(Substitute("Invalid query option: $0", key));
   }
+  return SetQueryOption(static_cast<TImpalaQueryOptions::type>(option_int),
+      value, query_options, set_query_options_mask);
+}
 
+Status impala::SetQueryOption(TImpalaQueryOptions::type option, const string& value,
+    TQueryOptions* query_options, QueryOptionsMask* set_query_options_mask) {
   QueryConstants qc;
-  TImpalaQueryOptions::type option = static_cast<TImpalaQueryOptions::type>(option_int);
 
   if (value.empty()) {
     ResetQueryOption(option, query_options);
@@ -241,7 +276,8 @@ Status impala::SetQueryOption(const string& key, const string& value,
       };
       case TImpalaQueryOptions::NUM_NODES: {
         int32_t int32_t_val = 0;
-        RETURN_IF_ERROR(QueryOptionParser::Parse<int32_t>(option, value, &int32_t_val));
+        RETURN_IF_ERROR(QueryOptionParser::ParseAndCheckInclusiveRange<int32_t>(
+            option, value, 0, 1, &int32_t_val));
         query_options->__set_num_nodes(int32_t_val);
         break;
       };
@@ -524,8 +560,11 @@ Status impala::SetQueryOption(const string& key, const string& value,
       }
       case TImpalaQueryOptions::DEFAULT_JOIN_DISTRIBUTION_MODE: {
         TJoinDistributionMode::type enum_type;
+        // Not using the values from '_TJoinDistributionMode_VALUES_TO_NAMES' so that we
+        // can exclude 'DIRECTED' mode from the options.
+        std::map<int, const char*> values_to_names {{0, "BROADCAST"}, {1, "SHUFFLE"}};
         RETURN_IF_ERROR(GetThriftEnum(value, "default join distribution mode",
-            _TJoinDistributionMode_VALUES_TO_NAMES, &enum_type));
+            values_to_names, &enum_type));
         query_options->__set_default_join_distribution_mode(enum_type);
         break;
       }
@@ -833,14 +872,26 @@ Status impala::SetQueryOption(const string& key, const string& value,
       }
       case TImpalaQueryOptions::ENABLED_RUNTIME_FILTER_TYPES: {
         std::set<TRuntimeFilterType::type> filter_types;
-        if (iequals(value, "all")) {
+        // Impala backend expects comma separated values to be in quotes when executing
+        // SET statement. This is usually the case when running
+        // SET query_option="value1,value2" using a jdbc driver. When using Impala-shell
+        // client, the SET statement is not executed immediately but query options are
+        // updated in the client and applied as part of following statement, so no quotes
+        // are required for Impala-shell SET query_option=value1,value2.
+        // By removing double quotes from the beginning and ending of the option value,
+        // SET ENABLED_RUNTIME_FILTER_TYPES="BLOOM,MIN_MAX" works for jdbc driver,
+        // both SET ENABLED_RUNTIME_FILTER_TYPES="BLOOM,MIN_MAX" and
+        // SET ENABLED_RUNTIME_FILTER_TYPES=BLOOM,MIN_MAX work for Impala-shell.
+        const string filter_value = std::regex_replace(value, std::regex("^\"|\"$"), "");
+        if (iequals(filter_value, "all")) {
           for (const auto& kv : _TRuntimeFilterType_VALUES_TO_NAMES) {
             filter_types.insert(static_cast<TRuntimeFilterType::type>(kv.first));
           }
         } else {
           // Parse and verify the enabled runtime filter types.
           vector<string> str_types;
-          split(str_types, value, is_any_of(","), token_compress_on);
+          split(str_types, filter_value, is_any_of(","), token_compress_on);
+          TrimAndRemoveEmptyString(str_types);
           for (const auto& t : str_types) {
             TRuntimeFilterType::type filter_type;
             RETURN_IF_ERROR(GetThriftEnum(t, "runtime filter type",
@@ -906,6 +957,14 @@ Status impala::SetQueryOption(const string& key, const string& value,
         query_options->__set_convert_legacy_hive_parquet_utc_timestamps(IsTrue(value));
         break;
       }
+      case TImpalaQueryOptions::CONVERT_KUDU_UTC_TIMESTAMPS: {
+        query_options->__set_convert_kudu_utc_timestamps(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::DISABLE_KUDU_LOCAL_TIMESTAMP_BLOOM_FILTER: {
+        query_options->__set_disable_kudu_local_timestamp_bloom_filter(IsTrue(value));
+        break;
+      }
       case TImpalaQueryOptions::ENABLE_OUTER_JOIN_TO_INNER_TRANSFORMATION: {
         query_options->__set_enable_outer_join_to_inner_transformation(IsTrue(value));
         break;
@@ -1146,7 +1205,100 @@ Status impala::SetQueryOption(const string& key, const string& value,
         query_options->__set_mem_limit_coordinators(mem_spec_val.value);
         break;
       }
+      case TImpalaQueryOptions::ICEBERG_PREDICATE_PUSHDOWN_SUBSETTING: {
+        query_options->__set_iceberg_predicate_pushdown_subsetting(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::HDFS_SCANNER_NON_RESERVED_BYTES: {
+        MemSpec mem_spec_val{};
+        RETURN_IF_ERROR(QueryOptionParser::Parse<MemSpec>(option, value, &mem_spec_val));
+        query_options->__set_hdfs_scanner_non_reserved_bytes(mem_spec_val.value);
+        break;
+      }
+      case TImpalaQueryOptions::CODEGEN_OPT_LEVEL: {
+        TCodeGenOptLevel::type enum_type;
+        RETURN_IF_ERROR(GetThriftEnum(
+            value, "CodeGen Opt Level", _TCodeGenOptLevel_VALUES_TO_NAMES, &enum_type));
+        query_options->__set_codegen_opt_level(enum_type);
+        break;
+      }
+      case TImpalaQueryOptions::KUDU_TABLE_RESERVE_SECONDS: {
+        int32_t int32_t_val = 0;
+        RETURN_IF_ERROR(QueryOptionParser::ParseAndCheckNonNegative<int32_t>(
+            option, value, &int32_t_val));
+        query_options->__set_kudu_table_reserve_seconds(int32_t_val);
+        break;
+      }
+      case TImpalaQueryOptions::RUNTIME_FILTER_CARDINALITY_REDUCTION_SCALE: {
+        double double_val = 0.0;
+        RETURN_IF_ERROR(QueryOptionParser::ParseAndCheckInclusiveRange<double>(
+            option, value, 0.0, 1.0, &double_val));
+        query_options->__set_runtime_filter_cardinality_reduction_scale(double_val);
+        break;
+      }
+      case TImpalaQueryOptions::MAX_NUM_FILTERS_AGGREGATED_PER_HOST: {
+        int32_t int32_t_val = 0;
+        RETURN_IF_ERROR(QueryOptionParser::ParseAndCheckInclusiveLowerBound<int32_t>(
+            option, value, -1, &int32_t_val));
+        query_options->__set_max_num_filters_aggregated_per_host(int32_t_val);
+        break;
+      }
+      case TImpalaQueryOptions::QUERY_CPU_COUNT_DIVISOR: {
+        double double_val = 0.0f;
+        RETURN_IF_ERROR(QueryOptionParser::ParseAndCheckExclusiveLowerBound<double>(
+            option, value, 0.0, &double_val));
+        query_options->__set_query_cpu_count_divisor(double_val);
+        break;
+      }
+      case TImpalaQueryOptions::ENABLE_TUPLE_CACHE: {
+        bool enable_tuple_cache = IsTrue(value);
+        if (enable_tuple_cache && !FLAGS_allow_tuple_caching) {
+          return Status(
+              "Tuple caching is disabled, so enable_tuple_cache cannot be set to true.");
+        }
+        query_options->__set_enable_tuple_cache(enable_tuple_cache);
+        break;
+      }
+      case TImpalaQueryOptions::ICEBERG_DISABLE_COUNT_STAR_OPTIMIZATION: {
+        query_options->__set_iceberg_disable_count_star_optimization(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::RUNTIME_FILTER_IDS_TO_SKIP: {
+        std::set<int32_t> filter_ids;
+        // This does quote handling similar as ENABLED_RUNTIME_FILTER_TYPES option.
+        const string filter_value = std::regex_replace(value, std::regex("^\"|\"$"), "");
+        vector<string> str_ids;
+        split(str_ids, filter_value, is_any_of(","), token_compress_on);
+        TrimAndRemoveEmptyString(str_ids);
+        for (const auto& t : str_ids) {
+          try {
+            int32_t filter_id = std::stoi(t);
+            filter_ids.insert(filter_id);
+          } catch (std::exception&) {
+            return Status::Expected(
+                "RUNTIME_FILTER_IDS_TO_SKIP is not a valid comma separated integers.");
+          }
+        }
+        query_options->__set_runtime_filter_ids_to_skip(filter_ids);
+        break;
+      }
+      case TImpalaQueryOptions::SLOT_COUNT_STRATEGY: {
+        TSlotCountStrategy::type enum_type;
+        RETURN_IF_ERROR(GetThriftEnum(value, "Slot count strategy",
+            _TSlotCountStrategy_VALUES_TO_NAMES, &enum_type));
+        query_options->__set_slot_count_strategy(enum_type);
+        break;
+      }
+      case TImpalaQueryOptions::CLEAN_DBCP_DS_CACHE: {
+        query_options->__set_clean_dbcp_ds_cache(IsTrue(value));
+        break;
+      }
+      case TImpalaQueryOptions::USE_NULL_SLOTS_CACHE: {
+        query_options->__set_use_null_slots_cache(IsTrue(value));
+        break;
+      }
       default:
+        string key = to_string(option);
         if (IsRemovedQueryOption(key)) {
           LOG(WARNING) << "Ignoring attempt to set removed query option '" << key << "'";
           return Status::OK();
@@ -1169,7 +1321,20 @@ Status impala::ParseQueryOptions(const string& options, TQueryOptions* query_opt
     QueryOptionsMask* set_query_options_mask) {
   if (options.length() == 0) return Status::OK();
   vector<string> kv_pairs;
-  split(kv_pairs, options, is_any_of(","), token_compress_on);
+  int double_quote_ct = 0;
+  int begin = 0;
+  int end = 0;
+  while (end < options.length()) {
+    if (options.at(end) == '"') {
+      double_quote_ct = (double_quote_ct + 1) % 2;
+    } else if (options.at(end) == ',' && double_quote_ct == 0) {
+      // Found comma that is not within two double quotes. This is an option separator.
+      if (begin < end) kv_pairs.push_back(options.substr(begin, end - begin));
+      begin = end + 1;
+    }
+    end++;
+  }
+  if (begin < end) kv_pairs.push_back(options.substr(begin, end - begin));
   // Construct an error status which is used to aggregate errors encountered during
   // parsing. It is only returned if the number of error details is greater than 0.
   Status errorStatus = Status::Expected("Errors parsing query options");
