@@ -73,10 +73,12 @@ class StatestoreSubscriber {
   ///   heartbeat_address - the local address on which the heartbeat service which
   ///                       communicates with the statestore should be started.
   ///   statestore_address - the address of the statestore to register with
+  ///   statestore2_address - the address of the second statestore to register with
   ///   subscriber_type - subscriber type
   StatestoreSubscriber(const std::string& subscriber_id,
       const TNetworkAddress& heartbeat_address,
       const TNetworkAddress& statestore_address,
+      const TNetworkAddress& statestore2_address,
       MetricGroup* metrics,
       TStatestoreSubscriberType::type subscriber_type);
 
@@ -166,10 +168,10 @@ class StatestoreSubscriber {
 
   /// Returns true if the statestore has recovered and the configurable post-recovery
   /// grace period has not yet elapsed.
-  bool IsInPostRecoveryGracePeriod() const;
+  bool IsInPostRecoveryGracePeriod();
 
   /// Returns true if the registration with statestore is completed.
-  bool IsRegistered() const;
+  bool IsRegistered();
 
  private:
   /// Unique, but opaque, identifier for this subscriber.
@@ -247,11 +249,16 @@ class StatestoreSubscriber {
   /// tracks variety of bookkeeping information.
   class StatestoreStub {
    public:
-    StatestoreStub(StatestoreSubscriber* subscriber,
+    StatestoreStub(StatestoreSubscriber* subscriber, bool first_statestore,
         const TNetworkAddress& statestore_address, MetricGroup* metrics);
 
+    /// Start to register with the statestore server.
     /// Returns OK unless some error occurred, like a failure to connect.
-    Status Start();
+    /// Note: If the statestore server is not ready, the registration will fail and the
+    /// subscriber enter recovery mode. In recovery mode, the class-wide lock_ is taken
+    /// to ensure mutual exclusion with any operations in flight. To avoid accessing
+    /// member variable is_active_, return its value in the given output parameter.
+    Status Start(bool* startstore_is_active = nullptr);
 
     /// Adds a topic to the set of topics that updates will be received
     /// for. When a topic update is received, the supplied UpdateCallback
@@ -302,7 +309,7 @@ class StatestoreSubscriber {
     /// Called when the catalogd has been updated.
     void UpdateCatalogd(const TCatalogRegistration& catalogd_registration,
         const RegistrationId& registration_id, int64_t active_catalogd_version,
-        bool* update_skipped);
+        bool statestore_failover, bool* update_skipped);
 
     /// Run in a separate thread. In a loop, check failure_detector_ to see if the
     /// statestore is still sending heartbeat messages. If not, enter 'recovery mode'
@@ -334,6 +341,12 @@ class StatestoreSubscriber {
       return MonotonicMillis() - last_registration_ms_.Load();
     }
 
+    int64_t MilliSecondsSinceLastFailover() const {
+      int64_t time_ms = MonotonicMillis() - last_failover_time_.Load();
+      DCHECK_GE(time_ms, 0);
+      return time_ms;
+    }
+
     bool IsInPostRecoveryGracePeriod() const;
 
     /// Check if the subscriber is interesting to receive the notification of catalogd
@@ -343,6 +356,25 @@ class StatestoreSubscriber {
     /// Returns true if the registration with statestore is completed.
     bool IsRegistered();
 
+    /// Set the active state of the registered statestore instance.
+    void SetStatestoreActive(bool is_active, int64_t active_statestored_version,
+        bool has_failover = false);
+
+    /// Return the version of active statestore.
+    int64_t GetActiveVersion(bool* is_active);
+
+    /// Return registration-Id and statestore-Id.
+    void GetRegistrationIdAndStatestoreId(
+        RegistrationId* registration_id, TUniqueId* statestore_id);
+
+    /// Increase count for receiving UpdateStatestoredRole RPC.
+    void IncCountForUpdateStatestoredRoleRPC();
+
+    /// Get connection state with the registered statestore instance.
+    TStatestoreConnState::type GetStatestoreConnState();
+
+    std::string GetAddress() const;
+
    private:
     /// Pointer to parent StatestoreSubscriber object
     StatestoreSubscriber* subscriber_;
@@ -350,6 +382,19 @@ class StatestoreSubscriber {
     /// Address of the statestore
     TNetworkAddress statestore_address_;
 
+    /// True if the registered statestore instance is active.
+    bool is_active_ = false;
+
+    /// The version of active statestored.
+    int64_t active_statestored_version_ = 0;
+
+    /// Monotonic timestamp of the last failover.
+    AtomicInt64 last_failover_time_{0};
+
+    /// Protects is_active_ and active_statestored_version_. Must be taken after lock_
+    /// if both are to be taken together.
+    std::mutex active_lock_;
+
     /// Object-wide lock that protects the below members. Must be held exclusively when
     /// modifying the members, except when modifying TopicRegistrations - see
     /// TopicRegistration::update_lock for details of locking there. Held in shared mode
@@ -399,6 +444,13 @@ class StatestoreSubscriber {
     /// Metric to count the total number of UpdateCatalogd RPCs received by subscriber.
     IntCounter* update_catalogd_rpc_metric_;
 
+    /// Metric that tracks if the statestored is active when statestored HA is enabled.
+    BooleanProperty* active_status_metric_;
+
+    /// Metric to count the total number of UpdateStatestoredRole RPCs received by
+    /// subscriber.
+    IntCounter* update_statestored_role_rpc_metric_;
+
     /// Metric to count the total number of re-registration attempt.
     IntCounter* re_registr_attempt_metric_;
 
@@ -446,8 +498,22 @@ class StatestoreSubscriber {
     AtomicInt64 last_registration_ms_{0};
   };
 
-  // statestore instances
+  /// Set to true if statestored HA is enabled.
+  bool enable_statestored_ha_ = false;
+
+  /// Pointers of two StatestoreStub objects
   StatestoreStub* statestore_ = nullptr;
+  StatestoreStub* statestore2_ = nullptr;
+
+  /// Lock to protect statestore HA related variables: active_statestore_,
+  /// standby_statestore_
+  std::mutex statestore_ha_lock_;
+
+  /// Pointers for active and standby StatestoreStub objects. Subscriber find active
+  /// statestore during registration, or get notification when statestore service
+  /// fail over to standby statestore.
+  StatestoreStub* active_statestore_ = nullptr;
+  StatestoreStub* standby_statestore_ = nullptr;
 
   /// Called when the statestore sends a topic update.
   Status UpdateState(const TopicDeltaMap& incoming_topic_deltas,
@@ -456,13 +522,25 @@ class StatestoreSubscriber {
 
   /// Called when the statestore sends a heartbeat message.
   void Heartbeat(const RegistrationId& registration_id,
-      const TUniqueId& statestore_id);
+      const TUniqueId& statestore_id, bool request_active_conn_state,
+      TStatestoreConnState::type* active_statestore_conn_state);
 
-  /// Called when the catalogd has been updated.
+  /// Called when the active catalogd has been updated.
   void UpdateCatalogd(const TCatalogRegistration& catalogd_registration,
       const RegistrationId& registration_id, const TUniqueId& statestore_id,
       int64_t active_catalogd_version, bool* update_skipped);
 
+  /// Called when the active statestore has been updated.
+  void UpdateStatestoredRole(bool is_active,
+      const RegistrationId& registration_id, const TUniqueId& statestore_id,
+      int64_t active_statestored_version, bool update_active_catalogd,
+      const TCatalogRegistration* catalogd_registration, int64_t active_catalogd_version,
+      bool* update_skipped);
+
+  /// Return the pointer of the active/standby StatestoreStub.
+  StatestoreStub* GetActiveStatestore();
+  StatestoreStub* GetStandbyStatestore();
+
   /// Subscriber thrift implementation, needs to access UpdateState
   friend class StatestoreSubscriberThriftIf;
 };
