@@ -31,6 +31,7 @@
 
 #include "common/atomic.h"
 #include "common/status.h"
+#include "gen-cpp/StatestoreHaService.h"
 #include "gen-cpp/StatestoreService.h"
 #include "gen-cpp/StatestoreService_types.h"
 #include "gen-cpp/StatestoreSubscriber.h"
@@ -39,6 +40,7 @@
 #include "runtime/client-cache.h"
 #include "statestore/failure-detector.h"
 #include "statestore/statestore-catalogd-mgr.h"
+#include "statestore/statestore-service-client-wrapper.h"
 #include "statestore/statestore-subscriber-client-wrapper.h"
 #include "util/aligned-new.h"
 #include "util/condition-variable.h"
@@ -53,6 +55,7 @@ namespace impala {
 class Status;
 
 typedef ClientCache<StatestoreSubscriberClientWrapper> StatestoreSubscriberClientCache;
+typedef ClientCache<StatestoreHaServiceClientWrapper> StatestoreHaClientCache;
 
 std::string SubscriberTypeToString(TStatestoreSubscriberType::type t);
 
@@ -115,6 +118,42 @@ std::string SubscriberTypeToString(TStatestoreSubscriberType::type t);
 /// versions. This ensure all of components in a cluster could communicate with compatible
 /// APIs and message format in the wire.
 ///
+/// Statestore HA:
+/// To support statestore HA, we add the preemptive behavior for statestored. When HA is
+/// enabled, the preemptive behavior allows the statestored with the higher priority to
+/// become active and the paired statestored becomes standby. The active statestored acts
+/// as the owner of the Impala cluster and provides statestore service for the cluster
+/// members.
+/// By default, preemption is disabled on the statestored which is running as single
+/// statestore instance in an Impala cluster. For deployment with statestore HA enabled,
+/// the preemption must be enabled with starting flag "enable_statestored_ha" on two
+/// statestoreds in the HA pair and all the subscribers.
+/// To avoid introduce external dependency like Apache ZooKeeper, standby statestored is
+/// used to monitor the health of active statestored. The basic idea is to have two
+/// statestore instances as HA pair in Active-Passive mode. The both primary and standby
+/// statestored send heartbeats to all subscribers. Subscribers send heartbeat response
+/// to standby statestored with their connection state with active statestored so that
+/// standby statestored could monitor the connection state between active statestore and
+/// the subscribers. Active statestored also sends heartbeats to standby statestored.
+/// Automatic fail-over will be triggered if standby statestored does not receive
+/// heartbeats from active statestored and the majority of the subscribers do not receive
+/// heartbeats from active statestored.
+/// In the scenario where standby statestored is still receiving heartbeats from active
+/// statestored, but majority of subscribers are unable to reach the active statestored,
+/// warning messages will be logged on standby statestored.
+/// The statestored in an Active-Passive HA pair can be assigned an instance priority
+/// value to indicate a preference for which statestored should assume the active role.
+/// The statestore ID which is generated for each statestored can be used as instance
+/// priority value. The lower numerical value in statestore ID corresponds to a higher
+/// priority. The statestored with the higher priority is designated as active, the other
+/// statestored is designated as standby after initial HA handshake. Only the active
+/// statestored propagates the IMPALA_CATALOG_TOPIC and elected active catalogd to the
+/// cluster members.
+/// To make a specific statestored in the HA pair as active instance, the statestored must
+/// be started with starting flag "statestore_force_active" so that the statestored will
+/// be assigned with active role in HA handshake. This allows administrator to manually
+/// perform statestore service fail over.
+///
 /// +================+
 /// | Implementation |
 /// +================+
@@ -122,10 +161,11 @@ std::string SubscriberTypeToString(TStatestoreSubscriberType::type t);
 /// Locking:
 /// --------
 /// The lock acquisition order is:
-/// 1. 'subscribers_lock_'
-/// 2. 'topics_map_lock_'
-/// 3. Subscriber::transient_entry_lock_
-/// 4. Topic::lock_ (terminal)
+/// 1. 'ha_lock_'
+/// 2. 'subscribers_lock_'
+/// 3. 'topics_map_lock_'
+/// 4. Subscriber::transient_entry_lock_
+/// 5. Topic::lock_ (terminal)
 class Statestore : public CacheLineAligned {
  public:
   /// A TopicId uniquely identifies a single topic
@@ -185,6 +225,12 @@ class Statestore : public CacheLineAligned {
     return thrift_iface_;
   }
 
+  /// Returns the Thrift API interface that proxies requests onto the local Statestore
+  /// HA Service.
+  const std::shared_ptr<StatestoreHaServiceIf>& ha_thrift_iface() const {
+    return ha_thrift_iface_;
+  }
+
   /// Names of prioritized topics that are handled in a separate threadpool. The topic
   /// names are hardcoded here for expediency. Ideally we would have a more generic
   /// interface for specifying prioritized topics, but for now we only have a small
@@ -200,12 +246,52 @@ class Statestore : public CacheLineAligned {
   /// after it stops responding to heartbeats.
   static int64_t FailedExecutorDetectionTimeMs();
 
+  /// Return statestore-ID.
   const TUniqueId& GetStateStoreId() const { return statestore_id_; }
 
+  /// Return protocol version of statestore service.
   StatestoreServiceVersion::type GetProtocolVersion() const {
     return protocol_version_;
   }
 
+  /// Disable network if the input parameter disable_network is set as true.
+  /// statestored will not send heartbeat and HA handshake messages when network is
+  /// disabled. This is used to simulate network failure for unit-test.
+  void SetStatestoreDebugAction(bool disable_network);
+
+  /// Initialize statestore HA.
+  Status InitStatestoreHa(
+      int32_t state_store_ha_port, const TNetworkAddress& peer_statestore_addr);
+
+  // Return true if this statestore instance is in active role.
+  bool IsActive();
+
+  // Return the version of active status.
+  int64_t GetActiveVersion(bool* is_active);
+
+  /// Send RPC to negotiate the role with peer statestored for Statestore HA.
+  Status SendHaHandshake(TStatestoreHaHandshakeResponse* response);
+
+  /// Called to process HA handshake request from peer statstore
+  Status ReceiveHaHandshakeRequest(const TUniqueId& subscriber_id,
+      const string& src_statestore_address, bool src_force_active,
+      bool* dst_statestore_active);
+
+  /// Called to process HA heartbeat from active statestore
+  void HaHeartbeatRequest(const TUniqueId& dst_statestore_id,
+      const TUniqueId& src_statestore_id);
+
+  /// Sends and monitors the HA heartbeats between two statestore instances.
+  [[noreturn]] void MonitorStatestoredHaHeartbeat();
+
+  /// Send HA heartbeat to peer statestore instance.
+  Status SendHaHeartbeat();
+
+  /// Return the network address of peer statestore instance.
+  std::string GetPeerStatestoreHaAddress() {
+    return TNetworkAddressToString(peer_statestore_ha_addr_);
+  }
+
  private:
   /// A TopicEntry is a single entry in a topic, and logically is a <string, byte string>
   /// pair.
@@ -553,6 +639,9 @@ class Statestore : public CacheLineAligned {
   /// Condition variable for sending the notifications of updating catalogd.
   ConditionVariable update_catalod_cv_;
 
+  /// Condition variable for sending the notifications of updating role of statestored.
+  ConditionVariable update_statestored_cv_;
+
   /// Used to generated unique IDs for each new registration.
   boost::uuids::random_generator subscriber_uuid_generator_;
 
@@ -619,6 +708,9 @@ class Statestore : public CacheLineAligned {
   /// Thread to send notification of updating catalogd.
   std::unique_ptr<Thread> update_catalogd_thread_;
 
+  /// Thread to send notification of updating role of statestored.
+  std::unique_ptr<Thread> update_statestored_thread_;
+
   /// Indicates whether the statestore has been initialized and the service is ready.
   std::atomic_bool service_started_{false};
 
@@ -637,15 +729,33 @@ class Statestore : public CacheLineAligned {
   /// failure.
   boost::scoped_ptr<StatestoreSubscriberClientCache> update_catalogd_client_cache_;
 
+  /// Cache of subscriber clients used for UpdateStatestoredRole() RPCs. Only one client
+  /// per subscriber should be used, but the cache helps with the client lifecycle on
+  /// failure.
+  boost::scoped_ptr<StatestoreSubscriberClientCache> update_statestored_client_cache_;
+
+  /// statestore client cache - only one client is ever used. Initialized in constructor.
+  /// The StatestoreHaClientCache is created with num_retries = 1 and wait_ms = 0.
+  /// Connections are still retried, but the retry mechanism is driven by DoRpcWithRetry.
+  /// Clients should always use DoRpcWithRetry rather than DoRpc to ensure that both RPCs
+  /// and connections are retried.
+  boost::scoped_ptr<StatestoreHaClientCache> ha_client_cache_;
+
   /// Container for the internal statestore service.
   boost::scoped_ptr<ThriftServer> thrift_server_;
 
+  /// Container for the internal statestore HA service.
+  boost::scoped_ptr<ThriftServer> ha_thrift_server_;
+
   /// Pointer to the MetricGroup for this statestore. Not owned.
   MetricGroup* metrics_;
 
-  /// Thrift API implementation which proxies requests onto this Statestore
+  /// Thrift API implementation which proxies requests onto this Statestore Service
   std::shared_ptr<StatestoreServiceIf> thrift_iface_;
 
+  /// Thrift API implementation which proxies requests onto this Statestore HA Service
+  std::shared_ptr<StatestoreHaServiceIf> ha_thrift_iface_;
+
   /// Failure detector for subscribers. If a subscriber misses a configurable number of
   /// consecutive heartbeat messages, it is considered failed and a) its transient topic
   /// entries are removed and b) its entry in the subscriber map is erased. The
@@ -655,6 +765,67 @@ class Statestore : public CacheLineAligned {
   /// carry over to any new registrations of the previous subscriber.
   boost::scoped_ptr<MissedHeartbeatFailureDetector> failure_detector_;
 
+  /// Lock to protect statestored HA related variables:
+  std::mutex ha_lock_;
+
+  /// Network address of peer statestore instance for StatestoreHaService.
+  TNetworkAddress peer_statestore_ha_addr_;
+
+  /// Unique identifier for peer statestore instance.
+  TUniqueId peer_statestore_id_;
+
+  /// Local network address for StatestoreHaService.
+  TNetworkAddress local_statestore_ha_addr_;
+
+  /// True if the statestore instance is active.
+  bool is_active_ = false;
+
+  /// The version of active statestore. It's set as the number of microseconds that have
+  /// passed since the Unix epoch.
+  /// The HA notifications could be received by subscribers out of order due to network
+  /// delays. This variable is sent in HA notifications to guarantee that only latest
+  /// notification takes effect on subscribers. We can not use an incrementing counter
+  /// for this purpose, otherwise the version will be reset when statestored is restarted.
+  int64_t active_version_ = 0;
+
+  /// True if the peer statestore instance has been detected.
+  bool found_peer_ = false;
+
+  /// True if the statestore instance is in recovery mode.
+  bool in_recovery_mode_ = false;
+
+  /// Starting time to enter recovery mode.
+  int64_t recovery_start_time_;
+
+  /// Number of HA heartbeat received in active state.
+  /// Reset this variable whenever `is_active_` is set to true.
+  int num_received_heartbeat_in_active_ = 0;
+
+  /// Disable network if this variable is set as true by statestore service API.
+  /// This is only used for unit-test.
+  AtomicBool disable_network_{false};
+
+  /// Failure detector for active statestore. If the standby statestore misses a
+  /// configurable number of consecutive heartbeat messages, it is considered failed and
+  /// if it does not receive heartbeat responses from more than 50% of subscribers,
+  /// active statestore enter "looking" state .
+  boost::scoped_ptr<MissedHeartbeatFailureDetector> ha_standby_ss_failure_detector_;
+
+  /// Failure detector for standby statestore. It tracks heartbeat messages from the
+  /// active statestore.
+  boost::scoped_ptr<impala::TimeoutFailureDetector> ha_active_ss_failure_detector_;
+
+  /// Record of connection state between active statestore and subscribers.
+  typedef boost::unordered_map<SubscriberId, TStatestoreConnState::type>
+      ActiveConnStateMap;
+  ActiveConnStateMap active_conn_states_;
+
+  /// Number of subscriber which lost connection with active statestored.
+  int failed_conn_state_count_ = 0;
+
+  /// Thread that sends and monitors the HA heartbeats between two statestore instances.
+  std::unique_ptr<Thread> ha_monitoring_thread_;
+
   /// Metric that track the registered, non-failed subscribers.
   IntGauge* num_subscribers_metric_;
   SetMetric<std::string>* subscriber_set_metric_;
@@ -680,6 +851,14 @@ class Statestore : public CacheLineAligned {
   /// Metric to count the total number of failed UpdateCatalogd RPCs sent by statestore.
   IntCounter* failed_update_catalogd_rpc_metric_;
 
+  /// Metric to count the total number of successful UpdateCStatestoredRole RPCs sent by
+  /// statestore.
+  IntCounter* successful_update_statestored_role_rpc_metric_;
+
+  /// Metric to count the total number of failed UpdateStatestoredRole RPCs sent by
+  /// statestore.
+  IntCounter* failed_update_statestored_role_rpc_metric_;
+
   /// Metric to count the total number of requests for clearing topic entries from
   /// catalogd. Catalogd indicates to clear topic entries when it is restarted or its
   /// role has been changed from standby to active.
@@ -688,6 +867,19 @@ class Statestore : public CacheLineAligned {
   /// Metric that tracks the address of active catalogd for catalogd HA
   StringProperty* active_catalogd_address_metric_;
 
+  /// Metric that tracks if the statestored is active when statestored HA is enabled.
+  BooleanProperty* active_status_metric_;
+
+  /// Metric that tracks if the statestore service is started and ready to accept
+  /// connections.
+  BooleanProperty* service_started_metric_;
+
+  /// Metric that tracks if the statestored is in HA recovery mode.
+  BooleanProperty* in_ha_recovery_mode_metric_;
+
+  /// Metric that tracks if the statestored is connected with peer statestored.
+  BooleanProperty* connected_peer_metric_;
+
   /// Utility method to add an update to the given thread pool, and to fail if the thread
   /// pool is already at capacity. Assumes that subscribers_lock_ is held by the caller.
   Status OfferUpdate(const ScheduledSubscriberUpdate& update,
@@ -800,13 +992,23 @@ class Statestore : public CacheLineAligned {
   /// Monitors the notification of updating catalogd.
   [[noreturn]] void MonitorUpdateCatalogd();
 
+  /// Monitors the notification of updating statestored's role.
+  [[noreturn]] void MonitorUpdateStatestoredRole();
+
   /// Send notification of updating catalogd to the coordinators.
   void SendUpdateCatalogdNotification(int64_t* last_active_catalogd_version,
       vector<std::shared_ptr<Subscriber>>& receivers);
 
+  /// Send notification of updating statestored's role to all subscribers.
+  void SendUpdateStatestoredRoleNotification(int64_t* last_active_statestored_version,
+      vector<std::shared_ptr<Subscriber>>& receivers);
+
   /// Raw callback to indicate whether the service is ready.
   void HealthzHandler(const Webserver::WebRequest& req, std::stringstream* data,
       HttpStatusCode* response);
+
+  // Return true if this statestore instance is in recovery mode.
+  bool IsInRecoveryMode();
 };
 
 } // namespace impala
