@@ -61,6 +61,21 @@ struct BigIntVal;
 struct StringVal;
 struct TimestampVal;
 struct DateVal;
+class FunctionContext;
+
+/// Built-in functions exposed to UDFs
+struct BuiltInFunctions {
+ public:
+  /// Sends a prompt to the default endpoint and uses the default model, default
+  /// jceks api-key secret and default params.
+  StringVal (*ai_generate_text_default)(
+      FunctionContext* context, const StringVal& prompt);
+  /// Sends a prompt to the input AI endpoint using the input model, jceks api_key secret
+  /// and optional params.
+  StringVal (*ai_generate_text)(FunctionContext* context, const StringVal& endpoint,
+      const StringVal& prompt, const StringVal& model,
+      const StringVal& api_key_jceks_secret, const StringVal& params);
+};
 
 /// A FunctionContext is passed to every UDF/UDA and is the interface for the UDF to the
 /// rest of the system. It contains APIs to examine the system state, report errors and
@@ -246,6 +261,8 @@ class FunctionContext {
   /// use this. This is used internally.
   impala::FunctionContextImpl* impl() const { return impl_; }
 
+  const BuiltInFunctions* Functions() const;
+
   ~FunctionContext();
 
  private:
@@ -691,6 +708,10 @@ struct StringVal : public AnyVal {
     return ptr == other.ptr || memcmp(ptr, other.ptr, len) == 0;
   }
   bool operator!=(const StringVal& other) const { return !(*this == other); }
+
+ private:
+  static void AllocateStringValWithLenCheck(FunctionContext* ctx, uint64_t str_len,
+      StringVal* res);
 };
 
 struct DecimalVal : public impala_udf::AnyVal {
