@@ -22,6 +22,7 @@
 #include "gutil/strings/substitute.h"
 #include "kudu/util/flag_tags.h"
 #include "rpc/jni-thrift-util.h"
+#include "rpc/thrift-util.h"
 #include "util/backend-gflag-util.h"
 #include "util/logging-support.h"
 #include "util/os-util.h"
@@ -34,6 +35,7 @@ DECLARE_bool(enable_stats_extrapolation);
 DECLARE_bool(use_local_catalog);
 DECLARE_int32(local_catalog_cache_expiration_s);
 DECLARE_int32(local_catalog_cache_mb);
+DECLARE_int32(local_catalog_cache_concurrency_level);
 DECLARE_int32(non_impala_java_vlog);
 DECLARE_int32(num_metadata_loading_threads);
 DECLARE_int32(max_hdfs_partitions_parallel_load);
@@ -104,12 +106,29 @@ DECLARE_bool(enable_sync_to_latest_event_on_ddls);
 DECLARE_bool(pull_table_types_and_comments);
 DECLARE_bool(enable_reload_events);
 DECLARE_string(geospatial_library);
-DECLARE_int32(thrift_rpc_max_message_size);
 DECLARE_string(file_metadata_reload_properties);
 DECLARE_string(java_weigher);
 DECLARE_int32(iceberg_reload_new_files_threshold);
 DECLARE_bool(enable_skipping_older_events);
 DECLARE_bool(enable_json_scanner);
+DECLARE_bool(iceberg_allow_datafiles_in_table_location_only);
+DECLARE_int32(catalog_operation_log_size);
+DECLARE_string(hostname);
+DECLARE_bool(allow_catalog_cache_op_from_masked_users);
+DECLARE_int32(topic_update_log_gc_frequency);
+DECLARE_string(debug_actions);
+DECLARE_bool(invalidate_metadata_on_event_processing_failure);
+DECLARE_bool(invalidate_global_metadata_on_event_processing_failure);
+DECLARE_string(inject_process_event_failure_event_types);
+DECLARE_double(inject_process_event_failure_ratio);
+DECLARE_bool(enable_workload_mgmt);
+DECLARE_string(query_log_table_name);
+DECLARE_string(default_skipped_hms_event_types);
+DECLARE_string(common_hms_event_types);
+DECLARE_int32(dbcp_max_conn_pool_size);
+DECLARE_int32(dbcp_max_wait_millis_for_conn);
+DECLARE_int32(dbcp_data_source_idle_timeout_s);
+DECLARE_bool(enable_catalogd_ha);
 
 // HS2 SAML2.0 configuration
 // Defined here because TAG_FLAG caused issues in global-flags.cc
@@ -205,6 +224,7 @@ DEFINE_string(ignored_dir_prefix_list, ".,_tmp.,_spark_metadata",
     " skip in loading file metadata.");
 
 DEFINE_double_hidden(query_cpu_count_divisor, 1.0,
+    "(Deprecated) this is now deprecated in favor of query option with the same name. "
     "(Advance) Divide the CPU requirement of a query to fit the total available CPU in "
     "the executor group. For example, setting value 2 will fit the query with CPU "
     "requirement 2X to an executor group with total available CPU X. Note that setting "
@@ -249,13 +269,17 @@ DEFINE_double_hidden(max_filter_error_rate_from_full_scan, 0.9,
     "query option or max_filter_error_rate backend flag. Setting value less than 0 "
     "will disable this runtime filter reduction feature.");
 
+DEFINE_double_hidden(query_cpu_root_factor, 1.5,
+    "(Advance) The Nth root value to control sublinear scale down of unbounded "
+    "cpu requirement for executor group set selection.");
+
 using strings::Substitute;
 
 namespace impala {
 
 // Flag validation
 // ------------------------------------------------------------
-static bool ValidateCpuCountDivisor(const char* flagname, double value) {
+static bool ValidatePositiveDouble(const char* flagname, double value) {
   if (0.0 < value) {
     return true;
   }
@@ -273,8 +297,9 @@ static bool ValidateMinProcessingPerThread(const char* flagname, int64_t value)
   return false;
 }
 
-DEFINE_validator(query_cpu_count_divisor, &ValidateCpuCountDivisor);
+DEFINE_validator(query_cpu_count_divisor, &ValidatePositiveDouble);
 DEFINE_validator(min_processing_per_thread, &ValidateMinProcessingPerThread);
+DEFINE_validator(query_cpu_root_factor, &ValidatePositiveDouble);
 
 Status GetConfigFromCommand(const string& flag_cmd, string& result) {
   result.clear();
@@ -291,6 +316,8 @@ Status PopulateThriftBackendGflags(TBackendGflags& cfg) {
   cfg.__set_local_catalog_cache_mb(FLAGS_local_catalog_cache_mb);
   cfg.__set_local_catalog_cache_expiration_s(
     FLAGS_local_catalog_cache_expiration_s);
+  cfg.__set_local_catalog_cache_concurrency_level(
+    FLAGS_local_catalog_cache_concurrency_level);
   cfg.__set_server_name(FLAGS_server_name);
   cfg.__set_kudu_master_hosts(FLAGS_kudu_master_hosts);
   cfg.__set_enable_kudu_impala_hms_check(FLAGS_enable_kudu_impala_hms_check);
@@ -429,14 +456,43 @@ Status PopulateThriftBackendGflags(TBackendGflags& cfg) {
   cfg.__set_skip_resource_checking_on_last_executor_group_set(
       FLAGS_skip_resource_checking_on_last_executor_group_set);
   cfg.__set_file_metadata_reload_properties(FLAGS_file_metadata_reload_properties);
-  cfg.__set_thrift_rpc_max_message_size(FLAGS_thrift_rpc_max_message_size);
+  cfg.__set_thrift_rpc_max_message_size(ThriftInternalRpcMaxMessageSize());
   cfg.__set_scan_range_cost_factor(FLAGS_scan_range_cost_factor);
   cfg.__set_use_jamm_weigher(FLAGS_java_weigher == "jamm");
   cfg.__set_iceberg_reload_new_files_threshold(FLAGS_iceberg_reload_new_files_threshold);
   cfg.__set_enable_skipping_older_events(FLAGS_enable_skipping_older_events);
   cfg.__set_enable_json_scanner(FLAGS_enable_json_scanner);
+  cfg.__set_iceberg_allow_datafiles_in_table_location_only(
+      FLAGS_iceberg_allow_datafiles_in_table_location_only);
   cfg.__set_max_filter_error_rate_from_full_scan(
       FLAGS_max_filter_error_rate_from_full_scan);
+  cfg.__set_catalog_operation_log_size(FLAGS_catalog_operation_log_size);
+  cfg.__set_hostname(FLAGS_hostname);
+  cfg.__set_allow_catalog_cache_op_from_masked_users(
+      FLAGS_allow_catalog_cache_op_from_masked_users);
+  cfg.__set_topic_update_log_gc_frequency(FLAGS_topic_update_log_gc_frequency);
+  cfg.__set_debug_actions(FLAGS_debug_actions);
+  cfg.__set_invalidate_metadata_on_event_processing_failure(
+      FLAGS_invalidate_metadata_on_event_processing_failure);
+  cfg.__set_invalidate_global_metadata_on_event_processing_failure(
+      FLAGS_invalidate_global_metadata_on_event_processing_failure);
+  cfg.__set_inject_process_event_failure_event_types(
+      FLAGS_inject_process_event_failure_event_types);
+  cfg.__set_inject_process_event_failure_ratio(FLAGS_inject_process_event_failure_ratio);
+  cfg.__set_enable_workload_mgmt(FLAGS_enable_workload_mgmt);
+  cfg.__set_query_log_table_name(FLAGS_query_log_table_name);
+  cfg.__set_query_cpu_root_factor(FLAGS_query_cpu_root_factor);
+  cfg.__set_default_skipped_hms_event_types(FLAGS_default_skipped_hms_event_types);
+  cfg.__set_common_hms_event_types(FLAGS_common_hms_event_types);
+  cfg.__set_dbcp_max_conn_pool_size(FLAGS_dbcp_max_conn_pool_size);
+  cfg.__set_dbcp_max_wait_millis_for_conn(FLAGS_dbcp_max_wait_millis_for_conn);
+  cfg.__set_dbcp_data_source_idle_timeout(FLAGS_dbcp_data_source_idle_timeout_s);
+#ifdef NDEBUG
+  cfg.__set_is_release_build(true);
+#else
+  cfg.__set_is_release_build(false);
+#endif
+  cfg.__set_enable_catalogd_ha(FLAGS_enable_catalogd_ha);
   return Status::OK();
 }
 
