@@ -16,18 +16,9 @@
 // under the License.
 
 #include "util/bloom-filter.h"
-#include "util/hash-util.h"
 
 using namespace impala;
 
-void BloomFilter::InsertNoAvx2(const uint32_t hash) noexcept {
-  always_false_ = false;
-  const uint32_t bucket_idx = HashUtil::Rehash32to32(hash) & directory_mask_;
-  BucketInsert(bucket_idx, hash);
-}
-
-void BloomFilter::InsertAvx2(const uint32_t hash) noexcept {
-  always_false_ = false;
-  const uint32_t bucket_idx = HashUtil::Rehash32to32(hash) & directory_mask_;
-  BucketInsertAVX2(bucket_idx, hash);
+void IR_ALWAYS_INLINE BloomFilter::IrInsert(const uint32_t hash) noexcept {
+  block_bloom_filter_.Insert(hash);
 }
