@@ -166,7 +166,21 @@ const char* ImpaladMetricKeys::HEDGED_READ_OPS =
     "impala-server.hedged-read-ops";
 const char* ImpaladMetricKeys::HEDGED_READ_OPS_WIN =
     "impala-server.hedged-read-ops-win";
+const char* ImpaladMetricKeys::COMPLETED_QUERIES_QUEUED =
+    "impala-server.completed-queries.queued";
+const char* ImpaladMetricKeys::COMPLETED_QUERIES_WRITTEN =
+    "impala-server.completed-queries.written";
+const char* ImpaladMetricKeys::COMPLETED_QUERIES_FAIL =
+    "impala-server.completed-queries.failure";
+const char* ImpaladMetricKeys::COMPLETED_QUERIES_SCHEDULED_WRITES =
+    "impala-server.completed-queries.scheduled-writes";
+const char* ImpaladMetricKeys::COMPLETED_QUERIES_MAX_RECORDS_WRITES =
+    "impala-server.completed-queries.max-records-writes";
+const char* ImpaladMetricKeys::COMPLETED_QUERIES_WRITE_DURATIONS =
+    "impala-server.completed-queries.write-durations";
 const char* ImpaladMetricKeys::DEBUG_ACTION_NUM_FAIL = "impala.debug_action.fail";
+const char* ImpaladMetricKeys::QUERY_LOG_EST_TOTAL_BYTES =
+    "impala-server.query-log-est-total-bytes";
 
 // These are created by impala-server during startup.
 // =======
@@ -214,6 +228,10 @@ IntCounter* ImpaladMetrics::CATALOG_CACHE_MISS_COUNT = nullptr;
 IntCounter* ImpaladMetrics::CATALOG_CACHE_REQUEST_COUNT = nullptr;
 IntCounter* ImpaladMetrics::CATALOG_CACHE_TOTAL_LOAD_TIME = nullptr;
 IntCounter* ImpaladMetrics::DEBUG_ACTION_NUM_FAIL = nullptr;
+IntCounter* ImpaladMetrics::COMPLETED_QUERIES_WRITTEN = nullptr;
+IntCounter* ImpaladMetrics::COMPLETED_QUERIES_FAIL = nullptr;
+IntCounter* ImpaladMetrics::COMPLETED_QUERIES_SCHEDULED_WRITES = nullptr;
+IntCounter* ImpaladMetrics::COMPLETED_QUERIES_MAX_RECORDS_WRITES = nullptr;
 
 // Gauges
 IntGauge* ImpaladMetrics::CATALOG_NUM_DBS = nullptr;
@@ -237,12 +255,14 @@ IntGauge* ImpaladMetrics::NUM_FILES_OPEN_FOR_INSERT = nullptr;
 IntGauge* ImpaladMetrics::NUM_QUERIES_REGISTERED = nullptr;
 IntGauge* ImpaladMetrics::RESULTSET_CACHE_TOTAL_NUM_ROWS = nullptr;
 IntGauge* ImpaladMetrics::RESULTSET_CACHE_TOTAL_BYTES = nullptr;
+IntGauge* ImpaladMetrics::QUERY_LOG_EST_TOTAL_BYTES = nullptr;
 DoubleGauge* ImpaladMetrics::CATALOG_CACHE_AVG_LOAD_TIME = nullptr;
 DoubleGauge* ImpaladMetrics::CATALOG_CACHE_HIT_RATE = nullptr;
 DoubleGauge* ImpaladMetrics::CATALOG_CACHE_LOAD_EXCEPTION_RATE = nullptr;
 DoubleGauge* ImpaladMetrics::CATALOG_CACHE_MISS_RATE = nullptr;
 DoubleGauge* ImpaladMetrics::CATALOG_CACHE_ENTRY_MEDIAN_SIZE = nullptr;
 DoubleGauge* ImpaladMetrics::CATALOG_CACHE_ENTRY_99TH_SIZE = nullptr;
+IntGauge* ImpaladMetrics::COMPLETED_QUERIES_QUEUED = nullptr;
 
 // Properties
 BooleanProperty* ImpaladMetrics::CATALOG_READY = nullptr;
@@ -254,6 +274,7 @@ StringProperty* ImpaladMetrics::ACTIVE_CATALOGD_ADDRESS = nullptr;
 // Histograms
 HistogramMetric* ImpaladMetrics::QUERY_DURATIONS = nullptr;
 HistogramMetric* ImpaladMetrics::DDL_DURATIONS = nullptr;
+HistogramMetric* ImpaladMetrics::COMPLETED_QUERIES_WRITE_DURATIONS = nullptr;
 
 // Other
 StatsMetric<uint64_t, StatsType::MEAN>*
@@ -342,6 +363,8 @@ void ImpaladMetrics::CreateMetrics(MetricGroup* m) {
       ImpaladMetricKeys::RESULTSET_CACHE_TOTAL_NUM_ROWS, 0);
   RESULTSET_CACHE_TOTAL_BYTES = m->AddGauge(
       ImpaladMetricKeys::RESULTSET_CACHE_TOTAL_BYTES, 0);
+  QUERY_LOG_EST_TOTAL_BYTES =
+      m->AddGauge(ImpaladMetricKeys::QUERY_LOG_EST_TOTAL_BYTES, 0);
 
   // Initialize scan node metrics
   NUM_RANGES_PROCESSED = m->AddCounter(
@@ -353,6 +376,16 @@ void ImpaladMetrics::CreateMetrics(MetricGroup* m) {
   NUM_FILES_OPEN_FOR_INSERT = m->AddGauge(
       ImpaladMetricKeys::NUM_FILES_OPEN_FOR_INSERT, 0);
 
+  // Initialize completed queries metrics.
+  COMPLETED_QUERIES_QUEUED = m->AddGauge(ImpaladMetricKeys::COMPLETED_QUERIES_QUEUED, 0);
+  COMPLETED_QUERIES_WRITTEN = m->AddCounter(
+      ImpaladMetricKeys::COMPLETED_QUERIES_WRITTEN, 0);
+  COMPLETED_QUERIES_FAIL = m->AddCounter(ImpaladMetricKeys::COMPLETED_QUERIES_FAIL, 0);
+  COMPLETED_QUERIES_SCHEDULED_WRITES = m->AddCounter(
+      ImpaladMetricKeys::COMPLETED_QUERIES_SCHEDULED_WRITES, 0);
+  COMPLETED_QUERIES_MAX_RECORDS_WRITES = m->AddCounter(
+      ImpaladMetricKeys::COMPLETED_QUERIES_MAX_RECORDS_WRITES, 0);
+
   // Initialize IO mgr metrics
   IO_MGR_METRICS = m->GetOrCreateChildGroup("io-mgr");
   IO_MGR_NUM_OPEN_FILES = IO_MGR_METRICS->AddGauge(
@@ -429,6 +462,9 @@ void ImpaladMetrics::CreateMetrics(MetricGroup* m) {
       MetricDefs::Get(ImpaladMetricKeys::QUERY_DURATIONS), FIVE_HOURS_IN_MS, 3));
   DDL_DURATIONS = m->RegisterMetric(new HistogramMetric(
       MetricDefs::Get(ImpaladMetricKeys::DDL_DURATIONS), FIVE_HOURS_IN_MS, 3));
+  COMPLETED_QUERIES_WRITE_DURATIONS = m->RegisterMetric(new HistogramMetric(
+      MetricDefs::Get(ImpaladMetricKeys::COMPLETED_QUERIES_WRITE_DURATIONS),
+      FIVE_HOURS_IN_MS, 3));
 
   // Initialize Hedged read metrics
   HEDGED_READ_OPS = m->AddCounter(ImpaladMetricKeys::HEDGED_READ_OPS, 0);
