@@ -261,6 +261,32 @@ class ImpaladMetricKeys {
   /// --debug_actions is set.
   static const char* DEBUG_ACTION_NUM_FAIL;
 
+  // Estimated total size of query logs that are currently retained in memory.
+  // Associated metric is modified in ImpalaServer::ArchiveQuery and must hold
+  // ImpalaServer::query_log_lock_ on modification.
+  static const char* QUERY_LOG_EST_TOTAL_BYTES;
+
+  /// The number of completed queries queued up and waiting to be written to the query
+  /// log table.
+  static const char* COMPLETED_QUERIES_QUEUED;
+
+  /// The number of completed queries successfully written to the query log table.
+  static const char* COMPLETED_QUERIES_WRITTEN;
+
+  /// The number of completed queries that failed to be written to the query log
+  /// table.
+  static const char* COMPLETED_QUERIES_FAIL;
+
+  /// Number of writes to the query log table that happened at the regularly scheduled
+  /// time.
+  static const char* COMPLETED_QUERIES_SCHEDULED_WRITES;
+
+  /// Number of writes to the query log table that happened because the max queued
+  /// completed queries records was reached.
+  static const char* COMPLETED_QUERIES_MAX_RECORDS_WRITES;
+
+  /// Time spent writing completed queries to the query log table.
+  static const char* COMPLETED_QUERIES_WRITE_DURATIONS;
 };
 
 /// Global impalad-wide metrics.  This is useful for objects that want to update metrics
@@ -315,6 +341,10 @@ class ImpaladMetrics {
   static IntCounter* CATALOG_CACHE_REQUEST_COUNT;
   static IntCounter* CATALOG_CACHE_TOTAL_LOAD_TIME;
   static IntCounter* DEBUG_ACTION_NUM_FAIL;
+  static IntCounter* COMPLETED_QUERIES_WRITTEN;
+  static IntCounter* COMPLETED_QUERIES_FAIL;
+  static IntCounter* COMPLETED_QUERIES_SCHEDULED_WRITES;
+  static IntCounter* COMPLETED_QUERIES_MAX_RECORDS_WRITES;
 
   // Gauges
   static IntGauge* CATALOG_NUM_DBS;
@@ -344,6 +374,8 @@ class ImpaladMetrics {
   static IntGauge* NUM_QUERIES_REGISTERED;
   static IntGauge* RESULTSET_CACHE_TOTAL_NUM_ROWS;
   static IntGauge* RESULTSET_CACHE_TOTAL_BYTES;
+  static IntGauge* QUERY_LOG_EST_TOTAL_BYTES;
+  static IntGauge* COMPLETED_QUERIES_QUEUED;
 
   // Properties
   static BooleanProperty* CATALOG_READY;
@@ -354,6 +386,7 @@ class ImpaladMetrics {
   // Histograms
   static HistogramMetric* QUERY_DURATIONS;
   static HistogramMetric* DDL_DURATIONS;
+  static HistogramMetric* COMPLETED_QUERIES_WRITE_DURATIONS;
 
   // Other
   static StatsMetric<uint64_t, StatsType::MEAN>* IO_MGR_CACHED_FILE_HANDLES_HIT_RATIO;
