@@ -226,6 +226,8 @@ class JniCall {
 
   Status ObjectToResult(jobject obj, std::string* result) WARN_UNUSED_RESULT;
 
+  Status ObjectToResult(jobject obj, jobject* result) WARN_UNUSED_RESULT;
+
   const jmethodID method_;
   JNIEnv* const env_;
   JniLocalFrame frame_;
@@ -281,6 +283,16 @@ class JniUtil {
   static bool MethodExists(JNIEnv* env, jclass class_ref,
       const char* method_str, const char* method_signature);
 
+  /// Wrapper method around JNI's 'GetMethodID'. Returns the method reference for the
+  /// requested method.
+  static Status GetMethodID(JNIEnv* env, jclass class_ref, const char* method_str,
+      const char* method_signature, jmethodID* method_ref);
+
+  /// Wrapper method around JNI's 'GetStaticMethodID'. Returns the method reference for
+  /// the requested method.
+  static Status GetStaticMethodID(JNIEnv* env, jclass class_ref, const char* method_str,
+      const char* method_signature, jmethodID* method_ref);
+
   /// Returns a global JNI reference to the class specified by class_str into class_ref.
   /// The returned reference must eventually be freed by calling FreeGlobalRef() (or have
   /// the lifetime of the impalad process).
@@ -365,6 +377,10 @@ class JniUtil {
     return JniCall::static_method(cls, method).Call();
   }
 
+  template <typename T, typename R>
+  static Status CallStaticJniMethod(const jclass& cls, const jmethodID& method,
+      const T& arg, R* response) WARN_UNUSED_RESULT;
+
   template <typename T>
   static Status CallJniMethod(const jobject& obj, const jmethodID& method,
       const T& arg) WARN_UNUSED_RESULT;
@@ -377,6 +393,10 @@ class JniUtil {
   static Status CallJniMethod(const jobject& obj, const jmethodID& method,
       R* response) WARN_UNUSED_RESULT;
 
+  template <typename T>
+  static Status CallJniMethod(const jobject& obj, const jmethodID& method,
+      const T& arg, jobject* response) WARN_UNUSED_RESULT;
+
  private:
   // Slow-path for GetJNIEnv, used on the first call by any thread.
   static JNIEnv* GetJNIEnvSlowPath();
@@ -418,6 +438,12 @@ SPECIALIZE_PRIMITIVE_TO_VALUE(float, f);
 SPECIALIZE_PRIMITIVE_TO_VALUE(double, d);
 #undef SPECIALIZE_PRIMITIVE_TO_VALUE
 
+template <typename T, typename R>
+inline Status JniUtil::CallStaticJniMethod(const jclass& cls, const jmethodID& method,
+    const T& arg, R* response) {
+  return JniCall::static_method(cls, method).with_thrift_arg(arg).Call(response);
+}
+
 template <typename T>
 inline Status JniUtil::CallJniMethod(const jobject& obj, const jmethodID& method,
     const T& arg) {
@@ -442,6 +468,12 @@ inline Status JniUtil::CallJniMethod(const jobject& obj, const jmethodID& method
   return JniCall::instance_method(obj, method).Call(response);
 }
 
+template <typename T>
+inline Status JniUtil::CallJniMethod(const jobject& obj, const jmethodID& method,
+    const T& arg, jobject* response) {
+  return JniCall::instance_method(obj, method).with_thrift_arg(arg).Call(response);
+}
+
 inline JniCall::JniCall(jmethodID method)
   : method_(method),
     env_(JniUtil::GetJNIEnv()) {
@@ -506,6 +538,14 @@ inline Status JniCall::ObjectToResult(jobject obj, std::string* result) {
   return Status::OK();
 }
 
+inline Status JniCall::ObjectToResult(jobject obj, jobject* result) {
+  DCHECK(obj) << "Call returned unexpected null Thrift object";
+  JNIEnv* env = JniUtil::GetJNIEnv();
+  if (env == nullptr) return Status("Failed to get/create JVM");
+  RETURN_IF_ERROR(JniUtil::LocalToGlobalRef(env, obj, result));
+  return Status::OK();
+}
+
 }
 
 #endif
