@@ -33,6 +33,7 @@
 #include <rapidjson/stringbuffer.h>
 
 #include "common/logging.h"
+#include "common/global-flags.h"
 #include "gutil/endian.h"
 #include "gutil/strings/escaping.h"
 #include "gutil/strings/strip.h"
@@ -148,6 +149,9 @@ DEFINE_bool(disable_content_security_policy_header, false,
     "If true then the webserver will not add the Content-Security-Policy "
     "HTTP header to HTTP responses");
 
+DEFINE_int64(slow_http_response_warning_threshold_ms, 500,
+    "(Advanced) Threshold for considering a HTTP response to be unusually slow.");
+
 DECLARE_bool(enable_ldap_auth);
 DECLARE_string(hostname);
 DECLARE_bool(is_coordinator);
@@ -157,11 +161,13 @@ DECLARE_string(ssl_cipher_list);
 DECLARE_string(tls_ciphersuites);
 DECLARE_string(trusted_domain);
 DECLARE_bool(trusted_domain_use_xff_header);
+DECLARE_bool(trusted_domain_empty_xff_header_use_origin);
 DECLARE_bool(trusted_domain_strict_localhost);
 DECLARE_bool(jwt_token_auth);
 DECLARE_bool(jwt_validate_signature);
 DECLARE_string(jwt_custom_claim_username);
 DECLARE_string(trusted_auth_header);
+DECLARE_string(spnego_keytab_file);
 
 static const char* DOC_FOLDER = "/www/";
 static const int DOC_FOLDER_LEN = strlen(DOC_FOLDER);
@@ -474,7 +480,13 @@ Status Webserver::Start() {
     // propagated into this environment variable where the GSSAPI calls will
     // pick it up. In other words, we aren't expecting users to pass in this
     // environment variable specifically.
-    const char* kt_file = getenv("KRB5_KTNAME");
+
+    // If --spnego_keytab_file flag is not empty, web server uses keytab file location
+    // specified in --spnego_keytab_file instead of --keytab_file. This is for seperation
+    // of impala service keytab and spnego keytab for web console authentication.
+    const char* kt_file = FLAGS_spnego_keytab_file.empty() ?
+      getenv("KRB5_KTNAME") :
+      FLAGS_spnego_keytab_file.c_str();
     if (!kt_file || !kudu::Env::Default()->FileExists(kt_file)) {
       return Status("Unable to configure web server for SPNEGO authentication: "
                     "must configure a keytab file for the server");
@@ -735,6 +747,11 @@ sq_callback_result_t Webserver::BeginRequestCallback(struct sq_connection* conne
         xff_origin_string :
         GetRemoteAddress(request_info).ToString();
     StripWhiteSpace(&origin);
+    if (origin.empty() && FLAGS_trusted_domain_use_xff_header &&
+        FLAGS_trusted_domain_empty_xff_header_use_origin) {
+      origin = GetRemoteAddress(request_info).ToString();
+      StripWhiteSpace(&origin);
+    }
     if (!origin.empty()) {
       if (TrustedDomainCheck(origin, connection, request_info)) {
         total_trusted_domain_check_success_->Increment(1);
@@ -924,9 +941,16 @@ sq_callback_result_t Webserver::BeginRequestCallback(struct sq_connection* conne
         connection, req, *url_handler, &output, &content_type, cookie_rand_value);
   }
 
-  VLOG(3) << "Rendering page " << request_info->uri << " took "
-          << PrettyPrinter::Print(sw.ElapsedTime(), TUnit::TIME_NS);
-
+  uint64_t elapsed_time_ns = sw.ElapsedTime();
+  if (elapsed_time_ns > FLAGS_slow_http_response_warning_threshold_ms * 1000 * 1000) {
+    LOG(WARNING) << "Rendering page " << request_info->uri << " took "
+        << PrettyPrinter::Print(sw.ElapsedTime(), TUnit::TIME_NS)
+        << ". User: " << req.source_user << ". Address: " << req.source_socket
+        << ". Args: " << req.query_string;
+  } else {
+    VLOG(3) << "Rendering page " << request_info->uri << " took "
+            << PrettyPrinter::Print(sw.ElapsedTime(), TUnit::TIME_NS);
+  }
   SendResponse(connection, HttpStatusCodeToString(response),
       Webserver::GetMimeType(content_type), output.str(), response_headers);
 
