@@ -253,12 +253,17 @@ class ToolchainPackage(EnvVersionedPackage):
       target_comp = parts[1]
     compiler = get_toolchain_compiler()
     label = get_platform_release_label(release=platform_release).toolchain
-    toolchain_build_id = os.environ["IMPALA_TOOLCHAIN_BUILD_ID"]
+    # Most common return values for machine are x86_64 or aarch64
+    arch = platform.machine()
+    if arch not in ['aarch64', 'x86_64']:
+      raise Exception("Unsupported architecture '{}' for pre-built native-toolchain. "
+          "Fetch and build it locally by setting NATIVE_TOOLCHAIN_HOME".format(arch))
+    toolchain_build_id = os.environ["IMPALA_TOOLCHAIN_BUILD_ID_{}".format(arch.upper())]
     toolchain_host = os.environ["IMPALA_TOOLCHAIN_HOST"]
-    template_subs = {'compiler': compiler, 'label': label,
+    template_subs = {'compiler': compiler, 'label': label, 'arch': arch,
                      'toolchain_build_id': toolchain_build_id,
                      'toolchain_host': toolchain_host}
-    archive_basename_tmpl = "${name}-${version}-${compiler}-${label}"
+    archive_basename_tmpl = "${name}-${version}-${compiler}-${label}-${arch}"
     url_prefix_tmpl = "https://${toolchain_host}/build/${toolchain_build_id}/" + \
         "${name}/${version}-${compiler}/"
     unpack_directory_tmpl = "${name}-${version}"
@@ -469,9 +474,9 @@ def get_toolchain_downloads():
   toolchain_packages += [ToolchainPackage(p) for p in
       ["avro", "binutils", "boost", "breakpad", "bzip2", "calloncehack", "cctz",
        "cloudflarezlib", "cmake", "crcutil", "curl", "flatbuffers", "gdb", "gflags",
-       "glog", "gperftools", "gtest", "jwt-cpp", "libev", "libunwind", "lz4", "openldap",
-       "orc", "protobuf", "python", "rapidjson", "re2", "snappy", "tpc-h", "tpc-ds",
-       "zlib", "zstd"]]
+       "glog", "gperftools", "gtest", "jwt-cpp", "libev", "libunwind", "lz4", "mold",
+       "openldap", "orc", "protobuf", "python", "rapidjson", "re2", "snappy", "tpc-h",
+       "tpc-ds", "zlib", "zstd"]]
   python3_package = ToolchainPackage(
       "python", explicit_version=os.environ.get("IMPALA_PYTHON3_VERSION"))
   toolchain_packages += [python3_package]
@@ -480,6 +485,8 @@ def get_toolchain_downloads():
   protobuf_package_clang = ToolchainPackage(
       "protobuf", explicit_version=os.environ.get("IMPALA_PROTOBUF_CLANG_VERSION"))
   toolchain_packages += [protobuf_package_clang]
+  if platform.machine() == 'aarch64':
+    toolchain_packages.append(ToolchainPackage("hadoop-client"))
   # Check whether this platform is supported (or whether a valid custom toolchain
   # has been provided).
   if not try_get_platform_release_label() \
