@@ -70,7 +70,7 @@ fi
 # WARNING: If changing this value, also run these commands:
 # cd ${IMPALA_HOME}/java
 # mvn versions:set -DnewVersion=YOUR_NEW_VERSION
-export IMPALA_VERSION=4.3.0-RELEASE
+export IMPALA_VERSION=4.4.1-RELEASE
 
 # Whether to build on Apache Hive (or CDP Hive). Versions of some toolchain dependencies
 # (e.g. thrift) will also depend on this.
@@ -85,12 +85,24 @@ export USE_AVRO_CPP=${USE_AVRO_CPP:=false}
 # moving to a different build of the toolchain, e.g. when a version is bumped or a
 # compile option is changed. The build id can be found in the output of the toolchain
 # build jobs, it is constructed from the build number and toolchain git hash prefix.
-export IMPALA_TOOLCHAIN_BUILD_ID=358-e7cfab15d3
+export IMPALA_TOOLCHAIN_BUILD_ID_AARCH64=34-f93e2c9a86
+export IMPALA_TOOLCHAIN_BUILD_ID_X86_64=422-f93e2c9a86
 export IMPALA_TOOLCHAIN_REPO=\
 ${IMPALA_TOOLCHAIN_REPO:-https://github.com/cloudera/native-toolchain.git}
 export IMPALA_TOOLCHAIN_BRANCH=${IMPALA_TOOLCHAIN_BRANCH:-master}
 export IMPALA_TOOLCHAIN_COMMIT_HASH=\
-${IMPALA_TOOLCHAIN_COMMIT_HASH-e7cfab15d36ae051747252b676f0a11a9c58fe05}
+${IMPALA_TOOLCHAIN_COMMIT_HASH-f93e2c9a865c80cafd76b872ad04400877766a2f}
+# Compare the build ref in build IDs by removing everything up-to-and-including the
+# first hyphen.
+if [ "${IMPALA_TOOLCHAIN_BUILD_ID_AARCH64#*-}" \
+    != "${IMPALA_TOOLCHAIN_BUILD_ID_X86_64#*-}" ]; then
+  echo "IMPALA_TOOLCHAIN_BUILD_ID_AARCH64 and IMPALA_TOOLCHAIN_BUILD_ID_X86_64 must" \
+    "come from the same commit hash."
+  exit 1
+fi
+
+export ARCH_NAME=$(uname -p)
+
 # Versions of toolchain dependencies.
 # -----------------------------------
 if $USE_AVRO_CPP; then
@@ -135,7 +147,7 @@ export IMPALA_JWT_CPP_VERSION=0.5.0
 unset IMPALA_JWT_CPP_URL
 export IMPALA_LIBEV_VERSION=4.20-p1
 unset IMPALA_LIBEV_URL
-export IMPALA_LIBUNWIND_VERSION=1.5.0-p1
+export IMPALA_LIBUNWIND_VERSION=1.7.2-p1
 unset IMPALA_LIBUNWIND_URL
 export IMPALA_LLVM_VERSION=5.0.1-p7
 unset IMPALA_LLVM_URL
@@ -191,13 +203,26 @@ export IMPALA_CLOUDFLAREZLIB_VERSION=9e601a3f37
 unset IMPALA_CLOUDFLAREZLIB_URL
 export IMPALA_CALLONCEHACK_VERSION=1.0.0
 unset IMPALA_CALLONCEHACK_URL
+if [[ $ARCH_NAME == 'aarch64' ]]; then
+  export IMPALA_HADOOP_CLIENT_VERSION=3.3.6
+  unset IMPALA_HADOOP_CLIENT_URL
+fi
+export IMPALA_MOLD_VERSION=2.4.1
+unset IMPALA_MOLD_URL
+
+# Impala JDBC driver for testing.
+export IMPALA_SIMBA_JDBC_DRIVER_VERSION=42-2.6.32.1041
+
 # Thrift related environment variables.
 # IMPALA_THRIFT_POM_VERSION is used to populate IMPALA_THRIFT_JAVA_VERSION and
 # thrift.version in java/pom.xml.
-# If upgrading IMPALA_THRIFT_PY_VERSION, remember to also upgrade thrift version in
-# shell/ext-py and shell/packaging/requirements.txt. IMPALA_THRIFT_PY_VERSION is used
-# with Impyla and for the thrift compiler.
-export IMPALA_THRIFT_CPP_VERSION=0.16.0-p5
+# IMPALA_THRIFT_PY_VERSION is used to find the thrift compiler to produce
+# the generated Python code. The code that uses the generated Python code gets
+# the corresponding Thrift runtime library by pip installing thrift (and does not
+# respect this version). If upgrading IMPALA_THRIFT_PY_VERSION, also upgrade the
+# thrift version in shell/ext-py, shell/packaging/requirements.txt, and
+# infra/python/deps/requirements.txt.
+export IMPALA_THRIFT_CPP_VERSION=0.16.0-p7
 unset IMPALA_THRIFT_CPP_URL
 if $USE_APACHE_HIVE; then
   # Apache Hive 3 clients can't run on thrift versions >= 0.14 (IMPALA-11801)
@@ -205,16 +230,21 @@ if $USE_APACHE_HIVE; then
   export IMPALA_THRIFT_JAVA_VERSION=${IMPALA_THRIFT_POM_VERSION}-p5
 else
   export IMPALA_THRIFT_POM_VERSION=0.16.0
-  export IMPALA_THRIFT_JAVA_VERSION=${IMPALA_THRIFT_POM_VERSION}-p5
+  export IMPALA_THRIFT_JAVA_VERSION=${IMPALA_THRIFT_POM_VERSION}-p7
 fi
 unset IMPALA_THRIFT_JAVA_URL
-export IMPALA_THRIFT_PY_VERSION=0.16.0-p5
+export IMPALA_THRIFT_PY_VERSION=0.16.0-p7
 unset IMPALA_THRIFT_PY_URL
 
 # Find system python versions for testing
 export IMPALA_SYSTEM_PYTHON2="${IMPALA_SYSTEM_PYTHON2_OVERRIDE-$(command -v python2)}"
 export IMPALA_SYSTEM_PYTHON3="${IMPALA_SYSTEM_PYTHON3_OVERRIDE-$(command -v python3)}"
 
+# Additional Python versions to use when building the impala-shell prebuilt tarball
+# via make_shell_tarball.sh. That tarball includes precompiled packages, so it can be
+# used without additional system dependencies needed for pip install.
+# export IMPALA_EXTRA_PACKAGE_PYTHONS=python3.6;python3.10
+
 if [[ $OSTYPE == "darwin"* ]]; then
   IMPALA_CYRUS_SASL_VERSION=2.1.26
   unset IMPALA_CYRUS_SASL_URL
@@ -225,19 +255,19 @@ fi
 : ${IMPALA_TOOLCHAIN_HOST:=native-toolchain.s3.amazonaws.com}
 export IMPALA_TOOLCHAIN_HOST
 
-export CDP_BUILD_NUMBER=44206393
+export CDP_BUILD_NUMBER=45689292
 export CDP_MAVEN_REPOSITORY=\
 "https://${IMPALA_TOOLCHAIN_HOST}/build/cdp_components/${CDP_BUILD_NUMBER}/maven"
-export CDP_AVRO_JAVA_VERSION=1.8.2.7.2.18.0-273
-export CDP_HADOOP_VERSION=3.1.1.7.2.18.0-273
-export CDP_HBASE_VERSION=2.4.17.7.2.18.0-273
-export CDP_HIVE_VERSION=3.1.3000.7.2.18.0-273
-export CDP_ICEBERG_VERSION=1.1.0.7.2.18.0-273
-export CDP_KNOX_VERSION=1.3.0.7.2.18.0-273
-export CDP_OZONE_VERSION=1.3.0.7.2.18.0-273
-export CDP_PARQUET_VERSION=1.10.99.7.2.18.0-273
-export CDP_RANGER_VERSION=2.4.0.7.2.18.0-273
-export CDP_TEZ_VERSION=0.9.1.7.2.18.0-273
+export CDP_AVRO_JAVA_VERSION=1.8.2.7.2.18.0-369
+export CDP_HADOOP_VERSION=3.1.1.7.2.18.0-369
+export CDP_HBASE_VERSION=2.4.17.7.2.18.0-369
+export CDP_HIVE_VERSION=3.1.3000.7.2.18.0-369
+export CDP_ICEBERG_VERSION=1.3.1.7.2.18.0-369
+export CDP_KNOX_VERSION=1.3.0.7.2.18.0-369
+export CDP_OZONE_VERSION=1.3.0.7.2.18.0-369
+export CDP_PARQUET_VERSION=1.10.99.7.2.18.0-369
+export CDP_RANGER_VERSION=2.4.0.7.2.18.0-369
+export CDP_TEZ_VERSION=0.9.1.7.2.18.0-369
 
 # Ref: https://infra.apache.org/release-download-pages.html#closer
 : ${APACHE_MIRROR:="https://www.apache.org/dyn/closer.cgi"}
@@ -246,18 +276,15 @@ export APACHE_HIVE_VERSION=3.1.3
 export APACHE_HIVE_STORAGE_API_VERSION=2.7.0
 export APACHE_OZONE_VERSION=1.3.0
 
-export ARCH_NAME=$(uname -p)
-
 # Java dependencies that are not also runtime components. Declaring versions here allows
 # other branches to override them in impala-config-branch.sh for cleaner patches.
 export IMPALA_BOUNCY_CASTLE_VERSION=1.68
 export IMPALA_COS_VERSION=3.1.0-8.0.8
-export IMPALA_OSS_VERSION=3.1.1
 export IMPALA_DERBY_VERSION=10.14.2.0
-export IMPALA_GUAVA_VERSION=31.1-jre
+export IMPALA_GUAVA_VERSION=32.0.1-jre
 export IMPALA_HUDI_VERSION=0.5.0-incubating
 export IMPALA_HTTP_CORE_VERSION=4.4.14
-export IMPALA_JACKSON_DATABIND_VERSION=2.13.4.2
+export IMPALA_JACKSON_DATABIND_VERSION=2.15.3
 export IMPALA_JSON_SMART_VERSION=2.4.11
 export IMPALA_JUNIT_VERSION=4.12
 export IMPALA_KITE_VERSION=1.1.0
@@ -266,8 +293,9 @@ export IMPALA_PAC4J_VERSION=4.5.5
 export IMPALA_RELOAD4j_VERSION=1.2.22
 export IMPALA_SLF4J_VERSION=2.0.3
 export IMPALA_SPRINGFRAMEWORK_VERSION=5.3.27
-export IMPALA_XMLSEC_VERSION=2.2.3
+export IMPALA_XMLSEC_VERSION=2.2.6
 export IMPALA_OBS_VERSION=3.1.1-hw-42
+export IMPALA_DBCP2_VERSION=2.9.0
 
 # When Impala is building docker images on Redhat-based distributions,
 # it is useful to be able to customize the base image. Some users will
@@ -505,8 +533,11 @@ chmod 755 "${PYTHON_EGG_CACHE}"
 # If it's 0, Impala will be built with the compiler in the toolchain directory.
 export USE_SYSTEM_GCC=${USE_SYSTEM_GCC-0}
 
-# Use ld.gold instead of ld by default to speed up builds.
-export USE_GOLD_LINKER=${USE_GOLD_LINKER-true}
+# Allow the linker to be set to gold, mold, or regular ld. Gold is the default
+# as it has been for a long time. Mold is a new linker that is faster than gold.
+# Note: This is validated in the CMake code.
+# TODO: Add support for lld as well
+export IMPALA_LINKER=${IMPALA_LINKER-gold}
 
 # Override the default compiler by setting a path to the new compiler. The default
 # compiler depends on USE_SYSTEM_GCC and IMPALA_GCC_VERSION. The intended use case
@@ -587,15 +618,25 @@ export EXTERNAL_LISTEN_HOST="${EXTERNAL_LISTEN_HOST-0.0.0.0}"
 export DEFAULT_FS="${DEFAULT_FS-hdfs://${INTERNAL_LISTEN_HOST}:20500}"
 export WAREHOUSE_LOCATION_PREFIX="${WAREHOUSE_LOCATION_PREFIX-}"
 export LOCAL_FS="file:${WAREHOUSE_LOCATION_PREFIX}"
-export IMPALA_CLUSTER_NODES_DIR="${IMPALA_CLUSTER_NODES_DIR-$IMPALA_HOME/testdata/cluster/cdh$CDH_MAJOR_VERSION}"
+# Use different node directories for each filesystem so we don't need to recreate them
+# from scratch when switching.
+UNIQUE_FS_LABEL=
+if [[ "${TARGET_FILESYSTEM}" != "hdfs" ]]; then
+  UNIQUE_FS_LABEL="${UNIQUE_FS_LABEL}-${TARGET_FILESYSTEM}"
+fi
+if [[ "${ERASURE_CODING}" = true ]]; then
+  UNIQUE_FS_LABEL="${UNIQUE_FS_LABEL}-ec"
+fi
+DEFAULT_NODES_DIR="$IMPALA_HOME/testdata/cluster/cdh$CDH_MAJOR_VERSION$UNIQUE_FS_LABEL"
+export IMPALA_CLUSTER_NODES_DIR="${IMPALA_CLUSTER_NODES_DIR-$DEFAULT_NODES_DIR}"
 
-ESCAPED_IMPALA_HOME=$(sed "s/[^0-9a-zA-Z]/_/g" <<< "$IMPALA_HOME")
+ESCAPED_DB_UID=$(sed "s/[^0-9a-zA-Z]/_/g" <<< "$UNIQUE_FS_LABEL$IMPALA_HOME")
 if $USE_APACHE_HIVE; then
   export HIVE_HOME="$APACHE_COMPONENTS_HOME/apache-hive-${IMPALA_HIVE_VERSION}-bin"
   export HIVE_SRC_DIR="$APACHE_COMPONENTS_HOME/apache-hive-${IMPALA_HIVE_VERSION}-src"
   # if apache hive is being used change the metastore db name, so we don't have to
   # format the metastore db everytime we switch between hive versions
-  export METASTORE_DB=${METASTORE_DB-"$(cut -c-59 <<< HMS$ESCAPED_IMPALA_HOME)_apache"}
+  export METASTORE_DB=${METASTORE_DB-"$(cut -c-59 <<< HMS$ESCAPED_DB_UID)_apache"}
 else
   export HIVE_HOME=${HIVE_HOME_OVERRIDE:-\
 "$CDP_COMPONENTS_HOME/apache-hive-${IMPALA_HIVE_VERSION}-bin"}
@@ -603,7 +644,7 @@ else
 "${CDP_COMPONENTS_HOME}/hive-${IMPALA_HIVE_VERSION}"}
   # Previously, there were multiple configurations and the "_cdp" included below
   # allowed the two to be distinct. We keep this "_cdp" for historical reasons.
-  export METASTORE_DB=${METASTORE_DB-"$(cut -c-59 <<< HMS$ESCAPED_IMPALA_HOME)_cdp"}
+  export METASTORE_DB=${METASTORE_DB-"$(cut -c-59 <<< HMS$ESCAPED_DB_UID)_cdp"}
 fi
 # Set the path to the hive_metastore.thrift which is used to build thrift code
 export HIVE_METASTORE_THRIFT_DIR=${HIVE_METASTORE_THRIFT_DIR_OVERRIDE:-\
@@ -618,7 +659,7 @@ fi
 # Set the Hive binaries in the path
 export PATH="$HIVE_HOME/bin:$HBASE_HOME/bin:$OZONE_HOME/bin:$PATH"
 
-RANGER_POLICY_DB=${RANGER_POLICY_DB-$(cut -c-63 <<< ranger$ESCAPED_IMPALA_HOME)}
+RANGER_POLICY_DB=${RANGER_POLICY_DB-$(cut -c-63 <<< ranger$ESCAPED_DB_UID)}
 # The DB script in Ranger expects the database name to be in lower case.
 export RANGER_POLICY_DB=$(echo ${RANGER_POLICY_DB} | tr '[:upper:]' '[:lower:]')
 
@@ -851,6 +892,7 @@ export IMPALA_FE_TEST_LOGS_DIR="${IMPALA_LOGS_DIR}/fe_tests"
 export IMPALA_FE_TEST_COVERAGE_DIR="${IMPALA_FE_TEST_LOGS_DIR}/coverage"
 export IMPALA_BE_TEST_LOGS_DIR="${IMPALA_LOGS_DIR}/be_tests"
 export IMPALA_EE_TEST_LOGS_DIR="${IMPALA_LOGS_DIR}/ee_tests"
+export IMPALA_JS_TEST_LOGS_DIR="${IMPALA_LOGS_DIR}/js_tests"
 export IMPALA_CUSTOM_CLUSTER_TEST_LOGS_DIR="${IMPALA_LOGS_DIR}/custom_cluster_tests"
 export IMPALA_MVN_LOGS_DIR="${IMPALA_LOGS_DIR}/mvn"
 export IMPALA_TIMEOUT_LOGS_DIR="${IMPALA_LOGS_DIR}/timeout_stacktrace"
@@ -969,7 +1011,7 @@ fi
 # overall build type) and does not apply when using a local Kudu build.
 export USE_KUDU_DEBUG_BUILD=${USE_KUDU_DEBUG_BUILD-false}
 
-export IMPALA_KUDU_VERSION=${IMPALA_KUDU_VERSION-"1.17.0"}
+export IMPALA_KUDU_VERSION=${IMPALA_KUDU_VERSION-"e742f86f6d"}
 export IMPALA_KUDU_HOME=${IMPALA_TOOLCHAIN_PACKAGES_HOME}/kudu-$IMPALA_KUDU_VERSION
 export IMPALA_KUDU_JAVA_HOME=\
 ${IMPALA_TOOLCHAIN_PACKAGES_HOME}/kudu-${IMPALA_KUDU_VERSION}/java
@@ -1018,9 +1060,6 @@ CLASSPATH="$IMPALA_FE_DIR/src/test/resources:$CLASSPATH"
 # A marker in the environment to prove that we really did source this file
 export IMPALA_CONFIG_SOURCED=1
 
-# Whether to strip the impalad binary when generating deployment package
-export STRIP_DEPLOYMENT_IMPALAD=true
-
 echo "IMPALA_VERSION          = $IMPALA_VERSION"
 echo "IMPALA_HOME             = $IMPALA_HOME"
 echo "HADOOP_HOME             = $HADOOP_HOME"
@@ -1067,7 +1106,6 @@ echo "IMPALA_KUDU_VERSION     = $IMPALA_KUDU_VERSION"
 echo "IMPALA_RANGER_VERSION   = $IMPALA_RANGER_VERSION"
 echo "IMPALA_ICEBERG_VERSION  = $IMPALA_ICEBERG_VERSION"
 echo "IMPALA_COS_VERSION      = $IMPALA_COS_VERSION"
-echo "IMPALA_OSS_VERSION      = $IMPALA_OSS_VERSION"
 echo "IMPALA_OBS_VERSION      = $IMPALA_OBS_VERSION"
 echo "IMPALA_SYSTEM_PYTHON2   = $IMPALA_SYSTEM_PYTHON2"
 echo "IMPALA_SYSTEM_PYTHON3   = $IMPALA_SYSTEM_PYTHON3"
