@@ -61,6 +61,8 @@ fi
 # Run Cluster Tests
 : ${CLUSTER_TEST:=true}
 : ${CLUSTER_TEST_FILES:=}
+# Run JS tests
+: ${JS_TEST:=false}
 # Verifiers to run after all tests. Skipped if empty.
 : ${TEST_SUITE_VERIFIERS:=verifiers/test_banned_log_messages.py}
 : ${TEST_SUITE_VERIFIERS_LOG_DIR:=${IMPALA_LOGS_DIR}/verifiers}
@@ -114,6 +116,15 @@ if [[ "${ERASURE_CODING}" = true ]]; then
   FE_TEST=false
 fi
 
+if test -v CMAKE_BUILD_TYPE && [[ "${CMAKE_BUILD_TYPE}" =~ 'UBSAN' ]] \
+    && [[ "$(uname -p)" = "aarch64" ]]; then
+  # FE tests fail on ARM with
+  #   libfesupport.so: cannot allocate memory in static TLS block
+  # https://bugzilla.redhat.com/show_bug.cgi?id=1722181 mentions this is more likely
+  # on aarch64 due to how it uses thread-local storage (TLS). There's no clear fix.
+  FE_TEST=false
+fi
+
 # Indicates whether code coverage reports should be generated.
 : ${CODE_COVERAGE:=false}
 
@@ -261,6 +272,7 @@ do
       # If running with Java 9+, add-opens to JAVA_TOOL_OPTIONS for
       # CatalogdMetaProviderTest.testWeights with ehcache.sizeof.
       JAVA_OPTIONS=" --add-opens=java.base/java.io=ALL-UNNAMED"
+      JAVA_OPTIONS+=" --add-opens=java.base/java.lang.invoke=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=java.base/java.lang.module=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=java.base/java.lang.ref=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=java.base/java.lang.reflect=ALL-UNNAMED"
@@ -270,17 +282,23 @@ do
       JAVA_OPTIONS+=" --add-opens=java.base/java.nio.file.attribute=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=java.base/java.nio=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=java.base/java.security=ALL-UNNAMED"
+      JAVA_OPTIONS+=" --add-opens=java.base/java.util.concurrent.locks=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=java.base/java.util.concurrent=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=java.base/java.util.jar=ALL-UNNAMED"
+      JAVA_OPTIONS+=" --add-opens=java.base/java.util.regex=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=java.base/java.util.zip=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=java.base/java.util=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=java.base/jdk.internal.loader=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=java.base/jdk.internal.math=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=java.base/jdk.internal.module=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=java.base/jdk.internal.perf=ALL-UNNAMED"
+      JAVA_OPTIONS+=" --add-opens=java.base/jdk.internal.platform=ALL-UNNAMED"
+      JAVA_OPTIONS+=" --add-opens=java.base/jdk.internal.platform.cgroupv1=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=java.base/jdk.internal.ref=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=java.base/jdk.internal.reflect=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=java.base/jdk.internal.util.jar=ALL-UNNAMED"
+      JAVA_OPTIONS+=" --add-opens=java.base/sun.net.www.protocol.jar=ALL-UNNAMED"
+      JAVA_OPTIONS+=" --add-opens=java.base/sun.nio.ch=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=java.base/sun.nio.fs=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=jdk.dynalink/jdk.dynalink.beans=ALL-UNNAMED"
       JAVA_OPTIONS+=" --add-opens=jdk.dynalink/jdk.dynalink.linker.support=ALL-UNNAMED"
@@ -349,6 +367,12 @@ do
     fi
   fi
 
+  if [[ "$JS_TEST" == true ]]; then
+    if ! "${IMPALA_HOME}/tests/run-js-tests.sh"; then
+      TEST_RET_CODE=1
+    fi
+  fi
+
   if [[ "$JDBC_TEST" == true ]]; then
     # Run the JDBC tests with background loading disabled. This is interesting because
     # it requires loading missing table metadata.
