@@ -22,10 +22,10 @@
 # "-INTERNAL" appended. Parts of the code will look for this to distinguish
 # between released and internal versions.
 VERSION=${IMPALA_VERSION}
-GIT_HASH=$(git rev-parse HEAD 2> /dev/null)
+GIT_HASH=6edbf57651aa84652e0121fe9055e53c4d350871
 if [ -z $GIT_HASH ]
 then
-  GIT_HASH=056a9cc26f40cd491b52f5fa80d73646680ac13e
+  GIT_HASH="Could not obtain git hash"
 fi
 
 BUILD_TIME=`date`
