@@ -424,7 +424,7 @@ bootstrap_dependencies() {
         git remote add toolchain "${IMPALA_TOOLCHAIN_REPO}"
         git fetch toolchain "${IMPALA_TOOLCHAIN_BRANCH}"
         # Specifying a branch avoids a large message from git about detached HEADs.
-        git checkout "${IMPALA_TOOLCHAIN_COMMIT_HASH}" -b "${IMPALA_TOOLCHAIN_BUILD_ID}"
+        git checkout "${IMPALA_TOOLCHAIN_COMMIT_HASH}" -b "${IMPALA_TOOLCHAIN_BRANCH}"
       else
         pushd "${NATIVE_TOOLCHAIN_HOME}"
       fi
@@ -443,13 +443,10 @@ bootstrap_dependencies() {
     "$IMPALA_HOME/bin/bootstrap_toolchain.py"
     echo "Toolchain bootstrap complete."
   fi
-  # Download prebuilt Hadoop native binaries for aarch64
+  # Use prebuilt Hadoop native binaries for aarch64
   if [[ "$(uname -p)" = "aarch64" ]]; then
-    if ! [[ -d "$IMPALA_HOME/../hadoopAarch64NativeLibs" ]]; then
-      git clone https://github.com/zhaorenhai/hadoopAarch64NativeLibs \
-        "$IMPALA_HOME/../hadoopAarch64NativeLibs"
-    fi
-    cp $IMPALA_HOME/../hadoopAarch64NativeLibs/lib*  $HADOOP_HOME/lib/native/
+    cp $IMPALA_TOOLCHAIN_PACKAGES_HOME/hadoop-client-$IMPALA_HADOOP_CLIENT_VERSION/lib/* \
+        $HADOOP_HOME/lib/native/
   fi
   if [[ "${USE_APACHE_HIVE}" = true ]]; then
     "$IMPALA_HOME/testdata/bin/patch_hive.sh"
