@@ -36,6 +36,7 @@ import org.apache.hadoop.hive.common.ValidTxnList;
 import org.apache.hadoop.hive.common.ValidWriteIdList;
 import org.apache.hadoop.hive.metastore.IMetaStoreClient;
 import org.apache.hadoop.hive.metastore.RetryingMetaStoreClient;
+import org.apache.hadoop.hive.metastore.api.AlreadyExistsException;
 import org.apache.hadoop.hive.metastore.api.ColumnStatistics;
 import org.apache.hadoop.hive.metastore.api.ColumnStatisticsObj;
 import org.apache.hadoop.hive.metastore.api.Database;
@@ -65,6 +66,7 @@ import org.apache.hadoop.hive.metastore.txn.TxnUtils;
 import org.apache.hadoop.hive.metastore.utils.FileUtils;
 import org.apache.impala.catalog.CatalogException;
 import org.apache.impala.catalog.CatalogServiceCatalog;
+import org.apache.impala.catalog.DataSource;
 import org.apache.impala.catalog.Db;
 import org.apache.impala.catalog.HdfsPartition;
 import org.apache.impala.catalog.HdfsTable;
@@ -457,9 +459,15 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
    * eventIds returned.
    *
    * @see org.apache.hadoop.hive.metastore.HiveMetaStoreClient#getNextNotification
+   * @param msClient Metastore client
+   * @param eventRequest Notification event request
+   * @param eventTypeSkipList unused
+   * @return NotificationEventResponse
+   * @throws TException
    */
   public static NotificationEventResponse getNextNotification(IMetaStoreClient msClient,
-      NotificationEventRequest eventRequest) throws TException {
+      NotificationEventRequest eventRequest, List<String> eventTypeSkipList)
+      throws TException {
     return getThriftClient(msClient).get_next_notification(eventRequest);
   }
 
@@ -581,6 +589,15 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
         "getPartitionsForRefreshingFileMetadata is not supported.");
   }
 
+  /**
+   * CDP Hive-3 only function.
+   */
+  public static Map<String, Long> getLatestCompactions(MetaStoreClient client,
+      String dbName, String tableName, List<String> partitionNames,
+      String unPartitionedName, long lastCompactionId) throws TException {
+    throw new UnsupportedOperationException("getLatestCompactions is not supported.");
+  }
+
   /**
    * CDP Hive-3 only function.
    */
@@ -616,6 +633,11 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
 
     }
 
+    @Override
+    protected boolean onFailure(Exception e) {
+      return false;
+    }
+
     @Override
     protected boolean isEventProcessingDisabled() {
       return false;
@@ -940,4 +962,22 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
     //  materialized view is outdated for rewriting, so set the flag to "Unknown"
     formatOutput("Outdated for Rewriting:", "Unknown", tableInfo);
   }
+
+  public static void createDataSource(IMetaStoreClient client, DataSource dataSource)
+      throws InvalidObjectException, AlreadyExistsException, MetaException, TException {
+    // noop for unsupported operation IMetaStoreClient.createDataConnector().
+  }
+
+  public static void dropDataSource(IMetaStoreClient client, String name,
+      boolean ifExists) throws NoSuchObjectException, InvalidOperationException,
+          MetaException, TException {
+    // noop for unsupported operation IMetaStoreClient.dropDataConnector().
+  }
+
+  public static Map<String, DataSource> loadAllDataSources(IMetaStoreClient client)
+      throws MetaException, TException {
+    // Unsupported operation IMetaStoreClient.getAllDataConnectorNames() and
+    // IMetaStoreClient.getDataConnector().
+    return null;
+  }
 }
