@@ -27,6 +27,7 @@ import static org.apache.impala.util.HiveMetadataFormatUtils.formatOutput;
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import com.google.common.base.Stopwatch;
+import com.google.common.base.Strings;
 import com.google.common.collect.Iterables;
 
 import java.net.InetAddress;
@@ -46,9 +47,11 @@ import java.util.stream.Collectors;
 import org.apache.hadoop.hive.common.ValidWriteIdList;
 import org.apache.hadoop.hive.metastore.HiveMetaStoreClient;
 import org.apache.hadoop.hive.metastore.IMetaStoreClient;
+import org.apache.hadoop.hive.metastore.api.AlreadyExistsException;
 import org.apache.hadoop.hive.metastore.api.ColumnStatistics;
 import org.apache.hadoop.hive.metastore.api.ColumnStatisticsObj;
 import org.apache.hadoop.hive.metastore.api.CompactionInfoStruct;
+import org.apache.hadoop.hive.metastore.api.DataConnector;
 import org.apache.hadoop.hive.metastore.api.Database;
 import org.apache.hadoop.hive.metastore.api.FireEventRequest;
 import org.apache.hadoop.hive.metastore.api.FireEventRequestData;
@@ -90,6 +93,7 @@ import org.apache.impala.analysis.TableName;
 import org.apache.impala.catalog.CatalogException;
 import org.apache.impala.catalog.CatalogServiceCatalog;
 import org.apache.impala.catalog.CompactionInfoLoader;
+import org.apache.impala.catalog.DataSource;
 import org.apache.impala.catalog.DatabaseNotFoundException;
 import org.apache.impala.catalog.Db;
 import org.apache.impala.catalog.HdfsPartition;
@@ -131,6 +135,13 @@ import org.slf4j.LoggerFactory;
 public class MetastoreShim extends Hive3MetastoreShimBase {
   private static final Logger LOG = LoggerFactory.getLogger(MetastoreShim.class);
 
+  // Impala DataSource object is saved in HMS as DataConnector with type
+  // as 'impalaDataSource'
+  private static final String HMS_DATA_CONNECTOR_TYPE = "impalaDataSource";
+  private static final String HMS_DATA_CONNECTOR_DESC = "Impala DataSource Object";
+  private static final String HMS_DATA_CONNECTOR_PARAM_KEY_CLASS_NAME = "className";
+  private static final String HMS_DATA_CONNECTOR_PARAM_KEY_API_VERSION = "apiVersion";
+
   /**
    * Wrapper around IMetaStoreClient.alter_table with validWriteIds as a param.
    */
@@ -534,7 +545,7 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
       LOG.error("FireEventResponse does not have event ids set for table {}.{}. This "
               + "may cause the table to unnecessarily be refreshed when the " +
               "refresh/invalidate event is received.", dbName, tableName);
-      return Collections.EMPTY_LIST;
+      return Collections.emptyList();
     }
     return response.getEventIds();
   }
@@ -583,9 +594,19 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
   /**
    * Wrapper around IMetaStoreClient.getThriftClient().get_next_notification() to deal
    * with added arguments.
+   *
+   * @param msClient Metastore client
+   * @param eventRequest Notification event request
+   * @param eventTypeSkipList Unwanted event types
+   * @return NotificationEventResponse
+   * @throws TException
    */
   public static NotificationEventResponse getNextNotification(IMetaStoreClient msClient,
-      NotificationEventRequest eventRequest) throws TException {
+      NotificationEventRequest eventRequest, List<String> eventTypeSkipList)
+      throws TException {
+    if (eventTypeSkipList != null) {
+      eventRequest.setEventTypeSkipList(eventTypeSkipList);
+    }
     return msClient.getThriftClient().get_next_notification(eventRequest);
   }
 
@@ -722,39 +743,19 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
   }
 
   /**
-   * Fetches the latest compaction id from HMS and compares with partition metadata in
-   * cache. If a partition is stale due to compaction, removes it from metas.
+   * Fetches the latest compactions from HMS
    */
-  public static List<PartitionRef> checkLatestCompaction(MetaStoreClientPool msClientPool,
-      String dbName, String tableName, TableMetaRef table,
-      Map<PartitionRef, PartitionMetadata> metas, String unPartitionedName)
-      throws TException {
-    Preconditions.checkNotNull(table, "TableMetaRef must be non-null");
-    Preconditions.checkNotNull(metas, "Partition map must be non-null");
-    if (metas.isEmpty()) {
-      return Collections.emptyList();
-    }
-    Stopwatch sw = Stopwatch.createStarted();
-    List<PartitionRef> stalePartitions = new ArrayList<>();
-    if (!table.isTransactional() || metas.isEmpty()) return stalePartitions;
+  public static Map<String, Long> getLatestCompactions(MetaStoreClient client,
+      String dbName, String tableName, List<String> partitionNames,
+      String unPartitionedName, long lastCompactionId) throws TException {
     GetLatestCommittedCompactionInfoRequest request =
         new GetLatestCommittedCompactionInfoRequest(dbName, tableName);
-    if (table.isPartitioned()) {
-      request.setPartitionnames(metas.keySet().stream()
-          .map(PartitionRef::getName).collect(Collectors.toList()));
-    }
-    long lastCompactionId = metas.values().stream()
-        .mapToLong(p -> p.getLastCompactionId()).max().orElse(-1);
-    if (lastCompactionId > 0) {
-      request.setLastCompactionId(lastCompactionId);
-    }
-
+    request.setPartitionnames(partitionNames);
+    if (lastCompactionId > 0) request.setLastCompactionId(lastCompactionId);
     GetLatestCommittedCompactionInfoResponse response;
-    try (MetaStoreClientPool.MetaStoreClient client = msClientPool.getClient()) {
-      response = CompactionInfoLoader.getLatestCompactionInfo(client, request);
-    }
+    response = CompactionInfoLoader.getLatestCompactionInfo(client, request);
     Map<String, Long> partNameToCompactionId = new HashMap<>();
-    if (table.isPartitioned()) {
+    if (partitionNames != null) {
       for (CompactionInfoStruct ci : response.getCompactions()) {
         if (ci.getPartitionname() != null) {
           partNameToCompactionId.put(ci.getPartitionname(), ci.getId());
@@ -765,14 +766,41 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
         }
       }
     } else {
-      CompactionInfoStruct ci = Iterables.getOnlyElement(response.getCompactions(),
-          null);
-      if (ci != null) {
-        partNameToCompactionId.put(unPartitionedName, ci.getId());
-      }
+      CompactionInfoStruct ci = Iterables.getOnlyElement(response.getCompactions(), null);
+      if (ci != null) partNameToCompactionId.put(unPartitionedName, ci.getId());
     }
-    Iterator<Entry<PartitionRef, PartitionMetadata>> iter =
-        metas.entrySet().iterator();
+    return partNameToCompactionId;
+  }
+
+  /**
+   * Fetches the latest compaction id from HMS and compares with partition metadata in
+   * cache. If a partition is stale due to compaction, removes it from metas.
+   */
+  public static List<PartitionRef> checkLatestCompaction(MetaStoreClientPool msClientPool,
+      String dbName, String tableName, TableMetaRef table,
+      Map<PartitionRef, PartitionMetadata> metas, String unPartitionedName)
+      throws TException {
+    Preconditions.checkNotNull(table, "TableMetaRef must be non-null");
+    Preconditions.checkNotNull(metas, "Partition map must be non-null");
+    if (!table.isTransactional() || metas.isEmpty()) return Collections.emptyList();
+    Stopwatch sw = Stopwatch.createStarted();
+    List<String> partitionNames = null;
+    if (table.isPartitioned()) {
+      partitionNames =
+          metas.keySet().stream().map(PartitionRef::getName).collect(Collectors.toList());
+    }
+    long lastCompactionId = metas.values()
+                                .stream()
+                                .mapToLong(PartitionMetadata::getLastCompactionId)
+                                .max()
+                                .orElse(-1);
+    Map<String, Long> partNameToCompactionId = Collections.emptyMap();
+    try (MetaStoreClientPool.MetaStoreClient client = msClientPool.getClient()) {
+      partNameToCompactionId = getLatestCompactions(
+          client, dbName, tableName, partitionNames, unPartitionedName, lastCompactionId);
+    }
+    List<PartitionRef> stalePartitions = new ArrayList<>();
+    Iterator<Entry<PartitionRef, PartitionMetadata>> iter = metas.entrySet().iterator();
     while (iter.hasNext()) {
       Map.Entry<PartitionRef, PartitionMetadata> entry = iter.next();
       if (partNameToCompactionId.containsKey(entry.getKey().getName())) {
@@ -802,6 +830,7 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
   public static class CommitTxnEvent extends MetastoreEvent {
     private final CommitTxnMessage commitTxnMessage_;
     private final long txnId_;
+    private Set<TableWriteId> tableWriteIds_ = Collections.emptySet();
 
     public CommitTxnEvent(CatalogOpExecutor catalogOpExecutor, Metrics metrics,
         NotificationEvent event) {
@@ -811,6 +840,8 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
       commitTxnMessage_ = MetastoreEventsProcessor.getMessageDeserializer()
           .getCommitTxnMessage(event.getMessage());
       txnId_ = commitTxnMessage_.getTxnId();
+      LOG.info("EventId: {} EventType: COMMIT_TXN transaction id: {}", getEventId(),
+          txnId_);
     }
 
     @Override
@@ -818,6 +849,7 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
       // To ensure no memory leaking in case an exception is thrown, we remove entries
       // at first.
       Set<TableWriteId> committedWriteIds = catalog_.removeWriteIds(txnId_);
+      tableWriteIds_ = committedWriteIds;
       // Via getAllWriteEventInfo, we can get data insertion info for transactional tables
       // even though there are no insert events generated for transactional tables. Note
       // that we cannot get DDL info from this API.
@@ -846,6 +878,26 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
       }
     }
 
+    @Override
+    protected boolean onFailure(Exception e) {
+      if (!BackendConfig.INSTANCE.isInvalidateMetadataOnEventProcessFailureEnabled()
+          || !canInvalidateTable(e)) {
+        return false;
+      }
+      errorLog(
+          "Invalidating tables in transaction due to exception during event processing",
+          e);
+      Set<TableName> tableNames =
+          tableWriteIds_.stream()
+              .map(writeId -> new TableName(writeId.getDbName(), writeId.getTblName()))
+              .collect(Collectors.toSet());
+      for (TableName tableName : tableNames) {
+        errorLog("Invalidate table {}.{}", tableName.getDb(), tableName.getTbl());
+        catalog_.invalidateTableIfExists(tableName.getDb(), tableName.getTbl());
+      }
+      return true;
+    }
+
     private void addCommittedWriteIdsToTables(Set<TableWriteId> tableWriteIds)
         throws CatalogException {
       for (TableWriteId tableWriteId: tableWriteIds) {
@@ -893,7 +945,7 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
           }
         } else {
           catalog_.reloadTableIfExists(entry.getKey().getDb(), entry.getKey().getTbl(),
-               "CommitTxnEvent", getEventId(), /*isSkipFileMetadataReload*/false);
+              "CommitTxnEvent", getEventId(), /*isSkipFileMetadataReload*/false);
         }
       }
     }
@@ -952,4 +1004,82 @@ public class MetastoreShim extends Hive3MetastoreShimBase {
       tableInfo.append(metaDataTable.renderTable(isOutputPadded));
     }
   }
+
+  /**
+   * Wrapper around IMetaStoreClient.createDataConnector().
+   */
+  public static void createDataSource(IMetaStoreClient client, DataSource dataSource)
+      throws InvalidObjectException, AlreadyExistsException, MetaException, TException {
+    DataConnector connector = MetastoreShim.dataSourceToDataConnector(dataSource);
+    Preconditions.checkNotNull(connector);
+    client.createDataConnector(connector);
+  }
+
+  /**
+   * Wrapper around IMetaStoreClient.dropDataConnector().
+   */
+  public static void dropDataSource(IMetaStoreClient client, String name,
+      boolean ifExists) throws NoSuchObjectException, InvalidOperationException,
+          MetaException, TException {
+    // Set 'checkReferences' as false, e.g. drop DataSource object without checking its
+    // reference since the properties of DataSource object are copied to the table
+    // property of the referenced DataSource tables.
+    client.dropDataConnector(
+        name, /* ifNotExists */ !ifExists, /* checkReferences */ false);
+  }
+
+  /**
+   * Wrapper around IMetaStoreClient.getAllDataConnectorNames() and
+   * IMetaStoreClient.getDataConnector().
+   */
+  public static Map<String, DataSource> loadAllDataSources(IMetaStoreClient client)
+      throws MetaException, TException {
+    Map<String, DataSource> newDataSrcs = new HashMap<String, DataSource>();
+    // Load DataSource objects from HMS DataConnector objects.
+    List<String> allConnectorNames = client.getAllDataConnectorNames();
+    for (String connectorName: allConnectorNames) {
+      DataConnector connector = client.getDataConnector(connectorName);
+      if (connector != null) {
+        DataSource dataSrc = MetastoreShim.dataConnectorToDataSource(connector);
+        if (dataSrc != null) newDataSrcs.put(connectorName, dataSrc);
+      }
+    }
+    return newDataSrcs;
+  }
+
+  /**
+   * Convert DataSource object to DataConnector object.
+   */
+  private static DataConnector dataSourceToDataConnector(DataSource dataSource) {
+    DataConnector connector = new DataConnector(
+        dataSource.getName(), HMS_DATA_CONNECTOR_TYPE, dataSource.getLocation());
+    connector.setDescription(HMS_DATA_CONNECTOR_DESC);
+    connector.putToParameters(
+        HMS_DATA_CONNECTOR_PARAM_KEY_CLASS_NAME, dataSource.getClassName());
+    connector.putToParameters(
+        HMS_DATA_CONNECTOR_PARAM_KEY_API_VERSION, dataSource.getApiVersion());
+    return connector;
+  }
+
+  /**
+   * Convert DataConnector object to DataSource object.
+   */
+  private static DataSource dataConnectorToDataSource(DataConnector connector) {
+    if (!connector.isSetName() || !connector.isSetType()
+        || !connector.isSetDescription() || connector.getParametersSize() == 0
+        || !connector.getType().equalsIgnoreCase(HMS_DATA_CONNECTOR_TYPE)) {
+      return null;
+    }
+    String name = connector.getName();
+    String location = connector.isSetUrl() ? connector.getUrl() : "";
+    String className =
+        connector.getParameters().get(HMS_DATA_CONNECTOR_PARAM_KEY_CLASS_NAME);
+    String apiVersion =
+        connector.getParameters().get(HMS_DATA_CONNECTOR_PARAM_KEY_API_VERSION);
+    if (!Strings.isNullOrEmpty(name) && location != null &&
+        !Strings.isNullOrEmpty(className) && !Strings.isNullOrEmpty(apiVersion)) {
+      return new DataSource(name, location, className, apiVersion);
+    }
+    return null;
+  }
 }
