@@ -95,6 +95,7 @@ public class AnalysisContext {
     public boolean isQueryStmt() { return stmt_ instanceof QueryStmt; }
     public boolean isSetOperationStmt() { return stmt_ instanceof SetOperationStmt; }
     public boolean isInsertStmt() { return stmt_ instanceof InsertStmt; }
+    public boolean isOptimizeStmt() { return stmt_ instanceof OptimizeStmt; }
     public boolean isDropDbStmt() { return stmt_ instanceof DropDbStmt; }
     public boolean isDropTableOrViewStmt() {
       return stmt_ instanceof DropTableOrViewStmt;
@@ -118,6 +119,10 @@ public class AnalysisContext {
     public boolean isUseStmt() { return stmt_ instanceof UseStmt; }
     public boolean isSetStmt() { return stmt_ instanceof SetStmt; }
     public boolean isShowTablesStmt() { return stmt_ instanceof ShowTablesStmt; }
+    public boolean isShowMetadataTablesStmt() {
+      return stmt_ instanceof ShowMetadataTablesStmt;
+    }
+    public boolean isShowViewsStmt() { return stmt_ instanceof ShowViewsStmt; }
     public boolean isDescribeHistoryStmt() {
       return stmt_ instanceof DescribeHistoryStmt;
     }
@@ -175,11 +180,11 @@ public class AnalysisContext {
     }
 
     private boolean isViewMetadataStmt() {
-      return isShowFilesStmt() || isShowTablesStmt() || isShowDbsStmt() ||
-          isShowFunctionsStmt() || isShowRolesStmt() || isShowGrantPrincipalStmt() ||
-          isShowCreateTableStmt() || isShowDataSrcsStmt() || isShowStatsStmt() ||
-          isDescribeTableStmt() || isDescribeDbStmt() || isShowCreateFunctionStmt() ||
-          isDescribeHistoryStmt();
+      return isShowFilesStmt() || isShowTablesStmt() || isShowMetadataTablesStmt() ||
+          isShowViewsStmt() || isShowDbsStmt() || isShowFunctionsStmt() ||
+          isShowRolesStmt() || isShowGrantPrincipalStmt() || isShowCreateTableStmt() ||
+          isShowDataSrcsStmt() || isShowStatsStmt() || isDescribeTableStmt() ||
+          isDescribeDbStmt() || isShowCreateFunctionStmt() || isDescribeHistoryStmt();
     }
 
     private boolean isGrantRevokeStmt() {
@@ -187,7 +192,7 @@ public class AnalysisContext {
     }
 
     public boolean isDmlStmt() {
-      return isInsertStmt() || isUpdateStmt() || isDeleteStmt();
+      return isInsertStmt() || isUpdateStmt() || isDeleteStmt() || isOptimizeStmt();
     }
 
     /**
@@ -197,7 +202,7 @@ public class AnalysisContext {
     public boolean isHierarchicalAuthStmt() {
       return isQueryStmt() || isInsertStmt() || isUpdateStmt() || isDeleteStmt()
           || isCreateTableAsSelectStmt() || isCreateViewStmt() || isAlterViewStmt()
-          || isTestCaseStmt();
+          || isOptimizeStmt() || isTestCaseStmt();
     }
 
     /**
@@ -209,7 +214,8 @@ public class AnalysisContext {
      */
     public boolean isSingleColumnPrivStmt() {
       return isDescribeTableStmt() || isResetMetadataStmt() || isUseStmt()
-          || isShowTablesStmt() || isAlterTableStmt() || isShowFunctionsStmt();
+          || isShowTablesStmt() || isShowMetadataTablesStmt() || isShowViewsStmt()
+          || isAlterTableStmt() || isShowFunctionsStmt();
     }
 
     public boolean isConvertTableToIcebergStmt() {
@@ -296,6 +302,11 @@ public class AnalysisContext {
       return (QueryStmt) stmt_;
     }
 
+    public OptimizeStmt getOptimizeStmt() {
+      Preconditions.checkState(isOptimizeStmt());
+      return (OptimizeStmt) stmt_;
+    }
+
     public InsertStmt getInsertStmt() {
       if (isCreateTableAsSelectStmt()) {
         return getCreateTableAsSelectStmt().getInsertStmt();
@@ -320,6 +331,16 @@ public class AnalysisContext {
       return (ShowTablesStmt) stmt_;
     }
 
+    public ShowMetadataTablesStmt getShowMetadataTablesStmt() {
+      Preconditions.checkState(isShowMetadataTablesStmt());
+      return (ShowMetadataTablesStmt) stmt_;
+    }
+
+    public ShowViewsStmt getShowViewsStmt() {
+      Preconditions.checkState(isShowViewsStmt());
+      return (ShowViewsStmt) stmt_;
+    }
+
     public ShowDbsStmt getShowDbsStmt() {
       Preconditions.checkState(isShowDbsStmt());
       return (ShowDbsStmt) stmt_;
@@ -407,7 +428,7 @@ public class AnalysisContext {
     }
     public boolean requiresExprRewrite() {
       return isQueryStmt() || isInsertStmt() || isCreateTableAsSelectStmt()
-          || isUpdateStmt() || isDeleteStmt();
+          || isUpdateStmt() || isDeleteStmt() || isOptimizeStmt();
     }
     public boolean requiresSetOperationRewrite() {
       return analyzer_.containsSetOperation() && !isCreateViewStmt() && !isAlterViewStmt()
@@ -631,10 +652,9 @@ public class AnalysisContext {
       analysisResult_.stmt_.analyze(analysisResult_.analyzer_);
       analysisResult_.analyzer_.setEnablePrivChecks(true); // Always restore
     } catch (AnalysisException e) {
-      LOG.error(String.format("Error analyzing the rewritten query.\n" +
-          "Original SQL: %s\nRewritten SQL: %s", analysisResult_.stmt_.toSql(),
-          analysisResult_.stmt_.toSql(REWRITTEN)), e);
-      throw e;
+      logRewriteErrorNoThrow(analysisResult_.stmt_, e);
+      throw new AnalysisException("An error occurred after query rewrite: " +
+          e.getMessage(), e);
     }
     // Restore the original result types and column labels.
     analysisResult_.stmt_.castResultExprs(origResultTypes);
@@ -645,6 +665,18 @@ public class AnalysisContext {
     if (isExplain) analysisResult_.stmt_.setIsExplain();
   }
 
+  private void logRewriteErrorNoThrow(StatementBase stmt,
+      AnalysisException analysisException) {
+    try {
+      LOG.error(String.format("Error analyzing the rewritten query.\n" +
+          "Original SQL: %s\nRewritten SQL: %s", stmt.toSql(),
+          stmt.toSql(REWRITTEN)), analysisException);
+    } catch (Exception e) {
+      LOG.error("An exception occurred during printing out " +
+          "the rewritten SQL statement.", e);
+    }
+  }
+
   public Analyzer getAnalyzer() { return analysisResult_.getAnalyzer(); }
   public EventSequence getTimeline() { return timeline_; }
   // This should only be called after analyzeAndAuthorize().
