@@ -25,6 +25,7 @@ import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
 import java.util.ListIterator;
+import java.util.Objects;
 import java.util.Optional;
 import java.util.Set;
 
@@ -39,6 +40,7 @@ import org.apache.impala.catalog.TypeCompatibility;
 import org.apache.impala.common.AnalysisException;
 import org.apache.impala.common.InternalException;
 import org.apache.impala.common.SqlCastException;
+import org.apache.impala.common.ThriftSerializationCtx;
 import org.apache.impala.common.TreeNode;
 import org.apache.impala.rewrite.ExprRewriter;
 import org.apache.impala.service.FeSupport;
@@ -861,23 +863,29 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
 
   protected String toSqlImpl() { return toSqlImpl(DEFAULT); };
 
-  // Convert this expr, including all children, to its Thrift representation.
+  // For locations that don't need to handle tuple caching yet, keep the
+  // old signature with a default ThriftSerializationCtx.
   public TExpr treeToThrift() {
+    return treeToThrift(new ThriftSerializationCtx());
+  }
+
+  // Convert this expr, including all children, to its Thrift representation.
+  public TExpr treeToThrift(ThriftSerializationCtx serialCtx) {
     if (type_.isNull()) {
       // Hack to ensure BE never sees TYPE_NULL. If an expr makes it this far without
       // being cast to a non-NULL type, the type doesn't matter and we can cast it
       // arbitrarily.
       Preconditions.checkState(IS_NULL_LITERAL.apply(this) ||
           this instanceof SlotRef);
-      return NullLiteral.create(ScalarType.BOOLEAN).treeToThrift();
+      return NullLiteral.create(ScalarType.BOOLEAN).treeToThrift(serialCtx);
     }
     TExpr result = new TExpr();
-    treeToThriftHelper(result);
+    treeToThriftHelper(result, serialCtx);
     return result;
   }
 
   // Append a flattened version of this expr, including all children, to 'container'.
-  protected void treeToThriftHelper(TExpr container) {
+  protected void treeToThriftHelper(TExpr container, ThriftSerializationCtx serialCtx) {
     Preconditions.checkState(isAnalyzed_,
         "Must be analyzed before serializing to thrift. %s", this);
     Preconditions.checkState(!type_.isWildcardDecimal());
@@ -894,10 +902,10 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
       msg.setFn(thriftFn);
       if (fn_.hasVarArgs()) msg.setVararg_start_idx(fn_.getNumArgs() - 1);
     }
-    toThrift(msg);
+    toThrift(msg, serialCtx);
     container.addToNodes(msg);
     for (Expr child: children_) {
-      child.treeToThriftHelper(container);
+      child.treeToThriftHelper(container, serialCtx);
     }
   }
 
@@ -905,6 +913,14 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
   // msg.op as well as the expr-specific field.
   protected abstract void toThrift(TExprNode msg);
 
+  // Exprs should override this signature of toThrift() if they need access to
+  // the ThriftSerializationContext. That is necessary if the Expr is non-deterministic
+  // or uses SlotIds/TupleIds. Everything else can simply keep using the old signature
+  // for toThrift().
+  protected void toThrift(TExprNode msg, ThriftSerializationCtx serialCtx) {
+    toThrift(msg);
+  }
+
   /**
    * Returns the product of the given exprs' number of distinct values or -1 if any of
    * the exprs have an invalid number of distinct values. Uses saturating arithmetic,
@@ -924,14 +940,19 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
     return numDistinctValues;
   }
 
-  public static List<TExpr> treesToThrift(List<? extends Expr> exprs) {
+  public static List<TExpr> treesToThrift(List<? extends Expr> exprs,
+      ThriftSerializationCtx serialCtx) {
     List<TExpr> result = new ArrayList<>();
     for (Expr expr: exprs) {
-      result.add(expr.treeToThrift());
+      result.add(expr.treeToThrift(serialCtx));
     }
     return result;
   }
 
+  public static List<TExpr> treesToThrift(List<? extends Expr> exprs) {
+    return treesToThrift(exprs, new ThriftSerializationCtx());
+  }
+
   public boolean isAggregate() {
     return IS_AGGREGATE.apply(this);
   }
@@ -1008,6 +1029,22 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
     return obj instanceof Expr && matches((Expr) obj, SlotRef.SLOTREF_EQ_CMP);
   }
 
+  /**
+   * Local hash code that ignores children.
+   */
+  protected int localHash() {
+    return Objects.hash(getClass(), fn_);
+  }
+
+  /**
+   * Returns a hash code based on the same keys used for equals.
+   */
+  @Override
+  public int hashCode() {
+    // CastExpr and SlotRef overload hashCode rather than mirroring 'matches'.
+    return Objects.hash(localHash(), children_);
+  }
+
   /**
    * Return true if l1[i].equals(l2[i]) for all i.
    */
@@ -1049,16 +1086,6 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
     return result;
   }
 
-  @Override
-  public int hashCode() {
-    if (id_ == null) {
-      throw new UnsupportedOperationException(
-          "Expr.hashCode() is not implemented in " + this.getClass().getName());
-    } else {
-      return id_.asInt();
-    }
-  }
-
   /**
    * Gather conjuncts from this expr and return them in a list.
    * A conjunct is an expr that returns a boolean, e.g., Predicates, function calls,
@@ -1851,9 +1878,11 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
    * Analyzes and evaluates expression to an integral value, returned as a long.
    * Throws if the expression cannot be evaluated or if the value evaluates to null.
    * The 'name' parameter is used in exception messages, e.g. "LIMIT expression
-   * evaluates to NULL".
+   * evaluates to NULL". If 'acceptDate' is true, treat Date expression as integral
+   * expression as well.
    */
-  public long evalToInteger(Analyzer analyzer, String name) throws AnalysisException {
+  public long evalToInteger(Analyzer analyzer, String name, boolean acceptDate)
+      throws AnalysisException {
     // Check for slotrefs and subqueries before analysis so we can provide a more
     // helpful error message.
     if (contains(SlotRef.class) || contains(Subquery.class)) {
@@ -1865,7 +1894,7 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
       throw new AnalysisException(name + " expression must be a constant expression: " +
           toSql());
     }
-    if (!getType().isIntegerType()) {
+    if (!getType().isIntegerType() && !(acceptDate && getType().isDate())) {
       throw new AnalysisException(name + " expression must be an integer type but is '" +
           getType() + "': " + toSql());
     }
@@ -1875,6 +1904,16 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
     } catch (InternalException e) {
       throw new AnalysisException("Failed to evaluate expr: " + toSql(), e);
     }
+
+    try {
+      return evalToInteger(val, acceptDate);
+    } catch (AnalysisException e) {
+      throw new AnalysisException(name + " expression evaluates to NULL: " + toSql());
+    }
+  }
+
+  public static long evalToInteger(TColumnValue val, boolean acceptDate)
+      throws AnalysisException {
     long value;
     if (val.isSetLong_val()) {
       value = val.getLong_val();
@@ -1884,12 +1923,18 @@ abstract public class Expr extends TreeNode<Expr> implements ParseNode, Cloneabl
       value = val.getShort_val();
     } else if (val.isSetByte_val()) {
       value = val.getByte_val();
+    } else if (acceptDate && val.isSetDate_val()) {
+      value = val.getDate_val();
     } else {
-      throw new AnalysisException(name + " expression evaluates to NULL: " + toSql());
+      throw new AnalysisException("TColumnValue evaluates to NULL: " + val);
     }
     return value;
   }
 
+  public long evalToInteger(Analyzer analyzer, String name) throws AnalysisException {
+    return evalToInteger(analyzer, name, false);
+  }
+
   /**
    * Analyzes and evaluates expression to a non-negative integral value, returned as a
    * long. Throws if the expression cannot be evaluated, if the value evaluates to null,
