@@ -84,15 +84,13 @@ public class FromClause extends StmtNode implements Iterable<TableRef> {
     boolean hasJoiningUnnest = false;
     for (int i = 0; i < tableRefs_.size(); ++i) {
       TableRef tblRef = tableRefs_.get(i);
-      if (IcebergMetadataTable.isIcebergMetadataTable(tblRef.getPath())) {
-        analyzer.addMetadataVirtualTable(tblRef.getPath());
-      }
       tblRef = analyzer.resolveTableRef(tblRef);
       tableRefs_.set(i, Preconditions.checkNotNull(tblRef));
       tblRef.setLeftTblRef(leftTblRef);
       tblRef.analyze(analyzer);
       leftTblRef = tblRef;
       if (tblRef instanceof CollectionTableRef) {
+        checkIcebergCollectionSupport((CollectionTableRef)tblRef);
         checkTopLevelComplexAcidScan(analyzer, (CollectionTableRef)tblRef);
         if (firstZippingUnnestRef != null && tblRef.isZippingUnnest() &&
             firstZippingUnnestRef.getResolvedPath().getRootTable() !=
@@ -161,6 +159,19 @@ public class FromClause extends StmtNode implements Iterable<TableRef> {
     analyzer.setHasTopLevelAcidCollectionTableRef();
   }
 
+  private void checkIcebergCollectionSupport(CollectionTableRef tblRef)
+      throws AnalysisException {
+    Preconditions.checkNotNull(tblRef);
+    Preconditions.checkNotNull(tblRef.getDesc());
+    Preconditions.checkNotNull(tblRef.getDesc().getPath());
+    Preconditions.checkNotNull(tblRef.getDesc().getPath().getRootTable());
+    // IMPALA-12853: Collection types in FROM clause for Iceberg Metadata Tables
+    if (tblRef.getDesc().getPath().getRootTable() instanceof IcebergMetadataTable) {
+      throw new AnalysisException("Querying collection types (ARRAY/MAP) in FROM " +
+          "clause is not supported for Iceberg Metadata tables.");
+    }
+  }
+
   @Override
   public FromClause clone() {
     List<TableRef> clone = new ArrayList<>();
