@@ -20,6 +20,7 @@ package org.apache.impala.analysis;
 import java.util.Arrays;
 import java.util.List;
 import java.util.HashSet;
+import java.util.Objects;
 import java.util.Optional;
 import java.util.Set;
 
@@ -241,7 +242,7 @@ public class FunctionCallExpr extends Expr {
   }
 
   @Override
-  public boolean localEquals(Expr that) {
+  protected boolean localEquals(Expr that) {
     if (!super.localEquals(that)) return false;
     FunctionCallExpr o = (FunctionCallExpr)that;
     return fnName_.equals(o.fnName_) &&
@@ -250,6 +251,12 @@ public class FunctionCallExpr extends Expr {
         params_.isStar() == o.params_.isStar();
   }
 
+  @Override
+  protected int localHash() {
+    return Objects.hash(super.localHash(),
+        fnName_, params_.isDistinct(), params_.isIgnoreNulls(), params_.isStar());
+  }
+
   @Override
   public String toSqlImpl(ToSqlOptions options) {
     if (label_ != null) return label_;
