@@ -18,6 +18,7 @@
 package org.apache.impala.analysis;
 
 import java.util.List;
+import java.util.Objects;
 
 import com.google.common.collect.Lists;
 
@@ -100,6 +101,11 @@ public class OrderByElement {
     return expr_.equals(o.expr_) && isAsc_ == o.isAsc_ && nullsFirstEqual;
   }
 
+  @Override
+  public int hashCode() {
+    return Objects.hash(getClass(), nullsFirstParam_, expr_, isAsc_);
+  }
+
   @Override
   public OrderByElement clone() { return new OrderByElement(this); }
 
