@@ -29,6 +29,7 @@ import org.apache.impala.catalog.StructField;
 import org.apache.impala.catalog.StructType;
 import org.apache.impala.catalog.Type;
 import org.apache.impala.catalog.VirtualColumn;
+import org.apache.impala.catalog.VirtualTable;
 import org.apache.impala.catalog.iceberg.IcebergMetadataTable;
 import org.apache.impala.thrift.TVirtualColumnType;
 import org.apache.impala.util.AcidUtils;
@@ -349,7 +350,7 @@ public class Path {
       String dbName = (tblNameIdx == 0) ? sessionDb : path.get(0);
       String tblName = path.get(tblNameIdx);
       String vTblName = null;
-      if (IcebergMetadataTable.isIcebergMetadataTable(path)) {
+      if (IcebergMetadataTable.canBeIcebergMetadataTable(path)) {
         vTblName = path.get(2);
       }
       result.add(new TableName(dbName, tblName, vTblName));
@@ -447,11 +448,27 @@ public class Path {
     } else {
       result.add(rootTable_.getDb().getName());
       result.add(rootTable_.getName());
+      if (rootTable_ instanceof VirtualTable) {
+        result.add(((IcebergMetadataTable)rootTable_).getMetadataTableName());
+      }
     }
     result.addAll(rawPath_);
     return result;
   }
 
+  /**
+   * Returns whether the given path belongs to a (possibly nested) field from an Iceberg
+   * metadata table.
+   */
+  public boolean comesFromIcebergMetadataTable() {
+    Preconditions.checkState(rootTable_ != null || rootDesc_ != null);
+    if (rootDesc_ != null) {
+      return rootDesc_.getTable() instanceof IcebergMetadataTable;
+    } else {
+      return rootTable_ instanceof IcebergMetadataTable;
+    }
+  }
+
   /**
    * Returns the absolute explicit path starting from the fully-qualified table name.
    * The goal is produce a canonical non-ambiguous path that can be used as an
