@@ -562,6 +562,12 @@ public class SelectStmt extends QueryStmt {
       }
 
       for (Expr expr: resultExprs_) {
+        if (selectList_.isDistinct() && expr.getType().isComplexType()) {
+          throw new AnalysisException("Complex types are not supported " +
+              "in SELECT DISTINCT clauses. Expr: '" + expr.toSql() + "', type: '"
+              + expr.getType().toSql() + "'.");
+        }
+
         if (expr.getType().isArrayType()) {
           ArrayType arrayType = (ArrayType) expr.getType();
           if (!arrayType.getItemType().isSupported()) {
@@ -893,6 +899,7 @@ public class SelectStmt extends QueryStmt {
       Path starExpandedPath = Path.createRelPath(resolvedRootPath, relRawPath);
       Preconditions.checkState(starExpandedPath.resolve());
       if (starExpandedPath.destType().isComplexType() &&
+          !starExpandedPath.comesFromIcebergMetadataTable() &&
           !analyzer_.getQueryCtx().client_request.query_options.expand_complex_types) {
         return;
       }
@@ -1459,13 +1466,15 @@ public class SelectStmt extends QueryStmt {
           Analyzer.DB_DOES_NOT_EXIST_ERROR_MSG + tableName.getDb(), e);
     }
     if (!(table instanceof FeIcebergTable)) return;
-
+    if (analyzer_.getQueryOptions().iceberg_disable_count_star_optimization) {
+      return;
+    }
     analyzer_.checkStmtExprLimit();
-    Table iceTable = ((FeIcebergTable) table).getIcebergApiTable();
+    FeIcebergTable iceTable = ((FeIcebergTable) table);
     if (Utils.hasDeleteFiles(iceTable, tableRef.getTimeTravelSpec())) {
-      optimizePlainCountStarQueryV2(tableRef, (FeIcebergTable)table);
+      optimizePlainCountStarQueryV2(tableRef, iceTable);
     } else {
-      optimizePlainCountStarQueryV1(tableRef, iceTable);
+      optimizePlainCountStarQueryV1(tableRef, iceTable.getIcebergApiTable());
     }
   }
 
