@@ -29,7 +29,9 @@ import org.apache.impala.catalog.StructType;
 import org.apache.impala.catalog.TableLoadingException;
 import org.apache.impala.catalog.Type;
 import org.apache.impala.catalog.TypeCompatibility;
+import org.apache.impala.catalog.iceberg.IcebergMetadataTable;
 import org.apache.impala.common.AnalysisException;
+import org.apache.impala.common.ThriftSerializationCtx;
 import org.apache.impala.common.UnsupportedFeatureException;
 import org.apache.impala.thrift.TExprNode;
 import org.apache.impala.thrift.TExprNodeType;
@@ -37,7 +39,6 @@ import org.apache.impala.thrift.TSlotRef;
 
 import com.google.common.base.Joiner;
 import com.google.common.base.MoreObjects;
-import com.google.common.base.Objects;
 import com.google.common.base.Preconditions;
 
 public class SlotRef extends Expr {
@@ -213,7 +214,9 @@ public class SlotRef extends Expr {
         throw new AnalysisException("Unsupported type '"
             + fieldType.toSql() + "' in '" + toSql() + "'.");
       }
-      if (fieldType.isBinary()) {
+      if (fieldType.isBinary() && !desc_.getPath().comesFromIcebergMetadataTable()) {
+        // We allow BINARY fields in collections from Iceberg metadata tables but NULL
+        // them out.
         throw new AnalysisException("Struct containing a BINARY type is not " +
             "allowed in the select list (IMPALA-11491).");
       }
@@ -226,29 +229,37 @@ public class SlotRef extends Expr {
     if (resolvedPath_ != null) {
       FeTable rootTable = resolvedPath_.getRootTable();
       if (rootTable != null) {
-        if (!(rootTable instanceof FeFsTable)) {
-          throw new AnalysisException(
-              String.format("%s is not supported when querying STRUCT type %s", rootTable,
-                  type_.toSql()));
-        }
-        FeFsTable feTable = (FeFsTable) rootTable;
-        for (HdfsFileFormat format : feTable.getFileFormats()) {
-          if (!formatSupportsQueryingStruct(format)) {
-            throw new AnalysisException("Querying STRUCT is only supported for ORC and "
-                + "Parquet file formats.");
-          }
+        checkTableTypeSupportsStruct(rootTable);
+        if (rootTable instanceof FeFsTable) {
+          checkFileFormatSupportsStruct((FeFsTable)rootTable);
         }
       }
     }
   }
 
-  // Returns true if the given HdfsFileFormat supports querying STRUCT types. Iceberg
-  // tables also have ICEBERG as HdfsFileFormat. We can return TRUE in case of Iceberg
-  // because the data file formats in the Iceberg table will be also tested separately.
-  private static boolean formatSupportsQueryingStruct(HdfsFileFormat format) {
-    return format == HdfsFileFormat.PARQUET ||
+  private void checkTableTypeSupportsStruct(FeTable feTable) throws AnalysisException {
+    if (!(feTable instanceof FeFsTable) &&
+        !(feTable instanceof IcebergMetadataTable)) {
+      throw new AnalysisException(
+          String.format("%s is not supported when querying STRUCT type %s",
+              feTable, type_.toSql()));
+    }
+  }
+
+  // Throws exception if the given HdfsFileFormat does not support querying STRUCT types.
+  // Iceberg tables also have ICEBERG as HdfsFileFormat. In case of Iceberg there is no
+  // need to throw exception because the data file formats in the Iceberg table will be
+  // also tested separately.
+  private void checkFileFormatSupportsStruct(FeFsTable feFsTable)
+      throws AnalysisException {
+    for (HdfsFileFormat format : feFsTable.getFileFormats()) {
+      if (! (format == HdfsFileFormat.PARQUET ||
            format == HdfsFileFormat.ORC ||
-           format == HdfsFileFormat.ICEBERG;
+           format == HdfsFileFormat.ICEBERG)) {
+        throw new AnalysisException("Querying STRUCT is only supported for ORC and "
+            + "Parquet file formats.");
+      }
+    }
   }
 
   // Assumes this 'SlotRef' is a struct and that desc_.itemTupleDesc_ has already been
@@ -315,8 +326,13 @@ public class SlotRef extends Expr {
 
   @Override
   protected void toThrift(TExprNode msg) {
+    Preconditions.checkState(false, "Unexpected use of old toThrift() signature");
+  }
+
+  @Override
+  protected void toThrift(TExprNode msg, ThriftSerializationCtx serialCtx) {
     msg.node_type = TExprNodeType.SLOT_REF;
-    msg.slot_ref = new TSlotRef(desc_.getId().asInt());
+    msg.slot_ref = new TSlotRef(serialCtx.translateSlotId(desc_.getId()).asInt());
     // we shouldn't be sending exprs over non-materialized slots
     Preconditions.checkState(desc_.isMaterialized(), String.format(
         "Illegal reference to non-materialized slot: tid=%s sid=%s",
@@ -338,14 +354,19 @@ public class SlotRef extends Expr {
     return toStrHelper.toString();
   }
 
+  /**
+   * Overrides hashCode to ignore children, as done in
+   * {@link Expr#matches(Expr, Comparator)}.
+   */
   @Override
   public int hashCode() {
     if (desc_ != null) return desc_.getId().hashCode();
-    return Objects.hashCode(Joiner.on('.').join(rawPath_).toLowerCase());
+    if (label_ != null) return label_.toLowerCase().hashCode();
+    return super.localHash();
   }
 
   @Override
-  public boolean localEquals(Expr that) {
+  protected boolean localEquals(Expr that) {
     if (!super.localEquals(that)) return false;
     SlotRef other = (SlotRef) that;
     // check slot ids first; if they're both set we only need to compare those
