@@ -20,6 +20,7 @@ package org.apache.impala.analysis;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
+import java.util.Set;
 
 import org.apache.impala.analysis.AnalysisContext.AnalysisResult;
 import org.apache.impala.analysis.SetOperationStmt.SetOperand;
@@ -799,6 +800,9 @@ public class StmtRewriter {
         stmt.whereClause_ =
             CompoundPredicate.createConjunction(onClausePredicate, stmt.whereClause_);
         inlineView.setJoinOp(JoinOperator.CROSS_JOIN);
+        // Indicate this inline view returns at most one value through a
+        // non-correlated scalar subquery.
+        if (isScalarSubquery) inlineView.setIsNonCorrelatedScalarSubquery();
         // Indicate that the CROSS JOIN may add a new visible tuple to stmt's
         // select list (if the latter contains an unqualified star item '*')
         return true;
@@ -1824,9 +1828,12 @@ public class StmtRewriter {
       newCollPath.add(0, alias);
       // Remove the alias of the old collection ref from the analyzer. We need to add
       // the new collection ref with the same old alias.
-      analyzer.removeAlias(collTblRef.getUniqueAlias());
-      TableRef newCollTblRef =
-          TableRef.newTableRef(analyzer, newCollPath, collTblRef.getUniqueAlias());
+      String[] old_aliases = collTblRef.getAliases();
+      for (String old_alias: old_aliases) {
+        analyzer.removeAlias(old_alias);
+      }
+      TableRef newCollTblRef = TableRef.newTableRef(
+          analyzer, newCollPath, old_aliases[old_aliases.length - 1]);
       // Set JOIN attributes. Please note that we cannot use TableRef.setJoinAttrs()
       // because we only want to copy the ON clause and the plan hints. The col names
       // in USING have been already converted to an ON clause. We let the analyzer/
@@ -1850,4 +1857,21 @@ public class StmtRewriter {
       return rawTblPath;
     }
   }
+
+  /**
+   * The purpose of this rewriter is to add CollectionTableRefs to the FROM clause of
+   * select queries where unnest() is in the select list. One example of such a query:
+   * SELECT unnest(arr1), unnest(arr2) FROM complextypes_arrays;
+   * In the above example there will be two CollectionTableRefs added to the FROM clause,
+   * one for each unnested array.
+   */
+  static class ZippingUnnestRewriter extends StmtRewriter {
+    @Override
+    protected void rewriteSelectStmtHook(SelectStmt stmt, Analyzer analyzer)
+        throws AnalysisException {
+      Set<CollectionTableRef> unnestTableRefs = analyzer.getTableRefsFromUnnestExpr();
+      Preconditions.checkState(stmt.getResultExprs().size() >= unnestTableRefs.size());
+      for (TableRef tblRef : unnestTableRefs) stmt.fromClause_.add(tblRef);
+    }
+  }
 }
