@@ -44,6 +44,7 @@ import org.apache.impala.compat.MetastoreShim;
 import org.apache.impala.thrift.TCatalogObject;
 import org.apache.impala.thrift.TFunction;
 import org.apache.impala.thrift.THdfsPartition;
+import org.apache.impala.thrift.TImpalaTableType;
 import org.apache.impala.thrift.TPartitionKeyValue;
 import org.apache.impala.thrift.TPrincipalType;
 import org.apache.impala.thrift.TTable;
@@ -52,6 +53,7 @@ import org.apache.impala.thrift.TUniqueId;
 import org.apache.impala.util.EventSequence;
 import org.apache.impala.util.PatternMatcher;
 
+import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Joiner;
 import com.google.common.base.Preconditions;
 
@@ -79,7 +81,7 @@ public abstract class Catalog implements AutoCloseable {
   public static final TUniqueId INITIAL_CATALOG_SERVICE_ID = new TUniqueId(0L, 0L);
   public static final String DEFAULT_DB = "default";
 
-  private final MetaStoreClientPool metaStoreClientPool_;
+  private MetaStoreClientPool metaStoreClientPool_;
 
   // Cache of authorization policy metadata. Populated from data retried from the
   // Sentry Service, if configured.
@@ -233,20 +235,30 @@ public abstract class Catalog implements AutoCloseable {
   }
 
   /**
-   * Returns all tables in 'dbName' that match 'matcher'.
+   * Returns all tables and views in 'dbName' that match 'matcher'.
+   */
+  public List<String> getTableNames(String dbName, PatternMatcher matcher)
+      throws DatabaseNotFoundException {
+    return getTableNames(dbName, matcher, /*tableTypes*/ Collections.emptySet());
+  }
+
+  /**
+   * Returns all tables of types specified in 'tableTypes' under the database 'dbName'
+   * that match 'matcher'.
    *
    * dbName must not be null.
    *
    * Table names are returned unqualified.
    */
-  public List<String> getTableNames(String dbName, PatternMatcher matcher)
+  public List<String> getTableNames(String dbName, PatternMatcher matcher,
+      Set<TImpalaTableType> tableTypes)
       throws DatabaseNotFoundException {
     Preconditions.checkNotNull(dbName);
     Db db = getDb(dbName);
     if (db == null) {
       throw new DatabaseNotFoundException("Database '" + dbName + "' not found");
     }
-    return filterStringsByPattern(db.getAllTableNames(), matcher);
+    return filterStringsByPattern(db.getAllTableNames(tableTypes), matcher);
   }
 
   /**
@@ -259,8 +271,7 @@ public abstract class Catalog implements AutoCloseable {
   }
 
   /**
-   * Adds a data source to the in-memory map of data sources. It is not
-   * persisted to the metastore.
+   * Adds a data source to the in-memory map of data sources.
    * @return true if this item was added or false if the existing value was preserved.
    */
   public boolean addDataSource(DataSource dataSource) {
@@ -392,6 +403,14 @@ public abstract class Catalog implements AutoCloseable {
   @Override
   public void close() { metaStoreClientPool_.close(); }
 
+  @VisibleForTesting
+  public MetaStoreClientPool getMetaStoreClientPool() { return metaStoreClientPool_; }
+
+  @VisibleForTesting
+  public void setMetaStoreClientPool(MetaStoreClientPool pool) {
+    metaStoreClientPool_ = pool;
+  }
+
   /**
    * Returns a managed meta store client from the client connection pool.
    */
@@ -402,10 +421,18 @@ public abstract class Catalog implements AutoCloseable {
    */
   public MetaStoreClient getMetaStoreClient(EventSequence timeline) {
     MetaStoreClient client = getMetaStoreClient();
-    if (timeline != null) timeline.markEvent("Got Metastore client");
+    timeline.markEvent("Got Metastore client");
     return client;
   }
 
+  public int getNumHmsClientsIdle() {
+    return metaStoreClientPool_.getNumHmsClientsIdle();
+  }
+
+  public int getNumHmsClientsInUse() {
+    return metaStoreClientPool_.getNumHmsClientsInUse();
+  }
+
   /**
    * Return all members of 'candidates' that match 'matcher'.
    * The results are sorted in String.CASE_INSENSITIVE_ORDER.
@@ -510,6 +537,23 @@ public abstract class Catalog implements AutoCloseable {
    */
   public TCatalogObject getTCatalogObject(TCatalogObject objectDesc)
       throws CatalogException {
+    return getTCatalogObject(objectDesc, false);
+  }
+
+  /**
+   * Gets the thrift representation of a catalog object, given the "object
+   * description". The object description is just a TCatalogObject with only the
+   * catalog object type and object name set.
+   * If the object is not found, a CatalogException is thrown.
+   * @param objectDesc the object description.
+   * @param isHumanReadable whether the request is for human readable purpose or not.
+   *                        If False, return full object. Otherwise, return smaller
+   *                        catalog object that omit some field.
+   * @return the requested catalog object.
+   * @throws CatalogException
+   */
+  public TCatalogObject getTCatalogObject(
+      TCatalogObject objectDesc, boolean isHumanReadable) throws CatalogException {
     TCatalogObject result = new TCatalogObject();
     switch (objectDesc.getType()) {
       case DATABASE: {
@@ -535,7 +579,8 @@ public abstract class Catalog implements AutoCloseable {
         try {
           result.setType(table.getCatalogObjectType());
           result.setCatalog_version(table.getCatalogVersion());
-          result.setTable(table.toThrift());
+          result.setTable(
+              isHumanReadable ? table.toHumanReadableThrift() : table.toThrift());
         } finally {
           table.releaseReadLock();
         }
