@@ -23,15 +23,19 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.concurrent.locks.ReentrantLock;
+import java.util.stream.Collectors;
 
 import org.apache.hadoop.hive.metastore.api.Database;
+import org.apache.hadoop.hive.metastore.api.PrincipalType;
 import org.apache.impala.analysis.ColumnDef;
 import org.apache.impala.analysis.KuduPartitionParam;
 import org.apache.impala.catalog.events.InFlightEvents;
 import org.apache.impala.common.ImpalaException;
 import org.apache.impala.common.ImpalaRuntimeException;
+import org.apache.impala.service.BackendConfig;
 import org.apache.impala.thrift.TBriefTableMeta;
 import org.apache.impala.thrift.TCatalogObject;
 import org.apache.impala.thrift.TCatalogObjectType;
@@ -41,6 +45,7 @@ import org.apache.impala.thrift.TFunctionBinaryType;
 import org.apache.impala.thrift.TFunctionCategory;
 import org.apache.impala.thrift.TGetPartialCatalogObjectRequest;
 import org.apache.impala.thrift.TGetPartialCatalogObjectResponse;
+import org.apache.impala.thrift.TImpalaTableType;
 import org.apache.impala.thrift.TPartialDbInfo;
 import org.apache.impala.util.FunctionUtils;
 import org.apache.impala.util.PatternMatcher;
@@ -78,6 +83,9 @@ public class Db extends CatalogObjectImpl implements FeDb {
 
   public static final String FUNCTION_INDEX_PREFIX = "impala_registered_function_";
 
+  // Name of the standard system DB. Also used by Hive MetaStore.
+  public static final String SYS = "sys";
+
   // Hive metastore imposes a limit of 4000 bytes on the key and value strings
   // in DB parameters map. We need ensure that this limit isn't crossed
   // while serializing functions to the metastore.
@@ -99,6 +107,9 @@ public class Db extends CatalogObjectImpl implements FeDb {
   private boolean isSystemDb_ = false;
 
   // tracks the in-flight metastore events for this db
+  // Also used as a monitor object to synchronize access to it to avoid blocking on table
+  // lock during self-event check. If both this and dbLock_ or catalog version lock are
+  // taken, inFlightEvents_ must be the last to avoid deadlock.
   private final InFlightEvents inFlightEvents_ = new InFlightEvents();
 
   // lock to make sure modifications to the Db object are atomically done along with
@@ -196,11 +207,23 @@ public class Db extends CatalogObjectImpl implements FeDb {
    */
   public void addTable(Table table) { tableCache_.add(table); }
 
-  /**
-   * Gets all table names in the table cache.
-   */
   @Override
   public List<String> getAllTableNames() {
+    return getAllTableNames(/*tableTypes*/ Collections.emptySet());
+  }
+
+  /**
+   * Gets all table names in the table cache whose corresponding tables are of a table
+   * type specified in 'tableTypes'. Returns all table names if 'tableTypes' is empty.
+   */
+  @Override
+  public List<String> getAllTableNames(Set<TImpalaTableType> tableTypes) {
+    if (!tableTypes.isEmpty()) {
+      return tableCache_.getValues().stream()
+          .filter(table -> tableTypes.contains(table.getTableType()))
+          .map(table -> table.getName())
+          .collect(Collectors.toList());
+    }
     return Lists.newArrayList(tableCache_.keySet());
   }
 
@@ -209,6 +232,11 @@ public class Db extends CatalogObjectImpl implements FeDb {
    */
   public List<Table> getTables() { return tableCache_.getValues(); }
 
+  /**
+   * Returns the number of tables in this db.
+   */
+  public int getNumTables() { return tableCache_.size(); }
+
   @Override
   public boolean containsTable(String tableName) {
     return tableCache_.contains(tableName.toLowerCase());
@@ -565,10 +593,9 @@ public class Db extends CatalogObjectImpl implements FeDb {
    * @return true if version was successfully removed, false if didn't exist
    */
   public boolean removeFromVersionsForInflightEvents(long versionNumber) {
-    Preconditions.checkState(dbLock_.isHeldByCurrentThread(),
-        "removeFromVersionsForInflightEvents called without getting the db lock for "
-            + getName() + " database.");
-    return inFlightEvents_.remove(false, versionNumber);
+    synchronized (inFlightEvents_) {
+      return inFlightEvents_.remove(false, versionNumber);
+    }
   }
 
   /**
@@ -578,21 +605,28 @@ public class Db extends CatalogObjectImpl implements FeDb {
    * given version and does not add it
    * @param versionNumber version number to add
    */
-  public void addToVersionsForInflightEvents(long versionNumber) {
+  public boolean addToVersionsForInflightEvents(long versionNumber) {
+    // The lock is not needed for thread safety, just verifying existing behavior.
     Preconditions.checkState(dbLock_.isHeldByCurrentThread(),
         "addToVersionsForInFlightEvents called without getting the db lock for "
             + getName() + " database.");
-    if (!inFlightEvents_.add(false, versionNumber)) {
+    boolean added = false;
+    synchronized (inFlightEvents_) {
+      added = inFlightEvents_.add(false, versionNumber);
+    }
+    if (!added) {
       LOG.warn(String.format("Could not add version %s to the list of in-flight "
           + "events. This could cause unnecessary database %s invalidation when the "
           + "event is processed", versionNumber, getName()));
     }
+    return added;
   }
 
   @Override // FeDb
   public String getOwnerUser() {
     org.apache.hadoop.hive.metastore.api.Database db = getMetaStoreDb();
-    return db == null ? null : db.getOwnerName();
+    return db == null ? null :
+        (db.getOwnerType() == PrincipalType.USER ? db.getOwnerName() : null);
   }
 
   /**
