@@ -23,7 +23,6 @@ import com.google.common.collect.Lists;
 import com.google.common.primitives.Ints;
 
 import java.io.IOException;
-import java.net.URI;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -54,6 +53,7 @@ import org.apache.iceberg.Snapshot;
 import org.apache.iceberg.SnapshotSummary;
 import org.apache.iceberg.Table;
 import org.apache.iceberg.TableProperties;
+import org.apache.iceberg.transforms.Transform;
 import org.apache.iceberg.util.PropertyUtil;
 import org.apache.iceberg.util.SnapshotUtil;
 import org.apache.impala.analysis.IcebergPartitionField;
@@ -61,15 +61,18 @@ import org.apache.impala.analysis.IcebergPartitionSpec;
 import org.apache.impala.analysis.LiteralExpr;
 import org.apache.impala.analysis.TimeTravelSpec;
 import org.apache.impala.analysis.TimeTravelSpec.Kind;
+import org.apache.impala.catalog.CatalogObject.ThriftObjectType;
 import org.apache.impala.catalog.HdfsPartition.FileBlock;
 import org.apache.impala.catalog.HdfsPartition.FileDescriptor;
 import org.apache.impala.catalog.iceberg.GroupedContentFiles;
 import org.apache.impala.common.AnalysisException;
 import org.apache.impala.common.FileSystemUtil;
+import org.apache.impala.common.ImpalaRuntimeException;
 import org.apache.impala.common.Pair;
 import org.apache.impala.common.PrintUtils;
 import org.apache.impala.common.Reference;
 import org.apache.impala.fb.FbFileBlock;
+import org.apache.impala.service.BackendConfig;
 import org.apache.impala.thrift.TColumn;
 import org.apache.impala.thrift.TCompressionCodec;
 import org.apache.impala.thrift.THdfsCompression;
@@ -85,9 +88,11 @@ import org.apache.impala.thrift.TNetworkAddress;
 import org.apache.impala.thrift.TResultSet;
 import org.apache.impala.thrift.TResultSetMetadata;
 import org.apache.impala.util.HdfsCachingUtil;
+import org.apache.impala.util.IcebergSchemaConverter;
 import org.apache.impala.util.IcebergUtil;
 import org.apache.impala.util.ListMap;
 import org.apache.impala.util.TResultRowBuilder;
+import org.apache.thrift.TException;
 
 import org.json.simple.JSONValue;
 import org.slf4j.Logger;
@@ -195,6 +200,11 @@ public interface FeIcebergTable extends FeFsTable {
     return getIcebergApiTable().schema();
   }
 
+  @Override
+  default List<String> getPrimaryKeyColumnNames() throws TException {
+    return Lists.newArrayList(getIcebergSchema().identifierFieldNames());
+  }
+
   @Override
   default boolean isCacheable() {
     return getFeFsTable().isCacheable();
@@ -329,7 +339,7 @@ public interface FeIcebergTable extends FeFsTable {
     return false;
   }
 
-  THdfsTable transfromToTHdfsTable(boolean updatePartitionFlag);
+  THdfsTable transformToTHdfsTable(boolean updatePartitionFlag, ThriftObjectType type);
 
   /**
    * Returns the current snapshot id of the Iceberg API table if it exists, otherwise
@@ -544,7 +554,7 @@ public interface FeIcebergTable extends FeFsTable {
      * @return a list of {@link org.apache.hadoop.hive.metastore.api.FieldSchema}
      */
     public static List<FieldSchema> getPartitionTransformKeys(FeIcebergTable table)
-        throws TableLoadingException {
+        throws ImpalaRuntimeException {
       Table icebergTable = table.getIcebergApiTable();
 
       if (icebergTable.specs().isEmpty()) {
@@ -552,7 +562,7 @@ public interface FeIcebergTable extends FeFsTable {
       }
 
       PartitionSpec latestSpec = icebergTable.spec();
-      HashMap<String, Integer> transformParams = IcebergUtil.getPartitionTransformParams(
+      Map<String, Integer> transformParams = IcebergUtil.getPartitionTransformParams(
           latestSpec);
       List<FieldSchema> fieldSchemaList = Lists.newArrayList();
       for (PartitionField field : latestSpec.fields()) {
@@ -645,6 +655,11 @@ public interface FeIcebergTable extends FeFsTable {
     }
 
     public static TIcebergTable getTIcebergTable(FeIcebergTable icebergTable) {
+      return getTIcebergTable(icebergTable, ThriftObjectType.FULL);
+    }
+
+    public static TIcebergTable getTIcebergTable(FeIcebergTable icebergTable,
+        ThriftObjectType type) {
       TIcebergTable tIcebergTable = new TIcebergTable();
       tIcebergTable.setTable_location(icebergTable.getIcebergTableLocation());
 
@@ -654,7 +669,9 @@ public interface FeIcebergTable extends FeFsTable {
       tIcebergTable.setDefault_partition_spec_id(
           icebergTable.getDefaultPartitionSpecId());
 
-      tIcebergTable.setContent_files(icebergTable.getContentFileStore().toThrift());
+      if (type == ThriftObjectType.FULL) {
+        tIcebergTable.setContent_files(icebergTable.getContentFileStore().toThrift());
+      }
 
       tIcebergTable.setCatalog_snapshot_id(icebergTable.snapshotId());
       tIcebergTable.setParquet_compression_codec(
@@ -752,7 +769,7 @@ public interface FeIcebergTable extends FeFsTable {
      */
     public static IcebergContentFileStore loadAllPartition(
         IcebergTable table, GroupedContentFiles icebergFiles)
-        throws IOException {
+        throws IOException, ImpalaRuntimeException {
       Map<String, HdfsPartition.FileDescriptor> hdfsFileDescMap = new HashMap<>();
       Collection<HdfsPartition> partitions =
           ((HdfsTable)table.getFeFsTable()).partitionMap_.values();
@@ -772,16 +789,13 @@ public interface FeIcebergTable extends FeFsTable {
         pathHashAndFd = getPathHashAndFd(dataFile, table, hdfsFileDescMap);
         fileStore.addDataFileWithDeletes(pathHashAndFd.first, pathHashAndFd.second);
       }
-      for (DeleteFile deleteFile : icebergFiles.deleteFiles) {
+      for (DeleteFile deleteFile : icebergFiles.positionDeleteFiles) {
         pathHashAndFd = getPathHashAndFd(deleteFile, table, hdfsFileDescMap);
-        if (deleteFile.content().equals(FileContent.POSITION_DELETES)) {
-          fileStore.addPositionDeleteFile(pathHashAndFd.first, pathHashAndFd.second);
-        } else if (deleteFile.content().equals(FileContent.EQUALITY_DELETES)) {
-          fileStore.addEqualityDeleteFile(pathHashAndFd.first, pathHashAndFd.second);
-        } else {
-          Preconditions.checkState(false,
-              "Delete file with unknown kind: " + deleteFile.path().toString());
-        }
+        fileStore.addPositionDeleteFile(pathHashAndFd.first, pathHashAndFd.second);
+      }
+      for (DeleteFile deleteFile : icebergFiles.equalityDeleteFiles) {
+        pathHashAndFd = getPathHashAndFd(deleteFile, table, hdfsFileDescMap);
+        fileStore.addEqualityDeleteFile(pathHashAndFd.first, pathHashAndFd.second);
       }
       return fileStore;
     }
@@ -880,23 +894,26 @@ public interface FeIcebergTable extends FeFsTable {
      * Get iceberg partition spec by iceberg table metadata
      */
     public static List<IcebergPartitionSpec> loadPartitionSpecByIceberg(
-        FeIcebergTable table) throws TableLoadingException {
+        FeIcebergTable table) throws ImpalaRuntimeException {
       List<IcebergPartitionSpec> ret = new ArrayList<>();
-      for (PartitionSpec spec : table.getIcebergApiTable().specs().values()) {
-        ret.add(convertPartitionSpec(spec));
+      Table iceApiTable = table.getIcebergApiTable();
+      for (PartitionSpec spec : iceApiTable.specs().values()) {
+        ret.add(convertPartitionSpec(iceApiTable.schema(), spec));
       }
       return ret;
     }
 
-    public static IcebergPartitionSpec convertPartitionSpec(PartitionSpec spec)
-        throws TableLoadingException {
-      List<IcebergPartitionField> fields = new ArrayList<>();;
-      HashMap<String, Integer> transformParams =
+    public static IcebergPartitionSpec convertPartitionSpec(Schema schema,
+        PartitionSpec spec) throws ImpalaRuntimeException {
+      List<IcebergPartitionField> fields = new ArrayList<>();
+      Map<String, Integer> transformParams =
           IcebergUtil.getPartitionTransformParams(spec);
       for (PartitionField field : spec.fields()) {
         fields.add(new IcebergPartitionField(field.sourceId(), field.fieldId(),
             spec.schema().findColumnName(field.sourceId()), field.name(),
-            IcebergUtil.getPartitionTransform(field, transformParams)));
+            IcebergUtil.getPartitionTransform(field, transformParams),
+            IcebergSchemaConverter.toImpalaType(
+                field.transform().getResultType(schema.findType(field.sourceId())))));
       }
       return new IcebergPartitionSpec(spec.specId(), fields);
     }
@@ -968,17 +985,26 @@ public interface FeIcebergTable extends FeFsTable {
     }
 
     /**
-     * Return true if the Iceberg has DeleteFiles.
+     * Return true if the Iceberg has DeleteFiles. Only non-dangling delete files count
+     * so we don't use the snapshot summary for this.
      */
-    public static boolean hasDeleteFiles(Table icebergTable, TimeTravelSpec travelSpec) {
-      Map<String, String> summary = getSnapshotSummary(icebergTable, travelSpec);
-      if (summary == null) return false;
-      String totalDeleteFilesStr = summary.get(SnapshotSummary.TOTAL_DELETE_FILES_PROP);
-      if (!Strings.isNullOrEmpty(totalDeleteFilesStr)) {
-        long totalDeleteFiles = Long.parseLong(totalDeleteFilesStr);
-        return totalDeleteFiles > 0;
+    public static boolean hasDeleteFiles(FeIcebergTable table,
+        TimeTravelSpec travelSpec) throws AnalysisException {
+      if (travelSpec == null) {
+        IcebergContentFileStore fileStore = table.getContentFileStore();
+        return !fileStore.getPositionDeleteFiles().isEmpty()
+            || !fileStore.getEqualityDeleteFiles().isEmpty();
+      } else {
+        try {
+          GroupedContentFiles groupedFiles =
+              IcebergUtil.getIcebergFiles(table, Lists.newArrayList(), travelSpec);
+          return !groupedFiles.positionDeleteFiles.isEmpty()
+              || !groupedFiles.equalityDeleteFiles.isEmpty();
+        } catch (TableLoadingException e) {
+          throw new AnalysisException("Failed to get record count of Iceberg V2 table: "
+              + table.getFullName(), e);
+        }
       }
-      return false;
     }
 
     /**
@@ -1020,6 +1046,9 @@ public interface FeIcebergTable extends FeFsTable {
       Table icebergApiTable = icebergTable.getIcebergApiTable();
       Preconditions.checkNotNull(icebergApiTable);
       Map<String, String> properties = icebergApiTable.properties();
+      if (BackendConfig.INSTANCE.icebergAllowDatafileInTableLocationOnly()) {
+        return true;
+      }
       return !(PropertyUtil.propertyAsBoolean(properties,
           TableProperties.OBJECT_STORE_ENABLED,
           TableProperties.OBJECT_STORE_ENABLED_DEFAULT)
@@ -1044,5 +1073,27 @@ public interface FeIcebergTable extends FeFsTable {
       return new FileStatus(contentFile.fileSizeInBytes(), false, 0, 0,
           DEFAULT_MODIFICATION_TIME, path);
     }
+
+    public static long getTotalNumberOfFiles(FeIcebergTable icebergTable,
+        TimeTravelSpec travelSpec)
+        throws ImpalaRuntimeException {
+      Map<String, String> snapshotSummary = getSnapshotSummary(
+          icebergTable.getIcebergApiTable(),
+          travelSpec);
+      if (snapshotSummary == null) {
+        throw new ImpalaRuntimeException("Invalid Iceberg snapshot summary");
+      }
+      try {
+        String totalDataFilesProp = snapshotSummary.get(
+            SnapshotSummary.TOTAL_DATA_FILES_PROP);
+        String totalDeleteFilesProp = snapshotSummary.getOrDefault(
+            SnapshotSummary.TOTAL_DELETE_FILES_PROP, "0");
+        long totalDataFiles = Long.parseLong(totalDataFilesProp);
+        long totalDeleteFiles = Long.parseLong(totalDeleteFilesProp);
+        return totalDataFiles + totalDeleteFiles;
+      } catch (NumberFormatException e) {
+        throw new ImpalaRuntimeException("Invalid Iceberg snapshot summary value");
+      }
+    }
   }
 }
