@@ -260,17 +260,28 @@ public class HdfsPartition extends CatalogObjectImpl
 
     public String getRelativePath() { return fbFileDescriptor_.relativePath(); }
 
-    public String getAbsolutePath() { return fbFileDescriptor_.absolutePath(); }
+    public String getAbsolutePath() {
+      return StringUtils.isEmpty(fbFileDescriptor_.absolutePath()) ?
+          StringUtils.EMPTY :
+          fbFileDescriptor_.absolutePath();
+    }
 
     public String getAbsolutePath(String rootPath) {
-      return StringUtils.isNotEmpty(fbFileDescriptor_.relativePath())
-          ? rootPath + Path.SEPARATOR + fbFileDescriptor_.relativePath()
-          : fbFileDescriptor_.absolutePath();
+      if (StringUtils.isEmpty(fbFileDescriptor_.relativePath())
+          && StringUtils.isNotEmpty(fbFileDescriptor_.absolutePath())) {
+        return fbFileDescriptor_.absolutePath();
+      } else {
+        return rootPath + Path.SEPARATOR + fbFileDescriptor_.relativePath();
+      }
     }
 
     public String getPath() {
-      return StringUtils.isNotEmpty(fbFileDescriptor_.relativePath())
-          ? fbFileDescriptor_.relativePath() : fbFileDescriptor_.absolutePath();
+      if (StringUtils.isEmpty(fbFileDescriptor_.relativePath())
+          && StringUtils.isNotEmpty(fbFileDescriptor_.absolutePath())) {
+        return fbFileDescriptor_.absolutePath();
+      } else {
+        return fbFileDescriptor_.relativePath();
+      }
     }
 
     public long getFileLength() { return fbFileDescriptor_.length(); }
@@ -1402,6 +1413,9 @@ public class HdfsPartition extends CatalogObjectImpl
         isMarkedCached_ = HdfsCachingUtil.getCacheDirectiveId(
             msPartition.getParameters()) != null;
         numRows_ = FeCatalogUtils.getRowCount(msPartition.getParameters());
+        if (table_.getDebugMetadataScale() > 1.0 && numRows_ > 0) {
+          numRows_ *= table_.getDebugMetadataScale();
+        }
         hmsParameters_ = msPartition.getParameters();
         extractAndCompressPartStats();
         // Intern parameters after removing the incremental stats
@@ -1775,9 +1789,9 @@ public class HdfsPartition extends CatalogObjectImpl
           && hmsParameters_.equals(oldInstance.hmsParameters_)
           && partitionStats_ == oldInstance.partitionStats_
           && hasIncrementalStats_ == oldInstance.hasIncrementalStats_
-          && numRows_ == oldInstance.numRows_
-          && writeId_ == oldInstance.writeId_
-          && lastCompactionId_ == oldInstance.lastCompactionId_);
+          && numRows_ == oldInstance.numRows_ && writeId_ == oldInstance.writeId_
+          && lastCompactionId_ == oldInstance.lastCompactionId_
+          && lastRefreshEventId_ == oldInstance_.lastRefreshEventId_);
     }
   }
 
