@@ -128,9 +128,9 @@ public class IcebergContentFileStore {
   }
 
   public void addEqualityDeleteFile(String pathHash, FileDescriptor desc) {
-    if (equalityDeleteFiles_.add(pathHash, desc)) {
-      updateFileFormats(desc);
-    }
+    Preconditions.checkState(
+        desc.getFbFileMetadata().icebergMetadata().equalityFieldIdsLength() > 0);
+    if (equalityDeleteFiles_.add(pathHash, desc)) updateFileFormats(desc);
   }
 
   // This is only invoked during time travel, when we are querying a snapshot that has
