@@ -52,6 +52,7 @@ import org.apache.impala.thrift.TPartialPartitionInfo;
 import org.apache.impala.thrift.TTable;
 import org.apache.impala.thrift.TTableDescriptor;
 import org.apache.impala.thrift.TTableType;
+import org.apache.impala.util.EventSequence;
 import org.apache.impala.util.IcebergSchemaConverter;
 import org.apache.impala.util.IcebergUtil;
 import org.apache.thrift.TException;
@@ -93,6 +94,34 @@ public class IcebergTable extends Table implements FeIcebergTable {
   // table metadata. This property is only valid for tables in 'hive.catalog'.
   public static final String METADATA_LOCATION = "metadata_location";
 
+  // Internal Iceberg table property that specifies the absolute path of the previous
+  // table metadata. This property is only valid for tables in 'hive.catalog'.
+  public static final String PREVIOUS_METADATA_LOCATION = "previous_metadata_location";
+
+  // Internal Iceberg table property that specifies the current schema.
+  public static final String CURRENT_SCHEMA = "current-schema";
+
+  // Internal Iceberg table property that specifies the number of snapshots.
+  public static final String SNAPSHOT_COUNT = "snapshot-count";
+
+  // Internal Iceberg table property that specifies the current snapshot id.
+  public static final String CURRENT_SNAPSHOT_ID = "current-snapshot-id";
+
+  // Internal Iceberg table property that specifies the current snapshot summary.
+  public static final String CURRENT_SNAPSHOT_SUMMARY = "current-snapshot-summary";
+
+  // Internal Iceberg table property that specifies the current snapshot timestamp in
+  // milliseconds.
+  public static final String CURRENT_SNAPSHOT_TIMESTAMP_MS
+      = "current-snapshot-timestamp-ms";
+
+  // Internal Iceberg table property that specifies the current default partition
+  // specification of the table.
+  public static final String DEFAULT_PARTITION_SPEC = "default-partition-spec";
+
+  // Internal Iceberg table property that specifies the UUID of the table.
+  public static final String UUID = "uuid";
+
   // Parquet compression codec and compression level table properties.
   public static final String PARQUET_COMPRESSION_CODEC =
       "write.parquet.compression-codec";
@@ -331,7 +360,16 @@ public class IcebergTable extends Table implements FeIcebergTable {
     TTable table = super.toThrift();
     table.setTable_type(TTableType.ICEBERG_TABLE);
     table.setIceberg_table(Utils.getTIcebergTable(this));
-    table.setHdfs_table(transfromToTHdfsTable(true));
+    table.setHdfs_table(transformToTHdfsTable(true, ThriftObjectType.FULL));
+    return table;
+  }
+
+  @Override
+  public TTable toHumanReadableThrift() {
+    TTable table = super.toThrift();
+    table.setTable_type(TTableType.ICEBERG_TABLE);
+    table.setIceberg_table(Utils.getTIcebergTable(this));
+    table.setHdfs_table(transformToTHdfsTable(true, ThriftObjectType.DESCRIPTOR_ONLY));
     return table;
   }
 
@@ -344,8 +382,8 @@ public class IcebergTable extends Table implements FeIcebergTable {
    */
   @Override
   public void load(boolean reuseMetadata, IMetaStoreClient msClient,
-      org.apache.hadoop.hive.metastore.api.Table msTbl, String reason)
-      throws TableLoadingException {
+      org.apache.hadoop.hive.metastore.api.Table msTbl, String reason,
+      EventSequence catalogTimeline) throws TableLoadingException {
     final Timer.Context context =
         getMetrics().getTimer(Table.LOAD_DURATION_METRIC).time();
     verifyTable(msTbl);
@@ -362,8 +400,10 @@ public class IcebergTable extends Table implements FeIcebergTable {
           getMetrics().getTimer(Table.LOAD_DURATION_STORAGE_METADATA).time();
       try {
         icebergApiTable_ = IcebergUtil.loadTable(this);
+        catalogTimeline.markEvent("Loaded Iceberg API table");
         catalogSnapshotId_ = FeIcebergTable.super.snapshotId();
         loadSchemaFromIceberg();
+        catalogTimeline.markEvent("Loaded schema from Iceberg");
         // Loading hdfs table after loaded schema from Iceberg,
         // in case we create external Iceberg table skipping column info in sql.
         icebergFileFormat_ = IcebergUtil.getIcebergFileFormat(msTbl);
@@ -373,15 +413,16 @@ public class IcebergTable extends Table implements FeIcebergTable {
         icebergParquetDictPageSize_ = Utils.getIcebergParquetDictPageSize(msTbl);
         GroupedContentFiles icebergFiles = IcebergUtil.getIcebergFiles(this,
             new ArrayList<>(), /*timeTravelSpec=*/null);
+        catalogTimeline.markEvent("Loaded Iceberg files");
         hdfsTable_.setIcebergFiles(icebergFiles);
         hdfsTable_.setCanDataBeOutsideOfTableLocation(
             !Utils.requiresDataFilesInTableLocation(this));
-        hdfsTable_.load(reuseMetadata, msClient, msTable_, reason);
+        hdfsTable_.load(reuseMetadata, msClient, msTable_, reason, catalogTimeline);
         fileStore_ = Utils.loadAllPartition(this, icebergFiles);
         partitionStats_ = Utils.loadPartitionStats(this, icebergFiles);
         setIcebergTableStats();
-        loadAllColumnStats(msClient);
-        setAvroSchema(msClient, msTbl, fileStore_);
+        loadAllColumnStats(msClient, catalogTimeline);
+        setAvroSchema(msClient, msTbl, fileStore_, catalogTimeline);
       } catch (Exception e) {
         throw new IcebergTableLoadingException("Error loading metadata for Iceberg table "
             + icebergTableLocation_, e);
@@ -402,6 +443,7 @@ public class IcebergTable extends Table implements FeIcebergTable {
         msTable_.putToParameters(StatsSetupConst.DO_NOT_UPDATE_STATS,
             StatsSetupConst.TRUE);
         msClient.alter_table(msTable_.getDbName(), msTable_.getTableName(), msTable_);
+        catalogTimeline.markEvent("Updated table schema in Metastore");
       } catch (TException e) {
         throw new TableLoadingException(e.getMessage());
       }
@@ -437,7 +479,8 @@ public class IcebergTable extends Table implements FeIcebergTable {
   /**
    * Load schema and partitioning schemes directly from Iceberg.
    */
-  public void loadSchemaFromIceberg() throws TableLoadingException {
+  public void loadSchemaFromIceberg()
+      throws TableLoadingException, ImpalaRuntimeException {
     loadSchema();
     addVirtualColumns();
     partitionSpecs_ = Utils.loadPartitionSpecByIceberg(this);
@@ -466,9 +509,9 @@ public class IcebergTable extends Table implements FeIcebergTable {
    */
   private void setAvroSchema(IMetaStoreClient msClient,
       org.apache.hadoop.hive.metastore.api.Table msTbl,
-      IcebergContentFileStore fileStore) throws Exception {
+      IcebergContentFileStore fileStore, EventSequence catalogTimeline) throws Exception {
     if (fileStore.hasAvro()) {
-      hdfsTable_.setAvroSchemaInternal(msClient, msTbl);
+      hdfsTable_.setAvroSchemaInternal(msClient, msTbl, catalogTimeline);
     }
   }
 
@@ -488,6 +531,7 @@ public class IcebergTable extends Table implements FeIcebergTable {
     addVirtualColumn(VirtualColumn.FILE_POSITION);
     addVirtualColumn(VirtualColumn.PARTITION_SPEC_ID);
     addVirtualColumn(VirtualColumn.ICEBERG_PARTITION_SERIALIZED);
+    addVirtualColumn(VirtualColumn.ICEBERG_DATA_SEQUENCE_NUMBER);
   }
 
   @Override
@@ -529,7 +573,8 @@ public class IcebergTable extends Table implements FeIcebergTable {
           fields.add(new IcebergPartitionField(field.getSource_id(), field.getField_id(),
               field.getOrig_field_name(), field.getField_name(),
               new IcebergPartitionTransform(field.getTransform().getTransform_type(),
-                  transformParam)));
+                  transformParam),
+              Type.fromTScalarType(field.getType())));
         }
         ret.add(new IcebergPartitionSpec(param.getSpec_id(),
             fields));
@@ -545,13 +590,14 @@ public class IcebergTable extends Table implements FeIcebergTable {
       Set<Long> referencedPartitions) {
     TTableDescriptor desc = new TTableDescriptor(tableId, TTableType.ICEBERG_TABLE,
         getTColumnDescriptors(), numClusteringCols_, name_, db_.getName());
-    desc.setIcebergTable(Utils.getTIcebergTable(this));
-    desc.setHdfsTable(transfromToTHdfsTable(false));
+    desc.setIcebergTable(Utils.getTIcebergTable(this, ThriftObjectType.DESCRIPTOR_ONLY));
+    desc.setHdfsTable(transformToTHdfsTable(false, ThriftObjectType.DESCRIPTOR_ONLY));
     return desc;
   }
 
-  public THdfsTable transfromToTHdfsTable(boolean updatePartitionFlag) {
-    THdfsTable hdfsTable = hdfsTable_.getTHdfsTable(ThriftObjectType.FULL, null);
+  public THdfsTable transformToTHdfsTable(boolean updatePartitionFlag,
+      ThriftObjectType type) {
+    THdfsTable hdfsTable = hdfsTable_.getTHdfsTable(type, null);
     if (updatePartitionFlag) {
       // Iceberg table only has one THdfsPartition, we set this partition
       // file format by iceberg file format which depend on table properties
