@@ -24,6 +24,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
+import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLong;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
 import java.util.concurrent.locks.ReentrantReadWriteLock.ReadLock;
@@ -34,6 +35,7 @@ import org.apache.hadoop.hive.metastore.TableType;
 import org.apache.hadoop.hive.metastore.api.ColumnStatisticsData;
 import org.apache.hadoop.hive.metastore.api.ColumnStatisticsObj;
 import org.apache.hadoop.hive.metastore.api.FieldSchema;
+import org.apache.hadoop.hive.metastore.api.PrincipalType;
 import org.apache.impala.analysis.TableName;
 import org.apache.impala.catalog.events.InFlightEvents;
 import org.apache.impala.catalog.monitor.CatalogMonitor;
@@ -58,8 +60,10 @@ import org.apache.impala.thrift.TTableInfoSelector;
 import org.apache.impala.thrift.TTableStats;
 import org.apache.impala.thrift.TTableType;
 import org.apache.impala.util.AcidUtils;
+import org.apache.impala.util.EventSequence;
 import org.apache.impala.util.HdfsCachingUtil;
 
+import com.codahale.metrics.Gauge;
 import com.codahale.metrics.Timer;
 import com.google.common.base.Preconditions;
 import com.google.common.collect.Lists;
@@ -80,12 +84,13 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
   protected org.apache.hadoop.hive.metastore.api.Table msTable_;
   protected final Db db_;
   protected final String name_;
+  protected final String full_name_;
   protected final String owner_;
   protected TAccessLevel accessLevel_ = TAccessLevel.READ_WRITE;
-  // Lock protecting this table. A read lock must be table when we are serializing
+  // Lock protecting this table. A read lock must be held when we are serializing
   // the table contents over thrift (e.g when returning the table to clients over thrift
   // or when topic-update thread serializes the table in the topic update)
-  // A write lock must be table when the table is being modified (e.g. DDLs or refresh)
+  // A write lock must be held when the table is being modified (e.g. DDLs or refresh)
   private final ReentrantReadWriteLock tableLock_ = new ReentrantReadWriteLock(
       true /*fair ordering*/);
   private final ReadLock readLock_ = tableLock_.readLock();
@@ -148,8 +153,11 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
   protected volatile long createEventId_ = -1;
 
   // tracks the in-flight metastore events for this table. Used by Events processor to
-  // avoid unnecessary refresh when the event is received
-  private final InFlightEvents inFlightEvents = new InFlightEvents();
+  // avoid unnecessary refresh when the event is received.
+  // Also used as a monitor object to synchronize access to it to avoid blocking on table
+  // lock during self-event check. If both this and tableLock_ or catalog version lock are
+  // taken, inFlightEvents_ must be the last to avoid deadlock.
+  private final InFlightEvents inFlightEvents_ = new InFlightEvents();
 
   // Table metrics. These metrics are applicable to all table types. Each subclass of
   // Table can define additional metrics specific to that table type.
@@ -190,6 +198,14 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
   public static final String TBL_PROP_EXTERNAL_TABLE_PURGE = "external.table.purge";
   public static final String TBL_PROP_EXTERNAL_TABLE_PURGE_DEFAULT = "TRUE";
 
+  public static final AtomicInteger LOADING_TABLES = new AtomicInteger(0);
+
+  // Table property key to determine the table's events process duration
+  public static final String TBL_EVENTS_PROCESS_DURATION = "events-process-duration";
+
+  // The last sync event id of the table
+  public static final String LAST_SYNC_EVENT_ID = "last-sync-event-id";
+
   // this field represents the last event id in metastore upto which this table is
   // synced. It is used if the flag sync_to_latest_event_on_ddls is set to true.
   // Making it as volatile so that read and write of this variable are thread safe.
@@ -199,14 +215,24 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
 
   protected volatile long lastRefreshEventId_ = -1L;
 
+  // Test only stats that will be injected in place of stats obtained from HMS.
+  protected SideloadTableStats testStats_ = null;
+
+  // Scale factor to multiply table stats with. Only used for testing.
+  protected double testMetadataScale_ = 1.0;
+
   protected Table(org.apache.hadoop.hive.metastore.api.Table msTable, Db db,
       String name, String owner) {
     msTable_ = msTable;
     db_ = db;
     name_ = name.toLowerCase();
+    full_name_ = (db_ != null ? db_.getName() + "." : "") + name_;
     owner_ = owner;
     tableStats_ = new TTableStats(-1);
     tableStats_.setTotal_file_bytes(-1);
+    if (db != null && RuntimeEnv.INSTANCE.hasSideloadStats(db.getName(), name)) {
+      testStats_ = RuntimeEnv.INSTANCE.getSideloadStats(db.getName(), name);
+    }
     initMetrics();
   }
 
@@ -376,6 +402,9 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
     metrics_.addTimer(HMS_LOAD_TBL_SCHEMA);
     metrics_.addTimer(LOAD_DURATION_ALL_COLUMN_STATS);
     metrics_.addCounter(NUMBER_OF_INFLIGHT_EVENTS);
+    metrics_.addTimer(TBL_EVENTS_PROCESS_DURATION);
+    metrics_.addGauge(LAST_SYNC_EVENT_ID,
+        (Gauge<Long>) () -> Long.valueOf(lastSyncedEventId_));
   }
 
   public Metrics getMetrics() { return metrics_; }
@@ -406,15 +435,22 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
    * valid existing metadata.
    */
   public abstract void load(boolean reuseMetadata, IMetaStoreClient client,
-      org.apache.hadoop.hive.metastore.api.Table msTbl, String reason)
-      throws TableLoadingException;
+      org.apache.hadoop.hive.metastore.api.Table msTbl, String reason,
+      EventSequence catalogTimeline) throws TableLoadingException;
 
   /**
    * Sets 'tableStats_' by extracting the table statistics from the given HMS table.
    */
   public void setTableStats(org.apache.hadoop.hive.metastore.api.Table msTbl) {
-    tableStats_ = new TTableStats(FeCatalogUtils.getRowCount(msTbl.getParameters()));
-    tableStats_.setTotal_file_bytes(FeCatalogUtils.getTotalSize(msTbl.getParameters()));
+    long rowCount = FeCatalogUtils.getRowCount(msTbl.getParameters());
+    if (testStats_ != null) {
+      tableStats_.setTotal_file_bytes(testStats_.getTotalSize());
+      testMetadataScale_ = (double) testStats_.getNumRows() / rowCount;
+      rowCount = testStats_.getNumRows();
+    } else {
+      tableStats_.setTotal_file_bytes(FeCatalogUtils.getTotalSize(msTbl.getParameters()));
+    }
+    tableStats_.setNum_rows(rowCount);
   }
 
   public void addColumn(Column col) {
@@ -453,7 +489,8 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
    * errors are logged and ignored, since the absence of column stats is not critical to
    * the correctness of the system.
    */
-  protected void loadAllColumnStats(IMetaStoreClient client) {
+  protected void loadAllColumnStats(IMetaStoreClient client,
+      EventSequence catalogTimeline) {
     final Timer.Context columnStatsLdContext =
         getMetrics().getTimer(LOAD_DURATION_ALL_COLUMN_STATS).time();
     try {
@@ -473,7 +510,8 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
         LOG.warn("Could not load column statistics for: " + getFullName(), e);
         return;
       }
-      FeCatalogUtils.injectColumnStats(colStats, this);
+      FeCatalogUtils.injectColumnStats(colStats, this, testStats_);
+      catalogTimeline.markEvent("Loaded all column stats");
     } finally {
       columnStatsLdContext.stop();
     }
@@ -488,10 +526,12 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
     CatalogInterners.internFieldsInPlace(msTbl);
     Table table = null;
     // Create a table of appropriate type
-    if (TImpalaTableType.VIEW ==
-        MetastoreShim.mapToInternalTableType(msTbl.getTableType())) {
-      if (msTbl.getTableType().equalsIgnoreCase("MATERIALIZED_VIEW") &&
-          HdfsFileFormat.isHdfsInputFormatClass(msTbl.getSd().getInputFormat())) {
+    TImpalaTableType tableType =
+        MetastoreShim.mapToInternalTableType(msTbl.getTableType());
+    if (tableType == TImpalaTableType.VIEW) {
+        table = new View(msTbl, db, msTbl.getTableName(), msTbl.getOwner());
+    } else if (tableType == TImpalaTableType.MATERIALIZED_VIEW) {
+      if (HdfsFileFormat.isHdfsInputFormatClass(msTbl.getSd().getInputFormat())) {
         table = new MaterializedViewHdfsTable(msTbl, db, msTbl.getTableName(),
             msTbl.getOwner());
       } else {
@@ -509,6 +549,8 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
       // have a special table property to indicate that Impala should use an external
       // data source.
       table = new DataSourceTable(msTbl, db, msTbl.getTableName(), msTbl.getOwner());
+    } else if (SystemTable.isSystemTable(msTbl)) {
+      table = new SystemTable(msTbl, db, msTbl.getTableName(), msTbl.getOwner());
     } else if (HdfsFileFormat.isHdfsInputFormatClass(msTbl.getSd().getInputFormat())) {
       table = new HdfsTable(msTbl, db, msTbl.getTableName(), msTbl.getOwner());
     }
@@ -529,6 +571,8 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
       TImpalaTableType tblType;
       if (thriftTable.getTable_type() == TTableType.VIEW) {
         tblType = TImpalaTableType.VIEW;
+      } else if (thriftTable.getTable_type() == TTableType.MATERIALIZED_VIEW) {
+        tblType = TImpalaTableType.MATERIALIZED_VIEW;
       } else {
         // If the table is unloaded or --pull_table_types_and_comments flag is not set,
         // keep the legacy behavior as showing the table type as TABLE.
@@ -668,6 +712,12 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
     return table;
   }
 
+  /**
+   * Variant of toThrift() with lower detail. Intended to get a human readable output
+   * that omit any binary fields.
+   */
+  public TTable toHumanReadableThrift() { return toThrift(); }
+
   private boolean isLockedByCurrentThread() {
     return isReadLockedByCurrentThread() || tableLock_.isWriteLockedByCurrentThread();
   }
@@ -807,7 +857,7 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
   public String getName() { return name_; }
 
   @Override // FeTable
-  public String getFullName() { return (db_ != null ? db_.getName() + "." : "") + name_; }
+  public String getFullName() { return full_name_; }
 
   @Override // FeTable
   public TableName getTableName() {
@@ -885,8 +935,11 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
 
   @Override // FeTable
   public String getOwnerUser() {
-    if (msTable_ == null) return null;
-    return msTable_.getOwner();
+    if (msTable_ == null) {
+      LOG.warn("Owner of {} is unknown due to table is unloaded", getFullName());
+      return null;
+    }
+    return msTable_.getOwnerType() == PrincipalType.USER ? msTable_.getOwner() : null;
   }
 
   public void setMetaStoreTable(org.apache.hadoop.hive.metastore.api.Table msTbl) {
@@ -981,12 +1034,13 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
    */
   public boolean removeFromVersionsForInflightEvents(
       boolean isInsertEvent, long versionNumber) {
-    Preconditions.checkState(isWriteLockedByCurrentThread(),
-        "removeFromVersionsForInFlightEvents called without taking the table lock on "
-            + getFullName());
-    boolean removed = inFlightEvents.remove(isInsertEvent, versionNumber);
-    if (removed) {
-      metrics_.getCounter(NUMBER_OF_INFLIGHT_EVENTS).dec();
+    boolean removed = false;
+    synchronized (inFlightEvents_) {
+      removed = inFlightEvents_.remove(isInsertEvent, versionNumber);
+      // Locked updating of counters is not need for correctnes but tests may rely on it.
+      if (removed) {
+        metrics_.getCounter(NUMBER_OF_INFLIGHT_EVENTS).dec();
+      }
     }
     return removed;
   }
@@ -1003,18 +1057,26 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
    * @return True if version number was added, false if the collection is at its max
    * capacity
    */
-  public void addToVersionsForInflightEvents(boolean isInsertEvent, long versionNumber) {
-    // we generally don't take locks on Incomplete tables since they are atomically
-    // replaced during load
+  public boolean addToVersionsForInflightEvents(
+      boolean isInsertEvent, long versionNumber) {
+    // We generally don't take locks on Incomplete tables since they are atomically
+    // replaced during load.
+    // The lock is not needed for thread safety, just verifying existing behavior.
     Preconditions.checkState(
         this instanceof IncompleteTable || isWriteLockedByCurrentThread());
-    if (!inFlightEvents.add(isInsertEvent, versionNumber)) {
-      LOG.warn(String.format("Could not add %s version to the table %s. This could "
-          + "cause unnecessary refresh of the table when the event is received by the "
-              + "Events processor.", versionNumber, getFullName()));
-    } else {
-      metrics_.getCounter(NUMBER_OF_INFLIGHT_EVENTS).inc();
+    boolean added = false;
+    synchronized (inFlightEvents_) {
+      added = inFlightEvents_.add(isInsertEvent, versionNumber);
+      // Locked updating of counters is not need for correctnes but tests may rely on it.
+      if (!added) {
+        LOG.warn(String.format("Could not add %s version to the table %s. This could "
+            + "cause unnecessary refresh of the table when the event is received by the "
+                + "Events processor.", versionNumber, getFullName()));
+      } else {
+        metrics_.getCounter(NUMBER_OF_INFLIGHT_EVENTS).inc();
+      }
     }
+    return added;
   }
 
   @Override
@@ -1045,6 +1107,10 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
   public long getLastRefreshEventId() { return lastRefreshEventId_; }
 
   public void setLastRefreshEventId(long eventId) {
+    setLastRefreshEventId(eventId, true);
+  }
+
+  public void setLastRefreshEventId(long eventId, boolean isSetLastSyncEventId) {
     if (eventId > lastRefreshEventId_) {
       lastRefreshEventId_ = eventId;
     }
@@ -1053,8 +1119,10 @@ public abstract class Table extends CatalogObjectImpl implements FeTable {
     // TODO: Should we reset lastSyncedEvent Id if it is less than event Id?
     // If we don't reset it - we may start syncing table from an event id which
     // is less than refresh event id
-    if (lastSyncedEventId_ < eventId) {
+    if (lastSyncedEventId_ < eventId && isSetLastSyncEventId) {
       setLastSyncedEventId(eventId);
     }
   }
-}
\ No newline at end of file
+
+  public double getDebugMetadataScale() { return testMetadataScale_; }
+}
