@@ -234,6 +234,7 @@ public abstract class Type {
         || isScalarType(PrimitiveType.TIMESTAMP);
   }
 
+  public boolean isIntegerOrDateType() { return isIntegerType() || isDate(); }
   public boolean isComplexType() { return isStructType() || isCollectionType(); }
   public boolean isCollectionType() { return isMapType() || isArrayType(); }
   public boolean isMapType() { return this instanceof MapType; }
@@ -243,6 +244,28 @@ public abstract class Type {
     return this instanceof CollectionStructType;
   }
 
+  /**
+   * Returns true if this type
+   *  - is a struct type or
+   *  - contains a struct type (recursively); for example
+   *    ARRAY<STRUCT<i: INT>>.
+   */
+  public boolean containsStruct() {
+    if (isStructType()) return true;
+
+    if (isArrayType()) {
+      ArrayType arrayType = (ArrayType) this;
+      return arrayType.getItemType().containsStruct();
+    } else if (isMapType()) {
+      MapType mapType = (MapType) this;
+      return mapType.getKeyType().containsStruct() ||
+          mapType.getValueType().containsStruct();
+    }
+
+    return false;
+  }
+
+
   /**
    * Returns true if this type
    *  - is a collection type or
@@ -434,6 +457,23 @@ public abstract class Type {
     return t.first;
   }
 
+  public static Type fromTScalarType(TScalarType scalarType) {
+    if (scalarType.getType() == TPrimitiveType.CHAR) {
+      Preconditions.checkState(scalarType.isSetLen());
+      return ScalarType.createCharType(scalarType.getLen());
+    } else if (scalarType.getType() == TPrimitiveType.VARCHAR) {
+      Preconditions.checkState(scalarType.isSetLen());
+      return ScalarType.createVarcharType(scalarType.getLen());
+    } else if (scalarType.getType() == TPrimitiveType.DECIMAL) {
+      Preconditions.checkState(scalarType.isSetPrecision()
+          && scalarType.isSetScale());
+      return ScalarType.createDecimalType(scalarType.getPrecision(),
+          scalarType.getScale());
+    } else {
+      return ScalarType.createType(PrimitiveType.fromThrift(scalarType.getType()));
+    }
+  }
+
   /**
    * Constructs a ColumnType rooted at the TTypeNode at nodeIdx in TColumnType.
    * Returned pair: The resulting ColumnType and the next nodeIdx that is not a child
@@ -445,22 +485,7 @@ public abstract class Type {
     switch (node.getType()) {
       case SCALAR: {
         Preconditions.checkState(node.isSetScalar_type());
-        TScalarType scalarType = node.getScalar_type();
-        if (scalarType.getType() == TPrimitiveType.CHAR) {
-          Preconditions.checkState(scalarType.isSetLen());
-          type = ScalarType.createCharType(scalarType.getLen());
-        } else if (scalarType.getType() == TPrimitiveType.VARCHAR) {
-          Preconditions.checkState(scalarType.isSetLen());
-          type = ScalarType.createVarcharType(scalarType.getLen());
-        } else if (scalarType.getType() == TPrimitiveType.DECIMAL) {
-          Preconditions.checkState(scalarType.isSetPrecision()
-              && scalarType.isSetScale());
-          type = ScalarType.createDecimalType(scalarType.getPrecision(),
-              scalarType.getScale());
-        } else {
-          type = ScalarType.createType(
-              PrimitiveType.fromThrift(scalarType.getType()));
-        }
+        type = fromTScalarType(node.getScalar_type());
         ++nodeIdx;
         break;
       }
