@@ -35,7 +35,7 @@ import org.apache.iceberg.types.TypeUtil;
 import org.apache.impala.analysis.ColumnDef;
 import org.apache.impala.analysis.IcebergPartitionSpec;
 import org.apache.impala.catalog.CatalogException;
-import org.apache.impala.catalog.CatalogObject;
+import org.apache.impala.catalog.CatalogObject.ThriftObjectType;
 import org.apache.impala.catalog.Column;
 import org.apache.impala.catalog.CtasTargetTable;
 import org.apache.impala.catalog.Db;
@@ -49,7 +49,6 @@ import org.apache.impala.catalog.IcebergColumn;
 import org.apache.impala.catalog.IcebergContentFileStore;
 import org.apache.impala.catalog.IcebergStructField;
 import org.apache.impala.catalog.IcebergTable;
-import org.apache.impala.catalog.HdfsPartition.FileDescriptor;
 import org.apache.impala.catalog.local.LocalDb;
 import org.apache.impala.catalog.local.LocalFsTable;
 import org.apache.impala.common.ImpalaRuntimeException;
@@ -90,11 +89,11 @@ public class IcebergCtasTarget extends CtasTargetTable implements FeIcebergTable
   private HdfsStorageDescriptor hdfsSd_;
 
   public IcebergCtasTarget(FeDb db, org.apache.hadoop.hive.metastore.api.Table msTbl,
-      List<ColumnDef> columnDefs, IcebergPartitionSpec partSpec)
-      throws CatalogException {
+        List<ColumnDef> columnDefs, List<String> primaryKeyNames,
+        IcebergPartitionSpec partSpec) throws CatalogException, ImpalaRuntimeException {
     super(msTbl, db, msTbl.getTableName(), msTbl.getOwner());
     createFsTable(db, msTbl);
-    createIcebergSchema(columnDefs);
+    createIcebergSchema(columnDefs, primaryKeyNames);
     createPartitionSpec(partSpec);
     icebergCatalog_ = IcebergUtil.getTIcebergCatalog(msTbl);
     setLocations();
@@ -106,13 +105,14 @@ public class IcebergCtasTarget extends CtasTargetTable implements FeIcebergTable
     hdfsSd_ = HdfsStorageDescriptor.fromStorageDescriptor(name_, msTable_.getSd());
   }
 
-  private void createIcebergSchema(List<ColumnDef> columnDefs) throws CatalogException {
+  private void createIcebergSchema(List<ColumnDef> columnDefs,
+      List<String> primaryKeyNames) throws CatalogException {
     List<TColumn> tcols = new ArrayList<>();
     for (ColumnDef col : columnDefs) {
       tcols.add(col.toThrift());
     }
     try {
-      iceSchema_ = IcebergSchemaConverter.genIcebergSchema(tcols);
+      iceSchema_ = IcebergSchemaConverter.genIcebergSchema(tcols, primaryKeyNames);
       // In genIcebergSchema() we did our best to assign correct field ids to columns,
       // but to be sure, let's use Iceberg's API function to assign field ids.
       iceSchema_ = TypeUtil.assignIncreasingFreshIds(iceSchema_);
@@ -126,11 +126,11 @@ public class IcebergCtasTarget extends CtasTargetTable implements FeIcebergTable
   }
 
   private void createPartitionSpec(IcebergPartitionSpec partSpec)
-      throws CatalogException {
+      throws CatalogException, ImpalaRuntimeException {
     Preconditions.checkState(iceSchema_ != null);
     PartitionSpec iceSpec = null;
     try {
-      // Let's create an Iceberg PartitionSpec with the help of Icebeg from 'partSpec',
+      // Let's create an Iceberg PartitionSpec with the help of Iceberg from 'partSpec',
       // then convert it back to an IcebergPartitionSpec.
       if (partSpec == null) {
         iceSpec = PartitionSpec.unpartitioned();
@@ -142,7 +142,7 @@ public class IcebergCtasTarget extends CtasTargetTable implements FeIcebergTable
         "Exception caught during generating Iceberg schema:", ex);
     }
     IcebergPartitionSpec resolvedIcebergSpec =
-        FeIcebergTable.Utils.convertPartitionSpec(iceSpec);
+        FeIcebergTable.Utils.convertPartitionSpec(iceSchema_, iceSpec);
     partitionSpecs_.add(resolvedIcebergSpec);
   }
 
@@ -254,7 +254,8 @@ public class IcebergCtasTarget extends CtasTargetTable implements FeIcebergTable
   }
 
   @Override
-  public THdfsTable transfromToTHdfsTable(boolean updatePartitionFlag) {
+  public THdfsTable transformToTHdfsTable(boolean updatePartitionFlag,
+      ThriftObjectType type) {
     throw new IllegalStateException("not implemented here");
   }
 
@@ -290,7 +291,7 @@ public class IcebergCtasTarget extends CtasTargetTable implements FeIcebergTable
         getNumClusteringCols(),
         getName(), db_.getName());
 
-    desc.setIcebergTable(Utils.getTIcebergTable(this));
+    desc.setIcebergTable(Utils.getTIcebergTable(this, ThriftObjectType.DESCRIPTOR_ONLY));
     desc.setHdfsTable(transformToTHdfsTable());
     return desc;
   }
@@ -305,7 +306,7 @@ public class IcebergCtasTarget extends CtasTargetTable implements FeIcebergTable
 
   private THdfsTable transformOldToTHdfsTable() {
     THdfsTable hdfsTable = ((HdfsTable)fsTable_).getTHdfsTable(
-        CatalogObject.ThriftObjectType.FULL, null);
+        ThriftObjectType.FULL, null);
     hdfsTable.setPrototype_partition(createPrototypePartition());
     return hdfsTable;
   }
