@@ -17,11 +17,15 @@
 
 package org.apache.impala.catalog.local;
 
+import static org.apache.impala.util.TUniqueIdUtil.PrintId;
+
 import java.lang.management.ManagementFactory;
 import java.nio.ByteBuffer;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -57,11 +61,13 @@ import org.apache.impala.catalog.Catalog;
 import org.apache.impala.catalog.CatalogDeltaLog;
 import org.apache.impala.catalog.CatalogException;
 import org.apache.impala.catalog.CatalogObjectCache;
+import org.apache.impala.catalog.DataSource;
 import org.apache.impala.catalog.Function;
 import org.apache.impala.catalog.HdfsCachePool;
 import org.apache.impala.catalog.HdfsPartition.FileDescriptor;
 import org.apache.impala.catalog.HdfsPartitionLocationCompressor;
 import org.apache.impala.catalog.HdfsStorageDescriptor;
+import org.apache.impala.catalog.HdfsTable;
 import org.apache.impala.catalog.ImpaladCatalog.ObjectUpdateSequencer;
 import org.apache.impala.catalog.Principal;
 import org.apache.impala.catalog.PrincipalPrivilege;
@@ -70,6 +76,7 @@ import org.apache.impala.catalog.VirtualColumn;
 import org.apache.impala.catalog.local.LocalIcebergTable.TableParams;
 import org.apache.impala.common.InternalException;
 import org.apache.impala.common.Pair;
+import org.apache.impala.common.PrintUtils;
 import org.apache.impala.service.BackendConfig;
 import org.apache.impala.service.FeSupport;
 import org.apache.impala.service.FrontendProfile;
@@ -81,11 +88,14 @@ import org.apache.impala.thrift.TCatalogInfoSelector;
 import org.apache.impala.thrift.TCatalogObject;
 import org.apache.impala.thrift.TCatalogObjectType;
 import org.apache.impala.thrift.TColumn;
+import org.apache.impala.thrift.TDataSource;
 import org.apache.impala.thrift.TDatabase;
 import org.apache.impala.thrift.TDbInfoSelector;
 import org.apache.impala.thrift.TErrorCode;
 import org.apache.impala.thrift.TFunction;
 import org.apache.impala.thrift.TFunctionName;
+import org.apache.impala.thrift.TGetLatestCompactionsRequest;
+import org.apache.impala.thrift.TGetLatestCompactionsResponse;
 import org.apache.impala.thrift.TGetPartialCatalogObjectRequest;
 import org.apache.impala.thrift.TGetPartialCatalogObjectResponse;
 import org.apache.impala.thrift.THdfsFileDesc;
@@ -115,6 +125,7 @@ import org.github.jamm.MemoryMeter;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import com.google.common.base.Joiner;
 import com.google.common.base.Objects;
 import com.google.common.base.Preconditions;
 import com.google.common.base.Stopwatch;
@@ -236,6 +247,8 @@ public class CatalogdMetaProvider implements MetaProvider {
   private static final String FUNCTIONS_STATS_CATEGORY = "Functions";
   private static final String RPC_STATS_CATEGORY = "RPCs";
   private static final String STORAGE_METADATA_LOAD_CATEGORY = "StorageLoad";
+  private static final String DATA_SOURCE_LIST_STATS_CATEGORY = "DataSourceLists";
+  private static final Object DS_OBJ_LIST_CACHE_KEY = new Object();
   private static final String RPC_REQUESTS =
       CATALOG_FETCH_PREFIX + "." + RPC_STATS_CATEGORY + ".Requests";
   private static final String RPC_BYTES =
@@ -257,11 +270,6 @@ public class CatalogdMetaProvider implements MetaProvider {
   private final ListMap<TNetworkAddress> cacheHostIndex_ =
       new ListMap<TNetworkAddress>();
 
-  // TODO(todd): currently we haven't implemented catalogd thrift APIs for all pieces
-  // of metadata. In order to incrementally build this out, we delegate various calls
-  // to the "direct" provider for now and circumvent catalogd.
-  private DirectMetaProvider directProvider_ = new DirectMetaProvider();
-
   /**
    * Number of requests which piggy-backed on a concurrent request for the same key,
    * and resulted in success. Used only for test assertions.
@@ -363,12 +371,19 @@ public class CatalogdMetaProvider implements MetaProvider {
       cacheSizeBytes = flags.local_catalog_cache_mb * 1024 * 1024;
     }
     int expirationSecs = flags.local_catalog_cache_expiration_s;
-    LOG.info("Metadata cache configuration: capacity={} MB, expiration={} sec",
-        cacheSizeBytes/1024/1024, expirationSecs);
+    int concurrencyLevel = flags.local_catalog_cache_concurrency_level;
+    if (concurrencyLevel <= 0) {
+      // 4 is the default value of the local cache
+      concurrencyLevel = 4;
+    }
+    LOG.info("Metadata cache configuration: capacity={} MB, expiration={} sec, " +
+        "concurrencyLevel={}", cacheSizeBytes/1024/1024, expirationSecs,
+        concurrencyLevel);
 
     // TODO(todd) add end-to-end test cases which stress cache eviction (both time
     // and size-triggered) and make sure results are still correct.
     cache_ = CacheBuilder.newBuilder()
+        .concurrencyLevel(concurrencyLevel)
         .maximumWeight(cacheSizeBytes)
         .expireAfterAccess(expirationSecs, TimeUnit.SECONDS)
         .weigher(new SizeOfWeigher(
@@ -448,10 +463,11 @@ public class CatalogdMetaProvider implements MetaProvider {
       case TABLE_NOT_FOUND:
       case TABLE_NOT_LOADED:
       case PARTITION_NOT_FOUND:
+      case DATA_SOURCE_NOT_FOUND:
         invalidateCacheForObject(req.object_desc);
-        throw new InconsistentMetadataFetchException(
-            String.format("Fetching %s failed. Could not find %s",
-                req.object_desc.type.name(), req.object_desc.toString()));
+        throw new InconsistentMetadataFetchException(resp.lookup_status,
+            String.format("Fetching %s failed: %s. Could not find %s",
+                req.object_desc.type, resp.lookup_status, req.object_desc));
       default: break;
     }
     Preconditions.checkState(resp.lookup_status == CatalogLookupStatus.OK);
@@ -469,7 +485,7 @@ public class CatalogdMetaProvider implements MetaProvider {
       LOG.warn("Catalog object {} changed version from {} to {} while fetching metadata",
           req.object_desc.toString(), req.object_desc.catalog_version,
           resp.object_version_number);
-      throw new InconsistentMetadataFetchException(
+      throw new InconsistentMetadataFetchException(CatalogLookupStatus.VERSION_MISMATCH,
           String.format("Catalog object %s changed version between accesses.",
               req.object_desc.toString()));
     }
@@ -658,6 +674,14 @@ public class CatalogdMetaProvider implements MetaProvider {
     return req;
   }
 
+  private TGetPartialCatalogObjectRequest newReqForDataSource(String dsName) {
+    TGetPartialCatalogObjectRequest req = new TGetPartialCatalogObjectRequest();
+    req.object_desc = new TCatalogObject();
+    req.object_desc.setType(TCatalogObjectType.DATA_SOURCE);
+    if (dsName == null) dsName = "";
+    req.object_desc.setData_source(new TDataSource(dsName, "", "", ""));
+    return req;
+  }
 
   @Override
   public Database loadDb(final String dbName) throws TException {
@@ -722,6 +746,20 @@ public class CatalogdMetaProvider implements MetaProvider {
     return req;
   }
 
+  @Override
+  public Pair<Table, TableMetaRef> getTableIfPresent(String dbName, String tblName) {
+    TableCacheKey cacheKey = new TableCacheKey(dbName.toLowerCase(),
+        tblName.toLowerCase());
+    try {
+      Object value = getIfPresent(cacheKey);
+      if (value == null) return null;
+      TableMetaRefImpl ref = (TableMetaRefImpl) value;
+      return Pair.create(ref.msTable_, ref);
+    } catch (TException e) {
+      return null;
+    }
+  }
+
   @Override
   public Pair<Table, TableMetaRef> loadTable(final String dbName, final String tableName)
       throws NoSuchObjectException, MetaException, TException {
@@ -861,9 +899,13 @@ public class CatalogdMetaProvider implements MetaProvider {
     if (existing == null) return null;
     if (!(existing instanceof Future)) return existing;
     try {
-      return ((Future<Object>)existing).get();
-    } catch (InterruptedException | ExecutionException e) {
-      Throwables.propagateIfPossible(e, TException.class);
+      return Uninterruptibles.getUninterruptibly((Future<Object>) existing);
+    } catch (ExecutionException e) {
+      // Try throwing the cause which is the error in the loading thread.
+      // This is consistent with the error handling in loadWithCaching().
+      // So failures like InconsistentMetadataFetchException can be caught
+      // and retry in Frontend#getTExecRequest().
+      Throwables.propagateIfPossible(e.getCause(), TException.class);
       throw new RuntimeException(e);
     }
   }
@@ -914,8 +956,8 @@ public class CatalogdMetaProvider implements MetaProvider {
         hostIndex, partitionRefs);
     if (BackendConfig.INSTANCE.isAutoCheckCompaction()) {
       // If any partitions are stale after compaction, they will be removed from refToMeta
-      List<PartitionRef> stalePartitions = directProvider_.checkLatestCompaction(
-          refImpl.dbName_, refImpl.tableName_, refImpl, refToMeta);
+      List<PartitionRef> stalePartitions =
+          checkLatestCompaction(refImpl.dbName_, refImpl.tableName_, refImpl, refToMeta);
       cache_.invalidateAll(stalePartitions.stream()
           .map(PartitionRefImpl.class::cast)
           .map(PartitionRefImpl::getId)
@@ -954,6 +996,55 @@ public class CatalogdMetaProvider implements MetaProvider {
     return nameToMeta;
   }
 
+  /**
+   * Fetches the latest compactions from catalogd and compares with partition metadata in
+   * cache. If a partition is stale due to compaction, removes it from metas. And return
+   * the stale partitions.
+   */
+  private List<PartitionRef> checkLatestCompaction(String dbName, String tableName,
+      TableMetaRef table, Map<PartitionRef, PartitionMetadata> metas) throws TException {
+    Preconditions.checkNotNull(table, "TableMetaRef must be non-null");
+    Preconditions.checkNotNull(metas, "Partition map must be non-null");
+    if (!table.isTransactional() || metas.isEmpty()) return Collections.emptyList();
+    Stopwatch sw = Stopwatch.createStarted();
+    TGetLatestCompactionsRequest req = new TGetLatestCompactionsRequest();
+    req.db_name = dbName;
+    req.table_name = tableName;
+    req.non_parition_name = HdfsTable.DEFAULT_PARTITION_NAME;
+    if (table.isPartitioned()) {
+      req.partition_names =
+          metas.keySet().stream().map(PartitionRef::getName).collect(Collectors.toList());
+    }
+    req.last_compaction_id = metas.values()
+                                 .stream()
+                                 .mapToLong(PartitionMetadata::getLastCompactionId)
+                                 .max()
+                                 .orElse(-1);
+    byte[] ret = FeSupport.GetLatestCompactions(
+        new TSerializer(new TBinaryProtocol.Factory()).serialize(req));
+    TGetLatestCompactionsResponse resp = new TGetLatestCompactionsResponse();
+    new TDeserializer(new TBinaryProtocol.Factory()).deserialize(resp, ret);
+    if (resp.status.status_code != TErrorCode.OK) {
+      throw new TException(Joiner.on("\n").join(resp.status.getError_msgs()));
+    }
+    Map<String, Long> partNameToCompactionId = resp.partition_to_compaction_id;
+    Preconditions.checkNotNull(
+        partNameToCompactionId, "Partition name to compaction id map must be non-null");
+    List<PartitionRef> stalePartitions = new ArrayList<>();
+    Iterator<Map.Entry<PartitionRef, PartitionMetadata>> iter =
+        metas.entrySet().iterator();
+    while (iter.hasNext()) {
+      Map.Entry<PartitionRef, PartitionMetadata> entry = iter.next();
+      if (partNameToCompactionId.containsKey(entry.getKey().getName())) {
+        stalePartitions.add(entry.getKey());
+        iter.remove();
+      }
+    }
+    LOG.info("Checked the latest compaction info for {}.{}. Time taken: {}", dbName,
+        tableName, PrintUtils.printTimeMs(sw.stop().elapsed(TimeUnit.MILLISECONDS)));
+    return stalePartitions;
+  }
+
   /**
    * Load the specified partitions 'prefs' from catalogd. The partitions are made
    * relative to the given 'hostIndex' before being returned.
@@ -1155,7 +1246,7 @@ public class CatalogdMetaProvider implements MetaProvider {
           /** Called to load cache for cache misses */
           @Override
           public String call() throws Exception {
-            return directProvider_.loadNullPartitionKeyValue();
+            return FeSupport.GetNullPartitionName();
           }
         });
   }
@@ -1206,6 +1297,51 @@ public class CatalogdMetaProvider implements MetaProvider {
     return funcs.build();
   }
 
+  /**
+   * Get all DataSource objects from catalogd.
+   */
+  public ImmutableList<DataSource> loadDataSources() throws TException {
+    ImmutableList<TDataSource> thriftDataSrcs = loadWithCaching(
+        "DataSource object list", DATA_SOURCE_LIST_STATS_CATEGORY,
+        DS_OBJ_LIST_CACHE_KEY, new Callable<ImmutableList<TDataSource>>() {
+          @Override
+          public ImmutableList<TDataSource> call() throws Exception {
+            TGetPartialCatalogObjectRequest req = newReqForDataSource(null);
+            TGetPartialCatalogObjectResponse resp = sendRequest(req);
+            checkResponse(resp.data_srcs != null, req, "no existing DataSource");
+            return ImmutableList.copyOf(resp.data_srcs);
+          }
+      });
+    ImmutableList.Builder<DataSource> dataSrcBld = ImmutableList.builder();
+    for (TDataSource thriftDataSrc : thriftDataSrcs) {
+      dataSrcBld.add(DataSource.fromThrift(thriftDataSrc));
+    }
+    return dataSrcBld.build();
+  }
+
+  /**
+   * Get the DataSource object from catalogd for the given DataSource name.
+   */
+  public DataSource loadDataSource(String dsName) throws TException {
+    Preconditions.checkState(dsName != null && !dsName.isEmpty());
+    return loadWithCaching("DataSource object for " + dsName,
+        DATA_SOURCE_LIST_STATS_CATEGORY,
+        new DataSourceCacheKey(dsName.toLowerCase()), new Callable<DataSource>() {
+          @Override
+          public DataSource call() throws Exception {
+            TGetPartialCatalogObjectRequest req = newReqForDataSource(dsName);
+            TGetPartialCatalogObjectResponse resp = sendRequest(req);
+            checkResponse(resp.data_srcs != null, req, "missing expected DataSource");
+            if (resp.data_srcs.size() == 1) {
+              return DataSource.fromThrift(resp.data_srcs.get(0));
+            } else {
+              Preconditions.checkState(resp.data_srcs.size() == 0);
+              return null;
+            }
+          }
+      });
+  }
+
   /**
    * Invalidate portions of the cache as indicated by the provided request.
    *
@@ -1437,7 +1573,7 @@ public class CatalogdMetaProvider implements MetaProvider {
         if (!catalogServiceId_.equals(Catalog.INITIAL_CATALOG_SERVICE_ID)) {
           LOG.warn("Detected catalog service restart: service ID changed from " +
               "{} to {}. Invalidating all cached metadata on this coordinator.",
-              catalogServiceId_, serviceId);
+              PrintId(catalogServiceId_), PrintId(serviceId));
         }
         catalogServiceId_ = serviceId;
         cache_.invalidateAll();
@@ -1512,6 +1648,12 @@ public class CatalogdMetaProvider implements MetaProvider {
           DbCacheKey.DbInfoType.HMS_METADATA,
           DbCacheKey.DbInfoType.FUNCTION_NAMES), invalidated);
       break;
+    case DATA_SOURCE:
+      if (cache_.asMap().remove(DS_OBJ_LIST_CACHE_KEY) != null) {
+        invalidated.add("list of DataSource objects");
+      }
+      invalidateCacheForDataSource(obj.data_source.name, invalidated);
+      break;
     default:
       break;
     }
@@ -1573,6 +1715,18 @@ public class CatalogdMetaProvider implements MetaProvider {
     }
   }
 
+  /**
+   * Invalidate cached DataSource for the given DataSource name. If anything was
+   * invalidated, adds a human-readable string to 'invalidated' indicating the
+   * invalidated DataSource.
+   */
+  private void invalidateCacheForDataSource(String dsName, List<String> invalidated) {
+    DataSourceCacheKey key = new DataSourceCacheKey(dsName.toLowerCase());
+    if (cache_.asMap().remove(key) != null) {
+      invalidated.add("DataSource " + dsName);
+    }
+  }
+
   /**
    * Reference to a partition within a table. We remember the partition's ID and pass
    * that back to the catalog in subsequent requests back to fetch the details of the
@@ -2009,6 +2163,30 @@ public class CatalogdMetaProvider implements MetaProvider {
     }
   }
 
+  /**
+   * Cache key for a DataSource object.
+   */
+  @Immutable
+  private static class DataSourceCacheKey {
+    private final String dsName_;
+
+    DataSourceCacheKey(String dsName) {
+      dsName_ = dsName;
+    }
+
+    @Override
+    public int hashCode() {
+      return Objects.hashCode(dsName_, getClass());
+    }
+
+    @Override
+    public boolean equals(Object obj) {
+      if (!(obj instanceof DataSourceCacheKey)) return false;
+      DataSourceCacheKey other = (DataSourceCacheKey)obj;
+      return dsName_.equals(other.dsName_);
+    }
+  }
+
   @VisibleForTesting
   static class SizeOfWeigher implements Weigher<Object, Object> {
     // Bypass flyweight objects like small boxed integers, Boolean.TRUE, enums, etc.
