@@ -37,6 +37,7 @@ import org.apache.hadoop.hive.metastore.api.Table;
 import org.apache.hadoop.hive.metastore.api.TableMeta;
 import org.apache.impala.authorization.AuthorizationPolicy;
 import org.apache.impala.catalog.CatalogException;
+import org.apache.impala.catalog.DataSource;
 import org.apache.impala.catalog.FileMetadataLoader;
 import org.apache.impala.catalog.Function;
 import org.apache.impala.catalog.HdfsCachePool;
@@ -147,6 +148,16 @@ class DirectMetaProvider implements MetaProvider {
     }
   }
 
+  @Override
+  public Pair<Table, TableMetaRef> getTableIfPresent(String dbName, String tblName) {
+    try {
+      return loadTable(dbName, tblName);
+    } catch (TException e) {
+      LOG.error("Failed to load table", e);
+      return null;
+    }
+  }
+
   @Override
   public Pair<Table, TableMetaRef> loadTable(String dbName, String tableName)
       throws MetaException, NoSuchObjectException, TException {
@@ -335,6 +346,20 @@ class DirectMetaProvider implements MetaProvider {
         "Functions not supported by DirectMetaProvider");
   }
 
+  @Override
+  public ImmutableList<DataSource> loadDataSources() throws TException {
+    // See above.
+    throw new UnsupportedOperationException(
+        "DataSource not supported by DirectMetaProvider");
+  }
+
+  @Override
+  public DataSource loadDataSource(String dsName) throws TException {
+    // See above.
+    throw new UnsupportedOperationException(
+        "DataSource not supported by DirectMetaProvider");
+  }
+
   @Override
   public List<ColumnStatisticsObj> loadTableColumnStatistics(TableMetaRef table,
       List<String> colNames) throws TException {
