@@ -570,8 +570,20 @@ public class AggregationNode extends PlanNode {
     processingCost_ = ProcessingCost.zero();
     AggregationNode prevAgg = getPrevAggInputNode();
     for (AggregateInfo aggInfo : aggInfos_) {
-      ProcessingCost aggCost = aggInfo.computeProcessingCost(
-          getDisplayLabel(), getAggClassNumGroup(prevAgg, aggInfo));
+      // The cost is a function of both the input size and the output size (NDV). For
+      // estimating cost for any pre-aggregation we need to account for the likelihood of
+      // duplicate keys across fragments.  Calculate an overall "intermediate" output
+      // cardinality that attempts to account for the dups. Cap it at the input
+      // cardinality because an aggregation cannot increase the cardinality.
+      long inputCardinality = Math.max(0, getAggClassNumGroup(prevAgg, aggInfo));
+      long perInstanceNdv = fragment_.getPerInstanceNdvForCpuCosting(
+          inputCardinality, aggInfo.getGroupingExprs());
+      long intermediateOutputCardinality = Math.max(0,
+          Math.min(
+              inputCardinality, perInstanceNdv * fragment_.getNumInstancesForCosting()));
+      long aggClassNumGroup = Math.max(0, getAggClassNumGroup(prevAgg, aggInfo));
+      ProcessingCost aggCost = aggInfo.computeProcessingCost(getDisplayLabel(),
+          aggClassNumGroup, intermediateOutputCardinality);
       processingCost_ = ProcessingCost.sumCost(processingCost_, aggCost);
     }
   }
@@ -678,15 +690,21 @@ public class AggregationNode extends PlanNode {
         Preconditions.checkState(lowPerInstanceDataBytes <= perInstanceDataBytes);
 
         // Given N as number of non-literal grouping expressions,
-        // corrFactor = AGG_MEM_CORRELATION_FACTOR ^ max(0, N - 1)
+        //   memScale = (1.0 - AGG_MEM_CORRELATION_FACTOR) ^ max(0, N - 1)
+        // Note that high value of AGG_MEM_CORRELATION_FACTOR value (close to 1.0) means
+        // there is high correlation between grouping expressions / columns, while low
+        // value (close to 0.0) means there is low correlation between them.
+        // High correlation means aggregation node can be scheduled with lower memory
+        // estimation (lower memScale).
         long nonLiteralExprCount = aggInfo.getGroupingExprs()
                                        .stream()
                                        .filter(e -> !(e instanceof LiteralExpr))
                                        .count();
-        double corrFactor = Math.pow(queryOptions.getAgg_mem_correlation_factor(),
-            Math.max(0, nonLiteralExprCount - 1));
+        double corrFactor = queryOptions.getAgg_mem_correlation_factor();
+        double memScale =
+            Math.pow(1.0 - corrFactor, Math.max(0, nonLiteralExprCount - 1));
         long resolvedPerInstanceDataBytes = lowPerInstanceDataBytes
-            + Math.round(corrFactor * (perInstanceDataBytes - lowPerInstanceDataBytes));
+            + Math.round(memScale * (perInstanceDataBytes - lowPerInstanceDataBytes));
         if (LOG.isTraceEnabled() && perInstanceDataBytes > resolvedPerInstanceDataBytes) {
           LOG.trace("Node " + getDisplayLabel() + " reduce perInstanceDataBytes from "
               + perInstanceDataBytes + " to " + resolvedPerInstanceDataBytes);
