@@ -17,24 +17,30 @@
 
 package org.apache.impala.planner;
 
+import java.io.IOException;
 import java.util.ArrayList;
 import java.util.List;
 
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.hadoop.fs.Path;
 import org.apache.impala.analysis.Analyzer;
 import org.apache.impala.analysis.BinaryPredicate;
 import org.apache.impala.analysis.BoolLiteral;
+import org.apache.impala.analysis.CastExpr;
 import org.apache.impala.analysis.CompoundPredicate;
+import org.apache.impala.analysis.DateLiteral;
 import org.apache.impala.analysis.Expr;
 import org.apache.impala.analysis.LiteralExpr;
 import org.apache.impala.analysis.NumericLiteral;
 import org.apache.impala.analysis.SlotRef;
 import org.apache.impala.analysis.StringLiteral;
+import org.apache.impala.analysis.TimestampLiteral;
 import org.apache.impala.analysis.TupleDescriptor;
 import org.apache.impala.catalog.DataSource;
 import org.apache.impala.catalog.FeDataSourceTable;
+import org.apache.impala.common.FileSystemUtil;
 import org.apache.impala.common.ImpalaException;
 import org.apache.impala.common.InternalException;
 import org.apache.impala.extdatasource.ExternalDataSourceExecutor;
@@ -61,6 +67,7 @@ import org.apache.impala.thrift.TStatus;
 import com.google.common.base.Joiner;
 import com.google.common.base.MoreObjects;
 import com.google.common.base.Preconditions;
+import com.google.common.base.Strings;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Lists;
 
@@ -132,11 +139,36 @@ public class DataSourceScanNode extends ScanNode {
       case STRING:
         return new TColumnValue().setString_val(
             ((StringLiteral) expr).getUnescapedValue());
-      case DECIMAL:
       case DATE:
-      case DATETIME:
+        return new TColumnValue().setDate_val(
+            (int) ((DateLiteral) expr).getValue());
       case TIMESTAMP:
-        // TODO: we support DECIMAL, TIMESTAMP and DATE but no way to specify it in SQL.
+        /**
+         * timestamp is of the ISO 8601 format(SQL standard): 'yyyy-mm-dd hh:mm:ss.ms'
+         * We are using string representation and not binary representation here as all
+         * Literals will finally be converted to string for pushdown.
+         */
+        return new TColumnValue().setString_val(
+            ((TimestampLiteral) expr).getStringValue());
+      case DECIMAL:
+        /**
+         * Decimal type of predicate data in TBinaryPredicate is saved as 'binary'.
+         * Predicates are generated by Planner and sent to backend DataSourceScanNode.
+         * When backend DataSourceScanNode calls ExternalDataSource.open() API, it sends
+         * predicates to Java ExternalDataSourceExecutor object as TOpenParams.predicates
+         * through JNI API. There is an issue to deserialize 'binary' type of data in
+         * TBinaryPredicate when JniUtil.deserializeThrift() is called in function
+         * ExternalDataSourceExecutor.open(). 'binary' type of data are changed to wrong
+         * value. JniUtil.deserializeThrift() calls Thrift API TDeserializer.deserialize.
+         * To work around this Thrift deserialization issue, string representation of the
+         * decimal is returned by converting decimal to string with BigDecimal.toString()
+         * Java API.
+         * TODO: Retry using JNI & Thrift TDeserializer.deserialize for decimal saved as
+         * 'binary' when Thrift is upgraded.
+         */
+        return new TColumnValue().setString_val(
+            ((NumericLiteral) expr).getValue().toString());
+       case DATETIME:
         return null;
       default:
         Preconditions.checkState(false);
@@ -163,18 +195,28 @@ public class DataSourceScanNode extends ScanNode {
       }
     }
 
-    String hdfsLocation = table_.getDataSource().getHdfs_location();
-    TCacheJarResult cacheResult = FeSupport.CacheJar(hdfsLocation);
-    TStatus cacheJarStatus = cacheResult.getStatus();
-    if (cacheJarStatus.getStatus_code() != TErrorCode.OK) {
-      throw new InternalException(String.format(
-          "Unable to cache data source library at location '%s'. Check that the file " +
-          "exists and is readable. Message: %s",
-          hdfsLocation, Joiner.on("\n").join(cacheJarStatus.getError_msgs())));
-    }
-    String localPath = cacheResult.getLocal_path();
     String className = table_.getDataSource().getClass_name();
     String apiVersion = table_.getDataSource().getApi_version();
+    String dsLocation = table_.getDataSource().getHdfs_location();
+    // Check if the data source jar file exists in the HDFS location. If yes, then copy
+    // the jar file to the local file system, and get the instance of the data source
+    // class in URLClassLoader which is generated from copied local jar file. Otherwise
+    // assume the jar file is in the classpath and try to get the instance of the data
+    // source class in current ClassLoader.
+    String localPath = null;
+    if (!Strings.isNullOrEmpty(dsLocation)) {
+      LOG.trace("Get the instance from URLClassLoader");
+      try {
+        localPath = FileSystemUtil.copyFileFromUriToLocal(dsLocation);
+      } catch (IOException e) {
+        throw new InternalException(String.format(
+            "Unable to fetch data source jar from location '%s'.", dsLocation));
+      }
+      Preconditions.checkNotNull(localPath);
+    } else {
+      LOG.trace("Get the instance of the data source class in current ClassLoader");
+    }
+
     TPrepareResult prepareResult;
     TStatus prepareStatus;
     try {
@@ -191,6 +233,12 @@ public class DataSourceScanNode extends ScanNode {
       throw new InternalException(String.format(
           "Error calling prepare() on data source %s",
           DataSource.debugString(table_.getDataSource())), e);
+    } finally {
+      if (!Strings.isNullOrEmpty(localPath)) {
+        // Delete the jar file once its loaded
+        Path localJarPath = new Path("file://" + localPath);
+        FileSystemUtil.deleteIfExists(localJarPath);
+      }
     }
     if (prepareStatus.getStatus_code() != TErrorCode.OK) {
       throw new InternalException(String.format(
@@ -226,23 +274,42 @@ public class DataSourceScanNode extends ScanNode {
       List<TBinaryPredicate> predicates) {
     if (conjunct instanceof BinaryPredicate) {
       if (conjunct.getChildren().size() != 2) return false;
+      Expr colExpr = null;
       SlotRef slotRef = null;
       LiteralExpr literalExpr = null;
       TComparisonOp op = null;
       if ((conjunct.getChild(0).unwrapSlotRef(true) instanceof SlotRef) &&
           (conjunct.getChild(1) instanceof LiteralExpr)) {
-        slotRef = conjunct.getChild(0).unwrapSlotRef(true);
+        colExpr = conjunct.getChild(0);
+        slotRef = colExpr.unwrapSlotRef(true);
         literalExpr = (LiteralExpr) conjunct.getChild(1);
         op = ((BinaryPredicate) conjunct).getOp().getThriftOp();
       } else if ((conjunct.getChild(1).unwrapSlotRef(true) instanceof SlotRef) &&
                  (conjunct.getChild(0) instanceof LiteralExpr)) {
-        slotRef = conjunct.getChild(1).unwrapSlotRef(true);
+        colExpr = conjunct.getChild(1);
+        slotRef = colExpr.unwrapSlotRef(true);
         literalExpr = (LiteralExpr) conjunct.getChild(0);
         op = ((BinaryPredicate) conjunct).getOp().converse().getThriftOp();
       } else {
+        // Other binary predicate scenarios are not accepted, including:
+        // both operands are slot references, one or both of the operands is an
+        // expression using SlotRef such as explicit CAST, built-in functions,
+        // arithmetic expressions, etc.
         return false;
       }
 
+      if (colExpr instanceof CastExpr) {
+        CastExpr castExpr = (CastExpr)colExpr;
+        Preconditions.checkNotNull(castExpr.getType());
+        if (castExpr.getType().isDateOrTimeType()
+            || castExpr.getCompatibility().isUnsafe()) {
+          // Unsafe casting or casting to Date/Timestamp for a column cannot be pushed
+          // down to JDBC table since TBinaryPredicate does not have column Expr.
+          // Return false here to avoid the conjunct to be added to offered predicate
+          // list.
+          return false;
+        }
+      }
       TColumnValue val = literalToColumnValue(literalExpr);
       if (val == null) return false; // false if unsupported type, e.g.
 
@@ -361,12 +428,12 @@ public class DataSourceScanNode extends ScanNode {
     output.append(String.format("%s%s:%s [%s%s]\n", prefix, id_.toString(),
         displayName_, table_.getFullName(), aliasStr));
 
-    if (!acceptedConjuncts_.isEmpty()) {
-      output.append(prefix + "data source predicates: "
+    if (acceptedConjuncts_ != null && !acceptedConjuncts_.isEmpty()) {
+      output.append(detailPrefix + "data source predicates: "
           + Expr.getExplainString(acceptedConjuncts_, detailLevel) + "\n");
     }
-    if (!conjuncts_.isEmpty()) {
-      output.append(prefix + "predicates: " +
+    if (conjuncts_ != null && !conjuncts_.isEmpty()) {
+      output.append(detailPrefix + "predicates: " +
             Expr.getExplainString(conjuncts_, detailLevel) + "\n");
     }
 
