@@ -38,6 +38,8 @@ import org.apache.impala.thrift.TPlanNodeType;
 import org.apache.impala.thrift.TQueryOptions;
 import org.apache.impala.util.BitUtil;
 import org.apache.impala.util.ExprUtil;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import com.google.common.base.Joiner;
 import com.google.common.base.MoreObjects;
@@ -49,6 +51,16 @@ import com.google.common.base.Preconditions;
  * inversion (for outer/semi/cross joins) it could also be the left child.
  */
 public class HashJoinNode extends JoinNode {
+  private final static Logger LOG = LoggerFactory.getLogger(HashJoinNode.class);
+
+  // Coefficients for estimating hash join CPU processing cost.  Derived from
+  // benchmarking. Probe side cost per input row consumed
+  private static final double COST_COEFFICIENT_PROBE_INPUT = 0.2565;
+  // Probe side cost per output row produced
+  private static final double COST_COEFFICIENT_HASH_JOIN_OUTPUT = 0.1812;
+  // Build side cost per input row consumed
+  private static final double COST_COEFFICIENT_BUILD_INPUT = 1.0;
+
   public HashJoinNode(PlanNode outer, PlanNode inner, boolean isStraightJoin,
       DistributionMode distrMode, JoinOperator joinOp,
       List<BinaryPredicate> eqJoinConjuncts, List<Expr> otherJoinConjuncts) {
@@ -324,27 +336,32 @@ public class HashJoinNode extends JoinNode {
 
   @Override
   public Pair<ProcessingCost, ProcessingCost> computeJoinProcessingCost() {
-    // TODO: The cost should consider conjuncts_ as well.
-    // Assume 'eqJoinConjuncts_' will be applied to all rows from lhs and rhs side,
-    // and 'otherJoinConjuncts_' to the resultant rows.
-    float eqJoinPredicateEvalCost = ExprUtil.computeExprsTotalCost(eqJoinConjuncts_);
-    float otherJoinPredicateEvalCost =
-        ExprUtil.computeExprsTotalCost(otherJoinConjuncts_);
-
-    // Compute the processing cost for lhs.
-    ProcessingCost probeProcessingCost =
-        ProcessingCost.basicCost(getDisplayLabel() + " Probe side (eqJoinConjuncts_)",
-            getChild(0).getCardinality(), eqJoinPredicateEvalCost);
-    if (otherJoinPredicateEvalCost > 0) {
-      probeProcessingCost = ProcessingCost.sumCost(probeProcessingCost,
-          ProcessingCost.basicCost(getDisplayLabel() + " Probe side(otherJoinConjuncts_)",
-              getCardinality(), otherJoinPredicateEvalCost));
+    // Compute the processing cost for lhs. Benchmarking suggests we can estimate the
+    // probe cost as a linear function combining the probe input cardinality and the
+    // estimated output cardinality.
+    long outputCardinality = Math.max(0, getCardinality());
+    double totalProbeCost =
+        (getProbeCardinalityForCosting() * COST_COEFFICIENT_PROBE_INPUT)
+        + (outputCardinality * COST_COEFFICIENT_HASH_JOIN_OUTPUT);
+    if (LOG.isTraceEnabled()) {
+      LOG.trace("Probe CPU cost estimate: " + totalProbeCost + ", Input Card: "
+          + getProbeCardinalityForCosting() + ", Output Card: " + outputCardinality);
     }
+    ProcessingCost probeProcessingCost =
+        ProcessingCost.basicCost(getDisplayLabel(), totalProbeCost);
 
     // Compute the processing cost for rhs.
+    // TODO: For broadcast join builds we're underestimating cost here because we're using
+    // the overall plan cardinality without accounting for the fact that the broadcast
+    // effectively multiplies the cardinality by the number of hosts.
+    // In the end this probably doesn't matter to the CPU resource allocation since the
+    // build fragment count is fixed for broadcast(at num hosts) regardless of the cost
+    // computed here. But we should clean up the costing here to avoid any future
+    // confusion.
+    long buildCardinality = Math.max(0, getChild(1).getFilteredCardinality());
+    double totalBuildCost = buildCardinality * COST_COEFFICIENT_BUILD_INPUT;
     ProcessingCost buildProcessingCost =
-        ProcessingCost.basicCost(getDisplayLabel() + " Build side",
-            getChild(1).getCardinality(), eqJoinPredicateEvalCost);
+        ProcessingCost.basicCost(getDisplayLabel() + " Build side", totalBuildCost);
     return Pair.create(probeProcessingCost, buildProcessingCost);
   }
 }
