@@ -37,8 +37,10 @@ import org.apache.impala.catalog.HdfsFileFormat;
 import org.apache.impala.catalog.HdfsPartition.FileDescriptor;
 import org.apache.impala.catalog.Type;
 import org.apache.impala.common.ImpalaRuntimeException;
+import org.apache.impala.common.ThriftSerializationCtx;
 import org.apache.impala.fb.FbIcebergDataFileFormat;
 import org.apache.impala.thrift.TExplainLevel;
+import org.apache.impala.thrift.TPlanNode;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -51,8 +53,14 @@ import com.google.common.collect.Lists;
 public class IcebergScanNode extends HdfsScanNode {
   private final static Logger LOG = LoggerFactory.getLogger(IcebergScanNode.class);
 
+  // List of files needed to be scanned by this scan node. The list is sorted in case of
+  // partitioned tables, so partition range scans are scheduled more evenly.
+  // See IMPALA-12765 for details.
   private List<FileDescriptor> fileDescs_;
 
+  // Indicates that the files in 'fileDescs_' are sorted.
+  private boolean filesAreSorted_ = false;
+
   // Conjuncts on columns not involved in IDENTITY-partitioning. Subset of 'conjuncts_',
   // but this does not include conjuncts on IDENTITY-partitioned columns, because such
   // conjuncts have already been pushed to Iceberg to filter out partitions/files, so
@@ -63,9 +71,27 @@ public class IcebergScanNode extends HdfsScanNode {
   // already applied them and they won't filter any further rows.
   private List<Expr> skippedConjuncts_;
 
+  // The Iceberg snapshot id used for this scan.
+  private final long snapshotId_;
+
+  // This member is set when this scan node is the left child of an IcebergDeleteNode or
+  // in other words when this scan node reads data files that have delete files
+  // associated. Holds the scan node ID of the right child of the IcebergDeleteNode
+  // responsible for reading the delete files of the corresponding table.
+  private final PlanNodeId deleteFileScanNodeId;
+
   public IcebergScanNode(PlanNodeId id, TableRef tblRef, List<Expr> conjuncts,
       MultiAggregateInfo aggInfo, List<FileDescriptor> fileDescs,
-      List<Expr> nonIdentityConjuncts, List<Expr> skippedConjuncts)
+      List<Expr> nonIdentityConjuncts, List<Expr> skippedConjuncts, long snapshotId)
+      throws ImpalaRuntimeException {
+    this(id, tblRef, conjuncts, aggInfo, fileDescs, nonIdentityConjuncts,
+        skippedConjuncts, null, snapshotId);
+  }
+
+  public IcebergScanNode(PlanNodeId id, TableRef tblRef, List<Expr> conjuncts,
+      MultiAggregateInfo aggInfo, List<FileDescriptor> fileDescs,
+      List<Expr> nonIdentityConjuncts, List<Expr> skippedConjuncts, PlanNodeId deleteId,
+      long snapshotId)
       throws ImpalaRuntimeException {
     super(id, tblRef.getDesc(), conjuncts,
         getIcebergPartition(((FeIcebergTable)tblRef.getTable()).getFeFsTable()), tblRef,
@@ -74,7 +100,17 @@ public class IcebergScanNode extends HdfsScanNode {
     Preconditions.checkState(partitions_.size() == 1);
 
     fileDescs_ = fileDescs;
+    if (((FeIcebergTable)tblRef.getTable()).isPartitioned()) {
+      // Let's order the file descriptors for better scheduling.
+      // See IMPALA-12765 for details.
+      // Create a clone of the original file descriptor list to avoid getting
+      // ConcurrentModificationException when sorting.
+      fileDescs_ = new ArrayList<>(fileDescs_);
+      Collections.sort(fileDescs_);
+      filesAreSorted_ = true;
+    }
     nonIdentityConjuncts_ = nonIdentityConjuncts;
+    snapshotId_ = snapshotId;
     //TODO IMPALA-11577: optimize file format counting
     boolean hasParquet = false;
     boolean hasOrc = false;
@@ -96,6 +132,7 @@ public class IcebergScanNode extends HdfsScanNode {
     if (hasOrc) fileFormats_.add(HdfsFileFormat.ORC);
     if (hasAvro) fileFormats_.add(HdfsFileFormat.AVRO);
     this.skippedConjuncts_ = skippedConjuncts;
+    this.deleteFileScanNodeId = deleteId;
   }
 
   /**
@@ -187,7 +224,9 @@ public class IcebergScanNode extends HdfsScanNode {
 
     // Ensure a consistent ordering of files for repeatable runs.
     List<FileDescriptor> orderedFds = Lists.newArrayList(fileDescs_);
-    Collections.sort(orderedFds);
+    if (!filesAreSorted_) {
+      Collections.sort(orderedFds);
+    }
 
     Preconditions.checkState(partitions_.size() == 1);
     FeFsPartition part = partitions_.get(0);
@@ -221,13 +260,26 @@ public class IcebergScanNode extends HdfsScanNode {
     return result;
   }
 
+  @Override
+  protected void toThrift(TPlanNode msg, ThriftSerializationCtx serialCtx) {
+    super.toThrift(msg, serialCtx);
+    Preconditions.checkNotNull(msg.hdfs_scan_node);
+    if (deleteFileScanNodeId != null) {
+      msg.hdfs_scan_node.setDeleteFileScanNodeId(deleteFileScanNodeId.asInt());
+    }
+  }
+
   @Override
   protected String getDerivedExplainString(
       String indentPrefix, TExplainLevel detailLevel) {
+    StringBuilder output = new StringBuilder();
+    output.append(
+        indentPrefix + "Iceberg snapshot id: " + String.valueOf(snapshotId_) + "\n");
     if (!skippedConjuncts_.isEmpty()) {
-      return indentPrefix + String.format("skipped Iceberg predicates: %s\n",
-          Expr.getExplainString(skippedConjuncts_, detailLevel));
+      output.append(indentPrefix +
+          String.format("skipped Iceberg predicates: %s\n",
+              Expr.getExplainString(skippedConjuncts_, detailLevel)));
     }
-    return "";
+    return output.toString();
   }
 }
