@@ -22,6 +22,7 @@ import java.util.Collections;
 import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Stack;
 
 import org.apache.impala.analysis.AnalyticExpr;
 import org.apache.impala.analysis.Analyzer;
@@ -107,7 +108,8 @@ public abstract class JoinNode extends PlanNode {
   public enum DistributionMode {
     NONE("NONE"),
     BROADCAST("BROADCAST"),
-    PARTITIONED("PARTITIONED");
+    PARTITIONED("PARTITIONED"),
+    DIRECTED("DIRECTED");
 
     private final String description_;
 
@@ -117,9 +119,18 @@ public abstract class JoinNode extends PlanNode {
 
     @Override
     public String toString() { return description_; }
+
     public static DistributionMode fromThrift(TJoinDistributionMode distrMode) {
-      if (distrMode == TJoinDistributionMode.BROADCAST) return BROADCAST;
-      return PARTITIONED;
+      switch (distrMode) {
+        case BROADCAST:
+          return BROADCAST;
+        case SHUFFLE:
+          return PARTITIONED;
+        case DIRECTED:
+          return DIRECTED;
+        default:
+          throw new RuntimeException("Invalid distribution mode: " + distrMode);
+      }
     }
   }
 
@@ -205,7 +216,8 @@ public abstract class JoinNode extends PlanNode {
   // Returns true if we can share a join build between multiple consuming fragment
   // instances.
   public boolean canShareBuild() {
-    return distrMode_ == JoinNode.DistributionMode.BROADCAST;
+    return distrMode_ == JoinNode.DistributionMode.BROADCAST ||
+        distrMode_ == DistributionMode.DIRECTED;
   }
 
   public JoinOperator getJoinOp() { return joinOp_; }
@@ -375,8 +387,8 @@ public abstract class JoinNode extends PlanNode {
    * The returned result is >= 0.
    * The list of join conjuncts must be non-empty and the cardinalities must be >= 0.
    */
-  private long getFkPkJoinCardinality(List<EqJoinConjunctScanSlots> eqJoinConjunctSlots,
-      long lhsCard, long rhsCard) {
+  protected static long getFkPkJoinCardinality(
+      List<EqJoinConjunctScanSlots> eqJoinConjunctSlots, long lhsCard, long rhsCard) {
     Preconditions.checkState(!eqJoinConjunctSlots.isEmpty());
     Preconditions.checkState(lhsCard >= 0 && rhsCard >= 0);
 
@@ -418,7 +430,7 @@ public abstract class JoinNode extends PlanNode {
 
     // first collect all the join cardinalities
     for (EqJoinConjunctScanSlots slots: eqJoinConjunctSlots) {
-      joinCardList.add(getGenericJoinCardinalityInternal(slots.lhsNdv(), slots.rhsNdv(),
+      joinCardList.add(computeGenericJoinCardinality(slots.lhsNdv(), slots.rhsNdv(),
           slots.lhsNumRows(), slots.rhsNumRows(), lhsCard, rhsCard));
     }
 
@@ -450,11 +462,11 @@ public abstract class JoinNode extends PlanNode {
   }
 
   /**
-   * An internal utility method to compute generic join cardinality as described
+   * A utility method to compute generic join cardinality as described
    * in the comments for {@link JoinNode#getJoinCardinality}. The input
    * cardinalities must be >= 0.
    */
-  private long getGenericJoinCardinalityInternal(double lhsNdv, double rhsNdv,
+  public static long computeGenericJoinCardinality(double lhsNdv, double rhsNdv,
       double lhsNumRows, double rhsNumRows, long lhsCard, long rhsCard) {
     Preconditions.checkState(lhsCard >= 0 && rhsCard >= 0);
     // Adjust the NDVs on both sides to account for predicates. Intuitively, the NDVs
@@ -471,10 +483,10 @@ public abstract class JoinNode extends PlanNode {
   }
 
   /**
-   * This function mirrors the logic for {@link JoinNode#getGenericJoinCardinality} except
-   * that instead of the EqJoinConjunctScanSlots, it uses the {@link NdvAndRowCountStats}
-   * to directly access stats that were pre-populated. Currently, this function is
-   * restricted to inner and outer joins.
+   * This function mirrors the logic for {@link JoinNode#computeGenericJoinCardinality}
+   * except that instead of the EqJoinConjunctScanSlots, it uses the {@link
+   * NdvAndRowCountStats} to directly access stats that were pre-populated. Currently,
+   * this function is restricted to inner and outer joins.
    * TODO: check if applicable for anti and semi joins
    */
   private long getGenericJoinCardinality2(List<NdvAndRowCountStats> statsList,
@@ -487,7 +499,7 @@ public abstract class JoinNode extends PlanNode {
       analyzer.getQueryOptions().getJoin_selectivity_correlation_factor();
     double cumulativeSel = 1.0;
     for (NdvAndRowCountStats stats: statsList) {
-      long joinCard = getGenericJoinCardinalityInternal(stats.lhsNdv(), stats.rhsNdv(),
+      long joinCard = computeGenericJoinCardinality(stats.lhsNdv(), stats.rhsNdv(),
           stats.lhsNumRows(), stats.rhsNumRows(), lhsCard, rhsCard);
       if (result == -1) {
         result = joinCard;
@@ -511,8 +523,8 @@ public abstract class JoinNode extends PlanNode {
    */
   public static final class EqJoinConjunctScanSlots {
     private final Expr eqJoinConjunct_;
-    private final SlotDescriptor lhs_;
-    private final SlotDescriptor rhs_;
+    protected final SlotDescriptor lhs_;
+    protected final SlotDescriptor rhs_;
 
     private EqJoinConjunctScanSlots(Expr eqJoinConjunct, SlotDescriptor lhs,
         SlotDescriptor rhs) {
@@ -743,15 +755,11 @@ public abstract class JoinNode extends PlanNode {
   }
 
   /**
-   * Unwraps the SlotRef in expr and returns the NDVs of it.
-   * Returns -1 if the NDVs are unknown or if expr is not a SlotRef.
+   * Returns the NDVs of a given expression.
+   * Returns -1 if the NDVs are unknown.
    */
   public static long getNdv(Expr expr) {
-    SlotRef slotRef = expr.unwrapSlotRef(false);
-    if (slotRef == null) return -1;
-    SlotDescriptor slotDesc = slotRef.getDesc();
-    if (slotDesc == null) return -1;
-    ColumnStats stats = slotDesc.getStats();
+    ColumnStats stats = ColumnStats.fromExpr(expr);
     if (!stats.hasNumDistinctValues()) return -1;
     return stats.getNumDistinctValues();
   }
@@ -992,10 +1000,38 @@ public abstract class JoinNode extends PlanNode {
    */
   public abstract Pair<ProcessingCost, ProcessingCost> computeJoinProcessingCost();
 
-  /* Helper to return all predicates as a string. */
-  public String getAllPredicatesAsString(TExplainLevel level) {
-    return "Conjuncts=" + Expr.getExplainString(getConjuncts(), level)
-        + ", EqJoinConjuncts=" + Expr.getExplainString(getEqJoinConjuncts(), level)
-        + ", EqJoinConjuncts=" + Expr.getExplainString(getOtherJoinConjuncts(), level);
+  /**
+   * Get filtered cardinality of probe hand of join node.
+   * Sanitized unknown cardinality (-1) into 0.
+   */
+  protected long getProbeCardinalityForCosting() {
+    return Math.max(0, getChild(0).getFilteredCardinality());
+  }
+
+  @Override
+  protected void reduceCardinalityByRuntimeFilter(
+      Stack<PlanNode> nodeStack, double reductionScale) {
+    if (isSelectiveAndReducingJoin()) {
+      nodeStack.add(this);
+    } else {
+      nodeStack.clear();
+    }
+    int i = 0;
+    for (PlanNode child : getChildren()) {
+      if (i > 0) nodeStack.clear(); // not probe child
+      child.reduceCardinalityByRuntimeFilter(nodeStack, reductionScale);
+      ++i;
+    }
+  }
+
+  private boolean isSelectiveAndReducingJoin() {
+    if (eqJoinConjuncts_.isEmpty() || getChild(0).getCardinality() < 0
+        || getChild(1).getCardinality() < 0
+        || (!joinOp_.isInnerJoin() && !joinOp_.isLeftOuterJoin()
+            && !joinOp_.isLeftSemiJoin())) {
+      return false;
+    }
+    double selectivity = RuntimeFilterGenerator.getJoinNodeSelectivity(this);
+    return 0 <= selectivity && selectivity <= 1;
   }
 }
