@@ -44,7 +44,9 @@ import org.apache.impala.analysis.TupleDescriptor;
 import org.apache.impala.catalog.FeKuduTable;
 import org.apache.impala.catalog.KuduColumn;
 import org.apache.impala.catalog.Type;
+import org.apache.impala.common.AnalysisException;
 import org.apache.impala.common.ImpalaRuntimeException;
+import org.apache.impala.common.InternalException;
 import org.apache.impala.service.BackendConfig;
 import org.apache.impala.thrift.TExplainLevel;
 import org.apache.impala.thrift.TKuduReplicaSelection;
@@ -128,6 +130,16 @@ public class KuduScanNode extends ScanNode {
   // for Kudu primary keys by enabling small query optimization.
   boolean isPointLookupQuery_ = false;
 
+  // It is used to indicate current kudu predicate should not be removed from conjuncts.
+  // If the current predicate is a comparison predicate with ambiguous timestamp, it may
+  // need to check again after actually scanning. For example, if we have two rows with
+  // column 'ts' 01:40:00(Local, UTC is 05:40:00) and 01:20:00(Local, UTC is 06:20:00),
+  // and the predicate 'ts' < 01:30:00(Local, convert to UTC is 05:30:00 or 06:30:00), we
+  // should push ts < 06:30:00(UTC) to Kudu to avoid missing the row with ts
+  // 01:20:00(Local, UTC is 06:20:00) and also need to filter out the row with ts
+  // 01:40:00(Local, UTC is 05:40:00) after actually scanning.
+  boolean currentPredicateNeedCheckAgain_ = false;
+
   public KuduScanNode(PlanNodeId id, TupleDescriptor desc, List<Expr> conjuncts,
       MultiAggregateInfo aggInfo, TableRef kuduTblRef) {
     super(id, desc, "SCAN KUDU");
@@ -492,7 +504,11 @@ public class KuduScanNode extends ScanNode {
               primaryKeyColsInEqualPred) ||
           tryConvertInListKuduPredicate(analyzer, rpcTable, predicate) ||
           tryConvertIsNullKuduPredicate(analyzer, rpcTable, predicate)) {
-        it.remove();
+        if (currentPredicateNeedCheckAgain_) {
+          currentPredicateNeedCheckAgain_ = false;
+        } else {
+          it.remove();
+        }
       }
     }
     if (primaryKeyColsInEqualPred.size() >= 1 &&
@@ -564,11 +580,18 @@ public class KuduScanNode extends ScanNode {
             ((StringLiteral)literal).getUnescapedValue());
         break;
       }
+      case BINARY: {
+        kuduPredicate = KuduPredicate.newComparisonPredicate(column, op,
+            ((StringLiteral)literal).getUnescapedValue().getBytes());
+        break;
+      }
       case TIMESTAMP: {
         try {
           // TODO: Simplify when Impala supports a 64-bit TIMESTAMP type.
-          kuduPredicate = KuduPredicate.newComparisonPredicate(column, op,
-              ExprUtil.utcTimestampToUnixTimeMicros(analyzer, literal));
+          kuduPredicate = analyzer.getQueryOptions().isConvert_kudu_utc_timestamps() ?
+              convertLocalTimestampBinaryKuduPredicate(analyzer, column, op, literal) :
+              KuduPredicate.newComparisonPredicate(column, op,
+                  ExprUtil.utcTimestampToUnixTimeMicros(analyzer, literal));
         } catch (Exception e) {
           LOG.info("Exception converting Kudu timestamp predicate: " + expr.toSql(), e);
           return false;
@@ -601,6 +624,54 @@ public class KuduScanNode extends ScanNode {
     return true;
   }
 
+  private KuduPredicate convertLocalTimestampBinaryKuduPredicate(Analyzer analyzer,
+      ColumnSchema column, ComparisonOp op, LiteralExpr literal)
+      throws AnalysisException, InternalException {
+    Long preUnixTimeMicros =
+        ExprUtil.localTimestampToUnixTimeMicros(analyzer, literal, true);
+    Long postUnixTimeMicros =
+        ExprUtil.localTimestampToUnixTimeMicros(analyzer, literal, false);
+    // If the timestamp is not a valid local timestamp, EQUAL predicate should be always
+    // false. For other comparison predicates, we could use the transition point time as
+    // a common value for comparison.
+    if (preUnixTimeMicros == null || postUnixTimeMicros == null) {
+      if (preUnixTimeMicros == null) return null; // should not happen
+      if (op == ComparisonOp.EQUAL) {
+        // An empty IN LIST predicate is always false.
+        return KuduPredicate.newInListPredicate(column, Lists.newArrayList());
+      } else {
+        postUnixTimeMicros = preUnixTimeMicros;
+      }
+    }
+    // If the timestamp is unique, create the predicate normally.
+    if (preUnixTimeMicros.equals(postUnixTimeMicros)) {
+      return KuduPredicate.newComparisonPredicate(column, op, preUnixTimeMicros);
+    }
+    // If the timestamp is ambiguous, we should convert EQUAL predicate to an IN LIST
+    // predicate that include all ambiguous values. For comparison predicates, we need to
+    // use a larger range of possible values for comparison to avoid missing rows.
+    // Additionally, set currentPredicateNeedCheckAgain_ to true to indicate that the
+    // predicate should not removed from the conjuncts_ list.
+    switch (op) {
+      case EQUAL: return KuduPredicate.newInListPredicate(column,
+          Lists.newArrayList(preUnixTimeMicros, postUnixTimeMicros));
+      case LESS:
+      case LESS_EQUAL: {
+        currentPredicateNeedCheckAgain_ = true;
+        return KuduPredicate.newComparisonPredicate(column, op,
+            postUnixTimeMicros);
+      }
+      case GREATER:
+      case GREATER_EQUAL: {
+        currentPredicateNeedCheckAgain_ = true;
+        return KuduPredicate.newComparisonPredicate(column, op,
+            preUnixTimeMicros);
+      }
+      default:
+        throw new InternalException("Unexpected operator: " + op);
+    }
+  }
+
   /**
    * If the InList 'expr' can be converted to a KuduPredicate, returns true and updates
    * kuduPredicates_ and kuduConjuncts_.
@@ -628,7 +699,11 @@ public class KuduScanNode extends ScanNode {
 
       Object value = getKuduInListValue(analyzer, literal);
       if (value == null) return false;
-      values.add(value);
+      if (value instanceof List) {
+        values.addAll((List<?>) value);
+      } else {
+        values.add(value);
+      }
     }
 
     String colName = ((KuduColumn) ref.getDesc().getColumn()).getKuduName();
@@ -671,6 +746,10 @@ public class KuduScanNode extends ScanNode {
    * Return the value of the InList child expression 'e' as an Object that can be
    * added to a KuduPredicate. If the Expr is not supported by Kudu or the type doesn't
    * match the expected PrimitiveType 'type', null is returned.
+   * Additionally, if the query option 'convert_kudu_utc_timestamps' is enabled and when
+   * the expression 'e' is converted from a local timestamp to a UTC timestamp, it is
+   * invalid or ambiguous, the method will return either an empty list or a list
+   * containing two ambiguous values.
    */
   private static Object getKuduInListValue(Analyzer analyzer, LiteralExpr e) {
     switch (e.getType().getPrimitiveType()) {
@@ -685,6 +764,21 @@ public class KuduScanNode extends ScanNode {
       case TIMESTAMP: {
         try {
           // TODO: Simplify when Impala supports a 64-bit TIMESTAMP type.
+          if (analyzer.getQueryOptions().isConvert_kudu_utc_timestamps()) {
+            Long preUnixTimeMicros =
+                ExprUtil.localTimestampToUnixTimeMicros(analyzer, e, true);
+            Long postUnixTimeMicros =
+                ExprUtil.localTimestampToUnixTimeMicros(analyzer, e, false);
+            // If the timestamp is invalid in local time, return empty list.
+            if (preUnixTimeMicros == null || postUnixTimeMicros == null) {
+              if (preUnixTimeMicros == null) return null; // should not happen
+              return Lists.newArrayList();
+            }
+            // If the timestamp is unique, return the unique value.
+            if (preUnixTimeMicros.equals(postUnixTimeMicros)) return preUnixTimeMicros;
+            // If the timestamp is ambiguous, return a list of the two possible values.
+            return Lists.newArrayList(preUnixTimeMicros, postUnixTimeMicros);
+          }
           return ExprUtil.utcTimestampToUnixTimeMicros(analyzer, e);
         } catch (Exception ex) {
           LOG.info("Exception converting Kudu timestamp expr: " + e.toSql(), ex);
