@@ -48,6 +48,8 @@ import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Lists;
 import com.google.common.math.IntMath;
 
+import javax.annotation.Nullable;
+
 /**
  * PlanFragments form a tree structure via their ExchangeNodes. A tree of fragments
  * connected in that way forms a plan. The output of a plan is produced by the root
@@ -90,6 +92,8 @@ public class PlanFragment extends TreeNode<PlanFragment> {
   private PlanId planId_;
   private CohortId cohortId_;
 
+  private final boolean coordinatorOnly_;
+
   // root of plan tree executed by this fragment
   private PlanNode planRoot_;
 
@@ -171,6 +175,11 @@ public class PlanFragment extends TreeNode<PlanFragment> {
   private int thisTreeCpuCore_ = -1;
   private int subtreeCpuCore_ = -1;
 
+  // Determine whether this fragment is the dominant one in the plan tree based on
+  // calculation initiated by Planner.computeBlockingAwareCores().
+  // A fragment is dominant if it contribute towards the final CoreCount.
+  private boolean isDominantFragment_ = false;
+
   public long getProducedRuntimeFiltersMemReservationBytes() {
     return producedRuntimeFiltersMemReservationBytes_;
   }
@@ -178,12 +187,23 @@ public class PlanFragment extends TreeNode<PlanFragment> {
   /**
    * C'tor for fragment with specific partition; the output is by default broadcast.
    */
-  public PlanFragment(PlanFragmentId id, PlanNode root, DataPartition partition) {
+  public PlanFragment(PlanFragmentId id, PlanNode root, DataPartition partition,
+      boolean coordinatorOnly) {
     fragmentId_ = id;
     planRoot_ = root;
     dataPartition_ = partition;
     outputPartition_ = DataPartition.UNPARTITIONED;
     setFragmentInPlanTree(planRoot_);
+    coordinatorOnly_ = coordinatorOnly;
+
+    // Coordinator-only fragments must be unpartitined as there is only one instance of
+    // them.
+    Preconditions.checkState(!coordinatorOnly ||
+        dataPartition_.equals(DataPartition.UNPARTITIONED));
+  }
+
+  public PlanFragment(PlanFragmentId id, PlanNode root, DataPartition partition) {
+    this(id, root, partition, false);
   }
 
   /**
@@ -465,6 +485,13 @@ public class PlanFragment extends TreeNode<PlanFragment> {
     }
     for (RuntimeFilter f : producedFilters.values()) {
       producedRuntimeFiltersMemReservationBytes_ += f.getFilterSize();
+      if (analyzer.getQueryOptions().max_num_filters_aggregated_per_host > 1
+          && !f.isBroadcast()) {
+        // If distributed aggregation is enabled, any backend can be selected as
+        // an intermediate aggregator. Add the same amount of memory per backend to
+        // anticipate for it.
+        consumedGlobalRuntimeFiltersMemReservationBytes_ += f.getFilterSize();
+      }
     }
     for (RuntimeFilter f : consumedFilters.values()) {
       if (!producedFilters.containsKey(f.getFilterId())) {
@@ -473,6 +500,10 @@ public class PlanFragment extends TreeNode<PlanFragment> {
     }
   }
 
+  public boolean coordinatorOnly() {
+    return coordinatorOnly_;
+  }
+
   public ResourceProfile getPerInstanceResourceProfile() {
     return perInstanceResourceProfile_;
   }
@@ -539,18 +570,29 @@ public class PlanFragment extends TreeNode<PlanFragment> {
       return ((HdfsTableSink)sink_).getNumInstances();
     } else {
       if (originalInstanceCount_ > -1) {
-        Preconditions.checkState(hasAdjustedInstanceCount());
-        Preconditions.checkState(
-            planRoot_.getNumInstances() == getAdjustedInstanceCount(),
+        int adjustedCount = getAdjustedInstanceCount();
+        Preconditions.checkState(planRoot_.getNumInstances() == adjustedCount,
             "Instance count of " + getId() + " with plan root "
                 + planRoot_.getDisplayLabel() + " (" + planRoot_.getNumInstances()
-                + ") does not follow cost based adjustment (" + getAdjustedInstanceCount()
-                + ")!");
+                + ") does not follow cost based adjustment (" + adjustedCount + ")!");
       }
       return planRoot_.getNumInstances();
     }
   }
 
+  /**
+   * Return early estimate of total number of instances of this fragment across all hosts.
+   * This is intended for use by CPC code for estimating instance counts prior to any
+   * call to setEffectiveNumInstance(). Currently we simply return the estimate from
+   * getNumInstances() but we should try to improve this in the future.
+   * TODO: Come up with a better prediction here of the instance count we're likely to
+   * end up with after subsequent ProcessingCost adjustments.
+   */
+  public int getNumInstancesForCosting() {
+    Preconditions.checkState(originalInstanceCount_ < 0);
+    return getNumInstances();
+  }
+
   /**
    * Estimates the number of distinct values of exprs per fragment instance based on the
    * data partition of this fragment, the number of nodes, and the degree of parallelism.
@@ -589,6 +631,49 @@ public class PlanFragment extends TreeNode<PlanFragment> {
     return result;
   }
 
+  /**
+   * Estimate the number of distinct values of exprs per fragment instance accounting
+   * for the likelihood of duplicate keys across fragment instances (see IMPALA-2945).
+   * We currently only use this estimate for CPU costing when CPC is enabled but we
+   * should use this approach more broadly for resource estimates because the approach
+   * in getPerInstanceNdv() ignores dups and can underestimate badly especially for low
+   * NDV.
+   */
+  public long getPerInstanceNdvForCpuCosting(long inputCardinality, List<Expr> exprs) {
+    Preconditions.checkState(inputCardinality > -1);
+
+    long result = 1;
+    int numInstances = getNumInstancesForCosting();
+    Preconditions.checkState(numInstances >= 0);
+    // TODO: Should we use AggregationNode.DEFAULT_SKEW_FACTOR when calculating
+    // per instance input cardinality here?
+    long perInstanceInputCardinality =
+        (long) Math.ceil(((double) inputCardinality / numInstances));
+    for (Expr expr : exprs) {
+      long exprGlobalNdv = expr.getNumDistinctValues();
+      if (exprGlobalNdv == -1) {
+        // A worst-case cardinality is better than an unknown cardinality.
+        exprGlobalNdv = inputCardinality;
+      }
+      // Estimate the per instance NDV for this expr accounting for the probability of
+      // duplicates
+      double probGivenValueIncludedInFragment = 1.0
+          - Math.pow(((double) (exprGlobalNdv - 1) / exprGlobalNdv),
+                perInstanceInputCardinality);
+      long exprPerInstanceNdv = (long) (probGivenValueIncludedInFragment * exprGlobalNdv);
+      if (LOG.isTraceEnabled()) {
+        LOG.trace("Expr per instance NDV: " + exprPerInstanceNdv
+            + ", expr: " + expr.debugString() + ", Input Cardinality: " + inputCardinality
+            + ", perInstanceInputCardinality: " + perInstanceInputCardinality);
+      }
+      // Include in combined NDV
+      result = PlanNode.checkedMultiply(result, exprPerInstanceNdv);
+    }
+
+    if (result > inputCardinality) result = inputCardinality;
+    return result;
+  }
+
   public TPlanFragment toThrift() {
     validateResourceProfiles();
     TPlanFragment result = new TPlanFragment();
@@ -610,7 +695,11 @@ public class PlanFragment extends TreeNode<PlanFragment> {
     result.setBackend_min_mem_reservation_bytes(
         perBackendResourceProfile_.getMinMemReservationBytes());
     result.setThread_reservation(perInstanceResourceProfile_.getThreadReservation());
-    result.setEffective_instance_count(getAdjustedInstanceCount());
+    if (hasAdjustedInstanceCount()) {
+      result.setEffective_instance_count(adjustedInstanceCount_);
+    }
+    result.setIs_coordinator_only(coordinatorOnly_);
+    result.setIs_dominant(isDominantFragment_);
     return result;
   }
 
@@ -748,11 +837,7 @@ public class PlanFragment extends TreeNode<PlanFragment> {
       // Print processing cost.
       builder.append(detailPrefix);
       builder.append("max-parallelism=");
-      if (maxParallelism_ > 0) {
-        builder.append(maxParallelism_);
-      } else {
-        builder.append(getAdjustedInstanceCount());
-      }
+      builder.append(maxParallelism_);
       builder.append(" segment-costs=");
       builder.append(costingSegmentSummary());
       if (thisTreeCpuCore_ > 0 && subtreeCpuCore_ > 0) {
@@ -801,6 +886,7 @@ public class PlanFragment extends TreeNode<PlanFragment> {
     planRoot_ = root;
     setFragmentInPlanTree(planRoot_);
   }
+  protected void markDominant() { isDominantFragment_ = true; }
 
   /**
    * Set the destination node of this fragment's sink, i.e. an ExchangeNode or a JoinNode.
@@ -909,6 +995,22 @@ public class PlanFragment extends TreeNode<PlanFragment> {
     return maxCostingSegment.getNumInstanceMax(getNumNodes());
   }
 
+  protected int getCappedCostBasedMaxParallelism(int minCap) {
+    int costBasedMaxParallelism = Math.max(minCap, getCostBasedMaxParallelism());
+    Preconditions.checkState(costBasedMaxParallelism > 0);
+    return costBasedMaxParallelism;
+  }
+
+  /**
+   * A fragment is either distributed evenly to all nodes or only at subset of nodes
+   * (ie., there is only 1 fragment instances with PlanRootSink and MAX_FS_WRITER
+   * fragment instances with HdfsTableSink). Increment by num nodes for the former case
+   * and 1 for the latter case.
+   */
+  protected int getNodeStepCountForParallelismTraversal() {
+    return getNumInstances() % getNumNodes() == 0 ? getNumNodes() : 1;
+  }
+
   protected boolean hasBlockingNode() {
     if (sink_ instanceof JoinBuildSink) return true;
     for (PlanNode p : collectPlanNodes()) {
@@ -948,7 +1050,23 @@ public class PlanFragment extends TreeNode<PlanFragment> {
     }
   }
 
-  protected int getAdjustedInstanceCount() { return adjustedInstanceCount_; }
+  /**
+   * Only valid to call after calling {@link #traverseEffectiveParallelism(int, int,
+   * PlanFragment)}.
+   */
+  protected int getAdjustedInstanceCount() {
+    Preconditions.checkState(adjustedInstanceCount_ > -1);
+    return adjustedInstanceCount_;
+  }
+
+  /**
+   * Only valid to call after calling {@link #traverseEffectiveParallelism(int, int,
+   * PlanFragment)}.
+   */
+  protected int getMaxParallelism() {
+    Preconditions.checkState(maxParallelism_ > -1);
+    return maxParallelism_;
+  }
 
   protected ProcessingCost getLastCostingSegment() {
     return rootSegment_.getProcessingCost();
@@ -983,7 +1101,7 @@ public class PlanFragment extends TreeNode<PlanFragment> {
     List<CostingSegment> costingSegments = rootSegment_.getNodesPreOrder();
     for (CostingSegment costingSegment : costingSegments) {
       ProcessingCost cost = costingSegment.getProcessingCost();
-      Preconditions.checkState(cost.isValid());
+      Preconditions.checkState(cost.isValid(), "Segment cost is invalid! %s", cost);
       Preconditions.checkState(
           cost.getNumInstancesExpected() == getAdjustedInstanceCount());
     }
@@ -998,18 +1116,19 @@ public class PlanFragment extends TreeNode<PlanFragment> {
    * @param maxThreadPerNode Maximum thread per fragment per node based on
    *                         {@code max(PROCESSING_COST_MIN_THREADS,
    *                         TExecutorGroupSet.num_cores_per_executor)}.
-   * @param parentParallelism Number of instance of parent fragment.
+   * @param parentFragment parent fragment of this fragment.
    */
-  protected void traverseEffectiveParallelism(
-      int minThreadPerNode, int maxThreadPerNode, int parentParallelism) {
+  protected void traverseEffectiveParallelism(final int minThreadPerNode,
+      final int maxThreadPerNode, final @Nullable PlanFragment parentFragment,
+      TQueryOptions queryOptions) {
     Preconditions.checkNotNull(
         rootSegment_, "ProcessingCost Fragment %s has not been computed!", getId());
-    int nodeStepCount = getNumInstances() % getNumNodes() == 0 ? getNumNodes() : 1;
+    int nodeStepCount = getNodeStepCountForParallelismTraversal();
 
     // step 1: Set initial parallelism to the maximum possible.
     //   Subsequent steps after this will not exceed maximum parallelism sets here.
     boolean canTryLower = adjustToMaxParallelism(
-        minThreadPerNode, maxThreadPerNode, parentParallelism, nodeStepCount);
+        minThreadPerNode, maxThreadPerNode, parentFragment, nodeStepCount, queryOptions);
 
     if (canTryLower) {
       // step 2: Try lower parallelism by comparing output ProcessingCost of the input
@@ -1039,27 +1158,84 @@ public class PlanFragment extends TreeNode<PlanFragment> {
     for (PlanFragment child : getChildren()) {
       if (child.getSink() instanceof JoinBuildSink) {
         child.traverseEffectiveParallelism(
-            minThreadPerNode, maxThreadPerNode, getAdjustedInstanceCount());
+            minThreadPerNode, maxThreadPerNode, this, queryOptions);
       }
     }
   }
 
+  protected boolean isPartitionedJoinBuildFragment() {
+    return (sink_ instanceof JoinBuildSink) && !((JoinBuildSink) sink_).isShared();
+  }
+
+  protected int getMaxParallelismForUnionFragment(
+      UnionNode node, boolean findUnboundedCount, @Nullable TQueryOptions queryOptions) {
+    Preconditions.checkState(this == node.getFragment());
+    // We set parallelism of union fragment as a max between its input fragments and
+    // its collocated ScanNode's expected parallelism
+    // (see Scheduler::CreateCollocatedAndScanInstances()).
+    // We skip any join builder child fragment here because their parallelism
+    // is not adjusted yet.
+    int nodeStepCount = getNodeStepCountForParallelismTraversal();
+    int costBasedMaxParallelism = getCappedCostBasedMaxParallelism(nodeStepCount);
+    int maxParallelism = 1;
+    for (PlanFragment child : getChildren()) {
+      if (child.getSink() instanceof JoinBuildSink) continue;
+      Preconditions.checkState(child.hasAdjustedInstanceCount());
+      maxParallelism = Math.max(maxParallelism,
+          (findUnboundedCount ? child.getMaxParallelism() :
+                                child.getAdjustedInstanceCount()));
+    }
+
+    List<ScanNode> scanNodes = Lists.newArrayList();
+    collectPlanNodes(Predicates.instanceOf(ScanNode.class), scanNodes);
+    int maxScannerThreads = 0;
+    if (!scanNodes.isEmpty()) {
+      // The existence of scan node may justify an increase of parallelism for this
+      // union fargment, but it should be capped at costBasedMaxParallelism.
+      maxScannerThreads = ScanNode.MIN_NUM_SCAN_THREADS;
+      for (ScanNode scanNode : scanNodes) {
+        // if findUnboundedCount is True, use estScanRangeAfterRuntimeFilter()
+        // because computeMaxScannerThreadsForCPC() is capped by
+        //   maxThreadsGlobal = getNumNodes() * maxThreadsPerNode.
+        // while estScanRangeAfterRuntimeFilter() is not.
+        int thisScannerThreads = ScanNode.MIN_NUM_SCAN_THREADS;
+        if (findUnboundedCount) {
+          thisScannerThreads = scanNode.estScanRangeAfterRuntimeFilter();
+        } else {
+          Preconditions.checkNotNull(queryOptions);
+          thisScannerThreads = scanNode.computeMaxScannerThreadsForCPC(queryOptions);
+        }
+        maxScannerThreads = Math.max(maxScannerThreads, thisScannerThreads);
+      }
+      maxParallelism =
+          Math.max(maxParallelism, Math.min(costBasedMaxParallelism, maxScannerThreads));
+    }
+    if (LOG.isDebugEnabled()) {
+      LOG.debug("Set maxParallelism to: " + maxParallelism + ", costBasedMaxParallelism: "
+          + costBasedMaxParallelism + ", maxScannerThreads: " + maxScannerThreads
+          + ", findUnboundedCount: " + findUnboundedCount);
+    }
+    return maxParallelism;
+  }
+
   /**
    * Adjust parallelism of this fragment to the maximum allowed.
+   * This method initialize maxParallelism_ and adjustedInstanceCount_.
    *
    * @param minThreadPerNode Minimum thread per fragment per node based on
    *                         {@code PROCESSING_COST_MIN_THREADS} query option.
    * @param maxThreadPerNode Maximum thread per fragment per node based on
    *                         {@code max(PROCESSING_COST_MIN_THREADS,
    *                         TExecutorGroupSet.num_cores_per_executor)}.
-   * @param parentParallelism Parallelism of parent fragment.
+   * @param parentFragment Parent fragment of this fragment.
    * @param nodeStepCount The step count used to increase this fragment's parallelism.
    *                      Usually equal to number of nodes or just 1.
    * @return True if it is possible to lower this fragment's parallelism through
    * ProcessingCost comparison. False if the parallelism should not be changed anymore.
    */
-  private boolean adjustToMaxParallelism(int minThreadPerNode, int maxThreadPerNode,
-      int parentParallelism, int nodeStepCount) {
+  private boolean adjustToMaxParallelism(final int minThreadPerNode,
+      final int maxThreadPerNode, final @Nullable PlanFragment parentFragment,
+      final int nodeStepCount, TQueryOptions queryOptions) {
     int maxThreadAllowed = IntMath.saturatedMultiply(maxThreadPerNode, getNumNodes());
     boolean canTryLower = true;
 
@@ -1067,46 +1243,27 @@ public class PlanFragment extends TreeNode<PlanFragment> {
     int selectedParallelism = getNumInstances();
     if (isFixedParallelism_) {
       selectedParallelism = getAdjustedInstanceCount();
+      maxParallelism_ = selectedParallelism;
       canTryLower = false;
-    } else if ((sink_ instanceof JoinBuildSink) && !((JoinBuildSink) sink_).isShared()) {
+    } else if (isPartitionedJoinBuildFragment()) {
       // This is a non-shared (PARTITIONED) join build fragment.
       // Parallelism of this fragment is equal to its parent parallelism.
-      Preconditions.checkState(parentParallelism > 0);
+      Preconditions.checkNotNull(parentFragment);
+      final int parentParallelism = parentFragment.getAdjustedInstanceCount();
       if (LOG.isTraceEnabled() && selectedParallelism != parentParallelism) {
         logCountAdjustmentTrace(selectedParallelism, parentParallelism,
             "Partitioned join build fragment follow parent's parallelism.");
       }
       selectedParallelism = parentParallelism;
+      maxParallelism_ = parentFragment.getMaxParallelism();
       canTryLower = false; // no need to compute effective parallelism anymore.
     } else {
-      int costBasedMaxParallelism = Math.max(nodeStepCount, getCostBasedMaxParallelism());
+      UnionNode unionNode = getUnionNode();
 
-      if (hasUnionNode()) {
-        // We set parallelism of union fragment as a max between its input fragments and
-        // its collocated ScanNode's expected parallelism
-        // (see Scheduler::CreateCollocatedAndScanInstances()).
-        // We skip any join builder child fragment here because their parallelism
-        // is not adjusted yet.
-        maxParallelism_ = 1;
-        for (PlanFragment child : getChildren()) {
-          if (child.getSink() instanceof JoinBuildSink) continue;
-          Preconditions.checkState(child.hasAdjustedInstanceCount());
-          maxParallelism_ = Math.max(maxParallelism_, child.getAdjustedInstanceCount());
-        }
-
-        List<ScanNode> scanNodes = Lists.newArrayList();
-        collectPlanNodes(Predicates.instanceOf(ScanNode.class), scanNodes);
-        if (!scanNodes.isEmpty()) {
-          // The existence of scan node may justify an increase of parallelism for this
-          // union fargment, but it should be capped at costBasedMaxParallelism.
-          long maxRangesPerScanNode = 1;
-          for (ScanNode scanNode : scanNodes) {
-            maxRangesPerScanNode =
-                Math.max(maxRangesPerScanNode, scanNode.getEffectiveNumScanRanges());
-          }
-          maxParallelism_ = Math.max(maxParallelism_,
-              (int) Math.min(costBasedMaxParallelism, maxRangesPerScanNode));
-        }
+      if (unionNode != null) {
+        // Special handling for Union fragment.
+        maxParallelism_ =
+            getMaxParallelismForUnionFragment(unionNode, false, queryOptions);
 
         if (maxParallelism_ > maxThreadAllowed) {
           selectedParallelism = maxThreadAllowed;
@@ -1118,7 +1275,7 @@ public class PlanFragment extends TreeNode<PlanFragment> {
           selectedParallelism = maxParallelism_;
           if (LOG.isTraceEnabled()) {
             logCountAdjustmentTrace(getNumInstances(), selectedParallelism,
-                "Follow minimum work per thread.");
+                "Follow minimum work per thread or max child count.");
           }
         }
         canTryLower = false;
@@ -1126,7 +1283,13 @@ public class PlanFragment extends TreeNode<PlanFragment> {
         // This is an interior fragment or fragment with single scan node.
         // We calculate maxParallelism_, minParallelism, and selectedParallelism across
         // all executors in the selected executor group.
+        int maxScannerThreads = Integer.MAX_VALUE;
+        int costBasedMaxParallelism = getCappedCostBasedMaxParallelism(nodeStepCount);
+
+        // track maximum parallelism unbounded by minThreadPerNode and maxThreadPerNode.
         maxParallelism_ = costBasedMaxParallelism;
+        // track maximum parallelism bounded by minThreadPerNode and maxThreadPerNode.
+        int boundedParallelism = costBasedMaxParallelism;
 
         // Bound maxParallelism_ by ScanNode's effective scan range count if
         // this fragment has ScanNode.
@@ -1134,9 +1297,18 @@ public class PlanFragment extends TreeNode<PlanFragment> {
         collectPlanNodes(Predicates.instanceOf(ScanNode.class), scanNodes);
         if (!scanNodes.isEmpty()) {
           Preconditions.checkState(scanNodes.size() == 1);
-          ScanNode scanNode = scanNodes.get(0);
-          maxParallelism_ =
-              (int) Math.min(maxParallelism_, scanNode.getEffectiveNumScanRanges());
+          maxScannerThreads =
+              scanNodes.get(0).computeMaxScannerThreadsForCPC(queryOptions);
+          maxParallelism_ = Math.max(ScanNode.MIN_NUM_SCAN_THREADS,
+              Math.min(
+                  maxParallelism_, scanNodes.get(0).estScanRangeAfterRuntimeFilter()));
+          boundedParallelism = Math.max(ScanNode.MIN_NUM_SCAN_THREADS,
+              Math.min(boundedParallelism, maxScannerThreads));
+          if (LOG.isDebugEnabled()) {
+            LOG.debug("Set maxParallelism to: " + maxParallelism_
+                + ", costBasedMaxParallelism: " + costBasedMaxParallelism
+                + ", maxScannerThreads: " + maxScannerThreads);
+          }
 
           // Prevent caller from lowering parallelism if fragment has ScanNode
           // because there is no child fragment to compare with.
@@ -1145,26 +1317,28 @@ public class PlanFragment extends TreeNode<PlanFragment> {
 
         int minParallelism = Math.min(
             maxThreadAllowed, IntMath.saturatedMultiply(minThreadPerNode, getNumNodes()));
+        LOG.info("maxParallelism_=" + maxParallelism_ + " boundedParallelism="
+            + boundedParallelism + " minParallelism=" + minParallelism);
 
-        if (maxParallelism_ > maxThreadAllowed) {
+        if (boundedParallelism > maxThreadAllowed) {
           selectedParallelism = maxThreadAllowed;
           if (LOG.isTraceEnabled()) {
             logCountAdjustmentTrace(
                 getNumInstances(), selectedParallelism, "Follow maxThreadPerNode.");
           }
         } else {
-          if (maxParallelism_ < minParallelism && scanNodes.isEmpty()) {
-            maxParallelism_ = minParallelism;
+          if (boundedParallelism < minParallelism && minParallelism < maxScannerThreads) {
+            boundedParallelism = minParallelism;
             canTryLower = false;
             if (LOG.isTraceEnabled()) {
               logCountAdjustmentTrace(
-                  getNumInstances(), maxParallelism_, "Follow minThreadPerNode.");
+                  getNumInstances(), boundedParallelism, "Follow minThreadPerNode.");
             }
           } else if (LOG.isTraceEnabled()) {
             logCountAdjustmentTrace(
-                getNumInstances(), maxParallelism_, "Follow minimum work per thread.");
+                getNumInstances(), boundedParallelism, "Follow minimum work per thread.");
           }
-          selectedParallelism = maxParallelism_;
+          selectedParallelism = boundedParallelism;
         }
       }
     }
@@ -1177,44 +1351,54 @@ public class PlanFragment extends TreeNode<PlanFragment> {
 
     // Initialize this fragment's parallelism to the selectedParallelism.
     setAdjustedInstanceCount(selectedParallelism);
-    return canTryLower;
+    return canTryLower && selectedParallelism > 1;
   }
 
-  private boolean hasUnionNode() {
-    List<ScanNode> nodes = Lists.newArrayList();
+  /**
+   * Get the top-most UnionNode of this fragment.
+   * If there is none, return null.
+   */
+  protected @Nullable UnionNode getUnionNode() {
+    List<PlanNode> nodes = Lists.newArrayList();
     collectPlanNodes(Predicates.instanceOf(UnionNode.class), nodes);
-    return !nodes.isEmpty();
-  }
-
-  private boolean hasScanNode() {
-    List<ScanNode> nodes = Lists.newArrayList();
-    collectPlanNodes(Predicates.instanceOf(ScanNode.class), nodes);
-    return !nodes.isEmpty();
+    return nodes.isEmpty() ? null : ((UnionNode) nodes.get(0));
   }
 
   /**
    * Compute {@link CoreCount} of this fragment and populate it into 'fragmentCoreState'.
    * @param fragmentCoreState A map holding per-fragment core state.
+   * @param findUnboundedCount if True, return the unbounded core count.
+   *                           Otherwise, return the bounded one.
    * All successor of this fragment must already have its CoreCount registered into this
    * map.
    */
   protected void computeBlockingAwareCores(
-      Map<PlanFragmentId, Pair<CoreCount, List<CoreCount>>> fragmentCoreState) {
+      Map<PlanFragmentId, Pair<CoreCount, List<CoreCount>>> fragmentCoreState,
+      boolean findUnboundedCount) {
     Preconditions.checkNotNull(
         rootSegment_, "ProcessingCost Fragment %s has not been computed!", getId());
     ImmutableList.Builder<CoreCount> subtreeCoreBuilder =
         new ImmutableList.Builder<CoreCount>();
-    CoreCount coreReq =
-        rootSegment_.traverseBlockingAwareCores(fragmentCoreState, subtreeCoreBuilder);
+    CoreCount coreReq = rootSegment_.traverseBlockingAwareCores(
+        fragmentCoreState, subtreeCoreBuilder, findUnboundedCount);
     fragmentCoreState.put(getId(), Pair.create(coreReq, subtreeCoreBuilder.build()));
   }
 
-  protected CoreCount maxCore(CoreCount thisTreeCpuCore, CoreCount subtreeCpuCore) {
-    thisTreeCpuCore_ = thisTreeCpuCore.total();
-    subtreeCpuCore_ = subtreeCpuCore.total();
+  protected CoreCount maxCore(
+      CoreCount thisTreeCpuCore, CoreCount subtreeCpuCore, boolean findUnboundedCount) {
+    if (!findUnboundedCount) {
+      // Only initialize thisTreeCpuCore_ and subtreeCpuCore_ when doing bounded counting.
+      // This will be displayed in query profile to show how the maximum selection goes,
+      // such as:
+      //   cpu-comparison-result=220 [max(120 (self) vs 220 (sum children))]
+      // "self" is 'thisTreeCpuCore_' while "sum children" is 'subtreeCpuCore_'.
+      thisTreeCpuCore_ = thisTreeCpuCore.total();
+      subtreeCpuCore_ = subtreeCpuCore.total();
+    }
     if (LOG.isTraceEnabled()) {
-      LOG.trace("At {}, compare {} ({}) vs {} ({})", getId(), thisTreeCpuCore,
-          thisTreeCpuCore.total(), subtreeCpuCore, subtreeCpuCore.total());
+      LOG.trace("At {}, do {} comparison of {} ({}) vs {} ({})", getId(), thisTreeCpuCore,
+          (findUnboundedCount ? "unbounded" : "bounded"), thisTreeCpuCore.total(),
+          subtreeCpuCore, subtreeCpuCore.total());
     }
     return CoreCount.max(thisTreeCpuCore, subtreeCpuCore);
   }
@@ -1231,14 +1415,13 @@ public class PlanFragment extends TreeNode<PlanFragment> {
       originalInstanceCount_ = getNumInstances();
     }
 
-    if (LOG.isTraceEnabled() && originalInstanceCount_ != getAdjustedInstanceCount()) {
-      logCountAdjustmentTrace(originalInstanceCount_, getAdjustedInstanceCount(),
-          "Finalize effective parallelism.");
+    int adjustedCount = getAdjustedInstanceCount();
+    if (LOG.isTraceEnabled() && originalInstanceCount_ != adjustedCount) {
+      logCountAdjustmentTrace(
+          originalInstanceCount_, adjustedCount, "Finalize effective parallelism.");
     }
 
-    for (PlanNode node : collectPlanNodes()) {
-      node.numInstances_ = getAdjustedInstanceCount();
-    }
+    for (PlanNode node : collectPlanNodes()) { node.numInstances_ = adjustedCount; }
 
     if (LOG.isTraceEnabled()) {
       LOG.trace("ProcessingCost Fragment {}:\n{}", getId(), debugProcessingCosts());
