@@ -21,23 +21,30 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
-import java.util.Iterator;
+import java.util.HashMap;
 import java.util.HashSet;
+import java.util.Iterator;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
+import java.util.Stack;
 
 import org.apache.impala.analysis.Analyzer;
 import org.apache.impala.analysis.BinaryPredicate;
+import org.apache.impala.analysis.CompoundPredicate;
+import org.apache.impala.analysis.DescriptorTable;
 import org.apache.impala.analysis.Expr;
 import org.apache.impala.analysis.ExprId;
 import org.apache.impala.analysis.ExprSubstitutionMap;
 import org.apache.impala.analysis.SlotDescriptor;
+import org.apache.impala.analysis.SlotId;
 import org.apache.impala.analysis.SlotRef;
 import org.apache.impala.analysis.ToSqlOptions;
 import org.apache.impala.analysis.TupleDescriptor;
 import org.apache.impala.analysis.TupleId;
 import org.apache.impala.common.ImpalaException;
 import org.apache.impala.common.PrintUtils;
+import org.apache.impala.common.ThriftSerializationCtx;
 import org.apache.impala.common.TreeNode;
 import org.apache.impala.planner.RuntimeFilterGenerator.RuntimeFilter;
 import org.apache.impala.thrift.TExecNodePhase;
@@ -123,10 +130,16 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
   // TODO for 2.3: Save this state in the PlannerContext instead.
   protected Set<ExprId> assignedConjuncts_;
 
-  // estimate of the output cardinality of this node; set in computeStats();
+  // estimate of the output cardinality of this node; set in computeStats() and can be
+  // replaced through filterCardinality();
   // invalid: -1
   protected long cardinality_;
 
+  // Estimate of the output cardinality of this node after reduction by runtime filter.
+  // Set in setFilteredCardinality(). Valid value is > -1.
+  // TODO: merge this with cardinality_.
+  protected long filteredCardinality_ = -1;
+
   // Estimated number of nodes on which the plan tree rooted at this node would be
   // scheduled;
   // set in computeStats(); invalid: -1
@@ -148,6 +161,10 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
   // sum of tupleIds_' avgSerializedSizes; set in computeStats()
   protected float avgRowSize_;
 
+  // Sum of tupleIds_' padding bytes that is included in avgRowSize_.
+  // Set in computeStats().
+  protected float rowPadSize_;
+
   // If true, disable codegen for this plan node.
   protected boolean disableCodegen_;
 
@@ -163,6 +180,8 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
   // is involved.
   protected boolean hasHardEstimates_ = false;
 
+  protected TupleCacheInfo tupleCacheInfo_;
+
   protected PlanNode(PlanNodeId id, List<TupleId> tupleIds, String displayName) {
     this(id, displayName);
     tupleIds_.addAll(tupleIds);
@@ -235,6 +254,10 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
   public int getNumInstances() { return numInstances_; }
   public ResourceProfile getNodeResourceProfile() { return nodeResourceProfile_; }
   public float getAvgRowSize() { return avgRowSize_; }
+  public float getRowPadSize() { return rowPadSize_; }
+  public float getAvgRowSizeWithoutPad() {
+    return Math.max(0.0F, avgRowSize_ - rowPadSize_);
+  }
   public void setFragment(PlanFragment fragment) { fragment_ = fragment; }
   public PlanFragment getFragment() { return fragment_; }
   public List<Expr> getConjuncts() { return conjuncts_; }
@@ -376,8 +399,15 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
       if (detailLevel == TExplainLevel.STANDARD) expBuilder.append(detailPrefix);
       expBuilder.append("row-size=")
           .append(PrintUtils.printBytes(Math.round(avgRowSize_)))
-          .append(" cardinality=")
-          .append(PrintUtils.printEstCardinality(cardinality_));
+          .append(" cardinality=");
+      if (filteredCardinality_ > -1) {
+        expBuilder.append(PrintUtils.printEstCardinality(filteredCardinality_))
+            .append("(filtered from ")
+            .append(PrintUtils.printEstCardinality(cardinality_))
+            .append(")");
+      } else {
+        expBuilder.append(PrintUtils.printEstCardinality(cardinality_));
+      }
       if (queryOptions.isCompute_processing_cost()) {
         // Show processing cost total.
         expBuilder.append(" cost=");
@@ -476,42 +506,68 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
     return result;
   }
 
-  // Append a flattened version of this plan node, including all children in the same
-  // fragment, to 'container'.
-  private void treeToThriftHelper(TPlan container) {
-    TPlanNode msg = new TPlanNode();
-    msg.node_id = id_.asInt();
+  /**
+   * Common initialization of TPlanNode used for all different types of PlanNodes.
+   * This is called before calling toThrift().
+   */
+  private void initThrift(TPlanNode msg, ThriftSerializationCtx serialCtx) {
     msg.limit = limit_;
 
-    TExecStats estimatedStats = new TExecStats();
-    estimatedStats.setCardinality(cardinality_);
-    estimatedStats.setMemory_used(nodeResourceProfile_.getMemEstimateBytes());
-    msg.setLabel(getDisplayLabel());
-    msg.setLabel_detail(getDisplayLabelDetail());
-    msg.setEstimated_stats(estimatedStats);
+    if (!serialCtx.isTupleCache()) {
+      msg.node_id = id_.asInt();
+      TExecStats estimatedStats = new TExecStats();
+      estimatedStats.setCardinality(
+          filteredCardinality_ > -1 ? filteredCardinality_ : cardinality_);
+      estimatedStats.setMemory_used(nodeResourceProfile_.getMemEstimateBytes());
+      msg.setLabel(getDisplayLabel());
+      msg.setLabel_detail(getDisplayLabelDetail());
+      msg.setEstimated_stats(estimatedStats);
+    } else {
+      // Do not set node_id for tuple caching, as it is a global index that is impacted
+      // by the shape of the rest of the query.
+      msg.node_id = 0;
+    }
 
     Preconditions.checkState(tupleIds_.size() > 0);
     msg.setRow_tuples(Lists.<Integer>newArrayListWithCapacity(tupleIds_.size()));
     msg.setNullable_tuples(Lists.<Boolean>newArrayListWithCapacity(tupleIds_.size()));
     for (TupleId tid: tupleIds_) {
-      msg.addToRow_tuples(tid.asInt());
+      serialCtx.registerTuple(tid);
+      msg.addToRow_tuples(serialCtx.translateTupleId(tid).asInt());
       msg.addToNullable_tuples(nullableTupleIds_.contains(tid));
     }
     for (Expr e: conjuncts_) {
-      msg.addToConjuncts(e.treeToThrift());
+      msg.addToConjuncts(e.treeToThrift(serialCtx));
     }
     // Serialize any runtime filters
     for (RuntimeFilter filter : runtimeFilters_) {
       msg.addToRuntime_filters(filter.toThrift());
     }
     msg.setDisable_codegen(disableCodegen_);
-    Preconditions.checkState(nodeResourceProfile_.isValid());
-    msg.resource_profile = nodeResourceProfile_.toThrift();
     msg.pipelines = new ArrayList<>();
-    for (PipelineMembership pipe : pipelines_) {
-      msg.pipelines.add(pipe.toThrift());
+    if (serialCtx.isTupleCache()) {
+      // At the point when TupleCachePlanner runs, the pipelines and resource profile
+      // have not been calculated yet. These should not be in the cache key anyway, so
+      // mask them out.
+      //
+      // resource_profile is a required field, so use a generic struct.
+      msg.resource_profile = ResourceProfile.noReservation(0).toThrift();
+    } else {
+      Preconditions.checkState(nodeResourceProfile_.isValid());
+      msg.resource_profile = nodeResourceProfile_.toThrift();
+      for (PipelineMembership pipe : pipelines_) {
+        msg.pipelines.add(pipe.toThrift());
+      }
     }
-    toThrift(msg);
+  }
+
+  // Append a flattened version of this plan node, including all children in the same
+  // fragment, to 'container'.
+  private void treeToThriftHelper(TPlan container) {
+    TPlanNode msg = new TPlanNode();
+    ThriftSerializationCtx serialCtx = new ThriftSerializationCtx();
+    initThrift(msg, serialCtx);
+    toThrift(msg, serialCtx);
     container.addToNodes(msg);
     // For the purpose of the BE consider cross-fragment children (i.e.
     // ExchangeNodes and separated join builds) to have no children.
@@ -662,9 +718,11 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
    */
   public void computeStats(Analyzer analyzer) {
     avgRowSize_ = 0.0F;
+    rowPadSize_ = 0.0F;
     for (TupleId tid: tupleIds_) {
       TupleDescriptor desc = analyzer.getTupleDesc(tid);
       avgRowSize_ += desc.getAvgSerializedSize();
+      rowPadSize_ += desc.getSerializedPadSize();
     }
     if (!children_.isEmpty()) {
       numNodes_ = getChild(0).numNodes_;
@@ -705,19 +763,59 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
    * 1. The individual selectivities of conjuncts may be unknown.
    * 2. Two selectivities, whether known or unknown, could be correlated. Assuming
    *    independence can lead to significant underestimation.
+   * 3. Two BinaryPredicate may be derived from single BetweenPredicate that have
+   *    lower selectivity if analyzed as a pair.
    *
    * The first issue is addressed by using a single default selectivity that is
    * representative of all conjuncts with unknown selectivities.
    * The second issue is addressed by an exponential backoff when multiplying each
    * additional selectivity into the final result.
+   * The third issue is addressed by checking BinaryPredicate.derivedFromBetween()
+   * property. If it is True, calculate the expression selectivity from
+   * BinaryPredicate.getBetweenSelectivity(). Both of the BinaryPredicates should have
+   * the same getBetweenSelectivity() value, thus it is OK to inspect just from one of
+   * them.
+   * TODO: Fix the third issue with more general solution when there are multiple range
+   * predicates involved, even if some of them are not derived from BetweenPredicate.
    */
   static protected double computeCombinedSelectivity(List<Expr> conjuncts) {
     // Collect all estimated selectivities.
     List<Double> selectivities = new ArrayList<>();
+    // Map between a slot id and the lowest between selectivity targeting that slot.
+    Map<SlotId, Double> perSlotBetweenSelectivities = new HashMap<>();
+
+    int conjunctSize = 0;
     for (Expr e: conjuncts) {
-      if (e.hasSelectivity()) selectivities.add(e.getSelectivity());
+      if (e instanceof BinaryPredicate && ((BinaryPredicate) e).derivedFromBetween()) {
+        // This is one of two BinaryPredicate that derived from a BetweenPredicate.
+        // A pair of BetweenPredicate must have been assigned and pushed down to the same
+        // 'targetSlotId', or one of them maybe removed by Expr.removeDuplicates()
+        // somewhere by Analyzer. But it is enough to get the selectivity from one of
+        // them. Analyzer.getBoundPredicates() ensure that BinaryPredicate derived from
+        // BetweenPredicate gets priority over regular BinaryPredicate.
+        // If multiple BetweenPredicate target the same 'targetSlotId', pick the least
+        // selectivity.
+        BinaryPredicate pred = (BinaryPredicate) e;
+        insertBetweenSelectivity(pred, perSlotBetweenSelectivities);
+      } else if (e instanceof CompoundPredicate
+          && ((CompoundPredicate) e).derivedFromBetween()
+          && (((CompoundPredicate) e).getChild(0) instanceof BinaryPredicate)) {
+        BinaryPredicate pred = (BinaryPredicate) ((CompoundPredicate) e).getChild(0);
+        insertBetweenSelectivity(pred, perSlotBetweenSelectivities);
+      } else {
+        // For everything else.
+        if (e.hasSelectivity()) selectivities.add(e.getSelectivity());
+        conjunctSize++;
+      }
     }
-    if (selectivities.size() != conjuncts.size()) {
+
+    // Add all values from perSlotBetweenSelectivities to selectivities.
+    if (!perSlotBetweenSelectivities.isEmpty()) {
+      selectivities.addAll(perSlotBetweenSelectivities.values());
+      conjunctSize += perSlotBetweenSelectivities.size();
+    }
+
+    if (selectivities.size() != conjunctSize) {
       // Some conjuncts have no estimated selectivity. Use a single default
       // representative selectivity for all those conjuncts.
       selectivities.add(Expr.DEFAULT_SELECTIVITY);
@@ -735,6 +833,15 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
     return Math.max(0.0, Math.min(1.0, result));
   }
 
+  static private void insertBetweenSelectivity(
+      BinaryPredicate pred, Map<SlotId, Double> perSlotBetweenSelectivities) {
+    Preconditions.checkNotNull(pred.getBoundSlot());
+    Preconditions.checkState(pred.getBetweenSelectivity() >= 0.0);
+    SlotId targetSlotId = pred.getBoundSlot().getSlotId();
+    double sel = pred.getBetweenSelectivity();
+    perSlotBetweenSelectivities.merge(targetSlotId, sel, (v1, v2) -> Math.min(v1, v2));
+  }
+
   protected double computeSelectivity() {
     return computeCombinedSelectivity(conjuncts_);
   }
@@ -753,6 +860,14 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
     return cardinality;
   }
 
+  protected void toThrift(TPlanNode msg, ThriftSerializationCtx serialCtx) {
+    // This should only be used for PlanNodes that don't support tuple
+    // caching. Any PlanNode that supports tuple caching must override
+    // this method with its own implementation.
+    Preconditions.checkState(!isTupleCachingImplemented());
+    toThrift(msg);
+  }
+
   // Convert this plan node into msg (excluding children), which requires setting
   // the node type and the node-specific field.
   protected abstract void toThrift(TPlanNode msg);
@@ -930,7 +1045,8 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
    */
   public void computeRowConsumptionAndProductionToCost() {
     Preconditions.checkState(processingCost_.isValid(),
-        "Processing cost of PlanNode " + getDisplayLabel() + " is invalid!");
+        "Processing cost of PlanNode %s is invalid! %s", getDisplayLabel(),
+        processingCost_);
     processingCost_.setNumRowToConsume(getInputCardinality());
     processingCost_.setNumRowToProduce(getCardinality());
   }
@@ -1157,4 +1273,82 @@ abstract public class PlanNode extends TreeNode<PlanNode> {
     }
     return numCols;
   }
+
+  protected void setFilteredCardinality(long newCardinality) {
+    Preconditions.checkState(newCardinality > -1);
+    Preconditions.checkState(newCardinality < cardinality_);
+    filteredCardinality_ = newCardinality;
+  }
+
+  // May return -1.
+  // TODO: merge this with getCardinality().
+  protected long getFilteredCardinality() {
+    return filteredCardinality_ > -1 ? filteredCardinality_ : getCardinality();
+  }
+
+  /**
+   * Find contiguous probe pipeline consisting of ScanNode, ExchangeNodes, and JoinNodes
+   * and reduce their cardinality with assumption that runtime filter targeting
+   * the ScanNode can filter out rows effectively (as selective as the originating
+   * join node) before the join operation. 'reductionScale' is a [0.0..1.0] range to
+   * control the scale of reduction. Higher value means more reduction (lower cardinality
+   * estimate).
+   */
+  protected void reduceCardinalityByRuntimeFilter(
+      Stack<PlanNode> nodeStack, double reductionScale) {
+    nodeStack.clear();
+    for (PlanNode child : getChildren()) {
+      child.reduceCardinalityByRuntimeFilter(nodeStack, reductionScale);
+    }
+  }
+
+  public TupleCacheInfo getTupleCacheInfo() {
+    return tupleCacheInfo_;
+  }
+
+  /**
+   * Compute the tuple cache eligibility and keys via a bottom-up tree traversal.
+   *
+   * In order to be eligible, a node's children need to all be eligible and the node
+   * must explicitly support tuple caching. TODO: The computation of eligibility will
+   * incorporate more information about whether the computation is non-deterministic.
+   *
+   * This computes the cache key by hashing Thrift structures, but it only computes
+   * the key if the node is eligible to avoid overhead.
+   */
+  public void computeTupleCacheInfo(DescriptorTable descTbl) {
+    tupleCacheInfo_ = new TupleCacheInfo(descTbl);
+    // computing the tuple cache information is a bottom-up tree traversal,
+    // so visit and merge the children before processing this node's contents
+    for (int i = 0; i < getChildCount(); i++) {
+      getChild(i).computeTupleCacheInfo(descTbl);
+      tupleCacheInfo_.mergeChild(getChild(i).getTupleCacheInfo());
+    }
+
+    if (!isTupleCachingImplemented()) {
+      tupleCacheInfo_.setIneligible(TupleCacheInfo.IneligibilityReason.NOT_IMPLEMENTED);
+    }
+
+    // If we are already ineligible, there is no need to process the Thrift
+    if (!tupleCacheInfo_.isEligible()) {
+      tupleCacheInfo_.finalize();
+      return;
+    }
+
+    // Incorporate this node's information
+    // TODO: This will also calculate eligibility via initThrift/toThrift.
+    // TODO: This will adjust the output of initThrift/toThrift to mask out items.
+    TPlanNode msg = new TPlanNode();
+    ThriftSerializationCtx serialCtx = new ThriftSerializationCtx(tupleCacheInfo_);
+    initThrift(msg, serialCtx);
+    toThrift(msg, serialCtx);
+    tupleCacheInfo_.hashThrift(msg);
+    tupleCacheInfo_.finalize();
+  }
+
+  /**
+   * Whether this node supports tuple caching. All nodes start ineligible and then
+   * need to explicitly enable it.
+   */
+  public boolean isTupleCachingImplemented() { return false; }
 }
