@@ -19,16 +19,20 @@ package org.apache.impala.planner;
 
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
+import java.util.Stack;
 import java.util.stream.Collectors;
 
 import org.apache.impala.analysis.AnalysisContext;
 import org.apache.impala.analysis.AnalysisContext.AnalysisResult;
 import org.apache.impala.analysis.Analyzer;
 import org.apache.impala.analysis.ColumnLineageGraph;
-import org.apache.impala.analysis.DeleteStmt;
 import org.apache.impala.analysis.ColumnLineageGraph.ColumnLabel;
+import org.apache.impala.analysis.DeleteStmt;
+import org.apache.impala.analysis.DmlStatementBase;
 import org.apache.impala.analysis.Expr;
 import org.apache.impala.analysis.ExprSubstitutionMap;
 import org.apache.impala.analysis.InsertStmt;
@@ -52,7 +56,6 @@ import org.apache.impala.thrift.TQueryCtx;
 import org.apache.impala.thrift.TQueryExecRequest;
 import org.apache.impala.thrift.TQueryOptions;
 import org.apache.impala.thrift.TRuntimeFilterMode;
-import org.apache.impala.thrift.TSortingOrder;
 import org.apache.impala.thrift.TTableName;
 import org.apache.impala.util.EventSequence;
 import org.apache.impala.util.KuduUtil;
@@ -160,7 +163,7 @@ public class Planner {
       InsertStmt insertStmt = ctx_.getAnalysisResult().getInsertStmt();
       insertStmt.substituteResultExprs(rootNodeSmap, ctx_.getRootAnalyzer());
       if (!ctx_.isSingleNodeExec()) {
-        // repartition on partition keys
+        // Repartition on partition keys
         rootFragment = distributedPlanner.createDmlFragment(
             rootFragment, insertStmt, ctx_.getRootAnalyzer(), fragments);
       }
@@ -168,30 +171,29 @@ public class Planner {
       createPreInsertSort(insertStmt, rootFragment, ctx_.getRootAnalyzer());
       // set up table sink for root fragment
       rootFragment.setSink(insertStmt.createDataSink());
-    } else {
+    } else if (ctx_.isUpdate() || ctx_.isDelete() || ctx_.isOptimize()) {
+      DmlStatementBase stmt;
+      if (ctx_.isUpdate()) {
+        stmt = ctx_.getAnalysisResult().getUpdateStmt();
+      } else if (ctx_.isDelete()) {
+        stmt = ctx_.getAnalysisResult().getDeleteStmt();
+      } else {
+        stmt = ctx_.getAnalysisResult().getOptimizeStmt();
+      }
+      Preconditions.checkNotNull(stmt);
+      stmt.substituteResultExprs(rootNodeSmap, ctx_.getRootAnalyzer());
+      if (stmt.getTargetTable() instanceof FeIcebergTable) {
+        rootFragment = createIcebergDmlPlanFragment(
+            rootFragment, distributedPlanner, stmt, fragments);
+      }
+      // Set up update sink for root fragment
+      rootFragment.setSink(stmt.createDataSink());
+    } else if (ctx_.isQuery()) {
       QueryStmt queryStmt = ctx_.getQueryStmt();
       queryStmt.substituteResultExprs(rootNodeSmap, ctx_.getRootAnalyzer());
       List<Expr> resultExprs = queryStmt.getResultExprs();
-      if (ctx_.isUpdate()) {
-        // Set up update sink for root fragment
-        rootFragment.setSink(
-            ctx_.getAnalysisResult().getUpdateStmt().createDataSink(resultExprs));
-      } else if (ctx_.isDelete()) {
-        // Set up delete sink for root fragment
-        DeleteStmt deleteStmt = ctx_.getAnalysisResult().getDeleteStmt();
-        if (deleteStmt.getTargetTable() instanceof FeIcebergTable) {
-          if (!ctx_.isSingleNodeExec()) {
-            // repartition on partition keys
-            rootFragment = distributedPlanner.createDmlFragment(
-                rootFragment, deleteStmt, ctx_.getRootAnalyzer(), fragments);
-          }
-          createPreDeleteSort(deleteStmt, rootFragment, ctx_.getRootAnalyzer());
-        }
-        rootFragment.setSink(deleteStmt.createDataSink());
-      } else if (ctx_.isQuery()) {
-        rootFragment.setSink(
-            ctx_.getAnalysisResult().getQueryStmt().createDataSink(resultExprs));
-      }
+      rootFragment.setSink(
+          ctx_.getAnalysisResult().getQueryStmt().createDataSink(resultExprs));
     }
 
     // The check for disabling codegen uses estimates of rows per node so must be done
@@ -210,6 +212,7 @@ public class Planner {
 
     Collections.reverse(fragments);
     ctx_.getTimeline().markEvent("Distributed plan created");
+    ctx_.getRootAnalyzer().logCacheStats();
 
     ColumnLineageGraph graph = ctx_.getRootAnalyzer().getColumnLineageGraph();
     if (BackendConfig.INSTANCE.getComputeLineage() || RuntimeEnv.INSTANCE.isTestEnv()) {
@@ -271,6 +274,23 @@ public class Planner {
     return fragments;
   }
 
+  public PlanFragment createIcebergDmlPlanFragment(PlanFragment rootFragment,
+      DistributedPlanner distributedPlanner, DmlStatementBase stmt,
+      List<PlanFragment> fragments) throws ImpalaException {
+    if (!ctx_.isSingleNodeExec()) {
+      // Repartition on partition keys
+      rootFragment = distributedPlanner.createDmlFragment(
+          rootFragment, stmt, ctx_.getRootAnalyzer(), fragments);
+    }
+    // We don't need to add a SORT node for DELETE operations as we are using the
+    // IcebergBufferedDeleteSink. UPDATE/MERGE statements will still require to
+    // sort their data records.
+    if (!(stmt instanceof DeleteStmt)) {
+      createPreDmlSort(stmt, rootFragment, ctx_.getRootAnalyzer());
+    }
+    return rootFragment;
+  }
+
   /**
    * Return a list of plans, each represented by the root of their fragment trees. May
    * return a single-node, distributed, or parallel plan depending on the query and
@@ -279,13 +299,23 @@ public class Planner {
   public List<PlanFragment> createPlans() throws ImpalaException {
     List<PlanFragment> distrPlan = createPlanFragments();
     Preconditions.checkNotNull(distrPlan);
-    if (!useParallelPlan(ctx_)) {
-      return Collections.singletonList(distrPlan.get(0));
+    if (useParallelPlan(ctx_)) {
+      ParallelPlanner parallelPlanner = new ParallelPlanner(ctx_);
+      distrPlan = parallelPlanner.createPlans(distrPlan.get(0));
+      ctx_.getTimeline().markEvent("Parallel plans created");
+    } else {
+      distrPlan = Collections.singletonList(distrPlan.get(0));
     }
-    ParallelPlanner planner = new ParallelPlanner(ctx_);
-    List<PlanFragment> parallelPlans = planner.createPlans(distrPlan.get(0));
-    ctx_.getTimeline().markEvent("Parallel plans created");
-    return parallelPlans;
+    // TupleCachePlanner comes last, because it needs to compute the eligibility of
+    // various locations in the PlanNode tree. Runtime filters and other modifications
+    // to the tree can change this, so this comes after all those modifications are
+    // complete.
+    if (useTupleCache(ctx_)) {
+      TupleCachePlanner cachePlanner = new TupleCachePlanner(ctx_);
+      distrPlan = cachePlanner.createPlans(distrPlan);
+      ctx_.getTimeline().markEvent("Tuple caching plan created");
+    }
+    return distrPlan;
   }
 
   /**
@@ -302,6 +332,11 @@ public class Planner {
     return queryOptions.getMt_dop() > 0 || queryOptions.isCompute_processing_cost();
   }
 
+  // Return true if ENABLE_TUPLE_CACHE=true
+  public static boolean useTupleCache(PlannerContext planCtx) {
+    return planCtx.getQueryOptions().isEnable_tuple_cache();
+  }
+
   /**
    * Return combined explain string for all plan fragments.
    * Includes the estimated resource requirements from the request if set.
@@ -441,8 +476,8 @@ public class Planner {
    * Only valid after {@link PlanFragment#computeCostingSegment(TQueryOptions)}
    * has been called for all plan fragments in the list.
    */
-  private static void computeEffectiveParallelism(
-      List<PlanFragment> postOrderFragments, int minThreadPerNode, int maxThreadPerNode) {
+  private static void computeEffectiveParallelism(List<PlanFragment> postOrderFragments,
+      int minThreadPerNode, int maxThreadPerNode, TQueryOptions queryOptions) {
     Preconditions.checkArgument(minThreadPerNode > 0);
     Preconditions.checkArgument(maxThreadPerNode >= minThreadPerNode);
     Preconditions.checkArgument(
@@ -451,7 +486,8 @@ public class Planner {
       if (!(fragment.getSink() instanceof JoinBuildSink)) {
         // Only adjust parallelism of non-join build fragment.
         // Join build fragment will be adjusted later by fragment hosting the join node.
-        fragment.traverseEffectiveParallelism(minThreadPerNode, maxThreadPerNode, -1);
+        fragment.traverseEffectiveParallelism(
+            minThreadPerNode, maxThreadPerNode, null, queryOptions);
       }
     }
 
@@ -464,11 +500,11 @@ public class Planner {
    * This method returns the effective CPU requirement of a query when considering
    * processing cost rate and blocking operator.
    * <p>
-   * Only valid after {@link #computeEffectiveParallelism(List, int, int)} has
-   * been called over the plan fragment list.
+   * Only valid after {@link #computeEffectiveParallelism(List, int, int, TQueryOptions)}
+   * has been called over the plan fragment list.
    */
   private static CoreCount computeBlockingAwareCores(
-      List<PlanFragment> postOrderFragments) {
+      List<PlanFragment> postOrderFragments, boolean findUnboundedCount) {
     // fragmentCoreState is a mapping between a fragment (through its PlanFragmentId) and
     // its CoreCount. The first element of the pair is the CoreCount of subtree rooted at
     // that fragment. The second element of the pair is the CoreCount of blocking-child
@@ -479,13 +515,32 @@ public class Planner {
         Maps.newHashMap();
 
     for (PlanFragment fragment : postOrderFragments) {
-      fragment.computeBlockingAwareCores(fragmentCoreState);
+      fragment.computeBlockingAwareCores(fragmentCoreState, findUnboundedCount);
     }
 
     PlanFragment root = postOrderFragments.get(postOrderFragments.size() - 1);
     Pair<CoreCount, List<CoreCount>> rootCores = fragmentCoreState.get(root.getId());
 
-    return root.maxCore(rootCores.first, CoreCount.sum(rootCores.second));
+    CoreCount maxCores = root.maxCore(
+        rootCores.first, CoreCount.sum(rootCores.second), findUnboundedCount);
+    return maxCores;
+  }
+
+  /**
+   * Reduce plan node cardinalities based on runtime filter information.
+   * Valid to call after runtime filter generation and before processing cost
+   * computation.
+   */
+  public static void reduceCardinalityByRuntimeFilter(
+      List<PlanFragment> planRoots, PlannerContext planCtx) {
+    double reductionScale = planCtx.getRootAnalyzer()
+                                .getQueryOptions()
+                                .getRuntime_filter_cardinality_reduction_scale();
+    if (reductionScale <= 0) return;
+    PlanFragment rootFragment = planRoots.get(0);
+    Stack<PlanNode> nodeStack = new Stack<>();
+    rootFragment.getPlanRoot().reduceCardinalityByRuntimeFilter(
+        nodeStack, reductionScale);
   }
 
   /**
@@ -495,15 +550,23 @@ public class Planner {
   public static void computeProcessingCost(
       List<PlanFragment> planRoots, TQueryExecRequest request, PlannerContext planCtx) {
     Analyzer rootAnalyzer = planCtx.getRootAnalyzer();
-    if (!rootAnalyzer.getQueryOptions().isCompute_processing_cost()) {
-      request.setCores_required(-1);
-      return;
-    }
+    TQueryOptions queryOptions = rootAnalyzer.getQueryOptions();
 
     PlanFragment rootFragment = planRoots.get(0);
-    List<PlanFragment> postOrderFragments = rootFragment.getNodesPostOrder();
-    for (PlanFragment fragment : postOrderFragments) {
-      fragment.computeCostingSegment(rootAnalyzer.getQueryOptions());
+    List<PlanFragment> postOrderFragments = new ArrayList<>();
+    boolean testCostCalculation = queryOptions.isEnable_replan()
+        && (RuntimeEnv.INSTANCE.isTestEnv() || queryOptions.isTest_replan());
+    if (queryOptions.isCompute_processing_cost() || testCostCalculation) {
+      postOrderFragments = rootFragment.getNodesPostOrder();
+      for (PlanFragment fragment : postOrderFragments) {
+        fragment.computeCostingSegment(queryOptions);
+      }
+    }
+
+    // Only do parallelism adjustment if COMPUTE_PROCESSING_COST is enabled.
+    if (!queryOptions.isCompute_processing_cost()) {
+      request.setCores_required(-1);
+      return;
     }
 
     if (LOG.isTraceEnabled()) {
@@ -515,11 +578,34 @@ public class Planner {
     }
 
     computeEffectiveParallelism(postOrderFragments,
-        rootAnalyzer.getMinParallelismPerNode(), rootAnalyzer.getMaxParallelismPerNode());
-    CoreCount effectiveCores = computeBlockingAwareCores(postOrderFragments);
-    request.setCores_required(effectiveCores.total());
+        rootAnalyzer.getMinParallelismPerNode(), rootAnalyzer.getMaxParallelismPerNode(),
+        queryOptions);
 
-    LOG.info("CoreCount=" + effectiveCores);
+    // Count bounded core count. This is taken from final instance count from previous
+    // step.
+    CoreCount boundedCores = computeBlockingAwareCores(postOrderFragments, false);
+    Set<PlanFragmentId> dominantFragmentIds =
+        new HashSet<>(boundedCores.getUniqueFragmentIds());
+    int coresRequired = Math.max(1, boundedCores.totalWithoutCoordinator());
+    if (boundedCores.hasCoordinator()) {
+      // exclude coordinator fragment from dominantFragmentIds.
+      dominantFragmentIds.remove(rootFragment.getId());
+    }
+    request.setCores_required(coresRequired);
+    LOG.info("CoreCount=" + boundedCores + ", coresRequired=" + coresRequired);
+
+    // Mark dominant fragment. This will be used by scheduler in scheduler.cc to count
+    // admission slot requirement.
+    for (PlanFragment fragment : postOrderFragments) {
+      if (dominantFragmentIds.contains(fragment.getId())) fragment.markDominant();
+    }
+
+    // Count unbounded core count. This is based on maximum parallelism of each fragment.
+    CoreCount unboundedCores = computeBlockingAwareCores(postOrderFragments, true);
+    int coresRequiredUnbounded = Math.max(1, unboundedCores.totalWithoutCoordinator());
+    request.setCores_required_unbounded(coresRequiredUnbounded);
+    LOG.info("CoreCountUnbounded=" + unboundedCores
+        + ", coresRequiredUnbounded=" + coresRequiredUnbounded);
   }
 
   /**
@@ -576,6 +662,8 @@ public class Planner {
     maxPerHostPeakResources = MIN_PER_HOST_RESOURCES.max(maxPerHostPeakResources);
 
     request.setPer_host_mem_estimate(maxPerHostPeakResources.getMemEstimateBytes());
+    request.setPlanner_per_host_mem_estimate(
+        maxPerHostPeakResources.getMemEstimateBytes());
     request.setIs_trivial_query(trivial);
     request.setMax_per_host_min_mem_reservation(
         maxPerHostPeakResources.getMinMemReservationBytes());
@@ -909,20 +997,26 @@ public class Planner {
     inputFragment.setPlanRoot(node);
   }
 
-  public void createPreDeleteSort(DeleteStmt deleteStmt, PlanFragment inputFragment,
+  public void createPreDmlSort(DmlStatementBase dmlStmt, PlanFragment inputFragment,
       Analyzer analyzer) throws ImpalaException {
     List<Expr> orderingExprs = new ArrayList<>();
 
-    orderingExprs.addAll(deleteStmt.getSortExprs());
+    List<Expr> partitionKeyExprs = dmlStmt.getPartitionKeyExprs();
+    orderingExprs.addAll(partitionKeyExprs);
+    orderingExprs.addAll(dmlStmt.getSortExprs());
+
+    if (orderingExprs.isEmpty()) return;
 
     // Build sortinfo to sort by the ordering exprs.
     List<Boolean> isAscOrder = Collections.nCopies(orderingExprs.size(), true);
     List<Boolean> nullsFirstParams = Collections.nCopies(orderingExprs.size(), false);
     SortInfo sortInfo = new SortInfo(orderingExprs, isAscOrder, nullsFirstParams,
-        TSortingOrder.LEXICAL);
-    sortInfo.createSortTupleInfo(deleteStmt.getResultExprs(), analyzer);
+        dmlStmt.getSortingOrder());
+    int numPartitionKeys = partitionKeyExprs.size();
+    sortInfo.setNumLexicalKeysInZOrder(numPartitionKeys);
+    sortInfo.createSortTupleInfo(dmlStmt.getResultExprs(), analyzer);
     sortInfo.getSortTupleDescriptor().materializeSlots();
-    deleteStmt.substituteResultExprs(sortInfo.getOutputSmap(), analyzer);
+    dmlStmt.substituteResultExprs(sortInfo.getOutputSmap(), analyzer);
 
     PlanNode node = SortNode.createTotalSortNode(
         ctx_.getNextNodeId(), inputFragment.getPlanRoot(), sortInfo, 0);
