@@ -43,6 +43,7 @@ import org.apache.impala.analysis.NullLiteral;
 import org.apache.impala.analysis.Predicate;
 import org.apache.impala.analysis.SlotId;
 import org.apache.impala.analysis.SlotRef;
+import org.apache.impala.analysis.StringLiteral;
 import org.apache.impala.analysis.TupleDescriptor;
 import org.apache.impala.analysis.TupleId;
 import org.apache.impala.analysis.TupleIsNullPredicate;
@@ -58,6 +59,7 @@ import org.apache.impala.common.AnalysisException;
 import org.apache.impala.common.IdGenerator;
 import org.apache.impala.common.InternalException;
 import org.apache.impala.planner.JoinNode.DistributionMode;
+import org.apache.impala.planner.JoinNode.EqJoinConjunctScanSlots;
 import org.apache.impala.service.BackendConfig;
 import org.apache.impala.service.FeSupport;
 import org.apache.impala.thrift.TColumnValue;
@@ -482,6 +484,12 @@ public final class RuntimeFilterGenerator {
       if (isTimestampTruncation) {
         Preconditions.checkArgument(srcExpr.isAnalyzed());
         Preconditions.checkArgument(srcExpr.getType() == Type.TIMESTAMP);
+        // The filter is targeted for Kudu scan node with source timestamp truncation.
+        if (analyzer.getQueryOptions().isConvert_kudu_utc_timestamps()) {
+          List<Expr> params = Lists.newArrayList(srcExpr,
+              new StringLiteral(analyzer.getQueryCtx().getLocal_time_zone()));
+          srcExpr = new FunctionCallExpr("to_utc_timestamp", params);
+        }
         Expr toUnixTimeExpr =
             new FunctionCallExpr("utc_to_unix_micros", Lists.newArrayList(srcExpr));
         try {
@@ -636,6 +644,15 @@ public final class RuntimeFilterGenerator {
       return null;
     }
 
+    public boolean isPartitionFilterAt(PlanNodeId targetPlanNodeId) {
+      for (RuntimeFilterTarget target : targets_) {
+        if (target.node.getId() == targetPlanNodeId) {
+          return target.isBoundByPartitionColumns;
+        }
+      }
+      return false;
+    }
+
     public List<RuntimeFilterTarget> getTargets() { return targets_; }
     public boolean hasTargets() { return !targets_.isEmpty(); }
     public Expr getSrcExpr() { return srcExpr_; }
@@ -646,6 +663,26 @@ public final class RuntimeFilterGenerator {
     public Operator getExprCompOp() { return exprCmpOp_; }
     public long getFilterSize() { return filterSizeBytes_; }
     public boolean isTimestampTruncation() { return isTimestampTruncation_; }
+    public boolean isBroadcast() { return isBroadcastJoin_; }
+    public PlanNode getSrc() { return src_; }
+
+    private long getBuildKeyNumRowStats() {
+      long minNumRows = src_.getChild(1).getCardinality();
+      SlotRef buildSlotRef = srcExpr_.unwrapSlotRef(true);
+      if (buildSlotRef == null || !buildSlotRef.hasDesc()
+          || buildSlotRef.getDesc().getParent() == null
+          || buildSlotRef.getDesc().getParent().getTable() == null
+          || buildSlotRef.getDesc().getParent().getTable().getNumRows() <= -1) {
+        return minNumRows;
+      }
+      return buildSlotRef.getDesc().getParent().getTable().getNumRows();
+    }
+
+    /**
+     * Return the estimated false-positive probability of this filter.
+     * Bloom filter type will return value in [0.0 - 1.0] range. Otherwise, return 0.
+     */
+    public double getEstFpp() { return est_fpp_; }
 
     /**
      * Return TIMESTAMP if the isTimestampTruncation_ is set as true so that
@@ -665,14 +702,7 @@ public final class RuntimeFilterGenerator {
      * associated source join node over the cardinality of that join node's left
      * child.
      */
-    public double getSelectivity() {
-      if (src_.getCardinality() == -1
-          || src_.getChild(0).getCardinality() == -1
-          || src_.getChild(0).getCardinality() == 0) {
-        return -1;
-      }
-      return src_.getCardinality() / (double) src_.getChild(0).getCardinality();
-    }
+    public double getSelectivity() { return getJoinNodeSelectivity(src_); }
 
     public void addTarget(RuntimeFilterTarget target) { targets_.add(target); }
 
@@ -745,6 +775,88 @@ public final class RuntimeFilterGenerator {
       for (RuntimeFilterTarget target: targets_) target.node.addRuntimeFilter(this);
     }
 
+    /**
+     * Return true if this filter is deemed highly selective, will arrive on-time, and
+     * applied for all rows without ever being disabled by the scanner node.
+     * Mainly used by {@link ScanNode#reduceCardinalityByRuntimeFilter(java.util.Stack,
+     * double)} to lower the scan cardinality estimation. These properties are hard to
+     * predict during planning, so this method is set very conservative to avoid severe
+     * underestimation.
+     */
+    public boolean isHighlySelective() {
+      return level_ <= 3 && (type_ != TRuntimeFilterType.BLOOM || est_fpp_ < 0.33);
+    }
+
+    /**
+     * Return a reduced cardinality estimate for given scanNode.
+     * @param scanNode the ScanNode to inspect.
+     * @param scanCardinality the cardinality estimate before reducing through this
+     *                        filter. This can be lower than scanNode.inputCardinality_.
+     * @param partitionSelectivities a map of column name to selectivity on that column.
+     *                               Will be populated if this filter target a partition
+     *                               column.
+     * @return a reduced cardinality estimate or -1 if unknown.
+     */
+    public long reducedCardinalityForScanNode(ScanNode scanNode, long scanCardinality,
+        Map<String, Double> partitionSelectivities) {
+      Preconditions.checkState(isHighlySelective());
+      SlotRef buildSlot = srcExpr_.unwrapSlotRef(true);
+      PlanNodeId scanNodeId = scanNode.getId();
+      Expr targetExpr = getTargetExpr(scanNodeId);
+      SlotRef targetSlot = targetExpr.unwrapSlotRef(true);
+      long scanColumnNdv = targetExpr.getNumDistinctValues();
+      if (scanColumnNdv < 0) {
+        // For cardinality reduction, it is OK to skip this filter if 'scanColumnNdv'
+        // is unknown rather than trying to estimate it.
+        return -1;
+      }
+
+      long buildKeyNdv = ndvEstimate_;
+      long buildKeyCard = src_.getChild(1).getCardinality();
+      long buildKeyNumRows = getBuildKeyNumRowStats();
+      // The raw input cardinality without applying scan conjuct and limits.
+      long scanNumRows = scanNode.inputCardinality_;
+
+      boolean isPartitionFilter = isPartitionFilterAt(scanNodeId);
+      if (isPartitionFilter) {
+        // Partition filter can apply at file-level filtering.
+        // Keep the most selective partition filter per column to reduce
+        // inputCardinality_. Ignore a column if its name is unknown.
+        double thisPartSel = (double) buildKeyNdv / scanColumnNdv;
+        if (targetSlot != null && targetSlot.hasDesc()
+            && targetSlot.getDesc().getColumn() != null) {
+          String colName = targetSlot.getDesc().getColumn().getName();
+          double currentSel = partitionSelectivities.computeIfAbsent(colName, c -> 1.0);
+          if (thisPartSel < currentSel) partitionSelectivities.put(colName, thisPartSel);
+        }
+      }
+
+      List<EqJoinConjunctScanSlots> eligibleFkPkJoinConjuncts = new ArrayList<>();
+      boolean isEvalAtStorageLayer =
+          isPartitionFilter || (scanNode instanceof KuduScanNode);
+      if (isEvalAtStorageLayer && src_.fkPkEqJoinConjuncts_ != null && targetSlot != null
+          && targetSlot.hasDesc() && buildSlot != null && buildSlot.hasDesc()) {
+        for (EqJoinConjunctScanSlots fkPk : src_.fkPkEqJoinConjuncts_) {
+          if (targetSlot.getSlotId().equals(fkPk.lhs_.getId())
+              && buildSlot.getSlotId().equals(fkPk.rhs_.getId())) {
+            eligibleFkPkJoinConjuncts.add(fkPk);
+          }
+        }
+      }
+
+      if (eligibleFkPkJoinConjuncts.isEmpty()) {
+        return JoinNode.computeGenericJoinCardinality(scanColumnNdv, buildKeyNdv,
+            scanNumRows, buildKeyNumRows, scanCardinality, buildKeyCard);
+      } else {
+        // JoinNode.getFkPkJoinCardinality() can return lower number than
+        // JoinNode.computeGenericJoinCardinality(). But be conservative to only use it
+        // for filters that evaluate in storage layer (such as partition filter and
+        // pushed-down Kudu filter) to avoid severe underestimation.
+        return JoinNode.getFkPkJoinCardinality(
+            eligibleFkPkJoinConjuncts, scanCardinality, buildKeyCard);
+      }
+    }
+
     public String debugString() {
       StringBuilder output = new StringBuilder();
       output.append("FilterID: " + id_)
@@ -991,7 +1103,7 @@ public final class RuntimeFilterGenerator {
               ctx.getRootAnalyzer(), conjunct, joinNode, filterType, filterSizeLimits_,
               /* isTimestampTruncation */ false, level);
           if (filter != null) {
-            registerRuntimeFilter(filter);
+            registerRuntimeFilter(ctx, filter);
             filters.add(filter);
           }
           // For timestamp bloom filters, we also generate a RuntimeFilter with the
@@ -1004,7 +1116,7 @@ public final class RuntimeFilterGenerator {
                 ctx.getRootAnalyzer(), conjunct, joinNode, filterType, filterSizeLimits_,
                 /* isTimestampTruncation */ true, level);
             if (filter2 == null) continue;
-            registerRuntimeFilter(filter2);
+            registerRuntimeFilter(ctx, filter2);
             filters.add(filter2);
           }
         }
@@ -1028,26 +1140,29 @@ public final class RuntimeFilterGenerator {
    * Registers a runtime filter with the tuple id of every scan node that is a candidate
    * destination node for that filter.
    */
-  private void registerRuntimeFilter(RuntimeFilter filter) {
+  private void registerRuntimeFilter(PlannerContext ctx, RuntimeFilter filter) {
     Map<TupleId, List<SlotId>> targetSlotsByTid = filter.getTargetSlots();
     Preconditions.checkState(targetSlotsByTid != null && !targetSlotsByTid.isEmpty());
     for (TupleId tupleId: targetSlotsByTid.keySet()) {
-      registerRuntimeFilter(filter, tupleId);
+      registerRuntimeFilter(ctx, filter, tupleId);
     }
   }
 
   /**
    * Registers a runtime filter with a specific target tuple id.
    */
-  private void registerRuntimeFilter(RuntimeFilter filter, TupleId targetTid) {
-    Preconditions.checkState(filter.getTargetSlots().containsKey(targetTid));
-    List<RuntimeFilter> filters = runtimeFiltersByTid_.get(targetTid);
-    if (filters == null) {
-      filters = new ArrayList<>();
-      runtimeFiltersByTid_.put(targetTid, filters);
+  private void registerRuntimeFilter(
+      PlannerContext ctx, RuntimeFilter filter, TupleId targetTid) {
+    Preconditions.checkArgument(filter.getTargetSlots().containsKey(targetTid),
+        "filter does not contain target slot!");
+    Preconditions.checkArgument(!filter.isFinalized(), "filter must not finalized yet!");
+    if (ctx.getQueryOptions().isSetRuntime_filter_ids_to_skip()
+        && ctx.getQueryOptions().getRuntime_filter_ids_to_skip().contains(
+            filter.getFilterId().asInt())) {
+      // Skip this filter because it is explicitly excluded via query option.
+      return;
     }
-    Preconditions.checkState(!filter.isFinalized());
-    filters.add(filter);
+    runtimeFiltersByTid_.computeIfAbsent(targetTid, t -> new ArrayList<>()).add(filter);
   }
 
   /**
@@ -1087,7 +1202,8 @@ public final class RuntimeFilterGenerator {
    *    scan node.
    * 3. Only Hdfs and Kudu scan nodes are supported:
    *     a. If the target is an HdfsScanNode, the filter must be type BLOOM/IN_LIST for
-   *        non Parquet tables, or type BLOOM/MIN_MAX/IN_LIST for Parquet tables.
+   *        ORC tables, or type BLOOM/MIN_MAX for Parquet tables, or type BLOOM for other
+   *        kind of tables.
    *     b. If the target is a KuduScanNode, the filter could be type MIN_MAX, and/or
    *        BLOOM, the target must be a slot ref on a column, and the comp op cannot
    *        be 'not distinct'.
@@ -1169,6 +1285,19 @@ public final class RuntimeFilterGenerator {
           }
           SlotRef slotRef = (SlotRef) targetExpr;
           if (slotRef.getDesc().getColumn() == null) continue;
+          if (filter.isTimestampTruncation() &&
+              analyzer.getQueryOptions().isConvert_kudu_utc_timestamps() &&
+              analyzer.getQueryOptions().isDisable_kudu_local_timestamp_bloom_filter()) {
+            // Local timestamp convert to UTC could be ambiguous in the case of DST
+            // change. We can only put one of the two possible UTC timestamps in the bloom
+            // filter for now, which may cause missing rows that have the other UTC
+            // timestamp.
+            // For those regions that do not observe DST, could set this flag to false
+            // to re-enable kudu local timestamp bloom filter.
+            LOG.info("Skipping runtime filter because kudu local timestamp bloom filter "
+                + "is disabled: " + filter.getSrcExpr().toSql());
+            continue;
+          }
         } else if (filter.getType() == TRuntimeFilterType.MIN_MAX) {
           Preconditions.checkState(
               enabledRuntimeFilterTypes.contains(TRuntimeFilterType.MIN_MAX),
@@ -1262,6 +1391,17 @@ public final class RuntimeFilterGenerator {
     return false;
   }
 
+  /**
+   * Get selectivity of joinNode.
+   */
+  public static double getJoinNodeSelectivity(JoinNode joinNode) {
+    if (joinNode.getCardinality() == -1 || joinNode.getChild(0).getCardinality() == -1
+        || joinNode.getChild(0).getCardinality() == 0) {
+      return -1;
+    }
+    return joinNode.getCardinality() / (double) joinNode.getChild(0).getCardinality();
+  }
+
   /**
    * Computes the target expr for a specified runtime filter 'filter' to be applied at
    * the scan node with target tuple descriptor 'targetTid'.
@@ -1278,16 +1418,19 @@ public final class RuntimeFilterGenerator {
       targetExpr.collect(SlotRef.class, exprSlots);
       List<SlotId> sids = filter.getTargetSlots().get(targetTid);
       for (SlotRef slotRef: exprSlots) {
+        boolean matched = false;
         for (SlotId sid: sids) {
           if (analyzer.hasValueTransfer(slotRef.getSlotId(), sid)) {
             SlotRef newSlotRef = new SlotRef(analyzer.getSlotDesc(sid));
             newSlotRef.analyzeNoThrow(analyzer);
             smap.put(slotRef, newSlotRef);
+            matched = true;
             break;
           }
         }
+        Preconditions.checkState(matched,
+            "No SlotId found for RuntimeFilter %s SlotRef %s", filter, slotRef);
       }
-      Preconditions.checkState(exprSlots.size() == smap.size());
       try {
         targetExpr = targetExpr.substitute(smap, analyzer, false);
       } catch (Exception e) {
