@@ -23,6 +23,7 @@ import org.apache.impala.thrift.TExplainLevel;
 import org.apache.impala.thrift.TPlanNode;
 import org.apache.impala.thrift.TPlanNodeType;
 import org.apache.impala.thrift.TQueryOptions;
+import org.apache.impala.util.ExprUtil;
 
 import com.google.common.base.Preconditions;
 
@@ -66,6 +67,13 @@ public class SingularRowSrcNode extends PlanNode {
     numInstances_ = containingSubplanNode_.getChild(0).getNumInstances();
   }
 
+  @Override
+  public void computeProcessingCost(TQueryOptions queryOptions) {
+    processingCost_ = ProcessingCost.basicCost(getDisplayLabel(),
+        containingSubplanNode_.getChild(0).getCardinality(),
+        ExprUtil.computeExprsTotalCost(getConjuncts()));
+  }
+
   @Override
   public void computeNodeResourceProfile(TQueryOptions queryOptions) {
     // TODO: add an estimate
