@@ -18,14 +18,17 @@
 package org.apache.impala.service;
 
 import java.nio.ByteBuffer;
+import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Collections;
 
 import org.apache.iceberg.AppendFiles;
 import org.apache.iceberg.BaseTable;
 import org.apache.iceberg.DataFile;
 import org.apache.iceberg.DataFiles;
+import org.apache.iceberg.DeleteFile;
 import org.apache.iceberg.DeleteFiles;
 import org.apache.iceberg.ExpireSnapshots;
 import org.apache.iceberg.FileMetadata;
@@ -33,28 +36,32 @@ import org.apache.iceberg.ManageSnapshots;
 import org.apache.iceberg.Metrics;
 import org.apache.iceberg.PartitionSpec;
 import org.apache.iceberg.ReplacePartitions;
+import org.apache.iceberg.RewriteFiles;
 import org.apache.iceberg.RowDelta;
 import org.apache.iceberg.Schema;
-import org.apache.iceberg.SnapshotManager;
 import org.apache.iceberg.Table;
-import org.apache.iceberg.TableMetadata;
-import org.apache.iceberg.TableOperations;
 import org.apache.iceberg.Transaction;
+import org.apache.iceberg.UpdatePartitionSpec;
 import org.apache.iceberg.UpdateProperties;
 import org.apache.iceberg.UpdateSchema;
 import org.apache.iceberg.catalog.TableIdentifier;
 import org.apache.iceberg.exceptions.ValidationException;
 import org.apache.iceberg.expressions.Expressions;
+import org.apache.iceberg.expressions.Term;
+import org.apache.iceberg.hive.HiveCatalog;
 import org.apache.impala.analysis.IcebergPartitionSpec;
 import org.apache.impala.catalog.FeIcebergTable;
 import org.apache.impala.catalog.IcebergTable;
 import org.apache.impala.catalog.TableLoadingException;
 import org.apache.impala.catalog.TableNotFoundException;
 import org.apache.impala.catalog.events.MetastoreEvents.MetastoreEventPropertyKey;
+import org.apache.impala.catalog.iceberg.GroupedContentFiles;
 import org.apache.impala.catalog.iceberg.IcebergCatalog;
+import org.apache.impala.catalog.iceberg.IcebergHiveCatalog;
 import org.apache.impala.common.ImpalaRuntimeException;
 import org.apache.impala.fb.FbIcebergColumnStats;
 import org.apache.impala.fb.FbIcebergDataFile;
+import org.apache.impala.thrift.TAlterTableDropPartitionParams;
 import org.apache.impala.thrift.TAlterTableExecuteExpireSnapshotsParams;
 import org.apache.impala.thrift.TAlterTableExecuteRollbackParams;
 import org.apache.impala.thrift.TColumn;
@@ -64,6 +71,7 @@ import org.apache.impala.thrift.TIcebergPartitionSpec;
 import org.apache.impala.thrift.TRollbackType;
 import org.apache.impala.util.IcebergSchemaConverter;
 import org.apache.impala.util.IcebergUtil;
+import org.apache.thrift.TException;
 
 import com.google.common.base.Preconditions;
 import org.slf4j.Logger;
@@ -83,11 +91,16 @@ public class IcebergCatalogOpExecutor {
    */
   public static Table createTable(TIcebergCatalog catalog, TableIdentifier identifier,
       String location, List<TColumn> columns, TIcebergPartitionSpec partitionSpec,
+      List<String> primaryKeyColumnNames, String owner,
       Map<String, String> tableProperties) throws ImpalaRuntimeException {
     // Each table id increase from zero
-    Schema schema = createIcebergSchema(columns);
+    Schema schema = createIcebergSchema(columns, primaryKeyColumnNames);
     PartitionSpec spec = IcebergUtil.createIcebergPartition(schema, partitionSpec);
     IcebergCatalog icebergCatalog = IcebergUtil.getIcebergCatalog(catalog, location);
+    if (icebergCatalog instanceof IcebergHiveCatalog) {
+      // Put table owner to table properties for HiveCatalog.
+      tableProperties.put(HiveCatalog.HMS_TABLE_OWNER, owner);
+    }
     Table iceTable = icebergCatalog.createTable(identifier, schema, spec, location,
         tableProperties);
     LOG.info("Create iceberg table successful.");
@@ -172,21 +185,24 @@ public class IcebergCatalogOpExecutor {
    * Sets new default partition spec for an Iceberg table.
    */
   public static void alterTableSetPartitionSpec(FeIcebergTable feTable,
-      TIcebergPartitionSpec partSpec, String catalogServiceId, long catalogVersion)
-      throws TableLoadingException, ImpalaRuntimeException {
+      TIcebergPartitionSpec partSpec, Transaction transaction)
+      throws ImpalaRuntimeException {
+    try {
+      if (!feTable.getPrimaryKeyColumnNames().isEmpty()) {
+        throw new ImpalaRuntimeException("Not allowed to do partition evolution on " +
+            "Iceberg tables with primary keys.");
+      }
+    } catch (TException tEx) {
+      throw new ImpalaRuntimeException(tEx.getMessage());
+    }
+
     BaseTable iceTable = (BaseTable)feTable.getIcebergApiTable();
-    TableOperations tableOp = iceTable.operations();
-    TableMetadata metadata = tableOp.current();
-    Schema schema = metadata.schema();
-    PartitionSpec newPartSpec = IcebergUtil.createIcebergPartition(schema, partSpec);
-    TableMetadata newMetadata = metadata.updatePartitionSpec(newPartSpec);
-    Map<String, String> properties = new HashMap<>(newMetadata.properties());
-    properties.put(MetastoreEventPropertyKey.CATALOG_SERVICE_ID.getKey(),
-                   catalogServiceId);
-    properties.put(MetastoreEventPropertyKey.CATALOG_VERSION.getKey(),
-                   String.valueOf(catalogVersion));
-    newMetadata = newMetadata.replaceProperties(properties);
-    tableOp.commit(metadata, newMetadata);
+    UpdatePartitionSpec updatePartitionSpec = transaction.updateSpec();
+    iceTable.spec().fields().forEach(partitionField -> updatePartitionSpec.removeField(
+        partitionField.name()));
+    List<Term> partitioningTerms = IcebergUtil.getPartitioningTerms(partSpec);
+    partitioningTerms.forEach(updatePartitionSpec::addField);
+    updatePartitionSpec.commit();
   }
 
   /**
@@ -233,10 +249,26 @@ public class IcebergCatalogOpExecutor {
   }
 
   /**
-   * Drops a column from a Iceberg table.
+   * Deletes files related to specific set of partitions
    */
-  public static void dropColumn(Transaction txn, String colName)
-      throws TableLoadingException, ImpalaRuntimeException {
+  public static long alterTableDropPartition(
+      Transaction iceTxn, TAlterTableDropPartitionParams params) {
+    DeleteFiles deleteFiles = iceTxn.newDelete();
+    if (params.iceberg_drop_partition_request.is_truncate) {
+      deleteFiles.deleteFromRowFilter(Expressions.alwaysTrue());
+    } else {
+      for (String path : params.iceberg_drop_partition_request.paths) {
+        deleteFiles.deleteFile(path);
+      }
+    }
+    deleteFiles.commit();
+    return params.iceberg_drop_partition_request.num_partitions;
+  }
+
+  /**
+   * Drops a column from an Iceberg table.
+   */
+  public static void dropColumn(Transaction txn, String colName) {
     UpdateSchema schema = txn.updateSchema();
     schema.deleteColumn(colName);
     schema.commit();
@@ -276,9 +308,9 @@ public class IcebergCatalogOpExecutor {
   /**
    * Build iceberg schema by parameters.
    */
-  private static Schema createIcebergSchema(List<TColumn> columns)
-      throws ImpalaRuntimeException {
-    return IcebergSchemaConverter.genIcebergSchema(columns);
+  private static Schema createIcebergSchema(List<TColumn> columns,
+      List<String> primaryKeyColumnNames) throws ImpalaRuntimeException {
+    return IcebergSchemaConverter.genIcebergSchema(columns, primaryKeyColumnNames);
   }
 
   /**
@@ -333,51 +365,117 @@ public class IcebergCatalogOpExecutor {
     switch (icebergOp.operation) {
       case INSERT: appendFiles(feIcebergTable, txn, icebergOp); break;
       case DELETE: deleteRows(feIcebergTable, txn, icebergOp); break;
+      case UPDATE: updateRows(feIcebergTable, txn, icebergOp); break;
+      case OPTIMIZE: rewriteTable(feIcebergTable, txn, icebergOp); break;
       default: throw new ImpalaRuntimeException(
           "Unknown Iceberg operation: " + icebergOp.operation);
     }
   }
 
-  public static void deleteRows(FeIcebergTable feIcebergTable, Transaction txn,
+  private static void deleteRows(FeIcebergTable feIcebergTable, Transaction txn,
       TIcebergOperationParam icebergOp) throws ImpalaRuntimeException {
-    org.apache.iceberg.Table nativeIcebergTable = feIcebergTable.getIcebergApiTable();
     List<ByteBuffer> deleteFilesFb = icebergOp.getIceberg_delete_files_fb();
     RowDelta rowDelta = txn.newRowDelta();
     for (ByteBuffer buf : deleteFilesFb) {
-      FbIcebergDataFile deleteFile = FbIcebergDataFile.getRootAsFbIcebergDataFile(buf);
-
-      PartitionSpec partSpec = nativeIcebergTable.specs().get(deleteFile.specId());
-      IcebergPartitionSpec impPartSpec = feIcebergTable.getPartitionSpec(
-          deleteFile.specId());
-      Metrics metrics = buildDataFileMetrics(deleteFile);
-      FileMetadata.Builder builder = FileMetadata.deleteFileBuilder(partSpec)
-          .ofPositionDeletes()
-          .withMetrics(metrics)
-          .withPath(deleteFile.path())
-          .withFormat(IcebergUtil.fbFileFormatToIcebergFileFormat(deleteFile.format()))
-          .withRecordCount(deleteFile.recordCount())
-          .withFileSizeInBytes(deleteFile.fileSizeInBytes());
-      IcebergUtil.PartitionData partitionData = IcebergUtil.partitionDataFromDataFile(
-          partSpec.partitionType(), impPartSpec, deleteFile);
-      if (partitionData != null) builder.withPartition(partitionData);
-      rowDelta.addDeletes(builder.build());
+      DeleteFile deleteFile = createDeleteFile(feIcebergTable, buf);
+      rowDelta.addDeletes(deleteFile);
     }
     try {
+      // Validate that there are no conflicting data files, because if data files are
+      // added in the meantime, they potentially contain records that should have been
+      // affected by this DELETE operation.
       rowDelta.validateFromSnapshot(icebergOp.getInitial_snapshot_id());
       rowDelta.validateNoConflictingDataFiles();
+      rowDelta.validateDataFilesExist(
+          icebergOp.getData_files_referenced_by_position_deletes());
+      rowDelta.validateDeletedFiles();
       rowDelta.commit();
     } catch (ValidationException e) {
       throw new ImpalaRuntimeException(e.getMessage(), e);
     }
   }
 
+  private static void updateRows(FeIcebergTable feIcebergTable, Transaction txn,
+      TIcebergOperationParam icebergOp) throws ImpalaRuntimeException {
+    List<ByteBuffer> deleteFilesFb = icebergOp.getIceberg_delete_files_fb();
+    List<ByteBuffer> dataFilesFb = icebergOp.getIceberg_data_files_fb();
+    RowDelta rowDelta = txn.newRowDelta();
+    for (ByteBuffer buf : deleteFilesFb) {
+      DeleteFile deleteFile = createDeleteFile(feIcebergTable, buf);
+      rowDelta.addDeletes(deleteFile);
+    }
+    for (ByteBuffer buf : dataFilesFb) {
+      DataFile dataFile = createDataFile(feIcebergTable, buf);
+      rowDelta.addRows(dataFile);
+    }
+    try {
+      // Validate that there are no conflicting data files, because if data files are
+      // added in the meantime, they potentially contain records that should have been
+      // affected by this UPDATE operation. Also validate that there are no conflicting
+      // delete files, because we don't want to revive records that have been deleted
+      // in the meantime.
+      rowDelta.validateFromSnapshot(icebergOp.getInitial_snapshot_id());
+      rowDelta.validateNoConflictingDataFiles();
+      rowDelta.validateNoConflictingDeleteFiles();
+      rowDelta.validateDataFilesExist(
+          icebergOp.getData_files_referenced_by_position_deletes());
+      rowDelta.validateDeletedFiles();
+      rowDelta.commit();
+    } catch (ValidationException e) {
+      throw new ImpalaRuntimeException(e.getMessage(), e);
+    }
+  }
+
+  private static DataFile createDataFile(FeIcebergTable feIcebergTable, ByteBuffer buf)
+      throws ImpalaRuntimeException {
+    FbIcebergDataFile dataFile = FbIcebergDataFile.getRootAsFbIcebergDataFile(buf);
+
+    PartitionSpec partSpec = feIcebergTable.getIcebergApiTable().specs().get(
+        dataFile.specId());
+    IcebergPartitionSpec impPartSpec =
+        feIcebergTable.getPartitionSpec(dataFile.specId());
+    Metrics metrics = buildDataFileMetrics(dataFile);
+    DataFiles.Builder builder =
+        DataFiles.builder(partSpec)
+        .withMetrics(metrics)
+        .withPath(dataFile.path())
+        .withFormat(IcebergUtil.fbFileFormatToIcebergFileFormat(dataFile.format()))
+        .withRecordCount(dataFile.recordCount())
+        .withFileSizeInBytes(dataFile.fileSizeInBytes());
+    IcebergUtil.PartitionData partitionData = IcebergUtil.partitionDataFromDataFile(
+        partSpec.partitionType(), impPartSpec, dataFile);
+    if (partitionData != null) builder.withPartition(partitionData);
+    return builder.build();
+  }
+
+  private static DeleteFile createDeleteFile(FeIcebergTable feIcebergTable,
+      ByteBuffer buf) throws ImpalaRuntimeException {
+    FbIcebergDataFile deleteFile = FbIcebergDataFile.getRootAsFbIcebergDataFile(buf);
+
+    PartitionSpec partSpec = feIcebergTable.getIcebergApiTable().specs().get(
+        deleteFile.specId());
+    IcebergPartitionSpec impPartSpec = feIcebergTable.getPartitionSpec(
+        deleteFile.specId());
+    Metrics metrics = buildDataFileMetrics(deleteFile);
+    FileMetadata.Builder builder = FileMetadata.deleteFileBuilder(partSpec)
+        .ofPositionDeletes()
+        .withMetrics(metrics)
+        .withPath(deleteFile.path())
+        .withFormat(IcebergUtil.fbFileFormatToIcebergFileFormat(deleteFile.format()))
+        .withRecordCount(deleteFile.recordCount())
+        .withFileSizeInBytes(deleteFile.fileSizeInBytes());
+    IcebergUtil.PartitionData partitionData = IcebergUtil.partitionDataFromDataFile(
+        partSpec.partitionType(), impPartSpec, deleteFile);
+    if (partitionData != null) builder.withPartition(partitionData);
+    return builder.build();
+  }
+
   /**
    * Append the newly inserted data files to the Iceberg table using the AppendFiles
    * API.
    */
   public static void appendFiles(FeIcebergTable feIcebergTable, Transaction txn,
       TIcebergOperationParam icebergOp) throws ImpalaRuntimeException {
-    org.apache.iceberg.Table nativeIcebergTable = feIcebergTable.getIcebergApiTable();
     List<ByteBuffer> dataFilesFb = icebergOp.getIceberg_data_files_fb();
     BatchWrite batchWrite;
     if (icebergOp.isIs_overwrite()) {
@@ -386,25 +484,8 @@ public class IcebergCatalogOpExecutor {
       batchWrite = new Append(txn);
     }
     for (ByteBuffer buf : dataFilesFb) {
-      FbIcebergDataFile dataFile = FbIcebergDataFile.getRootAsFbIcebergDataFile(buf);
-      Preconditions.checkState(dataFile.specId() == icebergOp.getSpec_id());
-      int specId = icebergOp.getSpec_id();
-
-      PartitionSpec partSpec = nativeIcebergTable.specs().get(specId);
-      IcebergPartitionSpec impPartSpec =
-          feIcebergTable.getPartitionSpec(specId);
-      Metrics metrics = buildDataFileMetrics(dataFile);
-      DataFiles.Builder builder =
-          DataFiles.builder(partSpec)
-          .withMetrics(metrics)
-          .withPath(dataFile.path())
-          .withFormat(IcebergUtil.fbFileFormatToIcebergFileFormat(dataFile.format()))
-          .withRecordCount(dataFile.recordCount())
-          .withFileSizeInBytes(dataFile.fileSizeInBytes());
-      IcebergUtil.PartitionData partitionData = IcebergUtil.partitionDataFromDataFile(
-          partSpec.partitionType(), impPartSpec, dataFile);
-      if (partitionData != null) builder.withPartition(partitionData);
-      batchWrite.addFile(builder.build());
+      DataFile dataFile = createDataFile(feIcebergTable, buf);
+      batchWrite.addFile(dataFile);
     }
     try {
       batchWrite.commit();
@@ -440,11 +521,50 @@ public class IcebergCatalogOpExecutor {
         nullValueCounts, null, lowerBounds, upperBounds);
   }
 
+  private static void rewriteTable(FeIcebergTable feIcebergTable, Transaction txn,
+      TIcebergOperationParam icebergOp) throws ImpalaRuntimeException {
+    GroupedContentFiles contentFiles;
+    try {
+      // Get all files from the initial snapshot.
+      contentFiles = IcebergUtil.getIcebergFilesFromSnapshot(
+          feIcebergTable, /*predicates=*/Collections.emptyList(),
+          icebergOp.getInitial_snapshot_id());
+    } catch (TableLoadingException e) {
+      throw new ImpalaRuntimeException(e.getMessage(), e);
+    }
+    RewriteFiles rewrite = txn.newRewrite();
+    // Delete current data files from table.
+    for (DataFile dataFile : contentFiles.dataFilesWithDeletes) {
+      rewrite.deleteFile(dataFile);
+    }
+    for (DataFile dataFile : contentFiles.dataFilesWithoutDeletes) {
+      rewrite.deleteFile(dataFile);
+    }
+    // Delete current delete files from table.
+    for (DeleteFile deleteFile : contentFiles.positionDeleteFiles) {
+      rewrite.deleteFile(deleteFile);
+    }
+    for (DeleteFile deleteFile : contentFiles.equalityDeleteFiles) {
+      rewrite.deleteFile(deleteFile);
+    }
+    // Add newly written files to the table.
+    List<ByteBuffer> dataFilesToAdd = icebergOp.getIceberg_data_files_fb();
+    for (ByteBuffer buf : dataFilesToAdd) {
+      DataFile dataFile = createDataFile(feIcebergTable, buf);
+      rewrite.addFile(dataFile);
+    }
+    try {
+      rewrite.validateFromSnapshot(icebergOp.getInitial_snapshot_id());
+      rewrite.commit();
+    } catch (ValidationException e) {
+      throw new ImpalaRuntimeException(e.getMessage(), e);
+    }
+  }
+
   /**
    * Creates new snapshot for the iceberg table by deleting all data files.
    */
-  public static void truncateTable(Transaction txn)
-      throws ImpalaRuntimeException {
+  public static void truncateTable(Transaction txn) {
     DeleteFiles delete = txn.newDelete();
     delete.deleteFromRowFilter(Expressions.alwaysTrue());
     delete.commit();
@@ -463,4 +583,4 @@ public class IcebergCatalogOpExecutor {
                     String.valueOf(version));
     updateProps.commit();
   }
-}
\ No newline at end of file
+}
