@@ -21,14 +21,15 @@ import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Lists;
+import com.google.common.collect.Maps;
+import java.util.Map;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.UUID;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
-import java.util.concurrent.locks.ReentrantReadWriteLock.ReadLock;
-import java.util.concurrent.locks.ReentrantReadWriteLock.WriteLock;
 import java.util.stream.Collectors;
 import org.apache.hadoop.hive.metastore.api.CurrentNotificationEventId;
+import org.apache.hadoop.hive.conf.HiveConf;
 import org.apache.impala.analysis.TableName;
 import org.apache.impala.authorization.AuthorizationConfig;
 import org.apache.impala.authorization.AuthorizationFactory;
@@ -38,9 +39,12 @@ import org.apache.impala.catalog.CatalogException;
 import org.apache.impala.catalog.CatalogServiceCatalog;
 import org.apache.impala.catalog.Db;
 import org.apache.impala.catalog.FeDb;
+import org.apache.impala.catalog.FileMetadataLoader;
 import org.apache.impala.catalog.Function;
 import org.apache.impala.catalog.MetaStoreClientPool;
 import org.apache.impala.catalog.MetaStoreClientPool.MetaStoreClient;
+import org.apache.impala.catalog.ParallelFileMetadataLoader;
+import org.apache.impala.catalog.Table;
 import org.apache.impala.catalog.events.ExternalEventsProcessor;
 import org.apache.impala.catalog.events.MetastoreEvents.EventFactoryForSyncToLatestEvent;
 import org.apache.impala.catalog.events.MetastoreEvents.MetastoreEventFactory;
@@ -48,8 +52,6 @@ import org.apache.impala.catalog.events.MetastoreEventsProcessor;
 import org.apache.impala.catalog.events.NoOpEventProcessor;
 import org.apache.impala.catalog.metastore.ICatalogMetastoreServer;
 import org.apache.impala.catalog.metastore.NoOpCatalogMetastoreServer;
-import org.apache.impala.catalog.monitor.CatalogMonitor;
-import org.apache.impala.catalog.monitor.CatalogOperationMetrics;
 import org.apache.impala.common.ImpalaException;
 import org.apache.impala.common.InternalException;
 import org.apache.impala.common.JniUtil;
@@ -69,6 +71,9 @@ import org.apache.impala.thrift.TGetDbsParams;
 import org.apache.impala.thrift.TGetDbsResult;
 import org.apache.impala.thrift.TGetFunctionsRequest;
 import org.apache.impala.thrift.TGetFunctionsResponse;
+import org.apache.impala.thrift.TGetLatestCompactionsRequest;
+import org.apache.impala.thrift.TGetLatestCompactionsResponse;
+import org.apache.impala.thrift.TGetNullPartitionNameResponse;
 import org.apache.impala.thrift.TGetPartialCatalogObjectRequest;
 import org.apache.impala.thrift.TGetPartitionStatsRequest;
 import org.apache.impala.thrift.TGetPartitionStatsResponse;
@@ -83,9 +88,13 @@ import org.apache.impala.thrift.TTableUsage;
 import org.apache.impala.thrift.TUniqueId;
 import org.apache.impala.thrift.TUpdateCatalogRequest;
 import org.apache.impala.thrift.TUpdateTableUsageRequest;
+import org.apache.impala.thrift.TGetAllHadoopConfigsResponse;
 import org.apache.impala.util.AuthorizationUtil;
 import org.apache.impala.util.CatalogOpUtil;
+import org.apache.impala.util.EventSequence;
 import org.apache.impala.util.GlogAppender;
+import org.apache.impala.util.MetaStoreUtil;
+import org.apache.impala.util.NoOpEventSequence;
 import org.apache.impala.util.PatternMatcher;
 import org.apache.impala.util.ThreadNameAnnotator;
 import org.apache.impala.util.TUniqueIdUtil;
@@ -118,15 +127,13 @@ public class JniCatalog {
   private final static ReentrantReadWriteLock catalogServiceIdLock_ =
       new ReentrantReadWriteLock(true /*fair ordering*/);
 
-  // A singleton monitoring class that keeps track of the catalog usage metrics.
-  private final CatalogOperationMetrics catalogOperationUsage_ =
-      CatalogMonitor.INSTANCE.getCatalogOperationMetrics();
-
   private static TUniqueId generateId() {
     UUID uuid = UUID.randomUUID();
     return new TUniqueId(uuid.getMostSignificantBits(), uuid.getLeastSignificantBits());
   }
 
+  private static final HiveConf HIVE_CONF = new HiveConf();
+
   public JniCatalog(byte[] thriftBackendConfig) throws ImpalaException {
     TBackendGflags cfg = new TBackendGflags();
     JniUtil.deserializeThrift(protocolFactory_, cfg, thriftBackendConfig);
@@ -177,7 +184,7 @@ public class JniCatalog {
     catalogMetastoreServer_.start();
 
     try {
-      catalog_.reset();
+      catalog_.reset(NoOpEventSequence.INSTANCE);
     } catch (CatalogException e) {
       LOG.error("Error initializing Catalog. Please run 'invalidate metadata'", e);
     }
@@ -316,12 +323,10 @@ public class JniCatalog {
       throws ImpalaException, TException {
     TResetMetadataRequest req = new TResetMetadataRequest();
     JniUtil.deserializeThrift(protocolFactory_, req, thriftResetMetadataReq);
-    catalogOperationUsage_.increment(req);
     String shortDesc = CatalogOpUtil.getShortDescForReset(req);
 
     return execAndSerialize("resetMetadata", shortDesc,
-        () -> catalogOpExecutor_.execResetMetadata(req),
-        () -> catalogOperationUsage_.decrement(req));
+        () -> catalogOpExecutor_.execResetMetadata(req));
   }
 
   /**
@@ -397,7 +402,9 @@ public class JniCatalog {
 
   /**
    * Gets the json string of a catalog object. It can only be used in showing debug
-   * messages and can't be deserialized to a thrift object.
+   * messages and can't be deserialized to a thrift object. The returned object is also
+   * slimmer than the one obtained from getCatalogObject() because binary data fields
+   * are excluded.
    */
   public String getJsonCatalogObject(byte[] thriftParams)
       throws ImpalaException, TException {
@@ -408,7 +415,7 @@ public class JniCatalog {
         "Getting JSON catalog object of " + Catalog.toCatalogObjectKey(objectDesc);
 
     return execOp("getJsonCatalogObject", shortDesc, () -> {
-      String res = jsonSerializer.toString(catalog_.getTCatalogObject(objectDesc));
+      String res = jsonSerializer.toString(catalog_.getTCatalogObject(objectDesc, true));
       return Pair.create(res, (long) res.length());
     }, objectDesc);
   }
@@ -490,13 +497,11 @@ public class JniCatalog {
       throws ImpalaException, TException {
     TUpdateCatalogRequest request = new TUpdateCatalogRequest();
     JniUtil.deserializeThrift(protocolFactory_, request, thriftUpdateCatalog);
-    catalogOperationUsage_.increment(request);
     String shortDesc = "Update catalog for "
         + fullyQualifiedTableName(request.getDb_name(), request.getTarget_table());
 
     return execAndSerialize("updateCatalog", shortDesc,
-        () -> catalogOpExecutor_.updateCatalog(request),
-        () -> catalogOperationUsage_.decrement(request));
+        () -> catalogOpExecutor_.updateCatalog(request));
   }
 
   /**
@@ -544,8 +549,77 @@ public class JniCatalog {
         "getCatalogServerMetrics", shortDesc, () -> {
           response.setCatalog_partial_fetch_rpc_queue_len(
               catalog_.getPartialFetchRpcQueueLength());
+          response.setCatalog_num_file_metadata_loading_threads(
+              ParallelFileMetadataLoader.TOTAL_THREADS.get());
+          response.setCatalog_num_tables_loading_file_metadata(
+              ParallelFileMetadataLoader.TOTAL_TABLES.get());
+          response.setCatalog_num_file_metadata_loading_tasks(
+              FileMetadataLoader.TOTAL_TASKS.get());
+          response.setCatalog_num_tables_loading_metadata(Table.LOADING_TABLES.get());
+          response.setCatalog_num_tables_async_loading_metadata(
+              catalog_.getNumAsyncLoadingTables());
+          response.setCatalog_num_tables_waiting_for_async_loading(
+              catalog_.getNumAsyncWaitingTables());
+          response.setCatalog_num_dbs(catalog_.getNumDatabases());
+          response.setCatalog_num_tables(catalog_.getNumTables());
+          response.setCatalog_num_functions(catalog_.getNumFunctions());
+          response.setCatalog_num_hms_clients_idle(catalog_.getNumHmsClientsIdle());
+          response.setCatalog_num_hms_clients_in_use(catalog_.getNumHmsClientsInUse());
           response.setEvent_metrics(catalog_.getEventProcessorMetrics());
           return response;
         });
   }
+
+  /**
+   * Refresh data sources from metadata store.
+   */
+  public void refreshDataSources() throws TException {
+    catalog_.refreshDataSources();
+  }
+
+  public byte[] getNullPartitionName() throws ImpalaException, TException {
+    return execAndSerialize("getNullPartitionName", "Getting null partition name", () -> {
+      TGetNullPartitionNameResponse response = new TGetNullPartitionNameResponse();
+      try (MetaStoreClient msClient = catalog_.getMetaStoreClient()) {
+        response.setPartition_value(
+            MetaStoreUtil.getNullPartitionKeyValue(msClient.getHiveClient()));
+        response.setStatus(new TStatus(TErrorCode.OK, Lists.newArrayList()));
+      }
+      return response;
+    });
+  }
+
+  public byte[] getLatestCompactions(byte[] thriftParams)
+      throws ImpalaException, TException {
+    TGetLatestCompactionsRequest request = new TGetLatestCompactionsRequest();
+    JniUtil.deserializeThrift(protocolFactory_, request, thriftParams);
+    return execAndSerialize("getLatestCompactions", "Getting latest compactions", () -> {
+      TGetLatestCompactionsResponse response = new TGetLatestCompactionsResponse();
+      try (MetaStoreClient msClient = catalog_.getMetaStoreClient()) {
+        response.setPartition_to_compaction_id(MetastoreShim.getLatestCompactions(
+            msClient, request.db_name, request.table_name, request.partition_names,
+            request.non_parition_name, request.last_compaction_id));
+        response.setStatus(new TStatus(TErrorCode.OK, Lists.newArrayList()));
+      }
+      return response;
+    });
+  }
+
+  /**
+   * Returns the serialized byte array of TGetAllHadoopConfigsResponse
+   */
+  public byte[] getAllHadoopConfigs() throws ImpalaException {
+    Map<String, String> configs = Maps.newHashMap();
+    for (Map.Entry<String, String> e: HIVE_CONF) {
+      configs.put(e.getKey(), e.getValue());
+    }
+    TGetAllHadoopConfigsResponse result = new TGetAllHadoopConfigsResponse();
+    result.setConfigs(configs);
+    try {
+      TSerializer serializer = new TSerializer(protocolFactory_);
+      return serializer.serialize(result);
+    } catch (TException e) {
+      throw new InternalException(e.getMessage());
+    }
+  }
 }
