@@ -22,6 +22,7 @@ import com.google.common.base.Preconditions;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 import org.apache.hadoop.conf.Configuration;
+import org.apache.hadoop.hive.conf.HiveConf;
 import org.apache.hadoop.fs.CommonConfigurationKeys;
 import org.apache.hadoop.fs.CommonConfigurationKeysPublic;
 import org.apache.hadoop.fs.FileSystem;
@@ -38,8 +39,10 @@ import org.apache.impala.authentication.saml.WrappedWebContext;
 import org.apache.impala.authorization.AuthorizationFactory;
 import org.apache.impala.authorization.ImpalaInternalAdminUser;
 import org.apache.impala.authorization.User;
+import org.apache.impala.catalog.DatabaseNotFoundException;
 import org.apache.impala.catalog.FeDataSource;
 import org.apache.impala.catalog.FeDb;
+import org.apache.impala.catalog.FeTable;
 import org.apache.impala.catalog.Function;
 import org.apache.impala.catalog.StructType;
 import org.apache.impala.catalog.Type;
@@ -53,6 +56,7 @@ import org.apache.impala.service.Frontend.PlanCtx;
 import org.apache.impala.thrift.TBackendGflags;
 import org.apache.impala.thrift.TBuildTestDescriptorTableParams;
 import org.apache.impala.thrift.TCatalogObject;
+import org.apache.impala.thrift.TColumnValue;
 import org.apache.impala.thrift.TDatabase;
 import org.apache.impala.thrift.TDescribeDbParams;
 import org.apache.impala.thrift.TDescribeOutputStyle;
@@ -74,6 +78,7 @@ import org.apache.impala.thrift.TGetHadoopConfigResponse;
 import org.apache.impala.thrift.TGetHadoopGroupsRequest;
 import org.apache.impala.thrift.TGetHadoopGroupsResponse;
 import org.apache.impala.thrift.TGetTableHistoryResult;
+import org.apache.impala.thrift.TGetMetadataTablesParams;
 import org.apache.impala.thrift.TGetTablesParams;
 import org.apache.impala.thrift.TGetTablesResult;
 import org.apache.impala.thrift.TLoadDataReq;
@@ -89,7 +94,9 @@ import org.apache.impala.thrift.TShowGrantPrincipalParams;
 import org.apache.impala.thrift.TShowRolesParams;
 import org.apache.impala.thrift.TShowStatsOp;
 import org.apache.impala.thrift.TShowStatsParams;
+import org.apache.impala.thrift.TStringLiteral;
 import org.apache.impala.thrift.TDescribeHistoryParams;
+import org.apache.impala.thrift.TSessionState;
 import org.apache.impala.thrift.TTableName;
 import org.apache.impala.thrift.TUniqueId;
 import org.apache.impala.thrift.TUpdateCatalogCacheRequest;
@@ -112,6 +119,7 @@ import org.slf4j.LoggerFactory;
 import java.io.File;
 import java.io.IOException;
 import java.lang.IllegalArgumentException;
+import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Enumeration;
 import java.util.List;
@@ -126,6 +134,8 @@ public class JniFrontend {
   private final static TBinaryProtocol.Factory protocolFactory_ =
       new TBinaryProtocol.Factory();
   private final Frontend frontend_;
+  public final static String KEYSTORE_ERROR_MSG = "Failed to get password from" +
+      "keystore, error: invalid key '%s' or password doesn't exist";
 
   /**
    * Create a new instance of the Jni Frontend.
@@ -251,10 +261,11 @@ public class JniFrontend {
   }
 
   /**
-   * Implement Hive's pattern-matching semantics for "SHOW TABLE [[LIKE] 'pattern']", and
-   * return a list of table names matching an optional pattern.
-   * The only metacharacters are '*' which matches any string of characters, and '|'
-   * which denotes choice.  Doing the work here saves loading tables or databases from the
+   * Returns a list of table names matching an optional pattern.
+   *
+   * Implements Hive's pattern-matching semantics for "SHOW TABLE [[LIKE] 'pattern']". The
+   * only metacharacters are '*' which matches any string of characters, and '|' which
+   * denotes choice. Doing the work here saves loading tables or databases from the
    * metastore (which Hive would do if we passed the call through to the metastore
    * client). If the pattern is null, all strings are considered to match. If it is an
    * empty string, no strings match.
@@ -267,13 +278,47 @@ public class JniFrontend {
     Preconditions.checkNotNull(frontend_);
     TGetTablesParams params = new TGetTablesParams();
     JniUtil.deserializeThrift(protocolFactory_, params, thriftGetTablesParams);
-    // If the session was not set it indicates this is an internal Impala call.
-    User user = params.isSetSession() ?
-        new User(TSessionStateUtil.getEffectiveUser(params.getSession())) :
-        ImpalaInternalAdminUser.getInstance();
+
+    TSessionState session = params.isSetSession() ? params.getSession() : null;
+    User user = getUser(session);
 
     Preconditions.checkState(!params.isSetSession() || user != null );
     List<String> tables = frontend_.getTableNames(params.db,
+        PatternMatcher.createHivePatternMatcher(params.pattern), user,
+        params.getTable_types());
+
+    TGetTablesResult result = new TGetTablesResult();
+    result.setTables(tables);
+
+    try {
+      TSerializer serializer = new TSerializer(protocolFactory_);
+      return serializer.serialize(result);
+    } catch (TException e) {
+      throw new InternalException(e.getMessage());
+    }
+  }
+
+  /**
+   * Returns the metadata tables available for the given table. Currently only Iceberg
+   * metadata tables are supported.
+   *
+   * Pattern matching is done as in getTableNames().
+   *
+   * The argument is a serialized TGetMetadataTablesParams object.
+   * The return type is a serialised TGetTablesResult object.
+   * @see Frontend#getTableNames
+   */
+  public byte[] getMetadataTableNames(byte[] thriftGetMetadataTablesParams)
+      throws ImpalaException {
+    Preconditions.checkNotNull(frontend_);
+    TGetMetadataTablesParams params = new TGetMetadataTablesParams();
+    JniUtil.deserializeThrift(protocolFactory_, params, thriftGetMetadataTablesParams);
+
+    TSessionState session = params.isSetSession() ? params.getSession() : null;
+    User user = getUser(session);
+
+    Preconditions.checkState(!params.isSetSession() || user != null );
+    List<String> tables = frontend_.getMetadataTableNames(params.db, params.tbl,
         PatternMatcher.createHivePatternMatcher(params.pattern), user);
 
     TGetTablesResult result = new TGetTablesResult();
@@ -320,10 +365,10 @@ public class JniFrontend {
     Preconditions.checkNotNull(frontend_);
     TGetDbsParams params = new TGetDbsParams();
     JniUtil.deserializeThrift(protocolFactory_, params, thriftGetTablesParams);
-    // If the session was not set it indicates this is an internal Impala call.
-    User user = params.isSetSession() ?
-        new User(TSessionStateUtil.getEffectiveUser(params.getSession())) :
-        ImpalaInternalAdminUser.getInstance();
+
+    TSessionState session = params.isSetSession() ? params.getSession() : null;
+    User user = getUser(session);
+
     List<? extends FeDb> dbs = frontend_.getDbs(
         PatternMatcher.createHivePatternMatcher(params.pattern), user);
     TGetDbsResult result = new TGetDbsResult();
@@ -490,18 +535,9 @@ public class JniFrontend {
     Preconditions.checkNotNull(frontend_);
     TDescribeTableParams params = new TDescribeTableParams();
     JniUtil.deserializeThrift(protocolFactory_, params, thriftDescribeTableParams);
-
     Preconditions.checkState(params.isSetTable_name() ^ params.isSetResult_struct());
     User user = new User(TSessionStateUtil.getEffectiveUser(params.getSession()));
-    TDescribeResult result = null;
-    if (params.isSetTable_name()) {
-      result = frontend_.describeTable(params.getTable_name(), params.output_style, user);
-    } else {
-      Preconditions.checkState(params.output_style == TDescribeOutputStyle.MINIMAL);
-      StructType structType = (StructType)Type.fromThrift(params.result_struct);
-      result = DescribeResultFactory.buildDescribeMinimalResult(structType);
-    }
-
+    TDescribeResult result = frontend_.describeTable(params, user);
     try {
       TSerializer serializer = new TSerializer(protocolFactory_);
       return serializer.serialize(result);
@@ -614,17 +650,27 @@ public class JniFrontend {
     frontend_.waitForCatalog();
   }
 
-  // Caching this saves ~50ms per call to getHadoopConfigAsHtml
+  FeTable getCatalogTable(byte[] tableNameParam) throws ImpalaException {
+    Preconditions.checkNotNull(frontend_);
+    TTableName tableName = new TTableName();
+    JniUtil.deserializeThrift(protocolFactory_, tableName, tableNameParam);
+    return frontend_.getCatalog().getTable(tableName.db_name, tableName.table_name);
+  }
+
+  // Caching this saves ~50ms per call to getHadoopConfig
   private static final Configuration CONF = new Configuration();
   private static final Groups GROUPS = Groups.getUserToGroupsMappingService(CONF);
 
+  // Caching this saves ~50ms per call to getAllHadoopConfigs
+  // org.apache.hadoop.hive.conf.HiveConf inherrits org.apache.hadoop.conf.Configuration
+  private static final HiveConf HIVE_CONF = new HiveConf();
+
   /**
-   * Returns a string of all loaded Hadoop configuration parameters as a table of keys
-   * and values. If asText is true, output in raw text. Otherwise, output in html.
+   * Returns the serialized byte array of TGetAllHadoopConfigsResponse
    */
   public byte[] getAllHadoopConfigs() throws ImpalaException {
     Map<String, String> configs = Maps.newHashMap();
-    for (Map.Entry<String, String> e: CONF) {
+    for (Map.Entry<String, String> e: HIVE_CONF) {
       configs.put(e.getKey(), e.getValue());
     }
     TGetAllHadoopConfigsResponse result = new TGetAllHadoopConfigsResponse();
@@ -762,6 +808,31 @@ public class JniFrontend {
     }
   }
 
+  /**
+   * Returns secret from the configured KeyStore.
+   * @param secretKeyRequest the serialized secret key to be used for extracting secret.
+   */
+  public static String getSecretFromKeyStore(byte[] secretKeyRequest)
+      throws ImpalaException {
+    final TStringLiteral secretKey = new TStringLiteral();
+    JniUtil.deserializeThrift(protocolFactory_, secretKey, secretKeyRequest);
+    String secret = null;
+    try {
+      char[] secretCharArray = CONF.getPassword(secretKey.getValue());
+      if (secretCharArray != null) {
+        secret = new String(secretCharArray);
+      } else {
+        String errMsg = String.format(KEYSTORE_ERROR_MSG, secretKey.getValue());
+        LOG.error(errMsg);
+        throw new InternalException(errMsg);
+      }
+    } catch (IOException e) {
+      LOG.error("Failed to get password from keystore, error: " + e);
+      throw new InternalException(e.getMessage());
+    }
+    return secret;
+  }
+
   public String validateSaml2Bearer(byte[] serializedRequest) throws ImpalaException{
     Preconditions.checkNotNull(frontend_);
     Preconditions.checkNotNull(frontend_.getSaml2Client());
@@ -916,6 +987,14 @@ public class JniFrontend {
     return output.toString();
   }
 
+  private User getUser(TSessionState session) {
+    // If the session was not set it indicates this is an internal Impala call.
+    User user = session == null ?
+        ImpalaInternalAdminUser.getInstance() :
+        new User(TSessionStateUtil.getEffectiveUser(session));
+    return user;
+  }
+
   /**
    * Return an empty string if the default FileSystem configured in CONF refers to a
    * DistributedFileSystem and Impala can list the root directory "/". Otherwise,
