@@ -37,6 +37,7 @@ import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.fs.permission.FsAction;
 import org.apache.hadoop.fs.permission.FsPermission;
+import org.apache.hadoop.security.AccessControlException;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.hadoop.hdfs.protocol.AclException;
 import static org.apache.hadoop.hdfs.client.HdfsClientConfigKeys.DeprecatedKeys.DFS_PERMISSIONS_SUPERUSERGROUP_KEY;
@@ -310,6 +311,26 @@ public class FsPermissionChecker {
     return new Permissions(fileStatus, aclStatus);
   }
 
+  /**
+   * Returns true if the current user can perform the given action given these
+   * permissions.
+   */
+  public boolean checkAccess(Path path, FileSystem fs, FsAction action,
+      boolean isAuthzEnabled) throws IOException {
+    // When Ranger authz is enabled, we invoke method
+    // FileSystem#access(Path path, FsAction mode) to check the actual access permission.
+    if (isAuthzEnabled) {
+      try{
+        fs.access(path, action);
+      } catch (AccessControlException e) {
+        LOG.warn(e.getMessage());
+        return false;
+      }
+      return true;
+    }
+    return this.getPermissions(fs, path).checkPermissions(action);
+  }
+
   /**
    * Returns the FsPermissionChecker singleton.
    */
