@@ -27,6 +27,7 @@ import com.google.common.primitives.Longs;
 import com.google.flatbuffers.FlatBufferBuilder;
 import java.io.IOException;
 import java.nio.ByteBuffer;
+import java.time.DateTimeException;
 import java.time.Instant;
 import java.time.LocalDateTime;
 import java.time.OffsetDateTime;
@@ -56,19 +57,33 @@ import org.apache.iceberg.TableScan;
 import org.apache.iceberg.Transaction;
 import org.apache.iceberg.catalog.TableIdentifier;
 import org.apache.iceberg.expressions.Expression;
+import org.apache.iceberg.expressions.Expressions;
+import org.apache.iceberg.expressions.Literal;
+import org.apache.iceberg.expressions.Term;
 import org.apache.iceberg.hadoop.HadoopFileIO;
 import org.apache.iceberg.io.CloseableIterable;
 import org.apache.iceberg.mr.Catalogs;
 import org.apache.iceberg.transforms.PartitionSpecVisitor;
+import org.apache.iceberg.transforms.Transforms;
 import org.apache.iceberg.types.Conversions;
 import org.apache.iceberg.types.Type;
 import org.apache.iceberg.types.Types;
+import org.apache.impala.analysis.Analyzer;
+import org.apache.impala.analysis.Expr;
+import org.apache.impala.analysis.FunctionCallExpr;
+import org.apache.impala.analysis.FunctionName;
+import org.apache.impala.analysis.FunctionParams;
 import org.apache.impala.analysis.IcebergPartitionField;
 import org.apache.impala.analysis.IcebergPartitionSpec;
 import org.apache.impala.analysis.IcebergPartitionTransform;
+import org.apache.impala.analysis.NumericLiteral;
+import org.apache.impala.analysis.StatementBase;
+import org.apache.impala.analysis.StringLiteral;
 import org.apache.impala.analysis.TimeTravelSpec;
 import org.apache.impala.analysis.TimeTravelSpec.Kind;
+import org.apache.impala.catalog.BuiltinsDb;
 import org.apache.impala.catalog.Catalog;
+import org.apache.impala.catalog.Column;
 import org.apache.impala.catalog.FeIcebergTable;
 import org.apache.impala.catalog.HdfsFileFormat;
 import org.apache.impala.catalog.IcebergColumn;
@@ -81,6 +96,7 @@ import org.apache.impala.catalog.iceberg.IcebergCatalogs;
 import org.apache.impala.catalog.iceberg.IcebergHadoopCatalog;
 import org.apache.impala.catalog.iceberg.IcebergHadoopTables;
 import org.apache.impala.catalog.iceberg.IcebergHiveCatalog;
+import org.apache.impala.common.AnalysisException;
 import org.apache.impala.common.FileSystemUtil;
 import org.apache.impala.common.ImpalaRuntimeException;
 import org.apache.impala.common.Pair;
@@ -101,8 +117,7 @@ import org.apache.impala.thrift.TIcebergPartitionTransformType;
 
 @SuppressWarnings("UnstableApiUsage")
 public class IcebergUtil {
-
-  private static final int ICEBERG_EPOCH_YEAR = 1970;
+  public static final int ICEBERG_EPOCH_YEAR = 1970;
   private static final int ICEBERG_EPOCH_MONTH = 1;
   @SuppressWarnings("unused")
   private static final int ICEBERG_EPOCH_DAY = 1;
@@ -232,13 +247,60 @@ public class IcebergUtil {
       } else if (transformType == TIcebergPartitionTransformType.VOID) {
         builder.alwaysNull(partitionField.getOrig_field_name());
       } else {
-        throw new ImpalaRuntimeException(String.format("Skip partition: %s, %s",
-            partitionField.getOrig_field_name(), transformType));
+        throw new ImpalaRuntimeException(
+            String.format("Unknown partition transform '%s' for field '%s",
+                transformType, partitionField.getOrig_field_name()));
       }
     }
     return builder.build();
   }
 
+  /**
+   * Transforms TIcebergPartitionSpec to a list of Iceberg terms.
+   * @param partSpec The partition spec Thrift object
+   * @return List of Iceberg terms.
+   * @throws ImpalaRuntimeException When the transform type is not supported
+   */
+  public static List<Term> getPartitioningTerms(TIcebergPartitionSpec partSpec)
+      throws ImpalaRuntimeException {
+    List<Term> result = new ArrayList<>();
+    for (TIcebergPartitionField field : partSpec.getPartition_fields()) {
+      result.add(getPartitioningTerm(field));
+    }
+    return result;
+  }
+
+  /**
+   * Retrieves the Iceberg partitioning term for a given partition field.
+   * @param field partition field for which the partitioning term is generated.
+   * @return The Iceberg partitioning term as a Term.
+   * @throws ImpalaRuntimeException When the transform type is not supported
+   * @see TIcebergPartitionField
+   * @see Expressions
+   * @see Transforms
+   */
+  private static Term getPartitioningTerm(TIcebergPartitionField field)
+      throws ImpalaRuntimeException {
+    TIcebergPartitionTransformType transformType = field.getTransform()
+        .getTransform_type();
+      String fieldName = field.getOrig_field_name();
+    int transformParam = field.getTransform().getTransform_param();
+    switch (transformType) {
+      case IDENTITY: return Expressions.transform(fieldName, Transforms.identity());
+      case HOUR: return Expressions.hour(fieldName);
+      case DAY: return Expressions.day(fieldName);
+      case MONTH: return Expressions.month(fieldName);
+      case YEAR: return Expressions.year(fieldName);
+      case BUCKET: return Expressions.bucket(fieldName, transformParam);
+      case TRUNCATE: return Expressions.truncate(fieldName, transformParam);
+      case VOID: return Expressions.transform(fieldName, Transforms.alwaysNull());
+      default:
+        throw new ImpalaRuntimeException(
+            String.format("Unknown partition transform '%s' for field '%s",
+                transformType, fieldName));
+      }
+  }
+
   /**
    * Returns true if 'msTable' uses HiveCatalog.
    */
@@ -390,28 +452,27 @@ public class IcebergUtil {
     return pSize;
   }
 
-  public static IcebergPartitionTransform getPartitionTransform(
-      PartitionField field, HashMap<String, Integer> transformParams)
-      throws TableLoadingException {
+  public static IcebergPartitionTransform getPartitionTransform(PartitionField field,
+      Map<String, Integer> transformParams) throws ImpalaRuntimeException {
     String type = field.transform().toString();
     String transformMappingKey = getPartitionTransformMappingKey(field.sourceId(),
         getPartitionTransformType(type));
     return getPartitionTransform(type, transformParams.get(transformMappingKey));
   }
 
-  public static IcebergPartitionTransform getPartitionTransform(String transformType,
-      Integer transformParam) throws TableLoadingException {
+  public static IcebergPartitionTransform getPartitionTransform(
+      String transformType, Integer transformParam) throws ImpalaRuntimeException {
     return new IcebergPartitionTransform(getPartitionTransformType(transformType),
         transformParam);
   }
 
   public static IcebergPartitionTransform getPartitionTransform(String transformType)
-      throws TableLoadingException {
+      throws ImpalaRuntimeException {
     return getPartitionTransform(transformType, null);
   }
 
-  private static TIcebergPartitionTransformType getPartitionTransformType(
-      String transformType) throws TableLoadingException {
+  public static TIcebergPartitionTransformType getPartitionTransformType(
+      String transformType) throws ImpalaRuntimeException {
     Preconditions.checkNotNull(transformType);
     transformType = transformType.toUpperCase();
     if ("IDENTITY".equals(transformType)) {
@@ -428,8 +489,8 @@ public class IcebergUtil {
       case "YEAR":  case "YEARS":  return TIcebergPartitionTransformType.YEAR;
       case "VOID": return TIcebergPartitionTransformType.VOID;
       default:
-        throw new TableLoadingException("Unsupported iceberg partition type: " +
-            transformType);
+        throw new ImpalaRuntimeException(
+            "Unsupported Iceberg partition type: " + transformType);
     }
   }
 
@@ -449,7 +510,7 @@ public class IcebergUtil {
    * expose the interface of the transform types outside their package and the only
    * way to get the transform's parameter is implementing this visitor class.
    */
-  public static HashMap<String, Integer> getPartitionTransformParams(PartitionSpec spec) {
+  public static Map<String, Integer> getPartitionTransformParams(PartitionSpec spec) {
     List<Pair<String, Integer>> transformParams = PartitionSpecVisitor.visit(
         spec, new PartitionSpecVisitor<Pair<String, Integer>>() {
           @Override
@@ -585,6 +646,12 @@ public class IcebergUtil {
     }
   }
 
+  public static long getSnapshotId(FeIcebergTable table, TimeTravelSpec timeTravelSpec) {
+    if (timeTravelSpec == null) return table.snapshotId();
+    TableScan scan = createScanAsOf(table, timeTravelSpec);
+    return scan.snapshot().snapshotId();
+  }
+
   private static TableScan createScanAsOf(FeIcebergTable table,
       TimeTravelSpec timeTravelSpec) {
     TableScan scan = table.getIcebergApiTable().newScan();
@@ -613,6 +680,24 @@ public class IcebergUtil {
     return scan;
   }
 
+  public static GroupedContentFiles getIcebergFilesFromSnapshot(
+      FeIcebergTable table, List<Expression> predicates, long snapshotId)
+      throws TableLoadingException {
+    if (table.snapshotId() == -1) {
+      return new GroupedContentFiles(CloseableIterable.empty());
+    }
+    TableScan scan = table.getIcebergApiTable().newScan();
+    scan = scan.useSnapshot(snapshotId);
+    for (Expression predicate : predicates) {
+      scan = scan.filter(predicate);
+    }
+    try (CloseableIterable<FileScanTask> fileScanTasks = scan.planFiles()) {
+      return new GroupedContentFiles(fileScanTasks);
+    } catch (IOException e) {
+      throw new TableLoadingException("Error during reading Iceberg manifest files.", e);
+    }
+  }
+
   /**
    * Use ContentFile path to generate 128-bit Murmur3 hash as map key, cached in memory
    */
@@ -745,6 +830,39 @@ public class IcebergUtil {
     throw new ImpalaRuntimeException("Unexpected partition transform: " + transformType);
   }
 
+  /**
+   * Returns the integer representation of date transforms
+   * @param transformType type of the transform
+   * @param stringValue date value as a string
+   * @return Integer representation of a transform value, or an empty optional if the
+   * parse failed, or the supplied transform is not supported.
+   */
+  public static Integer getDateTimeTransformValue(
+      TIcebergPartitionTransformType transformType, String stringValue)
+      throws ImpalaRuntimeException {
+    try {
+      switch (transformType) {
+        case YEAR: return parseYearToTransformYear(stringValue);
+        case MONTH: return parseMonthToTransformMonth(stringValue);
+        case DAY: return parseDayToTransformMonth(stringValue);
+        case HOUR: return parseHourToTransformHour(stringValue);
+      }
+    } catch (NumberFormatException | DateTimeException | IllegalStateException e) {
+      throw new ImpalaRuntimeException(
+          String.format("Unable to parse value '%s' as '%s' transform value", stringValue,
+              transformType));
+    }
+    throw new ImpalaRuntimeException("Unexpected partition transform: " + transformType);
+  }
+
+  public static boolean isDateTimeTransformType(
+      TIcebergPartitionTransformType transformType) {
+    return transformType.equals(TIcebergPartitionTransformType.YEAR)
+        || transformType.equals(TIcebergPartitionTransformType.MONTH)
+        || transformType.equals(TIcebergPartitionTransformType.DAY)
+        || transformType.equals(TIcebergPartitionTransformType.HOUR);
+  }
+
   /**
    * In the partition path years are represented naturally, e.g. 1984. However, we need
    * to convert it to an integer which represents the years from 1970. So, for 1984 the
@@ -769,13 +887,23 @@ public class IcebergUtil {
     return years * 12 + months;
   }
 
+  /**
+   * In the partition path days are represented as <year>-<month>-<day>, e.g. 2023-12-12.
+   * This functions converts this string to an integer which represents the days from
+   * '1970-01-01' with the help of Iceberg's type converter.
+   */
+  private static Integer parseDayToTransformMonth(String monthStr) {
+    Literal<Integer> days = Literal.of(monthStr).to(Types.DateType.get());
+    return days.value();
+  }
+
   /**
    * In the partition path hours are represented as <year>-<month>-<day>-<hour>, e.g.
    * 1970-01-01-01. We need to convert it to a single integer which represents the hours
    * from '1970-01-01 00:00:00'.
    */
   private static Integer parseHourToTransformHour(String hourStr) {
-    final OffsetDateTime EPOCH = Instant.ofEpochSecond(0).atOffset(ZoneOffset.UTC);
+    final OffsetDateTime epoch = Instant.ofEpochSecond(0).atOffset(ZoneOffset.UTC);
     String[] parts = hourStr.split("-", -1);
     Preconditions.checkState(parts.length == 4);
     int year = Integer.parseInt(parts[0]);
@@ -785,7 +913,7 @@ public class IcebergUtil {
     OffsetDateTime datetime = OffsetDateTime.of(
         LocalDateTime.of(year, month, day, hour, /*minute=*/0),
         ZoneOffset.UTC);
-    return (int)ChronoUnit.HOURS.between(EPOCH, datetime);
+    return (int) ChronoUnit.HOURS.between(epoch, datetime);
   }
 
   public static TCompressionCodec parseParquetCompressionCodec(
@@ -929,6 +1057,12 @@ public class IcebergUtil {
     if (spec != null && !spec.fields().isEmpty()) {
       partKeysOffset = createPartitionKeys(feTbl, fbb, spec, cf);
     }
+    int eqFieldIdsOffset = -1;
+    List<Integer> eqFieldIds = cf.equalityFieldIds();
+    if (eqFieldIds != null && !eqFieldIds.isEmpty()) {
+      eqFieldIdsOffset = FbIcebergMetadata.createEqualityFieldIdsVector(fbb,
+          eqFieldIds.stream().mapToInt(i -> i).sorted().toArray());
+    }
     FbIcebergMetadata.startFbIcebergMetadata(fbb);
     byte fileFormat = -1;
     if (cf.format() == FileFormat.PARQUET) fileFormat = FbIcebergDataFileFormat.PARQUET;
@@ -942,6 +1076,22 @@ public class IcebergUtil {
     if (partKeysOffset != -1) {
       FbIcebergMetadata.addPartitionKeys(fbb, partKeysOffset);
     }
+    if (cf.dataSequenceNumber() != null) {
+      FbIcebergMetadata.addDataSequenceNumber(fbb, cf.dataSequenceNumber());
+    } else {
+      // According to comments from the Iceberg code, data sequence numbers could be null
+      // when files were written with "older" Iceberg versions. Quote from the code
+      // comments of Iceberg's ContentFile.dataSequenceNumber():
+      // "This method can return null if the data sequence number is unknown. This may
+      // happen while reading a v2 manifest that did not persist the data sequence number
+      // for manifest entries with status DELETED (older Iceberg versions)."
+      FbIcebergMetadata.addDataSequenceNumber(fbb, -1l);
+    }
+
+    if (eqFieldIdsOffset != -1) {
+      FbIcebergMetadata.addEqualityFieldIds(fbb, eqFieldIdsOffset);
+    }
+
     return FbIcebergMetadata.endFbIcebergMetadata(fbb);
   }
 
@@ -1082,4 +1232,122 @@ public class IcebergUtil {
 
     return props;
   }
+
+  /**
+   * Checks the table before insert for unsupported types and file formats.
+   */
+  public static void validateIcebergTableForInsert(FeIcebergTable iceTable)
+      throws AnalysisException {
+    for (Types.NestedField field : iceTable.getIcebergSchema().columns()) {
+      org.apache.iceberg.types.Type iceType = field.type();
+      if (iceType.isPrimitiveType() && iceType instanceof Types.TimestampType) {
+        Types.TimestampType tsType = (Types.TimestampType)iceType;
+        if (tsType.shouldAdjustToUTC()) {
+          throw new AnalysisException("The Iceberg table has a TIMESTAMPTZ " +
+              "column that Impala cannot write.");
+        }
+      }
+    }
+    for (Column c : iceTable.getColumns()) {
+      if (c.getType().isComplexType()) {
+        throw new AnalysisException(String.format("Impala does not support writing " +
+                "tables with complex types. Table '%s' has column '%s' " +
+                "with type: %s", iceTable.getFullName(), c.getName(),
+            c.getType().toSql()));
+      }
+    }
+    if (iceTable.getIcebergFileFormat() != TIcebergFileFormat.PARQUET) {
+      throw new AnalysisException(String.format("Impala can only write Parquet data " +
+              "files, while table '%s' expects '%s' data files.",
+          iceTable.getFullName(), iceTable.getIcebergFileFormat().toString()));
+    }
+  }
+
+  /**
+   * Creates partition key expressions in the order of Iceberg partition spec fields.
+   */
+  public static void populatePartitionExprs(Analyzer analyzer,
+      List<Expr> widestTypeExprList, List<Column> selectExprTargetColumns,
+      List<Expr> selectListExprs, FeIcebergTable targetTable,
+      List<Expr> outPartitionExprs, List<Integer> outPartitionColPos)
+      throws AnalysisException {
+    Preconditions.checkNotNull(outPartitionExprs);
+    Preconditions.checkState(outPartitionExprs.isEmpty());
+    Preconditions.checkState(outPartitionColPos == null || outPartitionColPos.isEmpty());
+    IcebergPartitionSpec icebergPartSpec = targetTable.getDefaultPartitionSpec();
+    if (!icebergPartSpec.hasPartitionFields()) return;
+    String tableName = targetTable.getFullName();
+    for (IcebergPartitionField partField : icebergPartSpec.getIcebergPartitionFields()) {
+      if (partField.getTransformType() == TIcebergPartitionTransformType.VOID) continue;
+      for (int i = 0; i < selectListExprs.size(); ++i) {
+        IcebergColumn targetColumn = (IcebergColumn)selectExprTargetColumns.get(i);
+        if (targetColumn.getFieldId() != partField.getSourceId()) continue;
+        // widestTypeExpr is the widest type expression for column i
+        Expr widestTypeExpr =
+            (widestTypeExprList != null) ? widestTypeExprList.get(i) : null;
+        Expr icebergPartitionTransformExpr = getIcebergPartitionTransformExpr(analyzer,
+            tableName, partField, targetColumn, selectListExprs.get(i), widestTypeExpr);
+        outPartitionExprs.add(icebergPartitionTransformExpr);
+        if (outPartitionColPos != null) {
+          outPartitionColPos.add(targetColumn.getPosition());
+        }
+        break;
+      }
+    }
+  }
+
+  private static Expr getIcebergPartitionTransformExpr(Analyzer analyzer,
+      String targetTableName, IcebergPartitionField partField, IcebergColumn targetColumn,
+      Expr sourceExpr, Expr widestTypeExpr) throws AnalysisException {
+    Preconditions.checkState(targetColumn.getFieldId() == partField.getSourceId());
+    Expr compatibleExpr = StatementBase.checkTypeCompatibility(targetTableName,
+        targetColumn, sourceExpr, analyzer, widestTypeExpr);
+    Expr ret = IcebergUtil.getIcebergPartitionTransformExpr(partField, compatibleExpr);
+    ret.analyze(analyzer);
+    return ret;
+  }
+
+  /**
+   * Returns the partition transform expression. E.g. if the partition transform is DAY,
+   * it returns 'to_date(compatibleExpr)'. If the partition transform is BUCKET,
+   * it returns 'iceberg_bucket_transform(compatibleExpr, transformParam)'.
+   */
+  private static Expr getIcebergPartitionTransformExpr(IcebergPartitionField partField,
+                                                Expr compatibleExpr) {
+    String funcNameStr = transformTypeToFunctionName(partField.getTransformType());
+    if (funcNameStr == null || funcNameStr.equals("")) return compatibleExpr;
+    FunctionName fnName = new FunctionName(BuiltinsDb.NAME, funcNameStr);
+    List<Expr> paramList = new ArrayList<>();
+    paramList.add(compatibleExpr);
+    Integer transformParam = partField.getTransformParam();
+    if (transformParam != null) {
+      paramList.add(NumericLiteral.create(transformParam));
+    }
+    if (partField.getTransformType() == TIcebergPartitionTransformType.MONTH) {
+      paramList.add(new StringLiteral("yyyy-MM"));
+    }
+    else if (partField.getTransformType() == TIcebergPartitionTransformType.HOUR) {
+      paramList.add(new StringLiteral("yyyy-MM-dd-HH"));
+    }
+    FunctionCallExpr fnCall = new FunctionCallExpr(fnName, new FunctionParams(paramList));
+    fnCall.setIsInternalFnCall(true);
+    return fnCall;
+  }
+
+  /**
+   * Returns the builtin function to use for the given partition transform.
+   */
+  private static String transformTypeToFunctionName(
+      TIcebergPartitionTransformType transformType) {
+    switch (transformType) {
+      case IDENTITY: return "";
+      case HOUR:
+      case MONTH: return "from_timestamp";
+      case DAY: return "to_date";
+      case YEAR: return "year";
+      case BUCKET: return "iceberg_bucket_transform";
+      case TRUNCATE: return "iceberg_truncate_transform";
+    }
+    return "";
+  }
 }
