@@ -1895,4 +1895,43 @@ public class ToSqlTest extends FrontendTestBase {
     testToSql(":shutdown('hostname', 1000)");
     testToSql(":shutdown(1000)");
   }
+
+  /**
+   * Tests that SHOW TABLES statements are output correctly.
+   */
+  @Test
+  public void testShowTables() {
+    testToSql("SHOW TABLES", "default", "SHOW TABLES");
+    testToSql("SHOW TABLES IN functional");
+    testToSql("SHOW TABLES LIKE 'alltypes*'", "functional",
+        "SHOW TABLES LIKE 'alltypes*'");
+    testToSql("SHOW TABLES IN functional LIKE 'alltypes*'");
+  }
+
+  /**
+   * Tests that SHOW METADATA TABLES statements are output correctly.
+   */
+  @Test
+  public void testShowMetadataTables() {
+    String q1 = "SHOW METADATA TABLES IN iceberg_query_metadata";
+    testToSql(q1, "functional_parquet", q1);
+    String q2 = "SHOW METADATA TABLES IN iceberg_query_metadata LIKE '*file*'";
+    testToSql(q2, "functional_parquet", q2);
+
+    testToSql("SHOW METADATA TABLES IN functional_parquet.iceberg_query_metadata");
+    testToSql("SHOW METADATA TABLES IN functional_parquet.iceberg_query_metadata " +
+        "LIKE '*file*'");
+  }
+
+  /**
+   * Test SHOW VIEWS statements are output correctly.
+   */
+  @Test
+  public void testShowViews() {
+    testToSql("SHOW VIEWS", "default", "SHOW VIEWS");
+    testToSql("SHOW VIEWS IN functional");
+    testToSql("SHOW VIEWS LIKE 'alltypes*'", "functional",
+        "SHOW VIEWS LIKE 'alltypes*'");
+    testToSql("SHOW VIEWS IN functional LIKE 'alltypes*'");
+  }
 }
