@@ -68,6 +68,7 @@ import org.apache.impala.thrift.TPrivilege;
 import org.apache.impala.thrift.TPrivilegeLevel;
 import org.apache.impala.thrift.TPrivilegeScope;
 import org.apache.impala.thrift.TTableName;
+import org.apache.impala.util.NoOpEventSequence;
 import org.junit.After;
 import org.junit.Assume;
 import org.junit.Before;
@@ -364,7 +365,7 @@ public class CatalogTest {
     stats.reset();
     catalog_.invalidateTable(new TTableName("functional", "alltypes"),
         /*tblWasRemoved=*/new Reference<Boolean>(),
-        /*dbWasAdded=*/new Reference<Boolean>());
+        /*dbWasAdded=*/new Reference<Boolean>(), NoOpEventSequence.INSTANCE);
 
     HdfsTable table = (HdfsTable)catalog_.getOrLoadTable("functional", "AllTypes",
         "test", null);
@@ -390,7 +391,7 @@ public class CatalogTest {
 
     // Now test REFRESH on the table...
     stats.reset();
-    catalog_.reloadTable(table, "test");
+    catalog_.reloadTable(table, "test", NoOpEventSequence.INSTANCE);
 
     // Again, we expect only one getFileStatus call, for the top-level directory.
     assertEquals(1L, (long)opsCounts.getLong(GET_FILE_STATUS));
@@ -409,7 +410,7 @@ public class CatalogTest {
         new TPartitionKeyValue("year", "2010"),
         new TPartitionKeyValue("month", "10"));
     catalog_.reloadPartition(table, partitionSpec, new Reference<>(false),
-        CatalogObject.ThriftObjectType.NONE, "test");
+        CatalogObject.ThriftObjectType.NONE, "test", NoOpEventSequence.INSTANCE);
     assertEquals(0L, (long)opsCounts.getLong(GET_FILE_BLOCK_LOCS));
 
     // Loading or reloading an unpartitioned table with some files in it should not make
@@ -419,7 +420,7 @@ public class CatalogTest {
         "functional", "alltypesaggmultifilesnopart", "test", null);
     assertEquals(0L, (long)opsCounts.getLong(GET_FILE_BLOCK_LOCS));
     stats.reset();
-    catalog_.reloadTable(unpartTable, "test");
+    catalog_.reloadTable(unpartTable, "test", NoOpEventSequence.INSTANCE);
     assertEquals(0L, (long)opsCounts.getLong(GET_FILE_BLOCK_LOCS));
 
     // Simulate an empty partition, which will trigger the full
@@ -432,7 +433,7 @@ public class CatalogTest {
     table.updatePartition(partBuilder);
     stats.reset();
     catalog_.reloadPartition(table, partitionSpec, new Reference<>(false),
-        CatalogObject.ThriftObjectType.NONE, "test");
+        CatalogObject.ThriftObjectType.NONE, "test", NoOpEventSequence.INSTANCE);
 
     // Should not scan the directory file-by-file, should use a single
     // listLocatedStatus() to get the whole directory (partition)
@@ -736,8 +737,8 @@ public class CatalogTest {
     // Partitioned table with stats. Invalidate the table prior to fetching.
     Reference<Boolean> tblWasRemoved = new Reference<Boolean>();
     Reference<Boolean> dbWasAdded = new Reference<Boolean>();
-    catalog_.invalidateTable(
-        new TTableName("functional", "alltypesagg"), tblWasRemoved, dbWasAdded);
+    catalog_.invalidateTable(new TTableName("functional", "alltypesagg"),
+        tblWasRemoved, dbWasAdded, NoOpEventSequence.INSTANCE);
     expectStatistics("functional", "alltypesagg", 11);
 
     // Unpartitioned table with no stats.
