@@ -440,6 +440,11 @@ public class FrontendTestBase extends AbstractFrontendTest {
           public void postAnalyze(AuthorizationContext authzCtx) {
           }
 
+          @Override
+          public boolean roleExists(String roleName) {
+            return catalog_.getAuthPolicy().getRole(roleName) != null;
+          }
+
           @Override
           public AuthorizationContext createAuthorizationContext(boolean doAudits,
               String sqlStmt, TSessionState sessionState,
