@@ -40,6 +40,8 @@ import org.apache.thrift.transport.THttpClient;
 import org.apache.thrift.protocol.TBinaryProtocol;
 import org.junit.ClassRule;
 import org.junit.Test;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 @CreateDS(name = "myDS",
     partitions = { @CreatePartition(name = "test", suffix = "dc=myorg,dc=com") })
@@ -51,6 +53,8 @@ import org.junit.Test;
  * ldap authentication is being used.
  */
 public class LdapHS2Test {
+  private static final Logger LOG = LoggerFactory.getLogger(LdapHS2Test.class);
+
   @ClassRule
   public static CreateLdapServerRule serverRule = new CreateLdapServerRule();
 
@@ -92,7 +96,9 @@ public class LdapHS2Test {
     verifySuccess(fetchResp.getStatus());
     List<TColumn> columns = fetchResp.getResults().getColumns();
     assertEquals(1, columns.size());
-    assertEquals(expectedResult, columns.get(0).getStringVal().getValues().get(0));
+    if (expectedResult != null) {
+      assertEquals(expectedResult, columns.get(0).getStringVal().getValues().get(0));
+    }
 
     return execResp.getOperationHandle();
   }
@@ -494,6 +500,86 @@ public class LdapHS2Test {
     hiveserver2TrustedDomainAuthTestBody(false);
   }
 
+  @Test
+  public void testHiveserver2TrustedDomainEmptyXffHeaderUseOrigin() throws Exception {
+    setUp("--trusted_domain=localhost --trusted_domain_use_xff_header=true " +
+          "--trusted_domain_empty_xff_header_use_origin=true");
+    verifyMetrics(0, 0);
+    verifyTrustedDomainMetrics(0);
+    THttpClient transport = new THttpClient("http://localhost:28000");
+    Map<String, String> headers = new HashMap<String, String>();
+
+    // Case 1: Authenticate as 'Test1Ldap' without password, send X-Forwarded-For header
+    headers.put("Authorization", "Basic VGVzdDFMZGFwOg==");
+    headers.put("X-Forwarded-For", "127.0.0.1");
+    transport.setCustomHeaders(headers);
+    transport.open();
+    TCLIService.Iface client = new TCLIService.Client(new TBinaryProtocol(transport));
+
+    // Open a session which will get username 'Test1Ldap'.
+    TOpenSessionReq openReq = new TOpenSessionReq();
+    TOpenSessionResp openResp = client.OpenSession(openReq);
+    // One successful authentication.
+    verifyMetrics(0, 0);
+    verifyTrustedDomainMetrics(1);
+    // Running a query should succeed.
+    TOperationHandle operationHandle = execAndFetch(
+            client, openResp.getSessionHandle(), "select logged_in_user()", "Test1Ldap");
+    // Two more successful authentications - for the Exec() and the Fetch().
+    verifyMetrics(0, 0);
+    verifyTrustedDomainMetrics(3);
+
+    // Case 2: Authenticate as 'Test1Ldap' without password, do not send X-Forwarded-For
+    // header
+    headers.put("Authorization", "Basic VGVzdDFMZGFwOg==");
+    headers.remove("X-Forwarded-For");
+    transport.setCustomHeaders(headers);
+    openResp = client.OpenSession(openReq);
+    verifyMetrics(0, 0);
+    verifyTrustedDomainMetrics(4);
+    operationHandle = execAndFetch(client, openResp.getSessionHandle(),
+            "select logged_in_user()", "Test1Ldap");
+    verifyMetrics(0, 0);
+    // Two more successful authentications - for the Exec() and the Fetch().
+    verifyTrustedDomainMetrics(6);
+
+    // Case 3: Authenticate as 'Test1Ldap' without password, send X-Forwarded-For header
+    // that does not match trusted_domain
+    headers.put("Authorization", "Basic VGVzdDFMZGFwOg==");
+    headers.put("X-Forwarded-For", "126.0.23.1");
+    transport.setCustomHeaders(headers);
+    try {
+      openResp = client.OpenSession(openReq);
+      fail("Authentication should fail without password.");
+    } catch (Exception e) {
+      assertEquals(e.getMessage(), "HTTP Response code: 401");
+      // One basic auth failure
+      verifyMetrics(0, 1);
+      // And no more successful authentication
+      verifyTrustedDomainMetrics(6);
+    }
+
+    // Case 4: Authenticate as 'Test1Ldap' without password, do not send X-Forwarded-For
+    // header and the origin does not match trusted_domain
+    setUp("--trusted_domain=any.domain --trusted_domain_use_xff_header=true " +
+            "--trusted_domain_empty_xff_header_use_origin=true");
+    headers.put("Authorization", "Basic VGVzdDFMZGFwOg==");
+    headers.remove("X-Forwarded-For");
+    transport.setCustomHeaders(headers);
+    verifyMetrics(0, 0);
+    verifyTrustedDomainMetrics(0);
+    try {
+      openResp = client.OpenSession(openReq);
+      fail("Authentication should fail without password.");
+    } catch (Exception e) {
+      assertEquals(e.getMessage(), "HTTP Response code: 401");
+      // One basic auth failure
+      verifyMetrics(0, 1);
+      // No successful trusted domain authentications
+      verifyTrustedDomainMetrics(0);
+    }
+  }
+
   /**
    * Tests if authentication is skipped when connections to the HTTP hiveserver2
    * endpoint have trusted auth header.
@@ -624,4 +710,137 @@ public class LdapHS2Test {
       assertEquals(e.getMessage(), "HTTP Response code: 401");
     }
   }
+
+  /**
+   * Tests LDAP for reading Impala table through JDBC external data source.
+   * Assume that Impala JDBC driver has been downloaded and copied to
+   * ${FILESYSTEM_PREFIX}/test-warehouse/data-sources/jdbc-drivers/.
+   */
+  @Test
+  public void testImpalaExtJdbcTables() throws Exception {
+    setUp("");
+    verifyMetrics(0, 0);
+    THttpClient transport = new THttpClient("http://localhost:28000");
+    Map<String, String> headers = new HashMap<String, String>();
+    // Authenticate as 'Test1Ldap' with password '12345'
+    headers.put("Authorization", "Basic VGVzdDFMZGFwOjEyMzQ1");
+    transport.setCustomHeaders(headers);
+    transport.open();
+    TCLIService.Iface client = new TCLIService.Client(new TBinaryProtocol(transport));
+
+    // Open a session which will get username 'Test1Ldap'.
+    TOpenSessionReq openReq = new TOpenSessionReq();
+    TOpenSessionResp openResp = client.OpenSession(openReq);
+    TSessionHandle session = openResp.getSessionHandle();
+    // One successful authentication.
+    verifyMetrics(1, 0);
+
+    // Define queries.
+    String fileSystemPrefix = System.getenv("FILESYSTEM_PREFIX");
+    String internalListenHost = System.getenv("INTERNAL_LISTEN_HOST");
+    String dropTableQuery = "DROP TABLE IF EXISTS %s";
+    // Set JDBC authentication mechanisms as LDAP (3) with username/password as
+    // TEST_USER_1/TEST_PASSWORD_1.
+    String createTableQuery =
+        String.format("CREATE EXTERNAL TABLE impala_jdbc_ext_test_table (" +
+        "id INT, bool_col BOOLEAN, tinyint_col TINYINT, smallint_col SMALLINT, " +
+        "int_col INT, bigint_col BIGINT, float_col FLOAT, double_col DOUBLE, " +
+        "date_string_col STRING, string_col STRING, timestamp_col TIMESTAMP) " +
+        "STORED BY JDBC TBLPROPERTIES (" +
+        "\"database.type\"=\"IMPALA\", " +
+        "\"jdbc.url\"=\"jdbc:impala://%s:21050/functional\", " +
+        "\"jdbc.auth\"=\"AuthMech=3\", " +
+        "\"jdbc.driver\"=\"com.cloudera.impala.jdbc.Driver\", " +
+        "\"driver.url\"=\"%s/test-warehouse/data-sources/jdbc-drivers/" +
+        "ImpalaJDBC42.jar\", " +
+        "\"dbcp.username\"=\"%s\", " +
+        "\"dbcp.password\"=\"%s\", " +
+        "\"table\"=\"alltypes\")",
+        internalListenHost, fileSystemPrefix, TEST_USER_1, TEST_PASSWORD_1);
+    // Set JDBC authentication mechanisms as LDAP with wrong password.
+    String createTableWithWrongPassword =
+        String.format("CREATE EXTERNAL TABLE impala_jdbc_tbl_wrong_password (" +
+        "id INT, bool_col BOOLEAN, tinyint_col TINYINT, smallint_col SMALLINT, " +
+        "int_col INT, bigint_col BIGINT, float_col FLOAT, double_col DOUBLE, " +
+        "date_string_col STRING, string_col STRING, timestamp_col TIMESTAMP) " +
+       "STORED BY JDBC TBLPROPERTIES (" +
+        "\"database.type\"=\"IMPALA\", " +
+        "\"jdbc.url\"=\"jdbc:impala://%s:21050/functional\", " +
+        "\"jdbc.auth\"=\"AuthMech=3\", " +
+        "\"jdbc.driver\"=\"com.cloudera.impala.jdbc.Driver\", " +
+        "\"driver.url\"=\"%s/test-warehouse/data-sources/jdbc-drivers/" +
+        "ImpalaJDBC42.jar\", " +
+        "\"dbcp.username\"=\"%s\", " +
+        "\"dbcp.password\"=\"wrong-password\", " +
+        "\"table\"=\"alltypes\")",
+        internalListenHost, fileSystemPrefix, TEST_USER_1);
+    // Set JDBC authentication mechanisms as LDAP without AuthMech.
+    String createTableWithoutAuthMech =
+        String.format("CREATE EXTERNAL TABLE impala_jdbc_tbl_without_auth_mech (" +
+        "id INT, bool_col BOOLEAN, tinyint_col TINYINT, smallint_col SMALLINT, " +
+        "int_col INT, bigint_col BIGINT, float_col FLOAT, double_col DOUBLE, " +
+        "date_string_col STRING, string_col STRING, timestamp_col TIMESTAMP) " +
+        "STORED BY JDBC TBLPROPERTIES (" +
+        "\"database.type\"=\"IMPALA\", " +
+        "\"jdbc.url\"=\"jdbc:impala://%s:21050/functional\", " +
+        "\"jdbc.driver\"=\"com.cloudera.impala.jdbc.Driver\", " +
+        "\"driver.url\"=\"%s/test-warehouse/data-sources/jdbc-drivers/" +
+        "ImpalaJDBC42.jar\", " +
+        "\"dbcp.username\"=\"%s\", " +
+        "\"dbcp.password\"=\"%s\", " +
+        "\"table\"=\"alltypes\")",
+        internalListenHost, fileSystemPrefix, TEST_USER_1, TEST_PASSWORD_1);
+    String selectQuery = "select string_col from %s where id=9";
+
+    // Run queries.
+    //
+    // Create JDBC tables.
+    execAndFetch(client, session,
+        String.format(dropTableQuery, "impala_jdbc_ext_test_table"), null);
+    execAndFetch(client, session, createTableQuery, "Table has been created.");
+    execAndFetch(client, session,
+        String.format(dropTableQuery, "impala_jdbc_tbl_wrong_password"), null);
+    execAndFetch(client, session, createTableWithWrongPassword,
+        "Table has been created.");
+    execAndFetch(client, session,
+        String.format(dropTableQuery, "impala_jdbc_tbl_without_auth_mech"), null);
+    execAndFetch(client, session, createTableWithoutAuthMech, "Table has been created.");
+
+    // Successfully access JDBC data source table with LDAP.
+    execAndFetch(client, session,
+        String.format(selectQuery, "impala_jdbc_ext_test_table"), "9");
+    // Negative case for JDBC data source table with wrong password.
+    String expectedError = "Error initialized or created transport for authentication";
+    try {
+      execAndFetch(client, session,
+          String.format(selectQuery, "impala_jdbc_tbl_wrong_password"), "9");
+      fail("Expected error: " + expectedError);
+    } catch (Exception e) {
+      assertTrue(e.getMessage().contains(expectedError));
+    }
+    // Negative case for JDBC data source table without AuthMech.
+    expectedError = "Communication link failure. Failed to connect to server";
+    try {
+      execAndFetch(client, session,
+          String.format(selectQuery, "impala_jdbc_tbl_without_auth_mech"), "9");
+      fail("Expected error: " + expectedError);
+    } catch (Exception e) {
+      assertTrue(String.format("Authentication failed with error: %s", e.getMessage()),
+          e.getMessage().contains(expectedError));
+    }
+
+    // Drop JDBC tables.
+    execAndFetch(client, session,
+        String.format(dropTableQuery, "impala_jdbc_ext_test_table"),
+        "Table has been dropped.");
+    execAndFetch(client, session,
+        String.format(dropTableQuery, "impala_jdbc_tbl_wrong_password"),
+        "Table has been dropped.");
+    execAndFetch(client, session,
+        String.format(dropTableQuery, "impala_jdbc_tbl_without_auth_mech"),
+        "Table has been dropped.");
+
+    // Two successful authentications for each ExecAndFetch().
+    verifyMetrics(25, 0);
+  }
 }
