@@ -22,6 +22,7 @@ import static org.junit.Assert.assertEquals;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
 
 import org.apache.impala.catalog.Catalog;
@@ -32,6 +33,7 @@ import org.apache.impala.catalog.HBaseColumn;
 import org.apache.impala.catalog.Type;
 import org.apache.impala.common.ImpalaException;
 import org.apache.impala.datagenerator.HBaseTestDataRegionAssignment;
+import org.apache.impala.service.BackendConfig;
 import org.apache.impala.service.Frontend.PlanCtx;
 import org.apache.impala.testutil.TestUtils;
 import org.apache.impala.testutil.TestUtils.IgnoreValueFilter;
@@ -51,6 +53,7 @@ import com.google.common.base.Preconditions;
 import com.google.common.collect.Collections2;
 import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Lists;
+import com.google.common.collect.Maps;
 
 // All planner tests, except for S3 specific tests should go here.
 public class PlannerTest extends PlannerTestBase {
@@ -577,7 +580,11 @@ public class PlannerTest extends PlannerTestBase {
   public void testSmallQueryOptimization() {
     TQueryOptions options = new TQueryOptions();
     options.setExec_single_node_rows_threshold(8);
-    runPlannerTestFile("small-query-opt", options);
+    addTestDb("kudu_planner_test", "Test DB for Kudu Planner.");
+    addTestTable("CREATE EXTERNAL TABLE kudu_planner_test.no_stats STORED AS KUDU "
+        + "TBLPROPERTIES ('kudu.table_name' = 'impala::functional_kudu.alltypes');");
+    runPlannerTestFile("small-query-opt", options,
+        ImmutableSet.of(PlannerTestOption.VALIDATE_CARDINALITY));
   }
 
   @Test
@@ -679,9 +686,6 @@ public class PlannerTest extends PlannerTestBase {
     options.unsetEnabled_runtime_filter_types();
     options.addToEnabled_runtime_filter_types(TRuntimeFilterType.BLOOM);
     options.addToEnabled_runtime_filter_types(TRuntimeFilterType.MIN_MAX);
-    addTestDb("kudu_planner_test", "Test DB for Kudu Planner.");
-    addTestTable("CREATE EXTERNAL TABLE kudu_planner_test.no_stats STORED AS KUDU " +
-        "TBLPROPERTIES ('kudu.table_name' = 'impala::functional_kudu.alltypes');");
     runPlannerTestFile("kudu", options);
   }
 
@@ -965,7 +969,8 @@ public class PlannerTest extends PlannerTestBase {
     TQueryOptions options = defaultQueryOptions();
     runPlannerTestFile("tablesample", options,
         ImmutableSet.of(PlannerTestOption.EXTENDED_EXPLAIN,
-            PlannerTestOption.DO_NOT_VALIDATE_ROWCOUNT_ESTIMATION_FOR_PARTITIONS));
+            PlannerTestOption.DO_NOT_VALIDATE_ROWCOUNT_ESTIMATION_FOR_PARTITIONS,
+            PlannerTestOption.VALIDATE_ICEBERG_SNAPSHOT_IDS));
   }
 
   @Test
@@ -1182,6 +1187,8 @@ public class PlannerTest extends PlannerTestBase {
         filter.transform(" foo=bar cardinality=10.3K"));
     assertEquals(" foo=bar cardinality=",
         filter.transform(" foo=bar cardinality=unavailable"));
+    assertEquals(" foo=bar cardinality=",
+        filter.transform(" foo=bar cardinality=1.58K(filtered from 2.88M)"));
     filter = TestUtils.ROW_SIZE_FILTER;
     assertEquals(" row-size= cardinality=10.3K",
         filter.transform(" row-size=10B cardinality=10.3K"));
@@ -1274,13 +1281,29 @@ public class PlannerTest extends PlannerTestBase {
         ImmutableSet.of(PlannerTestOption.VALIDATE_CARDINALITY));
   }
 
+  /**
+   * Checks exercising predicate pushdown with Iceberg tables, without predicate
+   * subsetting.
+   */
+  @Test
+  public void testDisabledIcebergPredicateSubsetting() {
+    TQueryOptions queryOptions = new TQueryOptions();
+    queryOptions.setIceberg_predicate_pushdown_subsetting(false);
+    runPlannerTestFile("iceberg-predicates-disabled-subsetting", "functional_parquet",
+        queryOptions, ImmutableSet.of(PlannerTestOption.VALIDATE_CARDINALITY,
+            PlannerTestOption.VALIDATE_ICEBERG_SNAPSHOT_IDS));
+  }
+
   /**
    * Check that Iceberg V2 table scans work as expected.
    */
   @Test
   public void testIcebergV2TableScans() {
-    runPlannerTestFile("iceberg-v2-tables", "functional_parquet",
-        ImmutableSet.of(PlannerTestOption.VALIDATE_CARDINALITY));
+    TQueryOptions options = defaultQueryOptions();
+    options.setTimezone("UTC");
+    runPlannerTestFile("iceberg-v2-tables", "functional_parquet", options,
+        ImmutableSet.of(PlannerTestOption.VALIDATE_CARDINALITY,
+            PlannerTestOption.VALIDATE_ICEBERG_SNAPSHOT_IDS));
   }
 
   /**
@@ -1291,7 +1314,8 @@ public class PlannerTest extends PlannerTestBase {
     TQueryOptions options = defaultQueryOptions();
     options.setDisable_optimized_iceberg_v2_read(true);
     runPlannerTestFile("iceberg-v2-tables-hash-join", "functional_parquet", options,
-        ImmutableSet.of(PlannerTestOption.VALIDATE_CARDINALITY));
+        ImmutableSet.of(PlannerTestOption.VALIDATE_CARDINALITY,
+            PlannerTestOption.VALIDATE_ICEBERG_SNAPSHOT_IDS));
   }
 
   /**
@@ -1300,6 +1324,25 @@ public class PlannerTest extends PlannerTestBase {
   @Test
   public void testIcebergV2Delete() {
     runPlannerTestFile("iceberg-v2-delete", "functional_parquet",
+        ImmutableSet.of(PlannerTestOption.VALIDATE_CARDINALITY,
+            PlannerTestOption.VALIDATE_ICEBERG_SNAPSHOT_IDS));
+  }
+
+  @Test
+  public void testIcebergV2Update() {
+    runPlannerTestFile("iceberg-v2-update", "functional_parquet",
+        ImmutableSet.of(PlannerTestOption.VALIDATE_CARDINALITY));
+  }
+
+  /**
+   * Test that OPTIMIZE TABLE statements on Iceberg tables work as expected.
+   */
+  @Test
+  public void testIcebergOptimize() {
+    TQueryOptions options = defaultQueryOptions();
+    options.setMax_fs_writers(2);
+    options.setExplain_level(TExplainLevel.EXTENDED);
+    runPlannerTestFile("iceberg-optimize", "functional_parquet", options,
         ImmutableSet.of(PlannerTestOption.VALIDATE_CARDINALITY));
   }
 
@@ -1310,6 +1353,12 @@ public class PlannerTest extends PlannerTestBase {
   public void testIcebergMetadataTableScans() {
     runPlannerTestFile("iceberg-metadata-table-scan", "functional_parquet",
         ImmutableSet.of(PlannerTestOption.VALIDATE_CARDINALITY));
+
+    TQueryOptions options = defaultQueryOptions();
+    options.setExplain_level(TExplainLevel.EXTENDED);
+    runPlannerTestFile("iceberg-metadata-table-joined-with-regular-table",
+        "functional_parquet", options, ImmutableSet.of(
+        PlannerTestOption.DO_NOT_VALIDATE_ROWCOUNT_ESTIMATION_FOR_PARTITIONS));
   }
 
   /**
@@ -1364,6 +1413,11 @@ public class PlannerTest extends PlannerTestBase {
     runPlannerTestFile("tpcds-dist-method", "tpcds");
   }
 
+  @Test
+  public void testOrcStatsAgg() {
+    runPlannerTestFile("orc-stats-agg");
+  }
+
   /**
    * Test new hint of 'TABLE_NUM_ROWS'
    */
@@ -1391,8 +1445,19 @@ public class PlannerTest extends PlannerTestBase {
     options.setCompute_processing_cost(true);
     options.setProcessing_cost_min_threads(2);
     options.setMax_fragment_instances_per_node(16);
-    runPlannerTestFile(
-        "tpcds-processing-cost", "tpcds_parquet", options, tpcdsParquetTestOptions());
+    runPlannerTestFile("tpcds-processing-cost", "tpcds_partitioned_parquet_snap", options,
+        tpcdsParquetTestOptions());
+  }
+
+  /**
+   * Test that shows query plan for the same test cases at EE test
+   * test_processing_cost.py::TestProcessingCost::test_admission_slots.
+   */
+  @Test
+  public void testProcessingCostPlanAdmissionSlots() {
+    TQueryOptions options = tpcdsParquetQueryOptions();
+    runPlannerTestFile("processing-cost-plan-admission-slots",
+        "tpcds_partitioned_parquet_snap", options, tpcdsParquetTestOptions());
   }
 
   /**
@@ -1406,20 +1471,111 @@ public class PlannerTest extends PlannerTestBase {
 
   /**
    * Test that memory estimate of aggregation node equals to the default estimation
-   * (with NDV multiplication method) if AGG_MEM_CORRELATION_FACTOR=1.0 or
+   * (with NDV multiplication method) if AGG_MEM_CORRELATION_FACTOR=0.0 or
    * LARGE_AGG_MEM_THRESHOLD is less than or equal to zero.
    */
   @Test
   public void testAggNodeMaxMemEstimate() {
     Set<PlannerTestOption> testOptions = tpcdsParquetTestOptions();
     TQueryOptions options = tpcdsParquetQueryOptions();
-    double defaultAggMemCorrelationFactor = options.getAgg_mem_correlation_factor();
-    options.setAgg_mem_correlation_factor(1.0);
+    options.setAgg_mem_correlation_factor(0.0);
+    options.setLarge_agg_mem_threshold(512 * 1024 * 1024);
     runPlannerTestFile(
         "agg-node-max-mem-estimate", "tpcds_parquet", options, testOptions);
 
+    options.setAgg_mem_correlation_factor(0.5);
     options.setLarge_agg_mem_threshold(0);
     runPlannerTestFile(
         "agg-node-max-mem-estimate", "tpcds_parquet", options, testOptions);
   }
+
+  /**
+   * Test that high AGG_MEM_CORRELATION_FACTOR results in low memory estimate
+   * for aggregation node.
+   */
+  @Test
+  public void testAggNodeLowMemEstimate() {
+    Set<PlannerTestOption> testOptions = tpcdsParquetTestOptions();
+    TQueryOptions options = tpcdsParquetQueryOptions();
+    options.setAgg_mem_correlation_factor(0.9);
+    runPlannerTestFile(
+        "agg-node-low-mem-estimate", "tpcds_parquet", options, testOptions);
+  }
+
+  /**
+   * Test that low AGG_MEM_CORRELATION_FACTOR results in high memory estimate
+   * for aggregation node.
+   */
+  @Test
+  public void testAggNodeHighMemEstimate() {
+    Set<PlannerTestOption> testOptions = tpcdsParquetTestOptions();
+    TQueryOptions options = tpcdsParquetQueryOptions();
+    options.setAgg_mem_correlation_factor(0.1);
+    runPlannerTestFile(
+        "agg-node-high-mem-estimate", "tpcds_parquet", options, testOptions);
+  }
+
+  /**
+   * Test cardinality reduction by runtime filter.
+   */
+  @Test
+  public void testRuntimeFilterCardinalityReduction() {
+    runPlannerTestFile("runtime-filter-cardinality-reduction", "tpcds_parquet",
+        ImmutableSet.of(
+            PlannerTestOption.EXTENDED_EXPLAIN, PlannerTestOption.VALIDATE_CARDINALITY));
+  }
+
+  /**
+   * Test cardinality reduction by runtime filter against Kudu.
+   */
+  @Test
+  public void testRuntimeFilterCardinalityReductionOnKudu() {
+    runPlannerTestFile("runtime-filter-cardinality-reduction-on-kudu", "tpch_kudu",
+        ImmutableSet.of(PlannerTestOption.VALIDATE_CARDINALITY));
+  }
+
+  /**
+   * Test that how the different equality delete field ID lists are ordered for reading.
+   */
+  @Test
+  public void testEqualityDeleteFieldIdOrdering() {
+    Map<List<Integer>, Long> inp = Maps.newHashMap();
+    inp.put(Lists.newArrayList(2, 3), 2L);
+    inp.put(Lists.newArrayList(1, 2, 3), 2L);
+    inp.put(Lists.newArrayList(1, 2), 2L);
+    inp.put(Lists.newArrayList(4), 3L);
+    inp.put(Lists.newArrayList(1), 3L);
+    Assert.assertEquals(
+        Lists.newArrayList(
+            Lists.newArrayList(1),
+            Lists.newArrayList(4),
+            Lists.newArrayList(1, 2, 3),
+            Lists.newArrayList(1, 2),
+            Lists.newArrayList(2, 3)),
+        IcebergScanPlanner.getOrderedEqualityFieldIds(inp));
+  }
+
+  /**
+   * Performance test of planning for a query with a large number of expressions,
+   * as commonly seen in auto-generated queries.
+   */
+  @Test
+  public void testManyExpressionPerformance() {
+    addTestDb("test_many_expressions", "Test DB for many-expression perf test.");
+    addTestTable("create table test_many_expressions.i6bd46c0a (i20868d03 bigint, " +
+        "i89d57f06 bigint, idc633f9e bigint, i9e32db9e bigint, ic02dc181 date, " +
+        "ia74a4bc0 string, i3ee20502 bigint, ic05c995e bigint, i49dcf5c5 string, " +
+        "i99bd997e string, i510ab56c string, i11be1ec1 bigint, i81012615 bigint) " +
+        "location '/'");
+    runPlannerTestFile("many-expression", "test_many_expressions");
+  }
+
+  /**
+   * Test that runtime filters with the same expression repeated multiple times are
+   * planned successfully (IMPALA-13270).
+   */
+  @Test
+  public void testRuntimeFilterRepeatedExpr() {
+    runPlannerTestFile("runtime-filter-repeated-expr", "functional");
+  }
 }
