@@ -38,10 +38,12 @@ import org.apache.impala.analysis.ColumnLineageGraph;
 import org.apache.impala.analysis.DescriptorTable;
 import org.apache.impala.catalog.Catalog;
 import org.apache.impala.catalog.CatalogException;
+import org.apache.impala.catalog.SideloadTableStats;
 import org.apache.impala.common.FrontendTestBase;
 import org.apache.impala.common.ImpalaException;
 import org.apache.impala.common.RuntimeEnv;
 import org.apache.impala.service.Frontend.PlanCtx;
+import org.apache.impala.testutil.StatsJsonParser;
 import org.apache.impala.testutil.TestFileParser;
 import org.apache.impala.testutil.TestFileParser.Section;
 import org.apache.impala.testutil.TestFileParser.TestCase;
@@ -65,8 +67,10 @@ import org.apache.impala.thrift.TPlanNode;
 import org.apache.impala.thrift.TQueryCtx;
 import org.apache.impala.thrift.TQueryExecRequest;
 import org.apache.impala.thrift.TQueryOptions;
+import org.apache.impala.thrift.TReplicaPreference;
 import org.apache.impala.thrift.TScanRangeLocationList;
 import org.apache.impala.thrift.TScanRangeSpec;
+import org.apache.impala.thrift.TSlotCountStrategy;
 import org.apache.impala.thrift.TTableDescriptor;
 import org.apache.impala.thrift.TTableSink;
 import org.apache.impala.thrift.TTupleDescriptor;
@@ -90,8 +94,8 @@ import com.google.common.collect.Sets;
 public class PlannerTestBase extends FrontendTestBase {
   private final static Logger LOG = LoggerFactory.getLogger(PlannerTest.class);
   private final static boolean GENERATE_OUTPUT_FILE = true;
-  private final java.nio.file.Path testDir_ = Paths.get("functional-planner", "queries",
-      "PlannerTest");
+  private final static java.nio.file.Path testDir_ =
+      Paths.get("functional-planner", "queries", "PlannerTest");
   protected static java.nio.file.Path outDir_;
   private static KuduClient kuduClient_;
 
@@ -102,15 +106,14 @@ public class PlannerTestBase extends FrontendTestBase {
   // Map from table ID (TTableId) to the table descriptor with that ID.
   private final Map<Integer, TTableDescriptor> tableMap_ = Maps.newHashMap();
 
-  @BeforeClass
-  public static void setUp() throws Exception {
-    // Mimic the 3 node test mini-cluster.
+  protected static void setUpWithSize(int num_executors, int expected_num_executors)
+      throws Exception {
     TUpdateExecutorMembershipRequest updateReq = new TUpdateExecutorMembershipRequest();
     updateReq.setIp_addresses(Sets.newHashSet("127.0.0.1"));
     updateReq.setHostnames(Sets.newHashSet("localhost"));
     TExecutorGroupSet group_set = new TExecutorGroupSet();
-    group_set.curr_num_executors = 3;
-    group_set.expected_num_executors = 20; // default num_expected_executors startup flag
+    group_set.curr_num_executors = num_executors;
+    group_set.expected_num_executors = expected_num_executors;
     updateReq.setExec_group_sets(new ArrayList<TExecutorGroupSet>());
     updateReq.getExec_group_sets().add(group_set);
     ExecutorMembershipSnapshot.update(updateReq);
@@ -121,6 +124,13 @@ public class PlannerTestBase extends FrontendTestBase {
     outDir_ = Paths.get(logDir, "PlannerTest");
   }
 
+  @BeforeClass
+  public static void setUp() throws Exception {
+    // Mimic the 3 node test mini-cluster.
+    // 20 is the default num_expected_executors startup flag.
+    setUpWithSize(3, 20);
+  }
+
   @Before
   public void setUpTest() throws Exception {
     // Reset the RuntimeEnv - individual tests may change it.
@@ -353,12 +363,13 @@ public class PlannerTestBase extends FrontendTestBase {
    * Extracts and returns the expected error message from expectedPlan.
    * Returns null if expectedPlan is empty or its first element is not an error message.
    * The accepted format for error messages is the exception string. We currently
-   * support only NotImplementedException and InternalException.
+   * support only NotImplementedException, InternalException, and AnalysisException.
    */
   private String getExpectedErrorMessage(ArrayList<String> expectedPlan) {
     if (expectedPlan == null || expectedPlan.isEmpty()) return null;
     if (!expectedPlan.get(0).contains("NotImplementedException") &&
-        !expectedPlan.get(0).contains("InternalException")) return null;
+        !expectedPlan.get(0).contains("InternalException") &&
+        !expectedPlan.get(0).contains("AnalysisException")) return null;
     return expectedPlan.get(0).trim();
   }
 
@@ -411,6 +422,15 @@ public class PlannerTestBase extends FrontendTestBase {
     return options;
   }
 
+  protected static TQueryOptions tpcdsParquetCpuCostQueryOptions() {
+    return tpcdsParquetQueryOptions()
+        .setCompute_processing_cost(true)
+        .setMax_fragment_instances_per_node(12)
+        .setReplica_preference(TReplicaPreference.REMOTE)
+        .setSlot_count_strategy(TSlotCountStrategy.PLANNER_CPU_ASK)
+        .setPlanner_testcase_mode(true);
+  }
+
   protected static Set<PlannerTestOption> tpcdsParquetTestOptions() {
     return ImmutableSet.of(PlannerTestOption.EXTENDED_EXPLAIN,
         PlannerTestOption.INCLUDE_RESOURCE_HEADER, PlannerTestOption.VALIDATE_RESOURCES,
@@ -573,6 +593,9 @@ public class PlannerTestBase extends FrontendTestBase {
               PlannerTestOption.DO_NOT_VALIDATE_ROWCOUNT_ESTIMATION_FOR_PARTITIONS)) {
         resultFilters.add(TestUtils.PARTITIONS_FILTER);
       }
+      if (!testOptions.contains(PlannerTestOption.VALIDATE_ICEBERG_SNAPSHOT_IDS)) {
+        resultFilters.add(TestUtils.ICEBERG_SNAPSHOT_ID_FILTER);
+      }
 
       String planDiff = TestUtils.compareOutput(
           Lists.newArrayList(explainStr.split("\n")), expectedPlan, true, resultFilters);
@@ -893,7 +916,11 @@ public class PlannerTestBase extends FrontendTestBase {
     DISABLE_HDFS_NUM_ROWS_ESTIMATE,
     // If set, make no attempt to validate the estimated number of rows for any
     // partitions in an hdfs table.
-    DO_NOT_VALIDATE_ROWCOUNT_ESTIMATION_FOR_PARTITIONS
+    DO_NOT_VALIDATE_ROWCOUNT_ESTIMATION_FOR_PARTITIONS,
+    // Verify that the snapshot ids in the plan match to the expected values. We
+    // can only do this for tests that operate on pre-written Iceberg tables,
+    // e.g. functional_parquet.iceberg_partitioned.
+    VALIDATE_ICEBERG_SNAPSHOT_IDS
   }
 
   protected void runPlannerTestFile(String testFile, TQueryOptions options) {
@@ -1003,4 +1030,12 @@ public class PlannerTestBase extends FrontendTestBase {
   protected boolean scanRangeLocationsCheckEnabled() {
     return true;
   }
+
+  protected static Map<String, Map<String, SideloadTableStats>> loadStatsJson(
+      String statsJsonPath) {
+    String fileName = testDir_.resolve(statsJsonPath).toString();
+    StatsJsonParser statsParser = new StatsJsonParser(fileName);
+    statsParser.parseFile();
+    return statsParser.getDbStatsMap();
+  }
 }
