@@ -133,6 +133,7 @@ class ImpalaHttpClient(TTransportBase):
     self.__get_custom_headers_func = None
     self.__basic_auth = None
     self.__kerb_service = None
+    self.__add_custom_headers_funcs = []
 
   @staticmethod
   def basic_proxy_auth_header(proxy):
@@ -186,6 +187,18 @@ class ImpalaHttpClient(TTransportBase):
     custom_headers['Authorization'] = "Basic " + self.__basic_auth
     return custom_headers
 
+  def getCustomHeadersWithBearerAuth(self, cookie_header, has_auth_cookie):
+    custom_headers = {}
+    if cookie_header:
+      # Add cookies to HTTP header.
+      custom_headers['Cookie'] = cookie_header
+    # Add the 'Authorization' header to request even if the auth cookie is
+    # present to avoid a round trip in case the cookie is expired when server
+    # receive the request. Since the 'auth' value is calculated once, so it
+    # won't cause a performance issue.
+    custom_headers['Authorization'] = "Bearer {0}".format(self.__bearer_token)
+    return custom_headers
+
   def getCustomHeadersWithNegotiateAuth(self, cookie_header, has_auth_cookie):
     import kerberos
     custom_headers = {}
@@ -216,6 +229,12 @@ class ImpalaHttpClient(TTransportBase):
     self.__basic_auth = basic_auth
     self.__get_custom_headers_func = self.getCustomHeadersWithBasicAuth
 
+  # Set function to generate customized HTTP headers for JWT authorization.
+  def setJwtAuth(self, jwt):
+    # auth mechanism: JWT
+    self.__bearer_token = jwt
+    self.__get_custom_headers_func = self.getCustomHeadersWithBearerAuth
+
   # Set function to generate customized HTTP headers for Kerberos authorization.
   def setKerberosAuth(self, kerb_service):
     # auth mechanism: GSSAPI
@@ -227,13 +246,41 @@ class ImpalaHttpClient(TTransportBase):
     # auth mechanism: None
     self.__get_custom_headers_func = self.getCustomHeadersWithoutAuth
 
+  # Whenever http(s) calls are made to the backend impala, each function
+  # added through this method will be called.  Thus, arbitrary custom
+  # headers can be set on each request.
+  # parameters:
+  #  funcs - tuple of functions where each takes no arguments and returns
+  #      a dict of http headers
+  # Note:  if the custom function returns a http header with a name that
+  # does not start with "X-" or "x-", it will cause an error to be thrown
+  def addCustomHeaderFunc(self, *funcs):
+    if funcs is None:
+      return
+
+    for f in funcs:
+      self.__add_custom_headers_funcs.append(f)
+
   # Update HTTP headers based on the saved cookies and auth mechanism.
   def refreshCustomHeaders(self):
+    self.__custom_headers = {}
+
     if self.__get_custom_headers_func:
       cookie_header, has_auth_cookie = self.getHttpCookieHeaderForRequest()
       self.__custom_headers = \
           self.__get_custom_headers_func(cookie_header, has_auth_cookie)
 
+    for f in self.__add_custom_headers_funcs:
+      headers = f()
+      if headers is not None:
+        for key in headers:
+          assert key[0:2].lower() == "x-", \
+            "header '{0}' is not valid, all custom headers must start with "\
+            "'X-' or 'x-'".format(key)
+          assert key not in self.__custom_headers, \
+            "header '{0}' already exists in custom headers dictionary".format(key)
+          self.__custom_headers[key] = headers[key]
+
   # Return first value as a cookie list for Cookie header. It's a list of name-value
   # pairs in the form of <cookie-name>=<cookie-value>. Pairs in the list are separated by
   # a semicolon and a space ('; ').
