@@ -46,6 +46,7 @@ from TCLIService.TCLIService import (TExecuteStatementReq, TOpenSessionReq,
     TOperationState, TFetchResultsReq, TFetchOrientation, TGetLogReq,
     TGetResultSetMetadataReq, TTypeId, TCancelOperationReq, TCloseOperationReq)
 from ImpalaHttpClient import ImpalaHttpClient
+from kerberos_util import get_kerb_host_from_kerberos_host_fqdn
 from thrift.protocol import TBinaryProtocol
 from thrift_sasl import TSaslClientTransport
 from thrift.transport.TSocket import TSocket
@@ -310,9 +311,10 @@ class ImpalaClient(object):
   # differentiate between DML and non-DML.
   def close_dml(self, last_query_handle):
     """Fetches the results of a DML query. Returns a tuple containing the
-       number of rows modified and the number of row errors, in that order. If the DML
-       operation doesn't return 'num_row_errors', then the second element in the tuple
-       is None. Returns None if the query was not closed successfully. Not idempotent."""
+       number of rows modified, the number of rows deleted, and the number of row errors,
+       in that order. If the DML operation doesn't return 'rows_deleted' or
+       'num_row_errors', then the respective element in the tuple is None.
+       Returns None if the query was not closed successfully. Not idempotent."""
     raise NotImplementedError()
 
   def close_query(self, last_query_handle):
@@ -432,7 +434,7 @@ class ImpalaClient(object):
     elif self.use_kerberos or self.kerberos_host_fqdn:
       # Set the Kerberos service
       if self.kerberos_host_fqdn is not None:
-        kerb_host = self.kerberos_host_fqdn.split(':')[0].encode('ascii', 'ignore')
+        kerb_host = get_kerb_host_from_kerberos_host_fqdn(self.kerberos_host_fqdn)
       else:
         kerb_host = self.impalad_host
       kerb_service = "{0}@{1}".format(self.kerberos_service_name, kerb_host)
@@ -463,14 +465,13 @@ class ImpalaClient(object):
       # Systems. Only attempt to import TSSLSocket if the user wants an SSL connection.
       from TSSLSocketWithWildcardSAN import TSSLSocketWithWildcardSAN
 
-    # sasl does not accept unicode strings, explicitly encode the string into ascii.
     # The kerberos_host_fqdn option exposes the SASL client's hostname attribute to
     # the user. impala-shell checks to ensure this host matches the host in the kerberos
     # principal. So when a load balancer is configured to be used, its hostname is
     # expected by impala-shell. Setting this option to the load balancer hostname allows
     # impala-shell to connect directly to an impalad.
     if self.kerberos_host_fqdn is not None:
-      sasl_host = self.kerberos_host_fqdn.split(':')[0].encode('ascii', 'ignore')
+      sasl_host = get_kerb_host_from_kerberos_host_fqdn(self.kerberos_host_fqdn)
     else:
       sasl_host = self.impalad_host
 
@@ -661,6 +662,13 @@ class ImpalaClient(object):
     # to add arbitrary http headers.
     return None
 
+  def _process_dml_result(self, dml_result):
+    num_rows = sum([int(k) for k in dml_result.rows_modified.values()])
+    num_deleted_rows = None
+    if dml_result.rows_deleted:
+      num_deleted_rows = sum([int(k) for k in dml_result.rows_deleted.values()])
+    return (num_rows, num_deleted_rows, dml_result.num_row_errors)
+
 
 class ImpalaHS2Client(ImpalaClient):
   """Impala client. Uses the HS2 protocol plus Impala-specific extensions."""
@@ -972,10 +980,8 @@ class ImpalaHS2Client(ImpalaClient):
       self._check_hs2_rpc_status(resp.status)
       if not resp.dml_result:
         raise RPCException("Impala DML operation did not return DML statistics.")
-
-      num_rows = sum([int(k) for k in resp.dml_result.rows_modified.values()])
       last_query_handle.is_closed = True
-      return (num_rows, resp.dml_result.num_row_errors)
+      return self._process_dml_result(resp.dml_result)
     finally:
       self._clear_current_query_handle()
 
@@ -1293,7 +1299,7 @@ class StrictHS2Client(ImpalaHS2Client):
 
   def close_dml(self, last_query_handle):
     self.close_query(last_query_handle)
-    return (None, None)
+    return (None, None, None)
 
   def close_query(self, last_query_handle):
     # Set a member in the handle to make sure that it is idempotent
@@ -1445,13 +1451,12 @@ class ImpalaBeeswaxClient(ImpalaClient):
         return
 
   def close_dml(self, last_query_handle):
-    insert_result, rpc_status = self._do_beeswax_rpc(
+    dml_result, rpc_status = self._do_beeswax_rpc(
         lambda: self.imp_service.CloseInsert(last_query_handle))
     if rpc_status != RpcStatus.OK:
        raise RPCException()
-    num_rows = sum([int(k) for k in insert_result.rows_modified.values()])
     last_query_handle.is_closed = True
-    return (num_rows, insert_result.num_row_errors)
+    return self._process_dml_result(dml_result)
 
   def close_query(self, last_query_handle):
     # Set a member in the handle to make sure that it is idempotent
