@@ -1282,54 +1282,92 @@ class ImpalaShell(cmd.Cmd, object):
       if not summary:
         return
 
-      if summary.is_queued:
-        queued_msg = "Query queued. Latest queuing reason: %s\n" % summary.queued_reason
-        self.progress_stream.write(queued_msg)
-        self.last_summary = time.time()
-        return
-
-      data = ""
-      if summary.error_logs:
-        for error_line in summary.error_logs:
-          data += error_line + "\n"
-          if self.webserver_address:
-            query_id_search = re.search("Retrying query using query id: (.*)",
-                                        error_line)
-            if query_id_search and len(query_id_search.groups()) == 1:
-              retried_query_id = query_id_search.group(1)
-              data += "Retried query link: %s\n"\
-                      % self.imp_client.get_query_link(retried_query_id)
-
-      if summary.progress:
-        progress = summary.progress
-
-        # If the data is not complete return and wait for a good result.
-        if not progress.total_scan_ranges and not progress.num_completed_scan_ranges:
-          self.last_summary = time.time()
-          return
-
-        if self.live_progress and progress.total_scan_ranges > 0:
-          val = ((summary.progress.num_completed_scan_ranges * 100) //
-                 summary.progress.total_scan_ranges)
-          fragment_text = "[%s%s] %s%%\n" % ("#" * val, " " * (100 - val), val)
-          data += fragment_text
-
-        if self.live_summary:
-          table = self._default_summary_table()
-          output = []
-          self.imp_client.build_summary_table(summary, 0, False, 0, False, output)
-          formatter = PrettyOutputFormatter(table)
-          data += formatter.format(output) + "\n"
-
-      self.progress_stream.write(data)
+      summary_str = self._format_periodic_summary(summary)
+      if summary_str:
+        self.progress_stream.write(summary_str)
       self.last_summary = time.time()
 
+  def _format_periodic_summary(self, summary):
+    if summary.is_queued:
+      return "Query queued. Latest queuing reason: %s\n" % summary.queued_reason
+
+    data = ""
+    if summary.error_logs:
+      for error_line in summary.error_logs:
+        data += error_line + "\n"
+        if self.webserver_address:
+          query_id_search = re.search("Retrying query using query id: (.*)",
+                                      error_line)
+          if query_id_search and len(query_id_search.groups()) == 1:
+            retried_query_id = query_id_search.group(1)
+            data += "Retried query link: %s\n"\
+                    % self.imp_client.get_query_link(retried_query_id)
+
+    if summary.progress:
+      progress = summary.progress
+
+      has_scan_progress = progress.num_completed_scan_ranges is not None \
+          and progress.total_scan_ranges is not None \
+          and progress.total_scan_ranges > 0
+
+      if self.live_progress and has_scan_progress:
+        val = ((summary.progress.num_completed_scan_ranges * 100)
+                // summary.progress.total_scan_ranges)
+        scan_progress_text =\
+            " Scan Progress:[%s%s] %s%%\n" % ("#" * val, " " * (100 - val), val)
+        data += scan_progress_text
+
+      has_query_progress = progress.num_completed_fragment_instances is not None \
+          and progress.total_fragment_instances is not None \
+          and progress.total_fragment_instances > 0
+
+      if self.live_progress and has_query_progress:
+        val = ((progress.num_completed_fragment_instances * 100)
+                // progress.total_fragment_instances)
+        query_progress_text =\
+            "Query Progress:[%s%s] %s%%\n" % ("#" * val, " " * (100 - val), val)
+        data += query_progress_text
+
+      if self.live_summary:
+        table = self._default_summary_table()
+        output = []
+        self.imp_client.build_summary_table(summary, 0, False, 0, False, output)
+        formatter = PrettyOutputFormatter(table)
+        data += formatter.format(output) + "\n"
+
+    return data
+
   def _default_summary_table(self):
     return self.construct_table_with_header(["Operator", "#Hosts", "#Inst",
                                              "Avg Time", "Max Time", "#Rows",
                                              "Est. #Rows", "Peak Mem",
                                              "Est. Peak Mem", "Detail"])
 
+  def _format_num_rows_report(self, time_elapsed, num_fetched_rows=None, dml_result=None):
+    num_rows = None
+    verb = None
+    error_report = ""
+    if dml_result is not None:
+      (num_modified_rows, num_deleted_rows, num_row_errors) = dml_result
+      if num_modified_rows == 0 and num_deleted_rows is not None and num_deleted_rows > 0:
+        verb = "Deleted"
+        num_rows = num_deleted_rows
+      elif num_modified_rows is not None:
+        verb = "Modified"
+        num_rows = num_modified_rows
+      # Add the number of row errors if this DML and the operation supports it.
+      # num_row_errors is None if the DML operation doesn't return it.
+      if num_row_errors is not None:
+        error_report = ", %d row error(s)" % (num_row_errors)
+    elif num_fetched_rows is not None:
+      verb = "Fetched"
+      num_rows = num_fetched_rows
+
+    if verb is not None:
+      return "%s %d row(s)%s in %2.2fs" % (verb, num_rows, error_report, time_elapsed)
+    else:
+      return "Time elapsed: %2.2fs" % time_elapsed
+
   def _execute_stmt(self, query_str, is_dml=False, print_web_link=False):
     """Executes 'query_str' with options self.set_query_options on the Impala server.
     The query is run to completion and close with any results, warnings, errors or
@@ -1357,7 +1395,7 @@ class ImpalaShell(cmd.Cmd, object):
       self.last_summary = time.time()
       if print_web_link:
         self._print_if_verbose(
-            "Query progress can be monitored at: %s" % self.imp_client.get_query_link(
+            "Query state can be monitored at: %s" % self.imp_client.get_query_link(
              self.imp_client.get_query_id_str(self.last_query_handle)))
 
       wait_to_finish = self.imp_client.wait_to_finish(self.last_query_handle,
@@ -1368,7 +1406,7 @@ class ImpalaShell(cmd.Cmd, object):
       if is_dml:
         # retrieve the error log
         warning_log = self.imp_client.get_warning_log(self.last_query_handle)
-        (num_rows, num_row_errors) = self.imp_client.close_dml(self.last_query_handle)
+        dml_result = self.imp_client.close_dml(self.last_query_handle)
       else:
         # impalad does not support the fetching of metadata for certain types of queries.
         if not self.imp_client.expect_result_metadata(query_str, self.last_query_handle):
@@ -1400,23 +1438,14 @@ class ImpalaShell(cmd.Cmd, object):
 
       if warning_log:
         self._print_if_verbose(warning_log)
-      # print 'Modified' when is_dml is true (i.e. 1), or 'Fetched' otherwise.
-      verb = ["Fetched", "Modified"][is_dml]
+
       time_elapsed = end_time - start_time
-
-      # Add the number of row errors if this DML and the operation supports it.
-      # num_row_errors is None if the DML operation doesn't return it.
-      if is_dml and num_row_errors is not None:
-        error_report = ", %d row error(s)" % (num_row_errors)
+      row_report = ""
+      if is_dml:
+        row_report = self._format_num_rows_report(time_elapsed, dml_result=dml_result)
       else:
-        error_report = ""
-
-      if num_rows is not None:
-        self._print_if_verbose("%s %d row(s)%s in %2.2fs" %
-            (verb, num_rows, error_report, time_elapsed))
-      else:
-        self._print_if_verbose("Time elapsed: %2.2fs" %
-            (time_elapsed))
+        row_report = self._format_num_rows_report(time_elapsed, num_fetched_rows=num_rows)
+      self._print_if_verbose(row_report)
 
       if not is_dml:
         self.imp_client.close_query(self.last_query_handle)
