@@ -47,6 +47,8 @@ impala_shell_defaults = {
             'query': None,
             'query_file': None,
             'show_profiles': False,
+            'rpc_stdout': False,
+            'rpc_file': None,
             'ssl': False,
             'use_kerberos': False,
             'use_ldap': False,
@@ -58,5 +60,6 @@ impala_shell_defaults = {
             'http_socket_timeout_s': None,
             'global_config_default_path': '/etc/impalarc',
             'strict_hs2_protocol': False,
-            'fp_format_specification': None
+            'hs2_fp_format': None,
+            'no_http_tracing': False
     }
