@@ -36,6 +36,8 @@ ${COMPUTE_STATS_SCRIPT} --db_names=functional\
     alltypessmall,alltypestiny,jointbl,dimtbl,stringpartitionkey,nulltable,nullrows,
     date_tbl,chars_medium,part_strings_with_quotes,alltypes_date_partition,
     alltypes_date_partition_2,mv1_alltypes_jointbl,binary_tbl,binary_tbl_big"
+${COMPUTE_STATS_SCRIPT} --db_names=functional_parquet \
+    --table_names="unique_with_nulls"
 
 # We cannot load HBase on s3 and isilon yet.
 if [ "${TARGET_FILESYSTEM}" = "hdfs" ]; then
@@ -46,3 +48,8 @@ ${COMPUTE_STATS_SCRIPT} --db_names=tpch,tpch_parquet,tpch_orc_def \
     --table_names=customer,lineitem,nation,orders,part,partsupp,region,supplier
 ${COMPUTE_STATS_SCRIPT} --db_names=tpch_nested_parquet,tpcds,tpcds_parquet
 ${COMPUTE_STATS_SCRIPT} --db_names=functional_kudu,tpch_kudu
+
+# Compute tables of tpcds_partitioned_parquet_snap serially
+# due to large number of partitions in some of the fact tables.
+${COMPUTE_STATS_SCRIPT} --db_names=tpcds_partitioned_parquet_snap \
+    --parallelism=1
