@@ -165,6 +165,9 @@ function start-impala {
   : ${START_CLUSTER_ARGS=""}
   # Use a fast statestore update so that DDL operations run faster.
   START_CLUSTER_ARGS_INT="--state_store_args=--statestore_update_frequency_ms=50"
+  # Disable strict datafile location checks for Iceberg tables
+  DATAFILE_LOCATION_CHECK="-iceberg_allow_datafiles_in_table_location_only=false"
+  START_CLUSTER_ARGS_INT+=("--catalogd_args=$DATAFILE_LOCATION_CHECK")
   if [[ "${TARGET_FILESYSTEM}" == "local" ]]; then
     START_CLUSTER_ARGS_INT+=("--impalad_args=--abort_on_config_error=false -s 1")
   else
@@ -234,12 +237,12 @@ function load-data {
   TABLE_FORMATS=${3:-}
   FORCE_LOAD=${4:-}
 
-  MSG="Loading workload '$WORKLOAD'"
+  LOAD_MSG="Loading workload '$WORKLOAD'"
   ARGS=("--workloads $WORKLOAD")
-  MSG+=" using exploration strategy '$EXPLORATION_STRATEGY'"
+  LOAD_MSG+=" using exploration strategy '$EXPLORATION_STRATEGY'"
   ARGS+=("-e $EXPLORATION_STRATEGY")
   if [ $TABLE_FORMATS ]; then
-    MSG+=" in table formats '$TABLE_FORMATS'"
+    LOAD_MSG+=" in table formats '$TABLE_FORMATS'"
     ARGS+=("--table_formats $TABLE_FORMATS")
   fi
   if [ $LOAD_DATA_ARGS ]; then
@@ -282,7 +285,7 @@ function load-data {
   fi
 
   LOG_FILE=${IMPALA_DATA_LOADING_LOGS_DIR}/${LOG_BASENAME}
-  echo "$MSG. Logging to ${LOG_FILE}"
+  echo "$LOAD_MSG. Logging to ${LOG_FILE}"
   # Use unbuffered logging by executing with -u
   if ! impala-python -u ${IMPALA_HOME}/bin/load-data.py ${ARGS[@]} &> ${LOG_FILE}; then
     echo Error loading data. The end of the log file is:
@@ -291,6 +294,11 @@ function load-data {
   fi
 }
 
+function load-tpcds-data {
+  load-data "tpcds" "core"
+  load-data "tpcds_partitioned" "core"
+}
+
 function cache-test-tables {
   echo CACHING  tpch.nation AND functional.alltypestiny
   # uncaching the tables first makes this operation idempotent.
@@ -478,10 +486,17 @@ function custom-post-load-steps {
 
 function copy-and-load-ext-data-source {
   # Copy the test data source library into HDFS
-  ${IMPALA_HOME}/testdata/bin/copy-data-sources.sh
+  ${IMPALA_HOME}/testdata/bin/copy-ext-data-sources.sh
+  # Load the underlying data of the data source
+  ${IMPALA_HOME}/testdata/bin/load-ext-data-sources.sh
   # Create data sources table.
   ${IMPALA_HOME}/bin/impala-shell.sh -i ${IMPALAD} -f\
-    ${IMPALA_HOME}/testdata/bin/create-data-source-table.sql
+      ${IMPALA_HOME}/testdata/bin/create-ext-data-source-table.sql
+  # Create external JDBC tables for TPCH/TPCDS queries.
+  ${IMPALA_HOME}/testdata/bin/create-tpc-jdbc-tables.py --jdbc_db_name=tpch_jdbc \
+      --workload=tpch --database_type=impala --clean
+  ${IMPALA_HOME}/testdata/bin/create-tpc-jdbc-tables.py --jdbc_db_name=tpcds_jdbc \
+      --workload=tpcds --database_type=impala --clean
 }
 
 function check-hdfs-health {
@@ -536,6 +551,16 @@ function warm-up-hive {
   $HIVE_CMD -e "insert overwrite table hive_warm_up_tbl values (1);"
 }
 
+# IMPALA-13015, IMPALA-13026: This should be called during serial phase of data load.
+function create-hadoop-credential {
+  rm -f ${IMPALA_HOME}/testdata/jceks/test.jceks
+  hadoop credential create "openai-api-key-secret" -value "secret" -provider \
+    "localjceks://file/${IMPALA_HOME}/testdata/jceks/test.jceks"
+}
+
+run-step "Creating hadoop credential" create-hadoop-credential.log \
+    create-hadoop-credential
+
 # For kerberized clusters, use kerberos
 if ${CLUSTER_DIR}/admin is_kerberized; then
   LOAD_DATA_ARGS="${LOAD_DATA_ARGS} --use_kerberos --principal=${MINIKDC_PRINC_HIVE}"
@@ -568,7 +593,7 @@ if [ $SKIP_METADATA_LOAD -eq 0 ]; then
   run-step-backgroundable "Loading functional-query data" load-functional-query.log \
       load-data "functional-query" "exhaustive"
   run-step-backgroundable "Loading TPC-H data" load-tpch.log load-data "tpch" "core"
-  run-step-backgroundable "Loading TPC-DS data" load-tpcds.log load-data "tpcds" "core"
+  run-step-backgroundable "Loading TPC-DS data" load-tpcds.log load-tpcds-data
   run-step-wait-all
   # Load tpch nested data.
   # TODO: Hacky and introduces more complexity into the system, but it is expedient.
@@ -612,11 +637,6 @@ if [ "${TARGET_FILESYSTEM}" = "hdfs" ]; then
   # Caching tables in s3 returns an IllegalArgumentException, see IMPALA-1714
   run-step "Caching test tables" cache-test-tables.log cache-test-tables
 
-  # TODO: Modify the .sql file that creates the table to take an alternative location into
-  # account.
-  run-step "Loading external data sources" load-ext-data-source.log \
-      copy-and-load-ext-data-source
-
   run-step "Creating internal HBase table" create-internal-hbase-table.log \
       create-internal-hbase-table
 
@@ -626,6 +646,14 @@ if [ "${TARGET_FILESYSTEM}" = "hdfs" ]; then
   run-step "Logging created files" created-files.log hdfs dfs -ls -R /test-warehouse
 fi
 
+if [[ "${TARGET_FILESYSTEM}" = "hdfs" || "${TARGET_FILESYSTEM}" = "ozone" || \
+      "${TARGET_FILESYSTEM}" = "s3" ]]; then
+  # TODO: Modify the .sql file that creates the table to take an alternative location into
+  # account.
+  run-step "Loading external data sources" load-ext-data-source.log \
+      copy-and-load-ext-data-source
+fi
+
 # TODO: Investigate why all stats are not preserved. Theoretically, we only need to
 # recompute stats for HBase.
 run-step "Computing table stats" compute-table-stats.log \
