@@ -25,8 +25,8 @@ import os
 import pipes
 import psutil
 import socket
-import sys
 import time
+import requests
 from getpass import getuser
 from random import choice
 from signal import SIGKILL
@@ -39,7 +39,7 @@ from tests.common.impala_service import (
     CatalogdService,
     ImpaladService,
     StateStoredService)
-from tests.util.shell_util import exec_process, exec_process_async
+from tests.util.shell_util import exec_process
 
 LOG = logging.getLogger('impala_cluster')
 LOG.setLevel(level=logging.DEBUG)
@@ -55,6 +55,9 @@ DEFAULT_KRPC_PORT = 27000
 DEFAULT_CATALOG_SERVICE_PORT = 26000
 DEFAULT_STATE_STORE_SUBSCRIBER_PORT = 23000
 DEFAULT_CATALOGD_STATE_STORE_SUBSCRIBER_PORT = 23020
+DEFAULT_STATESTORE_SERVICE_PORT = 24000
+DEFAULT_STATESTORE_HA_SERVICE_PORT = 24020
+DEFAULT_PEER_STATESTORE_HA_SERVICE_PORT = 24021
 DEFAULT_IMPALAD_WEBSERVER_PORT = 25000
 DEFAULT_STATESTORED_WEBSERVER_PORT = 25010
 DEFAULT_CATALOGD_WEBSERVER_PORT = 25020
@@ -67,6 +70,20 @@ DEFAULT_CATALOGD_JVM_DEBUG_PORT = 30030
 # flakiness.
 CLUSTER_WAIT_TIMEOUT_IN_SECONDS = 240
 
+# Url format used to set JVM log level.
+SET_JAVA_LOGLEVEL_URL = "http://{0}:{1}/set_java_loglevel"
+
+
+def post_data(url, data):
+  """Helper method to post data to a url."""
+  response = requests.head(url)
+  assert response.status_code == requests.codes.ok, "URL: {0} Resp:{1}".format(
+    url, response.text)
+  response = requests.post(url, data=data)
+  assert response.status_code == requests.codes.ok, "URL: {0} Resp:{1}".format(
+    url, response.text)
+  return response
+
 
 # Represents a set of Impala processes.
 # Handles two cases:
@@ -117,6 +134,13 @@ class ImpalaCluster(object):
     # If no statestored process exists, return None.
     return self.__statestoreds[0] if len(self.__statestoreds) > 0 else None
 
+  def statestoreds(self):
+    """Returns a list of the known statestored processes"""
+    return self.__statestoreds
+
+  def get_first_statestored(self):
+    return self.statestoreds[0]
+
   @property
   def impalads(self):
     """Returns a list of the known impalad processes"""
@@ -198,13 +222,12 @@ class ImpalaCluster(object):
       assert self.statestored is not None
       assert self.catalogd is not None
 
-
     for impalad in self.impalads:
       impalad.service.wait_for_num_known_live_backends(expected_num_ready_impalads,
           timeout=CLUSTER_WAIT_TIMEOUT_IN_SECONDS, interval=2,
           early_abort_fn=check_processes_still_running)
-      if (impalad._get_arg_value("is_coordinator", default="true") == "true" and
-         impalad._get_arg_value("stress_catalog_init_delay_ms", default=0) == 0):
+      if (impalad._get_arg_value("is_coordinator", default="true") == "true"
+         and impalad._get_arg_value("stress_catalog_init_delay_ms", default=0) == 0):
         impalad.wait_for_catalog()
 
   def wait_for_num_impalads(self, num_impalads, retries=10):
@@ -238,7 +261,7 @@ class ImpalaCluster(object):
     environment this would need to enumerate each machine in the cluster.
     """
     impalads = list()
-    statestored = list()
+    statestoreds = list()
     catalogds = list()
     admissiond = None
     daemons = ['impalad', 'catalogd', 'statestored', 'admissiond']
@@ -259,15 +282,16 @@ class ImpalaCluster(object):
       if binary == 'impalad':
         impalads.append(ImpaladProcess(cmdline))
       elif binary == 'statestored':
-        statestored.append(StateStoreProcess(cmdline))
+        statestoreds.append(StateStoreProcess(cmdline))
       elif binary == 'catalogd':
         catalogds.append(CatalogdProcess(cmdline))
       elif binary == 'admissiond':
         admissiond = AdmissiondProcess(cmdline)
 
     self.__sort_impalads(impalads)
+    self.__sort_statestoreds(statestoreds)
     self.__sort_catalogds(catalogds)
-    return impalads, statestored, catalogds, admissiond
+    return impalads, statestoreds, catalogds, admissiond
 
   def __find_docker_containers(self):
     """
@@ -308,6 +332,7 @@ class ImpalaCluster(object):
         admissiond = AdmissiondProcess(args, container_id=container_id,
                                        port_map=port_map)
     self.__sort_impalads(impalads)
+    self.__sort_statestoreds(statestoreds)
     self.__sort_catalogds(catalogds)
     return impalads, statestoreds, catalogds, admissiond
 
@@ -318,6 +343,13 @@ class ImpalaCluster(object):
     the containerised cluster."""
     impalads.sort(key=lambda i: i.service.hs2_port)
 
+  def __sort_statestoreds(self, statestoreds):
+    """Does an in-place sort of a list of StateStoredService objects into a canonical
+    order. We order them by their service port, so that get_first_statestored() always
+    returns the first one. We need to use a port that is exposed and mapped to a host
+    port for the containerised cluster."""
+    statestoreds.sort(key=lambda i: i.service.service_port)
+
   def __sort_catalogds(self, catalogds):
     """Does an in-place sort of a list of CatalogdProcess objects into a canonical order.
     We order them by their service port, so that get_first_catalogd() always returns the
@@ -369,7 +401,7 @@ class Process(object):
     """Gets the PIDs of the process. In some circumstances, a process can run multiple
     times, e.g. when it forks in the Breakpad crash handler. Returns an empty list if no
     PIDs can be determined."""
-    pids = self.__get_pids()
+    pids = [proc['pid'] for proc in self.__get_procs()]
     if pids:
       LOG.info("Found PIDs %s for %s" % (", ".join(map(str, pids)), " ".join(self.cmd)))
     else:
@@ -377,37 +409,53 @@ class Process(object):
                " ".join(self.cmd))
     return pids
 
-  def __get_pid(self):
-    pids = self.__get_pids()
-    assert len(pids) < 2, "Expected single pid but found %s" % ", ".join(map(str, pids))
-    return len(pids) == 1 and pids[0] or None
+  def __procs_str(self, procs):
+    return "\n".join([str(proc) for proc in procs])
 
-  def __get_pids(self):
+  def __get_pid(self):
+    procs = self.__get_procs()
+    # Return early for containerized environments
+    if len(procs) == 1:
+      return procs[0]['pid']
+
+    result = None
+    # In some circumstances - notably ubsan tests - child processes can be slow to exit.
+    # Only return the original process, i.e. one who's parent has a different cmd.
+    pids = [proc['pid'] for proc in procs]
+    for process in procs:
+      if process['ppid'] not in pids:
+        assert result is None,\
+            "Multiple non-child processes:\n%s" % self.__procs_str(procs)
+        result = process['pid']
+      else:
+        LOG.info("Child process active:\n%s" % self.__procs_str(procs))
+
+    return result
+
+  def __get_procs(self):
+    """
+    Returns a list of dicts containing {pid, ppid, cmdline} for related processes.
+    """
     if self.container_id is not None:
       container_info = get_container_info(self.container_id)
       if container_info["State"]["Status"] != "running":
         return []
-      return [container_info["State"]["Pid"]]
+      return [{'pid': container_info["State"]["Pid"], 'ppid': 0, 'cmdline': self.cmd}]
 
     # In non-containerised case, search for process based on matching command lines.
-    pids = []
-    for pid in psutil.pids():
-      try:
-        process = psutil.Process(pid)
-        if set(self.cmd) == set(process.cmdline()):
-          pids.append(pid)
-      except psutil.NoSuchProcess:
-        # A process from psutil.pids() no longer exists, continue. We don't log this
-        # error since it can refer to arbitrary processes outside of our testing code.
-        pass
-    return pids
+    procs = []
+    for process in psutil.process_iter(['pid', 'ppid', 'cmdline']):
+      # Use info because it won't throw NoSuchProcess exceptions.
+      if set(self.cmd) == set(process.info['cmdline']):
+        procs.append(process.info)
+    return procs
 
   def kill(self, signal=SIGKILL):
     """
     Kills the given processes.
     """
     if self.container_id is None:
-      pid = self.__get_pid()
+      pid = self.get_pid()
       assert pid is not None, "No processes for %s" % self
       LOG.info('Killing %s with signal %s' % (self, signal))
       exec_process("kill -%d %d" % (signal, pid))
@@ -420,7 +468,7 @@ class Process(object):
     if self.container_id is None:
       binary = os.path.basename(self.cmd[0])
       restart_args = self.cmd[1:]
-      LOG.info("Starting {0} with arguments".format(binary, restart_args))
+      LOG.info("Starting {0} with arguments {1}".format(binary, restart_args))
       run_daemon(binary, restart_args)
     else:
       LOG.info("Starting container: {0}".format(self.container_id))
@@ -469,6 +517,11 @@ class BaseImpalaProcess(Process):
     """Return the port for the webserver of this process."""
     return int(self._get_port('webserver_port', self._get_default_webserver_port()))
 
+  def set_jvm_log_level(self, class_name, level="info"):
+    """Helper method to set JVM log level for certain class name.
+    Some daemon might not have JVM in it."""
+    raise NotImplementedError()
+
   def _get_default_webserver_port(self):
     """Different daemons have different defaults. Subclasses must override."""
     raise NotImplementedError()
@@ -544,8 +597,8 @@ class ImpaladProcess(BaseImpalaProcess):
     hs2_port_is_open = False
     num_dbs = 0
     num_tbls = 0
-    while ((time.time() - start_time < CLUSTER_WAIT_TIMEOUT_IN_SECONDS) and
-        not (beeswax_port_is_open and hs2_port_is_open)):
+    while ((time.time() - start_time < CLUSTER_WAIT_TIMEOUT_IN_SECONDS)
+        and not (beeswax_port_is_open and hs2_port_is_open)):
       try:
         num_dbs, num_tbls = self.service.get_metric_values(
             ["catalog.num-databases", "catalog.num-tables"])
@@ -563,17 +616,39 @@ class ImpaladProcess(BaseImpalaProcess):
           "Unable to open client ports within {num_seconds} seconds.".format(
               num_seconds=CLUSTER_WAIT_TIMEOUT_IN_SECONDS))
 
+  def set_jvm_log_level(self, class_name, level):
+    """Helper method to set JVM log level for certain class name."""
+    url = SET_JAVA_LOGLEVEL_URL.format(
+      self.webserver_interface, self.get_webserver_port())
+    return post_data(url, {"class": class_name, "level": level})
+
 
 # Represents a statestored process
 class StateStoreProcess(BaseImpalaProcess):
   def __init__(self, cmd, container_id=None, port_map=None):
     super(StateStoreProcess, self).__init__(cmd, container_id, port_map)
     self.service = StateStoredService(self.hostname, self.webserver_interface,
-        self.get_webserver_port(), self._get_webserver_certificate_file())
+        self.get_webserver_port(), self._get_webserver_certificate_file(),
+        self.__get_port())
 
   def _get_default_webserver_port(self):
     return DEFAULT_STATESTORED_WEBSERVER_PORT
 
+  def __get_port(self):
+    return int(self._get_port('state_store_port', DEFAULT_STATESTORE_SERVICE_PORT))
+
+  def start(self, wait_until_ready=True, additional_args=None):
+    """Starts statestored and waits until the service is started and ready to accept
+    connections."""
+    restart_args = self.cmd[1:]
+    if additional_args:
+      restart_args = restart_args + [additional_args]
+    LOG.info("Starting Statestored process: {0}".format(restart_args))
+    run_daemon("statestored", restart_args)
+    if wait_until_ready:
+      self.service.wait_for_metric_value('statestore.service-started',
+                                         expected_value=1, timeout=30)
+
 
 # Represents a catalogd process
 class CatalogdProcess(BaseImpalaProcess):
@@ -600,6 +675,12 @@ class CatalogdProcess(BaseImpalaProcess):
       self.service.wait_for_metric_value('statestore-subscriber.connected',
                                          expected_value=1, timeout=30)
 
+  def set_jvm_log_level(self, class_name, level):
+    """Helper method to set JVM log level for certain class name."""
+    url = SET_JAVA_LOGLEVEL_URL.format(
+      self.webserver_interface, self.get_webserver_port())
+    return post_data(url, {"class": class_name, "level": level})
+
 
 # Represents an admission control process.
 class AdmissiondProcess(BaseImpalaProcess):
@@ -611,6 +692,7 @@ class AdmissiondProcess(BaseImpalaProcess):
   def _get_default_webserver_port(self):
     return DEFAULT_ADMISSIOND_WEBSERVER_PORT
 
+
 def find_user_processes(binaries):
   """Returns an iterator over all processes owned by the current user with a matching
   binary name from the provided list. Return a iterable of tuples, with each tuple
