@@ -77,7 +77,8 @@ from tests.util.filesystem_utils import (
     S3GUARD_ENABLED,
     ADLS_STORE_NAME,
     FILESYSTEM_PREFIX,
-    FILESYSTEM_NAME)
+    FILESYSTEM_NAME,
+    FILESYSTEM_URI_SCHEME)
 
 from tests.util.hdfs_util import (
   HdfsConfig,
@@ -495,6 +496,7 @@ class ImpalaTestSuite(BaseTestSuite):
     repl = dict(('$' + k, globs[k]) for k in [
         "FILESYSTEM_PREFIX",
         "FILESYSTEM_NAME",
+        "FILESYSTEM_URI_SCHEME",
         "GROUP_NAME",
         "NAMENODE",
         "IMPALA_HOME",
@@ -688,6 +690,9 @@ class ImpalaTestSuite(BaseTestSuite):
         if needed_hive_major_version != HIVE_MAJOR_VERSION:
           continue
 
+      if 'IS_HDFS_ONLY' in test_section and not IS_HDFS:
+        continue
+
       if 'SHELL' in test_section:
         assert len(test_section) == 1, \
             "SHELL test sections can't contain other sections"
@@ -771,6 +776,10 @@ class ImpalaTestSuite(BaseTestSuite):
         rt_profile_info = 'RUNTIME_PROFILE'
 
       if rt_profile_info is not None:
+        if test_file_vars:
+          # only do test_file_vars replacement if it exist.
+          test_section[rt_profile_info] = self.__do_replacements(
+              test_section[rt_profile_info], extra=test_file_vars)
         rt_profile = verify_runtime_profile(test_section[rt_profile_info],
                                result.runtime_profile,
                                update_section=pytest.config.option.update_results)
@@ -941,6 +950,16 @@ class ImpalaTestSuite(BaseTestSuite):
     assert len(result.data) <= 1, 'Multiple values returned from scalar'
     return result.data[0] if len(result.data) == 1 else None
 
+  @classmethod
+  @execute_wrapper
+  def execute_scalar_expect_success(cls, impalad_client, query, query_options=None,
+      user=None):
+    """Executes a query and asserts if the query fails"""
+    result = cls.__execute_query(impalad_client, query, query_options, user)
+    assert result.success
+    assert len(result.data) <= 1, 'Multiple values returned from scalar'
+    return result.data[0] if len(result.data) == 1 else None
+
   def exec_and_compare_hive_and_impala_hs2(self, stmt, compare = lambda x, y: x == y):
     """Compare Hive and Impala results when executing the same statment over HS2"""
     # execute_using_jdbc expects a Query object. Convert the query string into a Query
@@ -1016,11 +1035,25 @@ class ImpalaTestSuite(BaseTestSuite):
     # This should never happen.
     assert 0, 'Unable to get location for table: ' + table_name
 
+  def run_impala_stmt_in_beeline(self, stmt, username=None, default_db='default'):
+    """ Run a statement in impala by Beeline. """
+    url = 'jdbc:hive2://localhost:' + pytest.config.option.impalad_hs2_port + '/'\
+      + default_db + ";auth=noSasl"
+    return self.run_stmt_in_beeline(url, username, stmt)
+
   # TODO(todd) make this use Thrift to connect to HS2 instead of shelling
   # out to beeline for better performance
-  def run_stmt_in_hive(self, stmt, username=None):
+  @classmethod
+  def run_stmt_in_hive(cls, stmt, username=None):
+    """Run a statement in Hive by Beeline."""
+    LOG.info("-- executing in HiveServer2\n\n" + stmt + "\n")
+    url = 'jdbc:hive2://' + pytest.config.option.hive_server2
+    return cls.run_stmt_in_beeline(url, username, stmt)
+
+  @classmethod
+  def run_stmt_in_beeline(cls, url, username, stmt):
     """
-    Run a statement in Hive, returning stdout if successful and throwing
+    Run a statement by Beeline, returning stdout if successful and throwing
     RuntimeError(stderr) if not.
     """
     # Remove HADOOP_CLASSPATH from environment. Beeline doesn't need it,
@@ -1033,8 +1066,10 @@ class ImpalaTestSuite(BaseTestSuite):
     env.pop("HADOOP_CLASSPATH", None)
     call = subprocess.Popen(
         ['beeline',
+         # TODO IMPALA-2228: Prevents query output from being confused with log.
+         '--silent=true',
          '--outputformat=csv2',
-         '-u', 'jdbc:hive2://' + pytest.config.option.hive_server2,
+         '-u', url,
          '-n', username or getuser(),
          '-e', stmt],
         stdout=subprocess.PIPE,
@@ -1166,6 +1201,11 @@ class ImpalaTestSuite(BaseTestSuite):
     else:
       return "UNIDENTIFIED"
 
+  @classmethod
+  def has_value(cls, value, lines):
+    """Check if lines contain value."""
+    return any([line.find(value) != -1 for line in lines])
+
   def wait_for_db_to_appear(self, db_name, timeout_s):
     """Wait until the database with 'db_name' is present in the impalad's local catalog.
     Fail after timeout_s if the doesn't appear."""
@@ -1228,20 +1268,24 @@ class ImpalaTestSuite(BaseTestSuite):
         "Check failed to return True after {0} tries and {1} seconds{2}".format(
           count, timeout_s, error_msg_str))
 
-  def assert_impalad_log_contains(self, level, line_regex, expected_count=1, timeout_s=6):
+  def assert_impalad_log_contains(self, level, line_regex, expected_count=1, timeout_s=6,
+      dry_run=False):
     """
     Convenience wrapper around assert_log_contains for impalad logs.
     """
-    self.assert_log_contains("impalad", level, line_regex, expected_count, timeout_s)
+    return self.assert_log_contains(
+        "impalad", level, line_regex, expected_count, timeout_s, dry_run)
 
   def assert_catalogd_log_contains(self, level, line_regex, expected_count=1,
-      timeout_s=6):
+      timeout_s=6, dry_run=False):
     """
     Convenience wrapper around assert_log_contains for catalogd logs.
     """
-    self.assert_log_contains("catalogd", level, line_regex, expected_count, timeout_s)
+    return self.assert_log_contains(
+        "catalogd", level, line_regex, expected_count, timeout_s, dry_run)
 
-  def assert_log_contains(self, daemon, level, line_regex, expected_count=1, timeout_s=6):
+  def assert_log_contains(self, daemon, level, line_regex, expected_count=1, timeout_s=6,
+      dry_run=False):
     """
     Assert that the daemon log with specified level (e.g. ERROR, WARNING, INFO) contains
     expected_count lines with a substring matching the regex. When expected_count is -1,
@@ -1252,6 +1296,9 @@ class ImpalaTestSuite(BaseTestSuite):
     make sure that log buffering has been disabled, for example by adding
     '-logbuflevel=-1' to the daemon startup options or set timeout_s to a value higher
     than the log flush interval.
+
+    Returns the result of the very last call to line_regex.search or None if
+    expected_count is 0 or the line_regex did not match any lines.
     """
     pattern = re.compile(line_regex)
     start_time = time.time()
@@ -1265,19 +1312,23 @@ class ImpalaTestSuite(BaseTestSuite):
         log_file_path = os.path.join(log_dir, daemon + "." + level)
         # Resolve symlinks to make finding the file easier.
         log_file_path = os.path.realpath(log_file_path)
+        last_re_result = None
         with open(log_file_path) as log_file:
           for line in log_file:
-            if pattern.search(line):
+            re_result = pattern.search(line)
+            if re_result:
               found += 1
-        if expected_count == -1:
-          assert found > 0, "Expected at least one line in file %s matching regex '%s'"\
-            ", but found none." % (log_file_path, line_regex)
-        else:
-          assert found == expected_count, \
-            "Expected %d lines in file %s matching regex '%s', but found %d lines. "\
-            "Last line was: \n%s" %\
-            (expected_count, log_file_path, line_regex, found, line)
-        return
+              last_re_result = re_result
+        if not dry_run:
+          if expected_count == -1:
+            assert found > 0, "Expected at least one line in file %s matching regex '%s'"\
+              ", but found none." % (log_file_path, line_regex)
+          else:
+            assert found == expected_count, \
+              "Expected %d lines in file %s matching regex '%s', but found %d lines. "\
+              "Last line was: \n%s" %\
+              (expected_count, log_file_path, line_regex, found, line)
+        return last_re_result
       except AssertionError as e:
         # Re-throw the exception to the caller only when the timeout is expired. Otherwise
         # sleep before retrying.
