@@ -89,13 +89,24 @@ class SkipIfFS:
   eventually_consistent = pytest.mark.skipif(IS_ADLS or IS_COS or IS_OSS or IS_OBS,
       reason="The client is slow to realize changes to file metadata")
 
+
 class SkipIfKudu:
-  no_hybrid_clock = pytest.mark.skipif(
-      get_kudu_master_flag("--use_hybrid_clock") == "false",
-      reason="Test relies on --use_hybrid_clock=true in Kudu.")
-  hms_integration_enabled = pytest.mark.skipif(
-      get_kudu_master_flag("--hive_metastore_uris") != "",
-      reason="Test assumes Kudu/HMS integration is not enabled.")
+  """Expose decorators as methods so that kudu web pages can be checked lazily when
+     needed, instead of whenever this module is imported. This helps to run Kudu
+     unrelated tests without launching the Kudu cluster."""
+
+  @classmethod
+  def no_hybrid_clock(cls):
+    return pytest.mark.skipif(
+        get_kudu_master_flag("--use_hybrid_clock") == "false",
+        reason="Test relies on --use_hybrid_clock=true in Kudu.")
+
+  @classmethod
+  def hms_integration_enabled(cls):
+    return pytest.mark.skipif(
+        get_kudu_master_flag("--hive_metastore_uris") != "",
+        reason="Test assumes Kudu/HMS integration is not enabled.")
+
 
 class SkipIf:
   skip_hbase = pytest.mark.skipif(pytest.config.option.skip_hbase,
@@ -116,6 +127,8 @@ class SkipIf:
   is_buggy_el6_kernel = pytest.mark.skipif(
       IS_BUGGY_EL6_KERNEL, reason="Kernel is affected by KUDU-1508")
   is_test_jdk = pytest.mark.skipif(IS_TEST_JDK, reason="Testing with different JDK")
+  runs_slowly = pytest.mark.skipif(IMPALA_TEST_CLUSTER_PROPERTIES.runs_slowly(),
+      reason="Test cluster runs slowly due to enablement of code coverage or sanitizer")
 
 class SkipIfLocal:
   # These are skipped due to product limitations.
@@ -148,6 +161,8 @@ class SkipIfNotHdfsMinicluster:
              "with no EC")
 
 class SkipIfBuildType:
+  dev_build = pytest.mark.skipif(IMPALA_TEST_CLUSTER_PROPERTIES.is_dev(),
+      reason="Test takes too much time on debug build.")
   not_dev_build = pytest.mark.skipif(not IMPALA_TEST_CLUSTER_PROPERTIES.is_dev(),
       reason="Test depends on debug build startup option.")
   remote = pytest.mark.skipif(IMPALA_TEST_CLUSTER_PROPERTIES.is_remote_cluster(),
@@ -177,6 +192,8 @@ class SkipIfDockerizedCluster:
       reason="IMPALA-8384: insert ACL tests are broken on dockerised minicluster.")
   insufficient_mem_limit = pytest.mark.skipif(
       IS_DOCKERIZED_TEST_CLUSTER, reason="Test require high per-process mem_limit.")
+  runs_slowly = pytest.mark.skipif(
+      IS_DOCKERIZED_TEST_CLUSTER, reason="Dockerized env too slow for test.")
   jira = partial(pytest.mark.skipif, IS_DOCKERIZED_TEST_CLUSTER)
 
 
@@ -218,13 +235,6 @@ class SkipIfCatalogV2:
       IMPALA_TEST_CLUSTER_PROPERTIES.is_catalog_v2_cluster(),
       reason="Test is specific to old implementation of catalog.")
 
-  # TODO: IMPALA-7131: add support or update tests to reflect expected behaviour.
-  @classmethod
-  def data_sources_unsupported(self):
-    return pytest.mark.skipif(
-      IMPALA_TEST_CLUSTER_PROPERTIES.is_catalog_v2_cluster(),
-      reason="IMPALA-7131: data sources not supported.")
-
   # TODO: IMPALA-8489: fix this bug.
   @classmethod
   def impala_8489(self):
@@ -260,3 +270,6 @@ class SkipIfOS:
 class SkipIfApacheHive():
   feature_not_supported = pytest.mark.skipif(IS_APACHE_HIVE,
       reason="Apache Hive does not support this feature")
+  data_connector_not_supported = pytest.mark.skipif(
+      IS_APACHE_HIVE and HIVE_MAJOR_VERSION <= 3,
+      reason="Apache Hive 3.1 or older version do not support DataConnector")
