@@ -185,16 +185,17 @@ def try_compile_regex(row_string):
     return regex
   return None
 
+
 # If comparing against a float or double, don't do a strict comparison
-# See: http://www.cygnus-software.com/papers/comparingfloats/comparingfloats.htm
-def compare_float(x, y, epsilon):
+# See: https://peps.python.org/pep-0485/#proposed-implementation
+def compare_float(x, y, rel_tol=1e-9, abs_tol=0.0):
   # For the purposes of test validation, we want to treat nans as equal.  The
   # floating point spec defines nan != nan.
   if math.isnan(x) and math.isnan(y):
     return True
   if math.isinf(x) or math.isinf(y):
     return x == y
-  return abs(x - y) <= epsilon
+  return abs(x - y) <= max(rel_tol * max(abs(x), abs(y)), abs_tol)
 
 # Represents a column in a row
 class ResultColumn(object):
@@ -227,9 +228,9 @@ class ResultColumn(object):
     if (self.value == 'NULL' or other.value == 'NULL'):
       return self.value == other.value
     elif self.column_type == 'float':
-      return compare_float(float(self.value), float(other.value), 10e-5)
+      return compare_float(float(self.value), float(other.value), abs_tol=10e-5)
     elif self.column_type == 'double':
-      return compare_float(float(self.value), float(other.value), 10e-10)
+      return compare_float(float(self.value), float(other.value), abs_tol=10e-10)
     elif self.column_type == 'boolean':
       return str(self.value).lower() == str(other.value).lower()
     else:
@@ -294,11 +295,27 @@ def verify_query_result_is_equal(expected_results, actual_results):
   assert_args_not_none(expected_results, actual_results)
   assert expected_results == actual_results
 
-def verify_query_result_is_not_in(expected_results, actual_results):
-  assert_args_not_none(expected_results, actual_results)
-  expected_set = set(map(unicode, expected_results.rows))
+
+def verify_query_result_is_not_in(banned_results, actual_results):
+  assert_args_not_none(banned_results, actual_results)
+  banned_literals, banned_non_literals = banned_results.separate_rows()
+
+  # Part 1: No intersection with the banned literals
+  banned_literals_set = set([unicode(row) for row in banned_literals])
   actual_set = set(map(unicode, actual_results.rows))
-  assert expected_set.isdisjoint(actual_set)
+  assert banned_literals_set.isdisjoint(actual_set)
+
+  # Part 2: Walk through each banned non-literal / regex and make sure that no row
+  # in the actual output matches.
+  for banned_row in banned_non_literals:
+    matched = False
+    for actual_row in actual_results.rows:
+      # Equals is overloaded, so this is doing a regex check
+      if actual_row == banned_row:
+        matched = True
+        break
+    assert not matched, u"Found banned row {0} in actual rows:\n{1}".format(
+      unicode(banned_row), unicode(actual_results))
 
 # Global dictionary that maps the verification type to appropriate verifier.
 # The RESULTS section of a .test file is tagged with the verifier type. We may
