@@ -17,7 +17,7 @@
 
 '''This module generates a docker environment for a job'''
 
-from __future__ import division
+from __future__ import absolute_import, division, print_function
 try:
   from fabric.api import sudo, run, settings
 except ImportError as e:
@@ -141,7 +141,7 @@ class ImpalaDockerEnv(object):
           volume_ops = ' '.join(
               ['-v {host_path}:{container_path}'.format(host_path=host_path,
                                                         container_path=container_path)
-               for host_path, container_path in volume_map.iteritems()])
+               for host_path, container_path in volume_map.items()])
         start_command += (
             'docker run -d -t {volume_ops} -p {postgres_port}:5432 -p {ssh_port}:22 '
             '-p {impala_port}:21050 {docker_image_name} /bin/docker-boot-daemon').format(
@@ -302,6 +302,7 @@ class ImpalaDockerEnv(object):
     # data as a volume to bypass AUFS. See also the README for Leopard.
     LOG.info('Warming testdata cluster external volume')
     self.start_new_container()
+    volume_map = None
     with settings(
         warn_only=True,
         host_string=self.host,
@@ -325,8 +326,6 @@ class ImpalaDockerEnv(object):
       volume_map = {
           HOST_TESTDATA_EXTERNAL_VOLUME_PATH: DOCKER_TESTDATA_VOLUME_PATH,
       }
-    else:
-      volume_map = None
 
     self.start_new_container(volume_map=volume_map)
     LOG.info('Container Started')
