@@ -18,14 +18,24 @@
 from __future__ import absolute_import, division, print_function
 import logging
 import re
+import time
 
+from beeswaxd.BeeswaxService import QueryState
+from builtins import round
 from tests.common.custom_cluster_test_suite import CustomClusterTestSuite
 from tests.common.environ import build_flavor_timeout
+from tests.util.filesystem_utils import IS_S3, get_fs_path
 from time import sleep
 
 LOG = logging.getLogger('catalogd_ha_test')
 DEFAULT_STATESTORE_SERVICE_PORT = 24000
 DEFAULT_CATALOG_SERVICE_PORT = 26000
+SLOW_BUILD_SYNC_DDL_DELAY_S = 20
+SYNC_DDL_DELAY_S = build_flavor_timeout(
+    10, slow_build_timeout=SLOW_BUILD_SYNC_DDL_DELAY_S)
+# s3 can behave as a slow build.
+if IS_S3:
+  SYNC_DDL_DELAY_S = SLOW_BUILD_SYNC_DDL_DELAY_S
 
 
 class TestCatalogdHA(CustomClusterTestSuite):
@@ -183,6 +193,7 @@ class TestCatalogdHA(CustomClusterTestSuite):
   @CustomClusterTestSuite.with_args(
     statestored_args="--use_subscriber_id_as_catalogd_priority=true "
                      "--statestore_heartbeat_frequency_ms=1000",
+    catalogd_args="--catalogd_ha_reset_metadata_on_failover=false",
     start_args="--enable_catalogd_ha")
   def test_catalogd_auto_failover(self):
     """Tests for Catalog Service auto fail over without failed RPCs."""
@@ -200,6 +211,7 @@ class TestCatalogdHA(CustomClusterTestSuite):
     statestored_args="--use_subscriber_id_as_catalogd_priority=true "
                      "--statestore_heartbeat_frequency_ms=1000 "
                      "--debug_actions=SEND_UPDATE_CATALOGD_RPC_FIRST_ATTEMPT:FAIL@1.0",
+    catalogd_args="--catalogd_ha_reset_metadata_on_failover=false",
     start_args="--enable_catalogd_ha")
   def test_catalogd_auto_failover_with_failed_rpc(self):
     """Tests for Catalog Service auto fail over with failed RPCs."""
@@ -282,6 +294,7 @@ class TestCatalogdHA(CustomClusterTestSuite):
   @CustomClusterTestSuite.with_args(
     statestored_args="--use_subscriber_id_as_catalogd_priority=true "
                      "--statestore_heartbeat_frequency_ms=1000",
+    catalogd_args="--catalogd_ha_reset_metadata_on_failover=false",
     start_args="--enable_catalogd_ha")
   def test_catalogd_manual_failover(self):
     """Tests for Catalog Service manual fail over without failed RPCs."""
@@ -299,6 +312,7 @@ class TestCatalogdHA(CustomClusterTestSuite):
     statestored_args="--use_subscriber_id_as_catalogd_priority=true "
                      "--statestore_heartbeat_frequency_ms=1000 "
                      "--debug_actions=SEND_UPDATE_CATALOGD_RPC_FIRST_ATTEMPT:FAIL@1.0",
+    catalogd_args="--catalogd_ha_reset_metadata_on_failover=false",
     start_args="--enable_catalogd_ha")
   def test_catalogd_manual_failover_with_failed_rpc(self):
     """Tests for Catalog Service manual fail over with failed RPCs."""
@@ -312,6 +326,38 @@ class TestCatalogdHA(CustomClusterTestSuite):
     assert(successful_update_catalogd_rpc_num >= 10)
     assert(failed_update_catalogd_rpc_num == successful_update_catalogd_rpc_num)
 
+  @CustomClusterTestSuite.with_args(
+    statestored_args="--use_subscriber_id_as_catalogd_priority=true "
+                     "--statestore_heartbeat_frequency_ms=1000",
+    impalad_args="--debug_actions=IGNORE_NEW_ACTIVE_CATALOGD_ADDR:FAIL@1.0",
+    start_args="--enable_catalogd_ha")
+  def test_manual_failover_with_coord_ignore_notification(self):
+    """Tests for Catalog Service manual failover with coordinators to ignore failover
+    notification."""
+    # Verify two catalogd instances are created with one as active.
+    catalogds = self.cluster.catalogds()
+    assert(len(catalogds) == 2)
+    catalogd_service_1 = catalogds[0].service
+    catalogd_service_2 = catalogds[1].service
+    assert(catalogd_service_1.get_metric_value("catalog-server.active-status"))
+    assert(not catalogd_service_2.get_metric_value("catalog-server.active-status"))
+
+    # Restart standby catalogd with force_catalogd_active as true.
+    catalogds[1].kill()
+    catalogds[1].start(wait_until_ready=True,
+                       additional_args="--force_catalogd_active=true")
+    # Wait until original active catalogd becomes in-active.
+    catalogd_service_1 = catalogds[0].service
+    catalogd_service_1.wait_for_metric_value(
+        "catalog-server.active-status", expected_value=False, timeout=15)
+    assert(not catalogd_service_1.get_metric_value("catalog-server.active-status"))
+
+    # Run query to create a table. Coordinator still send request to catalogd_service_1
+    # so that the request will be rejected.
+    ddl_query = "CREATE TABLE coordinator_ignore_notification (c int)"
+    ex = self.execute_query_expect_failure(self.client, ddl_query)
+    assert "Request for Catalog service is rejected" in str(ex)
+
   @CustomClusterTestSuite.with_args(
     statestored_args="--use_subscriber_id_as_catalogd_priority=true",
     start_args="--enable_catalogd_ha")
@@ -389,3 +435,83 @@ class TestCatalogdHA(CustomClusterTestSuite):
 
     # Verify simple queries are ran successfully.
     self.__run_simple_queries()
+
+  @CustomClusterTestSuite.with_args(
+    statestored_args="--use_subscriber_id_as_catalogd_priority=true",
+    catalogd_args="--debug_actions='catalogd_wait_sync_ddl_version_delay:SLEEP@{0}'"
+                  .format(SYNC_DDL_DELAY_S * 1000),
+    start_args="--enable_catalogd_ha")
+  def test_catalogd_failover_with_sync_ddl(self, unique_database):
+    """Tests for Catalog Service force fail-over when running DDL with SYNC_DDL
+    enabled."""
+    # Verify two catalogd instances are created with one as active.
+    catalogds = self.cluster.catalogds()
+    assert(len(catalogds) == 2)
+    catalogd_service_1 = catalogds[0].service
+    catalogd_service_2 = catalogds[1].service
+    assert(catalogd_service_1.get_metric_value("catalog-server.active-status"))
+    assert(not catalogd_service_2.get_metric_value("catalog-server.active-status"))
+
+    # Run DDL with SYNC_DDL enabled.
+    client = self.cluster.impalads[0].service.create_beeswax_client()
+    assert client is not None
+    try:
+      self.execute_query_expect_success(client, "set SYNC_DDL=1")
+      ddl_query = "CREATE TABLE {database}.failover_sync_ddl (c int)"
+      handle = client.execute_async(ddl_query.format(database=unique_database))
+
+      # Restart standby catalogd with force_catalogd_active as true.
+      start_s = time.time()
+      catalogds[1].kill()
+      catalogds[1].start(wait_until_ready=True,
+                         additional_args="--force_catalogd_active=true")
+      # Wait until original active catalogd becomes in-active.
+      catalogd_service_1 = catalogds[0].service
+      catalogd_service_1.wait_for_metric_value(
+          "catalog-server.active-status", expected_value=False, timeout=15)
+      assert(not catalogd_service_1.get_metric_value("catalog-server.active-status"))
+      elapsed_s = time.time() - start_s
+      assert elapsed_s < SYNC_DDL_DELAY_S, \
+          "Catalogd failover took %s seconds to complete" % (elapsed_s)
+      LOG.info("Catalogd failover took %s seconds to complete" % round(elapsed_s, 1))
+
+      # Verify that the query is failed due to the Catalogd HA fail-over.
+      self.wait_for_state(
+          handle, QueryState.EXCEPTION, SYNC_DDL_DELAY_S * 2 + 10, client=client)
+    finally:
+      client.close()
+
+  @CustomClusterTestSuite.with_args(
+    statestored_args="--use_subscriber_id_as_catalogd_priority=true",
+    catalogd_args="--catalogd_ha_reset_metadata_on_failover=true",
+    start_args="--enable_catalogd_ha")
+  def test_metadata_after_failover(self, unique_database):
+    """Verify that the metadata is correct after failover."""
+    catalogds = self.cluster.catalogds()
+    assert(len(catalogds) == 2)
+    catalogd_service_1 = catalogds[0].service
+    catalogd_service_2 = catalogds[1].service
+    assert(catalogd_service_1.get_metric_value("catalog-server.active-status"))
+    assert(not catalogd_service_2.get_metric_value("catalog-server.active-status"))
+
+    create_func_impala = ("create function {database}.identity_tmp(bigint) "
+                          "returns bigint location '{location}' symbol='Identity'")
+    self.client.execute(create_func_impala.format(
+        database=unique_database,
+        location=get_fs_path('/test-warehouse/libTestUdfs.so')))
+    self.execute_query_expect_success(
+        self.client, "select %s.identity_tmp(10)" % unique_database)
+
+    # Kill active catalogd
+    catalogds[0].kill()
+
+    # Wait for long enough for the statestore to detect the failure of active catalogd
+    # and assign active role to standby catalogd.
+    catalogd_service_2.wait_for_metric_value(
+        "catalog-server.active-status", expected_value=True, timeout=30)
+    assert(catalogd_service_2.get_metric_value(
+        "catalog-server.ha-number-active-status-change") > 0)
+    assert(catalogd_service_2.get_metric_value("catalog-server.active-status"))
+
+    self.execute_query_expect_success(
+        self.client, "select %s.identity_tmp(10)" % unique_database)
