@@ -36,7 +36,7 @@ RETRY_PROFILE_MSG = 'Retried query planning due to inconsistent metadata'
 CATALOG_VERSION_LOWER_BOUND = 'catalog.catalog-object-version-lower-bound'
 
 
-class TestCompactCatalogUpdates(CustomClusterTestSuite):
+class TestLocalCatalogCompactUpdates(CustomClusterTestSuite):
 
   @pytest.mark.execute_serially
   @CustomClusterTestSuite.with_args(
@@ -449,7 +449,42 @@ class TestLocalCatalogRetries(CustomClusterTestSuite):
     for i in t.imap_unordered(do_table, range(NUM_ITERS)):
       pass
 
-class TestObservability(CustomClusterTestSuite):
+  @CustomClusterTestSuite.with_args(
+      impalad_args="--use_local_catalog=true "
+                   "--inject_failure_ratio_in_catalog_fetch=0.1 "
+                   "--inject_latency_after_catalog_fetch_ms=100",
+      catalogd_args="--catalog_topic_mode=minimal")
+  def test_fetch_metadata_retry_in_piggybacked_failures(self, unique_database):
+    test_self = self
+
+    class ThreadLocalClient(threading.local):
+      def __init__(self):
+        self.c = test_self.create_impala_client()
+
+    NUM_THREADS = 8
+    t = ThreadPool(processes=NUM_THREADS)
+    tls = ThreadLocalClient()
+
+    self.execute_query(
+        "create table {0}.tbl (i int) partitioned by (p int)".format(unique_database))
+    self.execute_query(
+        "insert into {0}.tbl partition(p) values (0,0)".format(unique_database))
+
+    def read_part(i):
+      self.execute_query_expect_success(
+          tls.c, "select * from {0}.tbl where p=0".format(unique_database))
+
+    # Prior to fixing IMPALA-12670, this test would fail within 20 iterations,
+    # so 100 should be quite reliable as a regression test.
+    NUM_ITERS = 100
+    for k in range(NUM_ITERS):
+      # Read the same partition in concurrent queries so requests can be piggybacked.
+      for i in t.imap_unordered(read_part, range(NUM_THREADS)):
+        pass
+      # Refresh to invalidate the partition in local catalog cache
+      self.execute_query("refresh {0}.tbl partition(p=0)".format(unique_database))
+
+class TestLocalCatalogObservability(CustomClusterTestSuite):
   def get_catalog_cache_metrics(self, impalad):
     """ Returns catalog cache metrics as a dict by scraping the json metrics page on the
     given impalad"""
@@ -517,7 +552,7 @@ class TestObservability(CustomClusterTestSuite):
 
           cache_entry_median_size = cache_metrics[cache_entry_median_size_key]
           cache_entry_99th_size = cache_metrics[cache_entry_99th_size_key]
-          assert cache_entry_median_size > 300 and cache_entry_median_size < 1000
+          assert cache_entry_median_size > 300 and cache_entry_median_size < 3000
           assert cache_entry_99th_size > 12500 and cache_entry_99th_size < 19000
 
           cache_hit_count_prev_run = cache_hit_count
@@ -525,6 +560,20 @@ class TestObservability(CustomClusterTestSuite):
     finally:
       client.close()
 
+  @pytest.mark.execute_serially
+  @CustomClusterTestSuite.with_args(
+      impalad_args="--use_local_catalog=true",
+      catalogd_args="--catalog_topic_mode=minimal")
+  def test_lightweight_rpc_metrics(self):
+    """Verify catalogd client cache for lightweight RPCs is used correctly"""
+    # Fetching the db and table list should be lightweight requests
+    self.execute_query("describe database functional")
+    self.execute_query("show tables in functional")
+    impalad = self.cluster.impalads[0].service
+    assert 0 == impalad.get_metric_value("catalog.server.client-cache.total-clients")
+    assert 1 == impalad.get_metric_value(
+        "catalog.server.client-cache.total-clients-for-lightweight-rpc")
+
 
 class TestFullAcid(CustomClusterTestSuite):
   @classmethod
