@@ -17,6 +17,8 @@
 #
 # Tests for IMPALA-2273
 
+from __future__ import absolute_import, division, print_function
+from builtins import range
 import os
 import pytest
 import random
@@ -89,10 +91,10 @@ class TestParquetMaxPageHeader(CustomClusterTestSuite):
     """Creates a file in HDFS containing two MAX_STRING_LENGTH lines."""
     file_name = os.path.join(dir, file)
     # Create two 10MB long strings.
-    random_text1 = "".join([random.choice(string.letters)
-        for i in xrange(self.MAX_STRING_LENGTH)])
-    random_text2 = "".join([random.choice(string.letters)
-        for i in xrange(self.MAX_STRING_LENGTH)])
+    random_text1 = "".join([random.choice(string.ascii_letters)
+        for i in range(self.MAX_STRING_LENGTH)])
+    random_text2 = "".join([random.choice(string.ascii_letters)
+        for i in range(self.MAX_STRING_LENGTH)])
     put = subprocess.Popen(["hdfs", "dfs", "-put", "-d", "-f", "-", file_name],
         stdin=subprocess.PIPE, bufsize=-1)
     put.stdin.write(random_text1 + "\n")
