@@ -24,12 +24,14 @@ import pytest
 import re
 import shutil
 import stat
+import subprocess
 import tempfile
+import time
 
 from tests.common.custom_cluster_test_suite import CustomClusterTestSuite
-from tests.verifiers.metric_verifier import MetricVerifier
 from tests.common.skip import SkipIf
 from tests.util.hdfs_util import NAMENODE
+from tests.verifiers.metric_verifier import MetricVerifier
 
 class TestScratchDir(CustomClusterTestSuite):
   @classmethod
@@ -96,6 +98,7 @@ class TestScratchDir(CustomClusterTestSuite):
   def teardown_method(self, method):
     for dir_path in self.created_dirs:
       shutil.rmtree(dir_path, ignore_errors=True)
+    self.check_deleted_file_fd()
 
   @pytest.mark.execute_serially
   def test_multiple_dirs(self, vector):
@@ -279,6 +282,38 @@ class TestScratchDir(CustomClusterTestSuite):
   def dfs_tmp_path(self):
     return "{}/tmp".format(NAMENODE)
 
+  def find_deleted_files_in_fd(self, pid):
+    fd_path = "/proc/{}/fd".format(pid)
+    # Look for the files with keywords 'scratch' and '(deleted)'.
+    # Limited to keyword 'scratch' because in IMPALA-12698 the process may
+    # create some reference deleted hdfs files, but the files are eventually
+    # removed in a few minutes. This limitation helps to mitigate false-positive
+    # checks.
+    command = "find {} -ls | grep -E 'scratch.*(deleted)'".format(fd_path)
+    try:
+      result = subprocess.check_output(command, shell=True)
+      return result.strip()
+    except subprocess.CalledProcessError as e:
+      if not e.output:
+        # If there is no output, return None.
+        return None
+      return "Error checking the fd path with error '{}'".format(e)
+
+  def check_deleted_file_fd(self):
+    # Check if we have deleted but still referenced files in fd.
+    # Regression test for IMPALA-12681.
+    pids = []
+    for impalad in self.cluster.impalads:
+      pids.append(impalad.get_pid())
+    assert pids
+    for pid in pids:
+      deleted_files = self.find_deleted_files_in_fd(pid)
+      if deleted_files is not None:
+        # Retry again if fails at the first time.
+        time.sleep(15)
+        deleted_files = self.find_deleted_files_in_fd(pid)
+      assert deleted_files is None
+
   @pytest.mark.execute_serially
   @SkipIf.not_scratch_fs
   def test_scratch_dirs_remote_spill(self, vector):
