@@ -17,6 +17,8 @@
 #
 # Superclass of all HS2 tests containing commonly used functions.
 
+from __future__ import absolute_import, division, print_function
+from builtins import range
 from getpass import getuser
 from TCLIService import TCLIService
 from ImpalaService import ImpalaHiveServer2Service
@@ -280,13 +282,13 @@ class HS2TestSuite(ImpalaTestSuite):
         num_rows = len(typed_col.values)
         break
 
-    for i in xrange(num_rows):
+    for i in range(num_rows):
       row = []
       for c in columns:
         for col_type in HS2TestSuite.HS2_V6_COLUMN_TYPES:
           typed_col = getattr(c, col_type)
           if typed_col != None:
-            indicator = ord(typed_col.nulls[i / 8])
+            indicator = ord(typed_col.nulls[i // 8])
             if indicator & (1 << (i % 8)):
               row.append("NULL")
             else:
