@@ -25,17 +25,17 @@ import time
 
 from beeswaxd.BeeswaxService import QueryState
 from copy import deepcopy
-from test_ddl_base import TestDdlBase
+from tests.metadata.test_ddl_base import TestDdlBase
 from tests.beeswax.impala_beeswax import ImpalaBeeswaxException
 from tests.common.environ import (HIVE_MAJOR_VERSION)
 from tests.common.file_utils import create_table_from_orc
 from tests.common.impala_test_suite import LOG
 from tests.common.parametrize import UniqueDatabase
 from tests.common.skip import (SkipIf, SkipIfFS, SkipIfKudu, SkipIfLocal,
-                               SkipIfCatalogV2, SkipIfHive2)
+                               SkipIfCatalogV2, SkipIfHive2, SkipIfDockerizedCluster)
 from tests.common.test_dimensions import create_single_exec_option_dimension
 from tests.common.test_dimensions import (create_exec_option_dimension,
-    create_client_protocol_dimension)
+    create_client_protocol_dimension, create_exec_option_dimension_from_dict)
 from tests.common.test_vector import ImpalaTestDimension
 from tests.util.filesystem_utils import (
     get_fs_path,
@@ -326,7 +326,7 @@ class TestDdlStatements(TestDdlBase):
         use_db=unique_database, multiple_impalad=self._use_multiple_impalad(vector))
 
   @UniqueDatabase.parametrize(sync_ddl=True)
-  @SkipIfKudu.no_hybrid_clock
+  @SkipIfKudu.no_hybrid_clock()
   def test_create_kudu(self, vector, unique_database):
     vector.get_value('exec_option')['abort_on_error'] = False
     vector.get_value('exec_option')['kudu_read_mode'] = "READ_AT_SNAPSHOT"
@@ -472,6 +472,12 @@ class TestDdlStatements(TestDdlBase):
     self.run_test_case('QueryTest/alter-table-set-column-stats', vector,
         use_db=unique_database, multiple_impalad=self._use_multiple_impalad(vector))
 
+  @SkipIfFS.hbase
+  @UniqueDatabase.parametrize(sync_ddl=True)
+  def test_alter_hbase_set_column_stats(self, vector, unique_database):
+    self.run_test_case('QueryTest/alter-hbase-table-set-column-stats', vector,
+        use_db=unique_database, multiple_impalad=self._use_multiple_impalad(vector))
+
   @SkipIfLocal.hdfs_client
   def test_drop_partition_with_purge(self, vector, unique_database):
     """Verfies whether alter <tbl> drop partition purge actually skips trash"""
@@ -1142,6 +1148,25 @@ class TestAsyncDDLTiming(TestDdlBase):
       client.close()
 
 
+class TestDdlLogs(TestDdlBase):
+  @classmethod
+  def add_test_dimensions(cls):
+    super(TestDdlLogs, cls).add_test_dimensions()
+    cls.ImpalaTestMatrix.add_dimension(create_exec_option_dimension_from_dict({
+        'enable_async_ddl_execution': [True, False]}))
+
+  @SkipIfDockerizedCluster.daemon_logs_not_exposed
+  def test_error_logs(self, vector, unique_database):
+    query_opts = vector.get_value('exec_option')
+    tbl_name = 'test_async' if query_opts['enable_async_ddl_execution'] else 'test_sync'
+    tbl_name = unique_database + '.' + tbl_name
+    result = self.execute_query_expect_failure(
+        self.client, 'invalidate metadata ' + tbl_name, query_opts)
+    err = "TableNotFoundException: Table not found: " + tbl_name
+    assert err in str(result)
+    self.assert_impalad_log_contains('INFO', err)
+
+
 # IMPALA-2002: Tests repeated adding/dropping of .jar and .so in the lib cache.
 class TestLibCache(TestDdlBase):
   @classmethod
@@ -1168,7 +1193,6 @@ class TestLibCache(TestDdlBase):
   # Run serially because this test inspects global impalad metrics.
   # TODO: The metrics checks could be relaxed to enable running this test in
   # parallel, but that might need a more general wait_for_metric_value().
-  @SkipIfCatalogV2.data_sources_unsupported()
   @pytest.mark.execute_serially
   def test_create_drop_data_src(self, vector, unique_database):
     """This will create, run, and drop the same data source repeatedly, exercising
