@@ -17,6 +17,7 @@
 
 # Targeted tests for date type.
 
+from __future__ import absolute_import, division, print_function
 import pytest
 from tests.common.file_utils import create_table_and_copy_files
 from tests.common.impala_test_suite import ImpalaTestSuite
@@ -39,9 +40,10 @@ class TestDateQueries(ImpalaTestSuite):
         'batch_size': [0, 1],
         'disable_codegen': ['false', 'true'],
         'disable_codegen_rows_threshold': [0]}))
-    # DATE type is only supported for text, parquet and avro fileformat on HDFS and HBASE.
+    # DATE type is only supported for text, parquet, avro, orc and json fileformat on HDFS
+    # and HBASE.
     cls.ImpalaTestMatrix.add_constraint(lambda v:
-        v.get_value('table_format').file_format in ('text', 'hbase', 'parquet')
+        v.get_value('table_format').file_format in ('text', 'hbase', 'parquet', 'json')
         or (v.get_value('table_format').file_format == 'avro'
             and v.get_value('table_format').compression_codec == 'snap'))
 
