@@ -27,7 +27,7 @@ import random
 import re
 import time
 
-from subprocess import check_call
+from subprocess import check_call, check_output
 # noinspection PyUnresolvedReferences
 from parquet.ttypes import ConvertedType
 
@@ -38,7 +38,7 @@ import json
 from tests.beeswax.impala_beeswax import ImpalaBeeswaxException
 from tests.common.iceberg_test_suite import IcebergTestSuite
 from tests.common.skip import SkipIf, SkipIfFS, SkipIfDockerizedCluster
-from tests.common.test_vector import ImpalaTestDimension
+from tests.common.test_dimensions import add_exec_option_dimension
 from tests.common.file_utils import (
   create_iceberg_table_from_directory,
   create_table_from_parquet)
@@ -69,8 +69,17 @@ class TestIcebergTable(IcebergTestSuite):
   def test_create_iceberg_tables(self, vector, unique_database):
     self.run_test_case('QueryTest/iceberg-create', vector, use_db=unique_database)
 
-  def test_alter_iceberg_tables(self, vector, unique_database):
-    self.run_test_case('QueryTest/iceberg-alter', vector, use_db=unique_database)
+  def test_alter_iceberg_tables_v1(self, vector, unique_database):
+    self.run_test_case('QueryTest/iceberg-alter-v1', vector, use_db=unique_database)
+
+  def test_alter_iceberg_tables_v2(self, vector, unique_database):
+    self.run_test_case('QueryTest/iceberg-alter-v2', vector, use_db=unique_database)
+
+  def test_alter_iceberg_tables_default(self, vector, unique_database):
+    self.run_test_case('QueryTest/iceberg-alter-default', vector, use_db=unique_database)
+
+  def test_external_iceberg_tables(self, vector, unique_database):
+    self.run_test_case('QueryTest/iceberg-external', vector, unique_database)
 
   def test_expire_snapshots(self, unique_database):
     tbl_name = unique_database + ".expire_snapshots"
@@ -176,8 +185,16 @@ class TestIcebergTable(IcebergTestSuite):
   def test_insert(self, vector, unique_database):
     self.run_test_case('QueryTest/iceberg-insert', vector, use_db=unique_database)
 
-  def test_partitioned_insert(self, vector, unique_database):
-    self.run_test_case('QueryTest/iceberg-partitioned-insert', vector,
+  def test_partitioned_insert_v1(self, vector, unique_database):
+    self.run_test_case('QueryTest/iceberg-partitioned-insert-v1', vector,
+        use_db=unique_database)
+
+  def test_partitioned_insert_v2(self, vector, unique_database):
+    self.run_test_case('QueryTest/iceberg-partitioned-insert-v2', vector,
+        use_db=unique_database)
+
+  def test_partitioned_insert_default(self, vector, unique_database):
+    self.run_test_case('QueryTest/iceberg-partitioned-insert-default', vector,
         use_db=unique_database)
 
   def test_insert_overwrite(self, vector, unique_database):
@@ -459,6 +476,12 @@ class TestIcebergTable(IcebergTestSuite):
           "select count(*) from {0} for system_time as of {1}".format(tbl_name, ts),
           expected)
 
+    def expect_snapshot_id_in_plan_t(ts, snapshot_id):
+      data = impalad_client.execute(
+        "explain select * from {0} for system_time as of {1}".format(
+          tbl_name, ts))
+      assert "   Iceberg snapshot id: {0}".format(snapshot_id) in data.data
+
     def expect_results_v(snapshot_id, expected_results, expected_cols):
       expect_results(
           "select * from {0} for system_version as of {1}".format(tbl_name, snapshot_id),
@@ -470,6 +493,13 @@ class TestIcebergTable(IcebergTestSuite):
               tbl_name, snapshot_id),
           expected)
 
+    def expect_snapshot_id_in_plan_v(snapshot_id):
+      data = impalad_client.execute(
+        "explain select * from {0} for system_version as of {1}".format(
+          tbl_name, snapshot_id))
+      assert "   Iceberg snapshot id: {0}".format(snapshot_id) in data.data
+
+
     def impala_now():
       now_data = impalad_client.execute("select now()")
       return now_data.data[0]
@@ -537,6 +567,18 @@ class TestIcebergTable(IcebergTestSuite):
       expect_results_v(snapshots[3].get_snapshot_id(), ['100'], i_cols)
       expect_results_v(snapshots[4].get_snapshot_id(), ['100\tNULL', '3\t103'], ij_cols)
 
+      expect_snapshot_id_in_plan_v(snapshots[0].get_snapshot_id())
+      expect_snapshot_id_in_plan_v(snapshots[1].get_snapshot_id())
+      expect_snapshot_id_in_plan_v(snapshots[2].get_snapshot_id())
+      expect_snapshot_id_in_plan_v(snapshots[3].get_snapshot_id())
+      expect_snapshot_id_in_plan_v(snapshots[4].get_snapshot_id())
+
+      expect_snapshot_id_in_plan_t(quote(ts_1), snapshots[0].get_snapshot_id())
+      expect_snapshot_id_in_plan_t(quote(ts_2), snapshots[1].get_snapshot_id())
+      expect_snapshot_id_in_plan_t(quote(ts_3), snapshots[2].get_snapshot_id())
+      expect_snapshot_id_in_plan_t(quote(ts_4), snapshots[3].get_snapshot_id())
+      expect_snapshot_id_in_plan_t(quote(ts_5), snapshots[4].get_snapshot_id())
+
       # Test of plain count star optimization
       # 'NumRowGroups' and 'NumFileMetadataRead' should not appear in profile
       expect_for_count_star_t("now()", '2')
@@ -667,7 +709,7 @@ class TestIcebergTable(IcebergTestSuite):
 
     os.remove(local_file)
 
-  # Get hdfs path to manifest list that belongs to the sanpshot identified by
+  # Get hdfs path to manifest list that belongs to the snapshot identified by
   # 'snapshot_counter'.
   def get_manifest_list_hdfs_path(self, tmp_path_prefix, db_name, table_name,
       snapshot_counter):
@@ -725,18 +767,147 @@ class TestIcebergTable(IcebergTestSuite):
         os.remove(local_path)
     return datafiles
 
-  @SkipIf.not_dfs
-  def test_writing_metrics_to_metadata(self, vector, unique_database):
-    # Create table
-    table_name = "ice_stats"
-    qualified_table_name = "%s.%s" % (unique_database, table_name)
-    query = 'create table %s ' \
-        '(s string, i int, b boolean, bi bigint, ts timestamp, dt date, ' \
-        'dc decimal(10, 3)) ' \
-        'stored as iceberg' \
-        % qualified_table_name
-    self.client.execute(query)
+  def get_latest_metadata_path(self, database_name, table_name):
+    describe = 'describe extended %s.%s ' % (database_name, table_name)
+    output = self.client.execute(describe)
+    metadata_location = [s for s in output.data if s.startswith('\tmetadata_location')]
+    assert len(metadata_location) == 1
+    metadata_location_split = metadata_location[0].split('\t')
+    assert len(metadata_location_split) == 3
+    metadata_path = metadata_location_split[2]
+    return metadata_path
 
+  # Get the current partition spec as JSON from the latest metadata file
+  def get_current_partition_spec(self, database_name, table_name):
+    hdfs_path = self.get_latest_metadata_path(database_name, table_name)
+    output = check_output(['hadoop', 'fs', '-cat', hdfs_path])
+
+    current_partition_spec = None
+    metadata = json.loads(output)
+
+    current_spec_id = metadata['default-spec-id']
+    for spec in metadata['partition-specs']:
+      if spec['spec-id'] == current_spec_id:
+        current_partition_spec = spec
+        break
+    return current_partition_spec
+
+  @SkipIf.not_dfs
+  def test_partition_spec_update_v1(self, vector, unique_database):
+    # Create table
+    table_name = "ice_part"
+    qualified_table_name = "%s.%s" % (unique_database, table_name)
+    create_table = """create table {}
+        (s string, i int) partitioned by spec(truncate(5, s), identity(i))
+        stored as iceberg
+        tblproperties ('format-version'='1')""".format(qualified_table_name)
+    self.client.execute(create_table)
+
+    partition_spec = self.get_current_partition_spec(unique_database, table_name)
+    assert len(partition_spec['fields']) == 2
+    truncate_s = partition_spec['fields'][0]
+    identity_i = partition_spec['fields'][1]
+    # At table creation, partition names does not contain the parameter value.
+    assert truncate_s['name'] == 's_trunc'
+    assert identity_i['name'] == 'i'
+    assert truncate_s['field-id'] == 1000
+    assert identity_i['field-id'] == 1001
+
+    # Partition evolution
+    partition_evolution = 'alter table %s set partition ' \
+        'spec(identity(i), truncate(6,s))' % qualified_table_name
+    self.client.execute(partition_evolution)
+
+    # V1 partition evolution keeps the old, modified fields, but changes their
+    # transform to VOID.
+    evolved_partition_spec = self.get_current_partition_spec(unique_database, table_name)
+    assert len(evolved_partition_spec['fields']) == 3
+    old_truncate_s = evolved_partition_spec['fields'][0]
+    identity_i = evolved_partition_spec['fields'][1]
+    truncate_s = evolved_partition_spec['fields'][2]
+    assert old_truncate_s['name'] == 's_trunc'
+    assert identity_i['name'] == 'i'
+    # Modified field name contains the parameter value.
+    assert truncate_s['name'] == 's_trunc_6'
+    assert old_truncate_s['field-id'] == 1000
+    assert identity_i['field-id'] == 1001
+    # field-id increases for the modified field.
+    assert truncate_s['field-id'] == 1002
+    assert old_truncate_s['transform'] == 'void'
+
+  @SkipIf.not_dfs
+  def test_partition_spec_update_v2(self, vector, unique_database):
+    # Create table
+    table_name = "ice_part"
+    qualified_table_name = "%s.%s" % (unique_database, table_name)
+    create_table = 'create table %s ' \
+        '(s string, i int) partitioned by spec(truncate(5, s), identity(i)) ' \
+        'stored as iceberg tblproperties ("format-version" = "2")' \
+        % qualified_table_name
+    self.client.execute(create_table)
+
+    partition_spec = self.get_current_partition_spec(unique_database, table_name)
+    assert len(partition_spec['fields']) == 2
+    truncate_s = partition_spec['fields'][0]
+    identity_i = partition_spec['fields'][1]
+    # At table creation, partition names does not contain the parameter value.
+    assert truncate_s['name'] == 's_trunc'
+    assert identity_i['name'] == 'i'
+    assert truncate_s['field-id'] == 1000
+    assert identity_i['field-id'] == 1001
+
+    # Partition evolution for s
+    partition_evolution = 'alter table %s set partition ' \
+        'spec(identity(i), truncate(6,s))' % qualified_table_name
+    self.client.execute(partition_evolution)
+
+    # V2 partition evolution updates the previous partitioning for
+    # the modified field.
+    evolved_partition_spec = self.get_current_partition_spec(unique_database, table_name)
+    assert len(evolved_partition_spec['fields']) == 2
+    identity_i = evolved_partition_spec['fields'][0]
+    truncate_s = evolved_partition_spec['fields'][1]
+    assert identity_i['name'] == 'i'
+    # Modified field name contains the parameter value.
+    assert truncate_s['name'] == 's_trunc_6'
+    assert identity_i['field-id'] == 1001
+    # field-id increased for the modified field.
+    assert truncate_s['field-id'] == 1002
+
+    # Partition evolution for i and s
+    partition_evolution = 'alter table %s set partition ' \
+        'spec(bucket(4, i), truncate(3,s))' \
+        % qualified_table_name
+    self.client.execute(partition_evolution)
+
+    evolved_partition_spec = self.get_current_partition_spec(unique_database, table_name)
+    assert len(evolved_partition_spec['fields']) == 2
+    bucket_i = evolved_partition_spec['fields'][0]
+    truncate_s = evolved_partition_spec['fields'][1]
+    # The field naming follows the transform changes.
+    assert bucket_i['name'] == 'i_bucket_4'
+    assert truncate_s['name'] == 's_trunc_3'
+    # field-id's are increasing with each modification
+    assert bucket_i['field-id'] == 1003
+    assert truncate_s['field-id'] == 1004
+
+  @SkipIf.not_dfs
+  def test_writing_metrics_to_metadata_v1(self, vector, unique_database):
+    self._test_writing_metrics_to_metadata_impl(unique_database, 'ice_stats_v1', '1')
+
+  @SkipIf.not_dfs
+  def test_writing_metrics_to_metadata_v2(self, vector, unique_database):
+    self._test_writing_metrics_to_metadata_impl(unique_database, 'ice_stats_v2', '2')
+
+  def _test_writing_metrics_to_metadata_impl(self, unique_database, table_name, version):
+    # Create table
+    qualified_table_name = "%s.%s" % (unique_database, table_name)
+    query = """create table {}
+        (s string, i int, b boolean, bi bigint, ts timestamp, dt date,
+        dc decimal(10, 3))
+        stored as iceberg
+        tblproperties ('format-version'='{}')""".format(qualified_table_name, version)
+    self.client.execute(query)
     # Insert data
     # 1st data file:
     query = 'insert into %s values ' \
@@ -895,7 +1066,7 @@ class TestIcebergTable(IcebergTestSuite):
     """IMPALA-10914: This test verifies that Impala schedules scan ranges consistently for
     Iceberg tables."""
     def collect_split_stats(profile):
-      splits = [l.strip() for l in profile.splitlines() if "Hdfs split stats" in l]
+      splits = [s.strip() for s in profile.splitlines() if "Hdfs split stats" in s]
       splits.sort()
       return splits
 
@@ -914,6 +1085,54 @@ class TestIcebergTable(IcebergTestSuite):
         split_stats = collect_split_stats(profile)
         assert ref_split_stats == split_stats
 
+  def test_scheduling_partitioned_tables(self, vector, unique_database):
+    """IMPALA-12765: Balance consecutive partitions better for Iceberg tables"""
+    # We are setting the replica_preference query option in this test, so let's create a
+    # local impala client.
+    inventory_tbl = "inventory_ice"
+    item_tbl = "item_ice"
+    date_dim_tbl = "date_dim_ice"
+    with self.create_impala_client() as impalad_client:
+      impalad_client.execute("use " + unique_database)
+      impalad_client.execute("set replica_preference=remote")
+      impalad_client.execute("""
+          CREATE TABLE {}
+          PARTITIONED BY SPEC (inv_date_sk)
+          STORED BY ICEBERG
+          AS SELECT * from tpcds_partitioned_parquet_snap.inventory;
+          """.format(inventory_tbl))
+      impalad_client.execute("""
+          CREATE TABLE {}
+          STORED BY ICEBERG
+          AS SELECT * from tpcds_partitioned_parquet_snap.item;
+          """.format(item_tbl))
+      impalad_client.execute("""
+          CREATE TABLE {}
+          STORED BY ICEBERG
+          AS SELECT * from tpcds_partitioned_parquet_snap.date_dim;
+          """.format(date_dim_tbl))
+      q22_result = impalad_client.execute("""
+          select i_product_name, i_brand, i_class, i_category,
+                avg(inv_quantity_on_hand) qoh
+          from inventory_ice, date_dim_ice, item_ice
+          where inv_date_sk=d_date_sk and
+                inv_item_sk=i_item_sk and
+                d_month_seq between 1199 and 1199 + 11
+          group by rollup(i_product_name, i_brand, i_class, i_category)
+          order by qoh, i_product_name, i_brand, i_class, i_category
+          limit 100
+          """)
+      profile = q22_result.runtime_profile
+      # "Files rejected:" contains the number of files being rejected by runtime
+      # filters. With IMPALA-12765 we should see similar numbers for each executor.
+      files_rejected_array = re.findall(r"Files rejected: \d+ \((\d+)\)", profile)
+      avg_files_rejected = int(files_rejected_array[0])
+      THRESHOLD = 3
+      for files_rejected_str in files_rejected_array:
+        files_rejected = int(files_rejected_str)
+        if files_rejected != 0:
+          assert abs(avg_files_rejected - files_rejected) < THRESHOLD
+
   def test_in_predicate_push_down(self, vector, unique_database):
     self.execute_query("SET RUNTIME_FILTER_MODE=OFF")
     self.run_test_case('QueryTest/iceberg-in-predicate-push-down', vector,
@@ -965,10 +1184,6 @@ class TestIcebergTable(IcebergTestSuite):
     args = ['-q', 'DROP TABLE {0}.{1}'.format(db_name, tbl_name)]
     results = run_impala_shell_cmd(vector, args)
 
-  def test_multiple_storage_locations(self, vector, unique_database):
-    self.run_test_case('QueryTest/iceberg-multiple-storage-locations-table',
-                       vector, unique_database)
-
   def test_mixed_file_format(self, vector, unique_database):
     self.run_test_case('QueryTest/iceberg-mixed-file-format', vector,
                       unique_database)
@@ -1127,6 +1342,20 @@ class TestIcebergTable(IcebergTestSuite):
       self.run_test_case('QueryTest/iceberg-migrate-from-external-hdfs-tables',
                          vector, unique_database)
 
+  def test_table_exists(self, unique_database):
+    """Test that iceberg AlreadyExistsException are correctly handled."""
+    tbl_name = unique_database + ".create_iceberg_exists"
+    # Attempt to create an iceberg table, simulating AlreadyExistsException
+    iceberg_created_options = {'debug_action': 'CATALOGD_ICEBERG_CREATE:EXCEPTION@'
+        'IcebergAlreadyExistsException@Table was created concurrently'}
+    err = self.execute_query_expect_failure(self.client,
+        "create table {0} (i int) stored as iceberg".format(tbl_name),
+        query_options=iceberg_created_options)
+    assert "AlreadyExistsException: Table already exists" in str(err)
+    self.execute_query_expect_success(self.client,
+        "create table if not exists {0} (i int) stored as iceberg".format(tbl_name),
+        query_options=iceberg_created_options)
+
   def test_abort_transaction(self, unique_database):
     """Test that iceberg operations fail correctly when an Iceberg transaction commit
     fails, and that the effects of the failed operation are not visible."""
@@ -1172,6 +1401,33 @@ class TestIcebergTable(IcebergTestSuite):
     assert len(data.data) == 1
     assert data.data[0] == '1'
 
+  def test_drop_partition(self, vector, unique_database):
+    self.run_test_case('QueryTest/iceberg-drop-partition', vector,
+      use_db=unique_database)
+
+  def test_rollback_after_drop_partition(self, vector, unique_database):
+    table_name = "iceberg_drop_partition_rollback"
+    qualified_table_name = "{}.{}".format(unique_database, table_name)
+    create_table_stmt = """CREATE TABLE {}(identity_int int, unpartitioned_int int)
+      PARTITIONED BY SPEC (identity_int) STORED AS ICEBERG""".format(qualified_table_name)
+    insert_into_stmt = """INSERT INTO {} values(1, 2)""".format(qualified_table_name)
+    drop_partition_stmt = """ALTER TABLE {} DROP PARTITION (identity_int = 1)""".format(
+      qualified_table_name)
+
+    self.execute_query(create_table_stmt)
+    self.execute_query(insert_into_stmt)
+    self.execute_query(drop_partition_stmt)
+
+    snapshots = get_snapshots(self.client, qualified_table_name, expected_result_size=2)
+    rollback = """ALTER TABLE {} EXECUTE ROLLBACK ({})""".format(
+      qualified_table_name, snapshots[0].get_snapshot_id())
+    # Rollback before DROP PARTITION
+    self.execute_query(rollback)
+    snapshots = get_snapshots(self.client, qualified_table_name, expected_result_size=3)
+    assert snapshots[0].get_snapshot_id() == snapshots[2].get_snapshot_id()
+    assert snapshots[0].get_parent_id() == snapshots[2].get_parent_id()
+    assert snapshots[0].get_creation_time() < snapshots[2].get_creation_time()
+
 
 class TestIcebergV2Table(IcebergTestSuite):
   """Tests related to Iceberg V2 tables."""
@@ -1185,8 +1441,14 @@ class TestIcebergV2Table(IcebergTestSuite):
     super(TestIcebergV2Table, cls).add_test_dimensions()
     cls.ImpalaTestMatrix.add_constraint(
       lambda v: v.get_value('table_format').file_format == 'parquet')
-    cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension(
-      'disable_optimized_iceberg_v2_read', 0, 1))
+    add_exec_option_dimension(cls, 'disable_optimized_iceberg_v2_read', [0, 1])
+
+  def should_run_for_hive(self, vector):
+    # Hive interop tests are very slow. Only run them for a subset of dimensions.
+    if vector.get_value('exec_option')['disable_optimized_iceberg_v2_read'] == 0:
+      return True
+    return False
+
   # The test uses pre-written Iceberg tables where the position delete files refer to
   # the data files via full URI, i.e. they start with 'hdfs://localhost:2050/...'. In the
   # dockerised environment the namenode is accessible on a different hostname/port.
@@ -1201,6 +1463,41 @@ class TestIcebergV2Table(IcebergTestSuite):
   def test_read_position_deletes(self, vector):
     self.run_test_case('QueryTest/iceberg-v2-read-position-deletes', vector)
 
+  @SkipIfFS.hive
+  def test_read_mixed_format_position_deletes(self, vector, unique_database):
+    self.run_test_case('QueryTest/iceberg-mixed-format-position-deletes',
+        vector, unique_database)
+
+  @SkipIfDockerizedCluster.internal_hostname
+  @SkipIf.hardcoded_uris
+  def test_read_null_delete_records(self, vector):
+    expected_error = 'NULL found as file_path in delete file'
+    query_options = vector.get_value('exec_option')
+    v2_op_disabled = query_options['disable_optimized_iceberg_v2_read'] == 1
+    result = self.execute_query(
+        'select * from functional_parquet.iceberg_v2_null_delete_record', query_options)
+    assert len(result.data) == 6
+    errors = result.log
+    print(errors)
+    assert expected_error in errors or v2_op_disabled
+    result = self.execute_query(
+        'select count(*) from functional_parquet.iceberg_v2_null_delete_record',
+        query_options)
+    assert result.data == ['6']
+    errors = result.log
+    assert expected_error in errors or v2_op_disabled
+    result = self.execute_query(
+        """select * from functional_parquet.iceberg_v2_null_delete_record
+           where j < 3""", query_options)
+    assert sorted(result.data) == ['1\t1', '2\t2']
+    errors = result.log
+    assert expected_error in errors or v2_op_disabled
+
+  @SkipIfDockerizedCluster.internal_hostname
+  @SkipIf.hardcoded_uris
+  def test_read_equality_deletes(self, vector):
+    self.run_test_case('QueryTest/iceberg-v2-read-equality-deletes', vector)
+
   @SkipIfDockerizedCluster.internal_hostname
   @SkipIf.hardcoded_uris
   def test_read_position_deletes_orc(self, vector):
@@ -1212,9 +1509,19 @@ class TestIcebergV2Table(IcebergTestSuite):
     self.run_test_case('QueryTest/iceberg-tablesample-v2', vector,
         use_db="functional_parquet")
 
-  def test_metadata_tables(self, vector):
-    self.run_test_case('QueryTest/iceberg-metadata-tables', vector,
-        use_db="functional_parquet")
+  @SkipIf.hardcoded_uris
+  def test_metadata_tables(self, vector, unique_database):
+    with self.create_impala_client() as impalad_client:
+      overwrite_snapshot_id = impalad_client.execute("select snapshot_id from "
+                             "functional_parquet.iceberg_query_metadata.snapshots "
+                             "where operation = 'overwrite';")
+      overwrite_snapshot_ts = impalad_client.execute("select committed_at from "
+                             "functional_parquet.iceberg_query_metadata.snapshots "
+                             "where operation = 'overwrite';")
+      self.run_test_case('QueryTest/iceberg-metadata-tables', vector,
+          unique_database,
+          test_file_vars={'$OVERWRITE_SNAPSHOT_ID': str(overwrite_snapshot_id.data[0]),
+                          '$OVERWRITE_SNAPSHOT_TS': str(overwrite_snapshot_ts.data[0])})
 
   def test_delete(self, vector, unique_database):
     self.run_test_case('QueryTest/iceberg-delete', vector,
@@ -1223,14 +1530,10 @@ class TestIcebergV2Table(IcebergTestSuite):
   def test_delete_partitioned(self, vector, unique_database):
     self.run_test_case('QueryTest/iceberg-delete-partitioned', vector,
         unique_database)
-    if IS_HDFS:
-      self._partitioned_hive_tests(unique_database)
+    if IS_HDFS and self.should_run_for_hive(vector):
+      self._delete_partitioned_hive_tests(unique_database)
 
-  def _partitioned_hive_tests(self, db):
-    # Hive needs table property 'format-version' explicitly set
-    for tbl in ["id_part", "trunc_part", "multi_part", "evolve_part", "ice_store_sales"]:
-      self.run_stmt_in_hive(
-          "ALTER TABLE {}.{} SET TBLPROPERTIES('format-version'='2')".format(db, tbl))
+  def _delete_partitioned_hive_tests(self, db):
     hive_output = self.run_stmt_in_hive("SELECT * FROM {}.{} ORDER BY i".format(
         db, "id_part"))
     assert hive_output == "id_part.i,id_part.s\n"
@@ -1248,6 +1551,13 @@ class TestIcebergV2Table(IcebergTestSuite):
     hive_output = self.run_stmt_in_hive("SELECT count(*) FROM {}.{}".format(
         db, "ice_store_sales"))
     assert hive_output == "_c0\n2601498\n"
+    hive_output = self.run_stmt_in_hive("SELECT * FROM {}.{}".format(
+        db, "ice_alltypes_part_v2"))
+    # Cut off the long header line.
+    hive_output = hive_output.split("\n", 1)
+    hive_output = hive_output[1]
+    assert hive_output == \
+        "2,true,1,11,1.1,2.222,123.321,2022-02-22,impala\n"
 
   @SkipIfFS.hive
   def test_delete_hive_read(self, vector, unique_database):
@@ -1301,3 +1611,201 @@ class TestIcebergV2Table(IcebergTestSuite):
     hive_output = self.run_stmt_in_hive("SELECT id FROM {} ORDER BY id".format(ice_t))
     # Test that Hive sees the same rows deleted.
     assert hive_output == "id\n4\n5\n6\n7\n8\n"
+
+  def test_update_basic(self, vector, unique_database):
+    self.run_test_case('QueryTest/iceberg-update-basic', vector,
+        unique_database)
+    self._test_update_basic_snapshots(unique_database)
+    if IS_HDFS and self.should_run_for_hive(vector):
+      self._update_basic_hive_tests(unique_database)
+
+  def _test_update_basic_snapshots(self, db):
+    """Verifies that the tables have the expected number of snapshots, and
+    the parent ids match the previous snapshot ids. See IMPALA-12708."""
+
+    self.validate_snapshots(db, "single_col", 3)
+    self.validate_snapshots(db, "ice_alltypes", 17)
+    self.validate_snapshots(db, "ice_id_partitioned", 4)
+
+  def validate_snapshots(self, db, tbl, expected_snapshots):
+    tbl_name = "{}.{}".format(db, tbl)
+    snapshots = get_snapshots(self.client, tbl_name,
+        expected_result_size=expected_snapshots)
+    parent_id = None
+    for s in snapshots:
+      assert s.get_parent_id() == parent_id
+      parent_id = s.get_snapshot_id()
+
+  def _update_basic_hive_tests(self, db):
+    def get_hive_results(tbl, order_by_col):
+      stmt = "SELECT * FROM {}.{} ORDER BY {}".format(db, tbl, order_by_col)
+      return self.run_stmt_in_hive(stmt).split("\n", 1)[1]
+
+    hive_results = get_hive_results("single_col", "i")
+    assert hive_results == "1\n3\n4\n"
+
+    hive_results = get_hive_results("ice_alltypes", "bool_col")
+    assert hive_results == \
+        "false,0,111,0.0,0.0,0,0.00,2023-11-07,2000-01-01 00:00:00.0,IMPALA,zerob\n" \
+        "true,3,222,1.0,1.0,23,123.12,2023-11-08,2001-01-01 01:01:01.0,ICEBERG,oneb\n"
+
+    hive_results = get_hive_results("ice_id_partitioned", "i")
+    assert hive_results == \
+        "1,0,Apache Impala\n"     \
+        "2,0,iceberg\n"    \
+        "3,0,hive\n"       \
+        "4,1,Apache Spark\n"      \
+        "5,2,Kudu\n"
+
+    hive_results = get_hive_results("ice_bucket_transform", "i")
+    assert hive_results == \
+        "2,a fairly long string value,1000,1999-09-19 12:00:01.0\n" \
+        "4,bbb,2030,2001-01-01 00:00:00.0\n" \
+        "6,cccccccccccccccccccccccccccccccccccccccc,-123,2023-11-24 17:44:30.0\n"
+
+    hive_results = get_hive_results("ice_time_transforms_timestamp", "id")
+    assert hive_results == \
+        "1.5000,2001-01-01 01:01:01.0,2001-01-01 01:01:01.0,2001-01-01 01:01:01.0,2001-01-01 01:01:01.0\n" \
+        "2.4690,2023-11-24 18:02:00.0,2023-11-24 18:02:00.0,2023-11-24 18:02:00.0,2023-11-24 18:02:00.0\n" \
+        "1999.9998,2199-12-31 23:59:59.0,2199-12-31 23:59:59.0,2199-12-31 23:59:59.0,2199-12-31 23:59:59.0\n"  # noqa: E501
+
+    hive_results = get_hive_results("ice_time_transforms_date", "id")
+    assert hive_results == \
+        "1.5000,2001-01-01,2001-01-01,2001-01-01\n" \
+        "2.4690,2023-11-24,2023-11-24,2023-11-24\n" \
+        "1999.9998,2199-12-31,2199-12-31,2199-12-31\n"
+
+    hive_results = get_hive_results("ice_part_transforms", "i")
+    assert hive_results == \
+        "1,2023-11-13 18:07:05.0,blue,1234\n" \
+        "3,2023-11-14 19:07:05.0,green,1700\n" \
+        "4,2023-11-13 18:07:23.0,gray,2500\n" \
+        "8,2023-11-01 00:11:11.0,black,722\n"
+
+  def test_update_partitions(self, vector, unique_database):
+    self.run_test_case('QueryTest/iceberg-update-partitions', vector,
+        unique_database)
+    if IS_HDFS and self.should_run_for_hive(vector):
+      self._update_partitions_hive_tests(unique_database)
+
+  def _update_partitions_hive_tests(self, db):
+    def get_hive_results(tbl, order_by_col):
+      stmt = "SELECT * FROM {}.{} ORDER BY {}".format(db, tbl, order_by_col)
+      return self.run_stmt_in_hive(stmt).split("\n", 1)[1]
+
+    hive_results = get_hive_results("id_part", "i, s")
+    assert hive_results == \
+        "2,FIVE\n" \
+        "2,FOUR\n" \
+        "3,SIX\n" \
+        "5,ONE\n" \
+        "10,TWO\n" \
+        "15,THREE\n"
+
+    hive_results = get_hive_results("trunc_part", "i")
+    assert hive_results == \
+        "1,one\n" \
+        "5,five\n" \
+        "103,three\n" \
+        "1004,FOURfour\n" \
+        "1006,SIXsix\n" \
+        "1102,TWOtwo\n"
+
+    hive_results = get_hive_results("multi_part", "i")
+    assert hive_results == \
+        "0,void,3.14\n" \
+        "0,void,3.14\n" \
+        "0,void,3.14\n" \
+        "1,one,1.1\n" \
+        "3,three,3.33\n" \
+        "5,five,5.5\n" \
+        "111,fox,1.1\n"
+
+    hive_results = get_hive_results("evolve_part", "i")
+    assert hive_results == \
+        "1,one,1.1\n" \
+        "30,thirty,30.3\n" \
+        "40,forty,40.4\n" \
+        "50,fifty,50.5\n" \
+        "1003,three,3.33\n" \
+        "1010,ten,10.1\n" \
+        "1020,twenty,20.2\n" \
+        "1222,two,2.2\n"
+
+    hive_results = get_hive_results("date_day_part", "i")
+    assert hive_results == \
+        "11,1978-01-01\n" \
+        "12,1979-12-31\n" \
+        "13,1980-01-01\n" \
+        "14,2033-11-15\n"
+
+    hive_results = get_hive_results("ts_hour_part", "i")
+    assert hive_results == \
+        "101,1958-01-01 01:02:03.0\n" \
+        "102,1959-12-31 23:59:00.0\n" \
+        "103,1960-01-01 00:00:00.0\n" \
+        "104,2013-11-15 15:31:00.0\n"
+
+    hive_results = get_hive_results("ts_evolve_part", "i")
+    assert hive_results == \
+        "1001,1988-02-02 01:02:03.0\n" \
+        "1002,1990-02-01 23:59:00.0\n" \
+        "1003,1990-02-02 00:00:00.0\n" \
+        "1004,2043-12-16 15:31:00.0\n" \
+        "1111,NULL\n"
+
+    hive_results = get_hive_results("numeric_truncate", "id")
+    assert hive_results == \
+        "11,21,2111,531,75.20\n"
+
+    # HIVE-28048: Hive cannot run ORDER BY queries for Iceberg tables partitioned by
+    # decimal columns, so we order the results ourselves.
+    hive_results = self.run_stmt_in_hive("SELECT * FROM {}.{}".format(
+        db, "ice_alltypes_part_v2"))
+    # Throw away the header line and sort the results.
+    hive_results = hive_results.split("\n", 1)[1]
+    hive_results = hive_results.strip().split("\n")
+    hive_results.sort()
+    assert hive_results == [
+        "2,true,2,11,1.1,2.222,123.321,2022-04-22,impala",
+        "3,true,3,11,1.1,2.222,123.321,2022-05-22,impala"]
+
+  def test_optimize(self, vector, unique_database):
+    self.run_test_case('QueryTest/iceberg-optimize', vector, unique_database)
+    expected_snapshots = 19
+    self.validate_snapshots(unique_database, "ice_optimize", expected_snapshots)
+
+    # The last operation was an OPTIMIZE TABLE statement.
+    # Check that time travel to the previous snapshot returns all results correctly.
+    tbl_name = unique_database + ".ice_optimize"
+    snapshots = get_snapshots(
+        self.client, tbl_name, expected_result_size=expected_snapshots)
+    snapshot_before_last = snapshots[-2]
+
+    result_after_opt = self.execute_query("SELECT * FROM {0}".format(tbl_name))
+    result_time_travel = self.execute_query(
+        "select * from {0} for system_version as of {1};".format(
+            tbl_name, snapshot_before_last.get_snapshot_id()))
+    assert result_after_opt.data.sort() == result_time_travel.data.sort()
+
+
+# Tests to exercise the DIRECTED distribution mode for V2 Iceberg tables. Note, that most
+# of the test coverage is in TestIcebergV2Table.test_read_position_deletes but since it
+# runs also with the V2 optimizations setting turned off, some tests were moved here.
+class TestIcebergDirectedMode(IcebergTestSuite):
+  """Tests related to Iceberg DIRECTED distribution mode."""
+
+  @classmethod
+  def get_workload(cls):
+    return 'functional-query'
+
+  @classmethod
+  def add_test_dimensions(cls):
+    super(TestIcebergDirectedMode, cls).add_test_dimensions()
+    cls.ImpalaTestMatrix.add_constraint(
+      lambda v: v.get_value('table_format').file_format == 'parquet')
+
+  @SkipIfDockerizedCluster.internal_hostname
+  @SkipIf.hardcoded_uris
+  def test_directed_mode(self, vector):
+      self.run_test_case('QueryTest/iceberg-v2-directed-mode', vector)
