@@ -19,6 +19,7 @@ from __future__ import absolute_import, division, print_function
 from collections import defaultdict
 from datetime import datetime
 from tests.beeswax.impala_beeswax import ImpalaBeeswaxException
+from tests.common.impala_cluster import ImpalaCluster
 from tests.common.impala_test_suite import ImpalaTestSuite
 from tests.common.skip import SkipIfFS, SkipIfLocal, SkipIfNotHdfsMinicluster
 from tests.util.filesystem_utils import IS_EC, WAREHOUSE
@@ -188,6 +189,20 @@ class TestObservability(ImpalaTestSuite):
     expected_str = expected_str.format(timezone=server_timezone)
     assert expected_str in profile, profile
 
+  def test_profile(self):
+    """Test that expected fields are populated in the profile."""
+    query = """select count(distinct a.int_col) from functional.alltypes a
+        inner join functional.alltypessmall b on (a.id = b.id + cast(sleep(15) as INT))"""
+    result = self.execute_query(query)
+
+    assert "Query Type: QUERY" in result.runtime_profile
+    assert "Query State: " in result.runtime_profile
+    assert "Default Db: default" in result.runtime_profile
+    tables = re.search(r'\n\s+Tables Queried:\s+(.*?)\n', result.runtime_profile)
+    assert tables is not None
+    assert sorted(tables.group(1).split(",")) \
+        == ["functional.alltypes", "functional.alltypessmall"]
+
   def test_exec_summary(self):
     """Test that the exec summary is populated correctly in every query state"""
     query = "select count(*) from functional.alltypes"
@@ -440,14 +455,34 @@ class TestObservability(ImpalaTestSuite):
       r'Query Timeline:',
       r'Planning finished'
     ]
-    # Events for DDLs. Currently just CreateTable has them.
-    ddl_event_regexes = [
+    # Events for CreateTable DDLs.
+    create_table_ddl_event_regexes = [
       r'Catalog Server Operation:',
       r'Got metastoreDdlLock:',
       r'Created table in Metastore:',
       r'Created table in catalog cache:',
       r'DDL finished:',
     ]
+    # Events for AlterTable DDLs.
+    alter_table_ddl_event_regexes = [
+      r'Catalog Server Operation:',
+      r'Got catalog version read lock:',
+      r'Got catalog version write lock and table write lock:',
+      r'Altered table in Metastore:',
+      r'Fetched table from Metastore:',
+      r'Loaded table schema:',
+      r'DDL finished:'
+    ]
+    # Events for LOAD statement.
+    load_data_ddl_event_regexes = [
+      r'Catalog Server Operation:',
+      r'Got catalog version read lock:',
+      r'Got catalog version write lock and table write lock:',
+      r'Fired Metastore events:',
+      r'Fetched table from Metastore:',
+      r'Loaded file metadata for 1 partitions:',
+      r'Finished updateCatalog request:'
+    ]
     # queries that explore different code paths in Frontend compilation
     queries = [
       'create table if not exists impala_6568 (i int)',
@@ -465,7 +500,13 @@ class TestObservability(ImpalaTestSuite):
       # Verify catalogOp timeline
       if query.startswith("create table"):
         self.__verify_profile_contains_every_event(
-            ddl_event_regexes, runtime_profile, query)
+            create_table_ddl_event_regexes, runtime_profile, query)
+      elif query.startswith("alter table"):
+        self.__verify_profile_contains_every_event(
+            alter_table_ddl_event_regexes, runtime_profile, query)
+      elif query.startswith("load data"):
+        self.__verify_profile_contains_every_event(
+            load_data_ddl_event_regexes, runtime_profile, query)
 
   def __verify_profile_contains_every_event(self, event_regexes, runtime_profile, query):
     """Test that all the expected events show up in a given query profile."""
@@ -475,14 +516,15 @@ class TestObservability(ImpalaTestSuite):
           "' in profile: \n" + runtime_profile
 
   def test_create_table_profile_events(self, unique_database):
+    """Test that specific DDL timeline event labels exist in the profile. Note that
+       labels of HMS events are not used so this test is expected to pass with or without
+       event processor enabled"""
     # Create normal table
-    stmt = "create table %s.t1(id int)" % unique_database
+    stmt = "create table %s.t1(id int) partitioned by (p int)" % unique_database
     self.__verify_event_labels_in_profile(stmt, [
         "Got metastoreDdlLock",
         "Got Metastore client",
-        "Got current Metastore event id",
         "Created table in Metastore",
-        "Fetched event batch from Metastore",
         "Created table in catalog cache",
         "DDL finished"
     ])
@@ -491,7 +533,6 @@ class TestObservability(ImpalaTestSuite):
            " partition by hash(id) partitions 3 stored as kudu" % unique_database
     self.__verify_event_labels_in_profile(stmt, [
         "Got Metastore client",
-        "Got current Metastore event id",
         "Got Kudu client",
         "Got kuduDdlLock",
         "Checked table existence in Kudu",
@@ -500,7 +541,6 @@ class TestObservability(ImpalaTestSuite):
         "Got Metastore client",
         "Checked table existence in Metastore",
         "Created table in Metastore",
-        "Fetched event batch from Metastore",
         "Created table in catalog cache",
         "DDL finished",
     ])
@@ -509,18 +549,50 @@ class TestObservability(ImpalaTestSuite):
     self.__verify_event_labels_in_profile(stmt, [
         "Got Metastore client",
         "Checked table existence in Metastore",
-        "Got current Metastore event id",
         "Created table using Iceberg Catalog HIVE_CATALOG",
-        "Fetched event batch from Metastore",
         "Created table in catalog cache",
-        "Set Iceberg table owner in Metastore",
         "DDL finished",
     ])
+    # INSERT into table
+    stmt = "insert into %s.t1 partition(p) values (0,0), (1,1)" % unique_database
+    self.__verify_event_labels_in_profile(stmt, [
+        "Got Metastore client",
+        "Added 2 partitions in Metastore",
+        "Loaded file metadata for 2 partitions",
+        "Finished updateCatalog request"
+    ])
+    # Compute stats
+    stmt = "compute stats %s.t1" % unique_database
+    self.__verify_event_labels_in_profile(stmt, [
+        "Got Metastore client",
+        "Updated column stats",
+        "Altered 2 partitions in Metastore",
+        "Altered table in Metastore",
+        "Fetched table from Metastore",
+        "Loaded file metadata for 2 partitions",
+        "DDL finished"
+    ])
+    # REFRESH
+    stmt = "refresh %s.t1" % unique_database
+    self.__verify_event_labels_in_profile(stmt, [
+      "Got Metastore client",
+      "Fetched table from Metastore",
+      "Loaded file metadata for 2 partitions",
+      "Finished resetMetadata request"
+    ])
+    # Drop table
+    stmt = "drop table %s.t1" % unique_database
+    self.__verify_event_labels_in_profile(stmt, [
+        "Got Metastore client",
+        "Dropped table in Metastore",
+        "Deleted table in catalog cache",
+        "DDL finished"
+    ])
 
   def __verify_event_labels_in_profile(self, stmt, event_labels):
     profile = self.execute_query(stmt).runtime_profile
     for label in event_labels:
-      assert label in profile
+      assert label in profile, "Missing '%s' in runtime profile:\n%s" % (label, profile)
 
   def test_compute_stats_profile(self, unique_database):
     """Test that the profile for a 'compute stats' query contains three unique query ids:
@@ -561,7 +633,8 @@ class TestObservability(ImpalaTestSuite):
     assert "  MemoryUsage" not in profile
     assert "- MemoryUsage" in profile
 
-    assert "ExchangeScanRatio: 4.63" in profile
+    # Usually 4.63. Ignoring last digit to deflake test (IMPALA-12500).
+    assert "ExchangeScanRatio: 4.6" in profile
 
     keys = ["TotalBytesSent", "TotalScanBytesSent", "TotalInnerBytesSent"]
     counters = defaultdict(int)
@@ -593,10 +666,10 @@ class TestObservability(ImpalaTestSuite):
     assert any(expected_str in line for line in profile.splitlines())
 
   def test_query_profile_host_resource_metrics_off(self):
-    """Tests that the query profile does not contain resource usage metrics by default or
+    """Tests that the query profile does not contain resource usage metrics
        when disabled explicitly."""
     query = "select count(*), sleep(1000) from functional.alltypes"
-    for query_opts in [None, {'resource_trace_ratio': 0.0}]:
+    for query_opts in [{'resource_trace_ratio': 0.0}]:
       profile = self.execute_query(query, query_opts).runtime_profile
       # Assert that no host resource counters exist in the profile
       for line in profile.splitlines():
@@ -605,23 +678,24 @@ class TestObservability(ImpalaTestSuite):
         assert not re.search("HostDiskReadThroughput", line)
 
   def test_query_profile_contains_host_resource_metrics(self):
-    """Tests that the query profile contains various CPU and network metrics."""
-    query_opts = {'resource_trace_ratio': 1.0}
+    """Tests that the query profile contains various CPU and network metrics
+       by default or when enabled explicitly."""
     query = "select count(*), sleep(1000) from functional.alltypes"
-    profile = self.execute_query(query, query_opts).runtime_profile
-    # We check for 500ms because a query with 1s duration won't hit the 64 values limit
-    # that would trigger resampling.
-    expected_strs = ["HostCpuIoWaitPercentage (500.000ms):",
-                     "HostCpuSysPercentage (500.000ms):",
-                     "HostCpuUserPercentage (500.000ms):",
-                     "HostNetworkRx (500.000ms):",
-                     "HostNetworkTx (500.000ms):",
-                     "HostDiskReadThroughput (500.000ms):",
-                     "HostDiskWriteThroughput (500.000ms):"]
+    for query_opts in [{}, {'resource_trace_ratio': 1.0}]:
+      profile = self.execute_query(query, query_opts).runtime_profile
+      # We check for 50ms because a query with 1s duration won't hit the 64 values limit
+      # that would trigger resampling.
+      expected_strs = ["HostCpuIoWaitPercentage (50.000ms):",
+                     "HostCpuSysPercentage (50.000ms):",
+                     "HostCpuUserPercentage (50.000ms):",
+                     "HostNetworkRx (50.000ms):",
+                     "HostNetworkTx (50.000ms):",
+                     "HostDiskReadThroughput (50.000ms):",
+                     "HostDiskWriteThroughput (50.000ms):"]
 
-    # Assert that all expected counters exist in the profile.
-    for expected_str in expected_strs:
-      assert any(expected_str in line for line in profile.splitlines()), expected_str
+      # Assert that all expected counters exist in the profile.
+      for expected_str in expected_strs:
+        assert any(expected_str in line for line in profile.splitlines()), expected_str
 
     # Check that there are some values for each counter.
     for line in profile.splitlines():
@@ -871,6 +945,50 @@ class TestObservability(ImpalaTestSuite):
     assert len(re.findall('Single node plan created:', runtime_profile, re.M)) == 2
     assert len(re.findall('Distributed plan created:', runtime_profile, re.M)) == 2
 
+  @SkipIfNotHdfsMinicluster.plans
+  def test_reduced_cardinality_by_filter(self):
+    """IMPALA-12702: Check that ExecSummary shows the reduced cardinality estimation."""
+    query_opts = {'compute_processing_cost': True}
+    query = """select STRAIGHT_JOIN count(*) from
+        (select l_orderkey from tpch_parquet.lineitem) a
+        join (select o_orderkey, o_custkey from tpch_parquet.orders) l1
+          on a.l_orderkey = l1.o_orderkey
+        where l1.o_custkey < 1000"""
+    result = self.execute_query(query, query_opts)
+    scan = result.exec_summary[10]
+    assert '00:SCAN' in scan['operator']
+    assert scan['num_rows'] == 39563
+    assert scan['est_num_rows'] == 575771
+    assert scan['detail'] == 'tpch_parquet.lineitem'
+    runtime_profile = result.runtime_profile
+    assert "cardinality=575.77K(filtered from 6.00M)" in runtime_profile
+
+  def test_query_profile_contains_get_inflight_profile_counter(self):
+    """Test that counter for getting inflight profiles appears in the profile"""
+    # This query runs 15s
+    query = "select count(*) from functional.alltypes where bool_col = sleep(50)"
+    handle = self.execute_query_async(query)
+    query_id = handle.get_handle().id
+
+    cluster = ImpalaCluster.get_e2e_test_cluster()
+    impalad = cluster.get_first_impalad()
+    profile_urls = [
+      "query_profile?query_id=",
+      "query_profile_encoded?query_id=",
+      "query_profile_json?query_id=",
+      "query_profile_plain_text?query_id=",
+    ]
+    for url in profile_urls:
+      impalad.service.read_debug_webpage(url + query_id)
+    profile = self.client.get_runtime_profile(handle)
+    assert "GetInFlightProfileTimeStats:" in profile
+    assert "ClientFetchLockWaitTimer:" in profile
+    # Make sure the counter actually records the requests
+    samples_search = re.search(r"GetInFlightProfileTimeStats:.*samples: (\d+)", profile)
+    num_samples = int(samples_search.group(1))
+    assert num_samples > 0
+
+
 class TestQueryStates(ImpalaTestSuite):
   """Test that the 'Query State' and 'Impala Query State' are set correctly in the
   runtime profile."""
