@@ -17,7 +17,7 @@
 #
 
 from __future__ import absolute_import, division, print_function
-from copy import deepcopy
+import math
 import os
 import pytest
 import re
@@ -27,14 +27,20 @@ from tests.common.environ import build_flavor_timeout, ImpalaTestClusterProperti
 from tests.common.impala_cluster import ImpalaCluster
 from tests.common.impala_test_suite import ImpalaTestSuite
 from tests.common.skip import SkipIfEC, SkipIfLocal, SkipIfFS
-from tests.common.test_dimensions import add_exec_option_dimension
-from tests.common.test_vector import ImpalaTestDimension
+from tests.common.test_dimensions import (
+  add_mandatory_exec_option, add_exec_option_dimension)
 from tests.verifiers.metric_verifier import MetricVerifier
 from tests.util.filesystem_utils import WAREHOUSE
 
 # slow_build_timeout is set to 200000 to avoid failures like IMPALA-8064 where the
 # runtime filters don't arrive in time.
 WAIT_TIME_MS = build_flavor_timeout(60000, slow_build_timeout=200000)
+DEFAULT_VARS = {'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)}
+
+# Dimensions exercised in some of the tests.
+DIM_MT_DOP_SMALL = [0, 1]
+DIM_MT_DOP_MEDIUM = [0, 4]
+DIM_MAX_NUM_FILTERS_AGGREGATED_PER_HOST = [0, 2]
 
 # Check whether the Impala under test in slow build. Query option ASYNC_CODEGEN will
 # be enabled when test runs for slow build like ASAN, TSAN, UBSAN, etc. This avoid
@@ -42,6 +48,7 @@ WAIT_TIME_MS = build_flavor_timeout(60000, slow_build_timeout=200000)
 # the slowness of codegen.
 build_runs_slowly = ImpalaTestClusterProperties.get_instance().runs_slowly()
 
+
 # Some of the queries in runtime_filters consume a lot of memory, leading to
 # significant memory reservations in parallel tests.
 # Skipping Isilon due to IMPALA-6998. TODO: Remove when there's a holistic revamp of
@@ -62,27 +69,37 @@ class TestRuntimeFilters(ImpalaTestSuite):
         lambda v: v.get_value('table_format').file_format not in ['hbase'])
     # Exercise both mt and non-mt code paths. Some tests assume 3 finstances, so
     # tests are not expected to work unmodified with higher mt_dop values.
-    cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension('mt_dop', 0, 1))
+    add_exec_option_dimension(cls, 'mt_dop', DIM_MT_DOP_SMALL)
+    # Exercise max_num_filters_aggregated_per_host.
+    add_exec_option_dimension(cls, 'max_num_filters_aggregated_per_host',
+      DIM_MAX_NUM_FILTERS_AGGREGATED_PER_HOST)
     # Don't test all combinations of file format and mt_dop, only test a few
     # representative formats.
     cls.ImpalaTestMatrix.add_constraint(
         lambda v: v.get_value('table_format').file_format in ['parquet', 'text', 'kudu']
         or v.get_value('mt_dop') == 0)
+    # Limit 'max_num_filters_aggregated_per_host' exercise for parquet only.
+    cls.ImpalaTestMatrix.add_constraint(
+      lambda v: v.get_value('table_format').file_format == 'parquet'
+      or v.get_value('max_num_filters_aggregated_per_host') == 0)
     # Enable query option ASYNC_CODEGEN for slow build
     if build_runs_slowly:
-      add_exec_option_dimension(cls, "async_codegen", 1)
+      add_mandatory_exec_option(cls, "async_codegen", 1)
 
   def test_basic_filters(self, vector):
-    new_vector = deepcopy(vector)
-    new_vector.get_value('exec_option')['mt_dop'] = vector.get_value('mt_dop')
-    self.run_test_case('QueryTest/runtime_filters', new_vector,
-        test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS' : str(WAIT_TIME_MS)})
+    num_filters_per_host = vector.get_exec_option('max_num_filters_aggregated_per_host')
+    num_backend = 2  # exclude coordinator
+    num_updates = (num_backend + 1 if num_filters_per_host == 0
+        else int(math.ceil(num_backend / num_filters_per_host)))
+    vars = {
+      '$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS),
+      '$NUM_FILTER_UPDATES': str(num_updates)
+    }
+    self.run_test_case('QueryTest/runtime_filters', vector, test_file_vars=vars)
 
   def test_wait_time(self, vector):
     """Test that a query that has global filters does not wait for them if run in LOCAL
     mode"""
-    new_vector = deepcopy(vector)
-    new_vector.get_value('exec_option')['mt_dop'] = vector.get_value('mt_dop')
     now = time.time()
     self.run_test_case('QueryTest/runtime_filters_wait', vector)
     duration_s = time.time() - now
@@ -94,8 +111,6 @@ class TestRuntimeFilters(ImpalaTestSuite):
   def test_wait_time_cancellation(self, vector):
     """Regression test for IMPALA-9065 to ensure that threads waiting for filters
     get woken up and exit promptly when the query is cancelled."""
-    new_vector = deepcopy(vector)
-    new_vector.get_value('exec_option')['mt_dop'] = vector.get_value('mt_dop')
     # Make sure the cluster is quiesced before we start this test
     self._verify_no_fragments_running()
 
@@ -106,7 +121,7 @@ class TestRuntimeFilters(ImpalaTestSuite):
     QUERY = """select straight_join *
                from alltypes t1
                     join /*+shuffle*/ alltypestiny t2 on t1.id = t2.id"""
-    self.client.set_configuration(new_vector.get_value('exec_option'))
+    self.client.set_configuration(vector.get_exec_option_dict())
     self.client.set_configuration_option("DEBUG_ACTION", "1:OPEN:WAIT")
     self.client.set_configuration_option("RUNTIME_FILTER_WAIT_TIME_MS", "10000000")
     # Run same query with different delays to better exercise call paths.
@@ -133,25 +148,21 @@ class TestRuntimeFilters(ImpalaTestSuite):
   def test_file_filtering(self, vector):
     if 'kudu' in str(vector.get_value('table_format')):
       return
-    new_vector = deepcopy(vector)
-    new_vector.get_value('exec_option')['mt_dop'] = vector.get_value('mt_dop')
     self.change_database(self.client, vector.get_value('table_format'))
     self.execute_query("SET RUNTIME_FILTER_MODE=GLOBAL")
     self.execute_query("SET RUNTIME_FILTER_WAIT_TIME_MS=10000")
     result = self.execute_query("""select STRAIGHT_JOIN * from alltypes inner join
                                 (select * from alltypessmall where smallint_col=-1) v
                                 on v.year = alltypes.year""",
-                                new_vector.get_value('exec_option'))
-    assert re.search("Files rejected: 8 \(8\)", result.runtime_profile) is not None
-    assert re.search("Splits rejected: [^0] \([^0]\)", result.runtime_profile) is None
+                                vector.get_exec_option_dict())
+    assert re.search(r"Files rejected: 8 \(8\)", result.runtime_profile) is not None
+    assert re.search(r"Splits rejected: [^0] \([^0]\)", result.runtime_profile) is None
 
   def test_file_filtering_late_arriving_filter(self, vector):
     """Test that late-arriving filters are applied to files when the scanner starts processing
     each scan range."""
     if 'kudu' in str(vector.get_value('table_format')):
       return
-    new_vector = deepcopy(vector)
-    new_vector.get_value('exec_option')['mt_dop'] = vector.get_value('mt_dop')
     self.change_database(self.client, vector.get_value('table_format'))
     self.execute_query("SET RUNTIME_FILTER_MODE=GLOBAL")
     self.execute_query("SET RUNTIME_FILTER_WAIT_TIME_MS=1")
@@ -166,8 +177,9 @@ class TestRuntimeFilters(ImpalaTestSuite):
                                       where smallint_col > sleep(100)) v
                                      on v.id = alltypes.id
                                    where alltypes.id < sleep(10);""",
-                                   new_vector.get_value('exec_option'))
-    assert re.search("Splits rejected: [^0] \([^0]\)", result.runtime_profile) is not None
+                                   vector.get_exec_option_dict())
+    splits_re = r"Splits rejected: [^0] \([^0]\)"
+    assert re.search(splits_re, result.runtime_profile) is not None
 
 
 @SkipIfLocal.multiple_impalad
@@ -179,38 +191,58 @@ class TestBloomFilters(ImpalaTestSuite):
   @classmethod
   def add_test_dimensions(cls):
     super(TestBloomFilters, cls).add_test_dimensions()
+    # Exercise max_num_filters_aggregated_per_host.
+    add_exec_option_dimension(cls, 'max_num_filters_aggregated_per_host',
+      DIM_MAX_NUM_FILTERS_AGGREGATED_PER_HOST)
     # Bloom filters are disabled on HBase
     cls.ImpalaTestMatrix.add_constraint(
         lambda v: v.get_value('table_format').file_format not in ['hbase'])
+    # Limit 'max_num_filters_aggregated_per_host' exercise for parquet only.
+    cls.ImpalaTestMatrix.add_constraint(
+      lambda v: v.get_value('table_format').file_format == 'parquet'
+      or v.get_value('max_num_filters_aggregated_per_host') == 0)
     # Enable query option ASYNC_CODEGEN for slow build
     if build_runs_slowly:
-      add_exec_option_dimension(cls, "async_codegen", 1)
+      add_mandatory_exec_option(cls, "async_codegen", 1)
 
   def test_bloom_filters(self, vector):
-    vector.get_value('exec_option')['ENABLED_RUNTIME_FILTER_TYPES'] = 'BLOOM'
     self.run_test_case('QueryTest/bloom_filters', vector)
 
+
+@SkipIfLocal.multiple_impalad
+class TestBloomFiltersOnParquet(ImpalaTestSuite):
+  @classmethod
+  def get_workload(cls):
+    return 'functional-query'
+
+  @classmethod
+  def add_test_dimensions(cls):
+    super(TestBloomFiltersOnParquet, cls).add_test_dimensions()
+    # Only enable bloom filter.
+    add_mandatory_exec_option(cls, 'enabled_runtime_filter_types', 'BLOOM')
+    # Exercise max_num_filters_aggregated_per_host.
+    add_exec_option_dimension(cls, 'max_num_filters_aggregated_per_host',
+      DIM_MAX_NUM_FILTERS_AGGREGATED_PER_HOST)
+    # Test exclusively on parquet
+    cls.ImpalaTestMatrix.add_constraint(
+        lambda v: v.get_value('table_format').file_format == 'parquet')
+    # Enable query option ASYNC_CODEGEN for slow build
+    if build_runs_slowly:
+      add_mandatory_exec_option(cls, "async_codegen", 1)
+
   def test_iceberg_dictionary_runtime_filter(self, vector, unique_database):
-    if (vector.get_value('table_format').file_format != 'parquet'):
-      pytest.skip()
-    vector.get_value('exec_option')['ENABLED_RUNTIME_FILTER_TYPES'] = 'BLOOM'
     self.run_test_case('QueryTest/iceberg-dictionary-runtime-filter', vector,
-      unique_database, test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
+      unique_database, test_file_vars=DEFAULT_VARS)
 
   def test_parquet_dictionary_runtime_filter(self, vector, unique_database):
-    if (vector.get_value('table_format').file_format != 'parquet'):
-      pytest.skip()
-    vector.get_value('exec_option')['ENABLED_RUNTIME_FILTER_TYPES'] = 'BLOOM'
-    vector.get_value('exec_option')['PARQUET_READ_STATISTICS'] = 'false'
+    vector.set_exec_option('PARQUET_READ_STATISTICS', 'false')
     self.run_test_case('QueryTest/parquet-dictionary-runtime-filter', vector,
-      unique_database, test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
+      unique_database, test_file_vars=DEFAULT_VARS)
 
   def test_iceberg_partition_runtime_filter(self, vector, unique_database):
-    if (vector.get_value('table_format').file_format != 'parquet'):
-      pytest.skip()
-    vector.get_value('exec_option')['ENABLED_RUNTIME_FILTER_TYPES'] = 'BLOOM'
     self.run_test_case('QueryTest/iceberg-partition-runtime-filter', vector,
-      unique_database, test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
+      unique_database, test_file_vars=DEFAULT_VARS)
+
 
 @SkipIfLocal.multiple_impalad
 class TestMinMaxFilters(ImpalaTestSuite):
@@ -226,22 +258,21 @@ class TestMinMaxFilters(ImpalaTestSuite):
         lambda v: v.get_value('table_format').file_format in ['kudu'])
     # Enable query option ASYNC_CODEGEN for slow build
     if build_runs_slowly:
-      add_exec_option_dimension(cls, "async_codegen", 1)
+      add_mandatory_exec_option(cls, "async_codegen", 1)
     # IMPALA-10715. Enable only min/max since the bloom filters will return
     # rows only satisfying the join predicates. This test requires the return
     # of non-qualifying rows to succeed.
-    add_exec_option_dimension(cls, "ENABLED_RUNTIME_FILTER_TYPES", "MIN_MAX")
+    add_mandatory_exec_option(cls, "ENABLED_RUNTIME_FILTER_TYPES", "MIN_MAX")
 
   def test_min_max_filters(self, vector):
     self.execute_query("SET MINMAX_FILTER_THRESHOLD=0.5")
-    self.run_test_case('QueryTest/min_max_filters', vector,
-        test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
+    self.run_test_case('QueryTest/min_max_filters', vector, test_file_vars=DEFAULT_VARS)
 
   def test_decimal_min_max_filters(self, vector):
     if self.exploration_strategy() != 'exhaustive':
       pytest.skip("skip decimal min max filter test with various joins")
     self.run_test_case('QueryTest/decimal_min_max_filters', vector,
-        test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
+                       test_file_vars=DEFAULT_VARS)
 
   def test_large_strings(self, cursor, unique_database):
     """Tests that truncation of large strings by min-max filters still gives correct
@@ -252,7 +283,7 @@ class TestMinMaxFilters(ImpalaTestSuite):
     # Min-max bounds are truncated at 1024 characters, so construct some strings that are
     # longer than that, as well as some that are very close to the min/max bounds.
     matching_vals =\
-        ('b' * 1100, 'b' * 1099 + 'c', 'd' * 1100, 'f'* 1099 + 'e', 'f' * 1100)
+        ('b' * 1100, 'b' * 1099 + 'c', 'd' * 1100, 'f' * 1099 + 'e', 'f' * 1100)
     cursor.execute("insert into %s values ('%s'), ('%s'), ('%s'), ('%s'), ('%s')"
         % ((table1,) + matching_vals))
     non_matching_vals = ('b' * 1099 + 'a', 'c', 'f' * 1099 + 'g')
@@ -300,10 +331,10 @@ class TestOverlapMinMaxFilters(ImpalaTestSuite):
     super(TestOverlapMinMaxFilters, cls).add_test_dimensions()
     # Overlap min-max filters are only implemented for parquet.
     cls.ImpalaTestMatrix.add_constraint(
-        lambda v: v.get_value('table_format').file_format in ['parquet'])
+        lambda v: v.get_value('table_format').file_format == 'parquet')
     # Enable query option ASYNC_CODEGEN for slow build
     if build_runs_slowly:
-      add_exec_option_dimension(cls, "async_codegen", 1)
+      add_mandatory_exec_option(cls, "async_codegen", 1)
 
   def test_overlap_min_max_filters(self, vector, unique_database):
     self.execute_query("SET MINMAX_FILTER_THRESHOLD=0.5")
@@ -311,18 +342,16 @@ class TestOverlapMinMaxFilters(ImpalaTestSuite):
     # on sorted columns (by default).
     self.execute_query("SET MINMAX_FILTER_PARTITION_COLUMNS=false")
     self.run_test_case('QueryTest/overlap_min_max_filters', vector, unique_database,
-        test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
+                       test_file_vars=DEFAULT_VARS)
     self.execute_query("SET MINMAX_FILTER_PARTITION_COLUMNS=true")
 
   def test_overlap_min_max_filters_on_sorted_columns(self, vector, unique_database):
     self.run_test_case('QueryTest/overlap_min_max_filters_on_sorted_columns', vector,
-                       unique_database,
-        test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
+                       unique_database, test_file_vars=DEFAULT_VARS)
 
   def test_overlap_min_max_filters_on_partition_columns(self, vector, unique_database):
     self.run_test_case('QueryTest/overlap_min_max_filters_on_partition_columns', vector,
-                       unique_database,
-        test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
+                       unique_database, test_file_vars=DEFAULT_VARS)
 
   @SkipIfLocal.hdfs_client
   def test_partition_column_in_parquet_data_file(self, vector, unique_database):
@@ -332,9 +361,10 @@ class TestOverlapMinMaxFilters(ImpalaTestSuite):
     self.execute_query("CREATE TABLE {0}.{1} (i INT) PARTITIONED BY (d DATE) "
                        "STORED AS PARQUET".format(unique_database, tbl_name))
     tbl_loc = "%s/%s/%s/d=2022-02-22/" % (WAREHOUSE, unique_database, tbl_name)
+    src_loc = (os.environ['IMPALA_HOME']
+               + '/testdata/data/partition_col_in_parquet.parquet')
     self.filesystem_client.make_dir(tbl_loc)
-    self.filesystem_client.copy_from_local(os.environ['IMPALA_HOME'] +
-        '/testdata/data/partition_col_in_parquet.parquet', tbl_loc)
+    self.filesystem_client.copy_from_local(src_loc, tbl_loc)
     self.execute_query("ALTER TABLE {0}.{1} RECOVER PARTITIONS".format(
         unique_database, tbl_name))
     self.execute_query("SET PARQUET_FALLBACK_SCHEMA_RESOLUTION=NAME")
@@ -356,11 +386,11 @@ class TestInListFilters(ImpalaTestSuite):
         lambda v: v.get_value('table_format').file_format in ['orc'])
     # Enable query option ASYNC_CODEGEN for slow build
     if build_runs_slowly:
-      add_exec_option_dimension(cls, "async_codegen", 1)
+      add_mandatory_exec_option(cls, "async_codegen", 1)
 
   def test_in_list_filters(self, vector):
-    vector.get_value('exec_option')['enabled_runtime_filter_types'] = 'in_list'
-    vector.get_value('exec_option')['runtime_filter_wait_time_ms'] = WAIT_TIME_MS
+    vector.set_exec_option('enabled_runtime_filter_types', 'in_list')
+    vector.set_exec_option('runtime_filter_wait_time_ms', WAIT_TIME_MS)
     self.run_test_case('QueryTest/in_list_filters', vector)
 
 
@@ -378,7 +408,7 @@ class TestAllRuntimeFilters(ImpalaTestSuite):
       lambda v: v.get_value('table_format').file_format in ['kudu'])
     # Enable query option ASYNC_CODEGEN for slow build
     if build_runs_slowly:
-      add_exec_option_dimension(cls, "async_codegen", 1)
+      add_mandatory_exec_option(cls, "async_codegen", 1)
 
   def test_all_runtime_filters(self, vector):
     self.execute_query("SET ENABLED_RUNTIME_FILTER_TYPES=ALL")
@@ -386,12 +416,12 @@ class TestAllRuntimeFilters(ImpalaTestSuite):
     self.execute_query("SET MINMAX_FILTER_SORTED_COLUMNS=false")
     self.execute_query("SET MINMAX_FILTER_PARTITION_COLUMNS=false")
     self.run_test_case('QueryTest/all_runtime_filters', vector,
-                       test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
+                       test_file_vars=DEFAULT_VARS)
 
   def test_diff_runtime_filter_types(self, vector):
     # compare number of probe rows when apply different types of runtime filter
     self.run_test_case('QueryTest/diff_runtime_filter_types', vector,
-                       test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
+                       test_file_vars=DEFAULT_VARS)
 
 
 @SkipIfLocal.multiple_impalad
@@ -407,27 +437,85 @@ class TestRuntimeRowFilters(ImpalaTestSuite):
         v.get_value('table_format').file_format in ['parquet'])
     # Exercise both mt and non-mt code paths. Some tests assume 3 finstances, so
     # tests are not expected to work unmodified with higher mt_dop values.
-    cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension('mt_dop', 0, 4))
+    add_exec_option_dimension(cls, 'mt_dop', DIM_MT_DOP_MEDIUM)
+    # Exercise max_num_filters_aggregated_per_host.
+    add_exec_option_dimension(cls, 'max_num_filters_aggregated_per_host',
+      DIM_MAX_NUM_FILTERS_AGGREGATED_PER_HOST)
     # Enable query option ASYNC_CODEGEN for slow build
     if build_runs_slowly:
-      add_exec_option_dimension(cls, "async_codegen", 1)
+      add_mandatory_exec_option(cls, "async_codegen", 1)
 
   def test_row_filters(self, vector):
-    new_vector = deepcopy(vector)
-    new_vector.get_value('exec_option')['mt_dop'] = vector.get_value('mt_dop')
     # Disable generating min/max filters for sorted columns
     self.execute_query("SET MINMAX_FILTER_SORTED_COLUMNS=false")
     self.execute_query("SET MINMAX_FILTER_PARTITION_COLUMNS=false")
     self.execute_query("SET PARQUET_DICTIONARY_RUNTIME_FILTER_ENTRY_LIMIT=0")
-    self.run_test_case('QueryTest/runtime_row_filters', new_vector,
-                       test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS)})
+    num_filters_per_host = vector.get_exec_option('max_num_filters_aggregated_per_host')
+    num_backend = 2  # exclude coordinator
+    num_updates = (num_backend + 1 if num_filters_per_host == 0
+        else int(math.ceil(num_backend / num_filters_per_host)))
+    vars = {
+      '$RUNTIME_FILTER_WAIT_TIME_MS': str(WAIT_TIME_MS),
+      '$NUM_FILTER_UPDATES': str(num_updates)
+    }
+    self.run_test_case('QueryTest/runtime_row_filters', vector, test_file_vars=vars)
+
+
+@SkipIfLocal.multiple_impalad
+class TestRuntimeRowFilterReservation(ImpalaTestSuite):
+  @classmethod
+  def get_workload(cls):
+    return 'functional-query'
+
+  @classmethod
+  def add_test_dimensions(cls):
+    super(TestRuntimeRowFilterReservation, cls).add_test_dimensions()
+    cls.ImpalaTestMatrix.add_constraint(
+      lambda v: v.get_value('table_format').file_format == 'parquet')
+    # Enable query option ASYNC_CODEGEN for slow build
+    if build_runs_slowly:
+      add_mandatory_exec_option(cls, "async_codegen", 1)
 
   def test_row_filter_reservation(self, vector):
     """Test handling of runtime filter memory reservations. Tuned for mt_dop=0."""
-    mt_dop = vector.get_value('mt_dop')
-    if mt_dop != 0:
-        pytest.skip("Memory reservations tuned for mt_dop=0")
-    new_vector = deepcopy(vector)
-    new_vector.get_value('exec_option')['mt_dop'] = mt_dop
-    self.run_test_case('QueryTest/runtime_row_filter_reservations', new_vector,
-                       test_file_vars={'$RUNTIME_FILTER_WAIT_TIME_MS' : str(WAIT_TIME_MS)})
+    self.run_test_case('QueryTest/runtime_row_filter_reservations', vector,
+                       test_file_vars=DEFAULT_VARS)
+
+
+class TestRuntimeFiltersLateRemoteUpdate(ImpalaTestSuite):
+  """Test that distributed runtime filter aggregation still works
+  when remote filter update is late to reach the intermediate aggregator."""
+
+  @classmethod
+  def get_workload(cls):
+    return 'functional-query'
+
+  @classmethod
+  def add_test_dimensions(cls):
+    super(TestRuntimeFiltersLateRemoteUpdate, cls).add_test_dimensions()
+    # Only run this test in parquet
+    cls.ImpalaTestMatrix.add_constraint(
+        lambda v: v.get_value('table_format').file_format in ['parquet'])
+    # Set mandatory query options.
+    add_mandatory_exec_option(cls, 'max_num_filters_aggregated_per_host', 2)
+    add_mandatory_exec_option(cls, 'runtime_filter_wait_time_ms', WAIT_TIME_MS // 10)
+    add_mandatory_exec_option(cls, 'debug_action',
+        'REMOTE_FILTER_UPDATE_DELAY:SLEEP@%d' % WAIT_TIME_MS)
+    # Enable query option ASYNC_CODEGEN for slow build
+    if build_runs_slowly:
+      add_mandatory_exec_option(cls, "async_codegen", 1)
+
+  def test_late_remote_update(self, vector):
+    """Test that a query with global filters does not wait for late remote filter
+    update."""
+    query = ('select count(*) from functional.alltypes p '
+             'join [SHUFFLE] functional.alltypestiny b '
+             'on p.month = b.int_col and b.month = 1 and b.string_col = "1"')
+    now = time.time()
+    exec_options = vector.get_value('exec_option')
+    result = self.execute_query_expect_success(self.client, query, exec_options)
+    assert result.data[0] == '620'
+    duration_s = time.time() - now
+    assert duration_s < (WAIT_TIME_MS / 1000), \
+        "Query took too long (%ss, possibly waiting for late filters?)" \
+        % str(duration_s)
