@@ -42,6 +42,8 @@ from tests.common.skip import (
     SkipIfLocal,
     SkipIfNotHdfsMinicluster)
 from tests.common.test_dimensions import (
+    add_exec_option_dimension,
+    add_mandatory_exec_option,
     create_single_exec_option_dimension,
     create_exec_option_dimension,
     create_uncompressed_text_dimension)
@@ -67,9 +69,10 @@ DEBUG_ACTION_DIMS = [None,
 DEBUG_ACTION_DIMS.append('HDFS_SCANNER_THREAD_CHECK_SOFT_MEM_LIMIT:FAIL@0.5')
 
 MT_DOP_VALUES = [0, 1, 4]
+BATCH_SIZES = [0, 1, 16]
+
 
 class TestScannersAllTableFormats(ImpalaTestSuite):
-  BATCH_SIZES = [0, 1, 16]
 
   @classmethod
   def get_workload(cls):
@@ -82,30 +85,34 @@ class TestScannersAllTableFormats(ImpalaTestSuite):
       # The purpose of this test is to get some base coverage of all the file formats.
       # Even in 'core', we'll test each format by using the pairwise strategy.
       cls.ImpalaTestMatrix.add_dimension(cls.create_table_info_dimension('pairwise'))
-    cls.ImpalaTestMatrix.add_dimension(
-        ImpalaTestDimension('batch_size', *TestScannersAllTableFormats.BATCH_SIZES))
-    cls.ImpalaTestMatrix.add_dimension(
-        ImpalaTestDimension('debug_action', *DEBUG_ACTION_DIMS))
-    cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension('mt_dop', *MT_DOP_VALUES))
+    cls.ImpalaTestMatrix.add_dimension(create_exec_option_dimension(
+        batch_sizes=BATCH_SIZES, debug_action_options=DEBUG_ACTION_DIMS))
+    add_exec_option_dimension(cls, 'mt_dop', MT_DOP_VALUES)
 
   def test_scanners(self, vector):
-    new_vector = deepcopy(vector)
-    # Copy over test dimensions to the matching query options.
-    new_vector.get_value('exec_option')['batch_size'] = vector.get_value('batch_size')
-    new_vector.get_value('exec_option')['debug_action'] = vector.get_value('debug_action')
-    new_vector.get_value('exec_option')['mt_dop'] = vector.get_value('mt_dop')
-    self.run_test_case('QueryTest/scanners', new_vector)
+    self.run_test_case('QueryTest/scanners', vector)
 
   def test_many_nulls(self, vector):
     if vector.get_value('table_format').file_format == 'hbase':
       # manynulls table not loaded for HBase
       pytest.skip()
     # Copy over test dimensions to the matching query options.
-    new_vector = deepcopy(vector)
-    new_vector.get_value('exec_option')['batch_size'] = vector.get_value('batch_size')
-    new_vector.get_value('exec_option')['debug_action'] = vector.get_value('debug_action')
-    new_vector.get_value('exec_option')['mt_dop'] = vector.get_value('mt_dop')
-    self.run_test_case('QueryTest/scanners-many-nulls', new_vector)
+    self.run_test_case('QueryTest/scanners-many-nulls', vector)
+
+
+class TestScannersAllTableFormatsDefaultOptions(ImpalaTestSuite):
+
+  @classmethod
+  def get_workload(cls):
+    return 'functional-query'
+
+  @classmethod
+  def add_test_dimensions(cls):
+    super(TestScannersAllTableFormatsDefaultOptions, cls).add_test_dimensions()
+    if cls.exploration_strategy() == 'core':
+      # The purpose of this test is to get some base coverage of all the file formats.
+      # Even in 'core', we'll test each format by using the pairwise strategy.
+      cls.ImpalaTestMatrix.add_dimension(cls.create_table_info_dimension('pairwise'))
 
   def test_hdfs_scanner_profile(self, vector):
     if vector.get_value('table_format').file_format in ('kudu', 'hbase') or \
@@ -123,7 +130,6 @@ class TestScannersAllTableFormats(ImpalaTestSuite):
 
 
 class TestScannersVirtualColumns(ImpalaTestSuite):
-  BATCH_SIZES = [0, 1, 16]
 
   @classmethod
   def get_workload(cls):
@@ -136,8 +142,8 @@ class TestScannersVirtualColumns(ImpalaTestSuite):
       # The purpose of this test is to get some base coverage of all the file formats.
       # Even in 'core', we'll test each format by using the pairwise strategy.
       cls.ImpalaTestMatrix.add_dimension(cls.create_table_info_dimension('pairwise'))
-    cls.ImpalaTestMatrix.add_dimension(
-        ImpalaTestDimension('batch_size', *TestScannersAllTableFormats.BATCH_SIZES))
+    cls.ImpalaTestMatrix.add_dimension(create_exec_option_dimension(
+        batch_sizes=BATCH_SIZES))
 
   def test_virtual_column_input_file_name(self, vector, unique_database):
     file_format = vector.get_value('table_format').file_format
@@ -169,8 +175,24 @@ class TestScannersVirtualColumns(ImpalaTestSuite):
     self.run_test_case('QueryTest/mixing-virtual-columns', vector, unique_database)
 
 
+class TestScannersVirtualColumnsNegative(ImpalaTestSuite):
+  @classmethod
+  def get_workload(cls):
+    return 'functional-query'
+
+  @classmethod
+  def add_test_dimensions(cls):
+    super(TestScannersVirtualColumnsNegative, cls).add_test_dimensions()
+    # In the tests we explicitly refer to the databases, i.e. no need to
+    # run this test with multiple file formats.
+    cls.ImpalaTestMatrix.add_dimension(
+        create_uncompressed_text_dimension(cls.get_workload()))
+
+  def test_virtual_column_file_position_negative(self, vector):
+    self.run_test_case('QueryTest/virtual-column-file-position-negative', vector)
+
+
 class TestIcebergVirtualColumns(ImpalaTestSuite):
-  BATCH_SIZES = [0, 1, 16]
 
   @classmethod
   def get_workload(cls):
@@ -181,10 +203,10 @@ class TestIcebergVirtualColumns(ImpalaTestSuite):
     super(TestIcebergVirtualColumns, cls).add_test_dimensions()
     if cls.exploration_strategy() == 'core':
       cls.ImpalaTestMatrix.add_dimension(cls.create_table_info_dimension('pairwise'))
-    cls.ImpalaTestMatrix.add_dimension(
-        ImpalaTestDimension('batch_size', *TestScannersAllTableFormats.BATCH_SIZES))
+    cls.ImpalaTestMatrix.add_dimension(create_exec_option_dimension(
+        batch_sizes=BATCH_SIZES))
     cls.ImpalaTestMatrix.add_constraint(
-      lambda v: v.get_value('table_format').file_format == 'parquet')
+        lambda v: v.get_value('table_format').file_format == 'parquet')
 
   def test_partition_columns(self, vector):
     """Tests partition-level Iceberg-only virtual columns."""
@@ -201,7 +223,8 @@ class TestScannersAllTableFormatsWithLimit(ImpalaTestSuite):
   @classmethod
   def add_test_dimensions(cls):
     super(TestScannersAllTableFormatsWithLimit, cls).add_test_dimensions()
-    cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension('mt_dop', *MT_DOP_VALUES))
+    add_exec_option_dimension(cls, 'mt_dop', MT_DOP_VALUES)
+    add_mandatory_exec_option(cls, 'batch_size', 100)
 
   def test_limit(self, vector):
     vector.get_value('exec_option')['abort_on_error'] = 1
@@ -213,7 +236,6 @@ class TestScannersAllTableFormatsWithLimit(ImpalaTestSuite):
 
   def _test_limit(self, vector):
     # Use a small batch size so changing the limit affects the timing of cancellation
-    vector.get_value('exec_option')['batch_size'] = 100
     iterations = 50
     query_template = "select * from alltypes limit %s"
     for i in range(1, iterations):
@@ -226,8 +248,8 @@ class TestScannersAllTableFormatsWithLimit(ImpalaTestSuite):
       # IMPALA-3337: The error log should be empty.
       assert not result.log
 
+
 class TestScannersMixedTableFormats(ImpalaTestSuite):
-  BATCH_SIZES = [0, 1, 16]
 
   @classmethod
   def get_workload(cls):
@@ -239,17 +261,13 @@ class TestScannersMixedTableFormats(ImpalaTestSuite):
     # Only run with a single dimension format, since the table includes mixed formats.
     cls.ImpalaTestMatrix.add_dimension(
         create_uncompressed_text_dimension(cls.get_workload()))
-    cls.ImpalaTestMatrix.add_dimension(
-        ImpalaTestDimension('batch_size', *TestScannersAllTableFormats.BATCH_SIZES))
-    cls.ImpalaTestMatrix.add_dimension(
-        ImpalaTestDimension('debug_action', *DEBUG_ACTION_DIMS))
-    cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension('mt_dop', *MT_DOP_VALUES))
+    cls.ImpalaTestMatrix.add_dimension(create_exec_option_dimension(
+        batch_sizes=BATCH_SIZES, debug_action_options=DEBUG_ACTION_DIMS))
+    add_exec_option_dimension(cls, 'mt_dop', MT_DOP_VALUES)
 
   def test_mixed_format(self, vector):
-    new_vector = deepcopy(vector)
-    new_vector.get_value('exec_option')['batch_size'] = vector.get_value('batch_size')
-    new_vector.get_value('exec_option')['debug_action'] = vector.get_value('debug_action')
-    self.run_test_case('QueryTest/mixed-format', new_vector)
+    self.run_test_case('QueryTest/mixed-format', vector)
+
 
 # Test case to verify the scanners work properly when the table metadata (specifically the
 # number of columns in the table) does not match the number of columns in the data file.
@@ -321,17 +339,9 @@ class TestWideRow(ImpalaTestSuite):
     super(TestWideRow, cls).add_test_dimensions()
     cls.ImpalaTestMatrix.add_dimension(
         create_exec_option_dimension(debug_action_options=DEBUG_ACTION_DIMS))
-    # I can't figure out how to load a huge row into hbase
-    cls.ImpalaTestMatrix.add_constraint(
-      lambda v: v.get_value('table_format').file_format != 'hbase')
 
-  def test_wide_row(self, vector):
-    if vector.get_value('table_format').file_format == 'kudu':
-      pytest.xfail("KUDU-666: Kudu support for large values")
-
-    new_vector = deepcopy(vector)
     # Use a 5MB scan range, so we will have to perform 5MB of sync reads
-    new_vector.get_value('exec_option')['max_scan_range_length'] = 5 * 1024 * 1024
+    add_mandatory_exec_option(cls, 'max_scan_range_length', 5 * 1024 * 1024)
     # We need > 10 MB of memory because we're creating extra buffers:
     # - 10 MB table / 5 MB scan range = 2 scan ranges, each of which may allocate ~20MB
     # - Sync reads will allocate ~5MB of space
@@ -340,10 +350,19 @@ class TestWideRow(ImpalaTestSuite):
     # The 132MB value used here was determined empirically by raising the limit until the
     # query succeeded for all file formats -- I don't know exactly why we need this much.
     # TODO: figure out exact breakdown of memory usage (IMPALA-681)
-    new_vector.get_value('exec_option')['mem_limit'] = 132 * 1024 * 1024
+    add_mandatory_exec_option(cls, 'mem_limit', 132 * 1024 * 1024)
     # Specify that the query should able to handle 10 MB MAX_ROW_SIZE.
-    new_vector.get_value('exec_option')['max_row_size'] = 10 * 1024 * 1024
-    self.run_test_case('QueryTest/wide-row', new_vector)
+    add_mandatory_exec_option(cls, 'max_row_size', 10 * 1024 * 1024)
+
+    # I can't figure out how to load a huge row into hbase
+    cls.ImpalaTestMatrix.add_constraint(
+        lambda v: v.get_value('table_format').file_format != 'hbase')
+
+  def test_wide_row(self, vector):
+    if vector.get_value('table_format').file_format == 'kudu':
+      pytest.xfail("KUDU-666: Kudu support for large values")
+    self.run_test_case('QueryTest/wide-row', vector)
+
 
 class TestWideTable(ImpalaTestSuite):
   # TODO: expand this to more rows when we have the capability
@@ -616,7 +635,8 @@ class TestParquet(ImpalaTestSuite):
        repetetion level is set to REPEATED succeeds without errors."""
     create_table_from_parquet(self.client, unique_database, "repeated_root_schema")
 
-    result = self.client.execute("select * from %s.repeated_root_schema" % unique_database)
+    result = self.client.execute(
+        "select * from %s.repeated_root_schema" % unique_database)
     assert len(result.data) == 300
 
   def test_huge_num_rows(self, vector, unique_database):
@@ -891,13 +911,13 @@ class TestParquet(ImpalaTestSuite):
     num_row_groups_list = re.findall('NumRowGroups: ([0-9]*)', result.runtime_profile)
     scan_ranges_complete_list = re.findall(
         'ScanRangesComplete: ([0-9]*)', result.runtime_profile)
-    num_rows_read_list = re.findall('RowsRead: [0-9.K]* \(([0-9]*)\)',
+    num_rows_read_list = re.findall(r'RowsRead: [0-9.K]* \(([0-9]*)\)',
         result.runtime_profile)
 
     REGEX_UNIT_SECOND = "[0-9]*[s]*[0-9]*[.]*[0-9]*[nm]*[s]*"
     REGEX_MIN_MAX_FOOTER_PROCESSING_TIME = \
-        ("FooterProcessingTime: \(Avg: %s ; \(Min: (%s) ; Max: (%s) ; "
-            "Number of samples: %s\)" % (REGEX_UNIT_SECOND, REGEX_UNIT_SECOND,
+        (r"FooterProcessingTime: \(Avg: %s ; \(Min: (%s) ; Max: (%s) ; "
+         r"Number of samples: %s\)" % (REGEX_UNIT_SECOND, REGEX_UNIT_SECOND,
             REGEX_UNIT_SECOND, "[0-9]*"))
     footer_processing_time_list = re.findall(
         REGEX_MIN_MAX_FOOTER_PROCESSING_TIME, result.runtime_profile)
@@ -985,10 +1005,10 @@ class TestParquet(ImpalaTestSuite):
     # Check that the schema does not use the UTF8 annotation except for CHAR and VARCHAR
     # columns
     a_schema_elt, b_schema_elt, c_schema_elt, d_schema_elt = get_schema_elements()
-    assert a_schema_elt.converted_type == None
+    assert a_schema_elt.converted_type is None
     assert b_schema_elt.converted_type == ConvertedType.UTF8
     assert c_schema_elt.converted_type == ConvertedType.UTF8
-    assert d_schema_elt.converted_type == None
+    assert d_schema_elt.converted_type is None
 
   def test_resolution_by_name(self, vector, unique_database):
     self.run_test_case('QueryTest/parquet-resolution-by-name', vector,
@@ -1005,8 +1025,10 @@ class TestParquet(ImpalaTestSuite):
 
     create_table_from_parquet(self.client, unique_database, 'decimal_stored_as_int32')
     create_table_from_parquet(self.client, unique_database, 'decimal_stored_as_int64')
-    create_table_from_parquet(self.client, unique_database, 'decimal_padded_fixed_len_byte_array')
-    create_table_from_parquet(self.client, unique_database, 'decimal_padded_fixed_len_byte_array2')
+    create_table_from_parquet(self.client, unique_database,
+        'decimal_padded_fixed_len_byte_array')
+    create_table_from_parquet(self.client, unique_database,
+        'decimal_padded_fixed_len_byte_array2')
 
     self.run_test_case('QueryTest/parquet-decimal-formats', vector, unique_database)
 
@@ -1268,6 +1290,42 @@ class TestParquet(ImpalaTestSuite):
     self.run_test_case(
         "QueryTest/parquet-decimal-precision-and-scale-altering", vector, unique_database)
 
+  def test_row_size_gt_4096_queries(self, unique_database):
+    table_format = 'parquet'
+    table_name = "{0}.{1}_{2}".format(
+        unique_database, "t_row_size_gt_4096", table_format)
+
+    # create table
+    field_string = ', '.join('field{} STRING'.format(i) for i in range(1, 601))
+    create_sql = "CREATE TABLE {} (id INT, {}) STORED AS {}".format(
+        table_name, field_string, table_format)
+    self.client.execute(create_sql)
+
+    # insert data
+    id_generation_sql = """
+    WITH ten AS (
+      SELECT 0 AS n
+      UNION ALL SELECT 1 UNION ALL SELECT 2 UNION ALL SELECT 3
+      UNION ALL SELECT 4 UNION ALL SELECT 5 UNION ALL SELECT 6
+      UNION ALL SELECT 7 UNION ALL SELECT 8 UNION ALL SELECT 9
+    )
+    SELECT
+      row_number() OVER (ORDER BY a.n) AS id
+    FROM
+      ten a, ten b, ten c, ten d
+    LIMIT
+      2000
+    """
+    field_string = ', '.join(['CAST(RAND() AS STRING) AS field{}'.format(i)
+                                for i in range(1, 601)])
+    insert_sql = "INSERT INTO {} SELECT CAST(s.id AS INT), {} FROM ({}) s;".format(
+        table_name, field_string, id_generation_sql)
+    self.execute_query_expect_success(self.client, insert_sql)
+
+    # do a query
+    query_sql = "SELECT * FROM {} where field1 = '123'".format(table_name)
+    self.execute_query_expect_success(self.client, query_sql)
+
 
 # We use various scan range lengths to exercise corner cases in the HDFS scanner more
 # thoroughly. In particular, it will exercise:
@@ -1278,7 +1336,9 @@ class TestParquet(ImpalaTestSuite):
 # 5. scan range fits at least one row
 MAX_SCAN_RANGE_LENGTHS = [0, 1, 2, 5, 16, 17, 32, 512]
 
+
 class TestScanRangeLengths(ImpalaTestSuite):
+
   @classmethod
   def get_workload(cls):
     return 'functional-query'
@@ -1286,15 +1346,12 @@ class TestScanRangeLengths(ImpalaTestSuite):
   @classmethod
   def add_test_dimensions(cls):
     super(TestScanRangeLengths, cls).add_test_dimensions()
-    cls.ImpalaTestMatrix.add_dimension(
-        ImpalaTestDimension('max_scan_range_length', *MAX_SCAN_RANGE_LENGTHS))
+    add_exec_option_dimension(cls, 'max_scan_range_length', MAX_SCAN_RANGE_LENGTHS)
     # Test doesn't need to be run for non-HDFS table formats.
     cls.ImpalaTestMatrix.add_constraint(
         lambda v: not v.get_value('table_format').file_format in ('kudu', 'hbase'))
 
   def test_scan_ranges(self, vector):
-    vector.get_value('exec_option')['max_scan_range_length'] =\
-        vector.get_value('max_scan_range_length')
     self.run_test_case('QueryTest/hdfs-tiny-scan', vector)
 
 
@@ -1302,8 +1359,10 @@ class TestScanRangeLengths(ImpalaTestSuite):
 # variation to the length is added by the test in order to exercise edge cases.
 TPCH_SCAN_RANGE_LENGTHS = [128 * 1024, 16 * 1024 * 1024]
 
+
 class TestTpchScanRangeLengths(ImpalaTestSuite):
   """Exercise different scan range lengths on the larger TPC-H data sets."""
+
   @classmethod
   def get_workload(cls):
     return 'tpch'
@@ -1374,7 +1433,7 @@ class TestParquetScanRangeAssigment(ImpalaTestSuite):
 # More tests for text scanner
 # 1. Test file that ends w/o tuple delimiter
 # 2. Test file with escape character
-class TestTextScanRangeLengths(ImpalaTestSuite):
+class TestTextScanRangeLengths1(ImpalaTestSuite):
   ESCAPE_TABLE_LIST = ["testescape_16_lf", "testescape_16_crlf",
       "testescape_17_lf", "testescape_17_crlf",
       "testescape_32_lf", "testescape_32_crlf"]
@@ -1385,16 +1444,12 @@ class TestTextScanRangeLengths(ImpalaTestSuite):
 
   @classmethod
   def add_test_dimensions(cls):
-    super(TestTextScanRangeLengths, cls).add_test_dimensions()
+    super(TestTextScanRangeLengths1, cls).add_test_dimensions()
     cls.ImpalaTestMatrix.add_dimension(
-        ImpalaTestDimension('max_scan_range_length', *MAX_SCAN_RANGE_LENGTHS))
-    cls.ImpalaTestMatrix.add_constraint(lambda v:
-        v.get_value('table_format').file_format == 'text' and
-        v.get_value('table_format').compression_codec == 'none')
+        create_uncompressed_text_dimension(cls.get_workload()))
+    add_exec_option_dimension(cls, 'max_scan_range_length', MAX_SCAN_RANGE_LENGTHS)
 
   def test_text_scanner(self, vector):
-    vector.get_value('exec_option')['max_scan_range_length'] =\
-        vector.get_value('max_scan_range_length')
     self.execute_query_expect_success(self.client, "drop stats "
         "functional.table_no_newline_part")
     self.execute_query_expect_success(self.client, "compute stats "
@@ -1410,8 +1465,10 @@ class TestTextScanRangeLengths(ImpalaTestSuite):
       result = self.client.execute("select count(*) from " + t)
       assert result.data == expected_result.data
 
+
 # Tests behavior of split "\r\n" delimiters.
 class TestTextSplitDelimiters(ImpalaTestSuite):
+
   @classmethod
   def get_workload(cls):
     return 'functional-query'
@@ -1419,9 +1476,8 @@ class TestTextSplitDelimiters(ImpalaTestSuite):
   @classmethod
   def add_test_dimensions(cls):
     super(TestTextSplitDelimiters, cls).add_test_dimensions()
-    cls.ImpalaTestMatrix.add_constraint(lambda v:
-        v.get_value('table_format').file_format == 'text' and
-        v.get_value('table_format').compression_codec == 'none')
+    cls.ImpalaTestMatrix.add_dimension(
+        create_uncompressed_text_dimension(cls.get_workload()))
 
   def test_text_split_delimiters(self, vector, unique_database):
     """Creates and queries a datafile that exercises interesting edge cases around split
@@ -1454,9 +1510,9 @@ class TestTextSplitDelimiters(ImpalaTestSuite):
     in the main text parsing algorithm. The second scan range exercises correctly
     identifying a split delimiter as the first in a scan range."""
     DEFAULT_IO_BUFFER_SIZE = 8 * 1024 * 1024
-    data = ('a' * (DEFAULT_IO_BUFFER_SIZE - 1) + "\r\n" + # first scan range
+    data = ('a' * (DEFAULT_IO_BUFFER_SIZE - 1) + "\r\n" +  # first scan range
             'b' * (DEFAULT_IO_BUFFER_SIZE - 3) + "\r\n" +
-            'a' * (DEFAULT_IO_BUFFER_SIZE - 1) + "\r\n" +     # second scan range
+            'a' * (DEFAULT_IO_BUFFER_SIZE - 1) + "\r\n" +  # second scan range
             'b' * (DEFAULT_IO_BUFFER_SIZE - 1))
     assert len(data) == DEFAULT_IO_BUFFER_SIZE * 4
 
@@ -1481,7 +1537,7 @@ class TestTextSplitDelimiters(ImpalaTestSuite):
       f.write(data)
       f.flush()
       self.filesystem_client.copy_from_local(f.name, location)
-    self.client.execute("refresh %s" % qualified_table_name);
+    self.client.execute("refresh %s" % qualified_table_name)
 
     vector.get_value('exec_option')['max_scan_range_length'] = max_scan_range_length
     query = "select * from %s" % qualified_table_name
@@ -1490,15 +1546,16 @@ class TestTextSplitDelimiters(ImpalaTestSuite):
 
     assert sorted(result.data) == sorted(expected_result)
 
+
 # Test for IMPALA-1740: Support for skip.header.line.count
-class TestTextScanRangeLengths(ImpalaTestSuite):
+class TestTextScanRangeLengths2(ImpalaTestSuite):
   @classmethod
   def get_workload(cls):
     return 'functional-query'
 
   @classmethod
   def add_test_dimensions(cls):
-    super(TestTextScanRangeLengths, cls).add_test_dimensions()
+    super(TestTextScanRangeLengths2, cls).add_test_dimensions()
     cls.ImpalaTestMatrix.add_constraint(lambda v:
         v.get_value('table_format').file_format == 'text' and
         v.get_value('table_format').compression_codec in ['none', 'gzip'])
@@ -1540,9 +1597,8 @@ class TestScanTruncatedFiles(ImpalaTestSuite):
     # strategy.
     # TODO: Test other file formats
     if cls.exploration_strategy() == 'exhaustive':
-      cls.ImpalaTestMatrix.add_constraint(lambda v:
-          v.get_value('table_format').file_format == 'text' and
-          v.get_value('table_format').compression_codec == 'none')
+      cls.ImpalaTestMatrix.add_dimension(
+          create_uncompressed_text_dimension(cls.get_workload()))
     else:
       cls.ImpalaTestMatrix.add_constraint(lambda v: False)
 
@@ -1573,6 +1629,7 @@ class TestScanTruncatedFiles(ImpalaTestSuite):
     assert(len(result.data) == 1)
     assert(result.data[0] == str(num_rows))
 
+
 class TestUncompressedText(ImpalaTestSuite):
   @classmethod
   def get_workload(cls):
@@ -1581,9 +1638,8 @@ class TestUncompressedText(ImpalaTestSuite):
   @classmethod
   def add_test_dimensions(cls):
     super(TestUncompressedText, cls).add_test_dimensions()
-    cls.ImpalaTestMatrix.add_constraint(lambda v:
-        v.get_value('table_format').file_format == 'text' and
-        v.get_value('table_format').compression_codec == 'none')
+    cls.ImpalaTestMatrix.add_dimension(
+        create_uncompressed_text_dimension(cls.get_workload()))
 
   # IMPALA-5315: Test support for date/time in unpadded format
   def test_scan_lazy_timestamp(self, vector, unique_database):
@@ -1592,6 +1648,7 @@ class TestUncompressedText(ImpalaTestSuite):
                                 unique_database, "lazy_ts", test_files)
     self.run_test_case('QueryTest/select-lazy-timestamp', vector, unique_database)
 
+
 class TestOrc(ImpalaTestSuite):
   @classmethod
   def get_workload(cls):
@@ -1602,7 +1659,7 @@ class TestOrc(ImpalaTestSuite):
     super(TestOrc, cls).add_test_dimensions()
     cls.ImpalaTestMatrix.add_constraint(
       lambda v: v.get_value('table_format').file_format == 'orc')
-    cls.ImpalaTestMatrix.add_dimension(ImpalaTestDimension('orc_schema_resolution', 0, 1))
+    add_exec_option_dimension(cls, 'orc_schema_resolution', [0, 1])
 
   @SkipIfFS.hdfs_small_block
   @SkipIfLocal.multiple_impalad
@@ -1645,7 +1702,8 @@ class TestOrc(ImpalaTestSuite):
   def _misaligned_orc_stripes_helper(
           self, table_name, rows_in_table, num_scanners_with_no_reads=0):
     """Checks if 'num_scanners_with_no_reads' indicates the expected number of scanners
-    that don't read anything because the underlying file is poorly formatted
+    that don't read anything because the underlying file is poorly formatted.
+    Additionally, test that select count(star) match with expected number of rows.
     """
     query = 'select * from %s' % table_name
     result = self.client.execute(query)
@@ -1666,10 +1724,17 @@ class TestOrc(ImpalaTestSuite):
       total += int(n)
     assert total == num_scanners_with_no_reads
 
+    # Test that select count(star) match with expected number of rows.
+    query = 'select count(*) from %s' % table_name
+    result = self.client.execute(query)
+    assert int(result.data[0]) == rows_in_table
+
   # Skip this test on non-HDFS filesystems, because orc-type-check.test contains Hive
   # queries that hang in some cases (IMPALA-9345). It would be possible to separate
   # the tests that use Hive and run most tests on S3, but I think that running these on
   # S3 doesn't add too much coverage.
+  # TODO(IMPALA-12349): Rename the columns to use the correct names (see
+  # test_type_conversions_hive3).
   @SkipIfFS.hive
   @SkipIfHive3.non_acid
   def test_type_conversions_hive2(self, vector, unique_database):
@@ -1729,23 +1794,28 @@ class TestOrc(ImpalaTestSuite):
     tmp_alltypes = unique_database + ".alltypes"
     create_plain_orc_table("functional.alltypestiny", tmp_alltypes)
     tbl_loc = self._get_table_location(tmp_alltypes, vector)
-    self.client.execute("""create table %s.illtypes (c1 boolean, c2 float,
-        c3 boolean, c4 tinyint, c5 smallint, c6 int, c7 boolean, c8 string, c9 int,
-        c10 float, c11 bigint) partitioned by (year int, month int) stored as ORC
+    self.client.execute("""create table %s.illtypes (id boolean, bool_col float,
+        tinyint_col boolean, smallint_col tinyint, int_col smallint, bigint_col int,
+        float_col boolean, double_col string, date_string_col int, string_col float,
+        timestamp_col bigint) partitioned by (year int, month int) stored as ORC
         location '%s'""" % (unique_database, tbl_loc))
-    self.client.execute("""create table %s.illtypes_ts_to_date (c1 boolean,
-        c2 float, c3 boolean, c4 tinyint, c5 smallint, c6 int, c7 boolean, c8 string,
-        c9 int, c10 float, c11 date) partitioned by (year int, month int) stored as ORC
+    self.client.execute("""create table %s.illtypes_ts_to_date (id boolean,
+        bool_col float, tinyint_col boolean, smallint_col tinyint, int_col smallint,
+        bigint_col int, float_col boolean, double_col string,
+        date_string_col int, string_col float, timestamp_col date)
+        partitioned by (year int, month int) stored as ORC
         location '%s'""" % (unique_database, tbl_loc))
-    self.client.execute("""create table %s.safetypes (c1 bigint, c2 boolean,
-        c3 smallint, c4 int, c5 bigint, c6 bigint, c7 double, c8 double, c9 char(3),
-        c10 varchar(3), c11 timestamp) partitioned by (year int, month int) stored as ORC
+    self.client.execute("""create table %s.safetypes (id bigint, bool_col boolean,
+        tinyint_col smallint, smallint_col int, int_col bigint, bigint_col bigint,
+        float_col double, double_col double, date_string_col char(3),
+        string_col varchar(3), timestamp_col timestamp)
+        partitioned by (year int, month int) stored as ORC
         location '%s'""" % (unique_database, tbl_loc))
     tmp_date_tbl = unique_database + ".date_tbl"
     create_plain_orc_table("functional.date_tbl", tmp_date_tbl)
     date_tbl_loc = self._get_table_location(tmp_date_tbl, vector)
-    self.client.execute("""create table %s.illtypes_date_tbl (c1 boolean,
-        c2 timestamp) partitioned by (date_part date) stored as ORC location '%s'"""
+    self.client.execute("""create table %s.illtypes_date_tbl (id boolean,
+        date_col timestamp) partitioned by (date_part date) stored as ORC location '%s'"""
         % (unique_database, date_tbl_loc))
     self.client.execute("alter table %s.illtypes recover partitions" % unique_database)
     self.client.execute("alter table %s.illtypes_ts_to_date recover partitions"
@@ -1799,17 +1869,16 @@ class TestOrc(ImpalaTestSuite):
         "CREATE TABLE {db}.{tbl} (id BIGINT) STORED AS ORC",
         unique_database, test_name, test_files)
     err = self.execute_query_expect_failure(self.client,
-        "select count(*) from {0}.{1}".format(unique_database, test_name))
+        "select count(id) from {0}.{1}".format(unique_database, test_name))
     assert expected_error in str(err)
 
   def test_invalid_schema(self, vector, unique_database):
     """Test scanning of ORC file with malformed schema."""
     self._run_invalid_schema_test(unique_database, "corrupt_schema",
-        "Encountered parse error during schema selection")
+        "Encountered parse error in tail of ORC file")
     self._run_invalid_schema_test(unique_database, "corrupt_root_type",
         "Root of the selected type returned by the ORC lib is not STRUCT: boolean.")
 
-
   def test_date_out_of_range_orc(self, vector, unique_database):
     """Test scanning orc files with an out of range date."""
     orc_tbl_name = "out_of_range_date_orc"
@@ -1907,6 +1976,7 @@ class TestScannerReservation(ImpalaTestSuite):
   def test_scanners(self, vector):
     self.run_test_case('QueryTest/scanner-reservation', vector)
 
+
 class TestErasureCoding(ImpalaTestSuite):
   @classmethod
   def get_workload(cls):
@@ -1925,13 +1995,10 @@ class TestBinaryType(ImpalaTestSuite):
   @classmethod
   def add_test_dimensions(cls):
     super(TestBinaryType, cls).add_test_dimensions()
-    # todo: IMPALA-5323: Support Kudu BINARY
     cls.ImpalaTestMatrix.add_constraint(
-      lambda v: v.get_value('table_format').file_format != 'kudu')
+      lambda v: v.get_value('table_format').file_format != 'json')
 
   def test_binary_type(self, vector):
-    if vector.get_value('table_format').file_format == 'json':
-      pytest.skip()
     self.run_test_case('QueryTest/binary-type', vector)
 
 
@@ -1948,3 +2015,48 @@ class TestParquetV2(ImpalaTestSuite):
 
   def test_parquet_v2(self, vector):
     self.run_test_case('QueryTest/parquet-v2', vector)
+
+
+class TestSingleFileTable(ImpalaTestSuite):
+  """IMPALA-12589: Regression test for corner case behavior where a table LOCATION might
+  point to a file instead of a directory. Expect SELECT and SHOW FILES to still work in
+  that case."""
+
+  @classmethod
+  def get_workload(cls):
+    return 'functional-query'
+
+  @classmethod
+  def add_test_dimensions(cls):
+    super(TestSingleFileTable, cls).add_test_dimensions()
+    cls.ImpalaTestMatrix.add_constraint(
+      lambda v: v.get_value('table_format').file_format == 'text')
+
+  def test_single_file_table(self, vector, unique_database):
+    # Create a simple table with one column.
+    params = {"db": unique_database, "tbl": "single_file_table"}
+    create_tbl_ddl = ("create external table {db}.{tbl} (c1 int) "
+                      "stored as textfile").format(**params)
+    self.execute_query_expect_success(self.client, create_tbl_ddl)
+
+    # Insert one value to the table.
+    insert_stmt = "insert into {db}.{tbl} values (1)".format(**params)
+    self.execute_query_expect_success(self.client, insert_stmt)
+
+    # Show files and get the path to the first data file.
+    show_files_stmt = "show files in {db}.{tbl}".format(**params)
+    res = self.execute_query_expect_success(self.client, show_files_stmt)
+    assert len(res.data) == 1
+    hdfs_file_path = res.data[0].split("\t")[0]
+    params['new_location'] = hdfs_file_path
+
+    # Alter location to point a data file.
+    alter_stmt = "alter table {db}.{tbl} set location '{new_location}'".format(**params)
+    self.execute_query_expect_success(self.client, alter_stmt)
+
+    # Show files and count star should still work.
+    res = self.execute_query_expect_success(self.client, show_files_stmt)
+    assert res.data[0].split("\t")[0] == (hdfs_file_path + '/')
+    select_stmt = "select count(*) from {db}.{tbl}".format(**params)
+    res = self.execute_query_expect_success(self.client, select_stmt)
+    assert res.data[0].split("\t")[0] == '1'
