@@ -18,9 +18,14 @@
 # specific language governing permissions and limitations
 # under the License.
 
+from __future__ import absolute_import, division, print_function
+
+import sys
 import unittest
 
 from datetime import datetime, timedelta
+from http.client import HTTPMessage
+
 from shell.cookie_util import (cookie_matches_path, get_cookie_expiry,
                                get_all_matching_cookies)
 
@@ -84,9 +89,11 @@ class TestCookieUtil(unittest.TestCase):
         cookies = get_all_matching_cookies(['a', 'b'], '/path', {})
         assert not cookies
 
-        headers = {'Set-Cookie': '''c_cookie=c_value
-        b_cookie=b_value
-        a_cookie=a_value'''}
+        headers = make_cookie_headers([
+            ('c_cookie', 'c_value'),
+            ('b_cookie', 'b_value'),
+            ('a_cookie', 'a_value')
+        ])
         cookies = get_all_matching_cookies(['a_cookie', 'b_cookie'], '/path', headers)
         assert len(cookies) == 2
         assert cookies[0].key == 'a_cookie' and cookies[0].value == 'a_value'
@@ -96,17 +103,21 @@ class TestCookieUtil(unittest.TestCase):
         assert cookies[0].key == 'b_cookie' and cookies[0].value == 'b_value'
         assert cookies[1].key == 'a_cookie' and cookies[1].value == 'a_value'
 
-        headers = {'Set-Cookie': '''c_cookie=c_value;Path=/
-        b_cookie=b_value;Path=/path
-        a_cookie=a_value;Path=/'''}
+        headers = make_cookie_headers([
+            ('c_cookie', 'c_value;Path=/'),
+            ('b_cookie', 'b_value;Path=/path'),
+            ('a_cookie', 'a_value;Path=/')
+        ])
         cookies = get_all_matching_cookies(['a_cookie', 'b_cookie'], '/path', headers)
         assert len(cookies) == 2
         assert cookies[0].key == 'a_cookie' and cookies[0].value == 'a_value'
         assert cookies[1].key == 'b_cookie' and cookies[1].value == 'b_value'
 
-        headers = {'Set-Cookie': '''c_cookie=c_value;Path=/
-        b_cookie=b_value;Path=/path
-        a_cookie=a_value;Path=/path/path2'''}
+        headers = make_cookie_headers([
+            ('c_cookie', 'c_value;Path=/'),
+            ('b_cookie', 'b_value;Path=/path'),
+            ('a_cookie', 'a_value;Path=/path/path2')
+        ])
         cookies = get_all_matching_cookies(['a_cookie', 'b_cookie'], '/path', headers)
         assert len(cookies) == 1
         assert cookies[0].key == 'b_cookie' and cookies[0].value == 'b_value'
@@ -114,25 +125,58 @@ class TestCookieUtil(unittest.TestCase):
         assert len(cookies) == 1
         assert cookies[0].key == 'b_cookie' and cookies[0].value == 'b_value'
 
-        headers = {'Set-Cookie': '''c_cookie=c_value;Path=/
-        b_cookie=b_value;Path=/path
-        a_cookie=a_value;Path=/path'''}
+        headers = make_cookie_headers([
+            ('c_cookie', 'c_value;Path=/'),
+            ('b_cookie', 'b_value;Path=/path'),
+            ('a_cookie', 'a_value;Path=/path')
+        ])
         cookies = get_all_matching_cookies(['a_cookie', 'b_cookie'], '/path/path1',
             headers)
         assert len(cookies) == 2
         assert cookies[0].key == 'a_cookie' and cookies[0].value == 'a_value'
         assert cookies[1].key == 'b_cookie' and cookies[1].value == 'b_value'
 
-        headers = {'Set-Cookie': '''c_cookie=c_value;Path=/
-        b_cookie=b_value;Path=/path1
-        a_cookie=a_value;Path=/path2'''}
+        headers = make_cookie_headers([
+            ('c_cookie', 'c_value;Path=/'),
+            ('b_cookie', 'b_value;Path=/path1'),
+            ('a_cookie', 'a_value;Path=/path2')
+        ])
         cookies = get_all_matching_cookies(['a_cookie', 'b_cookie'], '/path', headers)
         assert not cookies
 
-        headers = {'Set-Cookie': '''c_cookie=c_value;Path=/
-        b_cookie=b_value;Path=/path1
-        a_cookie=a_value;Path=/path2'''}
+        headers = make_cookie_headers([
+            ('c_cookie', 'c_value;Path=/'),
+            ('b_cookie', 'b_value;Path=/path1'),
+            ('a_cookie', 'a_value;Path=/path2')
+        ])
         cookies = get_all_matching_cookies(['a_cookie', 'b_cookie', 'c_cookie'], '/path',
             headers)
         assert len(cookies) == 1
         assert cookies[0].key == 'c_cookie' and cookies[0].value == 'c_value'
+
+
+def make_cookie_headers(cookie_vals):
+    """Make an HTTPMessage containing Set-Cookie headers for Python 2 or Python 3"""
+    if sys.version_info.major == 2:
+        # In Python 2 the HTTPMessage is a mimetools.Message object, and the
+        # Set-Cookie values all appear in a single header, separated by newlines.
+        cookies = ""
+        count = 0
+        for pair in cookie_vals:
+            name = pair[0]
+            value = pair[1]
+            cookies += name + '=' + value
+            if count + 1 < len(cookie_vals):
+                # Separate the cookies, unless it is the last cookie.
+                cookies += '\n '
+            count += 1
+        return {'Set-Cookie': cookies}
+    else:
+        # In Python 3 the HTTPMessage is an email.message.Message, and the
+        # Set-Cookie values appear as duplicate headers.
+        headers = HTTPMessage()
+        for pair in cookie_vals:
+            name = pair[0]
+            value = pair[1]
+            headers.add_header('Set-Cookie', name + "=" + value)
+        return headers
