@@ -866,7 +866,7 @@ class TestImpalaShell(ImpalaTestSuite):
 
   def test_query_time_and_link_message(self, vector, unique_database):
     shell_messages = ["Query submitted at: ", "(Coordinator: ",
-        "Query progress can be monitored at: "]
+        "Query state can be monitored at: "]
     # CREATE statements should not print query time and webserver address.
     results = run_impala_shell_cmd(
         vector, ['--query=create table %s.shell_msg_test (id int)' % unique_database])
@@ -915,42 +915,59 @@ class TestImpalaShell(ImpalaTestSuite):
     else:
       assert "Modified 1 row(s)" in results.stderr
 
-  def _validate_dml_stmt(self, vector, stmt, expected_rows_modified, expected_row_errors):
-    results = run_impala_shell_cmd(vector, ['--query=%s' % stmt])
-    expected_output = "Modified %d row(s), %d row error(s)" %\
-        (expected_rows_modified, expected_row_errors)
-    assert expected_output in results.stderr, results.stderr
-
   def test_kudu_dml_reporting(self, vector, unique_database):
     if vector.get_value('strict_hs2_protocol'):
       pytest.skip("Kudu not supported in strict hs2 mode.")
-    db = unique_database
-    run_impala_shell_cmd(vector, [
-        '--query=create table %s.dml_test (id int primary key, '
-        'age int null) partition by hash(id) partitions 2 stored as kudu' % db])
+    create_sql = 'create table %s (id int primary key, age int null)' \
+        'partition by hash(id) partitions 2 stored as kudu'
+    self._test_dml_reporting(vector, create_sql, unique_database, True)
 
-    self._validate_dml_stmt(
-        vector, "insert into %s.dml_test (id) values (7), (7)" % db, 1, 1)
-    self._validate_dml_stmt(vector, "insert into %s.dml_test (id) values (7)" % db, 0, 1)
-    self._validate_dml_stmt(
-        vector, "upsert into %s.dml_test (id) values (7), (7)" % db, 2, 0)
-    self._validate_dml_stmt(
-        vector, "update %s.dml_test set age = 1 where id = 7" % db, 1, 0)
-    self._validate_dml_stmt(vector, "delete from %s.dml_test where id = 7" % db, 1, 0)
+  def test_iceberg_dml_reporting(self, vector, unique_database):
+    if vector.get_value('strict_hs2_protocol'):
+      pytest.skip("DML results are not completely supported in strict hs2 mode.")
+    create_sql = 'create table %s (id int, age int) ' \
+        'stored as iceberg tblproperties("format-version"="2")'
+    self._test_dml_reporting(vector, create_sql, unique_database, False)
 
-    # UPDATE/DELETE where there are no matching rows; there are no errors because the
-    # scan produced no rows.
-    self._validate_dml_stmt(
-        vector, "update %s.dml_test set age = 1 where id = 8" % db, 0, 0)
-    self._validate_dml_stmt(vector, "delete from %s.dml_test where id = 7" % db, 0, 0)
+  def _test_dml_reporting(self, vector, create_sql, db, is_kudu):
+    """ Runs DMLs on Kudu or Iceberg tables and verifies that modifed / deleted row
+        count and number of row errors in Kudu are reported correctly. Kudu and Iceberg
+        can have different results when adding rows with the same primary key as this
+        leads to row errors in Kudu.
+    """
+    tbl = db + ".dml_test"
+    run_impala_shell_cmd(vector, ['--query=' + create_sql % tbl])
+
+    def validate(stmt, expected_rows_modified_iceberg, expected_rows_modified_kudu,
+                 expected_row_errors_kudu, is_delete=False):
+      results = run_impala_shell_cmd(vector, ['--query=' + stmt % tbl])
+      expected = ""
+      if is_kudu:
+        expected = "Modified %d row(s), %d row error(s)" \
+            % (expected_rows_modified_kudu, expected_row_errors_kudu)
+      elif is_delete and expected_rows_modified_iceberg > 0:
+        expected = "Deleted %d row(s)" % expected_rows_modified_iceberg
+      else:
+        expected = "Modified %d row(s)" % expected_rows_modified_iceberg
+      assert expected in results.stderr, results.stderr
+
+    validate("insert into %s (id) values (7), (7)", 2, 1, 1)
+    validate("insert into %s (id) values (7)", 1, 0, 1)
+    if is_kudu:
+      validate("upsert into %s (id) values (7), (7)", -1, 2, 0)
+    validate("update %s set age = 1 where id = 7", 3, 1, 0)
+    validate("delete from %s where id = 7", 3, 1, 0, is_delete=True)
+
+    # UPDATE/DELETE where there are no matching rows; there are no errors in Kudu because
+    # the scan produced no rows.
+    validate("update %s set age = 1 where id = 8", 0, 0, 0)
+    validate("delete from %s where id = 7", 0, 0, 0, is_delete=True)
 
     # WITH clauses, only apply to INSERT and UPSERT
-    self._validate_dml_stmt(vector,
-        "with y as (values(7)) insert into %s.dml_test (id) select * from y" % db, 1, 0)
-    self._validate_dml_stmt(vector,
-        "with y as (values(7)) insert into %s.dml_test (id) select * from y" % db, 0, 1)
-    self._validate_dml_stmt(vector,
-        "with y as (values(7)) upsert into %s.dml_test (id) select * from y" % db, 1, 0)
+    validate("with y as (values(7)) insert into %s (id) select * from y", 1, 1, 0)
+    validate("with y as (values(7)) insert into %s (id) select * from y", 1, 0, 1)
+    if is_kudu:
+      validate("with y as (values(7)) upsert into %s (id) select * from y", -1, 1, 0)
 
   def test_missing_query_file(self, vector):
     result = run_impala_shell_cmd(vector, ['-f', 'nonexistent.sql'], expect_success=False)
@@ -1057,7 +1074,9 @@ class TestImpalaShell(ImpalaTestSuite):
       assert "Could not resolve table reference: 'non_existence_large_table'" \
           in result.stderr
       end_time = time()
-      time_limit_s = 20
+      # Use higher timeout in ASAN/UBSAN to avoid flakiness (IMPALA-11921).
+      build_runs_slowly = ImpalaTestClusterProperties.get_instance().runs_slowly()
+      time_limit_s = 60 if build_runs_slowly else 20
       actual_time_s = end_time - start_time
       assert actual_time_s <= time_limit_s, (
           "It took {0} seconds to execute the query. Time limit is {1} seconds.".format(
@@ -1185,6 +1204,31 @@ class TestImpalaShell(ImpalaTestSuite):
     assert "| a    | b     |" in result.stdout, result.stdout
     assert "| true | false |" in result.stdout, result.stdout
 
+  def test_binary_display(self, vector):
+    """Test that binary values are displayed correctly."""
+    query = "select binary_col from functional.binary_tbl"
+    result = run_impala_shell_cmd(vector, ['-q', query])
+    assert "| binary1            |" in result.stdout, result.stdout
+    assert "| NULL               |" in result.stdout, result.stdout
+    assert "|                    |" in result.stdout, result.stdout
+    assert "| árvíztűrőtükörfúró |" in result.stdout, result.stdout
+    assert "| 你好hello          |" in result.stdout, result.stdout
+    assert "| \x00\xef\xbf\xbd\x00\xef\xbf\xbd                 |" in result.stdout, \
+        result.stdout
+    assert '| \xef\xbf\xbdD3"\x11\x00              |' in result.stdout, result.stdout
+
+  def test_binary_as_string(self, vector):
+    query = """select cast(binary_col as string) from functional.binary_tbl
+               where string_col != "invalid utf8" """
+    result = run_impala_shell_cmd(vector, ['-q', query])
+    # Column length omitted because some strict HS2 protocol returns header "binary_col"
+    # while others return "cast(binary_col as string)".
+    assert "| binary1            " in result.stdout, result.stdout
+    assert "| NULL               " in result.stdout, result.stdout
+    assert "|                    " in result.stdout, result.stdout
+    assert "| árvíztűrőtükörfúró " in result.stdout, result.stdout
+    assert "| 你好hello          " in result.stdout, result.stdout
+
   def test_null_values(self, vector):
     """Test that null values are displayed correctly."""
     if vector.get_value('strict_hs2_protocol'):
