@@ -293,10 +293,10 @@ class ImpalaShell(object):
     #   Server version: impalad version...
     #   Query: select sleep(10)
     #   Query submitted at:...
-    #   Query progress can be monitored at:...
+    #   Query state can be monitored at:...
     # We stop at 10 iterations to prevent an infinite loop if somehting goes wrong.
     iters = 0
-    while "Query progress" not in self.shell_process.stderr.readline() and iters < 10:
+    while "Query state" not in self.shell_process.stderr.readline() and iters < 10:
       iters += 1
 
   def get_result(self, stdin_input=None):
@@ -394,6 +394,6 @@ def get_impala_shell_executable(vector):
   return {
     'dev': [impala_shell_executable],
     'dev3': ['env', 'IMPALA_PYTHON_EXECUTABLE=python3', impala_shell_executable],
-    'python2': [os.path.join(IMPALA_HOME, 'shell/build/py2_venv/bin/impala-shell')],
-    'python3': [os.path.join(IMPALA_HOME, 'shell/build/py3_venv/bin/impala-shell')]
+    'python2': [os.path.join(IMPALA_HOME, 'shell/build/python2_venv/bin/impala-shell')],
+    'python3': [os.path.join(IMPALA_HOME, 'shell/build/python3_venv/bin/impala-shell')]
   }[vector.get_value_with_default('impala_shell', 'dev')]
