@@ -17,13 +17,15 @@
 # specific language governing permissions and limitations
 # under the License.
 
+from __future__ import absolute_import, division, print_function
+from builtins import round
 import logging
 from multiprocessing import Value
 import os
 import re
 from textwrap import dedent
 from time import sleep, time
-from sys import maxint
+from sys import maxsize
 
 from tests.stress.queries import QueryType
 from tests.stress.util import create_and_start_daemon_thread, increment
@@ -102,7 +104,7 @@ class QueryRunner(object):
     self.impalad_conn = self.impalad.impala.connect(impalad=self.impalad)
 
   def run_query(self, query, mem_limit_mb, run_set_up=False,
-                timeout_secs=maxint, cancel_mech=None, retain_profile=False):
+                timeout_secs=maxsize, cancel_mech=None, retain_profile=False):
     """Run a query and return an execution report. If 'run_set_up' is True, set up sql
     will be executed before the main query. This should be the case during the binary
     search phase of the stress test. 'cancel_mech' is optionally a CancelMechanism
@@ -187,10 +189,10 @@ class QueryRunner(object):
     if run_set_up and query.set_up_sql:
       LOG.debug("Running set up query:\n%s", query.set_up_sql)
       cursor.execute(query.set_up_sql)
-    for query_option, value in self.common_query_options.iteritems():
+    for query_option, value in self.common_query_options.items():
       cursor.execute(
           "SET {query_option}={value}".format(query_option=query_option, value=value))
-    for query_option, value in query.options.iteritems():
+    for query_option, value in query.options.items():
       cursor.execute(
           "SET {query_option}={value}".format(query_option=query_option, value=value))
     # Set a time limit if it is the expected method of cancellation, or as an additional
@@ -382,7 +384,7 @@ class QueryRunner(object):
 
   def get_metric_vals(self):
     """Get the current values of the all metrics as a list of (k, v) pairs."""
-    return [(k, v.value) for k, v in self._metrics.iteritems()]
+    return [(k, v.value) for k, v in self._metrics.items()]
 
   def increment_metric(self, name):
     """Increment the current value of the metric called 'name'."""
@@ -471,7 +473,7 @@ def _add_row_to_hash(row, curr_hash):
     curr_hash += _hash_val(idx, val)
     # Modulo the result to keep it "small" otherwise the math ops can be slow
     # since python does infinite precision math.
-    curr_hash %= maxint
+    curr_hash %= maxsize
   return curr_hash
 
 
