@@ -34,6 +34,8 @@ LOG = logging.getLogger('impala_test_suite')
 # constants
 SECTION_DELIMITER = "===="
 SUBSECTION_DELIMITER = "----"
+ALLOWED_TABLE_FORMATS = ['kudu', 'parquet', 'orc']
+
 
 # The QueryTestSectionReader provides utility functions that help to parse content
 # from a query test file
@@ -73,16 +75,18 @@ class QueryTestSectionReader(object):
     elif table_format.compression_codec == 'none':
       suffix = '_%s' % (table_format.file_format)
     elif table_format.compression_type == 'record':
-      suffix =  '_%s_record_%s' % (table_format.file_format,
+      suffix = '_%s_record_%s' % (table_format.file_format,
                                    table_format.compression_codec)
     else:
-      suffix =  '_%s_%s' % (table_format.file_format, table_format.compression_codec)
+      suffix = '_%s_%s' % (table_format.file_format, table_format.compression_codec)
     dataset = table_format.dataset.replace('-', '')
     return dataset + scale_factor + suffix
 
 
 def remove_comments(section_text):
-  return '\n'.join([l for l in section_text.split('\n') if not l.strip().startswith('#')])
+  lines = [line for line in section_text.split('\n') if not line.strip().startswith('#')]
+  return '\n'.join(lines)
+
 
 def parse_query_test_file(file_name, valid_section_names=None, encoding=None):
   """
@@ -98,10 +102,11 @@ def parse_query_test_file(file_name, valid_section_names=None, encoding=None):
   if section_names is None:
     section_names = ['QUERY', 'HIVE_QUERY', 'RESULTS', 'TYPES', 'LABELS', 'SETUP',
         'CATCH', 'ERRORS', 'USER', 'RUNTIME_PROFILE', 'SHELL', 'DML_RESULTS',
-        'HS2_TYPES', 'HIVE_MAJOR_VERSION', 'LINEAGE']
+        'HS2_TYPES', 'HIVE_MAJOR_VERSION', 'LINEAGE', 'IS_HDFS_ONLY']
   return parse_test_file(file_name, section_names, encoding=encoding,
       skip_unknown_sections=False)
 
+
 def parse_table_constraints(constraints_file):
   """Reads a table constraints file, if one exists"""
   schema_include = defaultdict(list)
@@ -134,11 +139,13 @@ def parse_table_constraints(constraints_file):
           raise ValueError('Unknown constraint type: %s' % constraint_type)
   return schema_include, schema_exclude, schema_only
 
+
 def parse_table_format_constraint(table_format_constraint):
   # TODO: Expand how we parse table format constraints to support syntax such as
   # a table format string with a wildcard character. Right now we don't do anything.
   return table_format_constraint
 
+
 def parse_test_file(test_file_name, valid_section_names, skip_unknown_sections=True,
     encoding=None):
   """
@@ -169,6 +176,24 @@ def parse_test_file(test_file_name, valid_section_names, skip_unknown_sections=T
     return parse_test_file_text(file_data, valid_section_names,
                                 skip_unknown_sections)
 
+
+def parse_runtime_profile_table_formats(subsection_comment):
+  prefix = "table_format="
+  if not subsection_comment.startswith(prefix):
+    raise RuntimeError('RUNTIME_PROFILE comment (%s) must be of the form '
+                       '"table_format=FORMAT[,FORMAT2,...]"' % subsection_comment)
+
+  parsed_formats = subsection_comment[len(prefix):].split(',')
+  table_formats = list()
+  for table_format in parsed_formats:
+    if table_format not in ALLOWED_TABLE_FORMATS:
+      raise RuntimeError('RUNTIME_PROFILE table format (%s) must be in: %s' %
+                         (table_format, ALLOWED_TABLE_FORMATS))
+    else:
+      table_formats.append(table_format)
+  return table_formats
+
+
 def parse_test_file_text(text, valid_section_names, skip_unknown_sections=True):
   sections = list()
   section_start_regex = re.compile(r'(?m)^%s' % SECTION_DELIMITER)
@@ -199,8 +224,11 @@ def parse_test_file_text(text, valid_section_names, skip_unknown_sections=True):
       subsection_comment = None
 
       subsection_info = [s.strip() for s in subsection_name.split(':')]
-      if(len(subsection_info) == 2):
+      if len(subsection_info) == 2:
         subsection_name, subsection_comment = subsection_info
+        if subsection_comment == "":
+          # ignore empty subsection_comment
+          subsection_comment = None
 
       lines_content = lines[1:-1]
 
@@ -237,7 +265,7 @@ def parse_test_file_text(text, valid_section_names, skip_unknown_sections=True):
 
       if subsection_name == 'CATCH':
         parsed_sections['CATCH'] = list()
-        if subsection_comment == None:
+        if subsection_comment is None:
           parsed_sections['CATCH'].append(subsection_str)
         elif subsection_comment == 'ANY_OF':
           parsed_sections['CATCH'].extend(lines_content)
@@ -254,28 +282,24 @@ def parse_test_file_text(text, valid_section_names, skip_unknown_sections=True):
       # will be verified against the DML_RESULTS. Using both DML_RESULTS and RESULTS is
       # not supported.
       if subsection_name == 'DML_RESULTS':
-        if subsection_comment is None or subsection_comment == '':
-          raise RuntimeError('DML_RESULTS requires that the table is specified ' \
+        if subsection_comment is None:
+          raise RuntimeError('DML_RESULTS requires that the table is specified '
               'in the comment.')
         parsed_sections['DML_RESULTS_TABLE'] = subsection_comment
         parsed_sections['VERIFIER'] = 'VERIFY_IS_EQUAL_SORTED'
 
       # The RUNTIME_PROFILE section is used to specify lines of text that should be
       # present in the query runtime profile. It takes an option comment containing a
-      # table format. RUNTIME_PROFILE secions with a comment are only evaluated for the
+      # table format. RUNTIME_PROFILE sections with a comment are only evaluated for the
       # specified format. If there is a RUNTIME_PROFILE section without a comment, it is
       # evaluated for all formats that don't have a commented section for this query.
       if subsection_name == 'RUNTIME_PROFILE':
-        if subsection_comment is not None and subsection_comment is not "":
-          allowed_formats = ['kudu']
-          if not subsection_comment.startswith("table_format="):
-            raise RuntimeError('RUNTIME_PROFILE comment (%s) must be of the form '
-              '"table_format=FORMAT"' % subsection_comment)
-          table_format = subsection_comment[13:]
-          if table_format not in allowed_formats:
-            raise RuntimeError('RUNTIME_PROFILE table format (%s) must be in: %s' %
-                (table_format, allowed_formats))
-          subsection_name = 'RUNTIME_PROFILE_%s' % table_format
+        if subsection_comment:
+          table_formats = parse_runtime_profile_table_formats(subsection_comment)
+          for table_format in table_formats:
+            subsection_name_for_format = 'RUNTIME_PROFILE_%s' % table_format
+            parsed_sections[subsection_name_for_format] = subsection_str
+          continue
 
       parsed_sections[subsection_name] = subsection_str
 
@@ -283,6 +307,7 @@ def parse_test_file_text(text, valid_section_names, skip_unknown_sections=True):
       sections.append(parsed_sections)
   return sections
 
+
 def split_section_lines(section_str):
   """
   Given a section string as produced by parse_test_file_text(), split it into separate
@@ -294,6 +319,7 @@ def split_section_lines(section_str):
   # Trim off the trailing newline and split into lines.
   return section_str[:-1].split('\n')
 
+
 def join_section_lines(lines):
   """
   The inverse of split_section_lines().
@@ -303,6 +329,7 @@ def join_section_lines(lines):
   """
   return '\n'.join(lines) + '\n'
 
+
 def write_test_file(test_file_name, test_file_sections, encoding=None):
   """
   Given a list of test file sections, write out the corresponding test file
