@@ -21,14 +21,20 @@ from tests.common.file_utils import grep_dir
 from tests.common.skip import SkipIfBuildType, SkipIfDockerizedCluster
 from tests.common.impala_cluster import ImpalaCluster
 from tests.common.impala_test_suite import ImpalaTestSuite
+from tests.util.parse_util import parse_duration_string_ms
 from datetime import datetime
+from multiprocessing import Process, Queue
+from time import sleep, time
 import itertools
 import json
+import logging
 import os
 import pytest
 import re
 import requests
 
+LOG = logging.getLogger(__name__)
+
 
 class TestWebPage(ImpalaTestSuite):
   ROOT_URL = "http://localhost:{0}/"
@@ -52,6 +58,8 @@ class TestWebPage(ImpalaTestSuite):
   PROMETHEUS_METRICS_URL = "http://localhost:{0}/metrics_prometheus"
   QUERIES_URL = "http://localhost:{0}/queries"
   HEALTHZ_URL = "http://localhost:{0}/healthz"
+  EVENT_PROCESSOR_URL = "http://localhost:{0}/events"
+  HADOOP_VARZ_URL = "http://localhost:{0}/hadoop-varz"
 
   # log4j changes do not apply to the statestore since it doesn't
   # have an embedded JVM. So we make two sets of ports to test the
@@ -107,6 +115,14 @@ class TestWebPage(ImpalaTestSuite):
       if build_flags["cmake_build_type"] in ["debug"]:
         assert not is_ndebug
 
+  def test_root_other_info(self):
+    """Tests to check glibc version and locale is available"""
+    for port in self.TEST_PORTS_WITH_SS:
+      other_info_page = requests.get(self.ROOT_URL.format(port) + "/?json").text
+      other_info = json.loads(other_info_page)
+      assert "effective_locale" in other_info
+      assert "glibc_version" in other_info
+
   def test_memz(self):
     """Tests /memz at impalad / statestored / catalogd"""
 
@@ -337,8 +353,11 @@ class TestWebPage(ImpalaTestSuite):
     self.__test_table_metrics(unique_database, "foo_part", "total-file-size-bytes")
     self.__test_table_metrics(unique_database, "foo_part", "num-files")
     self.__test_table_metrics(unique_database, "foo_part", "alter-duration")
+    self.__test_table_metrics(unique_database, "foo_part", "events-process-duration")
     self.__test_catalog_tablesfilesusage(unique_database, "foo_part", "1")
     self.__test_catalog_tables_loading_time(unique_database, "foo_part")
+    self.get_and_check_status(self.EVENT_PROCESSOR_URL, "events-consuming-delay",
+        ports_to_test=self.CATALOG_TEST_PORT)
 
   def __test_catalog_object(self, db_name, tbl_name, cluster_properties):
     """Tests the /catalog_object endpoint for the given db/table. Runs
@@ -526,6 +545,7 @@ class TestWebPage(ImpalaTestSuite):
       response_json = json.loads(responses[0].text)
     finally:
       self.client.cancel(query_handle)
+      self.client.close_query(query_handle)
     return response_json
 
   @pytest.mark.xfail(run=False, reason="IMPALA-8059")
@@ -607,6 +627,13 @@ class TestWebPage(ImpalaTestSuite):
         self.THREAD_GROUP_URL + "?group=disk-io-mgr&json", ports_to_test=[25000])
     assert len(responses) == 1
     response_json = json.loads(responses[0].text)
+    # Verify metric keys for each thread
+    for t in response_json['threads']:
+      assert "name" in t
+      assert "id" in t
+      assert "user_s" in t
+      assert "kernel_s" in t
+      assert "iowait_s" in t
     thread_names = [t["name"] for t in response_json['threads']]
     expected_name_patterns = ["ADLS remote", "S3 remote", "HDFS remote"]
     for pattern in expected_name_patterns:
@@ -889,6 +916,81 @@ class TestWebPage(ImpalaTestSuite):
     page = requests.head("http://localhost:25020/operations")
     assert page.status_code == requests.codes.ok
 
+  def test_catalog_operation_fields(self, unique_database):
+    """Verify the CREATE_DATABASE operation is consistent with the statement shown in the
+       /queries page of impalad."""
+    catalog_operations_page = requests.get("http://localhost:25020/operations?json").text
+    catalog_operations = json.loads(catalog_operations_page)
+    assert "finished_catalog_operations" in catalog_operations
+
+    queries_page = requests.get("http://localhost:25000/queries?json").text
+    queries = json.loads(queries_page)
+    assert "completed_queries" in queries
+
+    # Find the CREATE_DATABASE operation in catalogd
+    ts_format = "%Y-%m-%d %H:%M:%S.%f"
+    found = False
+    for op in catalog_operations["finished_catalog_operations"]:
+      if op["target_name"] == unique_database \
+          and op["catalog_op_name"] == "CREATE_DATABASE":
+        catalog_op_query_id = op["query_id"]
+        catalog_op_user = op["user"]
+        catalog_op_start_time = datetime.strptime(op["start_time"], ts_format)
+        catalog_op_end_time = datetime.strptime(op["finish_time"], ts_format)
+        catalog_op_duration = parse_duration_string_ms(op["duration"])
+        found = True
+        LOG.info("Found query id in catalog operations: " + catalog_op_query_id)
+        break
+    assert found
+
+    def verify_query_record(query):
+      assert "CREATE DATABASE" in query["stmt"]
+      assert unique_database in query["stmt"]
+      assert catalog_op_user == query["effective_user"]
+      assert datetime.strptime(query["start_time"], ts_format) <= catalog_op_start_time
+      assert datetime.strptime(query["end_time"], ts_format) >= catalog_op_end_time
+      assert parse_duration_string_ms(query["duration"]) >= catalog_op_duration
+
+    # Find the query in impalad
+    matched = False
+    for query in queries["completed_queries"]:
+      if query["query_id"] == catalog_op_query_id:
+        verify_query_record(query)
+        matched = True
+    if not matched:
+      LOG.info("Query id {0} not found in the completed queries list".format(
+          catalog_op_query_id))
+      # Try to find the query in the in-flight queries list. It could be waiting to
+      # be closed.
+      for query in queries["in_flight_queries"]:
+        if query["query_id"] == catalog_op_query_id:
+          verify_query_record(query)
+          matched = True
+
+    # Dump web pages for debug
+    if not matched:
+      LOG.info("Query id {0} not found in queries page!".format(catalog_op_query_id))
+      LOG.info("Catalog operations: " + catalog_operations_page)
+      LOG.info("Queries: " + queries_page)
+    assert matched
+
+  def test_catalog_metrics(self):
+    """Test /metrics of catalogd"""
+    url = self.METRICS_URL.format(*self.CATALOG_TEST_PORT) + "?json"
+    json_res = json.loads(requests.get(url).text)
+    metric_keys = {m["name"] for m in json_res["metric_group"]["metrics"]}
+    assert "catalog-server.metadata.file.num-loading-threads" in metric_keys
+    assert "catalog-server.metadata.file.num-loading-tasks" in metric_keys
+    assert "catalog-server.metadata.table.num-loading-file-metadata" in metric_keys
+    assert "catalog-server.metadata.table.num-loading-metadata" in metric_keys
+    assert "catalog-server.metadata.table.async-loading.num-in-progress" in metric_keys
+    assert "catalog-server.metadata.table.async-loading.queue-len" in metric_keys
+    assert "catalog.num-databases" in metric_keys
+    assert "catalog.num-tables" in metric_keys
+    assert "catalog.num-functions" in metric_keys
+    assert "catalog.hms-client-pool.num-idle" in metric_keys
+    assert "catalog.hms-client-pool.num-in-use" in metric_keys
+
   def test_query_progress(self):
     """Tests that /queries page shows query progress."""
     query = "select count(*) from functional_parquet.alltypes where bool_col = sleep(100)"
@@ -898,6 +1000,138 @@ class TestWebPage(ImpalaTestSuite):
       if query in json_part['stmt']:
         assert json_part["query_progress"] == "0 / 4 ( 0%)"
 
+  def try_until(self, desc, run, check, timeout=10, interval=0.1):
+    start_time = time()
+    while (time() - start_time < timeout):
+      result = run()
+      if check(result):
+        return result
+      sleep(interval)
+    assert False, "Timed out waiting for " + desc
+
+  def get_queries(self):
+    responses = self.get_and_check_status(
+      self.QUERIES_URL + "?json", ports_to_test=[25000])
+    assert len(responses) == 1
+    response_json = json.loads(responses[0].text)
+    return response_json
+
+  @pytest.mark.execute_serially
+  def test_query_cancel_created(self):
+    """Tests that if we cancel a query in the CREATED state, it still finishes and we can
+    cancel it."""
+    query = "select count(*) from functional_parquet.alltypes"
+    delay_created_action = "impalad_load_tables_delay:SLEEP@1000"
+
+    response_json = self.get_queries()
+    assert response_json['num_in_flight_queries'] == 0
+
+    # Start the query completely async. The server doesn't return a response until
+    # the query has exited the CREATED state, so we need to get the query ID another way.
+    self.client.set_configuration(dict(debug_action=delay_created_action))
+    proc = Process(target=lambda cli, q: cli.execute_async(q), args=(self.client, query))
+    proc.start()
+
+    response_json = self.try_until("query creation", self.get_queries,
+        lambda resp: resp['num_in_flight_queries'] > 0)
+    assert len(response_json['in_flight_queries']) == 1
+    assert response_json['in_flight_queries'][0]['state'] == 'CREATED'
+    query_id = response_json['in_flight_queries'][0]['query_id']
+
+    cancel_query_url = "{0}cancel_query?json&query_id={1}".format(self.ROOT_URL.format
+      ("25000"), query_id)
+    response = requests.get(cancel_query_url)
+    assert response.status_code == requests.codes.ok
+    response_json = json.loads(response.text)
+    assert response_json['error'] == "Query not yet running\n"
+
+    # Wait for query to start running. It should finish soon after.
+    proc.join()
+    response_json = self.try_until("query finished", self.get_queries,
+        lambda resp: resp['in_flight_queries'][0]['state'] == 'FINISHED')
+    assert response_json['num_in_flight_queries'] == 1
+
+    # We never fetch results for the async query, so it stays in-flight until cancelled.
+    response = requests.get(cancel_query_url)
+    assert response.status_code == requests.codes.ok
+    response_json = json.loads(response.text)
+    assert response_json['contents'] == "Query cancellation successful"
+
+    # Cancel request can return before cancellation is finalized. Retry for slow
+    # environments like ASAN.
+    response_json = self.try_until("query cancellation", self.get_queries,
+        lambda resp: resp['num_in_flight_queries'] == 0)
+    assert response_json['num_in_flight_queries'] == 0
+    assert response_json['num_waiting_queries'] == 0
+
+    expected_queries = [q for q in response_json['completed_queries']
+                        if q['query_id'] == query_id]
+    assert len(expected_queries) == 1
+
+  @pytest.mark.execute_serially
+  def test_query_cancel_exception(self):
+    """Tests that if we cancel a query in the CREATED state and it has an exception, we
+    can cancel it."""
+    # Trigger UDF ERROR: Cannot divide decimal by zero
+    query = "select *, 1.0/0 from functional_parquet.alltypes limit 10"
+    delay_created_action = "impalad_load_tables_delay:SLEEP@1000"
+
+    response_json = self.get_queries()
+    assert response_json['num_in_flight_queries'] == 0
+
+    def run(queue, client, query):
+      queue.put(client.execute_async(query))
+
+    # Start the query completely async. The server doesn't return a response until
+    # the query has exited the CREATED state, so we need to get the query ID another way.
+    self.client.set_configuration(dict(debug_action=delay_created_action))
+    queue = Queue()
+    proc = Process(target=run, args=(queue, self.client, query))
+    proc.start()
+
+    response_json = self.try_until("query creation", self.get_queries,
+        lambda resp: resp['num_in_flight_queries'] > 0)
+    assert len(response_json['in_flight_queries']) == 1
+    assert response_json['in_flight_queries'][0]['state'] == 'CREATED'
+    query_id = response_json['in_flight_queries'][0]['query_id']
+
+    cancel_query_url = "{0}cancel_query?json&query_id={1}".format(self.ROOT_URL.format
+      ("25000"), query_id)
+    response = requests.get(cancel_query_url)
+    assert response.status_code == requests.codes.ok
+    response_json = json.loads(response.text)
+    assert response_json['error'] == "Query not yet running\n"
+
+    # Fetch query results.
+    query_handle = queue.get()
+    proc.join()
+    assert query_handle
+    try:
+      self.client.fetch(query, query_handle)
+    except Exception as e:
+      re.match("UDF ERROR: Cannot divide decimal by zero", str(e))
+
+    # Failed query should be completed.
+    response_json = self.get_queries()
+    assert response_json['num_in_flight_queries'] == 0
+    assert response_json['num_waiting_queries'] == 0
+
+    expected_queries = [q for q in response_json['completed_queries']
+                        if q['query_id'] == query_id]
+    assert len(expected_queries) == 1
+
+  @pytest.mark.execute_serially
+  def test_hadoop_varz_page(self):
+    """test for /hadoop-var to check availablity of haqoop configuration like
+    hive warehouse dir, fs.defaultFS"""
+    responses = self.get_and_check_status(self.HADOOP_VARZ_URL,
+        "hive.metastore.warehouse.dir", ports_to_test=self.TEST_PORTS_WITHOUT_SS)
+    responses = self.get_and_check_status(self.HADOOP_VARZ_URL,
+        "hive.metastore.warehouse.external.dir", ports_to_test=self.TEST_PORTS_WITHOUT_SS)
+    responses = self.get_and_check_status(self.HADOOP_VARZ_URL,
+        "fs.defaultFS", ports_to_test=self.TEST_PORTS_WITHOUT_SS)
+    # check if response size is 2 , for both catalog and impalad webUI
+    assert len(responses) == 2
 
 class TestWebPageAndCloseSession(ImpalaTestSuite):
   ROOT_URL = "http://localhost:{0}/"
